package com.despark.tuning3d.model.brands
{
	import com.despark.tuning3d.Notifications;
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.cars.CarsParser;
import com.despark.tuning3d.model.core.AppMode;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Device;
import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.core.http.HttpClient;
	import com.despark.tuning3d.model.core.http.caching.HttpCachePolicy;
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.packages.PackagesParser;
import com.despark.tuning3d.view.components.brands.BrandsScreen;
	import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;

import flash.events.Event;
import flash.events.IOErrorEvent;

import flash.events.TimerEvent;

import flash.filesystem.File;
	
	import by.blooddy.crypto.serialization.JSONer;

import flash.filesystem.FileMode;

import flash.filesystem.FileStream;
import flash.net.URLLoader;
import flash.net.URLRequest;
import flash.system.Capabilities;

import flash.utils.Timer;

import org.as3commons.lang.Assert;

import org.puremvc.as3.patterns.proxy.Proxy;
	
	import starling.utils.AssetManager;
	
	public class BrandsProxy extends Proxy
	{
		public static var NAME : String = "BrandsProxy";     
		
		public var httpClient : HttpClient;
		public var assetManager : AssetManager;
		
		private var brandsResponseString : String;
		private var brandsParser : BrandsParser;  
		private var _brandsJSON:Array=null;

        private static var _isAllBrandsLoaded : Boolean = false;

		public function BrandsProxy()
		{
			super(NAME);
			
			brandsParser = new BrandsParser();
			brandsParser.carsParser = new CarsParser();
		}
		
		public static function makeAllCarsList(brands:Array):Object
		{
			var i:int=0;
			var j:int=0;
			var cars:Object={};
			var id:String;
			
			for(i=0; i<brands.length; i++)
			{
				var barand:Brand=brands[i] as Brand;
				for(j=0; j<barand.cars.length; j++)
				{
					var car:Car=barand.cars[j] as Car;
					if(car) 
					{
						id=car.id;
						cars[id]=car;
					}
				}
				
			}
			
			return cars;  
		}

		public function loadAllBrands(completeFunction : Function = null) : void
		{
			if(_isAllBrandsLoaded)
			{
                if(completeFunction)
                    completeFunction();
				return;
			}

            httpClient.getPathCached("carsAll", null, HttpCachePolicy.INTERNET_FIRST, function(success : Boolean, result : Object) : void
            {
                if (success){
                    var json : Object = JSONer.parse(result);
                    brandsParser.parseObjects(json);
					_isAllBrandsLoaded = true;

					if(completeFunction)
						completeFunction();
                }
			});
		}

		public function loadBrands() : void
		{			
			if(!Car.BlankCar)
			{
				Car.BlankCar = new Car();
				Car.BlankCar.displayName= "more cars?";
				var appDirectory : File = File.applicationDirectory;
				var path:File = appDirectory.resolvePath("assets/textures/more_cars.png");
				path.preventBackup = true;
				Car.BlankCar.imageURL="file:///"+path.nativePath;
				if(!AlertView.language) {
                    AlertView.language = new Language(null);
                    AlertView.language.code = "en-US";
                }
				var url : String = null;
				if (Constants.isIOS){
					url = AlertView.language.translate("mob_settings", "appleStoreUrl");
				}
				else{
					url = AlertView.language.translate("mob_settings", "googlePlayUrl");
				}
				Car.BlankCar.moreCarsURL = url;
			}

            ImageLoaderWithStates.ZIP_ERROR_REPORT_TIME = 60;

            httpClient.getPathCached("cars", null, HttpCachePolicy.INTERNET_FIRST, function(success : Boolean, result : Object) : void
            {

                if (success)
                {
                    var brandsJSONString : String = result as String;
                    if(Constants.TEST_TUNING_STATES){
                        brandsJSONString=brandsJSONString.replace("5319aae269584a880334ff17", "58981566e847a024876d4d7f").replace("5319aae269584a880334ff17", "58981566e847a024876d4d7f"); //убрать это
                    }
                    var brandsJSON : Array = JSONer.parse(brandsJSONString) as Array;

                    _brandsJSON = brandsJSON;
                    loadRecent();
                }
                else {
                    sendNotification(Notifications.BRANDS_LOADING_FAILED);
                }
            });
		}


		private function recentLoadComplete(success : Boolean, result : Object) : void
		{
            if (success)
            {
                var recentJSONString : String;
                var recentJSON : Array;

                try{
                    recentJSONString  = result as String;
                    recentJSON  = JSONer.parse(recentJSONString) as Array;
                }catch(e:Error){
                    sendNotification(Notifications.BRANDS_LOADING_FAILED);
                    return;
                }

                var brands:Array = handleBrandsLoaded(_brandsJSON);




                PackagesParser.allCarsList=BrandsProxy.makeAllCarsList(brands);
				var allCars:Object = PackagesParser.allCarsList;
				var isTuningScreen:Boolean = (AppMode.self.mode==AppMode.MODE_TUNING);
				var j:int;
				for(j = recentJSON.length-1; j >= 0; j--){
					var jsonData:Object = recentJSON[j];
                    if((jsonData.carType=="pickup hd")==isTuningScreen){
                        recentJSON.removeAt(j);
                    }
				}

				for(j = brands.length-1; j >= 0; j--){
					var br:Brand = brands[j] as Brand;
					for(var k:int = br.cars.length-1; k >= 0; k--){
						if((Car(br.cars[k]).carType=="pickup hd")==isTuningScreen){
                            br.cars.removeAt(k);
						}
					}
					if(br.cars.length==0){
						brands.removeAt(j);
					}
				}
				// **************************************
				getRecentOrder(function (orderData: Object): void {
					var featuredCars:Array=handleFeaturedLoaded(recentJSON);

					var lastCarAdded:Array=handleLastCarAddedLoaded(recentJSON, []/*featuredCars*/);
					var newFeaturedCars:Array=[];
					var newLastCarAdded:Array=[];
					var car:Car;
					var newCar:Car;
					var brand : Brand;
					var i: int = 1;
					for each(car in featuredCars)
					{

						newCar = getCarById(car.id, brands);


						if (newCar){
							newCar.bgIndex = car.bgIndex;
							newCar.hdsize = car.hdsize;
							newCar.carChooserBgName = car.carChooserBgName;
							newCar.defaultBgId = car.defaultBgId;
							newFeaturedCars.push(newCar);

							if (orderData && orderData[car.id]) {
								newCar.updateText
										= car.updateText
										= orderData[car.id].updateText;
								newCar.feathersCarOrder
										= car.feathersCarOrder
										= orderData[car.id].order;
							} else {
								newCar.feathersCarOrder = car.feathersCarOrder = i * 10;
							}


						}
						else{
							newFeaturedCars.push(car)
						}
						i++;
					}

					if (orderData) {
						newFeaturedCars.sortOn( ['feathersCarOrder'], [Array.NUMERIC]);
					}

					featuredCars=newFeaturedCars;
					for each(car in lastCarAdded) {
						newCar = getCarById(car.id, brands);
						if (newCar){
							newCar.defaultBgId = car.defaultBgId;
							newLastCarAdded.push(newCar)
						} else{
							newLastCarAdded.push(car)
						}
					}

					lastCarAdded = newLastCarAdded;

					brand = new Brand();
					brand.name = "Last Added";
					brand.imageURL = "";
					brand.lastAdded=true;
					brand.cars = lastCarAdded;
					brands.unshift(brand);

					var indexUnderConstruction:int=0;
					var indexComingSoon:int=0;

					for(i = 0; i < brands.length; i++) {
						(brands[i] as Brand).index=i;

						if ((brands[i] as Brand).name=="Under Construction"){
							indexUnderConstruction=i;
						}

						if ((brands[i] as Brand).name=="Coming Soon"){
							indexComingSoon=i;
						}
					}

					var comingSoonBrand:Brand = brands[indexComingSoon];

					brands[indexComingSoon] = brands[indexUnderConstruction];
					brands[indexUnderConstruction] = comingSoonBrand;


					sendNotification(Notifications.BRANDS_LOADED, {brands: brands, featuredCars: featuredCars});
					_brandsJSON=null;
				})


            }else{
                sendNotification(Notifications.BRANDS_LOADING_FAILED);
            }
		}

		private function loadRecent():void
		{ //
			httpClient.getPathCached("recent", null, HttpCachePolicy.INTERNET_FIRST, recentLoadComplete);
		}

		private function getRecentOrder(cb: Function): void {
			httpClient.getPath(Constants.API_URL_assets2_recent, {mini: true, platform: Device.isIos() ? 'ios' : 'android' }, function (success: Boolean, result: String): void {
				if (success && result) {
					var data: Array = JSONer.parse(result);
					var resultData: Object = {};

					var recentCount : int = 3;
					if(AppMode.self.mode === AppMode.MODE_TRUCK_CONFIGURATOR){
						recentCount = data.length;
					}

					for (var i: int = 0; i < recentCount; i++) {
						resultData[data[i]._id] = {
							updateText: data[i].updateText,
							order: i
						}
					}

					cb(resultData);
				} else {
					cb(null);
				}
			} )
		}

		private function getCarById(car_id:String, brands:Array):Car
		{
			for each(var brand : Brand in brands){
				for each(var car : Car in brand.cars){
					if (car.id == car_id) return car;
				}
			}
			return null;
		}
		
		private function handleBrandsLoaded(brandsJSON : Array) :Array
		{
			var brands : Array = brandsParser.parseObjects(brandsJSON);
			//var featuredCar : Car = new Car();
			
			for each(var brand : Brand in brands) 
			{
                var moreCars : Car = new Car();
                moreCars.displayName= "more cars?";
                //moreCars.imageURL="car-icon.gif";
                var appDirectory : File = File.applicationDirectory;
                var path:File = appDirectory.resolvePath("assets/textures/more_cars.png");
                path.preventBackup = true;
                moreCars.imageURL="file:///"+path.nativePath;

                var url : String = null;
                if (Constants.isIOS){
                    url = AlertView.language.translate("mob_settings", "appleStoreUrl");
                } else{
                    url = AlertView.language.translate("mob_settings", "googlePlayUrl");
                }
                moreCars.moreCarsURL = url;//"http:///www.google.com";
                brand.cars.push(moreCars);
				
			}
			
			return brands;
		}
		
		private function handleLastCarAddedLoaded(recentJSON : Array, exluded:Array) : Array
		{
			var car:Car;
			var cars : Array = brandsParser.carsParser.parseObjects(recentJSON);
			
//			var exludedIdList:Array = [];
//
//			for each( car in exluded){
//				exludedIdList.push(car.id);
//			}
			
			var lastAddedCars:Array = [];
			
			if (cars)
			{
				function orderSortKey(a,b):int
				{
					var a_i : Number = Number(a.updated);
					var b_i : Number = Number(b.updated);
					
					if(a_i<b_i)return 1;
					if(a_i>b_i)return -1;
					
					return 0;
				}
				
//				cars=cars.sort(orderSortKey);
				
				var carchooserOrder:int=0;
				for each( car in cars) {
					car.carchooserOrder=carchooserOrder;
					if(car.defaultBkg && car.defaultBkg.length){
						var str:String = car.defaultBkg[0];
						var reg:RegExp = /([^\/]+)$/gsm;
						var res:Object = reg.exec(str);
						car.defaultBgId = res[0];
					}
					carchooserOrder++;
				}
				
				for each( car in cars) {
//					if (exludedIdList.indexOf(car.id) >= 0) continue;
					lastAddedCars.push(car);
				}
				
				return lastAddedCars;
			}
			
			return null;	
		}
		
		
		private function handleFeaturedLoaded(recentJSON : Array) : Array
		{
			var car:Car;
			var cars : Array = brandsParser.carsParser.parseObjects(recentJSON);
			var featuredCar : Car = null;
			
			var featuredCars:Array=[];
			
			if(cars)
			{
				
				var carchooserOrder:int=0;
				for each( car in cars) 
				{
					car.carchooserOrder=carchooserOrder;
					carchooserOrder++;
				}
				var i:int=0;

				var recentCount : int = 3;
				if(AppMode.self.mode === AppMode.MODE_TRUCK_CONFIGURATOR){
					recentCount = cars.length;

				}
				
				for each( car in cars)
				{
					featuredCar=new Car();
					featuredCar.id = car.id;
					featuredCar.imageURL = car.featuredImageUrl.replace("default3.png", "default.png");
					featuredCar.featuredImageUrl = car.featuredImageUrl.replace("default3.png", "default.png");
					featuredCar.carChooserBgName = car.carChooserBgName;
					featuredCar.isFeatured=true;
					featuredCar.model = car.model;
					featuredCar.name = car.name;
					featuredCar.year = car.year;
					featuredCar.updateText = car.updateText;
					featuredCar.hdsize = car.hdsize;
					featuredCar.defaultBkg = car.defaultBkg;
					if(featuredCar.defaultBkg && featuredCar.defaultBkg.length){
						var str:String = featuredCar.defaultBkg[0];
						var reg:RegExp = /([^\/]+)$/gsm;
						var res:Object = reg.exec(str);
						featuredCar.defaultBgId = res[0];
					}


					if(featuredCars.length<recentCount) {
						featuredCar.bgIndex = i;
						i++;
						featuredCars.push(featuredCar);
					}
					else {
						break;
					}
				}
				
				return featuredCars;
			}
			
			return null;	
			
		}
	}
}


