package com.despark.tuning3d.model.cars
{
	import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.core.http.HttpClient;
	import com.despark.tuning3d.model.core.http.caching.HttpCachePolicy;
	import com.despark.tuning3d.model.tunings.Tuning;

	import flash.display.BitmapData;
	import flash.display.JPEGEncoderOptions;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.net.URLLoaderDataFormat;
	import flash.utils.ByteArray;
	
	import by.blooddy.crypto.serialization.JSONer;
	
	import org.puremvc.as3.patterns.proxy.Proxy;
	
	import ru.inspirit.net.MultipartURLLoader;
	
	public class CarBackgroundsProxy extends Proxy {
		public static var NAME : String = "CarBackgroundsProxy";
		
		public var httpClient : HttpClient;
		
		private var multipartURLLoader : MultipartURLLoader;
        public static var uploaded_background_id:String;
		
		public var CarBackgrounds:Array=null;

		/**
		 * Backgrounds that does'nt depend on user and carType,
		 */
		public static function get CAR_ALL_BGS() : Array {
			return CAR_ALL_BACKGROUNDS;
		}
		private static var CAR_ALL_BACKGROUNDS : Array;
		public function CarBackgroundsProxy() {
			super(NAME); 
		}
		
		public function loadBackgrounds(userId : String, refresh : Boolean = false, carType : String = "", cb: Function = null, withDelete:Boolean = false) : void
		{
			Tracer.print("loadBackgrounds:" +  userId + "   carType:" +  carType + "  refresh։ " + refresh, "LOAD_BACKGROUNDS");
            var params : Object = {};
			if(userId)	params.userId = userId;
			if(carType.length>0){
				params.carType = carType;
            }
			if(withDelete){
				params.withDelete = true;
			}

			httpClient.getPathCached("", params, refresh ? HttpCachePolicy.INTERNET_FIRST : HttpCachePolicy.CACHE_FIRST, function(success : Boolean, result : Object) : void
			{

                Tracer.print("loadBackgrounds:" +  success, "LOAD_BACKGROUNDS");
				if(success)
				{
					var backroundsJSON : Object = JSONer.parse(String(result));
					var parser : CarBackgroundsParser = new CarBackgroundsParser();
					var carBackgrounds : Array = parser.parseObjects(backroundsJSON as Array);
					
					CarBackgrounds = carBackgrounds;
					if(carType == "all")
					{
						CAR_ALL_BACKGROUNDS = carBackgrounds;
					}
					
					sendNotification(Notifications.CAR_BACKGROUNDS_LOADED, carBackgrounds);
					if (cb) {
						cb();
					}
				}
				else
				{
					sendNotification(Notifications.CAR_BACKGROUNDS_LOADING_FAILED);
				}
			},true, HttpClient.API_DOMAIN + '/api/v1/backgrounds');
		}

		public function getURLById(id:String, resultHandler:Function):void{
			var url:String = 'http://api.3dtuning.com/api/v1/backgrounds/' + id;
			var params:Object = {
				isArray: false,
				isDeleted: true
			};
			httpClient.getPath(url, params, resultHandler);
		}

		public function getUrlArrayById(tuning:Tuning, resultHandler:Function):void{
			var loadedCount:int = tuning.customSnapshotsBackgrounds.length;
			var arr:Array = [];
			for(var i:int = 0; i < tuning.customSnapshotsBackgrounds.length; i++){
				var func:Function = function(index:int):void{
					getURLById(tuning.customSnapshotsBackgrounds[index], function(success:Boolean, result:*):void{
						if(success){
							var obj:Object = JSON.parse(result);
							var camIndex:int = tuning.customSnapshotsCamera[index];
							if(index != -1){
								if(camIndex == 1 && obj.hasOwnProperty("imagePathSecondCam")){
									arr[index] = "http:" + obj.imagePathSecondCam;
								}else{
									arr[index] = "http:" + obj.imagePath;
								}
							}
						}
						loadedCount--;
						if(loadedCount == 0){
							resultHandler(arr);
						}
					});
				};
				func(i);
			}
		}

		public function upload(bitmapData : BitmapData) : void {
			var bytes : ByteArray = bitmapData.encode(bitmapData.rect, new JPEGEncoderOptions(80));
			
			multipartURLLoader = new MultipartURLLoader();
			multipartURLLoader.dataFormat = URLLoaderDataFormat.BINARY;
			//			multipartURLLoader.addVariable("files", "photo.jpg");
			//			multipartURLLoader.addFile(bytes, "photo.jpg", "files", "application/octet-stream");
			multipartURLLoader.addVariable("files[]", "photo.jpeg");
			multipartURLLoader.addFile(bytes, "photo.jpeg", "files[]", "image/jpeg");
			multipartURLLoader.loader.addEventListener(Event.COMPLETE, multipartURLLoaderCompleteHandler);
			multipartURLLoader.loader.addEventListener(IOErrorEvent.IO_ERROR, multipartURLLoaderIoErrorHandler);
			var url_str : String = httpClient.urlForPath("backgrounds");
			multipartURLLoader.load(url_str);
		}
		
		private function multipartURLLoaderCompleteHandler(event : Event) : void{
			var id : String = (event.target.data as ByteArray).toString();
			CarBackgroundsProxy.uploaded_background_id = JSONer.parse(id)._id;
			sendNotification(Notifications.CAR_BACKGROUND_UPLOADED, JSON.parse(id));
			multipartURLLoader = null;
		}
		
		private function multipartURLLoaderIoErrorHandler(event : IOErrorEvent) : void
		{
			var result : String = multipartURLLoader.loader.data;
			sendNotification(Notifications.CAR_BACKGROUND_UPLOADING_FAILED, result);
			
			multipartURLLoader = null;
		}
		
		public function cancelUploading() : void
		{
			if(multipartURLLoader)
			{
				multipartURLLoader.loader.removeEventListener(Event.COMPLETE, multipartURLLoaderCompleteHandler);
				multipartURLLoader.loader.removeEventListener(IOErrorEvent.IO_ERROR, multipartURLLoaderIoErrorHandler);
				multipartURLLoader.close();
				multipartURLLoader = null;
			}
		}
		
		public function deleteBackground(carBackground : CarBackground) : void {
			httpClient.deletePath("backgrounds/" + carBackground.id, null, function(success : Boolean, result : Object) : void {
				if(success) {
					sendNotification(Notifications.CAR_BACKGROUND_DELETED, {background: carBackground});
				} else {
					sendNotification(Notifications.CAR_BACKGROUND_DELETION_FAILED);
				}
			});
		}
	}
}

