package com.despark.tuning3d.model.cars {
import com.despark.tuning3d.model.brands.BrandsParser;
    import com.despark.tuning3d.model.cars.CarsParser;
    import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Device;
import com.despark.tuning3d.model.core.Parser;
import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.packages.Packet;
import com.despark.tuning3d.model.parts.Part;
import com.despark.tuning3d.model.parts.PartColor;
import com.despark.tuning3d.model.parts.PartDisk;
import com.despark.tuning3d.model.parts.PartDisk;
import com.despark.tuning3d.model.parts.PartMaterial;
import com.despark.tuning3d.model.parts.PartMaterial;
import com.despark.tuning3d.model.parts.PartMaterialImage;
import com.despark.tuning3d.model.parts.PartSlot;
import com.despark.tuning3d.model.parts.PartSlotDisk;
import com.despark.tuning3d.view.components.core.ZipManager;
    import com.utils.Console;
    import com.utils.Logger;
import com.utils.MemoryTracker;

import flash.utils.Dictionary;
import flash.utils.setTimeout;

public class CarsParser extends Parser {
    private static const SHADOW_OFFSET_X:int = 52;

    private var _parserProgressCallback:Function = null;
    private var _parserCompleteCallback:Function = null;

    private var _loadingCar:Car = null;
    private var _carJSON:Object = null;
    private var _componentsJSON:Array = null;
    private var _tuningJSON:Object = null;

    private var _imagesPath:String = null;

    private var _existsFrontBamper:Boolean = false;
    private var _shadowBlackX:Number = 0;
    public static var carArray:Array;


    private var _usedPartSlotsMap:Object = {};
    private var _partsMap:Dictionary = new Dictionary(true);

    private var _bodyId:String = null;

    private var _parsedComponents:int = 0;
    private var _totalComponents:int = 0;
    internal static const wheelOnlyList:Object = {};
    //private var components:Array = componentsJSON as Array;



    public function parseProductInfo(car:Car, packet:Packet = null):void {

        if (!car)
            return;


        if (car.id == "58981566e847a024876d4d7f") {
            car.isChopper = true;
        }
        if (car.id == "5cb5e47571923820d89c13d8") {
            car.isBeetle = true;
        }

        if (packet) {

            car.addPacket(packet);//packet=packet;
        }
    }


    private function parseGeneralInfoAndGetImagesPath(car:Car, carJSON:Object):String {
        car.id = carJSON._id;
        car.model = carJSON.model;
        car.year = carJSON.year;
        car.name = carJSON.make;
        car.suspension = carJSON.suspension;
        car.defaultSuspension = carJSON.suspension;
        car.maxSuspension = carJSON.maxSuspension;
        car.limiterMaxSuspension = carJSON.maxSuspension;
        car.minSuspension = carJSON.minSuspension;
        car.limiterMinSuspension = carJSON.minSuspension;
        if (car.hasOwnProperty('suspension')) {
            var susp : Number = carJSON.suspension;

            if (susp != 0) {
                susp = -car.maxSuspension + susp;
            }
            car.suspension = susp;
        }


        car.isFeatured = carJSON.featured != null;

        if(carJSON.hasOwnProperty("limitedOptions")) {
            car.limitedOptions = carJSON.limitedOptions;
        }

        if (carJSON.hasOwnProperty('useZipSplitCam')) {
            car.useZipSplitCam = carJSON.useZipSplitCam;
        } else {
            car.useZipSplitCam = false;
        }


        if(carJSON.hasOwnProperty("body"))
        {
            car.body = carJSON.body;
        }
        if(carJSON.hasOwnProperty("suspensionForState"))
        {
            car.suspensionForState =  carJSON.suspensionForState;
        }

        if(carJSON.hasOwnProperty("suspensionForComponentsStructure"))
        {
            if(carJSON.suspensionForComponentsStructure is Array)
            {
                var suspForComp : Array = carJSON.suspensionForComponentsStructure;
                var sortFunction : Function = function(a : Object, b : Object):Number
                {
                    return b.components.length - a.components.length
                };

                suspForComp.sort(sortFunction);
                //
                // parsing suspension settings for component groups
                var i:int;
                var j:int;
                var prop:Object;
                var prop2:Object;
                for(i = 0; i < suspForComp.length; i++){
                    prop = suspForComp[i];
                    if(prop.hasOwnProperty("components") && prop.components && prop.components.length > 1){
                        for(j = i + 1; j < suspForComp.length; j++){
                            prop2 = suspForComp[j];
                            if(prop2.hasOwnProperty("components") && prop2.components && prop2.components.length == 1) {
                                var index:int = prop.components.indexOf(prop2.components[0]);
                                if (index != -1) {
                                    prop2.settings = prop.settings;
                                }
                            }
                        }
                    }
                }

                var ln:int = suspForComp.length;
                for(i = 0; i < ln; i++){
                    prop = suspForComp[i];
                    if(prop.hasOwnProperty("components") && prop.components && prop.components.length > 1){
                        for(j = prop.components.length - 1; j > 0; j--){
                            prop2 = {components:[prop.components[j]], settings: prop.settings};
                            suspForComp.push(prop2);
                        }
                    }
                }
                //
                car.suspensionComponentsArray = suspForComp;
            }else {
                car.suspensionForComponents =  carJSON.suspensionForComponentsStructure;
            }
        }

        if (carJSON.hasOwnProperty("newMask")) {
            car.isNewMask = carJSON.newMask;
        }
        else {
            car.isNewMask = false;
        }


        if (carJSON.hasOwnProperty("carChooserBgName")) {
            car.carChooserBgName = carJSON.carChooserBgName;
        }
        else {
            car.carChooserBgName = null;
        }






        if (carJSON.hasOwnProperty("startFrame")) {
            car.startFrame = carJSON.startFrame;
        }
        else {
            car.startFrame = 0;
        }



        if(carJSON.hasOwnProperty("backgrounds")){
            car.backgrounds = carJSON.backgrounds;
        }

        if(carJSON.hasOwnProperty("partMaterialZip")){
            car.partMaterialZip=carJSON.partMaterialZip;
        }else{
            car.partMaterialZip = false;
        }

        ZipManager.getInctance().useForThisCar = car.partMaterialZip;

        if (carJSON.hasOwnProperty('haveSound')) {
            car.haveSound = carJSON.haveSound;
        } else{
            car.haveSound = false;
        }

        if (carJSON.hasOwnProperty('haveSoundRevs')) {
            car.hasSoundRevs = carJSON.haveSoundRevs;
        }

        if(carJSON.hasOwnProperty("partMaterialEncrypted")){
            car.partMaterialEncrypted=carJSON.partMaterialEncrypted;
        }else{
            car.partMaterialEncrypted = false;
        }

        ZipManager.getInctance().isEncrypted = car.partMaterialEncrypted;
        ZipManager.getInctance().useZipSplitCam = car.useZipSplitCam;

        if (!carJSON.displayName) {
            car.displayName = car.model;
        } else {
            car.displayName = carJSON.displayName;
        }

        if (car.id == "58981566e847a024876d4d7f") {

            car.isChopper = true;
        }
        if (car.id == "5cb5e47571923820d89c13d8") {
            car.isBeetle = true;
        }
        car.countAngels = 1;

        if (carJSON.hasOwnProperty("frameCount")) {

            car.countFrames = carJSON.frameCount;

        } else {

            car.countFrames = 50;
        }

        //car.countFrames = 50;

        if (carJSON.hasOwnProperty("haveIconFolder")) {

            car.haveIconFolder = carJSON.haveIconFolder;

        } else {

            car.haveIconFolder = false;
        }
        if (car.isBeetle) {
            car.haveIconFolder = true;
        }

        /*if(carJSON.hasOwnProperty("frameStep")){

				car.stepFrames=carJSON.frameStep;
			}else{

				car.stepFrames=2;
			}*/

        if (carJSON.hasOwnProperty("camera2Frames")) {
//{"0": 0, "10": 12, "20": 24, "30": 32, "40": 48}
            //{"0": 4, "12": 20, "24": 30, "36": 46};
            car.camera2Frames = carJSON.camera2Frames;

            var count:int = 0;
            for (var key:String in car.camera2Frames) {
                count++;
            }

            //car.camera2FramesStep=car.countFrames/count;
        }


        /////////////////////////////////////////////////

        var screenWidth:Number = Constants.STAGE.fullScreenWidth;
        if (car.countFrames == 50) {

            if (!car.camera2Frames) {
                car.camera2Frames = {"0": 0, "10": 12, "20": 24, "30": 32, "40": 48};
                car.stepOfFrameIndexUp = 10;
            }

            if(Device.isIos()){

                if (
                        (Constants.TOTAL_MEMORY>=0 && Constants.TOTAL_MEMORY <= 1024)
                ) {

                    car.camera1Frames = {
                        "0": 0,
                        "5": 4,
                        "10": 10,
                        "15": 14,
                        "20": 20,
                        "25": 24,
                        "30": 30,
                        "35": 34,
                        "40": 40,
                        "45": 44
                    };

                    car.stepOfFrameIndex = 5;
                    car.stepOfFrameIndexUp = 10;
                } else {
                    car.stepOfFrameIndex = 2;
                    car.stepOfFrameIndexUp = 10;
                }

            }else{


                if (
                        (screenWidth==1920 && Constants.TOTAL_MEMORY>=0 && Constants.TOTAL_MEMORY <= 2048) ||
                        (Constants.TOTAL_MEMORY>=0 && Constants.TOTAL_MEMORY < 1538)
                ) {

                    car.camera1Frames = {
                        "0": 0,
                        "5": 4,
                        "10": 10,
                        "15": 14,
                        "20": 20,
                        "25": 24,
                        "30": 30,
                        "35": 34,
                        "40": 40,
                        "45": 44
                    };

                    car.stepOfFrameIndex = 5;
                    car.stepOfFrameIndexUp = 10;

                }
                else {

                    car.stepOfFrameIndex = 2;
                    car.stepOfFrameIndexUp = 12;
                }
            }


            car.camera1LayerPositionStep = 2;
            car.camera2LayerPositionStep = 12;


        } else if (car.countFrames == 48) {

            if (Device.isIos()) {
                if (Constants.TOTAL_MEMORY <=1024) {
                    car.stepOfFrameIndex = 4;
                    car.stepOfFrameIndexUp = 12;
                } else {

                    car.stepOfFrameIndex = 2;
                    car.stepOfFrameIndexUp = 12;
                }
            } else {


                if (screenWidth > 2100) {
                    car.stepOfFrameIndex = 2;
                    car.stepOfFrameIndexUp = 12;
                }
                else {
                    if (
                            (/*screenWidth==1920 &&*/ Constants.TOTAL_MEMORY>1200 && Constants.TOTAL_MEMORY <= 2048) ||
                            (Constants.TOTAL_MEMORY>1200 && Constants.TOTAL_MEMORY < 1538)
                    ) {

                        car.stepOfFrameIndex = 4;
                        car.stepOfFrameIndexUp = 12;

                    } else if(Constants.TOTAL_MEMORY<=1200){

                        car.stepOfFrameIndex = 4;
                        car.stepOfFrameIndexUp = 12;


                    }
                    else {

                        car.stepOfFrameIndex = 2;
                        car.stepOfFrameIndexUp = 12;
                    }
                }

            }

            if (!car.camera2Frames) {
                car.camera2Frames = {"0": 4, "12": 20, "24": 30, "36": 46};
                car.stepOfFrameIndexUp = 12;
            }

            car.camera1LayerPositionStep = 2;
            car.camera2LayerPositionStep = 12;

        }


        if (carJSON.hasOwnProperty("layersPositions")) {
            car.layersPositions = carJSON.layersPositions;
        }

        if (carJSON.hasOwnProperty('offsetDifference')) {
            car.offsetDifference = true;
        }

        if(carJSON.hasOwnProperty("camera24Frames") && carJSON.camera24Frames || car.layersPositions.length==48){
            car.camera24Frames = true;
        }


        if(car.camera24Frames){


            car.camera2Frames = {
                "0": 0, "2": 2, "4": 4, "6": 6, "8": 8, "10": 10, "12": 12, "14": 14, "16": 16, "18": 18,
                "20": 20, "22": 22, "24": 24, "26": 26, "28": 28, "30": 30, "32": 32, "34": 34, "36": 36, "38": 38,
                "40": 40, "42": 42, "44": 44, "46": 46
            };

            car.camera1Frames = {
                "0": 0, "2": 2, "4": 4, "6": 6, "8": 8, "10": 10, "12": 12, "14": 14, "16": 16, "18": 18,
                "20": 20, "22": 22, "24": 24, "26": 26, "28": 28, "30": 30, "32": 32, "34": 34, "36": 36, "38": 38,
                "40": 40, "42": 42, "44": 44, "46": 46
            };



            car.stepOfFrameIndex = 2;
            car.stepOfFrameIndexUp = 2;


            car.camera1LayerPositionStep = 2;
            car.camera2LayerPositionStep = 2;

            if(Device.isIos()){

                if ((Constants.TOTAL_MEMORY>0 && Constants.TOTAL_MEMORY <= 1024)) {

                    car.stepOfFrameIndex = 4;
                    car.stepOfFrameIndexUp = 4;
                }

            }else
            {
                if ((screenWidth==1920 && Constants.TOTAL_MEMORY>1200 && Constants.TOTAL_MEMORY <= 2048) ||
                        (Constants.TOTAL_MEMORY>1200 && Constants.TOTAL_MEMORY < 1538) ) {

                    car.stepOfFrameIndex = 4;
                    car.stepOfFrameIndexUp = 4;
                }else if(Constants.TOTAL_MEMORY<=1200){

                    car.stepOfFrameIndex = 6;
                    car.stepOfFrameIndexUp = 6;

                    car.camera2Frames = {
                        "0": 0,
                        "6": 6,
                        "12": 12,
                        "18": 18,
                        "24": 24,
                        "30": 30,
                        "36": 36,
                        "42": 42
                    };

                    car.camera1Frames = {
                        "0": 0,
                        "6": 4,
                        "12": 12,
                        "18": 16,
                        "24": 24,
                        "30": 28,
                        "36": 36,
                        "42": 42
                    };
                }
            }

        }


        if (carJSON.hasOwnProperty("camera12Frames") && carJSON.camera12Frames == true) {

            car.camera12Frames = true;

            car.camera2Frames = {
                "0": 0, "2": 2, "4": 4, "6": 6, "8": 8, "10": 10, "12": 12, "14": 14, "16": 16, "18": 18,
                "20": 20, "22": 22, "24": 24, "26": 26, "28": 28, "30": 30, "32": 32, "34": 34, "36": 36, "38": 38,
                "40": 40, "42": 42, "44": 44, "46": 46
            };

            car.camera1Frames = {
                "0": 0, "2": 2, "4": 4, "6": 6, "8": 8, "10": 10, "12": 12, "14": 14, "16": 16, "18": 18,
                "20": 20, "22": 22, "24": 24, "26": 26, "28": 28, "30": 30, "32": 32, "34": 34, "36": 36, "38": 38,
                "40": 40, "42": 42, "44": 44, "46": 46
            };

            if(Device.isIos()){

                car.stepOfFrameIndex = 4;
                car.stepOfFrameIndexUp = 4;

            }else{
                if(Constants.TOTAL_MEMORY<=1200){
                    car.stepOfFrameIndex = 6;
                    car.stepOfFrameIndexUp = 6;

                    car.camera2Frames = {
                        "0": 0,
                        "6": 4,
                        "12": 12,
                        "18": 16,
                        "24": 24,
                        "30": 28,
                        "36": 36,
                        "42": 44

                    };

                    car.camera1Frames = {
                        "0": 0,
                        "6": 4,
                        "12": 12,
                        "18": 16,
                        "24": 24,
                        "30": 28,
                        "36": 36,
                        "42": 44
                    };
                }else{

                    car.stepOfFrameIndex = 4;
                    car.stepOfFrameIndexUp = 4;
                }
            }


            car.camera1LayerPositionStep = 4;
            car.camera2LayerPositionStep = 4;
        }


        ///////////////////////////////////////////////////////



        if(!car.camera1Frames){


            if(car.countFrames == 50){

                car.camera1Frames = {
                    "0": 0, "2": 2, "4": 4, "6": 6, "8": 8, "10": 10, "12": 12, "14": 14, "16": 16, "18": 18,
                    "20": 20, "22": 22, "24": 24, "26": 26, "28": 28, "30": 30, "32": 32, "34": 34, "36": 36, "38": 38,
                    "40": 40, "42": 42, "44": 44, "46": 46, "48": 48
                };

            }else if(car.countFrames == 48){

                car.camera1Frames = {
                    "0": 0, "2": 2, "4": 4, "6": 6, "8": 8, "10": 10, "12": 12, "14": 14, "16": 16, "18": 18,
                    "20": 20, "22": 22, "24": 24, "26": 26, "28": 28, "30": 30, "32": 32, "34": 34, "36": 36, "38": 38,
                    "40": 40, "42": 42, "44": 44, "46": 46
                };
            }
        }


        if (carJSON.carchooserOrder && carJSON.carchooserOrder['US'])
            car.carchooserOrder = carJSON.carchooserOrder['US'];
        else
            car.carchooserOrder = carJSON.carchooserOrder;
        if (carJSON.hasOwnProperty('hdsize')){
            car.hdsize = carJSON.hdsize;
        }
        else{
            car.hdsize = false;
        }
        if(carJSON.hasOwnProperty("custom_offset"))
        {
            car.carType = carJSON.custom_offset;
        }


        if (carJSON.hasOwnProperty("features")) {
            car.carFeaturesNew = carJSON.features['new'];
            car.carFeaturesFull = carJSON.features['full'];

            car.topCameraIcon = carJSON.features['topviewcamera'];
            car.ultraIcon = carJSON.features['ultrarealistic'];
            car.offsetIcon = carJSON.features['offset'];
            //if(!car.offsetIcon)
            car.stanceIcon = carJSON.features['stance'];


            if (car.topCameraIcon) {
                car.ultraIcon = false;
            }
        } else {
            car.carFeaturesNew = false;
            car.carFeaturesFull = false;
        }


        if (carJSON.hasOwnProperty("allMaterialChangeFallback")) {

            car.pizdec = carJSON.allMaterialChangeFallback;
            //car.isBigSize=carJSON.isBigSize;
        }
        //car.pizdec=true;


        if (carJSON.hasOwnProperty("isBigSize")) {

            car.isBigSize = carJSON.isBigSize;
        }

        if (carJSON.hasOwnProperty("fullsize")) {

            car.fullsize = carJSON.fullsize;
        }


        if (carJSON.hasOwnProperty("defaultBkgMobile")) {

            car.defaultBkgMobile = carJSON.defaultBkgMobile;
        }

        if (carJSON.hasOwnProperty("defaultBkg")) {

            car.defaultBkg = carJSON.defaultBkg;
        }

        if (carJSON.hasOwnProperty("hashTags")) {

            car.hashTags = carJSON.hashTags;
        }

        if (carJSON.hasOwnProperty("size")) {


            car.size = carJSON.size;
        }

        if (BrandsParser.allCarsList && !car.brandImageUrl) {

            if (BrandsParser.allCarsList[car.id])
                car.brandImageUrl = BrandsParser.allCarsList[car.id].brandImageUrl;
        }


        if (carJSON.hasOwnProperty("disks")) {

            car.disks = carJSON.disks;
        }

        if (carJSON.hasOwnProperty("disksPosition")) {

            car.disksPosition = carJSON.disksPosition;

            //Тут нужно еще добавить позиции для таир копируем их пока с wheel
            var newList:Object = {};
            var slot_name:String;

            for (slot_name in car.disksPosition) {
                var new_slot_name:String = slot_name.replace("wheel", "tair");
                //trace("slot_name:", slot_name, "new_slot_name:", new_slot_name);
                newList[new_slot_name] = car.disksPosition[slot_name];
            }

            for (slot_name in newList) {

                car.disksPosition[slot_name] = newList[slot_name];
            }
        }

        //if(carJSON.hasOwnProperty("componentsDependencies"))
        {

            car.componentsDependenciesWheel = [
                {"rear_wheel_left": ["rear_tair_left", "brake_rear_left", "tire_vinyl_rear_left"]},
                {"rear_wheel_right": ["rear_tair_right", "brake_rear_right", "tire_vinyl_rear_right"]},
                {"front_wheel_right": ["front_tair_right", "brake_front_right", "tire_vinyl_front_right"]},
                {"front_wheel_left": ["front_tair_left", "brake_front_left", "tire_vinyl_front_left"]}
            ]

            //Временно выставляем данные зависемостей правильно
        }


        if (carJSON.hasOwnProperty("componentsDependencies")) {

            car.componentsDependencies = carJSON.componentsDependencies;
        }


        if (carJSON.images) {
            car.imageURL = carJSON.images.side; //main;
            car.featuredImageUrl = carJSON.images.slides2[0];
        }

        car.path = carJSON.folderName + "/";

        var imagesPath:String = Constants.IMAGES_URL + encodeURI(car.path);

        car.imagePath = imagesPath;
        car.path = "Cars/" + encodeURI(car.path);

        return imagesPath;
    }



    private function parseColors():void {
        var defaultColors:Array = _carJSON.factoryColors;

        if (defaultColors) {
            _loadingCar.addPossiblePartColor(PartColor.customColor());

            var defaultColorsLength:int = defaultColors.length;
           // var isWhiteAppeared:Boolean = false;
            for (var i:int = 0; i < defaultColorsLength; i++) {
                var colorJSON:Object = defaultColors[i];

                var partColor:PartColor = new PartColor();
                var colorValue:uint = uint(parseInt(colorJSON.value, 16));

                partColor.name = colorJSON.name;
                partColor.value = colorValue == 0xFFFFFF ? 0xFEFFFF : colorValue;
                /*if (partColor.value == 0xfeffff) {
                    isWhiteAppeared = true;
                    continue;
                }*/
                _loadingCar.addPossiblePartColor(partColor);
            }

            /*
            if (!isWhiteAppeared) {
                _loadingCar.addWhiteColor();
            }
            */
        }
    }


    override public function parseObjects(objects:Object, objectHandler:Function = null):Array {
        var result:Array = super.parseObjects(objects, objectHandler);
        for(var i:int = result.length-1; i >=0; i--){
            if(Car(result[i]).carType == "wheel_only"){
                wheelOnlyList[Car(result[i]).id] = true;
                result.removeAt(i);
            }
        }
        return result;
    }

    public override function parseObject(carJSON:Object, key:String, ...args):Object {
        if(!carJSON){
            Console.log("empty carJSNO");
            return null;
        }
        var componentsJSON:Object = args[0];
        var tuningJSON:Object = args[2];

        var car:Car = new Car();
        var imagesPath:String = parseGeneralInfoAndGetImagesPath(car, carJSON);

        if (carJSON.hasOwnProperty("type")) {
            car.type = carJSON.type;
        } else {
            car.type = "";
        }

        if(carJSON.hasOwnProperty("carType") && carJSON.carType == "wheel_only"){
            car.wheelConfigurator = true;
        }else if (carJSON.hasOwnProperty("wheelConfigurator")) {
            car.wheelConfigurator = carJSON.wheelConfigurator;
        }else{
            car.wheelConfigurator = CarsParser.wheelOnlyList[carJSON.id];
        }

        if (tuningJSON && tuningJSON.suspension) {
            car.suspension = tuningJSON.suspension * -1;

        }

        if (carJSON.hasOwnProperty("updateText")) {
            car.updateText = carJSON.updateText;
        }

        if(carJSON.hasOwnProperty("suvTires"))
        {
            car.suvTires = carJSON.suvTires;
        }

        if(carJSON.hasOwnProperty("tireMasks"))
        {
            car.tireMasks = carJSON.tireMasks;
        }

        if(carJSON.hasOwnProperty("carType"))
        {
            car.carType = carJSON.carType;
        }



        if (carJSON.hasOwnProperty("defaultBgId")) {
            car.defaultBgId = carJSON.defaultBgId;
        }

        // "updated": "2015-08-21T16:30:55.300Z",
        if (carJSON.hasOwnProperty("updated")) {

            var updated:String = carJSON.updated;


            updated = updated.replace("T", " ");
            updated = updated.replace("Z", " ");
            //2015-08-20 21:06:07.456
            updated = updated.substr(0, 19);

            updated = updated.replace("-", "/");
            updated = updated.replace("-", "/");


            var update_date:Date = new Date();

            update_date.setTime(Date.parse(updated));


            car.updated = update_date.time;

        }

        return car;
    }

    public function parseObjectAsync(carJSON:Object, key:String, ...args):Object {
        _carJSON = carJSON;
        _componentsJSON = args[0];
        _tuningJSON = args[2];
        _parserProgressCallback = args[3];
        _parserCompleteCallback = args[4];


        setTimeout(parseCarJson, 20);




        return null;
    }


    private function parseCarJson():void{

        trace("parseCarJson");

        _loadingCar = new Car();

        _imagesPath = parseGeneralInfoAndGetImagesPath(_loadingCar, _carJSON);


        if (_carJSON.hasOwnProperty("type")) {

            _loadingCar.type = _carJSON.type;
        } else {

            _loadingCar.type = "";
        }

        if (_tuningJSON && _tuningJSON.suspension) {
            _loadingCar.suspension = _tuningJSON.suspension * -1;

        }

        if(_carJSON.hasOwnProperty("suvTires"))
        {
            _loadingCar.suvTires = _carJSON.suvTires;
        }

        if(_carJSON.hasOwnProperty("tireMasks"))
        {
            _loadingCar.tireMasks = _carJSON.tireMasks;
        }

        if(_carJSON.hasOwnProperty("carType"))
        {
            _loadingCar.carType = _carJSON.carType;
        }

        if (_carJSON.hasOwnProperty("updateText")) {
            _loadingCar.updateText = _carJSON.updateText;
        }

        if (_carJSON.hasOwnProperty("defaultBgId")) {
            _loadingCar.defaultBgId = _carJSON.defaultBgId;
        }

        // "updated": "2015-08-21T16:30:55.300Z",
        if (_carJSON.hasOwnProperty("updated")) {

            var updated:String = _carJSON.updated;
            updated = updated.replace("T", " ");
            updated = updated.replace("Z", " ");
            //2015-08-20 21:06:07.456
            updated = updated.substr(0, 19);

            updated = updated.replace("-", "/");
            updated = updated.replace("-", "/");


            var update_date:Date = new Date();

            update_date.setTime(Date.parse(updated));


            _loadingCar.updated = update_date.time;

        }


        parseColors();



        setTimeout(startParseComponents, 20);

    }



    private function startParseComponents():void{

        trace("startParseComponents");

        if (_componentsJSON) {
            PartMaterial.ResetPartMaterialImageMem();
            //_loadingCar, _imagesPath, _carJSON, componentsJSON, tuningJSON
            parseComponents();
        }

    }

    private function parseFiles(part:Part, addedPartSlot:PartSlot, car:Car, componentPath:String, componentJSON:Object):void {

        var partMaterialImage:PartMaterialImage;
        var shadowOffset:Number = 0;
        var imageURLPrefix:String;
        var filesKey:String;
        var partMaterial:PartMaterial;
        var imageURL:String;

        for (filesKey in componentJSON.files) {
            partMaterial = new PartMaterial(part);
            partMaterial.name = filesKey;

            imageURLPrefix = componentPath;
            imageURLPrefix += partMaterial.name;
            imageURLPrefix += "/";


            partMaterial.componentPath = imageURLPrefix;

            partMaterial.parentPart = part;

            shadowOffset = 0;
            if (addedPartSlot.name === 'shadow') {
                shadowOffset = -SHADOW_OFFSET_X;
            }

            part.addPossibleMaterial(partMaterial);

            partMaterial.isPartMaterialImagesLoaded = false;

            if(Constants.COMPONENT_OPTIMIZE && !defaultComponentMap[part.id]) continue;

            for each(imageURL in componentJSON.files[filesKey]) {
                partMaterial.addImage(imageURL, car.stepOfFrameIndex, shadowOffset);
            }

            partMaterial.isPartMaterialImagesLoaded = true;


        }

        if(partMaterial){
            if (addedPartSlot.name == "front_bumper") {
                _existsFrontBamper = true;
                partMaterialImage = partMaterial.getImage(0);


                if(partMaterialImage)
                    car.frontBumperX = partMaterialImage.x;
            }

            if (addedPartSlot.name == "black") {
                if(!partMaterialImage)
                    partMaterialImage = partMaterial.getImage(0);

                if(partMaterialImage)
                    _shadowBlackX = partMaterialImage.x;
            }
        }

        partMaterialImage = null;

    }



    private function parseFilesLinked(part:Part, addedPartSlot:PartSlot, car:Car, componentPath:String, componentJSON:Object):void {

        var partMaterialImage:PartMaterialImage;
        var partMaterial:PartMaterial;
        var filesKey:String;
        var imageURLPrefix:String;
        var shadowOffset:Number;
        var imageURL:String;


        for (filesKey in componentJSON.files) {

            partMaterial = part.getPossibleMaterial(filesKey)

            if (!partMaterial) {

                partMaterial = new PartMaterial(part);


                partMaterial.name = filesKey;
                //partMaterial.isPaintable = filesKey != "metal";


                part.addPossibleMaterial(partMaterial);

            }

            imageURLPrefix = componentPath;

            imageURLPrefix += partMaterial.name;
            imageURLPrefix += "/";

            partMaterial.componentPath = imageURLPrefix;

            shadowOffset = 0;
            if (addedPartSlot.name === 'shadow' || addedPartSlot.name.indexOf('shadow_') > -1) {
                shadowOffset = -SHADOW_OFFSET_X;
            }

            //part.imagesUrlBytesMaterialStart(partMaterial.name);

            for each(imageURL in componentJSON.files[filesKey]) {

                partMaterial.addImage(imageURL, car.stepOfFrameIndex, shadowOffset);
            }

        }


        if (addedPartSlot.name == "front_bumper") {
            _existsFrontBamper = true;
            partMaterialImage = partMaterial.getImage(0);

            if(partMaterialImage)
                car.frontBumperX = partMaterialImage.x;
        }

        if (addedPartSlot.name == "black") {
            partMaterialImage = partMaterial.getImage(0);

            if(partMaterialImage)
                _shadowBlackX = partMaterialImage.x;
        }
    }

    private var defaultComponentMap:Object;


    private function parseComponentsOld():void {
        //trace("parseComponents")
        _existsFrontBamper = false;

        _parsedComponents = 0;
        _totalComponents = _componentsJSON.length;

        _partsMap = new Dictionary();
        _usedPartSlotsMap = {};



        parseContinue();
    }

    private function parseComponents():void {
        //trace("parseComponents")
        _existsFrontBamper = false;

        _parsedComponents = 0;
        _totalComponents = _componentsJSON.length;

        _partsMap = new Dictionary();
        _usedPartSlotsMap = {};

        defaultComponentMap = {};

        trace("ZIP CAR");
        PartSlot.VINYLS = [];
        if(_carJSON.components){
            for(var i:int = 0; i < _carJSON.settings.length; i++){
                var componentInfoJSON:Object = _carJSON.settings[i];
                if(componentInfoJSON && componentInfoJSON.componentType==PartSlot.VINYL_GROUP){
                    if(componentInfoJSON.hasOwnProperty("grouping")) {
                        var str:String = componentInfoJSON.grouping;
                        str = str.replace(/\s+/gms, '');
                        PartSlot.VINYLS = str.split(',');
                        PartSlot.VINYLS.push(componentInfoJSON);
                        break;
                    }
                }
            }
        }


        if(Constants.COMPONENT_OPTIMIZE){

            var selectedComponents:Array = _tuningJSON ? _tuningJSON.components : _carJSON.components;
            var defaultComponentJSON:Object;

            for each(defaultComponentJSON in selectedComponents){

                if(!defaultComponentJSON) continue;

                defaultComponentMap[defaultComponentJSON.component] = true;
            }
        }

        parseContinue();
    }

    private function parseContinue():void{

        //trace("parseContinue");

        if(_parsedComponents%20==0){
            //trace("progress:", _parsedComponents/_totalComponents);
            this._parserProgressCallback( _parsedComponents/_totalComponents)
            setTimeout(parseNextComponent, 1);
        }else{

            parseNextComponent();
        }
    }

    private function parseNextComponent():void{

        //trace("parseNextComponent")

        var addedPartSlot:PartSlot;
        var partSlotDisk:PartSlotDisk;
        var part:Part;
        var isDiskSubpart:Boolean = false;
        var nested:Object;
        var partImageFileName:String;
        var componentPath:String;
        var partDisk:PartDisk;
        var maskPartMaterial:PartMaterial;



        if(_componentsJSON.length<=0){

            this.completeComponentsParse();
            return;
        }

        var componentJSON:Object = _componentsJSON.pop();
        if (componentJSON.type == "podium") {
            parseContinue();
            return;
        }

        if (componentJSON.type == "front_bumper") {
            trace();
        }

        addedPartSlot = _usedPartSlotsMap[componentJSON.type];

        if (!addedPartSlot) {
            if (Car.DISKS_SLOTS.indexOf(componentJSON.type) >= 0) { //Для слота "Диски" используем расширеную модель

                partSlotDisk = new PartSlotDisk(_loadingCar);
                partSlotDisk.parentCar = _loadingCar;
                addedPartSlot = partSlotDisk;
            } else {
                addedPartSlot = new PartSlot(_loadingCar);
                addedPartSlot.parentCar = _loadingCar;
            }
            addedPartSlot.name = componentJSON.type;

            _usedPartSlotsMap[componentJSON.type] = addedPartSlot;
        }

        if (componentJSON.type == "packs") {
            isDiskSubpart = false;
            for each(nested in componentJSON.nesteds) {

                if (nested.folderName == "disks") {
                    isDiskSubpart = true;
                    break;
                }

            }

            if (componentJSON.additionalTypes.indexOf('disks') < 0 && isDiskSubpart) {

                componentJSON.additionalTypes.push('disks');
            }

        } //Тут пока не понятно как сделать

        if (Car.DISKS_SLOTS.indexOf(componentJSON.type) >= 0) {

            part = new PartDisk(addedPartSlot);
        } else {

            part = new Part(addedPartSlot);
        }
        part.name = componentJSON.name;
        if(componentJSON.hasOwnProperty('removeTypes') && componentJSON.removeTypes && componentJSON.removeTypes.length){
            part.removeTypes = componentJSON.removeTypes;
        }

        if(componentJSON.hasOwnProperty("universalIcons") ){
            part.universalIcons = componentJSON.universalIcons;
        }else{
            part.universalIcons = false;
        }



        part.excludedPartSlotsNames = componentJSON.additionalTypes;



        if (componentJSON.hasOwnProperty("sortKey")) {

            part.sortKey = parseInt(componentJSON.sortKey);
        }

        if (part.name.charAt(part.name.length - 1) == " ") {
            part.name = part.name.substr(0, part.name.length - 1);
        }
        part.id = componentJSON._id;
        part.parentSlot = addedPartSlot;
        part.folder = encodeURI(componentJSON.folderName);

        if (part.parentSlot.name == "body" && _bodyId == null) {
            _bodyId = part.id;
        }
        if (componentJSON.hasOwnProperty("manufacturer")) {
            if (componentJSON.manufacturer == '5508389948c99bfc7e23fe0c') {
                part.manufacturer = null;
            } else {
                part.manufacturer = componentJSON.manufacturer;
            }
        }
        //var partImageFileName : String = componentJSON.folderName == "default" ? "default" : part.folder;
        //!!!!!!!part.isDefault = (componentJSON.folder == "default");
        part.isDefault = (componentJSON.folderName == "default");

        partImageFileName = part.isDefault ? "default" : part.folder;


        part.imageUrl = "";
        if(componentJSON.hasOwnProperty("typeFolderName")){
            part.parentSlotName = componentJSON.typeFolderName;
            part.imageUrl += componentJSON.typeFolderName;
        }else{
            part.parentSlotName = part.parentSlot.name;
            part.imageUrl += part.parentSlot.name;
        }
        part.imageUrl += "/";
        part.imageUrl += componentJSON.folderName;
        part.imageUrl += "/";

        componentPath = part.imageUrl;

        /*	if(car.id=="537f31ff984539401f0ad004"){

        part.imageUrl=part.imageUrl.replace("/cars/", "/cars2/");
        }*/


        part.imageUrl += partImageFileName;
        part.imageUrl += "_big.png";

        if (part.parentSlot.name == "body") {
            part.imageUrl = Constants.IMAGES_URL + "body_big.png";
        }else if (part.parentSlot.name == "windows") {
            //trace("windows:", part.remoteUrl)
            part.imageUrl = Constants.IMAGES_URL + "windows_big.png";
            trace("windows:",part.remoteUrl)
        }
        if(part.vinylIconUrl){
            part.imageUrl = "cars/" + part.vinylIconUrl;
        }
        /*else if (part.parentSlot.name == "windows") {
            part.imageUrl = Constants.IMAGES_URL + "windows_big.png";
        }*/

        part.imageUrl += "?v=1";

        if (componentJSON.hasOwnProperty("properties") && componentJSON.properties != null) {
            part.properties = componentJSON.properties;
        }

        if (Car.DISKS_SLOTS.indexOf(componentJSON.type) >= 0) {  //Диски обрабатываются расширеной моделью

            partDisk = part as PartDisk;

            partDisk.selectedDiskSize = 0;

            partDisk.radius = 0;// Если радиус не задан ставим нуль
            if (part.properties && part.properties.hasOwnProperty('size') && part.properties.size) {
                partDisk.radius = part.properties.size;
            }
        }

        if (componentJSON.hasOwnProperty("affectedComponents")) {
            part.affectedSlots = componentJSON.affectedComponents;
        } else {

            part.affectedSlots = [];
        }

        _partsMap[part.id] = part;
        addedPartSlot.addPossiblePart(part);


        if(part.id=="59a3d4516145a3f35d201c72"){

            trace("EMPTY COLOR");
        }

        if(componentJSON.hasOwnProperty("multiselect")){
            part.multiselect = componentJSON.multiselect;
            if(part.multiselect) trace("multiselect true");
        }

        if(componentJSON.hasOwnProperty("activeForInformation")){
            part.activeForInformation = componentJSON.activeForInformation;
        }

        parseFiles(part, addedPartSlot, _loadingCar, componentPath, componentJSON);


        parseSubparts(part, componentJSON, componentPath, _loadingCar);


        parseLinked(part, componentJSON);


        parseStates(part, componentJSON, componentPath, _loadingCar);

        //part.states-


        if (part.getPossibleMaterial("mask") != null) {


            addedPartSlot = _usedPartSlotsMap['masks'];

            if (!addedPartSlot) {
                addedPartSlot = new PartSlot(_loadingCar);
                addedPartSlot.parentCar = _loadingCar;
                addedPartSlot.name = 'masks';
                _usedPartSlotsMap['masks'] = addedPartSlot;
                addedPartSlot.isEditable = false;
                addedPartSlot.layerIndex = 200;

                addedPartSlot.selectedColor = PartColor.noColor();
            }
            maskPartMaterial = part.getPossibleMaterial("mask");


            if (Car.DISKS_SLOTS.indexOf(part.parentSlot.name) >= 0) {

                part = new PartDisk(addedPartSlot);
                part.name = maskPartMaterial.parentPart.name;
                (part as PartDisk).radius = (maskPartMaterial.parentPart as PartDisk).radius;

                (part as PartDisk).selectedDiskSize = (maskPartMaterial.parentPart as PartDisk).selectedDiskSize;
                part.id = maskPartMaterial.parentPart.id + "_";
                part.folder = maskPartMaterial.parentPart.folder;
                part.nameOfState = "";
                part.maskForSlots = maskPartMaterial.parentPart.affectedSlots;
                part.imageUrl = "";
                _partsMap[part.id] = part;

                //part.imagesUrlBytes = maskPartMaterial.parentPart.imagesUrlBytes;

                addedPartSlot.addPossiblePart(part);
                //part.nameOfState=
                part.addPossibleMaterial(maskPartMaterial);

                part.selectedMaterial = maskPartMaterial;



            } else {

                part = new Part(addedPartSlot);

                part.id = maskPartMaterial.parentPart.id + "_";
                part.folder = maskPartMaterial.parentPart.folder;
                part.name = maskPartMaterial.parentPart.name;
                part.nameOfState = "";
                part.maskForSlots = maskPartMaterial.parentPart.affectedSlots;
                part.imageUrl = "";
                _partsMap[part.id] = part;

                //part.imagesUrlBytes = maskPartMaterial.parentPart.imagesUrlBytes;


                addedPartSlot.addPossiblePart(part);
                //part.nameOfState=
                part.addPossibleMaterial(maskPartMaterial);

                part.selectedMaterial = maskPartMaterial;


            }


        }


        //if(!existsFrontBamper)
        {

            _loadingCar.frontBumperX = _shadowBlackX;
        }


        _parsedComponents++;

        parseContinue();

    }


    private function completeComponentsParse():void{
        trace("completeComponentsParse")

        var maskKey:int = 0 ;
        var masksJSON:Object;
        var addedPartSlot:PartSlot;
        var part:Part;
        var baseUrl:String;
        var filesKey:String;
        var partMaterial:PartMaterial;
        var linkedPartMaterial:PartMaterial;
        var emptyPartMaterial:PartMaterial;
        var imageURLPrefix:String;
        var imageURL:String;
        var partSlotName:String;
        var partSlot:PartSlot;
        var i:int;
        var z:int;
        var j:int;
        var linkedPart:Part;
        var find:Boolean = false;

        var frameIndex:int;
        var nextFrameIndex:int = 2000;
        var slotNames:Array;


        //Тут парсим маску как дополнительный слот, если она есть
        if (_carJSON.hasOwnProperty("masks")) {
            for (maskKey in _carJSON.masks) {
                masksJSON = _carJSON.masks[maskKey];
                addedPartSlot = _usedPartSlotsMap['masks'];
                if (!addedPartSlot) {
                    addedPartSlot = new PartSlot(_loadingCar);
                    addedPartSlot.parentCar = _loadingCar;

                    addedPartSlot.layerIndex = 200;
                    addedPartSlot.name = 'masks';
                    _usedPartSlotsMap['masks'] = addedPartSlot;
                    addedPartSlot.isEditable = false;

                    addedPartSlot.selectedColor = PartColor.noColor();
                }

                part = new Part(addedPartSlot);

                part.id = masksJSON._id;
                part.folder = encodeURI(masksJSON.folderName);
                part.name = masksJSON.state;
                part.nameOfState = masksJSON.state;
                part.maskForSlots = masksJSON.componentTypes;

                part.imageUrl = "";

                _partsMap[part.id] = part;
                addedPartSlot.addPossiblePart(part);

                baseUrl = '';
                baseUrl += "masks/disks/";
                baseUrl += part.nameOfState;
                baseUrl += "/";

                for (filesKey in masksJSON.files) {
                    partMaterial = new PartMaterial(part);
                    partMaterial.name = filesKey;

                    //partMaterial.isPaintable = PartMaterial.isColorable(filesKey);

                    imageURLPrefix = baseUrl;
                    imageURLPrefix += partMaterial.name;
                    imageURLPrefix += "/";

                    partMaterial.componentPath = imageURLPrefix;

                    //part.imagesUrlBytesMaterialStart(partMaterial.name);

                    for each(imageURL in masksJSON.files[filesKey]) {
                        partMaterial.addImage(imageURL, _loadingCar.stepOfFrameIndex);
                    }

                    //part.imagesUrlBytesMaterialEnd(partMaterial.name);

                    part.addPossibleMaterial(partMaterial);

                    if (!part.selectedMaterial)
                        part.selectedMaterial = partMaterial;
                }
            }
        }
        //Тут маска и заканчивается


        for (partSlotName in _usedPartSlotsMap) {
            partSlot = _usedPartSlotsMap[partSlotName] as PartSlot;
            partSlot.afterParseAllSlot();
        }




        parseSelectedComponents();

        var linked_id:String;
        var partId:String;
        var bodyPart:Part = _partsMap[_bodyId];
        for (partId in _partsMap) {
            part = _partsMap[partId] as Part;


            if (part && part.parentSlot && part.parentSlot.selectedColor && part.parentSlot.selectedColor.value == 0xFEFFFF) {
                if (part.selectedMaterial && part.selectedMaterial.name != "metal" && part.parentSlot.name != "windows")
                    part.parentSlot.selectedColor.value = bodyPart.parentSlot.selectedColor.value;
            }else if(part && part.parentSlot && !part.parentSlot.selectedColor){


            }




            //Ищем и добавляем в linkedParts

            if (part && part.linked && part.linked.length > 0) {

                for (i = 0; i < part.linked.length; i++) {

                    linked_id = part.linked[i];

                    if (_partsMap[linked_id] && part.linkedParts.indexOf(_partsMap[linked_id]) < 0 ) {

                        if (part.possibleMaterials.length == 0) {
                            linkedPart = _partsMap[linked_id];

                            for (z = 0; z < linkedPart.possibleMaterials.length; z++) {

                                linkedPartMaterial = linkedPart.possibleMaterials[z];

                                if (!part.getPossibleMaterial(linkedPartMaterial.name)) {

                                    emptyPartMaterial = new PartMaterial(part);

                                    emptyPartMaterial.name = linkedPartMaterial.name;

                                    //emptyPartMaterial.isPaintable = linkedPartMaterial.isPaintable;

                                    emptyPartMaterial.componentPath = "";

                                    part.addPossibleMaterial(emptyPartMaterial);

                                }

                            }

                        }

                        part.linkedParts.push(_partsMap[linked_id]);
                        (_partsMap[linked_id] as Part).parentLinkedParts.push(part);
                    }
                }
            }






            var newExcludePartSlot:Array = [];
            if (part.excludedPartSlotsNames) {
                for (j = 0; j < part.excludedPartSlotsNames.length; j++) {
                    find = false;
                    for (z = 0; z < part.linkedParts.length; z++) {
                        linkedPart = part.linkedParts[z];
                        if (linkedPart.parentSlot.name == part.excludedPartSlotsNames[j]) {

                            find = true;
                            break;
                        }
                    }
                    if (!find) {
                        newExcludePartSlot.push(part.excludedPartSlotsNames[j]);
                    }
                }
                if (newExcludePartSlot.length >= 1) {
                    trace('111')
                }
                part.excludedPartSlotsNames = newExcludePartSlot;
            }
        }



        if (_loadingCar.layersPositions && _loadingCar.layersPositions.length > 0) {
            for (i = 0; i < _loadingCar.layersPositions.length; i++) {
                slotNames = _loadingCar.layersPositions[i];

                Tracer.print("i: " +  i +  slotNames);
                for (partSlotName in _usedPartSlotsMap) {
                    partSlot = _usedPartSlotsMap[partSlotName] as PartSlot;
                    frameIndex = slotNames.indexOf(partSlot.name);
                    if (partSlot.name == "mask") frameIndex = 2500;
                    if (frameIndex < 0) {
                        frameIndex = nextFrameIndex;
                    } else {
                        frameIndex = frameIndex * 10;
                    }
                    partSlot.layerIndexForFrames.push(frameIndex);
                }
            }

            for (partSlotName in _usedPartSlotsMap) {
                partSlot = _usedPartSlotsMap[partSlotName] as PartSlot;
            }
        }


        //_partsMap = null;

        /*if(car.isChopper)*/

        _parserProgressCallback(1.025);

        this.prepareAll(_loadingCar);
        updateExcludeForTuning(_loadingCar, _bodyId, _usedPartSlotsMap,_partsMap,_carJSON, _tuningJSON);

        MemoryTracker.gcAndCheck("CarsParser.parseComponents()");

        _carJSON = null;
        if(_componentsJSON){
            _componentsJSON.length=0;
            _componentsJSON = null;
        }

        _tuningJSON = null;
        _imagesPath = null;
        _existsFrontBamper = false;
        _shadowBlackX = 0;

        _usedPartSlotsMap = {};

        _partsMap = null;

        _bodyId = null;



        _parserCompleteCallback(_loadingCar);

        _parserProgressCallback = null;
        _parserCompleteCallback = null;
        _loadingCar = null;

    }

    private function updateExcludeForTuning(car:Car, bodyId:String, usedPartSlotsMap:Object, partsMap:Dictionary, carJSON:Object, tunningJSON:Object):void{

        //return;
        if(!tunningJSON) return;

        var partSlots:Array = car.possiblePartSlots;
        var partSlot:PartSlot;
        var component:Object;
        var selPart:Part
        var bodyPart:Part = partsMap[bodyId];
        for each(partSlot in partSlots){

            //carJSON.components



            if (partSlot.selectedPart && partSlot.selectedPart.excludedPartSlotsNames) {
                for each(var excludedPartSlotName:String in partSlot.selectedPart.excludedPartSlotsNames) {
                    var excludedPartSlot:PartSlot = car.getPossiblePartSlot(excludedPartSlotName);


                    if (excludedPartSlot && partSlot.selectedPart.excludedPartSlotsNames.indexOf(excludedPartSlot.name) >= 0) {


                        if (!excludedPartSlot.selectedPart) {


                            selPart = findInArrayByType(partsMap, carJSON.components, excludedPartSlot.name);

                            if(selPart){

                                //trace("ВОССТАНОВИЛИ ДЕФАУЛТ ДЛЯ ",  excludedPartSlot.name)
                                excludedPartSlot.isExcluded = true;

                                excludedPartSlot.oldBeforeExludeSelectedPart = selPart //partsMap[component.component];


                                if(excludedPartSlot.selectedPart && excludedPartSlot.selectedPart.selectedMaterial){

                                    var partMaterial:PartMaterial = selPart.getPossibleMaterial(excludedPartSlot.selectedPart.selectedMaterial.name);

                                    if(partMaterial){
                                        selPart.selectedMaterial =  partMaterial;

                                        if(excludedPartSlot.selectedPart.color){
                                            selPart.color = excludedPartSlot.selectedPart.color;
                                        }

                                        if(!selPart.color && bodyPart){

                                            selPart.color = bodyPart.color.copy();
                                        }

                                    }
                                }



                            }
                        }

                    }
                }
            }


            if (partSlot.selectedPart && partSlot.selectedPart.linkedParts.length > 0) {

                //Tracer.print("Обрабатываем линкеды для восстановления после сохранки")
                var updateSlots:Array = [];
                for (var i:int = 0; i < partSlot.selectedPart.linkedParts.length; i++) {

                    var linkedPart:Part = partSlot.selectedPart.linkedParts[i];

                    //Tracer.print("linkedPart.parentSlot:" + linkedPart.parentSlot.name + linkedPart.parentSlot.savePartLinked);

                    selPart = findInArrayByType(partsMap, carJSON.components, linkedPart.parentSlot.name);

                    if(selPart){
                        //trace("ВОССТАНОВИЛИ ДЕФАУЛТ ДЛЯ ",  linkedPart.parentSlot.name)



                        //trace("selPart:", selPart.name, selPart.id, selPart.parentSlot.name);

                        linkedPart.parentSlot.savePartLinked = selPart;

                        //selPart.pa


                        if(linkedPart.selectedMaterial){
                            selPart.selectedMaterial =  selPart.getPossibleMaterial(linkedPart.selectedMaterial.name);
                            if(selPart.selectedMaterial){
                                if(linkedPart.color){
                                    selPart.color = linkedPart.color;
                                }

                                if(!selPart.color && bodyPart && bodyPart.color){

                                    selPart.color = bodyPart.color.copy();
                                }
                            }

                        }

                    }

                    //car.se
                }
            }

        }
    }

    private function findInArrayByType(partsMap:Dictionary, components:Array, type:String):Part{

        var component:Object = null;
        var part:Part=null;

        for each (component in components){

            part = partsMap[component.component];

            if(!part) continue;

            if(part.parentSlot.name==type) return part;

        }

        return null;
    }

    private function getPartById(part_id:String, car:Car):Part {

        var i:int = 0;

        var partSlot:PartSlot;
        var part:Part;

        if(_partsMap) return _partsMap[part_id];

        for each(partSlot in car.possiblePartSlots) {

            if (partSlot) {

                //for(i=0; i<partSlot.possibleParts.length; i++){
                part = partSlot.getPartById(part_id);

                if (part) return part;

                //}
            }

        }


        return null;
    }


    /*private function getPartsByWide(wide:String, skip_id:String, car:Car):Part{

			var i:int=0;

			var partSlot:PartSlot;
			var part:Part;

			for each(partSlot in car.possiblePartSlots){

				if(partSlot){

					for(i=0; i<partSlot.possibleParts.length; i++){



						part = partSlot.getPartById(part_id);

						if(part) return part;

					}
				}

			}


			return null;
		}
		*/

    private function prepareAll(car:Car):void {

        var i:int = 0;
        var j:int = 0;
        var partSlot:PartSlot;

        var part:Part;
        var newPart:Part;
        var partSlotIsDependence:Boolean;
        var partSlotIsWide:Boolean;
        var depId:String;
        for each(partSlot in car.possiblePartSlots) {

            if (partSlot != null && partSlot.name != "masks") {
                partSlotIsDependence = false;
                partSlotIsWide = false;

                for (j = 0; j < partSlot.possibleParts.length; j++) {
                    part = partSlot.possibleParts[j];
                    if (part.wide) partSlotIsWide = true;
                    if (part.dependence as String) {
                        depId = part.dependence;
                        newPart = getPartById(depId, car);//partSlot.getPartById(depId);

                        if (newPart && newPart.depsMap.indexOf(partSlot.name) < 0) {
                            newPart.depsMap.push(partSlot.name);
                        }
                    } else if (part.dependence.length > 0) {
                        for (i = 0; i < part.dependence.length; i++) {
                            depId = part.dependence[i];
                            newPart = getPartById(depId, car)//partSlot.getPartById(depId);

                            if (newPart && newPart.depsMap.indexOf(partSlot.name) < 0) {
                                newPart.depsMap.push(partSlot.name);
                            }
                        }

                        part.isDependence = true;
                        partSlotIsDependence = true;
                    }
                }

                partSlot.isDependence = partSlotIsDependence;
                partSlot.isWide = partSlotIsWide;
            }
        }


        var rearWheelPartSlot:PartSlot = car.getPossiblePartSlot("rear_wheel");
        if (rearWheelPartSlot) {

            for each(partSlot in car.possiblePartSlots) {
                if (partSlot == rearWheelPartSlot) continue;
                if (!partSlot.isWide) continue;

                for (j = 0; j < rearWheelPartSlot.possibleParts.length; j++) {
                    part = rearWheelPartSlot.possibleParts[j];
                    if (part.depsMap.indexOf(partSlot.name) < 0) {
                        part.depsMap.push(partSlot.name);
                    }
                }
            }
        }
    }


    private function parseLinked(part:Part, componentJSON:Object):void {

        if (!componentJSON.linked)
            return;

        part.linked = componentJSON.linked;
    }


    private function findComponentById(id:String, components:Array):Object {


        for (var i:int = 0; i < components.length; i++) {

            var comp:Object = components[i];

            if (comp._id == id) {
                return comp;
            }
        }

        return null;
    }


    private function orderLastName(a:Object, b:Object):int {
        var a_num:int = a.layerNumber;
        var b_num:int = b.layerNumber;
        if (a_num < b_num) {
            return -1;
        }
        else if (a_num > b_num) {
            return 1;
        }
        else {
            return 0;
        }
    }

    private function parseSelectedComponents():void {

        var car:Car = _loadingCar;
        var bodyId:String = _bodyId;
        var usedPartSlotsMap:Object = _usedPartSlotsMap;
        var partsMap:Dictionary=_partsMap;
        var carJSON:Object = _carJSON;
        var tunningJSON:Object = _tuningJSON;

        var selectedComponents:Array = tunningJSON ? tunningJSON.components : carJSON.components;
        var bodyColor:int = -1;

        var bodyComponent:Object = null;

        var colorString:String;

        var component:Object;

        var partSlot:PartSlot;
        var disksLayerIndex:int;

        var componentInfoJSON:Object;

        var defaultComponentJSON:Object;

        var parentSlot:PartSlot;
        var part:Part;

        if (bodyId == null && selectedComponents && selectedComponents.length > 0) bodyId = selectedComponents[0].component;

        for each(component  in selectedComponents) {

            part =_partsMap[component.component];
            if (part && part.parentSlot.name=="body" /*component.component == bodyId*/) {
                bodyId = part.id;
                bodyComponent = component;
                bodyColor = uint(parseInt(component.color, 16));
                bodyColor = bodyColor == 0xFFFFFF ? 0xFEFFFF : bodyColor;
            }
        }

        var bodyPartColor:PartColor = car.getPossiblePartColor(bodyColor);

        if (!bodyPartColor) {
            bodyPartColor = PartColor.customColor();
            bodyPartColor.name = "color";
            bodyPartColor.value = bodyColor;
        }

        car.bodyPartColor = bodyPartColor.copy();

        if (carJSON.settings) {
            car.maxPartSlotLayerIndex = -1;

            var arrOfSettings:Array = carJSON.settings;

            arrOfSettings.sort(orderLastName);

            var layerNumber:int = 1;
            for each(componentInfoJSON in arrOfSettings) {

                partSlot = usedPartSlotsMap[componentInfoJSON.componentType];
                disksLayerIndex = 0;
                if (partSlot && componentInfoJSON.componentType != "podium") {
                    if (Car.DISKS_SLOTS.indexOf(partSlot.name) >= 0) {
                        layerNumber++;

                    }

                    partSlot.name = componentInfoJSON.componentType;
                    partSlot.displayName = displayNameForComponentType(componentInfoJSON.componentType);
                    partSlot.layerIndex = componentInfoJSON.layerNumber * 10;


                    if (componentInfoJSON.sortKey) {
                        partSlot.sortKey = componentInfoJSON.sortKey;
                    }

                    partSlot.isEditable = componentInfoJSON.isEditable || partSlot.name == "body";
                    partSlot.isMandatory = componentInfoJSON.isMandatory;
                    partSlot.bestViewFrameIndex = int(componentInfoJSON.position/2);
                    partSlot.selectedColor = PartColor.noColor();

                    if (componentInfoJSON.hasOwnProperty('lockSuspension')) {
                        partSlot.lockSuspension = componentInfoJSON.lockSuspension;
                        if(partSlot.isAffectedBySuspension && partSlot.lockSuspension){
                            partSlot.isAffectedBySuspension = !componentInfoJSON.lockSuspension;
                        }
                    }

                    if (componentInfoJSON.hasOwnProperty('unselectableComponent')) {
                        partSlot.unselectableComponent = componentInfoJSON.unselectableComponent;
                    }

                    if (componentInfoJSON.hasOwnProperty('hideFromMenu')) {
                        partSlot.hideFromMenu = componentInfoJSON.hideFromMenu;
                    }


                    car.maxPartSlotLayerIndex = Math.max(car.maxPartSlotLayerIndex, partSlot.layerIndex);

                    if (partSlot.name.indexOf("aerography")>=0) {

                        partSlot.isMandatory = false;
                    }


                    if (bodyColor >= 0 && partSlot.name.indexOf("aerography")<0 && PartSlot.VINYLS.indexOf(partSlot.name)==-1) {
                        partSlot.selectedColor = bodyPartColor.copy();
                    }
                    partSlot.sizeInMegabytes = componentInfoJSON.size ? componentInfoJSON.size : 0.0;

                    car.addPossiblePartSlot(partSlot);
                }
            }

            //Для теста ставим слот Маска
            {
                partSlot = usedPartSlotsMap['masks'];

                if (partSlot) {
                    partSlot.name = "masks";
                    partSlot.layerIndex = 2000;//car.maxPartSlotLayerIndex+5 ;
                    partSlot.isEditable = false;
                    partSlot.isMandatory = false;
                    partSlot.isMask = true;
                    partSlot.isAffectedBySuspension = true;

                    car.maxPartSlotLayerIndex = Math.max(car.maxPartSlotLayerIndex, partSlot.layerIndex);
                    car.maxPartSlotLayerIndex++;
                    car.addPossiblePartSlot(partSlot);
                }
            }
        }


        for each(defaultComponentJSON in selectedComponents) {


            var addedPart:Part = partsMap[defaultComponentJSON.component];

            if (!addedPart)
                continue;

            if (Car.DISKS_SLOTS.indexOf(addedPart.parentSlot.name) >= 0) {

                var partDiskSlot:PartSlotDisk = addedPart.parentSlot as PartSlotDisk;
                var partDisk: PartDisk;

                partDisk = partDiskSlot.findMainPartDiskByName(addedPart.name);
                if (!partDisk) {
                    var prefix:String = PartSlotDisk.getPrefixDep(addedPart);
                    partDisk = partDiskSlot.findMainPartDiskByName(addedPart.name + prefix);
                }

                partDisk ||= partDiskSlot.factoryDiskPart;
                var selectedRadiusPartDisk:PartDisk = addedPart as PartDisk;
                partDisk ||= selectedRadiusPartDisk;
                if(selectedRadiusPartDisk && selectedRadiusPartDisk.id == defaultComponentJSON.component){
                    partDisk.selectedDiskSize = selectedRadiusPartDisk.radius;
                    addedPart = selectedRadiusPartDisk;
                }else if (partDisk) {
                    partDisk.selectedDiskSize = selectedRadiusPartDisk.radius;
                    addedPart = partDisk;
                }
            }

            parentSlot = addedPart.parentSlot;

            try {

                if (Car.DISKS_SLOTS.indexOf(parentSlot.name) >= 0) {

                    addedPart.selectedMaterial = addedPart.getPossibleMaterial(defaultComponentJSON.material);
                } else {

                    if (!addedPart.selectedMaterial) {
                        addedPart.selectedMaterial = addedPart.getPossibleMaterial(defaultComponentJSON.material);
                    } else if (addedPart.selectedMaterial.name != 'glossy') {

                        addedPart.selectedMaterial = addedPart.getPossibleMaterial(defaultComponentJSON.material);
                    }
                }


            } catch (e:Error) {

                Tracer.print("error:", e.message/*, e.getStackTrace()*/);
                Logger.e(e.message, this);

                if (addedPart) {

                } else {
                    continue;
                }
            }

            if (!addedPart.selectedMaterial) {
                addedPart.selectedMaterial = selectPossibleMaterial(addedPart.possibleMaterials);
                if (Car.CAR_DISKS_SLOTS.indexOf(parentSlot.name) >=0) {

                    addedPart.selectedMaterial = selectPossibleMaterial(addedPart.possibleMaterials, parentSlot.name);
                }
            }



            addedPart.brightness = defaultComponentJSON.hasOwnProperty("brightness") ? defaultComponentJSON.brightness : 1.0;

            if (parentSlot.name == "aerography") {

                if (!parentSlot.selectedParts)
                    parentSlot.selectedParts = [];

                if (addedPart.saveIndex == "")
                    addedPart.saveIndex = (Math.random() * 10000000000).toFixed(0);

                colorString = defaultComponentJSON.color;

                if (colorString == null) colorString = "ffffff";

                colorValue2 = uint(parseInt(colorString, 16));

                var color:PartColor = new PartColor();
                color.name = "color";
                color.value = colorValue2 == 0xFFFFFF ? 0xFEFFFF : colorValue2;

                addedPart.color = color;

                parentSlot.selectedParts.push(addedPart);
            } else {
                parentSlot.selectedPart = addedPart;
            }




            car.addSelectedPartSlot(parentSlot);

            addedPart.selectedStates = defaultComponentJSON.states;

            var selectedTheState:Boolean = false;

            if (addedPart.selectedStates) {
                for (var nameOfState:String in addedPart.selectedStates) {

                    var objectOfState:Object = addedPart.selectedStates[nameOfState];

                    var active:Boolean = false;
                    if (objectOfState && objectOfState.hasOwnProperty("active")) {
                        active = objectOfState.active;
                    }

                    if (active) {
                        var partState:Part = addedPart.states[nameOfState];
                        addedPart.selectedState = partState;
                        selectedTheState = true;
                        //обрабатываем только одно (возможно других вариантов и нет)
                        break;
                    }


                }


            }

            colorString = defaultComponentJSON.color;
            if (defaultComponentJSON.material == "metal") {
                colorString = null;
            }

            if (parentSlot.name == "aerography" || PartSlot.VINYLS.indexOf(parentSlot.name)!=-1) {

                if (colorString == null) colorString = "ffffff";
            }


            var colorValue2:uint;

            if (colorString) {
                colorValue2 = uint(parseInt(colorString, 16));

                var selectedPartColor:PartColor = car.getPossiblePartColor(colorValue2);

                if (!selectedPartColor) {
                    selectedPartColor = new PartColor();
                    selectedPartColor.name = "color";
                    selectedPartColor.value = colorValue2 == 0xFFFFFF ? 0xFEFFFF : colorValue2;
                }

                addedPart.color = selectedPartColor.copy();
                parentSlot.selectedColor = selectedPartColor;
            }
            else {
                parentSlot.selectedColor = PartColor.noColor();

                if (parentSlot.selectedPart && parentSlot.selectedPart.selectedMaterial && parentSlot.selectedPart.selectedMaterial.name == "metal") {
                    parentSlot.selectedColor.value = 0xFFFFFF;
                }

                if (bodyColor >= 0 && parentSlot.name != "aerography" && PartSlot.VINYLS.indexOf(parentSlot.name)==-1) {
                    var material:PartMaterial;
                    if (parentSlot.selectedPart) {
                        material = parentSlot.selectedPart.selectedMaterial;
                    }


                    if (!material || material.name != "metal") {
                        parentSlot.selectedColor = bodyPartColor.copy();
                    }

                }

                if (selectedTheState)//костыль (если без костыля то нужно эту логику согласовывать как на сайте делалось)
                {
                    parentSlot.selectedColor.value = 0xFFFFFF;
                }
            }
        }

        var bodyPart:Part = partsMap[bodyId];
        if (bodyComponent) {
            bodyPart.selectedMaterial = bodyPart.getPossibleMaterial(bodyComponent.material);
        }


        //зададим материал для деталей которым не задан материал
        for each(var tempPart:Part in partsMap) {




            if (tempPart.selectedMaterial) continue;

            var tempParentSlot:PartSlot = tempPart.parentSlot;
            if (tempParentSlot.selectedPart && tempParentSlot.selectedPart.selectedMaterial) {
                tempPart.selectedMaterial = tempPart.getPossibleMaterial(tempParentSlot.selectedPart.selectedMaterial.name);
                if (tempPart.selectedMaterial) {
                    continue;
                }
                continue;
            }

            if (bodyPart.selectedMaterial) {

                tempPart.selectedMaterial = tempPart.getPossibleMaterial(bodyPart.selectedMaterial.name);
            }

            if (tempPart.selectedMaterial) {
                continue;
            }


            tempPart.selectedMaterial = selectPossibleMaterial(tempPart.possibleMaterials);


            if (tempPart.selectedMaterial) {
                continue;
            }

        }

        car.arrangePartSlots();
    }

    private function selectPossibleMaterial(materials:Array, partSlotName:String = null):PartMaterial {
        var order:Array;
        //front_lights rear_lights
        if (!partSlotName)
            order = ["glossy", "metal", "mate", "white", "redblack"];
        else if (Car.DISKS_SLOTS.indexOf(partSlotName) >= 0)
            order = ["metal", "glossy", "mate", "white", "redblack"];
        else if (partSlotName == "front_lights")
            order = ["metal", "glossy", "mate", "white", "redblack"];
        else if (partSlotName == "rear_lights")
            order = ["metal", "glossy", "mate", "white", "redblack"];
        else
            order = ["glossy", "metal", "mate", "white", "redblack"];

        for (var i:int = 0; i < order.length; i++) {
            for each(var partMaterial:PartMaterial in materials) {
                if (partMaterial.name == order[i]) {
                    return partMaterial;
                }
            }
        }

        return materials[0];
    }


    private function parseSubparts(part:Part, componentJSON:Object, componentPath:String, car:Car):void {
        var nesteds:Array = componentJSON.nesteds as Array;
        var nested:Object;
        var subpart:Part;
        var filesKey:String;
        var partMaterial:PartMaterial;
        var imageURLPrefix:String;
        var imageURL:String;

        for each(nested in nesteds) {
            subpart = new Part(part.parentSlot);
            subpart.subposition = nested.position;
            subpart.parentSlot = part.parentSlot;
            part.addSubpart(subpart);

            for (filesKey in nested.files) {
                partMaterial = new PartMaterial(part);
                partMaterial.name = filesKey;

                imageURLPrefix = componentPath;

                imageURLPrefix += "nesteds/";
                imageURLPrefix += nested.folderName;
                imageURLPrefix += "/";
                imageURLPrefix += partMaterial.name;
                imageURLPrefix += "/";

                partMaterial.componentPath = imageURLPrefix;

                subpart.addPossibleMaterial(partMaterial);

                partMaterial.isPartMaterialImagesLoaded = false;

                if(Constants.COMPONENT_OPTIMIZE && !defaultComponentMap[part.id]) continue;

                for each(imageURL in nested.files[filesKey]) {
                    partMaterial.addImage(imageURL, car.stepOfFrameIndex);
                }

                partMaterial.isPartMaterialImagesLoaded = true;
            }
        }
    }

    private function getCountState(states:Object):int {

        if (!states)
            return 0;

        var i:int = 0;
        var state:String;
        for (state in states) {

            i++;
        }


        return i;

    }

    private function parseStates(part:Part, componentJSON:Object, componentPath:String, car:Car):void {


        var partMaterialState:PartMaterial;
        var prefixUrl:String;
        var imageURLofState:String;
        var imageURLPrefixOfState:String;
        var filesOfState:String;
        var subpartOfState:Part;
        var type:String;
        var state:Object;
        var nameOfState:String;
        var isTestDisk:Boolean = false;
        var states:Object = componentJSON.states as Object;




        if (states) {//добавим структуру states

            var isReplaceState:Boolean = Car.SlotNamesForStatesMap.indexOf(part.parentSlot.name)>=0;


            for (nameOfState in states) {

                if (nameOfState && !part.existCamber) {

                    if (nameOfState.indexOf('trr') >= 0 || nameOfState.indexOf('trl') >= 0 || nameOfState.indexOf('cm1') >= 0 ||
                            nameOfState.indexOf('cm2') >= 0 || nameOfState.indexOf('cm3') >= 0 || nameOfState.indexOf('cm4') >= 0 ||
                            nameOfState.indexOf('of1') >= 0 || nameOfState.indexOf('of2') >= 0 || nameOfState.indexOf('of3') >= 0 || nameOfState.indexOf('of4') >= 0) {

                        part.existCamber = true;

                        if (!Device.isIos() && Constants.DISABLED_CAMBER_FOR_ANDROID_TRUCK_SUV) {


                            if (car.type == "Truck" || car.type == "Suv") {

                                part.existCamber = false;
                            }
                        }

                    }
                }

                state = states[nameOfState];
                type = state.type;//например addition(фар зажженные) или replace(диски с неоном)

                if (!state.files) {
                    continue;
                }

                subpartOfState = new Part(part.parentSlot);

                if (type == "addition") {
                    subpartOfState.isAdditionState = true;
                }

                //subpartOfState.subposition = nested.position;
                subpartOfState.parentSlot = part.parentSlot;
                part.addState(subpartOfState, nameOfState);

                for (filesOfState in state.files) {
                    partMaterialState = new PartMaterial(subpartOfState);
                    if (!subpartOfState.selectedMaterial) {
                        subpartOfState.selectedMaterial = partMaterialState;
                    }



                    if(isReplaceState && Car.statesMap[nameOfState]){

                        nameOfState = Car.statesMap[nameOfState];
                    }


                    partMaterialState.name = filesOfState;
                    //partMaterialState.isPaintable = filesOfState != "metal";

                    imageURLPrefixOfState = componentPath;


                    imageURLPrefixOfState += "states/";
                    imageURLPrefixOfState += nameOfState;
                    imageURLPrefixOfState += "/";
                    imageURLPrefixOfState += partMaterialState.name;
                    imageURLPrefixOfState += "/";


                    partMaterialState.componentPath = imageURLPrefixOfState;


                    //subpartOfState.imagesUrlBytesMaterialStart(partMaterialState.name);
                    subpartOfState.addPossibleMaterial(partMaterialState);
                    partMaterialState.parentPart = part;

                     if(Constants.COMPONENT_OPTIMIZE  && !defaultComponentMap[part.id] && (part.parentSlot.name != 'tire_front_left' && part.parentSlot.name != 'tire_rear_left')) continue;
                    // TODO (to be discussed)


                    for each(imageURLofState in state.files[filesOfState]) {
                        partMaterialState.addImage(imageURLofState, car.stepOfFrameIndex);
                    }

                    //subpartOfState.imagesUrlBytesMaterialEnd(partMaterialState.name);

                }
            }
        }

    }

    private function displayNameForComponentType(type:String):String {
        var map:Object = {
            "neon": "Neon",
            "shadow": "Shadow",
            "aerography": "Vinilis",
            "spoiler_top": "Top spoiler",
            "mirors": "Mirrors",
            "windows": "Tinted glass",
            "sides": "Sides",
            "exhaus": "Exhaust",
            "air_intake_motor": "Front air intake",
            "fox_eyes": "Fox eyes",
            "front_lights": "Front lights",
            "disks": "Wheel",
            "podium": "Podium",
            "body": "Body",
            "radiator": "Radiator",
            "rear_lights": "Rear lights",
            "motor_cover": "Hood",
            "wings": "Wings",
            "pipe": "Pipe",
            "air_intake_wings": "Wings air intake",
            "protectors": "Protector",
            "air_intake_top": "Top air intake",
            "spoiler_back": "Spoiler",
            "packs": "Packs",
            "black": "Black",
            "rear_bumper": "Rear bumper",
            "front_bumper": "Front bumper"
        };

        return map[type];
    }
}
}

//import com.despark.tuning3d.model.cars;


