package com.despark.tuning3d.model.challenges {
    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.utils.DateParser;

    public class ChallengeData {
        public var views: Number;
        public var start_date: String;
        public var owner: Object;
        public var receiver: Object;
        public var _id: String;
        public var created: String;
        public var status:String;
        private var currentUser: User;
        private var myChallengeVotes: Object;

        public static const FINISHED:String = "finished";
        public static const ACCEPTED:String = "accepted";
        public static const VOTE_MAX_COUNT:int = 10;
        public static const DURATION:Number = 2 * 3600 * 1000; // two hours (in milliseconds)
        function ChallengeData(data: Object) {
            views = data.views;
            start_date = data.start_date;
            owner = data.owner;
            receiver = data.receiver;
            status = data.status;
            _id = data._id;
            created = data.created;
            var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
            currentUser = usersProxy.currentUser;
            var challengesProxy:ChallengesProxy = ApplicationFacade.getInstance().retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            myChallengeVotes = challengesProxy.myChallengeVotes;
            var timeIsComplete:Boolean = DURATION <= (new Date().time - DateParser.parseDate(start_date).time);
            if(status != FINISHED && owner.votes >= VOTE_MAX_COUNT || receiver.votes >= VOTE_MAX_COUNT || timeIsComplete){
                status = FINISHED;
                owner.winner = (owner.votes >= receiver.votes);
                receiver.winner = (receiver.votes >= owner.votes);
            }
        }

        public function canIVote(userType: String): Boolean {
            if(isOwnChallenge){
                return false;
            }

            if (owner.winner || receiver.winner) {
                return false;
            }

            return getAlreadyVoted(userType);
        }

        public function get isOwnChallenge(): Boolean {
            if (currentUser) {
                if ((owner.user.hasOwnProperty("_id") && this.owner.user._id == currentUser.id) || (this.receiver.user.hasOwnProperty("_id") && this.receiver.user._id === currentUser.id)) {
                    return true;
                }
                if ((owner.user is String && this.owner.user == currentUser.id) || (this.receiver.user is String && this.receiver.user == currentUser.id)) {
                    return true;
                }
            }
            return false;
        }

        public function getAlreadyVoted(userType:String):Boolean{
            if (this[userType] && this[userType].tuning) {
                var voteTuning: String = this[userType].tuning._id;
                return !(myChallengeVotes[this._id] && myChallengeVotes[this._id].indexOf(voteTuning) > -1);
            } else {
                return false;
            }
        }

        public function isWin(type: String): Boolean {
            return this[type].winner;
        }

        public function getBackgroundUrl(type: String): String {
            var backgroundImageUrl: String;
            if (this[type] && this[type].tuning &&  this[type].tuning.background && this[type].tuning.background.imagePath) {
                backgroundImageUrl = this[type].tuning.background.imagePath
            } else if (this[type] && this[type].tuning &&  this[type].tuning.car && this[type].tuning.car.defaultBkgMobile && this[type].tuning.car.defaultBkgMobile[0]) {
                backgroundImageUrl = this[type].tuning.car.defaultBkgMobile[0];
            }
            if (backgroundImageUrl) {
                backgroundImageUrl = 'http:' + backgroundImageUrl.replace('http:', '');
                return backgroundImageUrl;
            }

            return null;
        }

        public function getCarImageUrl(type: String): String {
            var carImageUrl: String;

            if (this[type] && this[type].tuning && this[type].tuning.snapshots && this[type].tuning.snapshots[1]) {
                carImageUrl = this[type].tuning.snapshots[1];
            }

            if (carImageUrl) {
                carImageUrl = 'http:' + carImageUrl.replace('http:', '');
                return carImageUrl;
            }

            return null;
        }

        public function getCarFullHdUrl(type: String): String {
            var carImageUrl: String;

            if (this[type] && this[type].tuning && this[type].tuning.fullHdSnapshots && this[type].tuning.fullHdSnapshots[0]) {
                carImageUrl = this[type].tuning.fullHdSnapshots[0];
            }

            if (carImageUrl) {
                carImageUrl = 'http:' + carImageUrl.replace('http:', '');
                return carImageUrl;
            }

            return null;
        }

    }
}
