package com.despark.tuning3d.model.challenges {

import by.blooddy.crypto.serialization.JSONer;

import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.http.HttpClient;
import com.despark.tuning3d.model.core.http.caching.HttpCachePolicy;
import com.despark.tuning3d.model.friend.Friend;
import com.despark.tuning3d.model.friend.FriendProxy;
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.tunings.TuningsProxy;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.gallery.data.RequestPeriod;
import com.utils.DateParser;
import com.utils.FileUtil;

import flash.filesystem.File;
import flash.net.URLRequestDefaults;
import flash.net.URLRequestMethod;

import org.puremvc.as3.patterns.proxy.Proxy;

public class ChallengesProxy extends Proxy {
    public static const NAME : String = "ChallengesProxy";
    public var httpClient:HttpClient;
    public static var USER_REQUEST_COUNT:int = 12;
    public static const MAX_COUNT_STATUS_CODE:int = 426;
    private static const MAX_COUNT_MESSAGE:String = "You have reached the maximum allowed amount of active challenges - 10! \n" + "\n" + "You will be able to start/accept new challenges as soon as one of your current challenges finishes. ";
    private static const MAX_COUNT_MESSAGE_OPPONENT:String = "Your opponent has reached the maximum allowed amount of active challenges - 10! \n" + "\n" + "You will be able to start/accept new challenges as soon as one of his/her current challenges finishes. ";
    private var showCountError:Boolean = false;
    private var challengeNotes:File;
    private var myChallengeVotesFile : File;
    public var myChallengeVotes: Object;


    public function ChallengesProxy() {
        super(NAME);
        myChallengeVotesFile = File.applicationStorageDirectory.resolvePath("myChallengeVotes.json");
        loadChallengeVotesFromFile();
    }

    private function loadChallengeVotesFromFile(): void {
        if(myChallengeVotesFile.exists){
            myChallengeVotes = JSONer.parse(FileUtil.fileGetContent(myChallengeVotesFile));
        }else{
            myChallengeVotes = {};
        }
    }

    private function saveVotesToFile(): void{
        var json:String=JSONer.stringify(myChallengeVotes);
        FileUtil.filePutContent(myChallengeVotesFile, json);
    }


    public function loadUserChallenges(userId:String, selectTuning:Boolean) : void {
        var params:Object = {
            user: userId,
            limit: 0,
            status: "accepted"
        };
        var url : String = HttpClient.CHALLENGE_ROOT_URL;
        httpClient.getPath(url, params, function(success : Boolean, result : Object) : void {
            if (success) {
                sendNotification(Notifications.CHALLENGES_LOADED, {user:userId, challenges:JSONer.parse(result), selectTuning:selectTuning});
            } else {
                trace("Failed to load challenges", JSON.stringify(result));
            }
        });
    }

    public function loadTunings(user:User, selectTuning:Boolean = true, skip:int = 0, limit:int = 100) : void{
        httpClient.getPath(
                "tunings", {
                    userId:user.id,
                    skip: skip,
                    limit: limit,
                    withCounters:true,
                    all:true
                },
                function(success : Boolean, result : Object) : void{
                    if(success){
                        var tuningsJSON : Object = JSON.parse(String(result));
                        var tunings : Array = [];
                        for each(var tuningJSON : Object in tuningsJSON) {
                            var tuning : Tuning = TuningsProxy.parseTuning(tuningJSON);
                            tunings.push(tuning);
                        }
                        sendNotification(Notifications.CHALLENGE_TUNINGS_LOADED, {tunings: tunings, selectTuning: selectTuning, lastPage: tunings.length == 0});
                    } else {

                    }
                });
    }

    /**
     * Loads current user friends
     */
    public function loadFriends(user:User, skip:int = 0, limit:int = 12):void {
        var url : String = "http://friends.3dtuning.com/api/v1/friends";
        var params : Object = {};
        params.limit = limit;
        params.skip  = skip;
        params.user  = user.id;
        httpClient.loadPath("", URLRequestMethod.GET, params, HttpCachePolicy.NONE,true,function(success:Boolean,result:Object):void {
            if(success){
                var friends : Array = [];
                var friendsJSON : Object = JSON.parse(String(result));
                var jsonCollection : Object;
                jsonCollection = friendsJSON.friends;
                var friend  : Friend;
                for each(var jsn : Object in jsonCollection) {
                    friend = new Friend(Friend.STATE_FRIEND, jsn);
                    friends.push(friend);
                }
                Constants.CURRENT_USER_FRIENDS = friends;
                sendNotification(Notifications.CHALLENGE_FRIENDS_LOADED, {friends: friends, total:friendsJSON.total});
            } else {
                // sendNotification(Notifications.FRIENDS_LOADING_FAILED); TODO
            }
        },url);
    }

    public function searchUsers(searchTerm:String, skip:int, completeHandler:Function, limit:int = 12):void{
        var friendProxy:FriendProxy = ApplicationFacade.getInstance().retrieveProxy(FriendProxy.NAME) as FriendProxy;
        friendProxy.search(searchTerm, skip, null, null, completeHandler, limit);
    }

    public function searchTunings(searchTerm:String, skip:int, limit:int, completeHandler:Function = null):void{
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        var user:User = usersProxy.currentUser;
        var tuningsProxy:TuningsProxy = ApplicationFacade.getInstance().retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
        tuningsProxy.search(user.id, searchTerm, skip, limit, completeHandler);
    }

    public function inviteForChallenge(challengeData:Object):void{
        var url:String = HttpClient.CHALLENGE_ROOT_URL;
        var params:Object = {owner:{tuning:challengeData.tuningId}, receiver:{user:challengeData.userId}, challengeType:challengeData.challengeType};
        showCountError = true;
        httpClient.postPath("", JSON.stringify(params),
            function(success : Boolean, result : Object, headers:Array = null) : void{
                if(success){

                } else {
                    if(httpClient.lastErrorStatusCode == MAX_COUNT_STATUS_CODE && showCountError){
                        var message:String = result.title + result.body;
                        sendNotification(Notifications.CHALLENGE_MAX_COUNT_EXCEEDED_ERROR, {statusCode: MAX_COUNT_STATUS_CODE, message:message});
                        showCountError = false;
                    }
                }
            }, url);
    }

    public function randomChallenge(userId:String, carId:String, carType:String, challengeType:String, tuningId:String):void{
        showCountError = true;
        var url:String = HttpClient.RANOM_CHALLENGE_ROOT_URL;
        var params:Object = {
            "user": userId,
            "car": carId,
            "carType": carType,
            "challengeType": challengeType,
            "tuning": tuningId
        };
        httpClient.postPath("", JSON.stringify(params),
            function(success : Boolean, result : Object, responseHeader:Array=null) : void{
                if(success){
                    sendNotification(Notifications.CREATE_RANDOM_CHALLENGE, {tuning: tuningId});
                } else {
                    if(httpClient.lastErrorStatusCode == MAX_COUNT_STATUS_CODE && showCountError){
                        var message:String = result.title + result.body;
                        sendNotification(Notifications.CHALLENGE_MAX_COUNT_EXCEEDED_ERROR, {statusCode: MAX_COUNT_STATUS_CODE, message:message});
                        showCountError = false;
                    }
                }
            }, url);
    }

    public function acceptChallenge(challengeId:String, tuningId:String, status:Boolean):void{
        showCountError = true;
        var url:String = HttpClient.CHALLENGE_ROOT_URL + challengeId;
        var params:Object = {"status":(status?"accepted":"rejected"),"receiver":{"tuning":tuningId}};
        httpClient.putPath("", JSON.stringify(params),
            function(success : Boolean, result : Object, responseHeader:Array=null) : void{
                if(success){
                    trace("Challenge", success);
                } else {
                    if(httpClient.lastErrorStatusCode == MAX_COUNT_STATUS_CODE && showCountError){
                        var message:String = result.title + result.body;
                        sendNotification(Notifications.CHALLENGE_MAX_COUNT_EXCEEDED_ERROR, {statusCode: MAX_COUNT_STATUS_CODE, message:message});
                        showCountError = false;
                    }
                }
            }, url);
    }

    public function getNotifications(skip:int = 0, limit:int = 7, completeCallback:Function = null):void{
        var url:String = "http://mailer.3dtuning.com/api/v1/notifications?mobile=true&limit="+limit.toString()+"&skip="+skip.toString()+"&typeGroup=challenge&populatebackground=true";
        httpClient.loadPath("", URLRequestMethod.GET, null, HttpCachePolicy.NONE, false,
                function (success : Boolean, result : Object) : void {
                    if(completeCallback!=null) {
                        if (success) {
                            completeCallback.apply(null, [JSON.parse(result as String)]);
                        }else{
                            completeCallback.apply(null, [{}]);
                        }
                    }
                }, url);
    }

    public function getRequests(skip:int = 0, limit:int = 20, completeCallback:Function = null):void{
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        var user:User = usersProxy.currentUser;
        var url:String = HttpClient.CHALLENGE_ROOT_URL + "?mobile=true&skip="+skip+"&limit="+limit+"&populate=owner.tuning,owner.user&receiver.user="+user.id+"&smart=true&sort=-updated&status=pending";
        httpClient.loadPath("", URLRequestMethod.GET, null, HttpCachePolicy.NONE, false,
            function (success : Boolean, result : Object) : void {
                if(completeCallback!=null) {
                    if (success) {
                        completeCallback.apply(null, [JSON.parse(result as String)]);
                    }else{
                        completeCallback.apply(null, [{}]);
                    }
                }
            }, url);
    }


    public function getChallenge(challengeId: String, cb: Function): void {
        var url: String = HttpClient.CHALLENGE_ROOT_URL + challengeId;
        httpClient.getPath(url, {}, function (success : Boolean, result : String): void {
            if (success) {
                var challengeData: Object = JSON.parse(result);

                var newChallengeData: Object = {
                    owner: {
                        votes: challengeData.owner.votes,
                        winner: challengeData.owner.winner
                    },
                    receiver: {
                        votes: challengeData.receiver.votes,
                        winner: challengeData.receiver.winner
                    }

                };
                cb(newChallengeData)
            } else {
                cb(null)
            }
        })
    }

    public function getChallengeData(challengeId: String, callback: Function): void {
        var url: String = HttpClient.CHALLENGE_ROOT_URL + challengeId + '?populate=owner.tuning,owner.user,receiver.tuning,receiver.user';
        httpClient.getPath(url, {}, function (success : Boolean, result : String): void {
            if (success) {
                var challengeObj: Object = JSON.parse(result);
                var challengeData:ChallengeData = new ChallengeData(challengeObj);
                callback.apply(null, [challengeData]);
            } else {
                callback.apply(null, [null]);
            }
        })
    }


    public function getChallenges(limit: int = 2, skip: int = 0, isAll: Boolean = true, toString: String = null, sorting: String = "-updated", isPulling: Boolean = false): void {


        var url: String = HttpClient.CHALLENGE_ROOT_URL;
        var params:Object = {};
        if (isAll) {

        } else {
            var currentUser: User;
            var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
            currentUser = usersProxy.currentUser;
            params = {
                user: currentUser.id
            };
        }
        params.status  = 'accepted,finished';
        params.skip = skip;
        params.limit = limit;
        params.mobile = true;
        params.populate = 'owner.tuning,owner.user,receiver.tuning,receiver.user';
        if (isAll) {
            params.sort =  sorting;
            if (params.sort === '-updated') {
                params.status  = 'finished';
            } else {
                params.status  = 'accepted';
            }

        } else {
            params.status  = 'accepted,finished';
            params.sort = '-updated';
        }
        params. smart =  'true';
        if (toString) {
            params.to = toString;
        }
        httpClient.getPath(url, params, function (success : Boolean, result : String): void {
            if (success) {
                var challengesData: Array = JSON.parse(result) as Array;
                for (var i: int = 0; i < challengesData.length; i++) {
                    challengesData[i] = new ChallengeData(challengesData[i]);
                }
                sendNotification(Notifications.CHALLENGES_LIST_LOADED, {data: challengesData, isPulling: isPulling});
            } else {
                sendNotification(Notifications.CHALLENGES_LIST_LOADED, {data: []});
            }
        })
    }

    public function getAllUnreadNotificationCount(completeCallback:Function = null):void{
        challengeNotes = File.applicationStorageDirectory.resolvePath("challenge-notes.json");
        var challengeNotesContent:Object;
        if(challengeNotes.exists){
            challengeNotesContent = JSON.parse(FriendProxy.fileGetContent(challengeNotes));
        }
        challengeNotesContent ||= {};
        var count:int = 0;
        getUnreadNotificationCount(function (val:Object):void {
            if(val && val.hasOwnProperty("count")){
                count += val.count;
            }
            getRequestIdList(function (idList:Object):void {
                if(idList && idList.data){
                    var arr:Array = idList.data;
                    for(var i:int = 0; i < arr.length; i++){
                        if(!challengeNotesContent.hasOwnProperty(arr[i])){
                            count++;
                        }
                    }
                }
                FriendProxy.filePutContent(challengeNotes, JSON.stringify(challengeNotesContent));
                if(completeCallback != null){
                    Constants.UNREAD_CHALLENGE_NOTIFICATIONS = count;
                    completeCallback.apply(null, [count]);
                }
            });
        })
    }

    public function markAllUnreadNotifications(completeCallback:Function = null):void{
        challengeNotes = File.applicationStorageDirectory.resolvePath("challenge-notes.json");
        var challengeNotesContent:Object;
        if(challengeNotes.exists){
            challengeNotesContent = JSON.parse(FriendProxy.fileGetContent(challengeNotes));
        }
        challengeNotesContent ||= {};
        getRequestIdList(function (idList:Object):void {
            if(idList && idList.data){
                var arr:Array = idList.data;
                for(var i:int = 0; i < arr.length; i++){
                    if(!challengeNotesContent.hasOwnProperty(arr[i])){
                        challengeNotesContent[arr[i]] = 1;
                    }
                }
            }
            FriendProxy.filePutContent(challengeNotes, JSON.stringify(challengeNotesContent));
            if(completeCallback != null){
                completeCallback.call();
            }
        });
        markUnreadNotifications();
    }

    public function markUnreadNotifications() : void{
        httpClient.loadPath('', URLRequestMethod.PUT, null, HttpCachePolicy.NONE,false, function (success:Boolean, result:Object):void{
            if(success){

            }
        }, 'http://mailer.3dtuning.com/api/v1/notifications/?type=challenge')
    }

    public function getRequestIdList(completeCallback:Function = null):void{
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        var user:User = usersProxy.currentUser;
        var url:String = HttpClient.CHALLENGE_ROOT_URL + "?limit=50000&populate=owner.tuning,owner.user&receiver.user="+user.id+"&smart=true&sort=-updated&status=pending&count=true";
        httpClient.loadPath("", URLRequestMethod.GET, null, HttpCachePolicy.NONE, false,
            function (success : Boolean, result : Object) : void {
                if(completeCallback!=null) {
                    if (success) {
                        completeCallback.apply(null, [JSON.parse(result as String)]);
                    }
                }
            }, url);
    }

    public function getUnreadNotificationCount(completeCallback:Function = null):void{
        var url:String = "http://mailer.3dtuning.com/api/v1/notifications?typeGroup=challenge&count=true";
        httpClient.loadPath("", URLRequestMethod.GET, null, HttpCachePolicy.NONE, false,
            function (success : Boolean, result : Object) : void {
                if(completeCallback!=null) {
                    if (success) {
                        completeCallback.apply(null, [JSON.parse(result as String)]);
                    }
                }
            }, url);
    }

    public function voteChallenge(challengeId: String, type: String, tuningId: String, cb: Function, userId: String = null, onFail:Function = null): void {
        if (myChallengeVotes[challengeId] && myChallengeVotes[challengeId].indexOf(tuningId) > -1) {
            cb(null);
        } else {
            var url:String = HttpClient.CHALLENGE_ROOT_URL + challengeId + '/vote?activation=true&userId=' + userId + '&newVersion=true';
            var params:Object = {targetUser: type};
            httpClient.putPath("", JSON.stringify(params),
                    function (success:Boolean, result:String, responseHeader:Array = null):void {
                        if (success) {
                            if (!myChallengeVotes[challengeId]) {
                                myChallengeVotes[challengeId] = []
                            }
                            myChallengeVotes[challengeId].push(tuningId);
                            saveVotesToFile();
                            cb(new ChallengeData(JSON.parse(result)))
                        }else{
                            trace('failed to like challenge');
                            if(onFail != null){
                                onFail.call();
                            }
                            sendNotification(Notifications.EMAIL_VERIFICATION_ERROR, {event:'challenge', stopPropagation:true, statusCode:httpClient.lastErrorStatusCode});
                        }
                    }, url);
        }

    }

    public function getRandomChallenge(cb: Function = null): void {
        var url: String = HttpClient.CHALLENGE_ROOT_URL + 'random?populate=owner.tuning,owner.user,receiver.tuning,receiver.user&smart=true';
        httpClient.loadPath("", URLRequestMethod.POST, {}, HttpCachePolicy.NONE, false,function (success : Boolean, result : Object) : void {
            if(cb!=null) {
                if (success) {

                    cb(new ChallengeData(JSON.parse(result as String)))
                } else {
                    cb(null);
                }
            }
        }, url);
    }

    public function getLoaderBoardLastMonth(limit:int = 10, skip:int = 0, callback:Function = null):void{
        var toDate:Date = new Date();
        var fromDate:Date = new Date();
        fromDate.month-=1;

        getLeaderBoard(fromDate, toDate, limit, skip, false, callback);
    }

    public function getLoaderBoardThisWeek(limit:int = 10, skip:int = 0, toDate:Date = null, country:Boolean = false, callback:Function = null):void{
        var fromDate:Date = new Date();
        toDate ||= new Date();
        if(fromDate.day>0){
            fromDate.setTime(fromDate.getTime() - fromDate.day * 24 * 60 * 60 * 1000);
        }
        fromDate.setHours(0);
        fromDate.setMinutes(0);
        fromDate.setSeconds(0);

        /*toDate.setTime(toDate.getTime() + (7-fromDate.day) * 24 * 60 * 60 * 1000);
        toDate.setHours(0);
        toDate.setMinutes(0);
        toDate.setSeconds(0);*/
        getLeaderBoard(fromDate, toDate, limit, skip, country, callback);
    }

    public function getLoaderBoardByPeriod(limit:int = 10, skip:int = 0, toDate:Date = null, country:Boolean = false, period:String = null, callback:Function = null):void{
        var fromDate:Date = new Date();
        toDate ||= new Date();
        if(period == RequestPeriod.THIS_WEEK){
            if(fromDate.day > 0){
                fromDate.setTime(fromDate.getTime() - fromDate.day * 24 * 60 * 60 * 1000);
            }
        }else if(period == RequestPeriod.THIS_MONTH){
            if(fromDate.date > 0){
                fromDate.setTime(fromDate.getTime() - fromDate.date * 24 * 60 * 60 * 1000);
            }
        }else if(period == RequestPeriod.THIS_YEAR){
            fromDate.setFullYear(toDate.fullYear, 0, 1);
        }else if(period == RequestPeriod.ALL){
            fromDate = null;
        }

        if(fromDate){
            fromDate.setHours(0);
            if(period == RequestPeriod.THIS_YEAR){
                fromDate.setMinutes(-fromDate.timezoneOffset);
            }
            fromDate.setSeconds(0);
        }
        getLeaderBoard(fromDate, toDate, limit, skip, country, callback);
    }

    public function getLeaderBoard(fromDate:Date, toDate:Date, limit:int = 10, skip:int = 0, country:Boolean = false, callback:Function = null): void {
        var fromDateStr:String;
        if(fromDate){
            fromDateStr = DateParser.toW3CDTF(fromDate);
        }
        var toDateStr:String = DateParser.toW3CDTF(toDate);
        var url: String = HttpClient.CHALLENGE_ROOT_V2_URL + 'ratedusers?limit='+limit+'&skip='+skip;
        if(fromDateStr){
            url += '&from='+fromDateStr+'&to='+toDateStr;
        }
        if(country){
            url += '&country=true';
        }
        httpClient.loadPath("", URLRequestMethod.GET, {}, HttpCachePolicy.NONE, false,function (success : Boolean, result : Object) : void {
            if(callback!=null) {
                if (success) {
                    callback(JSON.parse(result as String));
                } else {
                    callback(null);
                }
            }
        }, url);
    }

    public static function getChallengeShareUrl(challengeId: String, type: String): String {
        return 'http://www.3dtuning.com/en-US/challenge/' + challengeId + '/' + type;
    }


    public function checkTuningInChallenge(tuningId: String, cb: Function): void {
        httpClient = (ApplicationFacade.getInstance().retrieveProxy(TuningsProxy.NAME) as TuningsProxy).httpClient;
        var url:String = HttpClient.CHALLENGE_ROOT_URL + 'inchallenge/' + tuningId;
        httpClient.getPath(url, null, function (success:Boolean, result:Object):void {
            if (success) {
                var obj:Object = JSON.parse(result.toString());
                cb(obj.status);
            }
        });
    }

}
}
