package com.despark.tuning3d.model.challenges.feed {
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.model.challenges.ChallengeData;
import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.tunings.TuningsProxy;
import com.despark.tuning3d.view.components.challenges.controls.ChallengeLabel;
import com.despark.tuning3d.view.components.challenges.controls.ChallengeNotificationItemRenderer;
import com.despark.tuning3d.view.components.challenges.controls.TuningLabel;
import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;

import feathers.controls.LayoutGroup;
import feathers.layout.FlowLayout;

import starling.events.Event;

public class ChallengeNotificationData{
    private static const CHALLENGE_REQUEST_RECEIVED:String = "challenge_request";
    private static const CHALLENGE_ACCEPTED:String = "challenge_accepted";
    private static const CHALLENGE_REJECTED:String = "challenge_rejected";
    private static const CHALLENGE_LOST:String = "challenge_lose";
    private static const CHALLENGE_WIN:String = "challenge_win";
    protected var _id:String;
    protected var _time:Number;
    protected var _iconTextureId:String;
    protected var _messageLabel:LayoutGroup;
    protected var _tuningView:TuningLabel;
    protected var _challengeLabel:ChallengeLabel;
    protected var _avatar:UserCircularAvatar;
    protected var _buttonGroup:LayoutGroup;
    protected var _holder:ChallengeNotificationItemRenderer;
    protected var _data:Object;

    protected var _disposed:Boolean;
    public function ChallengeNotificationData(data:Object) {
        _data = data;
        _id = _data._id;
        _disposed = true;
        createView();
    }

    public function createView():void{
        if(!_disposed){
            return;
        }
        _messageLabel = new LayoutGroup();
        var layout:FlowLayout = new FlowLayout();
        layout.horizontalGap = 5;
        layout.verticalGap = 0;
        _messageLabel.layout = layout;
        _disposed = false;
    }

    public static function createInstance(data:Object):ChallengeNotificationData{
        var instance:ChallengeNotificationData;
        if(data.type == "challenge_answer"){
            if(data.text == "accepted"){
                data.type = CHALLENGE_ACCEPTED;
            }else if(data.text == "rejected"){
                data.type = CHALLENGE_REJECTED;
            }
        }else if(data.type == null){
            data.type = CHALLENGE_REQUEST_RECEIVED;
        }
        switch(data.type){
            case CHALLENGE_REQUEST_RECEIVED:
                instance = new ChallengeRequestNoteData(data);
                break;
            case CHALLENGE_ACCEPTED:
                instance = new ChallengeAcceptedNoteData(data);
                break;
            case CHALLENGE_REJECTED:
                instance = new ChallengeRejectedNoteData(data);
                break;
            case CHALLENGE_LOST:
                instance = new ChallengeLostNoteData(data);
                break;
            case CHALLENGE_WIN:
                instance = new ChallengeWonNoteData(data);
                break;
        }
        return instance;
    }

    public function destroy():void{
        dispose();
        _data = null;
        _holder = null;
    }

    public function dispose():void{
        if(_messageLabel){
            _messageLabel.removeFromParent(true);
            _messageLabel = null;
        }
        if(_tuningView){
            _tuningView.removeFromParent(true);
            _tuningView = null;
        }
        if(_avatar){
            _avatar.removeFromParent(true);
            _avatar = null;
        }
        if(_buttonGroup){
            _buttonGroup.removeFromParent(true);
            _buttonGroup = null;
        }
        if(_challengeLabel){
            _challengeLabel.removeFromParent(true);
            _challengeLabel = null;
        }
        _disposed = true;
    }

    public function remove():void{
        if(_messageLabel){
            _messageLabel.removeFromParent();
        }

        if(_tuningView){
            _tuningView.removeFromParent();
        }

        if(_avatar){
            _avatar.removeFromParent();
        }

        if(_buttonGroup){
            _buttonGroup.removeFromParent();
        }

        if(_challengeLabel){
            _challengeLabel.removeFromParent();
        }
    }

    public function get id():String {
        return _id;
    }

    public function get time():Number {
        return _time;
    }

    public function get iconTextureId():String {
        return _iconTextureId;
    }

    public function get messageLabel():LayoutGroup {
        return _messageLabel;
    }

    public function get tuningView():TuningLabel {
        return _tuningView;
    }

    public function get challengeLabel():ChallengeLabel {
        return _challengeLabel;
    }

    public function get avatar():UserCircularAvatar {
        return _avatar;
    }

    public function get buttonGroup():LayoutGroup {
        return _buttonGroup;
    }

    public function set holder(value:ChallengeNotificationItemRenderer):void {
        _holder = value;
        _holder.setRequiresRedraw();
    }

    public function get disposed():Boolean {
        return _disposed;
    }

    protected function showChallengePopup():void{
        var tuningsProxy:TuningsProxy = ApplicationFacade.getInstance().retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
        var loadReceiverTuning:Boolean = _data.challenge.receiver.tuning is String;
        var loadOwnerTuning:Boolean = _data.challenge.owner.tuning is String;
        if(loadOwnerTuning || loadReceiverTuning){
            var tuningsLoadComplete:Function = function():void{
                if(!loadOwnerTuning && !loadReceiverTuning){
                    var eventData:Object = {
                        receiverTuning: _data.challenge.receiver.tuning is Tuning?_data.challenge.receiver.tuning: TuningsProxy.parseTuning(_data.challenge.receiver.tuning),
                        senderTuning: _data.challenge.owner.tuning is Tuning?_data.challenge.owner.tuning: TuningsProxy.parseTuning(_data.challenge.owner.tuning),
                        challengeData: new ChallengeData(_data.challenge)
                    };
                    _holder.eventDispatcher.dispatchEvent(new Event(ChallengeEvent.VIEW_CHALLENGE_TUNING, false, eventData));
                }
            };
            if(loadOwnerTuning){
                tuningsProxy.getTuning(_data.challenge.owner.tuning, function(tuning:Tuning, json:Object):void{
                    _data.challenge.owner.tuning = tuning;
                    loadOwnerTuning = false;
                    tuningsLoadComplete.call();
                }, false);
            }
            if(loadReceiverTuning){
                tuningsProxy.getTuning(_data.challenge.receiver.tuning, function(tuning:Tuning, json:Object):void{
                    _data.challenge.receiver.tuning = tuning;
                    loadReceiverTuning = false;
                    tuningsLoadComplete.call();
                }, false);
            }
        }else{
            var eventData:Object = {
                receiverTuning: TuningsProxy.parseTuning(_data.challenge.receiver.tuning),
                senderTuning: TuningsProxy.parseTuning(_data.challenge.owner.tuning),
                challengeData: new ChallengeData(_data.challenge)
            };
            _holder.eventDispatcher.dispatchEvent(new Event(ChallengeEvent.VIEW_CHALLENGE_TUNING, false, eventData));
        }
    }
}
}
