package com.despark.tuning3d.model.challenges.feed {
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.tunings.TuningsProxy;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.challenges.controls.ChallengeNotesLabelFactory;
import com.despark.tuning3d.view.components.challenges.controls.TuningLabel;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
import com.utils.DateParser;

import feathers.controls.Button;
import feathers.controls.Label;
import feathers.controls.LayoutGroup;
import feathers.layout.HorizontalLayout;
import feathers.layout.VerticalAlign;

import starling.display.DisplayObject;

import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;

public class ChallengeRequestNoteData extends ChallengeNotificationData {
    private var _tuning:Tuning;
    private var _user:User;

    private var _creatorLabel:Button;
    private var _textLabel:Label;

    private var _acceptButton:Button;
    private var _declineButton:Button;
    private var _touchMoved:Boolean;
    public function ChallengeRequestNoteData(data:Object) {
        _iconTextureId = "challenge_request_icon";
        _user = new User(data.owner.user);
        super(data);
        _time =  new Date().time - DateParser.parseDate(data.created).time;
        if(data.owner.user){
            data.owner.tuning.user = data.owner.user;
        }
        _tuning = TuningsProxy.parseTuning(data.owner.tuning, false);
        createTuningView();
        _touchMoved = false;
    }


    override public function createView():void {
        if(!_disposed){
            return;
        }
        super.createView();
        _creatorLabel = new Button();
        _creatorLabel.labelFactory = ChallengeNotesLabelFactory.nameLabelRenderer;
        _creatorLabel.label = _user.nameToShow;
        _creatorLabel.addEventListener(Event.TRIGGERED, profileTriggeredHandler);

        _textLabel = new Label();
        _textLabel.textRendererFactory = ChallengeNotesLabelFactory.messageLabelRenderer;
        _textLabel.text = "challenged you with";

        _messageLabel.addChild(_creatorLabel);
        _messageLabel.addChild(_textLabel);

        _acceptButton = ButtonFactory.challengeAcceptButton("Accept");
        _declineButton = ButtonFactory.challengeDeclineButton("Decline");
        _buttonGroup = new LayoutGroup();
        _acceptButton.addEventListener(Event.TRIGGERED, acceptHandler);
        _declineButton.addEventListener(Event.TRIGGERED, declineHandler);

        var layout:HorizontalLayout = new HorizontalLayout();
        layout.gap = 10;
        layout.verticalAlign = VerticalAlign.MIDDLE;
        _buttonGroup.layout = layout;
        _buttonGroup.addChild(_acceptButton);
        _buttonGroup.addChild(_declineButton);


        _avatar = new UserCircularAvatar(_user, 40);
        _avatar.loadComplete = function ():void {
            _holder.updateView();
        };
        _avatar.touchable = true;
        _avatar.addEventListener(TouchEvent.TOUCH, profileTouchedHandler);

        _creatorLabel.validate();
        _textLabel.validate();
        _messageLabel.validate();

        createTuningView();
    }

    override public function dispose():void {
        super.dispose();
        if(_creatorLabel){
            _creatorLabel.removeFromParent(true);
            _creatorLabel = null;
        }
        if(_textLabel){
            _textLabel.removeFromParent(true);
            _textLabel = null;
        }
        if(_acceptButton){
            _acceptButton.removeFromParent(true);
            _acceptButton = null;
        }
        if(_declineButton){
            _declineButton.removeFromParent(true);
            _declineButton = null;
        }
    }

    private function profileTouchedHandler(event:TouchEvent):void {
        var touch:Touch = event.getTouch(event.target as DisplayObject, TouchPhase.ENDED);
        if (touch && !_holder.owner.isScrolling){
            profileTriggeredHandler(null);
        }
    }

    private function profileTriggeredHandler(event:Event):void {
        _holder.eventDispatcher.dispatchEvent(new Event(ChallengeEvent.VIEW_USER_TUNINGS, false, {user:_user}));
    }

    private function acceptHandler(event:Event):void{
        if(!_tuning){
            return;
        }
        var currentUser : User  = (ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy).currentUser;
        var tuning:Tuning = (_tuning.userObj._id == _user.id)?_tuning:null;
        var data:Object = ChallengeEvent.getData(_user, tuning, currentUser, null, true);
        data.challengeId = _data._id;
        data.notification = this;
        data.challengeData = _data;
        _holder.eventDispatcher.dispatchEvent(new Event(ChallengeEvent.CHALLENGE_REQUEST, false, data));
    }

    private function declineHandler(event:Event):void{
        if(!_tuning){
            return;
        }
        var currentUser : User  = (ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy).currentUser;
        var tuning:Tuning = (_tuning.userObj._id == _user.id)?_tuning:null;
        var data:Object = ChallengeEvent.getData(_user, tuning, currentUser, null, true);
        data.challengeId = _data._id;
        data.notification = this;
        _holder.eventDispatcher.dispatchEvent(new Event(ChallengeEvent.DECLINE_CHALLENGE, false, data));
    }


    private function createTuningView():void {
        if(!_tuning || _tuningView){
            return;
        }
        _tuningView = new TuningLabel(_tuning.thumbnailURL);
        _tuningView.loadComplete = function():void{
            _holder.updateView();
        };
        _tuningView.addEventListener(TouchEvent.TOUCH, onTuningSelect);
    }

    private function onTuningSelect(event:TouchEvent):void {
        var touch:Touch = event.getTouch(_tuningView, TouchPhase.ENDED);
        if(touch) {
            _holder.eventDispatcher.dispatchEvent(new Event(ChallengeEvent.VIEW_REQUEST_TUNING, false, {tuning:_tuning, user:_user}));
        }
    }
}
}
