package com.despark.tuning3d.model.challenges.feed {
import com.despark.tuning3d.model.challenges.ChallengeData;
import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.tunings.TuningsProxy;
import com.despark.tuning3d.view.components.challenges.controls.ChallengeNotesLabelFactory;
import com.despark.tuning3d.view.components.challenges.controls.TuningLabel;
import com.utils.DateParser;

import feathers.controls.Label;

import starling.display.DisplayObject;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;

public class ChallengeWonNoteData extends ChallengeNotificationData {
    private var _creatorLabel:Label;
    private var _textLabel:Label;
    private var _tuningLabel:Label;

    private var _tuning:Tuning;
    private var _touchMoved:Boolean;
    public function ChallengeWonNoteData(data:Object) {
        _iconTextureId = "challenge_won_icon";
        _tuning = TuningsProxy.parseTuning(data.tuning, false);
        super(data);
        _time = new Date().time - DateParser.parseDate(data.updated).time;
        _touchMoved = false;
    }

    override public function createView():void {
        if(!_disposed){
            return;
        }
        super.createView();

        _creatorLabel = new Label();
        _creatorLabel.textRendererFactory = ChallengeNotesLabelFactory.nameLabelRenderer;
        _creatorLabel.text = "You";

        _textLabel = new Label();
        _textLabel.textRendererFactory = ChallengeNotesLabelFactory.messageLabelRenderer;
        _textLabel.text = "have won tuning challenge with";

        _tuningLabel = new Label();
        _tuningLabel.textRendererFactory = ChallengeNotesLabelFactory.tuningLabelRenderer;
        _tuningLabel.text = _tuning.name;
        _tuningLabel.addEventListener(TouchEvent.TOUCH, touchTuningHandler);

        _messageLabel.addChild(_creatorLabel);
        _messageLabel.addChild(_textLabel);
        _messageLabel.addChild(_tuningLabel);

        _tuningView = new TuningLabel(_tuning.thumbnailURL);
        _tuningView.loadComplete = function():void{
            _holder.updateView();
        };
        _tuningView.addEventListener(TouchEvent.TOUCH, touchTuningHandler);
    }

    override public function dispose():void {
        super.dispose();
        if(_creatorLabel){
            _creatorLabel.removeFromParent(true);
            _creatorLabel = null;
        }
        if(_textLabel){
            _textLabel.removeFromParent(true);
            _textLabel = null;
        }
        if(_tuningLabel){
            _tuningLabel.removeFromParent(true);
            _tuningLabel = null;
        }
    }

    private function viewTuningHandler(event:Event):void {
        var eventData:Object = {receiverTuning: TuningsProxy.parseTuning(_data.challenge.receiver.tuning), senderTuning: TuningsProxy.parseTuning(_data.challenge.owner.tuning), challengeData:new ChallengeData(_data.challenge)};
        _holder.eventDispatcher.dispatchEvent(new Event(ChallengeEvent.VIEW_CHALLENGE_TUNING, false, eventData));
    }

    private function touchTuningHandler(event:TouchEvent):void {
        var touch:Touch = event.getTouch(event.target as DisplayObject, TouchPhase.ENDED);
        if (touch && !_holder.owner.isScrolling){
            showChallengePopup();
        }
    }
}
}
