package com.despark.tuning3d.model.core.http
{
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.view.components.core.AlertView;
	
	import flash.events.ErrorEvent;
	import flash.events.Event;
	import flash.events.HTTPStatusEvent;
	import flash.events.IOErrorEvent;
	import flash.filesystem.File;
	import flash.filesystem.FileMode;
	import flash.filesystem.FileStream;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.net.URLRequestMethod;

import mx.events.Request;

public class HttpRequestLoader
	{		
		private var resultHandler : Function;
		private var urlLoader : URLLoader;
		private var _urlRequest : URLRequest;
		
		public function HttpRequestLoader()
		{
			super();
		}
		
		public function get urlRequest() : URLRequest 
		{
			return _urlRequest;
		}
		
		private var statusFlag:Boolean=false;
		public function start(request : URLRequest, aResultHandler : Function) : void
		{
			_urlRequest = request;
			resultHandler = aResultHandler;

			var manualCache : File = getManualCacheDirectoryPath(request);

			if(manualCache.exists && !manualCache.isDirectory){

                var fileStream:FileStream =new FileStream();
                fileStream.open(manualCache, FileMode.READ);
                resultHandler(true, fileStream.readUTFBytes(manualCache.size));
				Tracer.print(manualCache.name, 'ManualCache');
				fileStream.close();
			}
			else if((request.url.indexOf("http://") == 0) || (request.url.indexOf("https://") == 0))
			{
				cancel();
				
				if(request.url.indexOf(HttpClient.TUNING_DOMAIN + "/api/v1/tunings?")==0 && request.method==URLRequestMethod.GET){
					request.idleTimeout = 10000;
				} else if (request.url.indexOf('mini=true') > -1) {
					request.idleTimeout = 5000;
					Tracer.print('Mini Filed');
				}

				urlLoader = new URLLoader();
				urlLoader.addEventListener(HTTPStatusEvent.HTTP_RESPONSE_STATUS, urlLoaderHttpStatusHandler);
				urlLoader.addEventListener(IOErrorEvent.IO_ERROR, urlLoaderIoErrorHandler);
				urlLoader.addEventListener(Event.COMPLETE, urlLoaderCompleteHandler);
				statusFlag=false;
				urlLoader.load(request);
			}
			else 
			{  				
				var f :File = new File();   
				f.url = request.url;
				var fs:FileStream =new FileStream();
				fs.open(f, FileMode.READ);
				resultHandler(true, fs.readUTFBytes(f.size));
				fs.close();
			}
		}

		public function getManualCacheDirectoryPath(req : URLRequest) : File{
			var path : String = req.url;
			if (path.indexOf('localhost') > -1) {
				path = path.split('localhost:3003')[1];
			} else if(path.indexOf("3dtuning.com") > -1){
				path = path.split('3dtuning.com')[1];
			} else if(path.indexOf("192.168.10.41:3006") > -1){
				path = path.split("192.168.10.41:3006")[1];
			}
			path = path.replace('?', '%3F');
			path = Constants.MANUAL_CACHE_PATH + path;

			return File.applicationDirectory.resolvePath(path);
		}
		
		public function cancel() : void
		{
			try 
			{
				urlLoader.close();
			}
			catch(error : Error) {}
			
			urlLoader = null;
		}
		
		private function urlLoaderHttpStatusHandler(event : HTTPStatusEvent) : void {
			statusFlag=true;
			var statusCode : int = event.status;
			var responseHeader:Array=event.responseHeaders;
			var completeHandler : Function = function(event : Event) : void {
				var isSuccessful : Boolean = int(statusCode / 100) == 2;
				if(isSuccessful) {
					resultHandler(true, urlLoader.data, responseHeader);
				} else {
					var errorText : String;
					
					if(!urlLoader.data) 
					{
						errorText = "requestFailed";//AlertView.language.translate("mob_alertsAndOverlays", "requestFailed") + statusCode;
					}
					else 
					{
						errorText = urlLoader.data as String;   
					}
					
					resultHandler(false, {errorText: errorText, statusCode: statusCode}, responseHeader);
				}
			};
			urlLoader.removeEventListener(Event.COMPLETE, urlLoaderCompleteHandler);    
			
			urlLoader.addEventListener(Event.COMPLETE, completeHandler);
			
			urlLoader.addEventListener(ErrorEvent.ERROR, completeHandler);   
		}
		
		private function urlLoaderCompleteHandler(event : Event) : void
		{
			if(!statusFlag){
				resultHandler(true, urlLoader.data);
				cancel();   
			}
		}
		
		private function urlLoaderIoErrorHandler(event : IOErrorEvent) : void
		{
			var errorText : String = "requestFailed";//AlertView.language ?  AlertView.language.translate("mob_alertsAndOverlays", "tryAgainLater") : "Error";
			resultHandler(false, {errorText: errorText});
			cancel();
		}
	}
}