package com.despark.tuning3d.model.packages
{
	import com.despark.tuning3d.model.brands.Brand;
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.cars.CarsParser;
	import com.despark.tuning3d.model.core.Parser;
	
	import flash.globalization.DateTimeFormatter;
	
	public class PackagesParser extends Parser
	{
		public var carsParser : Parser;
		public static var allCarsList: Object;
		
		public override function parseObject(packetJSON : Object, key : String, ...args) : Object
		{
			var i:int=0;
			var cars:Array;
			var id:String;
			var existsFlag:Boolean=false;
			var car_id:String;
			var packet :Packet = new Packet();
			
			packet.id = packetJSON._id;
			packet.title = packetJSON.title;
			packet.body = "";//packetJSON.body;
			if(packetJSON.cars && packetJSON.cars._all_ && packetJSON.cars._all_.list){
                packet.listNum = packetJSON.cars._all_.list.length;
            }

            if(packetJSON.hasOwnProperty('project_type')){

                packet.projectType=packetJSON.project_type;
            }
			
			if(packetJSON.hasOwnProperty('productId')){
				
				packet.productId=packetJSON.productId;
			}else{
				
				//trace("Error productId property not found");
			}
			
			if(packetJSON.hasOwnProperty('type')){
				
				packet.type=packetJSON.type;
				
			}else{
				
				packet.type=Packet.TYPE_CARS;
			}
			
			if(packet.type==Packet.TYPE_CARS){
				
				packet.show=false;
				
			}else if(packet.type==Packet.TYPE_COMPONENTS || packet.type==Packet.TYPE_SUBCOMPONENT){
				
				packet.show=true;
				
			}else{
				
				//packet.type=Packet.TYPE_CARS;
				packet.show=false;
			}
			
			
			
			if(packetJSON.hasOwnProperty('discounttime') && packetJSON.discounttime!=null){
				packet.discounttime=new Date(packetJSON.discounttime);
			}else{
				packet.discounttime=null;
			}
			
			if(packetJSON.hasOwnProperty('discountprice'))
				packet.discountprice=packetJSON.discountprice;
			else
				packet.discountprice=-1;
			
			
			if(packetJSON.hasOwnProperty('fee')){
				
				packet.fee=packetJSON.fee;
			}else{
				
				packet.fee=0;
			}
			
			if(packetJSON.hasOwnProperty('subpacks')){
				packet.subpacks = this.parseObjects(packetJSON.subpacks, function(packet :Packet) : void
				{
					//car.name = brand.name;
				});
			}else{
				
				packet.subpacks=null;
				
			}

			if (packetJSON.hasOwnProperty('coin3dFee')) {
				packet.coin3dFee = packetJSON.coin3dFee;
			}

			if(packetJSON.hasOwnProperty('packets') && packetJSON.packets!=null ){
				
				var packets:Array=packetJSON.packets;
				
				packet.packets=packets;
				
			}else{
				
				packet.packets=null;
			}
			
			
			
			if(packetJSON.hasOwnProperty('cars')  && packetJSON.cars!=null){
				
				
				if(packet.type!=Packet.TYPE_SUBCOMPONENT){
				
					cars=packetJSON.cars;
					packet.cars=[];
					for(i=0; i<cars.length; i++){
						id=cars[i];	
						
						if(id){
							var car:Car=allCarsList[id] as Car;
							
							if(car){
								(carsParser as CarsParser).parseProductInfo((car), packet);
								packet.cars.push(car);
							}
						}
					}
					
					packet.example=packet.cars;
				
				}else{
					
					
					packet.cars=[];
					
					
					for (var car_id_or_func:String in packetJSON.cars ){
						
						
						if(car_id_or_func=="_in_" || car_id_or_func=="_all_"){
							
							if(!packetJSON.cars[car_id_or_func])
								continue;
							
							if(car_id_or_func=="_in_" && !packetJSON.cars[car_id_or_func]['list'])
								continue;
							
							var func_name:String="";
							if(packetJSON.cars[car_id_or_func]['standart']){
								
								func_name="standart";
							}else if(packetJSON.cars[car_id_or_func]['camber']) {
								
								
								func_name="camber";
							}
							
							
							packet.funcName=func_name;
							
							
							if(packetJSON.cars[car_id_or_func][func_name]['slot_name'] is Array){
								packet.slotName=packetJSON.cars[car_id_or_func][func_name]['slot_name'];
							}else{
								
								packet.slotName=[packetJSON.cars[car_id_or_func][func_name]['slot_name']];
							}
							
							packet.parts={};
							for(var k:int=0; k<packet.slotName.length; k++){
							
							
								
								if(packet.slotName.length>1){
									
									
									packet.parts[packet.slotName[k]]=packetJSON.cars[car_id_or_func][func_name]['parts'][0][packet.slotName[k]];
								
								}else{
									
									
									packet.parts[packet.slotName[k]]=packetJSON.cars[car_id_or_func][func_name]['parts'];
									
								}
							
							}
							
							if(packetJSON.cars[car_id_or_func]['example']!=null){
								
								for(var index:* in packetJSON.cars[car_id_or_func]['example']){
									
									car_id=packetJSON.cars[car_id_or_func]['example'][index];
									
									car=allCarsList[car_id] as Car;
									
									if(car){
										if(!packet.example){
											packet.example=[];
										}
										
										(carsParser as CarsParser).parseProductInfo((car), packet);
										packet.example.push(car);
									}
								}
								
								
							}
							
							if(packetJSON.cars[car_id_or_func]['list']!=null){
							
								existsFlag=false;
							
								for(var ind:* in packetJSON.cars[car_id_or_func]['list']){
									
									//packetJSON.cars[car_id_or_func]['list']=null;
									
									existsFlag=true;
								    car_id=packetJSON.cars[car_id_or_func]['list'][ind];
									car=allCarsList[car_id] as Car;
									
									if(car){
										(carsParser as CarsParser).parseProductInfo((car), packet);
										packet.cars.push(car);
									}
								}
								
								if(!existsFlag){
									
									
									packetJSON.cars[car_id_or_func]['list']=null;
								}
								
							}else if(car_id_or_func=="_all_"){
								
								existsFlag=false;
								var exclude:Array=packetJSON.cars[car_id_or_func]['exclude'];
								
								for(car_id in allCarsList){
									
									if(exclude && exclude.indexOf(car_id)>=0){
										continue;
									}
									
									car=allCarsList[car_id] as Car;
									
									if(car){
										(carsParser as CarsParser).parseProductInfo((car), packet);
										packet.cars.push(car);
									}
									   
									
								}
								
							}
							
							
							if(!packet.example){
								
								packet.example=packet.cars;
								
								
								
							}
							
							if(packet.example && packet.example.length<=0){
								
								packet.example=packet.cars;
							}
							
							if(packet.example && packet.example.length>100){
								
								packet.example=[];
							}
							
							
							
							
							/*if(packetJSON.cars[car_id_or_func]['list']==null){
								
								// Здесь обработчки для применения ко всем машинам
								for(var index in packetJSON.cars[car_id_or_func]['list']){
									
									//packetJSON.cars[car_id_or_func]['list']=null;
									
									existsFlag=true;
									var car_id:String=packetJSON.cars[car_id_or_func]['list'][index]
									
									var car:Car=allCarsList[car_id] as Car;
									
									if(car){
										(carsParser as CarsParser).parseProductInfo((car), packet);
										packet.cars.push(car);
									}else{
										
										trace("car null");
									}
								}
								
							}*/
							
							
						}
					}
					
				}
				
			}else{
				
				packet.cars=null;
			}
			
			return packet;
		}
		
	}
}