package com.despark.tuning3d.model.tunings
{
import by.blooddy.crypto.serialization.JSONer;

import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.brands.BrandsParser;
import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.cars.CarBackground;
	import com.despark.tuning3d.model.cars.CarsParser;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.core.http.HttpClient;
import com.despark.tuning3d.model.core.http.caching.HttpCachePolicy;
import com.despark.tuning3d.model.packages.PackagesParser;
import com.despark.tuning3d.model.packages.Packet;
import com.despark.tuning3d.model.parts.Part;
import com.despark.tuning3d.model.parts.PartDisk;
import com.despark.tuning3d.model.parts.PartSlot;
import com.despark.tuning3d.model.sharing.SharingProxy;
import com.despark.tuning3d.model.social.SocialStats;
import com.despark.tuning3d.model.users.User;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.gallery.data.RequestPeriod;
import com.despark.tuning3d.view.components.timeline.ItemRenderers.Comments.CommentData;
import com.utils.DateParser;

import flash.display.BitmapData;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.filesystem.File;
import flash.filesystem.FileMode;
import flash.filesystem.FileStream;
import flash.geom.Matrix;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.net.URLRequestMethod;
import flash.system.Capabilities;
import flash.utils.ByteArray;
	import flash.utils.getTimer;

	import org.puremvc.as3.patterns.proxy.Proxy;

public class TuningsProxy extends Proxy {
		public static const NAME : String = "TuningsProxy";
		public var httpClient : HttpClient;
		private var pendingTuningFile : File;
		public function TuningsProxy() {
			super(NAME);
			if (!Constants.selectedUser) {
                pendingTuningFile = File.cacheDirectory.resolvePath("pendingTuning.json");
                pendingTuningFile.preventBackup = true;
            }
		}

		private var _lastPartLoadId : String;
        public function loadPartInfo(id:String) : void {
			var url : String = "http://assets2.3dtuning.com/api/v1/componentsInfo/";
			if(_lastPartLoadId) {
				if(_lastPartLoadId == id) {
					Tracer.print("return " + id, "PART_INFO");
					return;
				} else {
                    Tracer.print("cancel request " + id, "PART_INFO");
					httpClient.cancelAllRequests(url + _lastPartLoadId);
				}
			}

			_lastPartLoadId = id;
            //trace("http://tuning-staging-pr-26.herokuapp.com/api/v1/componentsInfo/" + id);
			httpClient.loadPath("", URLRequestMethod.GET, null, HttpCachePolicy.NONE,true,function(success:Boolean,result:Object):void
            {

                if (success) {
					var info : Object = JSONer.parse(String(result));
					Constants.partsInfoMap[_lastPartLoadId] = info;
                    _lastPartLoadId = null;
                    sendNotification(Notifications.COMPONENT_INFO_LOADED, {info:info});
	                Tracer.print("complete", "PART_INFO");
                }
                else {
                    _lastPartLoadId = null;
                    sendNotification(Notifications.COMPONENT_INFO_LOADED, {info:null});
                    Tracer.print("there is no part info for this component", "PART_INFO");
                }

            },url + id);

			Tracer.print(url + id);
        }

		public function getTuning(shortId : String, completeHandler:Function = null, notify:Boolean = true) : void {
			// http://www.3dtuning.com/api/v1/tunings/WPkTbPYWH?all=true&withDeleteds=false
			httpClient.getPath("tunings/"+shortId+"?all=true&withDeleteds=true",null, function (success: Boolean, result : String): void {
				if (success) {
					var json : Object = JSONer.parse(result);
					var tuning : Tuning = parseTuning(json);
					if(notify){
						sendNotification(Notifications.TUNING_LOADED, tuning);
                    }
					if(completeHandler!=null){
                        completeHandler(tuning, json);
					}
				}else{
					trace("Failed to load tuning");
                    if(notify){
						sendNotification(Notifications.TUNING_LOADED, null);
                    }
                    if(completeHandler!=null){
                        completeHandler(null, null, shortId);
                    }
				}

            } )
		}

        public function loadComments(tuningID:String) : void
        {
            var url : String = "http://ratings.3dtuning.com/api/v1/tunings/"+tuningID+"/comments";
            httpClient.loadPath("", URLRequestMethod.GET, null, HttpCachePolicy.NONE,true,function(success:Boolean,result:Object):void
            {
                if(success)
                {
                    var comments : Array = [];
                    var dataJSON : Object = JSONer.parse(String(result));

                    var comment  : CommentData;
                    for each(var jsn : Object in dataJSON)
                    {
                        comment = new CommentData();
                        comment.id = jsn._id;
                        comment.comment = jsn.comment;
                        comment.created = DateParser.parseDate(jsn.created);
                        comment.updatedDate = DateParser.parseDate(jsn.updated);
                        comment.tuningId = jsn.tuning;
                        comment.owner = new User({
                            username: jsn.user.name ? jsn.user.name : jsn.user.username,
                            id:jsn._id,
                            avatarPath: jsn.user.avatar ? jsn.user.avatar : ""
                        });
                        comments.push(comment);
                    }
                    sendNotification(Notifications.TUNING_COMMENTS_LOADED, {comments: comments});
                }
                else {
                    sendNotification(Notifications.TUNING_COMMENTS_LOAD_FAILED);
                }
            },url);
        }

		public function loadTunings(user:User, skip: int = 0, sortBy:String = null, period:String = null) : void{
			var params:Object = {
				userId:user.id,
				skip: skip,
				limit: 12,
				withCounters:true,
				"new":true,
				all:true
			};
			if(sortBy){
				params.sortBy = sortBy;
			}
			if(period){
				params.period = period;
			}
			httpClient.getPath(
				"tunings", params
				,
				function(success : Boolean, result : Object) : void{
				if(success){
					var tuningsJSON : Object = JSONer.parse(String(result));
					var tunings : Array = [];
					for each(var tuningJSON : Object in tuningsJSON) {
						var tuning : Tuning = parseTuning(tuningJSON);
						tunings.push(tuning);
					}
					sendNotification(Notifications.TUNINGS_LOADED, {tunnings: tunings, lastPage: tunings.length == 0});
				} else {
					sendNotification(Notifications.TUNINGS_LOADING_FAILED);
				}
			});
		}

		public static function parseTuning(tuningJSON : Object, parseCar:Boolean = true) : Tuning {
            var tuning : Tuning = new Tuning();
            tuning.id = tuningJSON._id;
            tuning.name = tuningJSON.name;
            tuning.customSnapshotsCamera = tuningJSON.customSnapshotsCamera;
			if(tuningJSON.fullHdSnapshots) {
                tuning.fullHdSnapshots = tuningJSON.fullHdSnapshots.concat();
            }
			if(tuningJSON.hasOwnProperty('haveDeletedComponents')) {
                tuning.hasDeletedComponents  = tuningJSON.haveDeletedComponents;
            }

            var i:int=0;
            var url:String="";
            if(tuningJSON.hasOwnProperty("shortId")){
                tuning.shortId = tuningJSON.shortId;
            }

            if(tuningJSON.hasOwnProperty("user")){
                tuning.userObj = tuningJSON.user;
            }
			//

			/*if(tuningJSON.hasOwnProperty("thumbnails") && tuningJSON.thumbnails.length){
                tuning.thumbnailURL = tuningJSON.thumbnails[tuningJSON.thumbnails.length-1];
            }else if(tuningJSON.hasOwnProperty("tuningThumbnail")){
                tuning.thumbnailURL = tuningJSON.tuningThumbnail;
            }
            if(!tuning.thumbnailURL && tuningJSON.snapshots && tuningJSON.snapshots.length){
                tuning.thumbnailURL = tuningJSON.snapshots[tuningJSON.snapshots.length-1];
                tuning.thumbnailURL = tuning.thumbnailURL.replace(".png", "_tmb.png");
            }
            if(tuning.thumbnailURL && tuning.thumbnailURL.indexOf("http:")!=0){
                tuning.thumbnailURL = "http:"+tuning.thumbnailURL;
            }*/

            //customSnapshotsBackgrounds
			var social : SocialStats;
            if (tuningJSON.counters) {
				social = new SocialStats();
                social.likes = tuningJSON.counters.likes;
                social.comments = tuningJSON.counters.comments;
                social.challangesCount = tuningJSON.counters.challangesCount;
				social.challangeWins = tuningJSON.counters.challangeWins;
				tuning.stats = social;
            }else if(tuningJSON.hasOwnProperty("total_wins_count")){
				social = new SocialStats();
				social.challangeWins = tuningJSON.total_wins_count;
				tuning.stats = social;
			}
            if (tuningJSON.customSnapshotsBackgrounds && tuningJSON.customSnapshotsBackgrounds.length > 0){
                tuning.backgroundId = tuningJSON.customSnapshotsBackgrounds[0];
            }else{
                tuning.backgroundId = null;
            }
            tuning.customSnapshotsBackgrounds = tuningJSON.customSnapshotsBackgrounds;
            if (tuningJSON.customSnapshots && tuningJSON.customSnapshots.length > 0) {
                tuning.customSnapshots = tuningJSON.customSnapshots;
                if (tuningJSON.customSnapshots[0].indexOf("http") >= 0){
                    tuning.imageURL = tuningJSON.customSnapshots[0];
                }else{
                    tuning.imageURL = "http:" + tuningJSON.customSnapshots[0];
                }
            }else if(tuningJSON.snapshots && tuningJSON.snapshots.length > 0) {
                if(tuningJSON.snapshots[0].indexOf("http") >= 0){
                    tuning.imageURL = tuningJSON.snapshots[0];
                }else{
                    tuning.imageURL = "http:" + tuningJSON.snapshots[0];
                }
            }

            tuning.snapshots=[];
            var isCustomSnapshots:Boolean=false;
            if(tuningJSON.customSnapshots && tuningJSON.customSnapshots.length > 0) {
                for(i=0; i<tuningJSON.customSnapshots.length; i++){
                    url=tuningJSON.customSnapshots[i];
                    if(url.indexOf("http://")<0){
                        url="http:"+url;
                    }
                    tuning.snapshots.push(url);
                    isCustomSnapshots=true;
                }
            }


            if(tuningJSON.snapshots && tuningJSON.snapshots.length > 0){
                var count:int=isCustomSnapshots ? tuningJSON.snapshots.length: 1;
                for(i=0; i<count - count > 2 ? 2 : 0; i++){
                    url=tuningJSON.snapshots[i];
                    if(url.indexOf("http://")<0){
                        url="http:"+url;
                    }
                    tuning.snapshots.push(url);
                }
            }

			if(parseCar) {
                var car2:Car;
                if (tuningJSON["car"] && tuningJSON["car"].hasOwnProperty("_id")) {
                    BrandsParser.allCarsList[tuningJSON.car._id];
                }
                if (!car2) {
                    car2 = new CarsParser().parseObject(tuningJSON.car, "") as Car;
					if(car2 && tuningJSON.car){
						car2.id = tuningJSON.car._id;
					}
                }
                tuning.car = car2;
            }
            if(tuningJSON.car && tuningJSON.car.hasOwnProperty("carChooserBgName")){
                tuning.carChooserBgName = tuningJSON.car.carChooserBgName;
            }
            //carChooserBgName	"a_garage.jpg"

            if(tuningJSON.car && tuningJSON.car.hasOwnProperty("hdsize")){
                tuning.hdsize = tuningJSON.car.hdsize;
            }

            tuning.carObj = tuningJSON.car;

			//
			var j:int;
			if(tuningJSON.hasOwnProperty('customSnapshots') && tuningJSON.customSnapshots){
                tuning.customSnapshots = tuningJSON.customSnapshots;
				for(j = 0; j < tuning.customSnapshots.length; j++){
					if(tuning.customSnapshots[j].indexOf('http://') != 0){
                        tuning.customSnapshots[j] = String(tuning.customSnapshots[j]).replace('//', 'http://');
                    }
				}
                tuning.thumbnailURL = tuning.customSnapshots[tuning.customSnapshots.length - 1];
            }
			////
			if(tuningJSON.hasOwnProperty('snapshots') && tuningJSON.snapshots && !tuning.snapshots.length){
                tuning.snapshots = tuningJSON.snapshots.concat();
				for(j = 0; j < tuning.snapshots.length; j++){
					if(tuning.snapshots[j].indexOf('http://') != 0){
                        tuning.snapshots[j] = String(tuning.snapshots[j]).replace('//', 'http://');
                    }
				}
				if(tuning.snapshots.length) {
                    tuning.thumbnailURL ||= tuning.snapshots[0];
					if(!tuning.customSnapshots.length){
                        tuning.customSnapshots = tuning.snapshots.concat();
					}
                }
            }
			return tuning;
		}

		public function deleteTuning(tuning : Tuning) : void {
			httpClient.deletePath("tunings/" + tuning.id, null, function(success : Boolean, result : Object) : void {
				if(success) {
					sendNotification(Notifications.TUNING_DELETED);
				} else {
					sendNotification(Notifications.TUNING_DELETION_FAILED);
				}
			});		
		}

		public function deleteTuningFrame(tuning : Tuning, frameIndex:int) : void {
			tuning.customSnapshots.removeAt(frameIndex);
			tuning.snapshots.removeAt(frameIndex);
			tuning.customSnapshotsBackgrounds.removeAt(frameIndex);
			tuning.customSnapshotsCamera.removeAt(frameIndex);
			tuning.fullHdSnapshots.removeAt(frameIndex);
            var data:Object = {
				customSnapshots: tuning.customSnapshots,
				customSnapshotsBackgrounds: tuning.customSnapshotsBackgrounds,
				customSnapshotsCamera: tuning.customSnapshotsCamera,
                fullHdSnapshots: tuning.fullHdSnapshots
			};
			httpClient.putPath("tunings/update/" + tuning.id + "?all=true&withDeleteds=false", data, function(success : Boolean, result : Object) : void {
				if(success) {
					sendNotification(Notifications.TUNING_FRAME_DELETED, result);
				} else {
					sendNotification(Notifications.TUNING_DELETION_FAILED);
				}
			});
		}

		public function savePendingTuning(tuning : Tuning) : void
		{
			var data : Object = {};
			data.car = tuning.car.id;
            var object : Object;
			if(tuning.car.selectedBackground)
			{
				data.background = tuning.car.selectedBackground.id;
			}
			
			data.components = [];
			
			
			
			for each(var partSlot : PartSlot in tuning.car.possiblePartSlots)
			{
                var componentParams : Object;
				if(partSlot.selectedParts)
				{
					for(var i:int=0; i<partSlot.selectedParts.length; i++){
						
						var selectedPart:Part=partSlot.selectedParts[i];
						
						if(!selectedPart) continue;
						//selectedPart
						//selectedPart.selectedMaterial.

                        componentParams = {};
						
						componentParams.component = selectedPart.id;
						
						if(!selectedPart.color){
							
							componentParams.color = null;
							
						}else if(selectedPart.color.isNoColor)
						{
							componentParams.color = null;
						}
						else
						{
							componentParams.color = selectedPart.color.value.toString(16);
						}
						
						if(selectedPart.selectedMaterial)
						{
							componentParams.material = selectedPart.selectedMaterial.name;
						}
						
						{
							componentParams.brightness = selectedPart.brightness;
						}
						
						if(selectedPart.selectedState)
						{

                            object = {};
							object[selectedPart.selectedState.nameOfState] = {active: true};
							
							componentParams.states = object;
						}
						
						data.components.push(componentParams);
						
					}
				}else
				{
					if(partSlot.selectedPart)
					{
                        componentParams = {};
						
						componentParams.component = partSlot.selectedPart.id;
						
						if(partSlot.selectedColor && partSlot.selectedColor.isNoColor)
						{
							componentParams.color = null;
						}
						else
						{	if(partSlot.selectedColor)
							componentParams.color = partSlot.selectedColor.value.toString(16);
							else componentParams.color = null;
						}
						
						if(partSlot.selectedPart.selectedMaterial)
						{
							componentParams.material = partSlot.selectedPart.selectedMaterial.name;
						}
						
						{
							componentParams.brightness = partSlot.selectedPart.brightness;
						}
						
						if(partSlot.selectedPart.selectedState) {
                            object = {};
							object[partSlot.selectedPart.selectedState.nameOfState] = {active: true};
							componentParams.states = object;
						}

						if(Car.CAR_DISKS_SLOTS.indexOf(partSlot.name)>=0){
							var partDisk:PartDisk=partSlot.selectedPart as  PartDisk;
							if(partDisk){
								componentParams.component = partDisk.getSelectedRadiusId();//partSlot.selectedPart.id;
							}
							//componentParams.sel_disk_size=partDisk.selectedDiskSize;
						}
						data.components.push(componentParams);
					}
				}
			}
			
			var bytes : ByteArray = new ByteArray();
			bytes.writeObject(data);
			bytes.position = 0;
			
			var fileStream : FileStream = new FileStream();
			var file1: File = File.cacheDirectory.resolvePath("pendingTuning.json");
			file1.preventBackup = true;
			fileStream.open(file1, FileMode.WRITE);
			fileStream.writeBytes(bytes);
			fileStream.close();
		}
		
		public function hasPendingTuning() : Boolean
		{
			return pendingTuningFile.exists;
		}
		
		public function getPendingTuningJSON() : Object
		{			
			var fileStream : FileStream = new FileStream();
			fileStream.open(pendingTuningFile, FileMode.READ);
			
			var bytes : ByteArray = new ByteArray();
			fileStream.readBytes(bytes);
			fileStream.close();
			
			var tuningJSON : Object = bytes.readObject();
			
			return tuningJSON;
		}
		
		public function deletePendingTuning() : void
		{
			if(hasPendingTuning())
			{
				pendingTuningFile.deleteFile();
			}
		}

		private var numberOfBitmapsDownloading : int;
		private var tuning : Tuning;
		public var token: String;
		private var byReplacing : Boolean;

		public function  prepareAddTuningWeakDevice(__tuning : Tuning,  __byReplacing : Boolean = false, isBackground:Boolean=false, completePrapare:Function=null) : void
		{
			//var base64String : String = Base64.encode(tuning.bitmapData.encode(tuning.bitmapData.rect, new PNGEncoderOptions(true)));
			//return;
			tuning = __tuning;
			byReplacing = __byReplacing;
			
			
			//получим token для snapshot	
			
			var paramsForGetTocken : Object;
			
			var url_str : String = "tunings/snapshots-upload-urls";
			
			
			if(isBackground){
				paramsForGetTocken = {
					forDefaults: "false",
					forCustom: "true",
					forFullHd: "true",
					newType:"true"
				};
				
			}else{
				paramsForGetTocken = {
					forDefaults: "false",
					forCustom: "false",
					newType:"true"
				}
			}
			
			
			var paramsForGetTockenString : String = JSONer.stringify(paramsForGetTocken);
			httpClient.getPath(url_str, paramsForGetTocken, function(success : Boolean, result : Object) : void 
			{
				
				completePrapare(success, result)
				//addingTokenHandler(success, result);
			});
			
		}

		public function addTuningComment(tuningId:String, text:String, onSuccess:Function = null):void{
            var url : String = "http://ratings.3dtuning.com/api/v1/tunings/"+ tuningId + "/comments?activation=true";
            httpClient.postPath("", {comment:text}, function (success : Boolean, result : Object) : void {
                if(success) {
                    var json : Object = JSONer.parse(String(result));
                    if(onSuccess != null){
                        onSuccess(json);
					}
                }else{
                    sendNotification(Notifications.EMAIL_VERIFICATION_ERROR, {event:'comment'});
                }
            }, url)
		}

        public function likeTuning(id:String, iLiked:Boolean, onLikeSuccess:Function = null, onLikeFail:Function = null, onUnlikeSuccess:Function = null, onUnlikeFail:Function = null):void {
            var url:String = 'http://ratings.3dtuning.com/api/v1/tunings/' + id + '/ratings?activation=true';
            if (iLiked) {
                httpClient.postPath('', null, function (success:Boolean, result:Object):void {
                    if(result == "requestFailed"){
                        success = false;
                    }
                    if (success) {
						if(onLikeSuccess != null){
							onLikeSuccess();
						}
                    } else {
						if(onLikeFail != null){
							onLikeFail();
						}
                        sendNotification(Notifications.EMAIL_VERIFICATION_ERROR, {event:'like'});
                    }
                }, url);
            } else {
                httpClient.deletePath('', null, function (success:Boolean, result:Object):void {
                    if(result == "requestFailed"){
                        success = false;
                    }
                    if (success) {
                        if(onUnlikeSuccess != null){
                            onUnlikeSuccess();
                        }
                    } else {
                        if(onUnlikeFail != null){
                            onUnlikeFail();
                        }
                        sendNotification(Notifications.EMAIL_VERIFICATION_ERROR, {event:'like'});
                    }
                }, url);
            }
        }

		/*public function addTuning(__tuning : Tuning, result_arr : Array, bitmapDataWithoutNeon: BitmapData,bitmapDataForBackground: BitmapData, __byReplacing : Boolean = false, bitmapDataOfBackground: BitmapData = null) : void
		{
			//var base64String : String = Base64.encode(tuning.bitmapData.encode(tuning.bitmapData.rect, new PNGEncoderOptions(true)));
			//return;
			tuning = __tuning;
			byReplacing = __byReplacing;
			
			
			//получим token для snapshot	
			
			
			
			
			
			var addingTokenHandler : Function = function(success : Boolean, result : Object) : void 
			{
				if(success)
				{
					var resultJSON : Object = JSONer.parse(result as String);
					
					token = resultJSON.token;
					var customSnapshot: String = resultJSON.customSnapshot;
					var snapshots: Array = resultJSON.snapshots;
					var fullHdSnapshot: String = resultJSON.fullHdSnapshot;
					//var snapshotObject: Object = snapshots[0];
					
					//result_arr
					//var bitmapDataForFullHD: BitmapData = null;
					//var bitmapDataForBackground: BitmapData = null;
					
					numberOfBitmapsDownloading = 0;
					numberOfBitmapsDownloading ++;//захват

					
					var min : int = result_arr.length>snapshots.length?snapshots.length:result_arr.length;
					
					for(var iii: int = 0;iii<min;iii++){
						var snapshotObject: Object = snapshots[iii];
						var bitmapData: BitmapData = result_arr[iii];
						
						var snapshot: String = snapshotObject.snapshot;
						var thumbnail: String = snapshotObject.thumbnail;
						
						//if(!customBitmapData){
						//	customBitmapData = bitmapData;
						//}
//						if(fullHdSnapshot && (iii==5)){
//							bitmapDataForFullHD = bitmapData.clone();
//						}
						
//						if(customSnapshot && (iii==0)){
//							bitmapDataForBackground = bitmapData.clone();
//						}
						
						if(snapshot){
							numberOfBitmapsDownloading ++;
							httpClient.sentBitmapData(bitmapDowloaded,iii,bitmapData,snapshot,bitmapData.height,bitmapData.width,false,true);
							
						}
						
						if(thumbnail){
							numberOfBitmapsDownloading ++;
							httpClient.sentBitmapData(bitmapDowloaded,iii,bitmapData,thumbnail,190);
						}
						
						
						bitmapData.dispose();
						
					}
					
					var additionalSnapshots: Object = resultJSON.additionalSnapshots;
					var pure: String = additionalSnapshots.pure;
					/!*if(pure && bitmapDataWithoutNeon){ // commented due to optimization of garage
						numberOfBitmapsDownloading ++;
						httpClient.sentBitmapData(bitmapDowloaded,500,bitmapDataWithoutNeon,pure,bitmapDataWithoutNeon.height,bitmapDataWithoutNeon.width,false,false);
					}*!/
					
					/!*if(customSnapshot){
						//var bdWithBack: BitmapData = createHdSnapshot(bitmapDataOfBackground,bitmapDataForBackground,true);
						//bitmapDataWithBackground это теперь не нужно и нужно удалить везде!!!!!
						numberOfBitmapsDownloading ++;
						httpClient.sentBitmapData(bitmapDowloaded,600,bitmapDataForBackground,customSnapshot,bitmapDataForBackground.height,bitmapDataForBackground.width,false,true);// тут должен быть с бекграундом
						//bdWithBack.dispose();
						
						if(fullHdSnapshot && bitmapDataOfBackground && bitmapDataForBackground){
							var newBD: BitmapData = createHdSnapshot(bitmapDataOfBackground,bitmapDataForBackground);
							numberOfBitmapsDownloading ++;
							httpClient.sentBitmapData(bitmapDowloaded,901,newBD,fullHdSnapshot,1080,1920,true,true);
							
							newBD.dispose();
						}
					}*!/
					numberOfBitmapsDownloading --;//уберем захват
					
					
					
					
					if(bitmapDataWithoutNeon){
						bitmapDataWithoutNeon.dispose();
					}
					
//					if(bitmapDataWithBackground){
//						bitmapDataWithBackground.dispose();
//					}
					
//					if(bitmapDataForFullHD){
//						bitmapDataForFullHD.dispose();
//					}
					
					if(bitmapDataForBackground){
						bitmapDataForBackground.dispose();
					}
					
					if(bitmapDataOfBackground){
						bitmapDataOfBackground.dispose();
					}
					
					
				}
				else
				{
					sendNotification(Notifications.TUNING_NOT_SAVED, tuning);
				}
				
			}
			//, 
			var paramsForGetTocken : Object;
			
			var url_str : String = "tunings/snapshots-upload-urls";
			
			
			if(bitmapDataForBackground){
				paramsForGetTocken = {
					forDefaults: "true",
					forCustom: "true",
					forFullHd: "true"
				};
				
			}else{
				paramsForGetTocken = {
					forDefaults: "true",
					forCustom: "false"
				}
			}
			
			 
			
			
				
			
			var paramsForGetTockenString : String = JSONer.stringify(paramsForGetTocken);
			httpClient.getPath(url_str, paramsForGetTocken, function(success : Boolean, result : Object) : void 
			{
				addingTokenHandler(success, result);
			});
			
			
			
			
			
		
			
		}
		
		private function bitmapDowloaded():void
		{
			numberOfBitmapsDownloading --;
			
			if(numberOfBitmapsDownloading == 0){
				continueAddingOfTuning();
			}
			
		}*/
		
		public function continueAddingOfTuning():void
		{
			
			var name:String=tuning.name;
			
			if(!name || name==""){
				
				name=tuning.car.name+" "+tuning.car.model+" "+tuning.car.year;
			}
			
			var params : Object = {
				//snapshotsData: [ base64String ],
				car: tuning.car.id,
					suspension: tuning.car.suspension*-1,
					components: [],     
					name: name,
					background: tuning.car.selectedBackground ? tuning.car.selectedBackground.id : CarBackground.DEFAULT_ID,
					isMain: false,
					isPublic: true,
					mobile: true,
					device: Capabilities.manufacturer,
					customSnapshotsCamera: tuning.customSnapshotsCamera
			};
			
			
			if(SharingProxy.LAST_USER_PAGE){
				
				params['fb_link']=SharingProxy.LAST_USER_PAGE;
			}
            var componentParams : Object
			
			for each(var partSlot : PartSlot in tuning.car.possiblePartSlots)
			{
				if(partSlot.selectedParts)
				{
					for(var i:int=0; i<partSlot.selectedParts.length; i++){
					
						var selectedPart:Part=partSlot.selectedParts[i];
						
						//selectedPart
						//selectedPart.selectedMaterial.

                        componentParams= {};
						
						componentParams.component = selectedPart.id;
						
						if(!selectedPart.color){
							
							componentParams.color = null;
						
						}else if(selectedPart.color.isNoColor)
						{
							componentParams.color = null;
						}
						else
						{
							componentParams.color = selectedPart.color.value.toString(16);
						}
						
						if(partSlot.selectedPart.selectedMaterial)
						{
							componentParams.material = selectedPart.selectedMaterial.name;
						}
						
						//if(partSlot.selectedPart.brightness != 1.0 || partSlot.selectedPart.parentSlot.name=='windows')
						{
							componentParams.brightness = selectedPart.brightness;
						}
						
						if(selectedPart.selectedState)
						{
							
							var object : Object = new Object();
							object[selectedPart.selectedState.nameOfState] = {active: true};
							
							componentParams.states = object;
						}
						
						params.components.push(componentParams);
					
					}
				}else
				if(partSlot.selectedPart)
				{
					componentParams = {};
					
					componentParams.component = partSlot.selectedPart.id;
					
					if(partSlot.selectedColor.isNoColor)
					{
						componentParams.color = null;
					}
					else
					{
						componentParams.color = partSlot.selectedColor.value.toString(16);
					}
					
					if(partSlot.selectedPart.selectedMaterial)
					{
						componentParams.material = partSlot.selectedPart.selectedMaterial.name;
					}
					
					//if(partSlot.selectedPart.brightness != 1.0 || partSlot.selectedPart.parentSlot.name=='windows')
					{
						componentParams.brightness = partSlot.selectedPart.brightness;
					}
					
					if(partSlot.selectedPart.selectedState)
					{
						
						var obj : Object = {};
						obj[partSlot.selectedPart.selectedState.nameOfState] = {active: true};
						
						componentParams.states = obj;
					}
					
					
					if(Car.CAR_DISKS_SLOTS.indexOf(partSlot.name)>=0){
						
						var partDisk:PartDisk=partSlot.selectedPart as  PartDisk;

					
						componentParams.component = partDisk.getSelectedRadiusId();//partSlot.selectedPart.id;
						

						//componentParams.sel_disk_size=partDisk.selectedDiskSize;
					}
					
					params.components.push(componentParams);
				}
				
				
			}
			
			params.token = token;
			if(tuning.tuningUserID)
			{
				params.user = tuning.tuningUserID;
			}
			var paramsString : String = JSONer.stringify(params);
			//Logger.d("SAVE TUNING:"+paramsString, this);
			var appLink:String;
			if(byReplacing) {
				appLink = "tunings";
				//Logger.d("will replace tuning!!!", this);
				httpClient.putPath(appLink + "/" + tuning.id, paramsString, function(success : Boolean, result : Object) : void {
					if(success) {
						sendNotification(Notifications.TUNING_SAVED, tuning);
					} else {
						sendNotification(Notifications.TUNING_NOT_SAVED, tuning);
					}
					if(tuning.bitmapData) {
						tuning.bitmapData.dispose();
					}
				});
			} else {
				//Logger.d("will save tuning!!!", this);
				appLink = "tuningsNew";
				httpClient.postPath(appLink, paramsString, function(success : Boolean, result : Object) : void {
					if(success) {
						var resultJSON : Object = JSONer.parse(result as String);
						tuning.id = resultJSON._id;
						tuning.shortId = resultJSON.shortId;
						tuning.tuningUserID = resultJSON.user;
						sendNotification(Notifications.TUNING_SAVED, tuning);
					}else {
						sendNotification(Notifications.TUNING_NOT_SAVED, tuning);
					}
					
					if(tuning.bitmapData) {
						tuning.bitmapData.dispose();
					}
				});
			}
			
		}
		
		private function createHdSnapshot(__bitmapDataOfBackground:BitmapData, __bitmapDataForFullHD:BitmapData, kostil: Boolean = false):BitmapData
		{
			var bitmapDataHD:BitmapData = new BitmapData(1920, 1080, true, 0x000000);
			
			
			bitmapDataHD.draw(__bitmapDataOfBackground, null, null, null, null, true);
			
			
			var matrix:Matrix = new Matrix();
			matrix.ty = 414;
			matrix.tx = 458;
			bitmapDataHD.draw(__bitmapDataForFullHD, matrix, null, null, null, true);
			
			if(kostil){
				//обрежем на 373,1004
				var matrix2:Matrix = new Matrix();
				var smallBMD:BitmapData = new BitmapData(1004, 373, true, 0x000000);
				
				matrix2.tx = (smallBMD.width-bitmapDataHD.width)/2;
				matrix2.ty = (smallBMD.height-bitmapDataHD.height)/2 - 60;
				
				smallBMD.draw(bitmapDataHD, matrix2, null, null, null, true);
				bitmapDataHD.dispose();
				return smallBMD;
				
			}
			
			return bitmapDataHD;
		}

        public function search(userId:String, searchTerm:String, skip:int, limit:int, completeHandler:Function):void {
			var url:String = HttpClient.TUNING_DOMAIN + "/api/v1/tunings/tuning-search?all=true&isDeleted=false&limit="+limit.toString()+"&q="+searchTerm+"&skip="+skip.toString()+"&userId="+userId+"&withDeleteds=false";
            httpClient.getPath(url, {}, function(success : Boolean, result : Object) : void {
				if(success) {
                    completeHandler(JSON.parse(result as String));
                }
            });
		}

		public function loadGalleryTunings(completeHandler:Function, timestamp:Number = 0, limit:int = 30, skip:int = 0, sortBy:String = "views", period:String = "all", hasBackground:Boolean = true, query:String = null, country:Boolean = false):void {
			var url:String = HttpClient.TUNING_DOMAIN + "/api/v1/tunings";
            var params:Object = {
				all:true,
				limit:limit,
				withDeleteds:false,
				withOld:true,
                timestamp: timestamp,
                orderBy: "desc"
			};

			if(country){
				params.country = true;
			}

            if(sortBy){
                params.sortBy = sortBy;
            }

            if(query){
                params.q = query;
            }

			if(period != RequestPeriod.ALL){
				params.period = period;
			}

			if(skip){
				params.skip = skip;
			}

			if(hasBackground){
				params.hasBackground = hasBackground;
			}
			httpClient.getPath(url, params, function(success : Boolean, result : Object) : void {
				if(success) {
                    completeHandler(JSON.parse(result as String));
                }
            });
		}


		public function savePromoVideoFrames(snapshots:Array, theme:String, completeHandler:Function):void{
			var snapshotUrls:Array = new Array(snapshots.length);
			for(var i:int = 0; i < snapshots.length; i++){
				sendVideoFrame(i, snapshots[i].snapshot, snapshotUrls, function(arr:Array):void{
					var createUrl:String = Constants.PROMO_VIDEO_URL + 'createTuningVideoTask';
					var loader:URLLoader = new URLLoader();
					var request:URLRequest = new URLRequest(createUrl);
					request.method = URLRequestMethod.POST;
					request.data = JSONer.stringify({"snapshots": arr, template: theme, isMobile: true});
					loader.addEventListener(Event.COMPLETE, function(event:Event):void{
						completeHandler(true, event.currentTarget.data);
					});
					loader.addEventListener(IOErrorEvent.IO_ERROR, function(event:IOErrorEvent):void{
						completeHandler(false);
					});
					loader.load(request);
				});
			}
			//httpClient.postPath(createUrl, JSONer.stringify({"snapshots": snapshots, template: theme}), completeHandler);
		}

		private function sendVideoFrame(frameIndex:int, frameContent:String, snapshotUrlList:Array, completeHandler:Function):void{
			var createUrl:String = Constants.PROMO_VIDEO_URL + 'uploadTuningVideoTask';
			var loader:URLLoader = new URLLoader();
			var request:URLRequest = new URLRequest(createUrl);
			request.method = URLRequestMethod.POST;
			request.data = JSONer.stringify({"snapshot": frameContent});
			loader.addEventListener(Event.COMPLETE, function(event:Event):void{
				var obj:Object = JSON.parse(URLLoader(event.currentTarget).data);
				snapshotUrlList[frameIndex] = {snapshot: obj["newFileName"]};
				for(var i:int = 0; i < snapshotUrlList.length; i++){
					if(!snapshotUrlList[i]){
						break;
					}
				}
				if(i == snapshotUrlList.length){
					completeHandler(snapshotUrlList);
				}
			});
			loader.addEventListener(IOErrorEvent.IO_ERROR, function(event:IOErrorEvent):void{
				snapshotUrlList[frameIndex] = "none";
				for(var i:int = 0; i < snapshotUrlList.length; i++){
					if(!snapshotUrlList[i]){
						break;
					}
				}
				if(i == snapshotUrlList.length){
					completeHandler(snapshotUrlList);
				}
			});
			loader.load(request);
		}

		public function checkPromoVideo(fileName:String, resultHandler:Function):void{
			var url:String = Constants.PROMO_VIDEO_URL + 'checkFile?fileName='+fileName;
			httpClient.getPath(url, null, resultHandler);
		}
    }
}