package com.despark.tuning3d.view {
    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.cars.CarBackgroundsProxy;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.languages.LanguagesProxy;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.brands.BrandsScreen;
    import com.despark.tuning3d.view.components.challenges.modals.*;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.core.StackedScreenExtendedMediator;
    import com.distriqt.AdsManager;
    import com.utils.ArrayUtil;

    import org.puremvc.as3.interfaces.INotification;

    import starling.events.Event;

    public class ChallengeLeaderBoardScreenMediator extends StackedScreenExtendedMediator {
    private var _proxy:ChallengesProxy;
    private var _leaderBoardScreen:ChallengeLeaderBoardScreen;
    public function ChallengeLeaderBoardScreenMediator(stackedScreen:StackedScreen) {
        super(stackedScreen);
        _leaderBoardScreen = stackedScreen as ChallengeLeaderBoardScreen;
    }

    override public function onRegister():void {
        super.onRegister();
        _proxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
        leaderBoardScreen.addEventListener(ChallengeEvent.REQUEST_LEADER_BOARD_DATA, leaderBoardRequestHandler);
        leaderBoardScreen.addEventListener(ChallengeEvent.LEADER_BOARD_USER_SELECTED, leaderBoardUserSelectedHandler);
        leaderBoardScreen.addEventListener(ChallengeLeaderBoardScreen.LOAD_BACKGROUNDS, loadUserTuningBackgroundHandler);
        leaderBoardScreen.addEventListener(ChallengeLeaderBoardScreen.LOAD_TUNING_BACKGROUNDS, loadTuningBackgroundHandler);
        leaderBoardScreen.addEventListener(StackedScreen.TUNING_SELECTED, tuningSelectHandler);
        leaderBoardScreen.addEventListener(StackedScreen.USER_DETAILS_SELECTED, userDetailsSelectedHandler);
        leaderBoardScreen.addEventListener(BrandsScreen.LANGUAGE_CHANGED, homeScreenLanguageChangedHandler);
    }

    private function leaderBoardUserSelectedHandler(event:Event):void {
        if(leaderBoardScreen.isLeaderBoard){
            sendNotification(Notifications.RELOAD_LEADER_BOARD_TUNING_LIST, event.data);
        }else{
            leaderBoardScreen.loadTuningList(event.data.tunings, event.data.user, event.data.num);
        }
    }

    private function tuningSelectHandler(event:Event):void{
        var restoreBanner:Boolean = AdsManager.$.bannerHide(false);
        var data:Object = event.data;
        data.deleteOtherScreens = false;
        data.enableCloseButton = true;
        data.eventDispatcher = leaderBoardScreen;
        data.closeHandler = function():void{
            if(restoreBanner){
                BannerStackedScreen.showAdBanner();
            }
        };
        var selectedUser:User = data.user as User;
        var currentUser:User = UsersProxy.loggedInUser();
        if((currentUser && selectedUser && selectedUser.id != currentUser.id) || !currentUser){
            Constants.selectedUser = selectedUser;
        }else{
            Constants.selectedUser = null;
        }
        sendNotification(Notifications.DISPLAY_GALLERY_TUNING_POPUP, data);
    }

    private function loadTuningBackgroundHandler(event:Event):void{
        var carBackgroundsProxy : CarBackgroundsProxy = ApplicationFacade.getInstance().retrieveProxy(CarBackgroundsProxy.NAME) as CarBackgroundsProxy;
        carBackgroundsProxy.getUrlArrayById(event.data.tuning, function(arr:Array):void{
            event.data.tuning.customSnapshotsBackgrounds = arr;
            if(event.data.tuning.customSnapshotsBackgrounds.length){
                event.data.tuning.backgroundId = event.data.tuning.customSnapshotsBackgrounds[0];
            }
            leaderBoardScreen.handleTuningBackgroundsLoaded(event.data.tuning);
        });
    }

    private function loadUserTuningBackgroundHandler(event:Event):void{
        var carBackgroundsProxy : CarBackgroundsProxy = ApplicationFacade.getInstance().retrieveProxy(CarBackgroundsProxy.NAME) as CarBackgroundsProxy;
        carBackgroundsProxy.loadBackgrounds(event.data.user.id, true, "all");
    }

    private function userDetailsSelectedHandler(event:Event):void {
        sendNotification(Notifications.DISPLAY_USER_DETAILS_POPUP, event.data);
    }

    override public function onRemove():void {
        super.onRemove();
        leaderBoardScreen.removeEventListener(ChallengeEvent.REQUEST_LEADER_BOARD_DATA, leaderBoardRequestHandler);
        leaderBoardScreen.removeEventListener(ChallengeEvent.LEADER_BOARD_USER_SELECTED, leaderBoardUserSelectedHandler);
        leaderBoardScreen.removeEventListener(ChallengeLeaderBoardScreen.LOAD_BACKGROUNDS, loadUserTuningBackgroundHandler);
        leaderBoardScreen.removeEventListener(ChallengeLeaderBoardScreen.LOAD_TUNING_BACKGROUNDS, loadTuningBackgroundHandler);
        leaderBoardScreen.removeEventListener(StackedScreen.TUNING_SELECTED, tuningSelectHandler);
        leaderBoardScreen.removeEventListener(StackedScreen.USER_DETAILS_SELECTED, userDetailsSelectedHandler);
        leaderBoardScreen.removeEventListener(BrandsScreen.LANGUAGE_CHANGED, homeScreenLanguageChangedHandler);
    }

    private function leaderBoardRequestHandler(event:Event):void {
        _proxy.getLoaderBoardByPeriod(event.data.limit, event.data.skip, event.data.toDate, event.data.country, event.data.period, function (data:Object):void {
            leaderBoardScreen.loadData(data as Array);
        });
    }

    private function backgroundsLoadedHandler(backgrounds:Array):void {
        leaderBoardScreen.handleCarBackgroundsLoaded(backgrounds);
    }

    private function homeScreenLanguageChangedHandler(event : Event) : void {
        var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
        languagesProxy.loadLanguage(leaderBoardScreen.selectedLanguage);
        if (alertView) {
            alertView.hide();
        }
    }

    private function handleLanguageLoaded(loadedLanguage : Language) : void {
        if(leaderBoardScreen) {
            leaderBoardScreen.updateWithLanguage(loadedLanguage);
            if(alertView) {
                alertView.hide();
            }
            sendNotification(Notifications.LANGUAGE_CHANGED);
        }
    }

    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
            Notifications.CAR_BACKGROUNDS_LOADED,
            Notifications.LANGUAGE_CHANGED,
            Notifications.LANGUAGE_LOADED
        ]);
    }

    override public function handleNotification(note:INotification):void {
        switch(note.getName()) {
            case Notifications.CAR_BACKGROUNDS_LOADED:
                backgroundsLoadedHandler(note.getBody() as Array);
                break;

            case Notifications.LANGUAGE_LOADED:
                handleLanguageLoaded(note.getBody().language);
                break;
        }
        super.handleNotification(note);
    }

        public function get leaderBoardScreen():ChallengeLeaderBoardScreen {
            return _leaderBoardScreen;
        }
    }
}
