package com.despark.tuning3d.view {
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.cars.CarBackgroundsProxy;
import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.friend.FriendProxy;
import com.despark.tuning3d.model.languages.LanguagesProxy;
import com.despark.tuning3d.model.parts.Part;
import com.despark.tuning3d.model.parts.PartMaterial;
import com.despark.tuning3d.model.parts.PartMaterialImageTest;
import com.despark.tuning3d.model.parts.PartSlot;
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.tunings.TuningsProxy;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.brands.BrandsScreen;
import com.despark.tuning3d.view.components.brands.BrandsScreen;
import com.despark.tuning3d.view.components.challenges.modals.ChallengeLeaderBoardTuningScreen;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
import com.despark.tuning3d.view.components.core.StackedScreen;
import com.despark.tuning3d.view.components.timeline.FeedData;
import com.despark.tuning3d.view.core.StackedScreenMediator;
import com.distriqt.AdsManager;
import com.utils.ArrayUtil;
import com.utils.RandomSplashManager;

import flash.filesystem.File;
import flash.utils.setTimeout;

import org.puremvc.as3.interfaces.INotification;

import starling.core.Starling;

import starling.events.Event;
import starling.utils.ScaleMode;

public class ChallengeLeaderBoardTuningScreenMediator extends StackedScreenMediator {
    private var _leaderBoardTuningListScreen:ChallengeLeaderBoardTuningScreen;
    private var _garage_delay:Number;
    public function ChallengeLeaderBoardTuningScreenMediator(stackedScreen:StackedScreen) {
        super(stackedScreen);
        _leaderBoardTuningListScreen = stackedScreen as ChallengeLeaderBoardTuningScreen;
    }

    override public function onRegister():void {
        super.onRegister();
        // Main menu events
        leaderBoardTuningListScreen.addEventListener(StackedScreen.HOME_SELECTED, mainMenuHomeSelectedHandler);
        leaderBoardTuningListScreen.addEventListener(StackedScreen.BRANDS_SELECTED, mainMenuBrandsSelectedHandler);
        leaderBoardTuningListScreen.addEventListener(StackedScreen.VOTE_SELECTED, mainMenuVotesSelectedHandler);
        leaderBoardTuningListScreen.addEventListener(StackedScreen.GARAGE_SELECTED, mainMenuGarageSelectedHandler);
        leaderBoardTuningListScreen.addEventListener(StackedScreen.FRIENDS_SELECTED, mainMenuFriendsSelectedHandler);
        leaderBoardTuningListScreen.addEventListener(StackedScreen.NOTIFICATION_SELECTED, mainMenuNotificationsSelectedHandler);
        leaderBoardTuningListScreen.addEventListener(StackedScreen.CHALLENGES_SELECTED, challengeSelectHandler);
        leaderBoardTuningListScreen.addEventListener(StackedScreen.CHALLENGE_NOTIFICATIONS_SELECTED, challengeNotesSelectHandler);
        leaderBoardTuningListScreen.addEventListener(StackedScreen.LEADER_BOARD_SELECTED, leaderBoardSelectHandler);
        leaderBoardTuningListScreen.addEventListener(BrandsScreen.LANGUAGE_CHANGED,mainMenuLanguageChangedHandler);
        leaderBoardTuningListScreen.addEventListener(BrandsScreen.LOGOUT_SELECTED, mainMenuLogoutSelectedHandler);
        leaderBoardTuningListScreen.addEventListener(BrandsScreen.LOGIN_SELECTED, mainMenuLoginSelectedHandler);
        leaderBoardTuningListScreen.addEventListener(StackedScreen.TIMELINE_SECLECTED, mainMenuTimelineSelectedHandler);
        leaderBoardTuningListScreen.addEventListener(StackedScreen.GALLERY_SELECTED, gallerySelectHandler);

        leaderBoardTuningListScreen.addEventListener(ChallengeEvent.LOAD_TUNING_COMMENTS, tuningScreenSocialSelectedHandler);
        leaderBoardTuningListScreen.addEventListener(ChallengeEvent.CHALLENGE_REQUEST, challengeRequestHandler);
        leaderBoardTuningListScreen.addEventListener(StackedScreen.TUNING_SELECTED, tuningSelectedHandler);
        leaderBoardTuningListScreen.addEventListener(StackedScreen.BACK_BUTTON_SELECTED, closeSelectedHandler);
        leaderBoardTuningListScreen.addEventListener(ChallengeLeaderBoardTuningScreen.LOAD_BACKGROUNDS, loadTuningBackgroundHandler);
    }

    override public function onRemove():void {
        super.onRemove();
        // Main menu events
        leaderBoardTuningListScreen.removeEventListener(StackedScreen.HOME_SELECTED, mainMenuHomeSelectedHandler);
        leaderBoardTuningListScreen.removeEventListener(StackedScreen.BRANDS_SELECTED, mainMenuBrandsSelectedHandler);
        leaderBoardTuningListScreen.removeEventListener(StackedScreen.VOTE_SELECTED, mainMenuVotesSelectedHandler);
        leaderBoardTuningListScreen.removeEventListener(StackedScreen.GARAGE_SELECTED, mainMenuGarageSelectedHandler);
        leaderBoardTuningListScreen.removeEventListener(StackedScreen.FRIENDS_SELECTED, mainMenuFriendsSelectedHandler);
        leaderBoardTuningListScreen.removeEventListener(StackedScreen.NOTIFICATION_SELECTED, mainMenuNotificationsSelectedHandler);
        leaderBoardTuningListScreen.removeEventListener(StackedScreen.CHALLENGES_SELECTED, challengeSelectHandler);
        leaderBoardTuningListScreen.removeEventListener(StackedScreen.CHALLENGE_NOTIFICATIONS_SELECTED, challengeNotesSelectHandler);
        leaderBoardTuningListScreen.removeEventListener(StackedScreen.LEADER_BOARD_SELECTED, leaderBoardSelectHandler);
        leaderBoardTuningListScreen.removeEventListener(BrandsScreen.LANGUAGE_CHANGED,mainMenuLanguageChangedHandler);
        leaderBoardTuningListScreen.removeEventListener(BrandsScreen.LOGOUT_SELECTED, mainMenuLogoutSelectedHandler);
        leaderBoardTuningListScreen.removeEventListener(BrandsScreen.LOGIN_SELECTED, mainMenuLoginSelectedHandler);
        leaderBoardTuningListScreen.removeEventListener(StackedScreen.TIMELINE_SECLECTED, mainMenuTimelineSelectedHandler);
        leaderBoardTuningListScreen.removeEventListener(StackedScreen.GALLERY_SELECTED, gallerySelectHandler);

        leaderBoardTuningListScreen.removeEventListener(ChallengeEvent.LOAD_TUNING_COMMENTS, tuningScreenSocialSelectedHandler);
        leaderBoardTuningListScreen.removeEventListener(ChallengeEvent.CHALLENGE_REQUEST, challengeRequestHandler);
        leaderBoardTuningListScreen.removeEventListener(StackedScreen.TUNING_SELECTED, tuningSelectedHandler);
        leaderBoardTuningListScreen.removeEventListener(ChallengeLeaderBoardTuningScreen.LOAD_BACKGROUNDS, loadTuningBackgroundHandler);
    }

    private function loadTuningBackgroundHandler(event:Event):void{
        var carBackgroundsProxy : CarBackgroundsProxy = ApplicationFacade.getInstance().retrieveProxy(CarBackgroundsProxy.NAME) as CarBackgroundsProxy;
        carBackgroundsProxy.loadBackgrounds(event.data.user.id, true, "all");
    }

    private function mainMenuTimelineSelectedHandler(event:Event):void{
        if(leaderBoardTuningListScreen.mainMenuView.pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_LOGIN) {
            userAuthenticate(BrandsScreenMediator.AUTH_ACTION_LOGIN);
        } else {
            userAuthenticate(BrandsScreenMediator.AUTH_ACTION_TIMELINE);
        }
    }

    private function challengeRequestHandler(event:Event):void{
        sendNotification(Notifications.DISPLAY_START_CHALLENGE_POPUP, event.data);
    }

    private function gallerySelectHandler(event : Event) : void {
        sendNotification(Notifications.DISPLAY_GALLERY_SCREEN);
    }

    private function closeSelectedHandler(event : Event) : void {
        if(BrandsScreen.FIRST_SHOW_DONE){
            sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP, {backLevel: stackedScreen.backLevel});
        }else{
            sendNotification(Notifications.DISPLAY_PREVIOUS_SCREEN, {backLevel: stackedScreen.backLevel});
        }
    }

    private function tuningSelectedHandler(event : Event) : void {
        var selectedTuning : Tuning = leaderBoardTuningListScreen.selectedTuning;
        var info : Object = {
            car : selectedTuning.car
        };
        var userProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(event.data && event.data.isEditing && userProxy.currentUser) {
            if (selectedTuning.tuningUserID == userProxy.currentUser.id || (selectedTuning.userObj && selectedTuning.userObj.hasOwnProperty("_id") && selectedTuning.userObj._id == userProxy.currentUser.id) || selectedTuning.userObj == userProxy.currentUser.id) {
                info["tuning"] = selectedTuning;
            }
        }

        var partSlot:PartSlot = new PartSlot(selectedTuning.car);
        var partCarSplash:Part = new Part(partSlot);
        var partMaterial:PartMaterial = new PartMaterial(partCarSplash);
        var randomSplashUrl:String = RandomSplashManager.getRandomUrl();

        trace("randomSplashUrl:", randomSplashUrl);

        var carSplashImageProvider:PartMaterialImageTest = new PartMaterialImageTest(randomSplashUrl,"0",null,partMaterial);
        carSplashImageProvider.frameIndex = 0;
        carSplashImageProvider.angleIndex = 0;
        carSplashImageProvider.isEncrypted = false;


        var carSplashImage:ImageLoaderWithStates = new ImageLoaderWithStates();
        carSplashImage.overrideDispose = true;
        carSplashImage.snapToPixels = false;
        if(Constants.TOTAL_MEMORY<1024 && Constants.SCALE_FACTOR<2){
            carSplashImage.bitmapScale = 0.5;
        }
        carSplashImage.visible = false;
        leaderBoardTuningListScreen.stage.addChild(carSplashImage);
        carSplashImage.completeHandler = function(image:ImageLoaderWithStates):void{
            info.carSplashImage=carSplashImage;
            sendNotification(Notifications.DISPLAY_TUNING_SCREEN, info);
        };
        carSplashImage.scaleMode =	ScaleMode.NONE;
        carSplashImage.persistentImageProvider = carSplashImageProvider;
    }

    private function userAuthenticate(pendingforauction : String):void {
        var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if (!usersProxy.currentUser) {
            _garage_delay = 1.5;
        } else {
            _garage_delay = 0.2;
        }

        ImageLoaderWithStates.disposeWhenRemovedFromStage = false;

        pendingForAuthAction = pendingforauction;
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function handleChallengesLoaded(body:Object):void {
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        var user:User = usersProxy.currentUser;
        var result:Array = [];
        var challenges:Array = body.challenges as Array;
        for(var i:int = 0; i < challenges.length; i++){
            if(challenges[i].owner.user == user.id){
                result.push(challenges[i].owner.tuning);
            }else if(challenges[i].receiver.user == user.id){
                result.push(challenges[i].receiver.tuning);
            }
        }
        leaderBoardTuningListScreen.appendChallenges(result);
    }

    private function reloadLeaderBoardData(body:Object):void {
        leaderBoardTuningListScreen.loadTuningList(body.tunings, body.user, body.num);
    }

    private function backgroundsLoadedHandler(backgrounds:Array):void {
        leaderBoardTuningListScreen.handleCarBackgroundsLoaded(backgrounds);
    }

    private function handleUserAuthenticated(wasAlreadyAuthenticated:Boolean):void {
        if (!stackedScreen.stage) {
            return;
        }
        var friendProxy:FriendProxy = ApplicationFacade.getInstance().retrieveProxy(FriendProxy.NAME) as FriendProxy;
        friendProxy.loadPendings();
        ImageLoaderWithStates.disposeWhenRemovedFromStage = true;
        switch (pendingForAuthAction){
            case BrandsScreenMediator.AUTH_ACTION_NOTIFICATIONS:
                sendNotification(Notifications.DISPLAY_IN_APP_NOTIFICATION_POPUP);
                break;
            case BrandsScreenMediator.AUTH_ACTION_CHALLENGE_NOTIFICATIONS:
                sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP);
                break;
            case BrandsScreenMediator.AUTH_ACTION_FRIENDS:
                sendNotification(Notifications.DISPLAY_FRIENDS_LIST_SCREEN);
                break;
            case BrandsScreenMediator.AUTH_ACTION_TIMELINE:
                Starling.juggler.delayCall(function ():void {
                    sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN);
                }, 2);
                break;
            case BrandsScreenMediator.AUTH_ACTION_GARAGE:
                Starling.juggler.delayCall(function ():void {
                    sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN);
                }, _garage_delay);
                break;

        }
        pendingForAuthAction = null;
    }

    private function leaderBoardSelectHandler(event : Event) : void {
        sendNotification(Notifications.DISPLAY_CHALLENGE_LEADERBOARD_POPUP, {isLeaderBoard:true});
    }

    private function challengeNotesSelectHandler(event : Event) : void {
        if(leaderBoardTuningListScreen.mainMenuView.pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_LOGIN) {
            userAuthenticate(BrandsScreenMediator.AUTH_ACTION_LOGIN);
        } else {
            userAuthenticate(BrandsScreenMediator.AUTH_ACTION_CHALLENGE_NOTIFICATIONS);
        }
    }

    private function mainMenuLoginSelectedHandler(event : Event) : void {
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(!usersProxy.currentUser) {
            _garage_delay=1.5;
        } else {
            _garage_delay=0.2;
        }
        pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_LOGIN;
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function challengeSelectHandler(event : Event) : void {
        sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN);
    }

    private function mainMenuLogoutSelectedHandler(e:Event):void {
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        usersProxy.logoutCurrentUser();
        AlertView.show(AlertView.language.translate("mob_saving", "loggedOut"), -1, false);
        leaderBoardTuningListScreen.updateNotifications();
        leaderBoardTuningListScreen.ShowDone();
    }

    private function mainMenuLanguageChangedHandler(event:Event):void {
        var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
        languagesProxy.loadLanguage(leaderBoardTuningListScreen.selectedLanguage);
    }

    private function mainMenuVotesSelectedHandler(event:Event):void {
        sendNotification(Notifications.DISPLAY_VOTES_LIST_SCREEN, null);
    }

    private function mainMenuFriendsSelectedHandler(event : Event) : void {
        if(leaderBoardTuningListScreen.mainMenuView.pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_LOGIN) {
            userAuthenticate(BrandsScreenMediator.AUTH_ACTION_LOGIN);
        } else {
            userAuthenticate(BrandsScreenMediator.AUTH_ACTION_FRIENDS);
        }
    }

    private function mainMenuNotificationsSelectedHandler(event : Event) : void {
        if(leaderBoardTuningListScreen.mainMenuView.pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_LOGIN) {
            userAuthenticate(BrandsScreenMediator.AUTH_ACTION_LOGIN);
        } else {
            userAuthenticate(BrandsScreenMediator.AUTH_ACTION_NOTIFICATIONS);
        }
    }

    private function mainMenuGarageSelectedHandler(event : Event): void {
        if((Constants.selectedUser && FeedData.CURRENT_USER && Constants.selectedUser.id != FeedData.CURRENT_USER.id) || (event.data && event.data.enableClosing)){
            Starling.juggler.delayCall(function () :void {
                sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN, {deleteOtherScreens : true});
            }, 0.5);
        }else {
            Starling.juggler.delayCall(function () : void{
                sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN, null);
            }, 0.5);
        }
    }

    private function mainMenuHomeSelectedHandler(event:Event):void {
        sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP);
    }

    private function mainMenuBrandsSelectedHandler(event:Event):void {
        if(HomeScreenPopUpMediator.isFirstOpen) {
            sendNotification(Notifications.DISPLAY_BRANDS_SCREEN);
        } else {
            sendNotification(Notifications.DISPLAY_ROOT_SCREEN);
        }
        HomeScreenPopUpMediator.isFirstOpen = false;
    }

    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
            Notifications.TUNING_COMMENTS_LOADED,
            Notifications.USER_AUTHENTICATED,
            Notifications.CHALLENGES_LOADED,
            Notifications.RELOAD_LEADER_BOARD_TUNING_LIST,
            Notifications.CAR_BACKGROUNDS_LOADED
        ]);
    }

    override public function handleNotification(note:INotification):void {
        switch(note.getName()) {
            case Notifications.TUNING_COMMENTS_LOADED:
                handleCommentDataLoaded(note.getBody());
                break;
            case Notifications.USER_AUTHENTICATED:
                handleUserAuthenticated(note.getBody().wasAlreadyAuthenticated);
                break;
            case Notifications.CHALLENGES_LOADED:
                handleChallengesLoaded(note.getBody());
                break;
            case Notifications.RELOAD_LEADER_BOARD_TUNING_LIST:
                reloadLeaderBoardData(note.getBody());
                break;
            case Notifications.CAR_BACKGROUNDS_LOADED:
                backgroundsLoadedHandler(note.getBody() as Array);
                break;

        }
        super.handleNotification(note);
    }

    private function handleCommentDataLoaded(info:Object):void {
        leaderBoardTuningListScreen.comments = info.comments;
        if(alertView) alertView.hide();
    }

    private function tuningScreenSocialSelectedHandler(event:Event):void {
        if(leaderBoardTuningListScreen.selectedTuning) {
            (facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy).loadComments(leaderBoardTuningListScreen.selectedTuning.id);
        }
    }

    public function get leaderBoardTuningListScreen():ChallengeLeaderBoardTuningScreen {
        return _leaderBoardTuningListScreen;
    }
}
}
