package com.despark.tuning3d.view {
import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.parts.Part;
import com.despark.tuning3d.model.parts.PartMaterial;
import com.despark.tuning3d.model.parts.PartMaterialImageTest;
import com.despark.tuning3d.model.parts.PartSlot;
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.tunings.TuningsProxy;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.challenges.modals.ChallengeNotificationTuningScreen;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
import com.despark.tuning3d.view.components.core.PopUp;
import com.despark.tuning3d.view.components.core.StackedScreen;
import com.despark.tuning3d.view.core.PopUpMediator;
import com.utils.ArrayUtil;
import com.utils.RandomSplashManager;

import org.puremvc.as3.interfaces.INotification;

import starling.events.Event;
import starling.utils.ScaleMode;

public class ChallengeNotificationTuningScreenMediator extends PopUpMediator {
    private var _tuningPopup:ChallengeNotificationTuningScreen;
    public function ChallengeNotificationTuningScreenMediator(stackedScreen:PopUp) {
        super(stackedScreen);
        _tuningPopup = popUp as ChallengeNotificationTuningScreen;
    }

    override public function onRegister():void {
        super.onRegister();
        // Main menu events

        tuningPopup.addEventListener(ChallengeEvent.LOAD_TUNING_COMMENTS, tuningScreenSocialSelectedHandler);
        tuningPopup.addEventListener(ChallengeEvent.CHALLENGE_REQUEST, challengeRequestHandler);
        tuningPopup.addEventListener(StackedScreen.TUNING_SELECTED, tuningSelectedHandler);
    }

    override public function onRemove():void {
        super.onRemove();
        // Main menu events

        tuningPopup.removeEventListener(ChallengeEvent.LOAD_TUNING_COMMENTS, tuningScreenSocialSelectedHandler);
        tuningPopup.removeEventListener(ChallengeEvent.CHALLENGE_REQUEST, challengeRequestHandler);
        tuningPopup.removeEventListener(StackedScreen.TUNING_SELECTED, tuningSelectedHandler);
    }

    private function challengeRequestHandler(event:Event):void{
        sendNotification(Notifications.DISPLAY_START_CHALLENGE_POPUP, event.data);
    }

    private function tuningSelectedHandler(event : Event) : void {

        var selectedTuning : Tuning = tuningPopup.selectedTuning;
        var info : Object = {
            car : selectedTuning.car
        };
        var userProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(event.data && event.data.isEditing && userProxy.currentUser) {
            if (selectedTuning.tuningUserID == userProxy.currentUser.id || (selectedTuning.userObj && selectedTuning.userObj.hasOwnProperty("_id") && selectedTuning.userObj._id == userProxy.currentUser.id) || selectedTuning.userObj == userProxy.currentUser.id) {
                info["tuning"] = selectedTuning;
            }
        }

        var partSlot:PartSlot = new PartSlot(selectedTuning.car);
        var partCarSplash:Part = new Part(partSlot);
        var partMaterial:PartMaterial = new PartMaterial(partCarSplash);
        var randomSplashUrl:String = RandomSplashManager.getRandomUrl();

        trace("randomSplashUrl:", randomSplashUrl);

        var carSplashImageProvider:PartMaterialImageTest = new PartMaterialImageTest(randomSplashUrl,"0",null,partMaterial);
        carSplashImageProvider.frameIndex = 0;
        carSplashImageProvider.angleIndex = 0;
        carSplashImageProvider.isEncrypted = false;


        var carSplashImage:ImageLoaderWithStates = new ImageLoaderWithStates();
        carSplashImage.overrideDispose = true;
        carSplashImage.snapToPixels = false;
        if(Constants.TOTAL_MEMORY<1024 && Constants.SCALE_FACTOR<2){
            carSplashImage.bitmapScale = 0.5;
        }
        carSplashImage.visible = false;
        tuningPopup.stage.addChild(carSplashImage);
        carSplashImage.completeHandler = function(image:ImageLoaderWithStates):void{
            info.carSplashImage=carSplashImage;
            sendNotification(Notifications.DISPLAY_TUNING_SCREEN, info);
        };
        carSplashImage.scaleMode =	ScaleMode.NONE;
        carSplashImage.persistentImageProvider = carSplashImageProvider;
    }

    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
            Notifications.TUNING_COMMENTS_LOADED
        ]);
    }

    override public function handleNotification(note:INotification):void {
        switch(note.getName()) {
            case Notifications.TUNING_COMMENTS_LOADED:
                handleCommentDataLoaded(note.getBody());
                break;
        }
        super.handleNotification(note);
    }

    private function handleCommentDataLoaded(info:Object):void {
        tuningPopup.comments = info.comments;
        if(alertView) alertView.hide();
    }

    private function tuningScreenSocialSelectedHandler(event:Event):void {
        if(tuningPopup.selectedTuning) {
            (facade.retrieveProxy(TuningsProxy.NAME) as TuningsProxy).loadComments(tuningPopup.selectedTuning.id);
        }
    }

    public function get tuningPopup():ChallengeNotificationTuningScreen {
        return _tuningPopup;
    }
}
}
