package com.despark.tuning3d.view {
import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.view.components.challenges.modals.*;
import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.PopUp;
import com.despark.tuning3d.view.core.PopUpMediator;
import com.utils.ArrayUtil;

import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;

    import starling.events.Event;

public class ChallengeNotificationsPopupMediator extends PopUpMediator {
    private var _proxy:ChallengesProxy;
    private var _notificationPopUp:ChallengeNotificationsPopup;
    public function ChallengeNotificationsPopupMediator(popUp:PopUp) {
        super(popUp);
        _notificationPopUp = popUp as ChallengeNotificationsPopup;
    }

    override public function onRegister():void {
        super.onRegister();
        _proxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;

        _notificationPopUp.addEventListener(ChallengeEvent.GET_NOTIFICATIONS, getNotificationsHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.GET_INVITATIONS, getRequestsHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.CHALLENGE_REQUEST, handleChallengeRequest);

        _notificationPopUp.addEventListener(ChallengeEvent.DECLINE_CHALLENGE, declineNotificationHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.VIEW_USER_TUNINGS, viewUserTuningsHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.VIEW_CHALLENGE_TUNING, viewChallengeTuningHandler);
        _notificationPopUp.addEventListener(ChallengeEvent.VIEW_REQUEST_TUNING, tuningSelectedHandler);


        _proxy.markAllUnreadNotifications(function ():void {
            sendNotification(Notifications.CHALLENGE_NOTIFICATIONS_VIEWED);
        });
        //
        Starling.juggler.delayCall(function():void{
            BannerStackedScreen.hideAdBanner(false);
        }, 0.2);
    }

    private function tuningSelectedHandler(event:Event):void {
        _notificationPopUp.close();
        var closeHandler:Function = function ():void {
            sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP);
        };
        var data:Object = event.data;
        data.deleteOtherScreens = false;
        data.enableCloseButton = true;
        data.closeHandler =  closeHandler;
        sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_TUNING_POPUP, data);
    }

    private function viewUserTuningsHandler(event:Event):void {
        // _notificationPopUp.close();
        Constants.selectedUser = event.data.user;
        var closeHandler:Function = function ():void {
            sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP);
        };
        // sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN, {deleteOtherScreens : false, enableCloseButton: true, closeHandler: closeHandler});
        sendNotification(Notifications.DISPLAY_USER_DETAILS_POPUP, event.data);
    }

    private function viewChallengeTuningHandler(event:Event):void {
        _notificationPopUp.close();
        var closeHandler:Function = function ():void {
            sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP);
        };
        sendNotification(Notifications.DISPLAY_CHALLENGE_TUNING_POPUP, {deleteOtherScreens : false, enableCloseButton: true, closeHandler: closeHandler, data:event.data});
    }

    override public function onRemove():void {
        super.onRemove();
        _notificationPopUp.removeEventListener(ChallengeEvent.GET_NOTIFICATIONS, getNotificationsHandler);
        _notificationPopUp.removeEventListener(ChallengeEvent.GET_INVITATIONS, getRequestsHandler);
        _notificationPopUp.removeEventListener(ChallengeEvent.CHALLENGE_REQUEST, handleChallengeRequest);

        _notificationPopUp.removeEventListener(ChallengeEvent.DECLINE_CHALLENGE, declineNotificationHandler);
        _notificationPopUp.removeEventListener(ChallengeEvent.VIEW_USER_TUNINGS, viewUserTuningsHandler);
    }

    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
            Notifications.CHALLENGE_ACCEPTED_NOTIFICATION,
            Notifications.CHALLENGE_DECLINED_NOTIFICATION,
            Notifications.CHALLENGE_MAX_COUNT_EXCEEDED_ERROR
        ]);
    }

    override public function handleNotification(note:INotification):void {
        switch(note.getName()) {
            case Notifications.CHALLENGE_ACCEPTED_NOTIFICATION:
                handlerAcceptChallenge(note.getBody());
                break;
            case Notifications.CHALLENGE_DECLINED_NOTIFICATION:
                handlerDeclineChallenge(note.getBody());
                break;

            case Notifications.CHALLENGE_MAX_COUNT_EXCEEDED_ERROR:
                handleStartChallengeDenied(note.getBody());
                note.getBody().shown = true;
                break;
        }
        super.handleNotification(note);
    }

    private function handlerDeclineChallenge(body:Object):void {
        _notificationPopUp.removeItem(body.item);
    }

    private function handlerAcceptChallenge(body:Object):void {
        _notificationPopUp.removeItem(body.item);
    }

    private function getRequestsHandler(event:Event):void {
        _proxy.getRequests(event.data.skip, event.data.limit, _notificationPopUp.loadInvitations);
    }

    private function getNotificationsHandler(event:Event):void {
        _proxy.getNotifications(event.data.skip, event.data.limit, _notificationPopUp.loadNotifications);
    }

    private function handleChallengeRequest(event:Event):void {
        sendNotification(Notifications.DISPLAY_START_CHALLENGE_POPUP, event.data);
    }

    private function declineNotificationHandler(event:Event):void {
        sendNotification(Notifications.CHALLENGE_DECLINED_NOTIFICATION, {item:event.data.notification});
    }

    private function handleStartChallengeDenied(data:Object):void {
        if(!data.hasOwnProperty("shown") || !data.shown){
            AlertView.show(data.message, 10, false);
        }
    }

}
}
