package com.despark.tuning3d.view {
    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.friend.FriendProxy;
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.brands.BrandsScreen;
    import com.despark.tuning3d.view.components.challenges.ChallengesListScreen;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.instructions.TutorialView;
    import com.despark.tuning3d.view.components.timeline.TimelineScreen;
    import com.despark.tuning3d.view.components.tunings.editing.TuningScreen;
import com.despark.tuning3d.view.core.HomeScreenPopUp;
    import com.despark.tuning3d.view.core.StackedScreenExtendedMediator;
    import com.despark.tuning3d.view.core.StackedScreenMediator;
import com.utils.ArrayUtil;

import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;

    import starling.events.Event;

    public class ChallengesListScreenMediator extends StackedScreenMediator {
        private var _garage_delay:Number = 0.5;
        public function ChallengesListScreenMediator(stackedScreen:StackedScreen) {
            super(stackedScreen);
        }

        override public function onRegister():void {
            super.onRegister();
            challengesListScreen.addEventListener(Event.ADDED_TO_STAGE, challengeScreenAddedToStageHandler);
            challengesListScreen.addEventListener(StackedScreen.VOTE_SELECTED, challengeScreenVoteSelectedHandler);

            challengesListScreen.addEventListener(BrandsScreen.LOGOUT_SELECTED, challengeScreenLogoutSelectedHandler);
            challengesListScreen.addEventListener(BrandsScreen.LOGIN_SELECTED, challengeScreenLoginSelectedHandler);

            challengesListScreen.addEventListener(StackedScreen.FRIENDS_SELECTED, challengeScreenFriendsSelectedHandler);
            challengesListScreen.addEventListener(ChallengeEvent.CHALLENGE_REQUEST, handleChallengeRequest);

            challengesListScreen.addEventListener(BrandsScreen.INSTRUCTION_SELECTED, challengeScreenInstructionSelectedHandler); // +
            challengesListScreen.addEventListener(StackedScreen.GARAGE_SELECTED, challengeScreenGarageSelectedHandler); //+
            challengesListScreen.addEventListener(StackedScreen.BRANDS_SELECTED, challengeScreenBrandsSelectedSelectedHandler); // +
            challengesListScreen.addEventListener(StackedScreen.HOME_SELECTED, challengeScreenHomeSelectedSelectedHandler); //+
            challengesListScreen.addEventListener(StackedScreen.TIMELINE_SECLECTED, challengeScreenTimelineSelectedHandler); //+
            challengesListScreen.addEventListener(StackedScreen.GALLERY_SELECTED, gallerySelectHandler); //+
            challengesListScreen.addEventListener(StackedScreen.MAILS_SELECTED, mailSelectedHandler);
            challengesListScreen.addEventListener(StackedScreen.USER_DETAILS_SELECTED, userDetailsSelectedHandler);


            challengesListScreen.addEventListener(StackedScreen.NOTIFICATION_SELECTED, challengeScreenNotificationsSelectedHandler); // +
            challengesListScreen.addEventListener(StackedScreen.CHALLENGE_NOTIFICATIONS_SELECTED, challengeNotesSelectHandler);
            challengesListScreen.addEventListener(ChallengeEvent.VIEW_CHALLENGE_TUNING, viewChallengeTuningHandler);
            challengesListScreen.addEventListener(ChallengesListScreen.UPDATE_CHALLENEGES, challengesListReload);
            challengesListScreen.addEventListener(StackedScreen.LEADER_BOARD_SELECTED, leaderBoardSelectHandler);
            challengesListScreen.addEventListener(BrandsScreen.LOGIN_SELECTED, loginSelectHandler);

        }

        override public function onRemove():void {
            super.onRemove();
            challengesListScreen.removeEventListener(Event.ADDED_TO_STAGE, challengeScreenAddedToStageHandler);
            challengesListScreen.removeEventListener(StackedScreen.VOTE_SELECTED, challengeScreenVoteSelectedHandler);
            challengesListScreen.removeEventListener(BrandsScreen.LOGOUT_SELECTED, challengeScreenLogoutSelectedHandler);
            challengesListScreen.removeEventListener(StackedScreen.FRIENDS_SELECTED, challengeScreenFriendsSelectedHandler);
            challengesListScreen.removeEventListener(BrandsScreen.INSTRUCTION_SELECTED, challengeScreenInstructionSelectedHandler);
            challengesListScreen.removeEventListener(StackedScreen.GARAGE_SELECTED, challengeScreenGarageSelectedHandler);
            challengesListScreen.removeEventListener(StackedScreen.BRANDS_SELECTED, challengeScreenBrandsSelectedSelectedHandler);
            challengesListScreen.removeEventListener(StackedScreen.HOME_SELECTED, challengeScreenHomeSelectedSelectedHandler);
            challengesListScreen.removeEventListener(StackedScreen.TIMELINE_SECLECTED, challengeScreenTimelineSelectedHandler);
            challengesListScreen.removeEventListener(StackedScreen.GALLERY_SELECTED, gallerySelectHandler);
            challengesListScreen.removeEventListener(StackedScreen.MAILS_SELECTED, mailSelectedHandler);
            challengesListScreen.removeEventListener(StackedScreen.USER_DETAILS_SELECTED, userDetailsSelectedHandler);

            challengesListScreen.removeEventListener(StackedScreen.NOTIFICATION_SELECTED, challengeScreenNotificationsSelectedHandler);
            challengesListScreen.removeEventListener(StackedScreen.CHALLENGE_NOTIFICATIONS_SELECTED, challengeNotesSelectHandler);
            challengesListScreen.removeEventListener(ChallengesListScreen.UPDATE_CHALLENEGES, challengesListReload);
            challengesListScreen.removeEventListener(BrandsScreen.LOGIN_SELECTED, challengeScreenLoginSelectedHandler);
            challengesListScreen.removeEventListener(ChallengeEvent.CHALLENGE_REQUEST, handleChallengeRequest);
            challengesListScreen.removeEventListener(ChallengeEvent.VIEW_CHALLENGE_TUNING, viewChallengeTuningHandler);
            challengesListScreen.removeEventListener(BrandsScreen.LOGIN_SELECTED, loginSelectHandler);
            challengesListScreen.dispose();
        }

        private function leaderBoardSelectHandler(event : Event) : void {
            BannerStackedScreen.hideAdBanner(false);
            sendNotification(Notifications.DISPLAY_CHALLENGE_LEADERBOARD_POPUP, {showBannerAfterExit:true});
        }

        private function viewChallengeTuningHandler(event:Event):void {
            BannerStackedScreen.hideAdBanner(false);
            var closeHandler:Function = function (): void {
                BannerStackedScreen.showAdBanner();
            };
            sendNotification(Notifications.DISPLAY_CHALLENGE_TUNING_POPUP, {deleteOtherScreens : false, enableCloseButton: true, closeHandler: closeHandler, data:event.data});
        }

        private function handleChallengeRequest(event:Event):void {
            sendNotification(Notifications.DISPLAY_START_CHALLENGE_POPUP, event.data);
        }

        private function gallerySelectHandler(event : Event) : void {
            sendNotification(Notifications.DISPLAY_GALLERY_SCREEN);
        }

        private function mailSelectedHandler(event:Event):void {
            pendingForAuthAction = StackedScreenExtendedMediator.AUTH_ACTION_MAILS;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        private function userDetailsSelectedHandler(event:Event):void {
            sendNotification(Notifications.DISPLAY_USER_DETAILS_POPUP, event.data);
        }

        private function loginSelectHandler(event : Event) : void {
            sendNotification(Notifications.DISPLAY_LOGIN_SCREEN);
        }



        private function challengesListReload(event:Event): void {
            var challengeProxy : ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            var skip: Number = 0;
            if ( event.data && event.data.skip) {
                skip = event.data.skip
            }

            if (challengesListScreen.showAllChallenge) {
                skip -= 2;
                if (skip < 0) {
                    skip = 0;
                }
            }
            var isPulling: Boolean = false;
            if (event.data && event.data.isPulling) {
                isPulling = event.data.isPulling
            }

            var sorting: String = 'timeleft';
            if (isPulling) {
                skip = 0;
                sorting = '-updated';
            }

            if (challengesListScreen.getChallengesCount() == 0 && challengesListScreen.showAllChallenge) {
                sorting = '-updated';
            }

            challengeProxy.getChallenges(2, skip, challengesListScreen.showAllChallenge, challengesListScreen.firstChallengeDate, sorting, isPulling);
        }

        public function get challengesListScreen():ChallengesListScreen {
            return viewComponent as ChallengesListScreen;
        }


        private function challengeScreenLogoutSelectedHandler(event: Event):void {
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            usersProxy.logoutCurrentUser();
            AlertView.show(AlertView.language.translate("mob_saving", "loggedOut"), -1, false);
            challengesListScreen.updateNotifications();
            this.challengesListScreen.checkTopButtonsAccess();
        }

        private function challengeScreenLoginSelectedHandler(e: Event): void {
            sendNotification(BrandsScreen.LOGIN_SELECTED);
        }

        private function challengeScreenInstructionSelectedHandler(event: Event):void {
            TutorialView.showCustomUrl();
        }

        private function challengeScreenVoteSelectedHandler(event: Event):void {
            sendNotification(Notifications.DISPLAY_VOTES_LIST_SCREEN, null);
        }

        private function challengeScreenGarageSelectedHandler(event: Event):void {
            if(challengesListScreen.mainMenuView.pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_LOGIN)
            {
                userAuthenticate(BrandsScreenMediator.AUTH_ACTION_LOGIN);
            }
            else {
                userAuthenticate(BrandsScreenMediator.AUTH_ACTION_GARAGE);
            }
        }

        private function challengeScreenBrandsSelectedSelectedHandler(event: Event):void {
            if(HomeScreenPopUpMediator.isFirstOpen) {
                sendNotification(Notifications.DISPLAY_BRANDS_SCREEN);
            } else {
                sendNotification(Notifications.DISPLAY_ROOT_SCREEN);
            }
            HomeScreenPopUpMediator.isFirstOpen = false;
        }

        private function challengeScreenHomeSelectedSelectedHandler(event: Event):void {
            sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP);
        }

        private function challengeScreenTimelineSelectedHandler(event: Event):void {
            if(challengesListScreen.mainMenuView.pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_LOGIN)
            {
                userAuthenticate(BrandsScreenMediator.AUTH_ACTION_LOGIN);
            }
            else {
                userAuthenticate(BrandsScreenMediator.AUTH_ACTION_TIMELINE);
            }
        }

        private function challengeScreenFriendsSelectedHandler(event: Event):void {
            if(challengesListScreen.mainMenuView.pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_LOGIN)
            {
                userAuthenticate(BrandsScreenMediator.AUTH_ACTION_LOGIN);
            }
            else {
                userAuthenticate(BrandsScreenMediator.AUTH_ACTION_FRIENDS);
            }
        }

        private function challengeScreenNotificationsSelectedHandler(event: Event):void {
            if(challengesListScreen.mainMenuView.pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_LOGIN)
            {
                userAuthenticate(BrandsScreenMediator.AUTH_ACTION_LOGIN);
            }
            else {
                userAuthenticate(BrandsScreenMediator.AUTH_ACTION_NOTIFICATIONS);
            }
        }
        private function userAuthenticate(pendingforauction : String):void
        {
            var usersProxy:UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;

            if (!usersProxy.currentUser) _garage_delay = 1.5;
            else _garage_delay = 0.2;

            ImageLoaderWithStates.disposeWhenRemovedFromStage = false;

            pendingForAuthAction = pendingforauction;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        private function challengeNotesSelectHandler(event: Event):void {
            if(challengesListScreen.mainMenuView.pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_LOGIN) {
                userAuthenticate(BrandsScreenMediator.AUTH_ACTION_LOGIN);
            } else {
                userAuthenticate(BrandsScreenMediator.AUTH_ACTION_CHALLENGE_NOTIFICATIONS);
            }
            //sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP);
        }

        private function challengeScreenAddedToStageHandler(event: Event): void {
            loadChallenges();
        }



        override public function listNotificationInterests():Array {
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
                Notifications.USER_AUTHENTICATED,
                Notifications.CHALLENGES_LIST_LOADED,
                Notifications.CHALLENGES_LIST_LOADING_FAILED,
                Notifications.APPLICATION_BECAME_ACTIVE,
                Notifications.USER_LOGGED_OUT,
                Notifications.USER_AUTHENTICATED,
                Notifications.USER_SESSION_EXTENDED,
                Notifications.USER_SESSION_EXTENDING_FAILED,
                Notifications.USER_SESSION_EXTENDING_NOT_POSSIBLE,
                Notifications.SHARING_READY,
                Notifications.TUNING_SHARING_MESSAGE_ENTERED,
                Notifications.TIMELINE_SCREEN_FEED_PREVIEW,
                Notifications.CHALLENGE_MAX_COUNT_EXCEEDED_ERROR
            ]);
        }


        override public function handleNotification(note:INotification):void {
            switch (note.getName()) {
                case Notifications.CHALLENGES_LIST_LOADED:
                    handleChallengesLoaded(note.getBody());
                    break;

                case Notifications.CHALLENGES_LIST_LOADING_FAILED:
                    handleChallengesLoadingFailed();
                    break;


                case Notifications.APPLICATION_BECAME_ACTIVE:
                    handleApplicationBecameActive();
                    break;

                case Notifications.USER_LOGGED_OUT:
                    handleUserLoggedOut();
                    break;

                case Notifications.USER_AUTHENTICATED:
                    handleUserAuthenticated();
                    break;

                case Notifications.USER_SESSION_EXTENDED:
                    handleUserSessionExtented();
                    break;

                case Notifications.USER_SESSION_EXTENDING_FAILED:
                    handleUserSessionExtendingFailed();
                    break;

                case Notifications.USER_SESSION_EXTENDING_NOT_POSSIBLE:
                    handleUserSessionExtendingNotPossible();
                    break;

                case Notifications.TIMELINE_SCREEN_FEED_PREVIEW:
                    handleDisplayTimelineScreenFeed(note.getBody());
                    break;

                case Notifications.CHALLENGE_MAX_COUNT_EXCEEDED_ERROR:
                    handleStartChallengeDenied(note.getBody());
                    note.getBody().shown = true;
                    break;

            }

            super.handleNotification(note);
        }



            private function handleApplicationBecameActive() : void {
            if(Device.isIos()) {
                return;
            }

            if(this.challengesListScreen) {
                challengesListScreen.invalidate();
            }
        }

        private function handleUserLoggedOut() : void {
            sendNotification(Notifications.PENDING_FRIENDS_LOADED, {friends: null, unread:false});
        }

        private function handleDisplayTimelineScreenFeed(data:Object):void {
            sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN, data);
        }

        private function handleUserAuthenticated() : void {

            trace("TuningScreenMediator handleUserAuthenticated 1");
            if (!stackedScreen.stage) {
                return;
            }


            trace("TuningScreenMediator handleUserAuthenticated 2");
            var friendProxy: FriendProxy = ApplicationFacade.getInstance().retrieveProxy(FriendProxy.NAME) as FriendProxy;
            friendProxy.loadPendings();
            ImageLoaderWithStates.disposeWhenRemovedFromStage = true;


            if (pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_NOTIFICATIONS) {
                BannerStackedScreen.hideAdBanner(false);
                sendNotification(Notifications.DISPLAY_IN_APP_NOTIFICATION_POPUP, {showBannerAfterExit: true});
            } else if(pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_CHALLENGE_NOTIFICATIONS) {
                BannerStackedScreen.hideAdBanner(false);
                sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP, {showBannerAfterExit: true});
            } else if (pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_FRIENDS){
                BannerStackedScreen.hideAdBanner(false);
                sendNotification(Notifications.DISPLAY_FRIENDS_LIST_SCREEN, {showBannerAfterExit: true});
            }

            if (pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_TIMELINE){
                Starling.juggler.delayCall(function ():void {
                    sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN);
                }, 2);
            }else if(pendingForAuthAction == StackedScreenExtendedMediator.AUTH_ACTION_MAILS){
                Starling.juggler.delayCall(function():void{
                    sendNotification(Notifications.DISPLAY_MAIL_SCREEN);
                }, 0.2);
            }else if (pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_GARAGE){
                Starling.juggler.delayCall(function ():void {
                    sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN);
                }, _garage_delay);
            }

            challengesListScreen.checkTopButtonsAccess();
            pendingForAuthAction = null;
        }

        private function handleUserSessionExtented() : void {

        }

        private function handleUserSessionExtendingFailed() : void {

        }

        private function handleUserSessionExtendingNotPossible() : void {

        }

        private function handleChallengesLoaded(data: Object): void {
            if (data.isPulling) {
                challengesListScreen.challenges = null;
            }
            if (data && data.data && data.data[0] && challengesListScreen.firstChallengeDate) {
                challengesListScreen.firstChallengeDate = data.data[0].start_date;
            }
            challengesListScreen.challenges = data.data;
            if (data.isPulling) {
                challengesListScreen.listPullingEnd();
            }
//
        }

        private function handleChallengesLoadingFailed(): void {

        }

        private function loadChallenges(): void {
            var challengeProxy: ChallengesProxy = facade.retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengeProxy.getChallenges(2, 0, challengesListScreen.showAllChallenge, challengesListScreen.firstChallengeDate);
        }

        private function handleStartChallengeDenied(data:Object):void {
            if(!data.hasOwnProperty("shown") || !data.shown){
                AlertView.show(data.message, 10, false);
            }
        }
    }
}
