package com.despark.tuning3d.view {
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.brands.BrandsProxy;
import com.despark.tuning3d.model.cars.Car;
import com.despark.tuning3d.model.cars.CarsProxy;
import com.despark.tuning3d.model.core.AppMode;
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.friend.FriendProxy;
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.LanguagesProxy;
import com.despark.tuning3d.model.packages.PackagesProxy;
import com.despark.tuning3d.model.parts.Part;
import com.despark.tuning3d.model.parts.PartMaterial;
import com.despark.tuning3d.model.parts.PartMaterialImageTest;
import com.despark.tuning3d.model.parts.PartSlot;
import com.despark.tuning3d.model.settings.Settings;
import com.despark.tuning3d.model.settings.SettingsProxy;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.timeline.TimelineProxy;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.brands.BrandsScreen;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.AlertViewHL;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
import com.despark.tuning3d.view.components.core.StackedScreen;
import com.despark.tuning3d.view.components.friends.FriendsListPopUp;
import com.despark.tuning3d.view.core.*;
    import com.despark.tuning3d.view.core.StackedScreenExtendedMediator;
    import com.distriqt.AdsManager;
    import com.utils.ArrayUtil;
import com.utils.RandomSplashManager;

import org.puremvc.as3.interfaces.INotification;

import starling.core.Starling;
import starling.events.Event;
import starling.utils.ScaleMode;

public class HomeScreenPopUpMediator extends StackedScreenMediator {

    public static var isFirstOpen : Boolean = true;
    private var _screen_delay:Number;
    public function HomeScreenPopUpMediator(screen:StackedScreen) {
        super(screen);
    }

    public function get homeScreenPopUp() : HomeScreenPopUp {
        return HomeScreenPopUp(stackedScreen);
    }


    override public function onRegister():void {
        super.onRegister();

        homeScreenPopUp.addEventListener(HomeScreenPopUp.MODE_SELECTED, onModeSelectedHandler);
        AppMode.self.addEventListener("featuredCompleteLoad", onFeaturedCompletedLoad);
        loadData();
        //
        homeScreenPopUp.addEventListener(StackedScreen.TUNING_SELECTED, carSelectedHandler);

        homeScreenPopUp.addEventListener(StackedScreen.VOTE_SELECTED, votesSelectedHandler);
        homeScreenPopUp.addEventListener(StackedScreen.GARAGE_SELECTED, garageSelectedHandler);
        homeScreenPopUp.addEventListener(StackedScreen.USER_GARAGE_SELECTED, userGarageSelectedHandler);
        homeScreenPopUp.addEventListener(StackedScreen.TIMELINE_SECLECTED, timelineSelectedHandler);
        homeScreenPopUp.addEventListener(StackedScreen.FRIENDS_SELECTED, friendsSelectedHandler);
        homeScreenPopUp.addEventListener(StackedScreen.NOTIFICATION_SELECTED, notificationsSelectedHandler);
        homeScreenPopUp.addEventListener(StackedScreen.GALLERY_SELECTED, gallerySelectHandler);
        homeScreenPopUp.addEventListener(StackedScreen.MAILS_SELECTED, mailSelectedHandler);

        homeScreenPopUp.addEventListener(BrandsScreen.LANGUAGE_CHANGED, languageChangedHandler);
        homeScreenPopUp.addEventListener(BrandsScreen.LOGIN_SELECTED, loginSelectedHandler);
        homeScreenPopUp.addEventListener(BrandsScreen.LOGOUT_SELECTED, logoutSelectedHandler);

        homeScreenPopUp.addEventListener(StackedScreen.CHALLENGES_SELECTED, challengeSelectHandler);
        homeScreenPopUp.addEventListener(StackedScreen.CHALLENGE_NOTIFICATIONS_SELECTED, challengeNotesSelectHandler);
        homeScreenPopUp.addEventListener(StackedScreen.LEADER_BOARD_SELECTED, leaderBoardSelectHandler);
        //
        if(isFirstOpen){
            var brandsProxy : BrandsProxy = facade.retrieveProxy(BrandsProxy.NAME) as BrandsProxy;
            brandsProxy.loadBrands();
            Starling.juggler.delayCall(updateNotifications, .5);
        }

        BannerStackedScreen.showAdBanner();
    }


    private function votesSelectedHandler(event:Event):void {
        AppMode.self.setModeTuning();
        sendNotification(Notifications.DISPLAY_VOTES_LIST_SCREEN, null);
    }

    private function garageSelectedHandler(event : Event): void {
        Constants.selectedUser = null;
        _screen_delay = 0.2;
        pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_GARAGE;
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function userGarageSelectedHandler(event : Event): void {
        AppMode.self.setModeTuning();
        if(Constants.selectedUser) {
            _screen_delay = 1.5;
            pendingForAuthAction = null;
            sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN);
        } else {
            _screen_delay = 0.2;
            pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_GARAGE;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }
    }

    private function timelineSelectedHandler( e : Event) : void {
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(!usersProxy.currentUser) {
            _screen_delay = 1.5;
        } else {
            _screen_delay = 0.2;
        }
        AppMode.self.setModeTuning();
        pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_TIMELINE;
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function friendsSelectedHandler(event : Event) : void {
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(!usersProxy.currentUser) {
            _screen_delay=1.5;
        } else {
            _screen_delay=0.2;
        }
        pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_FRIENDS;
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function notificationsSelectedHandler(event : Event) : void{
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(!usersProxy.currentUser) {
            _screen_delay = 1.5;
        } else {
            _screen_delay = 0.2;
        }
        pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_NOTIFICATIONS;
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function gallerySelectHandler(event : Event) : void {
        AppMode.self.setModeTuning();
        sendNotification(Notifications.DISPLAY_GALLERY_SCREEN);
    }

    private function mailSelectedHandler(event:Event):void {
        _screen_delay = 0.1;
        pendingForAuthAction = StackedScreenExtendedMediator.AUTH_ACTION_MAILS;
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function languageChangedHandler(event:Event):void {
        var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
        languagesProxy.loadLanguage(homeScreenPopUp.selectedLanguage);
    }

    private function loginSelectedHandler(event : Event) : void {
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(!usersProxy.currentUser) {
            _screen_delay = 1.5;
        } else {
            _screen_delay=0.2;
        }
        pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_LOGIN;
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function logoutSelectedHandler(e:Event):void {
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        usersProxy.logoutCurrentUser();
        AlertView.show(AlertView.language.translate("mob_saving", "loggedOut"), -1, false);

        updateNotifications();
    }

    private function leaderBoardSelectHandler(event : Event) : void {
        AdsManager.$.bannerHide(false);
        sendNotification(Notifications.DISPLAY_CHALLENGE_LEADERBOARD_POPUP, {showBannerAfterExit:true, bannerScreen:BannerScreen.HOME_SCREEN});
    }

    private function challengeNotesSelectHandler(event : Event) : void {
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(!usersProxy.currentUser) {
            _screen_delay = 1.5;
        } else {
            _screen_delay = 0.2;
        }

        pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_CHALLENGE_NOTIFICATIONS;
        sendNotification(Notifications.AUTHENTICATE_USER);
    }

    private function carSelectedHandler(event : Event) : void {
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(usersProxy.httpClient.isUserSigned){
            continueCarSelect(event);
            return;
        }

        var textOfMessage:String = AlertView.language.translate("carchooserTop", "agreement_text");
        var title:String = AlertView.language.translate("carchooserTop", "agreement_title");
        var agree:String = AlertView.language.translate("carchooserTop", "agree");
        var disagree:String = AlertView.language.translate("carchooserTop", "disagree");
        if(!textOfMessage){
            textOfMessage = "Вы согласны с <u><a href=\"http://google.com\">лицензионным соглашением</a></u>?";
        }
        SoundManager.playSound(SoundManager.SELECT_PART);
        AlertViewHL.show(title,textOfMessage, agree,disagree, -1, false, function(yesSelected : Boolean) : void {
            if(yesSelected){
                continueCarSelect(event);
            }
        },  0x49cc46);
    }


    private function continueCarSelect(event : Event) : void {
        if(event.data.screenMode == AppMode.MODE_TUNING){
            AppMode.self.setModeTuning();
        }else if(event.data.screenMode == AppMode.MODE_TRUCK_CONFIGURATOR){
            AppMode.self.setModeTruckConfigurator();
        }

        var car:Car = new Car();
        car.id = event.data.carId;
        var info : Object = {
            car : car
        };
        var partSlot:PartSlot = new PartSlot(car);
        var partCarSplash:Part = new Part(partSlot);
        var partMaterial:PartMaterial = new PartMaterial(partCarSplash);
        var randomSplashUrl:String = RandomSplashManager.getRandomUrl();

        var carSplashImageProvider:PartMaterialImageTest = new PartMaterialImageTest(randomSplashUrl,"0",null,partMaterial);
        carSplashImageProvider.frameIndex = 0;
        carSplashImageProvider.angleIndex = 0;
        carSplashImageProvider.isEncrypted = false;


        var carSplashImage:ImageLoaderWithStates = new ImageLoaderWithStates();
        carSplashImage.overrideDispose = true;
        carSplashImage.snapToPixels = false;
        if(Constants.TOTAL_MEMORY<1024 && Constants.SCALE_FACTOR<2){
            carSplashImage.bitmapScale = 0.5;
        }
        carSplashImage.visible = false;
        if(Starling.current.stage) {
            Starling.current.stage.addChild(carSplashImage);
            carSplashImage.completeHandler = function (image:ImageLoaderWithStates):void {
                info.carSplashImage = carSplashImage;
                sendNotification(Notifications.DISPLAY_TUNING_SCREEN, info);
            };
            carSplashImage.scaleMode = ScaleMode.NONE;
            carSplashImage.persistentImageProvider = carSplashImageProvider;
        }
    }


    private function challengeSelectHandler(event : Event) : void {
        sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN);
    }

    private function onFeaturedCompletedLoad(event:Event):void {
        if(homeScreenPopUp.brandsLoaded()) {
            AppMode.self.removeEventListener("featuredCompleteLoad", onFeaturedCompletedLoad);
        }
    }

    private function onModeSelectedHandler(event:Event):void {
        if(isFirstOpen) {
            sendNotification(Notifications.DISPLAY_BRANDS_SCREEN);
        } else {
            sendNotification(Notifications.DISPLAY_ROOT_SCREEN);
        }
        isFirstOpen = false;
    }

    override public function onRemove():void {
        super.onRemove();
        homeScreenPopUp.removeEventListener(HomeScreenPopUp.MODE_SELECTED, onModeSelectedHandler);
        AdsManager.$.bannerHide(true);
    }

    private function loadData() : void {
        var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
        languagesProxy.loadLanguages();

        var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
        var languageToLoad : Language = new Language(null);

        if(languagesProxy.defaultLanguageCode.indexOf("ru")<0 && settingsProxy.currentSettings.selectedLanguageCode && settingsProxy.currentSettings.selectedLanguageCode.indexOf("en")<0){
            settingsProxy.currentSettings.selectedLanguageCode = "en-US";
        }

        languageToLoad.code = settingsProxy.currentSettings.selectedLanguageCode;
        languagesProxy.loadLanguage(languageToLoad);
    }

    private function handleUserLoggedIn():void{
        var brandsProxy : BrandsProxy = facade.retrieveProxy(BrandsProxy.NAME) as BrandsProxy;
        brandsProxy.loadBrands();
    }

    private function handleUserAuthenticated() : void {
        if(!homeScreenPopUp.userSignedIn){
            var userProxy:UsersProxy =  facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(userProxy.currentUser && userProxy.currentUser.isSuperAdmin){
                var packagesProxy:PackagesProxy =  facade.retrieveProxy(PackagesProxy.NAME) as PackagesProxy;
                packagesProxy.packages = [];
                CarsProxy.IS_FULL = true;
                Constants.IN_APP_PURCHASE_ENABLED = false;
                Constants.update_API_URLS(true);
            }else{
                Constants.update_API_URLS(false);
                CarsProxy.IS_FULL = false;
                Constants.IN_APP_PURCHASE_ENABLED = true;
            }
        }

        if (homeScreenPopUp){
            homeScreenPopUp.userSignedIn = true;
            homeScreenPopUp.mainMenuView.loginButtonTriggeredHandler();
            if (homeScreenPopUp.mainMenuView.pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_LOGIN) {
                Starling.juggler.delayCall(updateNotifications, .5);
                Starling.juggler.delayCall(updateNotifications, 1);
                return;
            }
        }

        if(homeScreenPopUp && !homeScreenPopUp.stage) {
            return;
        }
        var data:Object = {showBannerAfterExit: true, bannerScreen: BannerScreen.BANNER_SCREEN};
        if(pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_FRIENDS) {
            Starling.juggler.delayCall(function():void{
                AppMode.self.setModeTuning();
                sendNotification(Notifications.DISPLAY_FRIENDS_LIST_SCREEN, data);
            }, _screen_delay);
        }

        if(pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_GARAGE){
            Starling.juggler.delayCall(function():void{
                AppMode.self.setModeTuning();
                sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN);
            }, _screen_delay);
        }

        if(pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_TIMELINE) {
            Starling.juggler.delayCall(function():void{
                AppMode.self.setModeTuning();
                sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN);
            }, _screen_delay);
        }else if(pendingForAuthAction == StackedScreenExtendedMediator.AUTH_ACTION_MAILS){
            Starling.juggler.delayCall(function():void{
                AppMode.self.setModeTuning();
                sendNotification(Notifications.DISPLAY_MAIL_SCREEN);
            }, _screen_delay);
        }else if(pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_NOTIFICATIONS){
            Starling.juggler.delayCall(function():void{
                sendNotification(Notifications.DISPLAY_IN_APP_NOTIFICATION_POPUP, data);
            }, _screen_delay);
        }else if(pendingForAuthAction == BrandsScreenMediator.AUTH_ACTION_CHALLENGE_NOTIFICATIONS){
            Starling.juggler.delayCall(function():void{
                sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP, data);
            }, _screen_delay);
        }
    }


    private function handleLanguageLoaded(loadedLanguage : Language) : void {
        if(homeScreenPopUp) {
            homeScreenPopUp.updateWithLanguage(loadedLanguage);
            if(alertView) {
                alertView.hide();
            }
            sendNotification(Notifications.LANGUAGE_CHANGED);
        }
    }

    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [

            Notifications.LANGUAGES_LOADED,
            Notifications.LANGUAGES_LOADING_FAILED,
            Notifications.LANGUAGE_LOADED,
            Notifications.LANGUAGE_LOADING_FAILED,
            Notifications.USER_LOGGED_OUT,

            Notifications.SHARING_READY,
            Notifications.TUNING_SHARING_MESSAGE_ENTERED,
            Notifications.SHARING_PREPARING_ERROR,
            Notifications.SHARING_PHOTO_UPLOADED,
            Notifications.SHARING_PHOTO_UPLOADING_ERROR,

            Notifications.CHALLENGE_NOTIFICATIONS_VIEWED,
            Notifications.SHARING_PHOTO_UPLOADING_ERROR,
            Notifications.AUTHENTICATE_USER,
            Notifications.CAR_BACKGROUNDS_LOADED,
            Notifications.USER_AUTHENTICATED,
            Notifications.USER_LOGGED_IN,
            Notifications.DISPLAY_PREVIOUS_SCREEN,
            Notifications.REMOVE_POP_UP,

            Notifications.BRANDS_LOADED,
            Notifications.PACKAGES_LOADED,
            Notifications.PENDING_FRIENDS_LOADED,
            Notifications.TIMELINE_SCREEN_FEED_PREVIEW
        ]);
    }

    override public function handleNotification(note:INotification):void {
        switch(note.getName()) {
            case Notifications.BRANDS_LOADED:
                handleBrandsLoaded(note.getBody().brands);
                break;
            case Notifications.LANGUAGE_LOADED:
                handleLanguageLoaded(note.getBody().language);
                break;
            case Notifications.INTERNET_CONNECTED:
                handleInternetConnected();
                break;

            case Notifications.INTERNET_DISCONNECTED:
                handleInternetDisconnected();
                break;

            case Notifications.USER_LOGGED_OUT:
                handleUserLoggedOut();
                break;

            case Notifications.USER_AUTHENTICATED:
                handleUserAuthenticated();
                break;

            case Notifications.USER_LOGGED_IN:
                handleUserLoggedIn();
                break;

            case Notifications.LANGUAGES_LOADED:
                handleLanguagesLoaded();
                break;

            case Notifications.LANGUAGES_LOADING_FAILED:
                handleLanguagesLoadingFailed();
                break;

            case Notifications.LANGUAGE_LOADING_FAILED:
                handleLanguageLoadingFailed();
                break;

            case Notifications.PENDING_FRIENDS_LOADED:
                handlePendingFriendsLoaded(note.getBody());
                homeScreenPopUp.updateNotificationCount();
                break;

            case Notifications.TIMELINE_SCREEN_FEED_PREVIEW:
                handleDisplayTimelineScreenFeed(note.getBody());
                break;

            case Notifications.PACKAGES_LOADED:
                handleCarPackagesLoaded(note.getBody().packages);
                break;

            case Notifications.AUTHENTICATE_USER:
                handleAuthenticateUser();
                break;


        }

        super.handleNotification(note);
    }

    private function handleAuthenticateUser():void {
        Starling.juggler.delayCall(AdsManager.$.bannerHide, 0.1);
    }

    private function handleBrandsLoaded(brands:Array):void{
        var packagesProxy:PackagesProxy =  facade.retrieveProxy(PackagesProxy.NAME) as PackagesProxy;
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(usersProxy.currentUser && usersProxy.currentUser.isSuperAdmin){
            packagesProxy.packages = [];
            CarsProxy.IS_FULL = true;
            Constants.IN_APP_PURCHASE_ENABLED = false;
            Constants.update_API_URLS(true);
        }else{
            Constants.update_API_URLS(false);
            CarsProxy.IS_FULL = false;
            Constants.IN_APP_PURCHASE_ENABLED = true;
            packagesProxy.loadPackages(BrandsProxy.makeAllCarsList(brands));
        }
    }

    private function handleCarPackagesLoaded(packages : Array) : void {
        var usersProxy:UsersProxy= facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        if(Constants.IN_APP_PURCHASE_ENABLED) {
            usersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            usersProxy.getPurchases();
        }
        // packagesLoaded = true;
    }

    private function handleDisplayTimelineScreenFeed(data:Object):void {
        sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN, data);
    }

    private function handlePendingFriendsLoaded(friends: Object): void {
        homeScreenPopUp.dispatchEventWith(FriendsListPopUp.PENDING_FRIENDS_READ, false, friends);
    }

    private function updateNotifications():void{
        var friendProxy:FriendProxy = ApplicationFacade.getInstance().retrieveProxy(FriendProxy.NAME) as FriendProxy;
        friendProxy.loadPendings();
        (facade.retrieveProxy(TimelineProxy.NAME) as TimelineProxy).loadUnreadNotification(function (obj:Object) : void {
            homeScreenPopUp.updateNotificationCount();
        });
    }

    private function handleInternetConnected():void{
        trace("handleInternetConnected");
    }


    private function handleInternetDisconnected():void{
        AlertView.show("Lost internet connection",-1, false);
    }


    private function handleUserLoggedOut() : void {
        sendNotification(Notifications.PENDING_FRIENDS_LOADED, {friends: null, unread:false});
        homeScreenPopUp.updateNotificationCount();
        handleUserLoggedIn();
    }


    private function handleLanguagesLoaded():void{
        Starling.juggler.delayCall(refreshSettings, 1.5);
    }

    private function handleLanguagesLoadingFailed() : void {
        homeScreenPopUp.selectedLanguage = null;
    }

    private function handleLanguageLoadingFailed():void {

    }

    private function refreshSettings() : void {
        var settingsProxy : SettingsProxy = facade.retrieveProxy(SettingsProxy.NAME) as SettingsProxy;
        var languagesProxy : LanguagesProxy = facade.retrieveProxy(LanguagesProxy.NAME) as LanguagesProxy;
        var settings : Settings = settingsProxy.currentSettings;

        if(!settings.selectedLanguageCode) {
            settings.selectedLanguageCode = languagesProxy.defaultLanguageCode;
        }

        homeScreenPopUp.selectedLanguage = languagesProxy.getLanguage(settings.selectedLanguageCode);
        if(!homeScreenPopUp.selectedLanguage){
            settings.selectedLanguageCode = "en-US";
            homeScreenPopUp.selectedLanguage = languagesProxy.getLanguage(settings.selectedLanguageCode);
        }

        homeScreenPopUp.languages = languagesProxy.languages;
        if(homeScreenPopUp.languages != null && homeScreenPopUp.languages.length<=0){
            languagesProxy.loadLanguages();
        }else if (homeScreenPopUp.languages == null){
            languagesProxy.loadLanguages();
        }
    }
}
}
