package com.despark.tuning3d.view {
import com.despark.tuning3d.Notifications;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.timeline.TimelineProxy;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.NotificationView.*;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.PopUp;
import com.despark.tuning3d.view.components.core.StackedScreen;
import com.despark.tuning3d.view.components.timeline.FeedData;
import com.despark.tuning3d.view.core.PopUpMediator;
import com.utils.ArrayUtil;

import org.puremvc.as3.interfaces.INotification;

import starling.core.Starling;

import starling.events.Event;

public class InAppNotificationPopUpMediator extends PopUpMediator{

    private var _loadingAlert:AlertView;
    private var _isFeedUpdating: Boolean;

    public function InAppNotificationPopUpMediator(popUp:PopUp) {
        super(popUp);
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        FeedData.CURRENT_USER = usersProxy.currentUser;
    }

    override public function onRegister():void {
        super.onRegister();
        var proxy : TimelineProxy = facade.retrieveProxy(TimelineProxy.NAME) as TimelineProxy;
        proxy.loadNotifications(FeedData.CURRENT_USER);
        if(Constants.UNREAD_NOTIFICATIONS_COUNT > 0){
            proxy.makeNotificationsRead();
        }

        notificationPopUp.addEventListener(StackedScreen.GARAGE_SELECTED, onGarageSelected);
        notificationPopUp.addEventListener(InAppNotificationPopUp.MORE_NOTIFICATIONS, onMoreNotificationRequest);
        notificationPopUp.addEventListener(InAppNotificationPopUp.VIEW_FEED, viewFeedHandler);
        notificationPopUp.addEventListener(StackedScreen.TUNING_SELECTED, tuningSelectedHandler);
        notificationPopUp.addEventListener(StackedScreen.USER_DETAILS_SELECTED, userDetailsSelectedHandler);
    }

    private function onGarageSelected(event:Event):void {
        notificationPopUp.close();
        var closeHandler:Function = function():void{
            sendNotification(Notifications.DISPLAY_IN_APP_NOTIFICATION_POPUP);
        };
        Starling.juggler.delayCall(function () :void {
            sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN, {deleteOtherScreens : false, enableCloseButton: true, closeHandler: closeHandler});
        }, 0.5)
    }

    override public function onRemove():void {
        super.onRemove();
        notificationPopUp.removeEventListener(StackedScreen.GARAGE_SELECTED, onMoreNotificationRequest);
        notificationPopUp.removeEventListener(InAppNotificationPopUp.MORE_NOTIFICATIONS, onMoreNotificationRequest);
        notificationPopUp.removeEventListener(InAppNotificationPopUp.VIEW_FEED, viewFeedHandler);
        notificationPopUp.removeEventListener(StackedScreen.TUNING_SELECTED, tuningSelectedHandler);
        notificationPopUp.removeEventListener(StackedScreen.USER_DETAILS_SELECTED, userDetailsSelectedHandler);

        if(_loadingAlert && _loadingAlert.parent)
        {
            _loadingAlert.hide();
        }
        _loadingAlert = null;
    }

    private function userDetailsSelectedHandler(event:Event):void {
        sendNotification(Notifications.DISPLAY_USER_DETAILS_POPUP, event.data);
    }

    private function onMoreNotificationRequest(event:Event):void {
        if(!_loadingAlert || !_loadingAlert.parent) {
            _loadingAlert = AlertView.show("Loading notifications ...", 4, true);
        }

        if(_isFeedUpdating ) return;

        _isFeedUpdating = true;
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        var timelineProxy : TimelineProxy = facade.retrieveProxy(TimelineProxy.NAME) as TimelineProxy;
        timelineProxy.loadNotifications(usersProxy.currentUser, 10);
    }

    private function viewFeedHandler(event:Event):void {
        event.data.enableCloseButton = true;
        notificationPopUp.close();
        sendNotification(Notifications.TIMELINE_SCREEN_FEED_PREVIEW, {firstPost:event.data.feed.id});
    }

    private function tuningSelectedHandler(event:Event):void {
        // notificationPopUp.close();
        var data:Object = event.data;
        data.deleteOtherScreens = false;
        data.enableCloseButton = true;
        data.eventDispatcher = notificationPopUp;
        /*data.closeHandler = function():void{
            sendNotification(Notifications.DISPLAY_IN_APP_NOTIFICATION_POPUP);
        };*/

        var selectedUser:User = data.user;
        var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
        var currentUser:User = usersProxy.currentUser;
        if((currentUser && selectedUser && selectedUser.id != currentUser.id) || !currentUser){
            Constants.selectedUser = selectedUser;
        }else{
            Constants.selectedUser = null;
        }
        sendNotification(Notifications.DISPLAY_GALLERY_TUNING_POPUP, data);
    }

    public function get notificationPopUp() : InAppNotificationPopUp {
        return InAppNotificationPopUp(popUp);
    }

    override public function listNotificationInterests() : Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
            Notifications.USER_NOTIFICATION_LOADED,
            Notifications.USER_NOTIFICATION_LOAD_FAILED
        ]);
    }

    override public function handleNotification(note:INotification):void {
        switch(note.getName()) {
            case Notifications.USER_NOTIFICATION_LOADED:
                handleNotificationLoaded(note.getBody());
                break;
            case Notifications.USER_NOTIFICATION_LOAD_FAILED:
                //Tracer.Warn("[InAppNotificationPopUpMediator]", "Failed to load notifications");
                break;
        }
        super.handleNotification(note);
    }

    private function handleNotificationLoaded(body:Object):void {

        if(_loadingAlert && _loadingAlert.parent)
        {
            _loadingAlert.hide();
        }
        _loadingAlert = null;
        _isFeedUpdating = false;
        notificationPopUp.notifications = body.feed;
    }
}
}
