package com.despark.tuning3d.view {
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.auth.UserDetailsPopup;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.core.*;
    import com.utils.ArrayUtil;
    import com.utils.ObjectUtils;

    import feathers.core.PopUpManager;

    import starling.core.Starling;

    import starling.events.Event;

    public class UserDetailsPopupMediator extends PopUpMediator {
        private var _detailsPopup:UserDetailsPopup;
    public function UserDetailsPopupMediator(popUp:PopUp) {
        super(popUp);
        _detailsPopup = popUp as UserDetailsPopup;
    }

    override public function listNotificationInterests():Array {
        return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [

        ]);
    }

    override public function onRegister():void {
        super.onRegister();
        _detailsPopup.addEventListener(ChallengeEvent.CHALLENGE_REQUEST, handleChallengeRequest);
        _detailsPopup.addEventListener(StackedScreen.TIMELINE_SECLECTED, timelineSelectedHandler);
        _detailsPopup.addEventListener(StackedScreen.GARAGE_SELECTED, garageSelectedHandler);
        _detailsPopup.addEventListener(StackedScreen.MAILS_SELECTED, mailsSelectedHandler);
    }

        private function garageSelectedHandler(e : Event) : void{
            var user:User = e.data.user;
            PopUpManager.removeAllPopUps(true);
            var closeHandler:Function;
            var referrer:String = ObjectUtils.getValue(e, "data.referrer");
            if(_detailsPopup.referrer == UserDetailsPopup.GARAGE){
                closeHandler = function():void{
                    sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP);
                };
            }else{
                closeHandler = function():void{
                    sendNotification(Notifications.DISPLAY_PREVIOUS_SCREEN);
                };
            }
            Starling.juggler.delayCall(function () :void {
                Constants.selectedUser = user;
                sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN, {deleteOtherScreens : false, enableCloseButton: true, referrer: referrer, closeHandler: closeHandler});
            }, 0.2);
        }

        private function handleChallengeRequest(event:Event):void {
            sendNotification(Notifications.DISPLAY_START_CHALLENGE_POPUP, event.data);
        }

        private function timelineSelectedHandler(e : Event) : void{
            PopUpManager.removeAllPopUps(true);
            var user:User = e.data.user;
            Constants.selectedUser = user;
            var closeHandler:Function;
            if(_detailsPopup.referrer == UserDetailsPopup.TIMELINE){
                closeHandler = function():void{
                    sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP);
                };
            }else{
                closeHandler = function():void{
                    sendNotification(Notifications.DISPLAY_PREVIOUS_SCREEN);
                };
            }
            var referrer:String = ObjectUtils.getValue(e, "data.referrer");
            Starling.juggler.delayCall(function () :void {
                sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN, {user : Constants.selectedUser, deleteOtherScreens : false, closeHandler:closeHandler, referrer:referrer});
            }, 0.5);
        }

        private function mailsSelectedHandler(e : Event) : void{
            if(UsersProxy.loggedInUser()){
                _detailsPopup.close();
                var user:User = e.data.user;
                Constants.selectedUser = user;
                Starling.juggler.delayCall(function () :void {
                    sendNotification(Notifications.DISPLAY_INSTANT_MAIL_POPUP, {toUser: Constants.selectedUser});
                }, 0.2);
            }else{
                sendNotification(Notifications.INSTANT_MAIL_POPUP_SELECTED, {toUser: Constants.selectedUser});
            }
        }

    override public function onRemove():void {
        super.onRemove();
    }

}
}
