/**
 * Created by Render-2 on 8/23/2017.
 */
package com.despark.tuning3d.view.components.NotificationView {
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.tunings.TuningsProxy;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.MultilingualLabel;
    import com.despark.tuning3d.view.components.core.PopUp;
import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
import com.despark.tuning3d.view.components.tunings.TuningItemRenderer;
    import com.distriqt.AdsManager;
    import com.utils.FontSelector;

    import feathers.controls.Label;

import feathers.controls.LayoutGroup;

import feathers.controls.List;
import feathers.controls.ScrollBarDisplayMode;
import feathers.controls.ScrollPolicy;
import feathers.controls.renderers.IListItemRenderer;
import feathers.data.ListCollection;
import feathers.layout.AnchorLayout;
import feathers.layout.AnchorLayoutData;
import feathers.layout.HorizontalAlign;
import feathers.layout.VerticalAlign;
import feathers.layout.VerticalLayout;

    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;

    import org.as3commons.lang.ArrayUtils;

    import starling.core.Starling;

    import starling.display.Canvas;
import starling.display.Image;
import starling.display.Image;

import starling.display.Quad;
import starling.events.Event;

public class InAppNotificationPopUp extends PopUp{


    public static const MORE_NOTIFICATIONS : String = "MoreNotifications";
    public static const VIEW_FEED : String = "viewFeedNotification";

    private var _notificationList:List;
    private var _notifications : Array;
    private var _showBannerAfterExit:Boolean;
    private var _bannerScreen:String;
    private var _messageLabel : Label;
    private static const MAX_COUNT:int = 200;

    public function InAppNotificationPopUp(data:Object = null) {
        super();
        _showBannerAfterExit = (data && data.hasOwnProperty("showBannerAfterExit") && data.showBannerAfterExit);
        _bannerScreen = (data && data.hasOwnProperty("bannerScreen"))?data["bannerScreen"]:BannerScreen.BANNER_SCREEN;
    }

    override protected function initialize():void
    {
        super.initialize();

        setSizeInternal(this.stage.stageWidth*1, this.stage.stageHeight,false);
        background.color = 0x555555;
        background.alpha = 0.8;
        contentBackgroundImage.visible = false;

        var layout : VerticalLayout = new VerticalLayout();
        layout.gap = 0;
        layout.horizontalAlign = HorizontalAlign.LEFT;
        layout.verticalAlign = VerticalAlign.TOP;

        _notificationList = new List();
        _notificationList.dataProvider = new ListCollection();
        _notificationList.layout = layout;
        _notificationList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
        _notificationList.horizontalScrollPolicy = ScrollPolicy.OFF;
        _notificationList.verticalScrollPolicy = ScrollPolicy.ON;
        _notificationList.backgroundSkin = new Quad(1,1,0xe8e8e8);
        _notificationList.addEventListener(Event.SCROLL, listScrollHandler);


        _notificationList.width = actualWidth;
        _notificationList.height = actualHeight;
        _notificationList.x = 0;
        _notificationList.y = 0;

        rendererWidth = _notificationList.width -_notificationList.paddingLeft - _notificationList.paddingRight;

        _notificationList.itemRendererFactory =  notificationItemRendererFactory;
        addChild(_notificationList);
        addChild(closeButton);
        closeButton.defaultSkin = new Image( assetManager.getTexture('garage_close'));
        closeButton.setSize(20, 20);

        _messageLabel = new MultilingualLabel();
        _messageLabel.includeInLayout = false;
        _messageLabel.visible = false;
        _messageLabel.height = 14;
        _messageLabel.textRendererFactory = LabelFactory.notificationsMessageField;
        _messageLabel.text = "You don't have any notifications at this moment";
        _messageLabel.width = actualWidth;
        addChild(_messageLabel);
    }

    override protected function closeButtonTriggeredHandler(event:Event):void{
        super .closeButtonTriggeredHandler(event);
        SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
        Starling.juggler.delayCall(function():void{
            if(_showBannerAfterExit){
                if(_bannerScreen == BannerScreen.BANNER_SCREEN){
                    BannerStackedScreen.showAdBanner();
                }else if(_bannerScreen == BannerScreen.HOME_SCREEN){
                    AdsManager.$.bannerShow(BannerScreen.HOME_SCREEN);
                }
            }
        }, 0.5);
    }

    public function get neededWidth() : Number{
        return actualWidth;
    }

    public function get neededHeight():Number{
        return actualHeight;
    }


    private function listScrollHandler(event:Event):void
    {
        if(!_notificationList || !_notifications || _notifications.length == 0 || _notificationList.dataProvider.length == 0) {
            return;
        }


        if(_notificationList.verticalScrollPosition >= _notificationList.maxVerticalScrollPosition-30/* && _notifications.length<MAX_COUNT*/) {
            dispatchEventWith(MORE_NOTIFICATIONS);
        }
    }

    override protected function draw():void {
        super.draw();

        if(isInvalid(INVALIDATION_FLAG_SIZE)) {
            _notificationList.width = actualWidth;
            _notificationList.height = actualHeight;
            _notificationList.x = 0;
            _notificationList.y = 0;

            rendererWidth = _notificationList.width -_notificationList.paddingLeft - _notificationList.paddingRight;

            if(closeButton){
                closeButton.x = actualWidth - 10-closeButton.width;
                closeButton.y = 10;
            }

            if(_messageLabel){
                _messageLabel.x = (actualWidth - _messageLabel.width) / 2;
                _messageLabel.y = (actualHeight - _messageLabel.height) / 2;
            }
        }
    }

    public function set notifications (value : Array) : void {
        if(_notifications && _notifications.length > 0) {
            var newListCollection : ListCollection = new ListCollection();
            for(var i : int = 0; i < value.length; i++) {
                newListCollection.addItem(value[i]);// its automatically adds in feed array too
            }
            _notificationList.dataProvider.addAll(newListCollection);
        }  else {
            _notifications = value;
            if(_notificationList) {
                _notificationList.dataProvider = new ListCollection(value);
                _notificationList.validate();
            }
        }
        _messageLabel.visible = (_notifications.length == 0);
    }

    private var rendererWidth : Number;

    private function notificationItemRendererFactory() : IListItemRenderer
    {
        var renderer : NotificationPanelViewItemRenderer = new NotificationPanelViewItemRenderer();
        renderer.setSize(rendererWidth, 80);
        renderer.eventDispatcher = this;
        return renderer;
    }

    override public function dispose():void {
        super.dispose();

        if(_notificationList)
        {
            _notificationList.removeEventListener(Event.SCROLL, listScrollHandler);
            _notificationList.dataProvider = null;
            _notificationList.removeFromParent(true);
            _notificationList = null;
        }

        if(_messageLabel){
            _messageLabel.removeFromParent(true);
            _messageLabel = null;
        }
        //UserCircularAvatar.disposeStoredAvatarTextures();
    }

    public function close():void {
        dispatchEventWith(CLOSE_BUTTON_SELECTED);
    }

    public static function generateNotificationIcon() : LayoutGroup{
        var _notContainer : LayoutGroup = new LayoutGroup();
        var canvas : Canvas = new Canvas();
        canvas.beginFill(0xB22222);
        canvas.drawCircle(21,20,20);
        canvas.endFill();
//        _notContainer.backgroundSkin = new Quad(1,1,0xb22222);
        _notContainer.backgroundSkin = canvas;
        _notContainer.layout = new AnchorLayout();
        _notContainer.width = 18;
        _notContainer.height = 18;

        var layData : AnchorLayoutData = new AnchorLayoutData();
        layData.verticalCenter = 0;
        layData.horizontalCenter = 0;

        var _notLabel : Label = new Label();
        _notLabel.textRendererFactory = NotificationPanelLabelFactory.notificationNotifierTextRenderer;
        _notLabel.layoutData = layData;
        _notContainer.addChild(_notLabel);
        return _notContainer;
    }

}
}
