package com.despark.tuning3d.view.components.NotificationView {
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.view.components.timeline.FeedData;
import com.utils.DateParser;

public class NotificationFeedData extends FeedData{
    public var thumbURL : String;

    public static const TYPE_POST_COMMENT : String = "post_comment";
    public static const TYPE_POST_LIKE : String = "post_like";
    public var feed:FeedData;
    public function NotificationFeedData(data:Object) {
        super(null); // we want to initialize custom data
        feedType  = feedActualType = data.type;
        if(data.hasOwnProperty("post") && data.post){
            feed ||= new FeedData(data.post, false);
        }
        if(data.hasOwnProperty("likes")){
            likes = data.likes;
        }
        owner = currentUser.copy();
        switch(data.type) {
            case TYPE_NEW_FRIEND:
                id = data._id;
                date =DateParser.parseDate(data.created);
                creator = new User(data.user);
                newFriend = new User(data.newFriend);
                updatedDate = DateParser.parseDate(data.updated);
                break;
            case TYPE_TUNING_LIKE:
            case TYPE_TUNING_COMMENT:
                id = data._id;
                date = DateParser.parseDate(data.created);
                creator = new User(data.user);
                updatedDate = DateParser.parseDate(data.updated);
                if(data.hasOwnProperty("tuning")){
                    tuning = data.tuning;
                    var url : String = tuning.tuningThumbnail;
                    if(tuning.hasOwnProperty('customSnapshots') && tuning.customSnapshots && tuning.customSnapshots.length){
                        url = tuning.customSnapshots[tuning.customSnapshots.length - 1];
                    }
                    if(url && url.indexOf("http") == - 1) {
                        url = "http:" + url;
                    }
                    thumbURL = url;
                }
                break;
            case TYPE_POST_COMMENT:
            case TYPE_POST_LIKE:
                id = data._id;
                date = DateParser.parseDate(data.created);
                creator = new User(data.user);
                updatedDate = DateParser.parseDate(data.updated);
                break;
        }
    }

    /**
     * "new_friend",
     * "tuning_like",
     * "tuning_comment",
     * "post_comment",
     */
    override public function  get descriptionLabel() : String {
        var str : String = "";
        switch(this.feedType){
            case TYPE_NEW_FRIEND:
                str = "became friends with you";
                break;
            case TYPE_TUNING_LIKE:
                str = "likes your";
                break;
            case TYPE_TUNING_COMMENT:
                str = "commented on your";
                break;
            case TYPE_POST_COMMENT:
                str = "commented on your post";
                break;
            case TYPE_POST_LIKE:
                str = "likes your post";
                break;
        }
        return str;
    }

    public function  get iconTextureName() : String {
        var str : String = "post_tuning_icon";
        switch(this.feedType){
            case TYPE_NEW_FRIEND:
                str = "post_friend_icon";
                break;
            case TYPE_TUNING_COMMENT:
            case TYPE_POST_COMMENT:
                str = "post_icon";
                break;
            case TYPE_TUNING_LIKE:
            case TYPE_POST_LIKE:
                str = "post_like_icon";
                break;
        }
        return str;
    }
}
}
