package com.despark.tuning3d.view.components.auth
{
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.FormScreen;
	import com.despark.tuning3d.view.components.core.MultilingualButton;
	import com.despark.tuning3d.view.components.core.TextInputView;
	
	import feathers.controls.Button;
	
	import starling.events.Event;

	public class ForgottenPasswordScreen extends FormScreen implements Multilingual
	{
		public static const EMAIL_CHANGED : String = "EmailChanged";
		public static const SEND_SELECTED : String = "SendSelected";
		
		private var emailInputView : TextInputView;
		private var sendButton : MultilingualButton;
		
		public function ForgottenPasswordScreen()
		{
			super();
			title = "RESET PASSWORD";
		}
		
		public function updateWithLanguage(language : Language) : void
		{
			titleLabel.updateWithLanguage(language);
			titleLabel.text = language.translate("mob_signup", "resetPassword").toUpperCase();
			
			emailInputView.updateWithLanguage(language);
			emailInputView.title = language.translate("mob_signup", "email").toUpperCase();
			
			sendButton.updateWithLanguage(language);
			sendButton.label = language.translate("mob_signup", "sendNewPassword").toUpperCase();
		}
		
		override protected function initialize() : void
		{
			super.initialize();
			
			emailInputView = new TextInputView();
			emailInputView.addEventListener(Event.CHANGE, emailInputViewChangeHandler);
			emailInputView.width = 200;
			scrollContainer.addChild(emailInputView);
			
			sendButton = ButtonFactory.defaultButton(null, 0xED3F25);
			sendButton.addEventListener(Event.TRIGGERED, sendButtonTriggeredHandler);
			scrollContainer.addChild(sendButton);
			
			emailInputView.fun_enterPress = function() : void
			{
				emailInputView.textInput.visible = false;
				emailInputView.textInput.visible = true;
				sendButtonTriggeredHandler(null);
			}
			
			super.initializeBackButton();
		}
		
		override protected function draw() : void
		{
			super.draw();
			
			emailInputView.x = int((actualWidth - emailInputView.width) / 2);
			emailInputView.y = underlineQuad.y + underlineQuad.height + 20;
			
			sendButton.x = emailInputView.x;
			sendButton.y = emailInputView.y + emailInputView.height + 20;
			sendButton.width = emailInputView.width;
		}
		
		public function get email() : String 
		{
			return emailInputView.text;
		}
		
		private function emailInputViewChangeHandler(event : Event) : void
		{
			dispatchEventWith(EMAIL_CHANGED);
		}

		override protected function backButtonTriggeredHandler(event:Event):void{
			super .backButtonTriggeredHandler(event);
			SoundManager.playSound(SoundManager.SELECT_PART);
		}
		
		private function sendButtonTriggeredHandler(event : Event) : void
		{
			SoundManager.playSound(SoundManager.SELECT_PART);
			dispatchEventWith(SEND_SELECTED);
		}
		
		public function setSendButtonEnabled(value : Boolean) : void
		{
			sendButton.alpha = value ? 1.0 : 0.5;
			sendButton.touchable = value;
		}
	}
}