package com.despark.tuning3d.view.components.auth
{
import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.model.core.Device;
	import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.ExtentedButton;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.despark.tuning3d.view.components.core.MultilingualLabel;
import com.despark.tuning3d.view.components.core.StackedScreen;
import com.despark.tuning3d.view.components.core.TextInputView;
	import com.distriqt.extension.applesignin.AppleSignIn;
	import com.utils.FontSelector;

import feathers.controls.Button;
import feathers.controls.Label;
import feathers.controls.ScrollContainer;

import flash.display.Bitmap;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.display.Image;
import starling.display.Quad;
import starling.events.Event;
import starling.events.KeyboardEvent;
import starling.textures.Texture;

public class LoginScreen extends StackedScreen implements Multilingual
	{
		public static const TYPE_NORMAL:int=0;
		public static const TYPE_RESTORE:int=1;
		public static const TYPE_FOR_PURCHASE:int=2;
		
		public static const USERNAME_CHANGED : String = "UsernameChanged";
		public static const PASSWORD_CHANGED : String = "PasswordChanged";
		public static const REGISTER_SELECTED : String = "RegisterSelected";
		public static const LOGIN_SELECTED : String = "LoginSelected";
		public static const PASSWORD_RESET_SELECTED : String = "PasswordResetSelected";
		public static const BACK_BUTTON_SELECTED : String = "BackButtonSelected";
		public static const VK_LOGIN_SELECTED : String = "VK_LOGIN_SELECTED";
		public static const FB_LOGIN_SELECTED : String = "FB_LOGIN_SELECTED";
		public static const TW_LOGIN_SELECTED : String = "TW_LOGIN_SELECTED";
		public static const APPLE_LOGIN_SELECTED : String = "APPLE_LOGIN_SELECTED";
		public static const CLOSE_WEBVIEW:String = "CLOSE_WEBVIEW";
		
		private var backgroundQuad : Quad;
		private var logoImage : Image;

		public var userInputView : TextInputView;
		public var passwordInputView : TextInputView;
		private var loginButton : MultilingualButton;
		private var registerLabel : Label;
		public var scrollContainer : ScrollContainer;
		private var registerButton : MultilingualButton;
		private var resetPasswordButton : MultilingualButton;
		
		
		private var restoreIcon:Image;
		private var titleLabel:MultilingualLabel;
		
		private var textLabel:MultilingualLabel;
		
		public var vkButton : Button;
		public var faceBookButton : Button;
		public var twButton : Button;
		public var appleButton : Button;
		public var closeWebviewButton:Button;
		
		private var _type:int=TYPE_NORMAL;

        [Embed(source="/assets/textures/logo4.png")]
        public static var LOGO_CLASS_4 : Class;

		[Embed(source="/assets/textures/watermark_for_snapshots.png")]
		public static var WATERMARK_FOR_SNAPSHOTS : Class;
		
		
		
		public function LoginScreen()
		{
			super("LoginScreen");
		}
		
		protected function onGlobalEventMemLeakTest(event:*):void{
			
		}
		
		public function set type(value:int):void{
			
			
			_type=value;
		}
		
		public function get type():int{
			
			return _type;
		}
		
		public function updateWithLanguage(language : Language) : void
		{
			userInputView.updateWithLanguage(language);
			userInputView.title = language.translate("mob_signup", "username").toUpperCase();
			
			passwordInputView.updateWithLanguage(language);
			passwordInputView.title = language.translate("mob_signup", "password").toUpperCase();
			
			resetPasswordButton.updateWithLanguage(language);
			resetPasswordButton.label = language.translate("mob_signup", "forgotPassword");
			
			loginButton.updateWithLanguage(language);
			loginButton.label = language.translate("mob_signup", "login");
			
			registerButton.updateWithLanguage(language);
			registerButton.label = language.translate("mob_signup", "or") + " <font color='#ED3F25'>" + language.translate("mob_signup", "createAccount") + "</font>";
			
			if(titleLabel){
				titleLabel.updateWithLanguage(language);
				
				titleLabel.text = ("Restore In-App purchases").toLocaleUpperCase(); //language.translate("mob_signup", "Text for example");
			}
			
			if(textLabel){
				textLabel.updateWithLanguage(language);
				
				textLabel.text="To restore previously made in-app purchases,<br/>"+ 
							   "please login with the UserName used for<br/>"+
							   "making those in-app purchases";    
			
			}
		}
		
		override protected function initialize() : void
		{
			super.initialize();
			
			backgroundQuad = new Quad(1, 1, 0x262D2F);
			addChild(backgroundQuad);
			
			
				
			closeWebviewButton  = ButtonFactory.circleButton("a_photo_cancel_button");
			(closeWebviewButton as ExtentedButton).color = 0x5C77A7;
			closeWebviewButton.addEventListener(Event.TRIGGERED, closeWebviewHandler);
			closeWebviewButton.visible = false;
			//faceBookButton.x = OffsetToSide + offsetMinBut;
			//faceBookButton.y = OffsetToSide + this.width*2 + offsetMinBut;
			closeWebviewButton.width = 30;
			closeWebviewButton.height = 30;
			
			addChild(closeWebviewButton);
			
			scrollContainer = new ScrollContainer();
			addChild(scrollContainer);
			
			if(type==TYPE_NORMAL){
				var logoBitmap: Bitmap = new LOGO_CLASS_4();
				logoImage = new Image(Texture.fromBitmap(logoBitmap, false, false, assetManager.scaleFactor));
                                logoImage.scaleX = logoImage.scaleY = 0.5;
				scrollContainer.addChild(logoImage);
			}
			else{
				restoreIcon=new Image(ButtonFactory.assetManager.getTexture("restoreIcon"));
				
				scrollContainer.addChild(restoreIcon);
			
				titleLabel=new MultilingualLabel();
				titleLabel.textFormat = new TextFormat(FontSelector.getFont(), 11, 0x0b8ae0 );
				titleLabel.textFormat.align = TextFormatAlign.CENTER;
				titleLabel.textFormat.bold=true;
				titleLabel.width=200;
				titleLabel.isHTML=true;   
					
				scrollContainer.addChild(titleLabel);
				
				
				textLabel=new MultilingualLabel();
				textLabel.textFormat = new TextFormat(FontSelector.getFont(), 9, 0x6A7275 );
				textLabel.textFormat.align = TextFormatAlign.CENTER;
				textLabel.width=200;
				textLabel.isHTML=true;
				
				scrollContainer.addChild(textLabel);
			
			
			}
			
			
			
			userInputView = new TextInputView();
			userInputView.addEventListener(Event.CHANGE, userInputViewChangeHandler);
			

		
			userInputView.width = 200;
			scrollContainer.addChild(userInputView);
			
			passwordInputView = new TextInputView();
			
			passwordInputView.addEventListener(Event.CHANGE, passwordInputViewChangeHandler);
			passwordInputView.password = true;
			passwordInputView.width = 200;
			
			userInputView.fun_enterPress = function() : void
			{
				passwordInputView.textInput.setFocus();
			}
			
			passwordInputView.fun_enterPress = function() : void
			{
				passwordInputView.textInput.visible = false;
				passwordInputView.textInput.visible = true;
				loginButtonTriggeredHandler(null);
			}
			
			
			
			passwordInputView.addEventListener(KeyboardEvent.KEY_DOWN,textInputHandler);
			//passwordInputView.addEventListener(FocusEvent.FOCUS_OUT,textInputHandlerOut);

			scrollContainer.addChild(passwordInputView);
			
			loginButton = ButtonFactory.defaultButton(null, 0xED3F25);
			loginButton.addEventListener(Event.TRIGGERED, loginButtonTriggeredHandler);
			scrollContainer.addChild(loginButton);
			
			
			
			registerButton = new MultilingualButton();
			registerButton.addEventListener(Event.TRIGGERED, registerButtonTriggeredHandler);
			registerButton.textFormat = new TextFormat(FontSelector.getFont(), 10, 0x6A7275);
			registerButton.textFormat.align = TextFormatAlign.CENTER;
			//registerButton
			//registerButton.textFormat.bold=true;
			registerButton.isHTML = true;
			scrollContainer.addChild(registerButton);
			
			registerButton.validate();
			
			
			
			resetPasswordButton = new MultilingualButton();
			resetPasswordButton.addEventListener(Event.TRIGGERED, resetPasswordButtonTriggeredHandler);
			resetPasswordButton.textFormat = new TextFormat(FontSelector.getFont(), 9, 0x6A7275 );
			resetPasswordButton.textFormat.align = TextFormatAlign.RIGHT;
			scrollContainer.addChild(resetPasswordButton);
			
			resetPasswordButton.validate();
			
			
			{
				vkButton  = ButtonFactory.circleButton("a_vk_icon2");
                (vkButton as ExtentedButton).color = 0x000000;
				vkButton.addEventListener(Event.TRIGGERED, vkLoginHandler);
				scrollContainer.addChild(vkButton);
				vkButton.validate();
			}
			
			{
				faceBookButton  = ButtonFactory.circleButton("a_fb_icon2");
                (faceBookButton as ExtentedButton).color = 0x000000;
				faceBookButton.addEventListener(Event.TRIGGERED, faceBookLoginHandler);
				//faceBookButton.x = OffsetToSide + offsetMinBut;
				//faceBookButton.y = OffsetToSide + this.width*2 + offsetMinBut;
				//faceBookButton.width = widthOfMin;
				//faceBookButton.height = widthOfMin;
				
				scrollContainer.addChild(faceBookButton);
				faceBookButton.validate();
			}

			{
				appleButton  = ButtonFactory.circleButton("a_apple_icon2");
				(appleButton as ExtentedButton).color = 0xfffffe;
				appleButton.addEventListener(Event.TRIGGERED, appleLoginHandler);

				if (!Device.isIos() || !AppleSignIn.isSupported) {
					appleButton.visible = false;
				} else {
					appleButton.visible = true;
				}
//				appleButton.visible = true;
				scrollContainer.addChild(appleButton);
				appleButton.validate();
			}

			{
				twButton  = ButtonFactory.circleButton("a_tw_icon2");
                (twButton as ExtentedButton).color = 0x5C77A7;
				twButton.addEventListener(Event.TRIGGERED, twLoginHandler);
				//faceBookButton.x = OffsetToSide + offsetMinBut;
				//faceBookButton.y = OffsetToSide + this.width*2 + offsetMinBut;
				//faceBookButton.width = widthOfMin;
				//faceBookButton.height = widthOfMin;
				
//				scrollContainer.addChild(twButton);
				twButton.validate();
			}

			initializeBackButton();
		}
		
		private function logout():void{
			
			var facade : ApplicationFacade = ApplicationFacade.getInstance();
			var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
			usersProxy.logoutCurrentUser();
			
			//AdsButton.turnOffAds=false;  
		}
		
		private function closeWebviewHandler(event:Event):void{
			
			dispatchEventWith(CLOSE_WEBVIEW);
		}
		
		private function vkLoginHandler(event : Event):void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			logout();
			dispatchEventWith(VK_LOGIN_SELECTED);
		}
		
		private function faceBookLoginHandler(event : Event):void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			logout();
			dispatchEventWith(FB_LOGIN_SELECTED);
		}

		private function twLoginHandler(event : Event):void {
			logout();
			dispatchEventWith(TW_LOGIN_SELECTED);
		}


		private function appleLoginHandler(event : Event): void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			logout();
			dispatchEventWith(APPLE_LOGIN_SELECTED);
		}

		override protected function draw() : void
		{
			super.draw();
			
			if(isInvalid(INVALIDATION_FLAG_SIZE))
			{
				backgroundQuad.width = actualWidth;
				backgroundQuad.height = actualHeight;
				
				scrollContainer.width = actualWidth;
				scrollContainer.height = actualHeight;
				
				var nextOffsetY:Number=20;
				if(type==TYPE_NORMAL){
					logoImage.x = int((actualWidth - logoImage.width) / 2);
					logoImage.y = nextOffsetY;

					nextOffsetY=logoImage.y+logoImage.height+20;
				}
				else
				{
					restoreIcon.x=(actualWidth-restoreIcon.width)/2;
					restoreIcon.y=nextOffsetY;
						
					titleLabel.x=int((actualWidth - titleLabel.width) / 2);
					titleLabel.y= restoreIcon.y+ restoreIcon.height + 4;
					titleLabel.height=NaN;
					titleLabel.validate();
							
					textLabel.x=int((actualWidth - titleLabel.width) / 2);
					textLabel.y=  titleLabel.y+ titleLabel.height + 4;     
					textLabel.height=NaN;
					textLabel.validate();
					
					nextOffsetY=textLabel.y+textLabel.height+6;   
				
				}
				
				userInputView.x = int((actualWidth - userInputView.width) / 2);
				userInputView.y = nextOffsetY;
				
				passwordInputView.x = int((actualWidth - userInputView.width) / 2);
				passwordInputView.y = userInputView.y + userInputView.height + 6;
				
				resetPasswordButton.x = userInputView.x;
				resetPasswordButton.width = userInputView.width;
				resetPasswordButton.y = passwordInputView.y + passwordInputView.height;
				resetPasswordButton.height = 35;
				
				loginButton.x = userInputView.x;
				loginButton.width = userInputView.width;
				loginButton.y = resetPasswordButton.y + resetPasswordButton.height + 3;
				
				registerButton.x = loginButton.x;
				registerButton.y = loginButton.y  + loginButton.height + 3;
				registerButton.width = loginButton.width;
				registerButton.height = 25;  
				
				
				faceBookButton.x = (actualWidth - (2 * faceBookButton.width + 10))/2;
				faceBookButton.y = registerButton.y  + registerButton.height + 1;
				
				vkButton.x = faceBookButton.x + faceBookButton.width + 10;
				vkButton.y = faceBookButton.y;
				
				twButton.x = vkButton.x + vkButton.width + 10;
				twButton.y = faceBookButton.y;

				appleButton.x = vkButton.x + vkButton.width + 10;
				appleButton.y = faceBookButton.y;
				
				this.closeWebviewButton.x=actualWidth-6-closeWebviewButton.width;
				this.closeWebviewButton.y=2.5;
				//vkButton3 -> faceBookButton
				//faceBookButton2 -> vkButton
			}
		}
		
		public function textInputHandler(event:KeyboardEvent):void {
			//our textfield has focus, we want to listen for keyboard events.
			//stage.addEventListener( KeyboardEvent.KEY_DOWN, keyDownHandler);
		}
		
		public function textInputHandlerOut(event:KeyboardEvent):void {
		}
		
		private function keyDownHandler( e:KeyboardEvent ):void{
			
		}
		private function registerButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.PART_MENU);
			dispatchEventWith(REGISTER_SELECTED);
		}
		
		public function setLoginButtonEnabled(value : Boolean) : void
		{
			if(loginButton)
			{
				loginButton.alpha = value ? 1.0 : 0.5;
				loginButton.touchable = value;
			}
		}
		
		public function get username() : String 
		{
			return userInputView.text;
		}
		
		public function get password() : String 
		{
			return passwordInputView.text;
		}
		
		private function userInputViewChangeHandler(event : Event) : void
		{
			dispatchEventWith(USERNAME_CHANGED);
		}
		
		private function passwordInputViewChangeHandler(event : Event) : void
		{
			
			dispatchEventWith(PASSWORD_CHANGED);
		}
		private function keyPressed(event:KeyboardEvent):void {
		}
		private function loginButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			logout();
			dispatchEventWith(LOGIN_SELECTED);
		}
		
		public function clearTextInputViews() : void
		{
			if(userInputView)
			{
				
			//	var t:StageT
				userInputView.text = null;
			}
			
			if(passwordInputView)
			{
				passwordInputView.text = null;
			}
		}
		
		private function resetPasswordButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.PART_MENU);
			logout();
			dispatchEventWith(PASSWORD_RESET_SELECTED);
		}
		
		protected override function backButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
			dispatchEventWith(BACK_BUTTON_SELECTED);
			super.backButtonTriggeredHandler(event);
		}
	}
}