package com.despark.tuning3d.view.components.auth {
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.utils.FontSelector;

import feathers.controls.Label;
import feathers.controls.TextInput;
import feathers.controls.text.TextFieldTextEditor;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.ITextEditor;
import feathers.core.ITextRenderer;

import flash.geom.Rectangle;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.display.Image;
import starling.events.Event;
import starling.textures.Texture;

public class ProfilePageLabel extends TextInput {
    private var _placeHolder:Label;
    private var _placeholderText:String;
    public function ProfilePageLabel(placeholder:String = '') {
        super();
        _placeholderText = placeholder;
    }


    override protected function initialize():void {
        super.initialize();

        paddingLeft = paddingRight = 10;
        textEditorFactory = function():ITextEditor {
            var editor : TextFieldTextEditor = new TextFieldTextEditor();
            editor.textFormat = new TextFormat("_sans", 11, 0x242424);
            editor.textFormat.letterSpacing = .3;
            editor.multiline = false;
            return editor;
        };

        var bgTexture:Texture = ButtonFactory.assetManager.getTexture("profile_field_bg");
        var bgImage:Image = new Image(bgTexture);
        bgImage.scale9Grid = new Rectangle(bgTexture.frameWidth * 30/65, 0/*bgTexture.frameHeight / 2*/, bgTexture.frameWidth * 5/65, bgTexture.frameHeight);
        backgroundSkin = bgImage;

        _placeHolder = new Label();
        _placeHolder.textRendererFactory = placeHolderFactory;
        _placeHolder.text = _placeholderText;
        _placeHolder.height = 15;
        addChild(_placeHolder);
    }

    public function setText(value:String):void {
        text = value;
        _placeHolder.visible = value.length == 0;
    }

    override protected function focusInHandler(event:Event):void {
        super.focusInHandler(event);
        _placeHolder.visible = false;
    }

    override protected function focusOutHandler(event:Event):void {
        super.focusOutHandler(event);
        _placeHolder.visible = (text.length==0);
    }

    override protected function draw():void {
        super.draw();
        if(_placeHolder){
            _placeHolder.x = 10;
            _placeHolder.y = (height - _placeHolder.height) / 2;
        }
    }

    override public function dispose():void {
        super.dispose();
        if(_placeHolder){
            _placeHolder.removeFromParent(true);
            _placeHolder = null;
        }
    }

    private static function placeHolderFactory() : ITextRenderer {
        var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
        var bold:Boolean = false;
        var italic:Boolean = false;
        var color:uint = 0x58595b;
        var size:int = 11;
        var align:String=TextFormatAlign.LEFT;
        textRenderer.textFormat = new TextFormat(FontSelector.getMonserratMedium(), size, color);
        textRenderer.textFormat.align = align;
        textRenderer.textFormat.bold = bold;
        textRenderer.textFormat.italic =italic;
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.wordWrap = false;
        textRenderer.isHTML = true;
        return textRenderer;
    }
}
}
