package com.despark.tuning3d.view.components.auth
{
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.FormScreen;
	import com.despark.tuning3d.view.components.core.MultilingualButton;
	import com.despark.tuning3d.view.components.core.MultilingualCheck;
	import com.despark.tuning3d.view.components.core.MultilingualLabel;
	import com.despark.tuning3d.view.components.core.TextInputView;
	import com.utils.FontSelector;
	
	import flash.text.TextFormat;
	
	import feathers.controls.Button;
	import feathers.controls.Check;
	import feathers.controls.Label;
	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.skins.ImageStateValueSelector;
	
	import starling.display.Image;
	import starling.display.Quad;
	import starling.events.Event;

	public class RegisterScreen extends FormScreen implements Multilingual
	{
		public static const REGISTER_SELECTED : String = "RegisterSelected";
		public static const AGREEMENT_SELECTED : String = "AgreemnetSelected";
		public static const AGREE_CHECK_CHANGED : String = "AgreeCheckChanged";
		public static const EMAIL_CHANGED : String = "EmailChanged";
		public static const USERNAME_CHANGED : String = "UsernameChanged";
		public static const PASSWORD_CHANGED : String = "PasswordChanged";
		
		private var registerButton : MultilingualButton;
		private var emailInputView : TextInputView;
		private var userInputView : TextInputView;
		private var passwordInputView : TextInputView;
		private var check : MultilingualCheck;
		private var dummyQuad : Quad;
		private var emailErrorLabel : MultilingualLabel;
		private var userErrorLabel : MultilingualLabel;
		private var passwordErrorLabel : MultilingualLabel;
		private var errorLabelsMap : Object;
		private var agreementButton : Button;
		private var subscribeCheck : MultilingualCheck;
		
		public function RegisterScreen()
		{
			super();
			
			title = "REGISTER";
			errorLabelsMap = {};   
			
			name="RegisterScreen";
		}
		
		public function updateWithLanguage(language : Language) : void
		{
			titleLabel.text = language.translate("signUp", "register").toUpperCase();
			titleLabel.updateWithLanguage(language);
			
			emailInputView.updateWithLanguage(language);
			emailInputView.title = language.translate("signUp", "email").toUpperCase()+' <font color="#ED3F25" ><i><b>(optional)</b></i></font>';   
			
			userInputView.updateWithLanguage(language);
			userInputView.title = language.translate("signUp", "username").toUpperCase();
			
			passwordInputView.updateWithLanguage(language);
			passwordInputView.title = language.translate("signUp", "password").toUpperCase();
			
			registerButton.updateWithLanguage(language);
			registerButton.label = language.translate("signUp", "register").toUpperCase();
			
			check.updateWithLanguage(language);
			check.label = language.translate("mob_signup", "readAndAgree") + " <font color='#ED3F25'>" + language.translate("mob_signup", "userAgreement") + "</font>      ";
			check.validate();
			
			subscribeCheck.updateWithLanguage(language);
			subscribeCheck.label = language.translate("signUp", "newsletter");
			
			emailErrorLabel.updateWithLanguage(language);
			userErrorLabel.updateWithLanguage(language);
			passwordErrorLabel.updateWithLanguage(language);
		}
		
		override protected function initialize() : void
		{
			super.initialize();
			
			emailErrorLabel = errorLabelWithKey("email");
			scrollContainer.addChild(emailErrorLabel);
			
			emailInputView = new TextInputView();
			emailInputView.addEventListener(Event.CHANGE, emailInputViewChangeHandler);
			emailInputView.width = 200;
			scrollContainer.addChild(emailInputView);
			
			userErrorLabel = errorLabelWithKey("username");
			scrollContainer.addChild(userErrorLabel);
			
			userInputView = new TextInputView();
			userInputView.addEventListener(Event.CHANGE, userInputViewChangeHandler);
			userInputView.width = 200;
			scrollContainer.addChild(userInputView);
			
			passwordErrorLabel = errorLabelWithKey("password");
			scrollContainer.addChild(passwordErrorLabel);
			
			passwordInputView = new TextInputView();
			passwordInputView.addEventListener(Event.CHANGE, passwordInputViewChangeHandler);
			passwordInputView.password = true;
			passwordInputView.width = 200;
			scrollContainer.addChild(passwordInputView);
			
			check = checkFactory();
			check.isSelected = true;
			check.addEventListener(Event.CHANGE, checkChangeHandler);
			scrollContainer.addChild(check);
			
			subscribeCheck = checkFactory();
			subscribeCheck.isSelected = true;
			subscribeCheck.addEventListener(Event.CHANGE, subscribeCheckTriggeredHandler);
			scrollContainer.addChild(subscribeCheck);

			registerButton = ButtonFactory.defaultButton("REGISTER", 0xED3F25);
			registerButton.addEventListener(Event.TRIGGERED, registerButtonTriggeredHandler);
			scrollContainer.addChild(registerButton);
			
			dummyQuad = new Quad(1, 1, 0xFFFFFF);
			dummyQuad.alpha = 0;
			scrollContainer.addChild(dummyQuad);
			
			agreementButton = new Button();
			agreementButton.addEventListener(Event.TRIGGERED, agreementButtonTriggeredHandler);
			scrollContainer.addChild(agreementButton);
			
			
			
			
			emailInputView.fun_enterPress = function() : void
			{
				userInputView.textInput.setFocus();
			}
				
			userInputView.fun_enterPress = function() : void
			{
				passwordInputView.textInput.setFocus();
			}
				
			
			passwordInputView.fun_enterPress = function() : void
			{
				passwordInputView.textInput.visible = false;
				passwordInputView.textInput.visible = true;
				registerButtonTriggeredHandler(null);
			}
			
			
			
			
			super.initializeBackButton();
		}
		
		private function checkFactory() : MultilingualCheck
		{
			var iconSelector : ImageStateValueSelector = new ImageStateValueSelector();
			iconSelector.defaultValue = assetManager.getTexture("check");
			iconSelector.defaultSelectedValue = assetManager.getTexture("checkSelected");
			
			var check : MultilingualCheck = new MultilingualCheck();
			check.labelOffsetX = 5;
			check.stateToIconFunction = iconSelector.updateValue;
			check.textFormat = new TextFormat(FontSelector.getFont(), 10, 0x6C7578);
			check.isHTML = true;
			check.defaultIcon = new Image(assetManager.getTexture("check"));
			check.selectedUpIcon = new Image(assetManager.getTexture("checkSelected"));
			check.selectedDownIcon = check.selectedUpIcon;
			check.x = 0;
			
			return check;
		}
		
		private function errorLabelWithKey(key : String) : MultilingualLabel
		{
			var errorLabel : MultilingualLabel = new MultilingualLabel();
			errorLabel.textFormat = new TextFormat(FontSelector.getFont(), 8, 0xFF0000);
			errorLabel.isHTML=true;
				
			errorLabelsMap[key] = errorLabel;
				
			return errorLabel;
		}
		
		override protected function draw() : void
		{
			super.draw();
			
		
			
		
			
			userInputView.x = int((actualWidth - userInputView.width) / 2);
			userInputView.y = underlineQuad.y + underlineQuad.height + 20;;
			
			userErrorLabel.x = underlineQuad.x;
			userErrorLabel.y = userInputView.y + userInputView.height + 5;
			
			passwordInputView.x = int((actualWidth - userInputView.width) / 2);
			passwordInputView.y = userInputView.y + userInputView.height + 20;   
			  
			passwordErrorLabel.x = underlineQuad.x;
			passwordErrorLabel.y = passwordInputView.y + passwordInputView.height + 5;    
			
			
			emailInputView.x = int((actualWidth - userInputView.width) / 2);
			emailInputView.y =passwordInputView.y + passwordInputView.height + 20;
			
			emailErrorLabel.x = underlineQuad.x;
			emailErrorLabel.y = emailInputView.y + emailInputView.height + 5;
			
			check.x = passwordInputView.x;
			check.y = emailInputView.y + emailInputView.height + 20;
			
			subscribeCheck.x = check.x;
			subscribeCheck.y = check.y + 20;
			
			agreementButton.x = check.x + 50;
			agreementButton.y = check.y - 2;
			agreementButton.width = check.width - 50;
			agreementButton.height = 20;
		
			registerButton.width = passwordInputView.width;
			registerButton.x = passwordInputView.x;
			registerButton.y = subscribeCheck.y + subscribeCheck.height + 30;
			
			dummyQuad.y = registerButton.y + registerButton.height + 15;
			
			scrollContainer.width = actualWidth;
			scrollContainer.height = actualHeight;
		}
		
		private function checkChangeHandler(event : Event) : void
		{
			SoundManager.playSound(SoundManager.SELECT_PART);
			dispatchEventWith(AGREE_CHECK_CHANGED);
		}
		
		public function get agree() : Boolean 
		{
			return check.isSelected;
		}
		
		public function setRegisterButtonEnabled(value : Boolean) : void {
			registerButton.alpha = value ? 1.0 : 0.5;
			registerButton.touchable = value;
		}
		
		public function get email() : String 
		{
			return emailInputView.text;
		}
		
		public function get username() : String 
		{
			return userInputView.text;
		}
		
		public function get password() : String 
		{
			return passwordInputView.text;
		}
		
		private function emailInputViewChangeHandler(event : Event) : void
		{
			dispatchEventWith(EMAIL_CHANGED);
		}
		
		private function userInputViewChangeHandler(event : Event) : void
		{
			dispatchEventWith(USERNAME_CHANGED);
		}
		
		private function passwordInputViewChangeHandler(event : Event) : void
		{
			dispatchEventWith(PASSWORD_CHANGED);
		}
		
		private function registerButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			dispatchEventWith(REGISTER_SELECTED);
		}

		private function subscribeCheckTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.SELECT_PART);
		}

		protected override function backButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
			super.backButtonTriggeredHandler(event);
		}
		
		private function agreementButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.SELECT_PART);
			dispatchEventWith(AGREEMENT_SELECTED);
		}
		
		public function clearErrorLabels() : void {
			for(var key : String in errorLabelsMap)
			{
				errorLabelsMap[key].text = null;
			}
		}
		
		public function setErrorText(key : String, text : String) : void {
			if(errorLabelsMap[key]) {
				errorLabelsMap[key].text = text;
			}
			else {
				AlertView.show("Unrecognized key: " + key, -1, false);
			}
		}
		
		public function get shouldSubscribe() : Boolean {
			return subscribeCheck.isSelected;
		}
	}
}