package com.despark.tuning3d.view.components.auth {
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.friend.Friend;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.timeline.TimelineGarage.TimelineAddFriendButton;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.distriqt.AdsManager;

    import feathers.controls.Button;
    import feathers.controls.LayoutGroup;
    import feathers.core.FeathersControl;
    import feathers.layout.Direction;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.VerticalAlign;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.textures.Texture;

    public class UserDetailsPopup extends PopUp {
        private var _mailsButton:Button;
        private var _garageButton:Button;
        private var _timelineButton:Button;
        private var _startChallengeButton:Button;
        private var _friendButton:TimelineAddFriendButton;

        private var _buttonContainer : LayoutGroup;
        private var _user:User;
        private var _avatar:UserCircularAvatar;
        private var _usernameLabel:FeathersControl;
        //
        private var showAdsOnClose:Boolean;
        private var _options:Array;
        //
        public static const CHALLENGE:String = "challenge";
        public static const TIMELINE:String = "timeline";
        public static const GARAGE:String = "garage";
        public static const EMAILS:String = "emails";
        //
        private var _sep:Quad;
        private var _referrer:String;
        private static const POPUP_WIDTH:int = 260;
        //
		public function UserDetailsPopup(user:User, options:Array = null, referrer:String = null) {
            _user = user;
            _options = options?options:[TIMELINE, GARAGE, EMAILS, CHALLENGE];
            _referrer = referrer;
		}

		override protected function initialize() : void {
			super.initialize();
            setSizeInternal(stage.stageWidth, stage.stageHeight, false);
            showAdsOnClose = AdsManager.$.bannerHide();
            //
            _avatar = new UserCircularAvatar(_user, 50, true, 2);
            addChild(_avatar);

            _usernameLabel = LabelFactory.createUserDetailsLabel();
            LabelFactory.setLabelText(_usernameLabel, _user.nameToShow);
            addChild(_usernameLabel);

            _buttonContainer = new LayoutGroup();
            var socialLayout:HorizontalLayout = new HorizontalLayout();
            socialLayout.gap = 10;
            socialLayout.verticalAlign = VerticalAlign.MIDDLE;
            _buttonContainer.layout = socialLayout;
            //
            _startChallengeButton = createSocialButton("request_challenge_btn", 25);
            _startChallengeButton.addEventListener(Event.TRIGGERED, startChallengeHandler);
            _buttonContainer.addChild(_startChallengeButton);
            if(_options.indexOf(CHALLENGE) == -1){
                _startChallengeButton.isEnabled = false;
            }

            _buttonContainer.addChild(createSeparator(Direction.VERTICAL, 25));

            _timelineButton = createSocialButton("clock-menu");
            _timelineButton.addEventListener(Event.TRIGGERED, timelineTriggeredHandler);
            _buttonContainer.addChild(_timelineButton);
            if(_options.indexOf(TIMELINE) == -1){
                _timelineButton.isEnabled = false;
            }

            _buttonContainer.addChild(createSeparator(Direction.VERTICAL, 25));

            _garageButton = createSocialButton("garage_icon");
            _garageButton.addEventListener(Event.TRIGGERED, garageTriggeredHandler);
            _buttonContainer.addChild(_garageButton);
            if(_options.indexOf(GARAGE) == -1){
                _garageButton.isEnabled = false;
            }

            _buttonContainer.addChild(createSeparator(Direction.VERTICAL, 25));

            _mailsButton = createSocialButton("messages_btn");
            _mailsButton.addEventListener(Event.TRIGGERED, mailTriggeredHandler);
            _buttonContainer.addChild(_mailsButton);
            if(_options.indexOf(EMAILS) == -1){
                _mailsButton.isEnabled = false;
            }

            _buttonContainer.addChild(createSeparator(Direction.VERTICAL, 25));
            var friend:Friend = new Friend(Friend.STATE_NO_FRIEND, null);
            friend.user = _user;
            _friendButton = new TimelineAddFriendButton(friend);
            _friendButton.width = _friendButton.height = 25;
            _buttonContainer.addChild(_friendButton);


            _buttonContainer.validate();
            addChild(_buttonContainer);

            _sep = createSeparator(Direction.HORIZONTAL, POPUP_WIDTH);
            addChild(_sep);
            //

            var closeTxT:Texture = ButtonFactory.assetManager.getTexture("details_close");
            var closeIcon:Image = new Image(closeTxT);
            closeIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            closeButton.width = closeIcon.width;
            closeButton.height = closeIcon.height;
            closeButton.defaultIcon = closeIcon;

            var closeDownIcon:Image = new Image(closeTxT);
            closeDownIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            closeDownIcon.alpha = 0.6;
            closeButton.width = closeDownIcon.width;
            closeButton.height = closeDownIcon.height;
            closeButton.downIcon = closeDownIcon;
            addChild(closeButton);
            //
            SoundManager.playSound(SoundManager.PART_MENU);
		}

        private static function createSocialButton(textureId:String, size:Number = 0):Button{
            var socialIcon:Image;
            var socColor:uint = 0x7e7e7e;
            var socSize:uint = size||30;
            var socTxt:Texture;
            var button:Button = new Button();
            socTxt = ButtonFactory.assetManager.getTexture(textureId);
            socialIcon = new Image(socTxt);
            socialIcon.color = socColor;
            socialIcon.width = socialIcon.height = socSize;
            button.defaultIcon = socialIcon;

            socialIcon = new Image(socTxt);
            socialIcon.color = socColor;
            socialIcon.alpha = 0.6;
            socialIcon.width = socialIcon.height = socSize;
            button.downIcon = socialIcon;

            socialIcon = new Image(socTxt);
            socialIcon.color = 0xdedede;
            socialIcon.width = socialIcon.height = socSize;
            button.disabledIcon = socialIcon;

            button.validate();
            return button;
        }

        public static function createSeparator(direction:String, size:Number, color:uint = 0xCCCCCC):Quad{
            var sep:Quad = new Quad(1, 1, color);
            if(direction == Direction.VERTICAL){
                sep.height = size;
            }else{
                sep.width = size;
            }
            return sep;
        }


        override protected function closeButtonTriggeredHandler(event:Event):void {
            super.closeButtonTriggeredHandler(event);
            if(showAdsOnClose){
                AdsManager.$.bannerShow(AdsManager.$.currentBannerScreen);
            }
            SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
        }


		override protected function draw() : void {
			super.draw();
			
			if(isInvalid(INVALIDATION_FLAG_SIZE)){
                contentBackgroundImage.width = POPUP_WIDTH;
                contentBackgroundImage.height = 125;
                contentBackgroundImage.x = (actualWidth - contentBackgroundImage.width) / 2;
                contentBackgroundImage.y = (actualHeight - contentBackgroundImage.height) / 2;

                closeButton.x = contentBackgroundImage.x + contentBackgroundImage.width - (closeButton.width + 3);
                closeButton.y = contentBackgroundImage.y + 3;

                _avatar.x = contentBackgroundImage.x + (contentBackgroundImage.width - _avatar.width) / 2;
                _avatar.y = contentBackgroundImage.y + 10;

                _usernameLabel.x = _avatar.x + (_avatar.width - _usernameLabel.width) / 2;
                _usernameLabel.y = _avatar.y + _avatar.height + 2;

                _buttonContainer.x = contentBackgroundImage.x + (contentBackgroundImage.width - _buttonContainer.width) / 2;
                _buttonContainer.y = contentBackgroundImage.y + contentBackgroundImage.height - (_buttonContainer.height + 5);

                _sep.x = contentBackgroundImage.x;
                _sep.y = _buttonContainer.y - _sep.height - 2.5;
			}
		}

        private function startChallengeHandler(event:Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            var data:Object  = ChallengeEvent.getData(UsersProxy.loggedInUser(), null, _user, null);
            data.referrer = _referrer;
            dispatchEvent(new Event(ChallengeEvent.CHALLENGE_REQUEST, false, data));
        }

        private function timelineTriggeredHandler(event:Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchEventWith(StackedScreen.TIMELINE_SECLECTED, false, {user: _user, referrer: _referrer});
        }

        private function garageTriggeredHandler(event:Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchEventWith(StackedScreen.GARAGE_SELECTED, false, {user: _user, referrer: _referrer});
        }

        private function mailTriggeredHandler(event:Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchEventWith(StackedScreen.MAILS_SELECTED, false, {user: _user, referrer: _referrer});
        }

        public function close():void{
            SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
            closeButtonTriggeredHandler(null);
        }


    override public function dispose():void {
        super.dispose();

        if(_mailsButton){
            _mailsButton.removeFromParent(true);
            _mailsButton = null;
        }
        if(_startChallengeButton){
            _startChallengeButton.removeFromParent(true);
            _startChallengeButton = null;
        }
        if(_garageButton){
            _garageButton.removeFromParent(true);
            _garageButton = null;
        }
        if(_timelineButton){
            _timelineButton.removeFromParent(true);
            _timelineButton = null;
        }
        if(_buttonContainer){
            _buttonContainer.removeFromParent(true);
            _buttonContainer = null;
        }
        if(_avatar){
            _avatar.removeFromParent(true);
            _avatar = null;
        }
        if(_usernameLabel){
            _usernameLabel.removeFromParent(true);
            _usernameLabel = null;
        }
    }

        public function get referrer():String {
            return _referrer;
        }
    }
}