package com.despark.tuning3d.view.components.brands
{
	import com.despark.tuning3d.model.brands.Brand;
	import com.despark.tuning3d.model.core.Constants;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.BrandListItemRenderer;
	import com.utils.FontSelector;
	
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.ITextRenderer;
	import starling.display.Quad;

	public class BrandItemRenderer extends BrandListItemRenderer
	{
		private var background:Quad;
		private var underline:Quad;
		
		
		public function BrandItemRenderer()
		{
			super();
			overflowMultiLang = true;
		}
		
		public function get brand() : Brand 
		{
			return data as Brand;
		}

		protected override function commitData() : void 
		{
			if (brand)
			{
				if (brand.name){

					var str : String = brand.name;
					if (str.indexOf("last added") != -1){
                        label.text = "LAST ADDED";
					}
					else {
                        label.text = brand.name;
                    }
				}
				else 
				{
					label.text = "";
				}
					
				if (brand.name && !(brand.lastAdded) && (brand.name=="Coming Soon" || brand.name=="Under Construction" || brand.name=="With \"stance\"" || brand.name=="With most parts"))
				{
					label.textFormat.italic = true;
					label.text=brand.name;  
				}
					
				label.height = (actualHeight - label.height) / 2; 
				
				if (brand.lastAdded) {
					label.text = AlertView.language.translate("carchooserTop", 'last_added').toLocaleUpperCase();
					label.textFormat.italic = true;
				}
			}
		}
		
		protected override function commitSelection() : void
		{
			super.commitSelection();
			
			underline.alpha = isSelected ? 1.0 : 0.85;
			
			textColor = isSelected ? 0xA80808 : 0x636365;
			label.textRendererFactory = getTextRenderer;
			label.paddingTop = 3;
		}
		
		override protected function initialize() : void      
		{
			background = new Quad(1, 1, 0xFFFFFF);
			background.alpha = 0; // ???????
			addChild(background);
			
			underline = new Quad(1, 1.2, 0xECECEC);
			underline.alpha = .85;
			addChild(underline);
			
			super.initialize();
			
			label.textRendererFactory = getTextRenderer;
			label.paddingLeft = 0;
			addChild(label);
		}
		
		private function getTextRenderer():ITextRenderer
		{
			var ft_size : Number;
			if (Constants.SCALE_FACTOR == 4) {
				ft_size = Constants.isIOS ? 13 : 13.5;
			}
			else if (Constants.SCALE_FACTOR == 2){
				ft_size = Constants.isIOS ? 14 : 15;
			}
			else {
				ft_size = 15;
			}
			var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
				textRenderer.textFormat = new TextFormat(FontSelector.getFont(), ft_size, textColor);
				textRenderer.textFormat.align = TextFormatAlign.LEFT;
				textRenderer.textFormat.italic = false;
				textRenderer.embedFonts = "NotoSans";
				textRenderer.textFormat.bold = true;
				textRenderer.embedFonts = true;
			
			return textRenderer;
		}
		
		protected override function layout():void
		{
			super.layout();

			label.width = actualWidth - 30;
			label.y = (actualHeight - label.height) / 2 - 3;
			label.x = 15;
			
			background.width  = actualWidth;
			background.height = actualHeight;
			
			underline.width = actualWidth - 30;
			underline.x = 15;
			underline.y = actualHeight - underline.height;
		}
		
		override protected function draw():void {
			overflowMultiLang = true;
			super.draw();
		}
		
		protected override function autoSizeIfNeeded() : Boolean
		{
			var needsWidth  : Boolean = isNaN(explicitWidth);
			var needsHeight : Boolean = isNaN(explicitHeight);
			
			if (!needsWidth && !needsHeight){
				return false;
			}
			
			label.width  = NaN;
			label.height = NaN;
			label.validate();
			
			var newWidth : Number = explicitWidth;
			
			if (needsWidth){
				newWidth = Math.ceil(label.width) + 20;
				label.width = newWidth;
			}
			
			var newHeight : Number = explicitHeight;   
			if  (needsHeight) {
				newHeight = 35;
				label.height = newHeight - 13;
			}
			
			label.y = (actualHeight - label.height) / 2 - 3;
			
			return setSizeInternal(newWidth, newHeight, false);
		}
	}
}