package com.despark.tuning3d.view.components.brands {
import com.despark.tuning3d.model.packages.Packet;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.despark.tuning3d.view.components.core.PopUp;
import com.utils.FontSelector;

import feathers.controls.Label;

import feathers.controls.LayoutGroup;

import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.ITextRenderer;
import feathers.layout.AnchorLayout;
import feathers.layout.AnchorLayoutData;

import flash.geom.Rectangle;
import flash.text.StyleSheet;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.core.Starling;
import starling.display.Image;
import starling.events.Event;

public class RewardConfirmPopUp   extends PopUp{
    private const REWARD_BUY_PACKET: String = "REWARD_BUY_PACKET";
    public var packet: Packet;
    protected var _mainCotnainer : LayoutGroup;
    private var cancelButton:MultilingualButton;
    private var confirmButton:MultilingualButton;
    private var newtitleLabel:MultilingualButton;

    public function RewardConfirmPopUp() {
        super();
        setSizeInternal(Starling.current.stage.width, Starling.current.stage.height, false);
    }
    override protected function initialize() : void {
        super.initialize();
        contentBackgroundImage.visible = false;
        this.closeButton.visible = false;
        var mainLayout : AnchorLayoutData = new AnchorLayoutData();
        mainLayout.verticalCenter = 0;
        mainLayout.horizontalCenter = 0;


        _mainCotnainer = new LayoutGroup();
        _mainCotnainer.layout = new AnchorLayout();
        _mainCotnainer.layoutData = mainLayout;
        addChild(_mainCotnainer);

        var borderImage: Image = new Image(assetManager.getTexture('frame'));
        borderImage.scale9Grid = new Rectangle(borderImage.width * .2, borderImage.height * .2, borderImage.width * .6, borderImage.height * .6);
        borderImage.width = borderImage.height = 10;
        _mainCotnainer.backgroundSkin = borderImage;

        var packetTitle: String = packet.title;

        if(AlertView.language && AlertView.language.translate(packet.id, 'title') && AlertView.language.translate(packet.id, 'title') != 'title')
        {
            packetTitle = AlertView.language.translate(packet.id, 'title');
        }

        var textLayout : AnchorLayoutData = new AnchorLayoutData();
        textLayout.verticalCenter = 0;
        textLayout.horizontalCenter = 0;


        packetTitle = packetTitle.replace('<br>', '').replace('<br/>', '').replace('<br />', '');

        var text: String = '<body>3DT Coin will grant you one-time access to any one of the vehicles available in <span class="title">' + packetTitle + '</span> package</body>';



        var textLabel: Label = new Label();
        textLabel.layoutData = textLayout;
        textLabel.textRendererFactory = function () :ITextRenderer {
            var styleSheet: StyleSheet = new StyleSheet();
            var bodyStyle: Object = {};
            var boldStyle: Object = {};
            bodyStyle.fontWeight = 'normal';
            boldStyle.fontWeight = 'bold';
            bodyStyle.fontSize = boldStyle.fontSize = 12;
            bodyStyle.fontFamily = FontSelector.getMonserratMedium();
            boldStyle.fontFamily = FontSelector.getMonserratSemiBold();

            bodyStyle.color = "#ffffff";
            boldStyle.color = "#FF9700";

            styleSheet.setStyle(".title",boldStyle );
            styleSheet.setStyle("body",bodyStyle);

            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.wordWrap = true;
            textRenderer.embedFonts = true;
            textRenderer.isHTML = true;
            textRenderer.styleSheet = styleSheet;
            return textRenderer;
        };
        textLabel.width = actualWidth * .6 - 20;
        textLabel.wordWrap = true;
        textLabel.text = text;
        textLabel.validate();
        textLabel.padding = 20;
        _mainCotnainer.addChild(textLabel);



        cancelButton = ButtonFactory.rewardCancelButton();
        confirmButton = ButtonFactory.rewardConfirmButton();

        confirmButton.addEventListener(Event.TRIGGERED, confirmButtonTriggeredHandler);
        addChild(confirmButton);

        cancelButton.addEventListener(Event.TRIGGERED, cancelButtonTriggeredHandler);
        addChild(cancelButton);

        newtitleLabel = ButtonFactory.rewardsPopUpTitle();
        addChild(newtitleLabel);
    }
    override protected function draw() : void {
        super.draw();

        _mainCotnainer.invalidate();
        _mainCotnainer.validate();

        _mainCotnainer.x = (actualWidth - _mainCotnainer.width) / 2;
        _mainCotnainer.y = (actualHeight - _mainCotnainer.height) / 2;

        newtitleLabel.width = _mainCotnainer.width;
        newtitleLabel.x = (actualWidth - _mainCotnainer.width) / 2;
        newtitleLabel.y = _mainCotnainer.y - 10 - newtitleLabel.height;

        confirmButton.x = (actualWidth - (cancelButton.width + 10 + confirmButton.width)) / 2;
        cancelButton.x = confirmButton.x + confirmButton.width + 10;
        confirmButton.y = cancelButton.y = _mainCotnainer.y + _mainCotnainer.height + 10;

    }

    private function cancelButtonTriggeredHandler(event: Event): void {
        dispatchEventWith(CLOSE_BUTTON_SELECTED);
    }

    private function confirmButtonTriggeredHandler(event: Event): void {
        dispatchEventWith(CLOSE_BUTTON_SELECTED);
        dispatchEventWith(REWARD_BUY_PACKET, false, {packet: packet});
    }

    override public function dispose(): void {
        super.dispose();
        confirmButton.removeEventListener(Event.TRIGGERED, confirmButtonTriggeredHandler);
        cancelButton.removeEventListener(Event.TRIGGERED, cancelButtonTriggeredHandler);
    }

}
}
