package com.despark.tuning3d.view.components.brands {
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.brands.BrandsScreen;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.despark.tuning3d.view.components.core.PopUp;
import com.distriqt.AdsManager;


import starling.core.Starling;
import starling.display.Image;
import starling.events.Event;
import starling.textures.Texture;

public class RewardInfoPopUp  extends PopUp {
    private const BUTTONS_BLOCK_PERCENT: Number = 72;
    public static const OPEN_NO_ADS_POP_UP : String = 'OPEN_NO_ADS_POP_UP';
    private var watchButton:MultilingualButton;
    private var laterButton:MultilingualButton;
    private var noButton:MultilingualButton;


    public function RewardInfoPopUp() {
        super();
        setSizeInternal(Starling.current.stage.width, Starling.current.stage.height, false);
    }

    override protected function initialize() : void {
        super.initialize();
        contentBackgroundImage.visible = false;
        this.closeButton.visible = false;

        var text: Texture = assetManager.getTexture("text");
        var textImage: Image = new Image(text);
        textImage.width = text.width;
        textImage.height = text.height;

        if (textImage.height > actualHeight * .8 || textImage.width > actualWidth * .8) {
            var coef: Number = actualWidth * .8 / textImage.width;
            if (coef * textImage.height > actualHeight) {
                coef = actualHeight * .8 / textImage.height;
            }

            textImage.width = text.width * coef;
            textImage.height = text.height * coef;
        }

        textImage.x = (actualWidth - textImage.width) / 2;
        textImage.y = (actualHeight - textImage.height) / 2;

        watchButton = ButtonFactory.rewardWatchButton(coef);
        laterButton = ButtonFactory.rewardLaterButton(coef);
        noButton = ButtonFactory.rewardNoButton(coef);

        watchButton.addEventListener(Event.TRIGGERED, watchButtonTriggerHandler);
        laterButton.addEventListener(Event.TRIGGERED, noOrLaterButtonTriggeredHandler);
        noButton.addEventListener(Event.TRIGGERED, noOrLaterButtonTriggeredHandler);

        watchButton.y = laterButton.y = noButton.y = textImage.y + textImage.height - watchButton.height / 2;

        var buttonBlockWidth: Number = textImage.width * BUTTONS_BLOCK_PERCENT / 100;
        var buttonBlockStartX: Number = textImage.x + (textImage.width - buttonBlockWidth) / 2;
        var blocksOffset: Number = (buttonBlockWidth - watchButton.width - laterButton.width - noButton.width) / 6;


        laterButton.x = buttonBlockStartX + blocksOffset;
        watchButton.x = laterButton.x + laterButton.width + 2 * blocksOffset;
        noButton.x = watchButton.x + watchButton.width + 2 * blocksOffset;


        addChild(textImage);
        addChild(watchButton);
        addChild(laterButton);
        addChild(noButton);

    }

    override protected function draw() : void {
        super.draw();
    }

    private function noOrLaterButtonTriggeredHandler(event: Event): void {
        SoundManager.playSound(SoundManager.SELECT_PART_BACK);
        dispatchEventWith(CLOSE_BUTTON_SELECTED);
        Starling.juggler.delayCall(function():void{
            AdsManager.$.bannerShow(BannerScreen.HOME_SCREEN);
        }, 0.5);
    }

    private function watchButtonTriggerHandler(event: Event): void {
        SoundManager.playSound(SoundManager.SELECT_PART);
        if (AdsManager.$.haveRewardAds) {
            AdsManager.$.showReward();
        } else {
            dispatchEventWith(OPEN_NO_ADS_POP_UP);
        }
        dispatchEventWith(CLOSE_BUTTON_SELECTED);
    }

    override public function dispose(): void {
        super.dispose();
        watchButton.removeEventListener(Event.TRIGGERED, watchButtonTriggerHandler);
        laterButton.removeEventListener(Event.TRIGGERED, noOrLaterButtonTriggeredHandler);
        noButton.removeEventListener(Event.TRIGGERED, noOrLaterButtonTriggeredHandler);
    }

}
}
