package com.despark.tuning3d.view.components.cars
{
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.model.cars.Car;
import com.despark.tuning3d.model.cars.CarsProxy;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.images.PersistentImageProvider;
import com.despark.tuning3d.model.packages.Packet;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.CarListItemRenderer;
import com.despark.tuning3d.view.components.core.ExtendedList;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;

import flash.utils.setTimeout;

import starling.display.Image;
import starling.display.Quad;
import starling.textures.Texture;

public class CarItemRenderer extends CarListItemRenderer
	{
		//protected var nf_icon:Image;
		//protected var ft_icon:Image;
		protected var carUnlock:Image;
		
		private var carLock:Image;
		private var quad:Quad;
		private var offsetIcon:Image;
		private var stanceIcon:Image;
		private var ultraIcon:Image;
		private var hdResolutionIcon:Image;
		private var topCameraIcon:Image;
		private var completeLoad:Boolean=false;
		
		private var _underlined : Boolean = true;
		private var _horizontalOffset : Number;
		private var underline : Quad;
		private var _verticalOffset : Number = 5;
		
		public  var _underOffset : Number = 0;
		public  var _carOffset : Number = 0;
		private var _overlay : Quad;
		
		public function CarItemRenderer(){
			super();
		}
		
		public function set horizontalOffset(value:Number):void{
			_horizontalOffset = value;
		}
		
		public function get horizontalOffset():Number{
			return _horizontalOffset;
		}
	
		public function set underlined(value:Boolean):void{
			_underlined = value;
		}

		public function get underlined():Boolean{
			return _underlined;
		}
		
		public function get car() : Car{
			return data as Car;       
		}

		protected override function initialize():void
		{
			quad = new Quad(actualWidth, actualHeight,0xECECEC);
			quad.alpha = 0;
			addChild(quad);  
			
			_overlay = new Quad(actualWidth, actualHeight,0xECECEC);
			_overlay.alpha = 0;
			
			underline = new Quad(1, 1.2, 0xECECEC);
			underline.visible = true;
			addChild(underline);
			
			super.initialize();
			
			var carLockTexture : Texture = AlertView.assetManager.getTexture("car_lock_big_icon");
			carLock=new Image(carLockTexture);		
			
			var carUnlockTexture:Texture = AlertView.assetManager.getTexture("buy_check_icon");	
			carUnlock=new Image(carUnlockTexture);
					
			var offsetTexture:Texture = AlertView.assetManager.getTexture("offsetIcon");
			offsetIcon=new Image(offsetTexture);
			offsetIcon.visible=false;
			
			var stanceTexture:Texture = AlertView.assetManager.getTexture("stance");
			stanceIcon=new Image(stanceTexture);
			stanceIcon.visible=false;
			
			var ultraTexture:Texture = AlertView.assetManager.getTexture("ulstrarealistic");
			ultraIcon=new Image(ultraTexture);
			ultraIcon.visible=false;

            var hdRes:Texture = AlertView.assetManager.getTexture("hd-resolution");
            hdResolutionIcon=new Image(hdRes);
            hdResolutionIcon.visible=false;
			
			var topCameraTexture:Texture = AlertView.assetManager.getTexture("top-vew-camera");
			topCameraIcon=new Image(topCameraTexture);
			topCameraIcon.visible=false;
			

			addChild(carLock);	
			addChild(carUnlock);
			addChild(stanceIcon);
			
			addChild(offsetIcon);
			addChild(offsetIcon);
			addChild(ultraIcon);
			addChild(topCameraIcon);
			addChild(hdResolutionIcon);
		}
		
		protected override function layout():void
		{
			
			var facade : ApplicationFacade = ApplicationFacade.getInstance();
			var carsProxy : CarsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;


			label.height = tlabel.height = ylabel.height = labelHeight;

			label.x = tlabel.x = 10; //pading * 1.5;
			label.y = ylabel.y = pading;
			ylabel.x = actualWidth - 22 - 10; //pading * 1.5;

			tlabel.y = label.y + label.height - pading * 1.5;

			tlabel.width = actualWidth - 20; //pading * 1.5;
			ylabel.width = 22;

			label.width = actualWidth - ylabel.width -20;

			//label.height=NaN;
			//label.validate();

			if (imageLoader){
				imageLoader.x = _carOffset;//pading;
				imageLoader.y = tlabel.y + tlabel.height - pading;
				imageLoader.width  = actualWidth  - 2 * _carOffset;//pading;
				imageLoader.height = actualHeight - (tlabel.y + tlabel.height + 1.5* pading);
			}

			var posx : Number = ylabel.x - carLock.width - 3;

			carLock.x = carUnlock.x = posx;
			carLock.y = carUnlock.y = label.y + carLock.height / 2;

			if( car && car.packet && car.packet.lock){
				carLock.visible = car.packet.type != Packet.TYPE_SUBCOMPONENT ;
			}
			else {
				carLock.visible=false;
			}

			if (car && car.carFeaturesNew && car.carFeaturesFull)
			{
				carLock.x = carUnlock.x = posx;

				if (car && car.packet && car.packet.lock)
				{
					if (car.packet.type != Packet.TYPE_SUBCOMPONENT)
					{
						if(carsProxy.bonusCarExists(car.id)){
							carLock.visible=false;
							carUnlock.visible=true;
						}
						else{
							carLock.visible=true;
							carUnlock.visible=false;
						}
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else if(car && car.packet && !car.packet.lock)
				{
					if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
						carLock.visible=false;
						carUnlock.visible=true;
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else{
						carLock.visible=false;
						carUnlock.visible=false;
				}
			}
			else if(car && car.carFeaturesFull)
			{
				carLock.x = carUnlock.x = posx;
				if (car && car.packet && car.packet.lock)
				{
					if (car.packet.type != Packet.TYPE_SUBCOMPONENT)
					{
						if(carsProxy.bonusCarExists(car.id)){
							carLock.visible=false;
							carUnlock.visible=true;
						}
						else{
							carLock.visible=true;
							carUnlock.visible=false;
						}
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else if(car && car.packet && !car.packet.lock)
				{
					if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
						carLock.visible=false;
						carUnlock.visible=true;
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else{
						carLock.visible=false;
					carUnlock.visible=false;
				}
			}
			else if(car && car.carFeaturesNew)
			{
				carLock.x = carUnlock.x = posx;

				if(car && car.packet && car.packet.lock)
				{
					if(car.packet.type!=Packet.TYPE_SUBCOMPONENT)
					{
						if(carsProxy.bonusCarExists(car.id)){
							carLock.visible=false;
							carUnlock.visible=true;
						}
						else{
							carLock.visible=true;
							carUnlock.visible=false;
						}
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else if(car && car.packet && !car.packet.lock)
				{
					if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
						carLock.visible=false;
						carUnlock.visible=true;
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else{
						carLock.visible=false;
					carUnlock.visible=false;
				}
			}
			else
			{
				carLock.x = carUnlock.x = posx;
				if(car && car.packet && car.packet.lock)
				{
					if(car.packet.type!=Packet.TYPE_SUBCOMPONENT)
					{
						if(carsProxy.bonusCarExists(car.id)){
							carLock.visible=false;
							carUnlock.visible=true;
						}
						else{
							carLock.visible=true;
							carUnlock.visible=false;
						}
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else if(car && car.packet && !car.packet.lock)
				{
					if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
						carLock.visible=false;
						carUnlock.visible=true;
					}
					else{
						carLock.visible=false;
						carUnlock.visible=false;
					}
				}
				else{
						carLock.visible=false;
						carUnlock.visible=false;
				}
			}


			//////////////////////////////////////////////////////////
			quad.x = 0;
			quad.width  = actualWidth;
			quad.height = actualHeight;

			if (_underlined)
			{
				underline.visible = true;
				underline.y = actualHeight - underline.height;
				underline.width = actualWidth - _underOffset;
				underline.x = _underOffset;

			}
			else {
				underline.visible = false;
			}
		  
			
			//addChild(_overlay);
		}
		
		private var imageWidth:Number=-1;
		private var imageHeight:Number=-1;
		protected function completeLoader(image:ImageLoaderWithStates=null):void{
			setTimeout(completeLoaderImage, 50);
		}
		
		protected function errorLoader():void{
			
			//completeLoad=false;
			//imageLoader.persistentImageProvider = Car.BlankCar;
			//completeLoad=true;
			//invalidate();
		}
		
		protected function completeLoaderImage():void
		{
			if (imageLoader && imageLoader.image)
			{
				completeLoad = true;
		
				imageWidth=imageLoader.image.width;
				imageHeight=imageLoader.image.height;
				
				underline.visible = _underlined;
				invalidate();
			}
			else
			{
				setTimeout(completeLoaderImage, 10);
			}		
		} 
			
		protected override function commitData() : void 
		{
			super.commitData();
			
			if (car)
			{
                ultraIcon.visible=false;
                offsetIcon.visible=false;
                stanceIcon.visible=false;
                topCameraIcon.visible=false;
                hdResolutionIcon.visible=false;


				var newPersistentImageProvider:PersistentImageProvider=car;
							
				if(!imageLoader.completeHandler){			
					imageLoader.completeHandler=completeLoader;
				}
				
				if(!imageLoader.errorHandler){			
					imageLoader.errorHandler=errorLoader;
				}
				
				if (imageLoader.persistentImageProvider && imageLoader.persistentImageProvider.remoteUrl != newPersistentImageProvider.remoteUrl)
				{
					completeLoad=false;
					imageLoader.persistentImageProvider = car;
				}
				else if (!imageLoader.persistentImageProvider)
				{
					completeLoad=false;
					imageLoader.persistentImageProvider = car;
				}


				var str : String = car.model;
				if(car.body && car.body.length>0)
				{
					str += " " + car.body;
				}

				label.text  = str;
				tlabel.text = (this.owner as ExtendedList).data.showBrand || car.limitedOptions ? car.name : "";

				var temp : String = car.year.toString();
				var  len : int = temp.length;
				if (len > 3){
					temp = temp.substr(len-2, 2);
				}
				else {
					temp = (len == 2) ? temp : "";
				}
				ylabel.text = len > 1 ? "'" + temp : "";

			}
		}
		
		protected override function   draw() : void
		{      
			super.draw();


			var facade : ApplicationFacade = ApplicationFacade.getInstance();
			var carsProxy : CarsProxy = facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;

			if(car && car.packet && car.packet.lock && !car.packet.show){
				carLock.visible = car.packet.type != Packet.TYPE_SUBCOMPONENT ;
			}
			else
			{
				if(car && !carsProxy.bonusCarExists(car.id) && car.packet && car.packet.type!=Packet.TYPE_SUBCOMPONENT){
					carLock.visible=false;
				}
				else {
					carLock.visible=false;
				}
			}

			if(car && car.packet && car.packet.lock)
			{
				if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
					carLock.visible=true;
					carUnlock.visible=false;
				}
				else{
					carLock.visible=false;
					carUnlock.visible=false;
				}
			}
			else if(car && car.packet && !car.packet.lock)
			{
				if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
					carLock.visible=false;
					carUnlock.visible=true;
				}
				else{
					carLock.visible=false;
					carUnlock.visible=false;
				}
			}
			else
			{
					carLock.visible=false;
					carUnlock.visible=false;
			}

		    updateIcons();

			layout();
		}

		private function updateIcons() : void
		{
            if (completeLoad)
            {
                if (!topCameraIcon) return;

                ultraIcon.visible=false;
                offsetIcon.visible=false;
                stanceIcon.visible=false;
                topCameraIcon.visible=false;
                hdResolutionIcon.visible=false;

                 if(car && (car.hdsize || car.ultraIcon || car.offsetIcon ||  car.topCameraIcon || car.stanceIcon))
                {

                    if (car.hdsize && (car.offsetIcon || car.stanceIcon))
                    {
                        hdResolutionIcon.x = (imageLoader.image.x + imageWidth - (hdResolutionIcon.width)) - 4;//
                        hdResolutionIcon.visible=true;
                        if(car.offsetIcon){

                            offsetIcon.x = (imageLoader.image.x + imageWidth - (offsetIcon.width)) + 4;// + 8;
                            offsetIcon.visible=true;
                        }
                        else if(car.stanceIcon){
                            stanceIcon.x = (imageLoader.image.x+imageWidth-(stanceIcon.width)) + 4;// + 8;
                            stanceIcon.visible=true;
                        }
                    }else if (car.ultraIcon && (car.offsetIcon || car.stanceIcon))
                    {
                        ultraIcon.x = (imageLoader.image.x + imageWidth - (ultraIcon.width)) - 4;//
                        ultraIcon.visible=true;

                        if(car.offsetIcon){
                            offsetIcon.x = (imageLoader.image.x + imageWidth - (offsetIcon.width)) + 4;// + 8;
                            offsetIcon.visible=true;
                        }
                        else if(car.stanceIcon){
                            stanceIcon.x = (imageLoader.image.x+imageWidth-(stanceIcon.width)) + 4;// + 8;
                            stanceIcon.visible=true;
                        }
                    }
                    else if(car.topCameraIcon && (car.offsetIcon || car.stanceIcon))
                    {

                        topCameraIcon.x=(imageLoader.image.x+imageWidth-(topCameraIcon.width)) - 4;//
                        topCameraIcon.visible=true;

                        if(car.offsetIcon){
                            offsetIcon.x=(imageLoader.image.x+imageWidth-(offsetIcon.width)) + 4;//+8;
                            offsetIcon.visible=true;
                        }
                        else if (car.stanceIcon){
                            stanceIcon.x=(imageLoader.image.x+imageWidth-(stanceIcon.width)) + 4;//+8;
                            stanceIcon.visible=true;
                        }
                    }
                    else if(car.topCameraIcon)
                    {
                        topCameraIcon.x=imageLoader.image.x+imageWidth-(topCameraIcon.width) - 4;
                        topCameraIcon.visible=true;
                    }
                    else if(car.hdsize)
                    {
                        hdResolutionIcon.x=imageLoader.image.x + imageWidth-(hdResolutionIcon.width) - 4;
                        hdResolutionIcon.visible=true;
                    }else if(car.ultraIcon)
                    {
                        ultraIcon.x=imageLoader.image.x + imageWidth-(ultraIcon.width) - 4;
                        ultraIcon.visible=true;
                    }
                    else if(car.offsetIcon)
                    {
                        offsetIcon.x=imageLoader.image.x+imageWidth-(offsetIcon.width) + 4;//+8;
                        offsetIcon.visible=true;
                    }
                    else if(car.stanceIcon)
                    {
                        stanceIcon.x=imageLoader.image.x+imageWidth-(stanceIcon.width) + 4;//+8;
                        stanceIcon.visible=true;
                    }

                    ultraIcon.y  		= (imageLoader.image.y  + imageHeight) - ultraIcon.height  + 8 + 12;
                    hdResolutionIcon.y  = (imageLoader.image.y  + imageHeight) - hdResolutionIcon.height  + 8 + 12;
                    offsetIcon.y 		= (imageLoader.image.y  + imageHeight) - offsetIcon.height + 8 + 12;
                    stanceIcon.y 		= (imageLoader.image.y  + imageHeight) - stanceIcon.height + 8 + 12;
                    topCameraIcon.y		= (imageLoader.image.y  + imageHeight) - topCameraIcon.height + 12 + 12;
                }
            }
		}
		
		protected override function autoSizeIfNeeded() : Boolean
		{
			return true;
		}
	}
}