package com.despark.tuning3d.view.components.cars
{
import com.despark.tuning3d.model.cars.Car;
import com.despark.tuning3d.model.core.AppMode;
import com.despark.tuning3d.model.core.AppMode;
import com.despark.tuning3d.model.core.AppMode;
    import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.packages.Packet;
import com.despark.tuning3d.view.BrandsScreenMediator;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.BrandsScreenBackgroundImage;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
import com.despark.tuning3d.view.components.core.ListItemRenderer;
import com.despark.tuning3d.view.components.core.MultilingualLabel;

import flash.display.Bitmap;
import flash.display.BitmapData;
import flash.display.Loader;
import flash.events.Event;
import flash.events.HTTPStatusEvent;
import flash.events.IEventDispatcher;
import flash.events.IOErrorEvent;
import flash.events.MouseEvent;
import flash.events.ProgressEvent;
import flash.filesystem.File;
import flash.geom.Rectangle;
import flash.net.URLRequest;
import flash.system.ApplicationDomain;
import flash.system.LoaderContext;
import flash.utils.setTimeout;

import starling.core.Starling;

import starling.display.Image;
import starling.display.Quad;
import starling.textures.Texture;
import starling.utils.AssetManager;

public class FeaturedCarItemRenderer extends ListItemRenderer
{
    private var quad:Quad;
    private var glowTweenAdded:Boolean=false;
    private var carLockIcon:Image;
    private var carUnlockIcon:Image;
    private var backgroundImage:BrandsScreenBackgroundImage; //BrandsScreenBackgroundImage
    private static var embbededBgTexture : Texture;
    private var loader:Loader;

    private var _clipRectQuad:Quad;
    private var imageWidth  : Number = -1;
    private var imageHeight : Number = -1;

    public static var assetManager : AssetManager;
    private var _carLastId : String = "";
    private static var _cachedTextures:Object;

    public var isLoaded: Boolean = false;

    public function FeaturedCarItemRenderer()
    {
        super();

        if(!embbededBgTexture)
        {
            embbededBgTexture = Texture.fromColor(600, 600/1.7);
        }
        _cachedTextures ||= {};
        _hitArea=new Rectangle(0,0,0,0);
        this._isQuickHitAreaEnabled=true;
    }

    public function get car() : Car{
        return data as Car;
    }

    override protected  function autoSizeIfNeeded():Boolean{
        return true;
    }

    public function get isCurrentImage():Boolean{
        var currentIndex:int = owner.dataProvider.getItemIndex(car);
        return (owner.horizontalPageIndex == currentIndex);
    }

    public function loading(): void {
        if (!imageLoader) {
            initialize();
        }
        if (!imageLoader.completeHandler) {
            imageLoader.completeHandler = completeLoadImage;
        }
        var url:String = car.featuredImageUrl;
        var newCar:Car;

        var isTruckImage:Boolean = AppMode.self.mode == AppMode.MODE_TRUCK_CONFIGURATOR;
        if(isTruckImage){
            imageLoader.sepcialDispose();
        }
        if (!imageLoader.persistentImageProvider || imageLoader.persistentImageProvider.remoteUrl != url || (isTruckImage && !imageLoader.currentTexture && isCurrentImage)) {
            newCar = new Car();
            newCar.id = car.id;
            newCar.imageURL = url;

            imageLoader.completeHandler = completeLoadImage;
            imageLoader.errorHandler2 = errorLoadImage;
            imageLoader.persistentImageProvider = newCar;
        }

        if (car && car.packet && car.packet.lock) {
            if (car.packet.type != Packet.TYPE_SUBCOMPONENT) {
                carLockIcon.visible = true;
                carUnlockIcon.visible = false;
            } else {
                carLockIcon.visible = false;
                carUnlockIcon.visible = false;
            }
        } else if (car && car.packet) {
            if (car.packet.type != Packet.TYPE_SUBCOMPONENT) {
                carLockIcon.visible = false;
                carUnlockIcon.visible = true;
            } else {
                carLockIcon.visible = false;
                carUnlockIcon.visible = false;
            }
        } else {
            carLockIcon.visible = false;
            carUnlockIcon.visible = false;
        }

        loadBg();

    }

    protected override function initialize():void
    {
        _clipRectQuad = new Quad(1,1);

        quad = new Quad(this.actualWidth, this.actualHeight, 0x008800);
        quad.alpha = 0;
        addChild(quad);

        backgroundImage = new BrandsScreenBackgroundImage(embbededBgTexture);
        addChild(backgroundImage);
        backgroundImage.width = actualWidth;
        backgroundImage.height = actualHeight;

        //super.initialize();

        label = new MultilingualLabel();
        addChild(label);

        imageLoader = new ImageLoaderWithStates();
        imageLoader.overrideDispose = true;
        addChild(imageLoader);

        var carLockTexture:Texture=ButtonFactory.assetManager.getTexture("car_lock_big_icon");
        carLockIcon=new Image(carLockTexture);
        carLockIcon.visible=false;
        addChild(carLockIcon);

        var carUnlockTexture:Texture=ButtonFactory.assetManager.getTexture("car_unlock_big_icon");
        carUnlockIcon=new Image(carUnlockTexture);
        carUnlockIcon.visible=false;
        addChild(carUnlockIcon);
    }



    public function specialDispose():void{

        _carLastId = "";
        if(backgroundImage && backgroundImage.texture  ){
            if(backgroundImage.texture != embbededBgTexture){
                backgroundImage.texture.dispose();
            }
        }

//        if(this.quad)
//        {
//            this.quad.removeFromParent(true);
//            this.quad = null;
//        }
//
//        if(this.carLockIcon)
//        {
//            this.carLockIcon.removeFromParent(true);
//            this.carLockIcon = null;
//        }
//
//        if(this.carUnlockIcon)
//        {
//            this.carUnlockIcon.removeFromParent(true);
//            this.carUnlockIcon = null;
//        }

//       / disposeLoader();

        /*if(imageLoader){
            imageLoader.removeFromParent(true);
            imageLoader = null;
        }*/
    }

    override public function dispose():void{
//        specialDispose();
        super.dispose();
    }

    protected override function layout():void
    {
        if(_clipRectQuad){
            _clipRectQuad.width = actualWidth;
            _clipRectQuad.height = actualHeight;
            this.mask = _clipRectQuad;
        }

        label.visible=false;
        resampleCar(imageLoader);

        quad.width=this.actualWidth;
        quad.height=this.actualHeight;

        carLockIcon.x=(actualWidth/2)+50;
        carLockIcon.y=imageLoader.y+((imageLoader.height-carLockIcon.height)/2)-20;
        carLockIcon.visible = false;

        carUnlockIcon.x=(actualWidth/2)+50;
        carUnlockIcon.y=imageLoader.y+((imageLoader.height-carLockIcon.height)/2)-20;
        carUnlockIcon.visible = false;

        _hitArea.width=actualWidth;
        _hitArea.height=actualHeight;
    }

    private function resampleCar(target:ImageLoaderWithStates): void {
        if(AppMode.self.mode == AppMode.MODE_TRUCK_CONFIGURATOR){
            resampleTruck(target);
            return;
        }

        var scale:Number = 1;
        var x_offset:Number = 0;//-13;
        var y_offset:Number = 0;// 27;
        var tmpScale:Number = 1.084*scale;
        var tmpWidth:Number  = actualWidth; //isEmulator ? actualWidth * (Constants.SCALE_FACTOR < 3.375 ? 1 : 4) : Starling.current.nativeStage.stageWidth / 2;
        var tmpHeight:Number = actualHeight;//isEmulator ? actualHeight* (Constants.SCALE_FACTOR < 3.375 ? 1 : 4) : Starling.current.nativeStage.stageHeight / 2;

        target.width = tmpWidth * 2 * 0.75 * tmpScale;
        target.height = tmpHeight * 1.3 * 0.75 * tmpScale;
        if(car && !car.hdsize) {
            y_offset+=23;//28;
            tmpScale = 0.59;
            target.width = tmpWidth * 2 * 0.75 * tmpScale;
            target.height = tmpHeight * 1.3 * 0.75 * tmpScale;
        }
        target.x = (actualWidth - target.width) / 2 + x_offset;
        target.y = (actualHeight - target.height) / 2 + y_offset;
    }

    private function resampleTruck(target:ImageLoaderWithStates): void {
        var tmpWidth:Number  = actualWidth;
        var tmpHeight:Number = actualHeight - 80;
        var actualRatio:Number = tmpWidth / tmpHeight;
        var targetRatio:Number = target.width / target.height;
        if(targetRatio > actualRatio){
            target.height = tmpHeight;
        }else{
            target.width = tmpWidth;
        }
        target.x = (actualWidth - target.width) / 2;
        target.y = (actualHeight - target.height) / 2;
    }


    private function reset() : void
    {
        /*if(backgroundImage && backgroundImage.texture && backgroundImage.texture != embbededBgTexture) {
            backgroundImage.texture.dispose();
            backgroundImage.texture = null;
        }*/

        disposeLoader();

        if(imageLoader )
        {
            imageLoader.sepcialDispose();
        }
    }

    protected override function commitData() : void
    {
        super.commitData();

        Starling.juggler.delayCall(completeLoadImage, 5, null); // call complete to hide menu, in case internet is to slow, and

        if(car.id == _carLastId)
        {
            completeLoadImage(null);
            return;
        }
        _carLastId = car.id;
        reset();
        if (car) {
            if(AppMode.self.mode == AppMode.MODE_TRUCK_CONFIGURATOR && car.featuredImageUrl.indexOf('default3.png') == -1){
                car.featuredImageUrl = car.featuredImageUrl.replace("default.png", "default3.png");
            }
            var url:String = car.featuredImageUrl;
            var newCar:Car;
            if(AppMode.self.mode == AppMode.MODE_TRUCK_CONFIGURATOR) {
                completeLoadImage(null);
                if(isCurrentImage) {
                    newCar = new Car();
                    newCar.id = car.id;
                    newCar.imageURL = url;

                    imageLoader.sepcialDispose();

                    imageLoader.completeHandler = completeLoadImage;
                    imageLoader.errorHandler2 = errorLoadImage;
                    imageLoader.persistentImageProvider = newCar;
                }
            } else {

                if (!imageLoader.completeHandler) {
                    imageLoader.completeHandler = completeLoadImage;
                }


                if (!imageLoader.persistentImageProvider || imageLoader.persistentImageProvider.remoteUrl != url) {
                    newCar = new Car();
                    newCar.id = car.id;
                    newCar.imageURL = url;

                    imageLoader.completeHandler = completeLoadImage;
                    imageLoader.errorHandler2 = errorLoadImage;
                    imageLoader.persistentImageProvider = newCar;
                }

                if (car && car.packet && car.packet.lock) {
                    if (car.packet.type != Packet.TYPE_SUBCOMPONENT) {
                        carLockIcon.visible = true;
                        carUnlockIcon.visible = false;
                    } else {
                        carLockIcon.visible = false;
                        carUnlockIcon.visible = false;
                    }
                } else if (car && car.packet) {
                    if (car.packet.type != Packet.TYPE_SUBCOMPONENT) {
                        carLockIcon.visible = false;
                        carUnlockIcon.visible = true;
                    } else {
                        carLockIcon.visible = false;
                        carUnlockIcon.visible = false;
                    }
                } else {
                    carLockIcon.visible = false;
                    carUnlockIcon.visible = false;
                }

                loadBg();
            }
        }
    }

    private function disposeLoader() : void
    {
        if(loader)
        {
            if(loader.contentLoaderInfo && loader.contentLoaderInfo.hasEventListener(Event.COMPLETE))
            {
                deleteListeners(loader.contentLoaderInfo);
            }
            try
            {
                loader.unloadAndStop(true);

            }
            catch (e:Error) {trace(e.getStackTrace())}

            loader=null;
        }
    }

    private function loadBg():void{
        if(!car ) return;
        destroyCachedTexture();
        var bgNameList:Array = ["", "a_country.jpg", "a_garage.jpg", 'a_ford.png', 'a_wrangler.png'];
        var randomSplashUrl:String;
        if(AppMode.self.mode == AppMode.MODE_TRUCK_CONFIGURATOR){ // TODO : temporary solution/ to be removed
            if(car.defaultBgId == null){
                car.carChooserBgName = "35.jpg";
            }else{
                car.carChooserBgName = car.defaultBgId;
            }
        }
        if(car.carChooserBgName){

            if(car.carChooserBgName.indexOf('http')>= 0)
            {
                randomSplashUrl = car.carChooserBgName;
                if(!Constants.isWide)
                {
                    randomSplashUrl = randomSplashUrl.replace('.png', '@2.png');
                }
            }
            else {
                if (!Constants.isWide) {
                    switch (car.carChooserBgName) {
                        case 'a_ford.png': car.carChooserBgName = 'a_ford_ipad.png'; break;
                        case 'a_wrangler.png': car.carChooserBgName = 'a_wrangler_ipad.png'; break;
                        case 'a_silverado.png': car.carChooserBgName = 'a_silverado_ipad.png'; break;
                        case 'a_dodge.png': car.carChooserBgName = 'a_dodge_ipad.png'; break;
                        case 'ford_2019.png': car.carChooserBgName = 'ford_2019_ipad.png'; break;
                        case 'silverado_1500_18.png': car.carChooserBgName = 'silverado_1500_18_ipad.png'; break;
                        case 'dodge_ram2500_14.png': car.carChooserBgName = 'dodge_ram2500_14_ipad.png'; break;
                        case 'dodge_ram3500_14.png': car.carChooserBgName = 'dodge_ram3500_14_ipad.png'; break;
                        case 'gmc_sierra.png': car.carChooserBgName = 'gmc_sierra_ipad.png'; break;
                        case 'ford_f_250_supercrew.png': car.carChooserBgName = 'ford_f_250_supercrew_ipad.png'; break;
                        case 'ford_f_350.png': car.carChooserBgName = 'ford_f_350_ipad.png'; break;
                        case 'silverado_2500.png': car.carChooserBgName = 'silverado_2500_ipad.png'; break;
                        case 'silverado_3500.png': car.carChooserBgName = 'silverado_3500_ipad.png'; break;
                        case 'a_gladiator.png': car.carChooserBgName = 'a_gladiator_ipad.png'; break;
                        case 'a_gladiator.jps': car.carChooserBgName = 'a_gladiator_ipad.jpg'; break;
                        case 'wrangler_jli.png': car.carChooserBgName = 'wrangler_jli_ipad.png'; break;
                        case 'dodge_ram1500_19.png': car.carChooserBgName = 'dodge_ram1500_19_ipad.png'; break;
                        case 'silverado_1500_19.png': car.carChooserBgName = 'silverado_1500_19_ipad.png'; break;
                        case 'ford_f-150_raptor_17.png': car.carChooserBgName = 'ford_f-150_raptor_17_Ipad.png'; break;
                        case 'tundra.png': car.carChooserBgName = 'tundra_ipad.png'; break;
                        case 'tacoma.png': car.carChooserBgName = 'tacoma_ipad.png'; break;
                        case 'silverado_2020.png': car.carChooserBgName = 'silverado_2020_ipad.png'; break;
                        case 'sierra_2020.png': car.carChooserBgName = 'sierra_2020_ipad.png'; break;
                        case 'tesla.png': car.carChooserBgName = 'tesla_ipad.png'; break;
                        case 'bronco.png': car.carChooserBgName = 'bronco_ipad.png'; break;
                        case 'ram25002020.png': car.carChooserBgName = 'ram25002020_ipad.png'; break;
                        case 'ram35002020.png': car.carChooserBgName = 'ram35002020_ipad.png'; break;
                        case 'fordf1501978.png': car.carChooserBgName = 'fordf1501978_ipad.png'; break;
                        case 'silveradoc101981.png': car.carChooserBgName = 'silveradoc101981_ipad.png'; break;
                        case 'c101962.png': car.carChooserBgName = 'c101962_Ipad.png'; break;
                        case 'fordf1502008.png': car.carChooserBgName = 'fordf1502008_Ipad.png'; break;
                        case 'sierra2500hd2020.png': car.carChooserBgName = 'sierra2500hd2020_ipad.png'; break;
                        case 'fordf1001956.png': car.carChooserBgName = 'fordf1001956_ipad.png'; break;
                        case '31001955.png': car.carChooserBgName = '31001955_ipad.png'; break;
                        case 'apache1958.png': car.carChooserBgName = 'apache1958_ipad.png'; break;
                        case 'ram1991.png': car.carChooserBgName = 'ram1991_ipad.png'; break;
                        case 'c10cheyenne.png': car.carChooserBgName = 'c10cheyenne_ipad.png'; break;
                        case 'f1502015.png': car.carChooserBgName = 'f1502015_ipad.png'; break;
                        case 'f150supercab2019.png': car.carChooserBgName = 'f150supercab2019_ipad.png'; break;
                        case 'regularcab2015.png': car.carChooserBgName = 'regularcab2015_ipad.png'; break;
                        case 'fordf11949.png': car.carChooserBgName = 'fordf11949_ipad.png'; break;
                        case 'regularcab2019.png': car.carChooserBgName = 'regularcab2019_ipad.png'; break;
                        case 'ram1500regularcab.png': car.carChooserBgName = 'ram1500regularcab_pad.png'; break;
                        case 'ram1500quadcab2014.png': car.carChooserBgName = 'ram1500quadcab2014_pad.png'; break;
                        case 'chevrolet31001954.png': car.carChooserBgName = 'chevrolet31001954_pad.png'; break;
                        case 'k1500regularcab1988.png': car.carChooserBgName = 'k1500regularcab1988_pad.png'; break;
                        case 'ramsrt102006.png': car.carChooserBgName = 'ramsrt102006_pad.png'; break;
                        case 'f150svtraptor2013.png': car.carChooserBgName = 'f150svtraptor2013_ipad.png'; break;
                        case 'gmck15001971.png': car.carChooserBgName = 'gmck15001971_Ipad.png'; break;
                        case 'silverado15002007.png': car.carChooserBgName = 'silverado15002007_Ipad.png'; break;
                        case 'silverado2006.png': car.carChooserBgName = 'silverado2006_Ipad.png'; break;
                        case 'f1501986.png': car.carChooserBgName = 'f1501986_Ipad.png'; break;
                        case 'f1501992.png': car.carChooserBgName = 'f1501992_Ipad.png'; break;
                        case 'silverado15002019.png': car.carChooserBgName = 'silverado15002019_Ipad.png'; break;
                        case 'sierra1500regcab2020.png': car.carChooserBgName = 'sierra1500regcab2020_Ipad.png'; break;

                    }
                }


                randomSplashUrl= File.applicationDirectory.resolvePath("assets/textures/bgs/"+car.carChooserBgName).url;
            }


            loader = new Loader();
            configureListeners(loader.contentLoaderInfo);
            loader.load(new URLRequest(randomSplashUrl), new LoaderContext(false, ApplicationDomain.currentDomain, null));

        }else{
            if(car.bgIndex>0){
                randomSplashUrl = File.applicationDirectory.resolvePath("assets/textures/"+bgNameList[car.bgIndex]).url;
                loader = new Loader();
                configureListeners(loader.contentLoaderInfo)	;
                loader.load(new URLRequest(randomSplashUrl), new LoaderContext(false, ApplicationDomain.currentDomain, null));
            }else if(backgroundImage.texture != embbededBgTexture){
                if(backgroundImage.texture){
                    backgroundImage.texture.dispose();
                }

                backgroundImage.texture =embbededBgTexture;
                backgroundImage.width = actualWidth;
                backgroundImage.height = actualHeight;
            }
        }
    }

    private function configureListeners(dispatcher:IEventDispatcher):void {
        dispatcher.addEventListener(Event.COMPLETE, completeHandler);
        dispatcher.addEventListener(HTTPStatusEvent.HTTP_STATUS, httpStatusHandler);
        dispatcher.addEventListener(Event.INIT, initHandler);
        dispatcher.addEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);
        dispatcher.addEventListener(Event.OPEN, openHandler);
        dispatcher.addEventListener(ProgressEvent.PROGRESS, progressHandler);
        dispatcher.addEventListener(Event.UNLOAD, unLoadHandler);

    }

    private function deleteListeners(dispatcher:IEventDispatcher):void {
        dispatcher.removeEventListener(Event.COMPLETE, completeHandler);
        dispatcher.removeEventListener(HTTPStatusEvent.HTTP_STATUS, httpStatusHandler);
        dispatcher.removeEventListener(Event.INIT, initHandler);
        dispatcher.removeEventListener(IOErrorEvent.IO_ERROR, ioErrorHandler);
        dispatcher.removeEventListener(Event.OPEN, openHandler);
        dispatcher.removeEventListener(ProgressEvent.PROGRESS, progressHandler);
        dispatcher.removeEventListener(Event.UNLOAD, unLoadHandler);
    }

    private function completeHandler(event:Event):void {
        isLoaded = true;
        var bitmap:Bitmap;
        try{
            bitmap = loader.content as Bitmap;
        }catch (e:Error) {

        }
        if(!bitmap) {
            loadBg();
            return;
        }

        var bitmapData:BitmapData = bitmap.bitmapData;
        try{
            bitmapData.width;
            if(!bitmapData) {
                loadBg();
                return;
            }
        }catch (e:Error) {
            loadBg();
            return;
        }
        _cachedTextures ||= {};
        if(_cachedTextures[loader.contentLoaderInfo.url]){
            Texture(_cachedTextures[loader.contentLoaderInfo.url]).dispose();
            delete _cachedTextures[loader.contentLoaderInfo.url];
        }
        var texture:Texture = Texture.fromBitmapData(bitmapData,false,false,1);
        _cachedTextures[loader.contentLoaderInfo.url] = texture;
        resetBackground(texture);

        bitmapData.dispose();
    }

    private function destroyCachedTexture():void{
        if(!_cachedTextures)
            return;
        for(var key:String in _cachedTextures){
            Texture(_cachedTextures[key]).dispose();
        }
        _cachedTextures = {};
    }

    private function httpStatusHandler(event:HTTPStatusEvent):void {
        Tracer.print(event.toString());
    }

    private function initHandler(event:Event):void {
    }

    private function ioErrorHandler(event:IOErrorEvent):void {
        Tracer.print(event.toString());
        resetBackground(embbededBgTexture);
    }

    private function resetBackground(texture:Texture):void{
        if(backgroundImage.texture && backgroundImage.texture != texture && backgroundImage.texture != embbededBgTexture){
            backgroundImage.texture.dispose();
        }
        backgroundImage.texture = texture;
        backgroundImage.width = actualWidth;
        backgroundImage.height = actualHeight;

        if(loader) {
            deleteListeners(loader.contentLoaderInfo);
        }
    }

    private function openHandler(event:Event):void {
    }

    private function progressHandler(event:ProgressEvent):void {
    }

    private function unLoadHandler(event:Event):void {
    }

    private function clickHandler(event:MouseEvent):void {
        var loader:Loader = Loader(event.target);
        loader.unload();
    }


    private function errorLoadImage(image:ImageLoaderWithStates):void{
        completeLoadImage(null)
    }

    private function completeLoadImage(image:ImageLoaderWithStates):void{
        BrandsScreenMediator.FIRST_LOAD_COMPLETE = true;
        BrandsScreenMediator.SECOND_LOAD_COMPLETE = true;
        if(this.owner)
            this.owner.dispatchEventWith("featuredCompleteLoad");
    }

    protected override function draw():void {
        super.draw();

        backgroundImage.width = actualWidth;
        backgroundImage.height = actualHeight;
        backgroundImage.setSize(actualWidth, actualHeight);
        backgroundImage.alignCenter();

        if (car) {
            var url:String=car.featuredImageUrl;
            var newCar:Car;
            var deleteFile:Boolean=false;

            if (imageLoader.persistentImageProvider) {
                var localFile: File = File.cacheDirectory.resolvePath(imageLoader.persistentImageProvider.localFile);

                if (localFile.exists) {
                    var creationDate:Date=new Date();
                    creationDate.setTime(Date.parse(localFile.creationDate));
                    var nowDate:Date=new Date();
                    var interval:int=(nowDate.getTime()/1000)-(creationDate.getTime()/1000);

                    if(interval>60*60*3){
                        localFile.deleteFile();
                        deleteFile=true;
                    }
                }
            }


            var b:Boolean = (AppMode.self.mode == AppMode.MODE_TUNING || (isCurrentImage && AppMode.self.mode == AppMode.MODE_TRUCK_CONFIGURATOR));

            if ((imageLoader.persistentImageProvider && imageLoader.persistentImageProvider.remoteUrl != url && b) || deleteFile) {
                newCar=new Car();
                newCar.id=car.id;
                newCar.imageURL = url;//.replace("3.png", ".png");
                imageLoader.persistentImageProvider = newCar;
            } else if (!imageLoader.persistentImageProvider && b) {
                newCar=new Car();
                newCar.id=car.id;
                newCar.imageURL = url;//.replace("3.png", ".png");
                imageLoader.persistentImageProvider = newCar;
            }

            if (car && car.packet && car.packet.lock) {
                if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
                    carLockIcon.visible=true;
                    carUnlockIcon.visible=false;
                }
                else{
                    carLockIcon.visible=false;
                    carUnlockIcon.visible=false;
                }
            } else if(car && car.packet) {
                if(car.packet.type!=Packet.TYPE_SUBCOMPONENT){
                    carLockIcon.visible=false;
                    carUnlockIcon.visible=true;
                }
                else{
                    carLockIcon.visible=false;
                    carUnlockIcon.visible=false;
                }
            } else{
                carLockIcon.visible=false;
                carUnlockIcon.visible=false;
            }
        }
        label.visible=false;
        if(imageLoader){
            resampleCar(imageLoader);
        }
    }
}
}