package com.despark.tuning3d.view.components.challenges {
    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.challenges.ChallengeImageLoader;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.tunings.TuningsProxy;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeListLabelFactory;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeListTimerLabel;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeNotesLabelFactory;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeTimerLabel;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ListItemRenderer;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.utils.DateParser;
    import com.utils.ImageUtils;

    import feathers.controls.Button;

    import feathers.controls.ImageLoader;
    import feathers.controls.Label;
    import feathers.controls.List;
    import feathers.controls.ToggleButton;

    import feathers.core.FeathersControl;

    import flash.display.Bitmap;
    import flash.display.Loader;
    import flash.display.StageAspectRatio;
    import flash.display.StageOrientation;
    import flash.events.Event;
    import flash.events.IOErrorEvent;
    import flash.geom.Rectangle;
    import flash.net.URLRequest;

    import starling.core.Starling;
    import starling.display.DisplayObject;

    import starling.display.Image;
import starling.display.Quad;
import starling.events.EnterFrameEvent;
    import starling.events.Event;
    import starling.events.EventDispatcher;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.textures.Texture;

    public class ChallengeListItemRenderer extends ListItemRenderer {
        private var _ownerImage: ImageLoader;
        private var _receiverImage: ImageLoader;

        private var _ownerBg: ChallengeImageLoader;
        private var _receiverBg: ChallengeImageLoader;

        private var _currentData: ChallengeData;

        private var _ownerUser: Button;
        private var _receiverUser: Button;


        private var _avatarSize : Number = 40;
        private var _vsIcon: Image;
        private var _timeLeftLabel: ChallengeListTimerLabel;
        private var _winner: Image;
        private var _winner2: Image;

        private var _ownerLikeButton: ToggleButton;
        private var _receiverLikeButton: ToggleButton;
        private var _itemOffsetHorizontal: int = 2.5;
        private var _interval: uint;
        public var list: List;

        private var _uiLeftOffset: int = 0;
        private var _uiRightOffset: int = 0;
        private var _eventDispatcher:EventDispatcher;

        public function ChallengeListItemRenderer() {
        }

        protected function get challengeData() : ChallengeData {
            return _data as ChallengeData;
        }

        override protected function initialize():void {
            super.initialize();
            if (Device.isIphoneXVariant()) {
                if (Constants.STAGE.deviceOrientation === StageOrientation.ROTATED_LEFT) {
                    _uiLeftOffset = 27;
                    _uiRightOffset = 0;
                } else {
                    _uiLeftOffset = 0;
                    _uiRightOffset = 27;
                }
            }
            if(_ownerBg){
                _ownerBg.removeFromParent(true);
            }


            _ownerBg = new ChallengeImageLoader();
            _ownerBg.addEventListener(starling.events.Event.COMPLETE, loadComplete);
            _ownerBg.addEventListener( starling.events.Event.IO_ERROR, loader_ioErrorHandler );
            _ownerBg.width = actualWidth / 2 - _itemOffsetHorizontal;
            _ownerBg.scaleY = Math.abs(_ownerBg.scaleX);
            _ownerBg.x = 0;
            _ownerBg.y = 0;
            _ownerBg.visible = false;
            _ownerBg.addEventListener(TouchEvent.TOUCH, selectChallengeHandler);
            addChild(_ownerBg);


            if(_receiverBg){
                _receiverBg.removeFromParent(true);
            }

            _receiverBg = new ChallengeImageLoader();
            _receiverBg.addEventListener(starling.events.Event.COMPLETE, loadComplete);
            _receiverBg.addEventListener( starling.events.Event.IO_ERROR, loader_ioErrorHandler );
            _receiverBg.width = actualWidth / 2 - _itemOffsetHorizontal;
            _receiverBg.scaleY = Math.abs(_receiverBg.scaleX);
            _receiverBg.x = actualWidth / 2 + _itemOffsetHorizontal;
            _receiverBg.y = 0;
            _receiverBg.visible = false;
            _receiverBg.addEventListener(TouchEvent.TOUCH, selectChallengeHandler);
            addChild(_receiverBg);


            if(_ownerImage){
                _ownerImage.removeFromParent(true);
            }
            _ownerImage = new ChallengeImageLoader();
            _ownerImage.addEventListener(starling.events.Event.COMPLETE, loadComplete);
            _ownerImage.addEventListener( starling.events.Event.IO_ERROR, loader_ioErrorHandler );
            _ownerImage.width = actualWidth / 2 - _itemOffsetHorizontal;
            _ownerImage.x = 0;
            _ownerImage.y = 0;
            _ownerImage.visible = false;
            _ownerImage.addEventListener(TouchEvent.TOUCH, selectChallengeHandler);
            addChild(_ownerImage);

            if(_receiverImage){
                _receiverImage.removeFromParent(true);
            }
            _receiverImage = new ChallengeImageLoader();
            _receiverImage.addEventListener(starling.events.Event.COMPLETE, loadComplete);
            _receiverImage.addEventListener( starling.events.Event.IO_ERROR, loader_ioErrorHandler );
            _receiverImage.width = actualWidth / 2 - _itemOffsetHorizontal;
            _receiverImage.scaleY = Math.abs(_receiverImage.scaleX);
            _receiverImage.x = actualWidth / 2 + _itemOffsetHorizontal;
            _receiverImage.y = 0;
            _receiverImage.visible = false;
            _receiverImage.addEventListener(TouchEvent.TOUCH, selectChallengeHandler);
            addChild(_receiverImage);




            if(_vsIcon){
                _vsIcon.removeFromParent(true);
            }
            _vsIcon = new Image(ButtonFactory.assetManager.getTexture('vs_big'));
            _vsIcon.height = actualHeight / 2;
            _vsIcon.scaleY = _vsIcon.scaleX;
            _vsIcon.x = actualWidth / 2 - _vsIcon.width / 2;
            _vsIcon.y = actualHeight / 2 - _vsIcon.height / 2;
            _vsIcon.touchable = false;
            addChild(_vsIcon);





            if(_ownerLikeButton){
                _ownerLikeButton.removeFromParent(true);
            }
            _ownerLikeButton = ButtonFactory.challengeLiteButton('challenge-like-1', 'challenge-like-selected-1', "left");
            _ownerLikeButton.x = 5 + _uiLeftOffset;
            _ownerLikeButton.setSize(60,30);
            _ownerLikeButton.y = actualHeight - 5 - _ownerLikeButton.height;
            addChild(_ownerLikeButton);

            if(_receiverLikeButton){
                _receiverLikeButton.removeFromParent(true);
            }
            _receiverLikeButton = ButtonFactory.challengeLiteButton('challenge-like-2', 'challenge-like-selected-2', "right");
            _receiverLikeButton.setSize(60,30);
            _receiverLikeButton.x = actualWidth - 5 - _receiverLikeButton.width - _uiRightOffset;
            _receiverLikeButton.y = actualHeight - 5 - _receiverLikeButton.height;
            addChild(_receiverLikeButton);

            _receiverLikeButton.addEventListener(starling.events.Event.TRIGGERED, receiverVote);
            _ownerLikeButton.addEventListener(starling.events.Event.TRIGGERED, ownerVote);

            _winner = new Image(AlertView.assetManager.getTexture('win'));
            _winner.visible = false;
            _winner.height = actualHeight / 4;
            _winner.scale = _winner.scaleY;
            _winner.touchable = false;
            addChild(_winner);

            if(_timeLeftLabel){
                _timeLeftLabel.removeFromParent(true);
            }
            _timeLeftLabel = new ChallengeListTimerLabel();

            _timeLeftLabel.width = _timeLeftLabel.background.texture.width;
            _timeLeftLabel.height = _timeLeftLabel.background.texture.height;
            _timeLeftLabel.x = actualWidth / 2 - _timeLeftLabel.width / 2;
            _timeLeftLabel.y = actualHeight - _timeLeftLabel.height;
            addChild(_timeLeftLabel);

        }

        private function monitorOrientation(e : EnterFrameEvent  ): void {
            if (Constants.STAGE.deviceOrientation === StageOrientation.ROTATED_LEFT) {
                _uiLeftOffset = 27;
                _uiRightOffset = 0;
            } else {
                _uiLeftOffset = 0;
                _uiRightOffset = 27;
            }
        }

        override protected function layout():void {
            super.layout();
        }

        override protected function commitData():void {
            if (data && _currentData != data) {
                _currentData = data as ChallengeData;
                //
                if(_winner2){
                    _winner2.removeFromParent(true);
                    _winner2 = null;
                }
                //
                update();
            }
        }

        private function selectChallengeHandler(e: starling.events.TouchEvent): void {
            var touch: Touch = e.getTouch(e.target as DisplayObject, TouchPhase.ENDED);

            var isSwitched: Boolean = false;


            if (touch && !list.isScrolling && _currentData.receiver &&  _currentData.owner && _currentData.receiver.tuning && _currentData.owner.tuning) {
                _currentData.receiver.tuning.user =  _currentData.receiver.user;
                _currentData.owner.tuning.user =  _currentData.owner.user;


                if (e.target == _receiverImage || e.target == _receiverBg) {
                    isSwitched = true;
                }
                var eventData:Object = {
                    receiverTuning: TuningsProxy.parseTuning(_currentData.receiver.tuning),
                    senderTuning: TuningsProxy.parseTuning(_currentData.owner.tuning),
                    challengeData: _currentData,
                    isSwitched: isSwitched
                };
                dispatchEvent(new starling.events.Event(ChallengeEvent.VIEW_CHALLENGE_TUNING, false, eventData));
            }
        }

        protected override function draw() : void {
            super.draw();
        }

        protected function update() : void {

            _ownerBg.scale = 1;
            _receiverBg.scale = 1;
            _ownerImage.scale = 1;
            _receiverImage.scale = 1;

            _ownerBg.visible = false;
            _receiverBg.visible = false;
            _ownerImage.visible = false;
            _receiverImage.visible = false;



            _ownerImage.source = _currentData.getCarFullHdUrl('owner');
            _receiverImage.source = _currentData.getCarFullHdUrl('receiver');

            if (_currentData.start_date && !_currentData.isWin('owner') && !_currentData.isWin('receiver')) {
                _timeLeftLabel.setTime(_currentData.start_date);
                _timeLeftLabel.x = actualWidth / 2 - _timeLeftLabel.background.texture.width / 2;
                _timeLeftLabel.y = actualHeight - _timeLeftLabel.background.texture.height;
                _timeLeftLabel.visible = !_timeLeftLabel.isFinished;
            } else {
                _timeLeftLabel.visible = false;
            }


            _ownerLikeButton.label = _currentData.owner.votes;
            _receiverLikeButton.label = _currentData.receiver.votes;


            if(_ownerUser){
                _ownerUser.removeFromParent(true);
            }

            if(_currentData && _currentData.owner && _currentData.owner.user) {
                _ownerUser = new Button();
                _ownerUser.defaultSkin = new UserCircularAvatar(new User(_currentData.owner.user), _avatarSize, true, 1.5);
                _ownerUser.addEventListener(starling.events.Event.TRIGGERED, onUserAvatarSelectHandler);
                _ownerUser.x = 5 + _uiLeftOffset;
                _ownerUser.y = 5;
                addChild(_ownerUser);
            }

            if(_receiverUser){
                _receiverUser.removeFromParent(true);
            }

            if(_currentData && _currentData.receiver && _currentData.receiver.user) {
                _receiverUser = new Button();
                _receiverUser.defaultSkin= new UserCircularAvatar(new User(_currentData.receiver.user), _avatarSize, true, 1.5);
                _receiverUser.x = actualWidth - 5 - _avatarSize - _uiRightOffset;
                _receiverUser.y = 5;
                _receiverUser.addEventListener(starling.events.Event.TRIGGERED, onUserAvatarSelectHandler);
                addChild(_receiverUser);
            }
            _ownerLikeButton.isSelected = !_currentData.canIVote('owner');
            _ownerLikeButton.touchable = _currentData.canIVote('owner');
            _receiverLikeButton.isSelected = !_currentData.canIVote('receiver');
            _receiverLikeButton.touchable = _currentData.canIVote('receiver');

            var haveWinner: Boolean = checkWins();

            if (!haveWinner) {
                if (_interval) {
                    Starling.juggler.removeByID(_interval);
                }
                _interval = Starling.juggler.repeatCall(function (): void {
                    var challengesProxy:ChallengesProxy = ApplicationFacade.getInstance().retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
                    challengesProxy.getChallenge(_currentData._id, function (newData: Object): void {
                        if (newData && newData.owner && newData.receiver) {
                            if (_ownerLikeButton) {
                                _ownerLikeButton.label = newData.owner.votes;
                            }
                            if (_receiverLikeButton) {
                                _receiverLikeButton.label = newData.receiver.votes;
                            }

                            _currentData.owner.votes = newData.owner.votes;
                            _currentData.receiver.votes = newData.receiver.votes;

                            _currentData.owner.winner = newData.owner.winner;
                            _currentData.receiver.winner = newData.receiver.winner;

                            if (_currentData) {
                                if (_ownerLikeButton){
                                    _ownerLikeButton.isSelected = !_currentData.canIVote('owner');
                                }

                                if (_ownerLikeButton) {
                                    _ownerLikeButton.touchable = _currentData.canIVote('owner');
                                }
                                if (_receiverLikeButton) {
                                    _receiverLikeButton.isSelected = !_currentData.canIVote('receiver');
                                }
                                if (_receiverLikeButton) {
                                    _receiverLikeButton.touchable = _currentData.canIVote('receiver');
                                }
                            }



                            checkWins();
                        }
                    })
                }, 5);
            }
        }

        private function onUserAvatarSelectHandler(e: starling.events.Event): void {
            var user: User;
            if (e.target == _ownerUser) {
                user = new User(_currentData.owner.user);
            } else if (e.target == _receiverUser) {
                user = new User(_currentData.receiver.user);
            }
            Constants.selectedUser = user;
            if(UsersProxy.loggedInUser() && UsersProxy.loggedInUser().id == user.id){
                return;
            }
            _eventDispatcher.dispatchEventWith(StackedScreen.USER_DETAILS_SELECTED, false, {user: Constants.selectedUser});
        }

        private function checkWins(): Boolean {
            var haveWinner: Boolean = false;
            var isTie:Boolean = _currentData.isWin('owner') && _currentData.isWin('receiver');
            if(isTie){
                _winner2 = new Image(AlertView.assetManager.getTexture('win'));
                _winner2.height = actualHeight / 4;
                _winner2.scale = _winner.scaleY;
                _winner2.touchable = false;
                addChild(_winner2);

                _winner.x = (actualWidth / 2 - _itemOffsetHorizontal ) - actualWidth / 4 - _winner.width / 2;
                _winner.y = actualHeight / 2 - _winner.height / 2;
                _winner.visible = true;

                _winner2.x = (actualWidth / 2 - _itemOffsetHorizontal) - actualWidth / 4 - _winner2.width / 2 + (actualWidth / 2 - _itemOffsetHorizontal);
                _winner2.y = actualHeight / 2 - _winner2.height / 2;
                _winner2.visible = true;
                haveWinner = true;
            }else if (_currentData.isWin('owner')) {
                _winner.x = (actualWidth / 2 - _itemOffsetHorizontal ) - actualWidth / 4 - _winner.width / 2;
                _winner.y = actualHeight / 2 - _winner.height / 2;
                _winner.visible = true;
                haveWinner = true;
            } else if (_currentData.isWin('receiver')) {
                _winner.x = (actualWidth / 2 - _itemOffsetHorizontal) - actualWidth / 4 - _winner.width / 2 + (actualWidth / 2 - _itemOffsetHorizontal);
                _winner.y = actualHeight / 2 - _winner.height / 2;
                _winner.visible = true;
                haveWinner = true;
            } else if(_winner){
                _winner.visible = false;
            }

            return haveWinner;
        }

        private function ownerVote(event: starling.events.Event): void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            var challengesProxy:ChallengesProxy = ApplicationFacade.getInstance().retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengesProxy.voteChallenge(_currentData._id, 'owner', _currentData.owner.tuning._id, function (newData: ChallengeData): void {
                _ownerLikeButton.isSelected = true;
                _ownerLikeButton.touchable = false;
                if (newData) {
                    _currentData.owner.votes = newData.owner.votes;
                    _ownerLikeButton.label = newData.owner.votes;
                }
            }, null, function():void{
                _ownerLikeButton.isSelected = false;
                _ownerLikeButton.touchable = true;
            });
        }

        private function receiverVote(event: starling.events.Event): void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            var challengesProxy:ChallengesProxy = ApplicationFacade.getInstance().retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengesProxy.voteChallenge(_currentData._id, 'receiver', _currentData.receiver.tuning._id, function (newData: ChallengeData): void {
                _receiverLikeButton.isSelected = true;
                _receiverLikeButton.touchable = false;
                if (newData) {
                    _currentData.receiver.votes = newData.receiver.votes;
                    _receiverLikeButton.label = newData.receiver.votes;
                }

            },null, function():void{
                _receiverLikeButton.isSelected = false;
                _receiverLikeButton.touchable = true;
            })
        }

        private function loader_ioErrorHandler(e: starling.events.Event): void {
            trace(e)
        }

        private function loadComplete(e: starling.events.Event): void {
            var koef: Number;
            switch (e.target) {
                case _ownerBg:
                    _ownerBg.x = _ownerBg.width;
                    _ownerBg.scaleX = -1;
                    _ownerBg.visible = true;
                    break;
                case _receiverBg:
                    _receiverBg.visible = true;
                    break;
                case _ownerImage:
                    koef = actualWidth / 2 / (_ownerImage as ChallengeImageLoader).imageTexture.width ;
                    _ownerImage.width = actualWidth / 2;
                    _ownerImage.height = (_ownerImage as ChallengeImageLoader).imageTexture.height * koef;
                    _ownerImage.y = 0;
                    _ownerImage.scaleY = Math.abs(_ownerImage.scaleY);
                    _ownerImage.x = 0;
                    _ownerImage.visible = true;
                    if(!_ownerImage.hasEventListener(TouchEvent.TOUCH, selectChallengeHandler)){
                        _ownerImage.addEventListener(TouchEvent.TOUCH, selectChallengeHandler);
                    }
                    break;
                case _receiverImage:
                    koef = actualWidth / 2 / (_receiverImage as ChallengeImageLoader).imageTexture.width ;
                    _receiverImage.width = actualWidth / 2;
                    _receiverImage.height = (_receiverImage as ChallengeImageLoader).imageTexture.height * koef;
                    _receiverImage.y = (actualHeight - _receiverImage.height) / 2;
                    _receiverImage.visible = true;
                    if(!_receiverImage.hasEventListener(TouchEvent.TOUCH, selectChallengeHandler)){
                        _receiverImage.addEventListener(TouchEvent.TOUCH, selectChallengeHandler);
                    }
                    break
            }
            if(_ownerBg && _ownerBg.image){
                if(_ownerBg.image.height < actualHeight){
                    _ownerBg.image.height = actualHeight;
                    _ownerBg.image.color = 0xFFFF00;
                }
            }

            if(_receiverBg && _receiverBg.image){
                if(_receiverBg.image.height < actualHeight){
                    _receiverBg.image.height = actualHeight;
                    _receiverBg.image.color = 0xFFFF00;
                }
            }
        }

        override public function dispose():void {
            super.dispose();
            if (_ownerBg) {
                _ownerBg.removeEventListener(TouchEvent.TOUCH, selectChallengeHandler);
                _ownerBg.removeEventListener(starling.events.Event.COMPLETE, loadComplete);
                _ownerBg.removeEventListener( starling.events.Event.IO_ERROR, loader_ioErrorHandler );
                _ownerBg.removeFromParent(true);
                _ownerBg = null;
            }

            if (_receiverLikeButton) {
                _receiverLikeButton.removeEventListener(starling.events.Event.TRIGGERED, receiverVote);
                _receiverLikeButton.removeFromParent(true);
                _receiverLikeButton = null;
            }

            if (_ownerLikeButton) {
                _ownerLikeButton.removeEventListener(starling.events.Event.TRIGGERED, ownerVote);
                _ownerLikeButton.removeFromParent(true);
                _ownerLikeButton = null;
            }

            if (_receiverBg) {
                _receiverBg.removeEventListener(TouchEvent.TOUCH, selectChallengeHandler);
                _receiverBg.removeEventListener(starling.events.Event.COMPLETE, loadComplete);
                _receiverBg.removeEventListener( starling.events.Event.IO_ERROR, loader_ioErrorHandler );
                _receiverBg.removeFromParent(true);
                _receiverBg = null;
            }

            if (_ownerImage) {
                _ownerImage.removeEventListener(TouchEvent.TOUCH, selectChallengeHandler);
                _ownerImage.removeEventListener(starling.events.Event.COMPLETE, loadComplete);
                _ownerImage.removeEventListener( starling.events.Event.IO_ERROR, loader_ioErrorHandler );
                _ownerImage.removeFromParent(true);
                _ownerImage = null;
            }

            if (_receiverImage) {
                _receiverImage.removeEventListener(TouchEvent.TOUCH, selectChallengeHandler);
                _receiverImage.removeEventListener(starling.events.Event.COMPLETE, loadComplete);
                _receiverImage.removeEventListener( starling.events.Event.IO_ERROR, loader_ioErrorHandler );
                _receiverImage.removeFromParent(true);
                _receiverImage = null;
            }

            if (_timeLeftLabel) {
                _timeLeftLabel.removeFromParent(true);
                _timeLeftLabel = null;
            }

            if (_vsIcon) {
                _vsIcon.removeFromParent(true);
                _vsIcon = null;
            }

            if (_ownerUser) {
                _ownerUser.removeEventListener(starling.events.Event.TRIGGERED, onUserAvatarSelectHandler);
                _ownerUser.removeFromParent(true);
                _ownerUser = null;
            }

            if (_receiverUser) {
                _receiverUser.removeEventListener(starling.events.Event.TRIGGERED, onUserAvatarSelectHandler);
                _receiverUser.removeFromParent(true);
                _receiverUser = null;
            }

            if(_winner){
                _winner.removeFromParent(true);
                _winner = null;
            }

            if(_winner2){
                _winner2.removeFromParent(true);
                _winner2 = null;
            }

            if (_interval) {
                Starling.juggler.removeByID(_interval);
            }
        }


        public function set eventDispatcher(value:EventDispatcher):void {
            _eventDispatcher = value;
        }
    }
}
