package com.despark.tuning3d.view.components.challenges.controls {
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.tunings.TuningsProxy;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
import com.despark.tuning3d.view.components.instructions.SimpleImageProvider;

import feathers.core.FeathersControl;

import starling.core.Starling;

import starling.display.Image;
import starling.events.Event;

public class ChallengeLabel extends FeathersControl {
    private var _vsIcon:Image;
    private var _creatorCar:ThumbnailLoader;
    private var _opponentCar:ThumbnailLoader;

    private var _ownerTuning:Tuning;
    private var _receiverTuning:Tuning;

    private var _creatorCarImage:Image;
    private var _opponentCarImage:Image;
    private var _onComplete:Function;
    private var _imageHeight:Number;
    public function ChallengeLabel(ownerTuningObj:Object, receiverTuningObj:Object, imageHeight:Number = 70) {
        super();
        _imageHeight = imageHeight;
        if(ownerTuningObj) {
            _ownerTuning = TuningsProxy.parseTuning(ownerTuningObj, false);
        }
        if(receiverTuningObj) {
            _receiverTuning = TuningsProxy.parseTuning(receiverTuningObj, false);
        }
        if(_receiverTuning && _ownerTuning){
            createView();
        }
    }

    override protected function draw():void {
        super.draw();
        updateView();
    }

    override protected function initialize():void {
        super.initialize();
        updateView();
    }

    override public function dispose():void {
        if(_creatorCar){
            _creatorCar.removeFromParent(true);
            _creatorCar = null;
        }
        if(_opponentCar){
            _opponentCar.removeFromParent(true);
            _opponentCar = null;
        }
        if(_creatorCarImage){
            _creatorCarImage.texture.dispose();
            _creatorCarImage.removeFromParent(true);
            _creatorCarImage = null;
        }
        if(_opponentCarImage){
            _opponentCarImage.texture.dispose();
            _opponentCarImage.removeFromParent(true);
            _opponentCarImage = null;
        }
        super.dispose();
    }

    public function createView():void{
        if(_creatorCar || _opponentCar){
            return;
        }
        _creatorCar = new ThumbnailLoader();
        //_creatorCar.overrideDispose = true;
        var simpleImageProvider : SimpleImageProvider = new SimpleImageProvider();
        simpleImageProvider.urlForDownloading = _ownerTuning.thumbnailURL;
        //_creatorCar.persistentImageProvider = simpleImageProvider;
        //_creatorCar.startLoading();
        addChild(_creatorCar);
        _creatorCar.addEventListener(Event.COMPLETE, creatorCompleteHandler);
        _creatorCar.source = _ownerTuning.thumbnailURL;
        /*completeHandler = function():void{
            _creatorCarImage = new Image(_creatorCar.currentTexture);
            addChildAt(_creatorCarImage, 0);
            _creatorCarImage.height = _imageHeight;
            _creatorCarImage.width = -_imageHeight * _creatorCarImage.texture.nativeWidth/_creatorCarImage.texture.nativeHeight;

            _creatorCar.removeFromParent();
            updateView();
            if(_onComplete!=null){
                _onComplete.call();
            }
            Starling.juggler.delayCall(validate, 1);
        };
        _creatorCar.errorHandler = function():void{
            trace("failed loading thumbnail");
        };*/

        _opponentCar = new ThumbnailLoader();
        //_opponentCar.overrideDispose = true;
        var simpleImageProvider2 : SimpleImageProvider = new SimpleImageProvider();
        simpleImageProvider2.urlForDownloading = _receiverTuning.thumbnailURL;
        //_opponentCar.persistentImageProvider = simpleImageProvider2;
        //_opponentCar.startLoading();
        addChild(_opponentCar);
        _opponentCar.addEventListener(Event.COMPLETE, opponentCompleteHandler);
        _opponentCar.source = _receiverTuning.thumbnailURL;
        /*_opponentCar.completeHandler = function():void{
            _opponentCarImage = new Image(_opponentCar.currentTexture);
            addChildAt(_opponentCarImage, 0);
            _opponentCarImage.width = _imageHeight * _opponentCarImage.texture.nativeWidth/_opponentCarImage.texture.nativeHeight;
            _opponentCarImage.height = _imageHeight;

            _opponentCar.removeFromParent();
            updateView();
            if(_onComplete!=null){
                _onComplete.call();
            }
            Starling.juggler.delayCall(validate, 1);
        };
        _opponentCar.errorHandler = function():void{
            trace("failed loading thumbnail");
        };*/

        _vsIcon = new Image(ButtonFactory.assetManager.getTexture("vs_small"));
        _vsIcon.scale = (_imageHeight/70)*_vsIcon.texture.scale/2;
    }

    private function opponentCompleteHandler(event:Event):void {
        _opponentCarImage = new Image(_opponentCar.texture);
        addChildAt(_opponentCarImage, 0);
        _opponentCarImage.width = _imageHeight * _opponentCarImage.texture.nativeWidth/_opponentCarImage.texture.nativeHeight;
        _opponentCarImage.height = _imageHeight;

        _opponentCar.removeFromParent();
        updateView();
        if(_onComplete!=null){
            _onComplete.call();
        }
        Starling.juggler.delayCall(validate, 1);
    }

    private function creatorCompleteHandler(event:Event):void {
        _creatorCarImage = new Image(_creatorCar.texture);
        addChildAt(_creatorCarImage, 0);
        _creatorCarImage.height = _imageHeight;
        _creatorCarImage.width = -_imageHeight * _creatorCarImage.texture.nativeWidth/_creatorCarImage.texture.nativeHeight;

        _creatorCar.removeFromParent();
        updateView();
        if(_onComplete!=null){
            _onComplete.call();
        }
        Starling.juggler.delayCall(validate, 1);
    }

    public function updateView():void{
        if(_creatorCarImage && _opponentCarImage){
            if(_creatorCarImage.width > _opponentCarImage.width){
                _creatorCarImage.width = -_opponentCarImage.width;
                _creatorCarImage.height = _creatorCarImage.texture.nativeHeight * _creatorCarImage.width / _creatorCarImage.texture.nativeWidth;
            }else if(_creatorCarImage.width < _opponentCarImage.width){
                _opponentCarImage.width = _creatorCarImage.width;
                _opponentCarImage.height = _opponentCarImage.texture.nativeHeight * _opponentCarImage.width / _opponentCarImage.texture.nativeWidth;
            }
        }
        if(_creatorCarImage){
            //_creatorCarImage.visible = Boolean(_opponentCarImage);
            _creatorCarImage.x = _creatorCarImage.width/2 + (actualWidth-_creatorCarImage.width)/2;
            _creatorCarImage.y = (actualHeight - _creatorCarImage.height)/2;
        }
        if(_opponentCarImage){
            //_opponentCarImage.visible = Boolean(_creatorCarImage);
            _opponentCarImage.x = _opponentCarImage.width/2 + (actualWidth-_opponentCarImage.width)/2;
            _opponentCarImage.y = (actualHeight-_opponentCarImage.height)/2;
        }
        if(_creatorCarImage || _opponentCarImage){
            addChild(_vsIcon);
        }
        if(_vsIcon) {
            _vsIcon.x = (actualWidth - _vsIcon.width) / 2;
            _vsIcon.y = (actualHeight - _vsIcon.height) / 2;
        }
        if(isInitialized){
            validate();
        }
    }

    public function set onComplete(value:Function):void {
        _onComplete = value;
    }

    public function get ownerTuning():Tuning {
        return _ownerTuning;
    }

    public function get receiverTuning():Tuning {
        return _receiverTuning;
    }
}
}

import feathers.controls.ImageLoader;

import starling.textures.Texture;

class ThumbnailLoader extends ImageLoader{
    public function ThumbnailLoader(){
        super ();
    }

    public function get texture():Texture{
        return _texture;
    }
}