package com.despark.tuning3d.view.components.challenges.controls {
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.ListItemRenderer;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;

import feathers.controls.Label;
import feathers.controls.LayoutGroup;
import feathers.layout.HorizontalAlign;
import feathers.layout.HorizontalLayout;
import feathers.layout.VerticalAlign;


import starling.display.Canvas;

import starling.display.Image;

import starling.display.Quad;
    import starling.events.EventDispatcher;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;

    public class ChallengeLeaderBoardItemRenderer extends ListItemRenderer{
    private static const gap:int = 5;
    private static const avatarGap:int = 10;
    private var _committedData:Object;

    private var _bottomBorder:Quad;
    private var _avatar:UserCircularAvatar;
    private var _userNameLabel:Label;
    //
    private var _winCountLabel:Label;
    private var _tuningCountLabel:Label;
    private var _challengeCountLabel:Label;
    private var _tuningIcon:Image;
    private var _challengeIcon:Image;
    private var _winIcon:Image;
    private var _bottomGroup:LayoutGroup;
    private var _background:Quad;
    private var _currentUser:User;
    private var _paddingLeft:Number;
        private var _eventDispatcher:EventDispatcher;
    public function ChallengeLeaderBoardItemRenderer() {
        super ();
        _paddingLeft = 0;
    }

    override protected function commitData():void {
        super.commitData();
        if(data && _committedData != data){
            _committedData = data;
            updateItem();
        }
    }

    override protected function draw():void {
        super.draw();
        updateItem();
        if(isSelected){
            if(_tuningIcon && _challengeIcon && _winIcon){
                _tuningIcon.color = _challengeIcon.color = _winIcon.color = 0xFFCC00;
            }
            if(_background) {
                if (_currentUser && _committedData.user.id == _currentUser.id) {
                    _background.color = 0x2d7600;
                    _background.alpha = 1;
                }else{
                    _background.color = 0x2a5475;
                    _background.alpha = 1;
                }
            }
        }else{
            if(_tuningIcon && _challengeIcon && _winIcon){
                _tuningIcon.color = _challengeIcon.color = _winIcon.color = 0xFFFFFE;
            }
            if(_background) {
                if (_currentUser && _committedData.user.id == _currentUser.id) {
                    _background.color = 0x64DD17;
                    _background.alpha = 0.8;
                } else {
                    _background.color = 0x000000;
                    _background.alpha = 0.8;
                }
            }
        }
    }

    override protected function initialize():void {
        super.initialize();
        _background = new Quad(1,1,0xCCCCCC);
        _background.alpha = 0;
        _background.width = actualWidth;
        _background.height = actualHeight;
        addChildAt(_background, 0);
    }

    private function updateItem():void {
        _bottomBorder ||= new Quad(1,1, 0xFFFFFF);
        _bottomBorder.alpha = 0.5;
        _bottomBorder.width = actualWidth - 2 * gap;
        _bottomBorder.height = 0.5;
        _bottomBorder.x = gap;
        _bottomBorder.y = actualHeight - 0.5;
        if(_background && _currentUser && _committedData.user.id == _currentUser.id){
            _background.color = 0x64DD17;
            _background.alpha = 0.4;
        }
        addChild(_bottomBorder);
        //
        _avatar ||= new UserCircularAvatar(_committedData.user, 30, true, 0.75);
        if(_avatar.user != _committedData.user){
            _avatar.user = _committedData.user;
        }
        if(!_avatar.parent){
            addChild(_avatar);
        }
        _avatar.x = avatarGap + _paddingLeft;
        _avatar.y = avatarGap;
        _avatar.addEventListener(TouchEvent.TOUCH, avatarTouchHandler);
        var avatarWidth:Number = _avatar.width + 2 * avatarGap;

        _committedData.num = owner.dataProvider.getItemIndex(_committedData)+1;
        _userNameLabel ||= new Label();
        _userNameLabel.width = actualWidth - (avatarWidth + gap);
        _userNameLabel.textRendererFactory = ChallengeTextItemRenderer.itemUserFieldFactory;
        _userNameLabel.text = _committedData.num.toString() + ". " + User(_committedData.user).nameToShow;
        _userNameLabel.x = avatarWidth + 2 + _paddingLeft;
        _userNameLabel.y = gap;
        _userNameLabel.validate();
        addChild(_userNameLabel);
        //
        _winCountLabel ||= new Label();
        _winCountLabel.textRendererFactory = ChallengeTextItemRenderer.itemDetailFieldFactory;
        _winCountLabel.paddingLeft = -2;

        _tuningCountLabel ||= new Label();
        _tuningCountLabel.textRendererFactory = ChallengeTextItemRenderer.itemDetailFieldFactory;
        _tuningCountLabel.paddingLeft = -2;

        _challengeCountLabel ||= new Label();
        _challengeCountLabel.textRendererFactory = ChallengeTextItemRenderer.itemDetailFieldFactory;
        _challengeCountLabel.paddingLeft = -2;
        if(!_bottomGroup){
            _bottomGroup = new LayoutGroup();
            var hLayout:HorizontalLayout = new HorizontalLayout();
            hLayout.gap = 2;
            hLayout.horizontalAlign = HorizontalAlign.JUSTIFY;
            hLayout.verticalAlign = VerticalAlign.MIDDLE;

            _tuningIcon = new Image(ButtonFactory.assetManager.getTexture("leaderboard_car_icon"));
            _tuningIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            _challengeIcon = new Image(ButtonFactory.assetManager.getTexture("leaderboard_cup_icon"));
            _challengeIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            _winIcon = new Image(ButtonFactory.assetManager.getTexture("leaderboard_crown_icon"));
            _winIcon.scale = ButtonFactory.assetManager.scaleFactor/2;

            _bottomGroup.layout = hLayout;
            _bottomGroup.addChild(_winIcon);
            _bottomGroup.addChild(_winCountLabel);
            _bottomGroup.addChild(getHBorder());
            _bottomGroup.addChild(_challengeIcon);
            _bottomGroup.addChild(_challengeCountLabel);
            _bottomGroup.addChild(getHBorder());
            _bottomGroup.addChild(_tuningIcon);
            _bottomGroup.addChild(_tuningCountLabel);
            //
            _bottomGroup.width = actualWidth - (avatarWidth + gap);
            var labelWidth:Number = (_bottomGroup.width - (_tuningIcon.width + _challengeIcon.width + _winIcon.width + 3 + hLayout.gap * (_bottomGroup.numChildren - 1)))/3;
            _winCountLabel.width = _tuningCountLabel.width = _challengeCountLabel.width = labelWidth;
            _winCountLabel.height = _tuningCountLabel.height = _challengeCountLabel.height = 12;
        }

        HorizontalLayout(_bottomGroup.layout).paddingLeft = _paddingLeft;
        _winCountLabel.text = "-" + _committedData.win_count.toString();
        _tuningCountLabel.text = "-" + _committedData.tunings_count.toString();
        _challengeCountLabel.text = "-" + _committedData.total_challenges_count.toString();
        _bottomGroup.x = avatarWidth;
        _bottomGroup.y = _userNameLabel.y + _userNameLabel.height + gap;
        addChild(_bottomGroup);
    }

        private function avatarTouchHandler(event:TouchEvent):void {
            var touch:Touch = event.getTouch(_avatar, TouchPhase.ENDED);
            if(touch){
                event.stopImmediatePropagation();
                _eventDispatcher.dispatchEventWith(StackedScreen.USER_DETAILS_SELECTED, false, {user:_avatar.user});
            }
        }

    private static function getHBorder():Quad{
        var quad:Quad = new Quad(1,1,0xFFFFFF);
        quad.height = 14;
        quad.alpha = 0.5;
        return quad;
    }

    public function set currentUser(value:User):void {
        _currentUser = value;
    }

    public function set paddingLeft(value:Number):void {
        _paddingLeft = value;
    }

    public function set eventDispatcher(value:EventDispatcher):void{
        _eventDispatcher = value;
    }
}
}
