package com.despark.tuning3d.view.components.challenges.controls {
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.utils.DateParser;
    import com.utils.FontSelector;

    import feathers.controls.Label;
    import feathers.controls.text.TextFieldTextRenderer;
    import feathers.core.ITextRenderer;

    import flash.text.TextFormat;

    import flash.text.TextFormatAlign;

    import starling.core.Starling;

    import starling.display.Image;
    import starling.display.Sprite;


    public class ChallengeListTimerLabel extends Sprite  {
        public var background: Image;
        public var isFinished: Boolean;

        private var label: Label;
        private var _timeLeft: Number;

        private static const SECOND:Number = 1000;
        private static const MINUTE:Number = SECOND * 60;
        private static const HOUR:Number = MINUTE * 60;

        public function ChallengeListTimerLabel() {
            isFinished = false;
            label = new Label();
            background = new Image(ButtonFactory.assetManager.getTexture("challenge-countdown-challenge"));
            label.textRendererFactory = labelChallengeListRenderer;
            label.width = background.texture.width;
            label.x = 0;
            addChild(background);
            addChild(label);
        }

        public function setTime(value: String): void {
            isFinished = false;
            label.visible = true;
            _timeLeft = new Date().time - DateParser.parseDate(value).time;
            _timeLeft = ChallengeData.DURATION - _timeLeft;
            Starling.juggler.removeDelayedCalls(updateTime);
            updateTime();
        }

        private function updateTime(): void {
            if(_timeLeft<=0){
                background.visible = false;
                label.visible = false;
                isFinished = true;
                return;
            }

            background.visible = true;
            label.visible = true;

            _timeLeft -= SECOND;


            var hours: Number= int(_timeLeft/HOUR);
            var minute: Number = int((_timeLeft - hours * HOUR) / MINUTE);
            var seconds: Number = int((_timeLeft - hours * HOUR - minute * MINUTE) / SECOND);

            label.text = getPair(hours) + ':' + getPair(minute) + ':' + getPair(seconds);

            label.validate();
            Starling.juggler.delayCall(updateTime, 1);
        }

        private static function getPair(num:int):String{
            var str:String = num.toString();
            return str.length==1?"0"+str:str;
        }

        private static function labelChallengeListRenderer() :ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getFont(true, false), 10, 0x242424);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.italic = true;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        }

    }
}
