package com.despark.tuning3d.view.components.challenges.controls {
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
import com.despark.tuning3d.view.components.core.ListItemRenderer;
import com.despark.tuning3d.view.components.instructions.SimpleImageProvider;
import com.utils.FontSelector;

import feathers.controls.Label;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.ITextRenderer;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.display.Quad;

public class ChallengeTuningItemRenderer extends ListItemRenderer {
    private var _thumbnail:ImageLoaderWithStates;
    private var _background:Quad;
    private var _label:Label;
    private var _searchResultList:SearchResultList;
    private var loaded:Boolean;
    public function ChallengeTuningItemRenderer() {
        super();
    }

    private function get tuning():Tuning{
        return data as Tuning;
    }

    override public function setSize(width:Number, height:Number):void {
        super.setSize(width, height);
        if(_background){
            _background.width = width;
            _background.height = height;
        }
        if(_thumbnail) {
            _thumbnail.width = width;
            _thumbnail.height = height;
        }
    }

    override protected function commitData():void {
        super.commitData();
        _searchResultList = owner as SearchResultList;
        if(!tuning)
            return;
        if(_thumbnail){
            _thumbnail.removeFromParent(true);
            if(!loaded){
                _searchResultList.loadingItems--;
            }
        }
        if(!_background) {
            _background = new Quad(1, 1, 0xFFFFFF);
        }
        _thumbnail = new ImageLoaderWithStates();
        _searchResultList.loadingItems++;
        loaded = false;
        var simpleImageProvider : SimpleImageProvider = new SimpleImageProvider();
        simpleImageProvider.urlForDownloading = tuning.thumbnailURL;
        _thumbnail.persistentImageProvider = simpleImageProvider;
        _thumbnail.completeHandler = function():void{
            _thumbnail.width = _background.width = actualWidth;
            _thumbnail.height = 3*actualHeight/4;
            _searchResultList.loadingItems = 0;
            loaded = true;
        };
        _thumbnail.errorHandler = function():void{
            _searchResultList.loadingItems--;
            loaded = true;
        };

        if(!_label) {
            _label = new Label();
            _label.textRendererFactory = titleTextRenderer;
        }

        _background.height = actualHeight;
        _label.text = tuning.name;
        _label.width = actualWidth - 10;
        _label.height = actualHeight/4;
        _label.x = 5;
        _label.y = 3*actualHeight/4;

        addChild(_background);
        addChild(_thumbnail);
        addChild(_label);
    }

    private static function  titleTextRenderer ():ITextRenderer {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 9, 0x424242);
        textRenderer.embedFonts = FontSelector.embedFonts();
        textRenderer.textFormat.bold = false;
        textRenderer.embedFonts = true;
        textRenderer.textFormat.align = TextFormatAlign.CENTER;
        textRenderer.wordWrap = true;
        return textRenderer;
    }

    override public function dispose():void {
        super.dispose();
        if(_thumbnail) {
            _thumbnail.removeFromParent(true);
            _thumbnail = null;
        }
        if(_background) {
            _background.removeFromParent(true);
            _background = null;
        }
        if(_label) {
            _label.removeFromParent(true);
            _label = null;
        }
    }
}
}
