/**
 * Created by Render-2 on 8/10/2017.
 */
package com.despark.tuning3d.view.components.challenges.controls
{
import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.auth.ProfileAlertView;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.StackedScreen;
import com.despark.tuning3d.view.components.timeline.TimelineGarage.*;
import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
import com.despark.tuning3d.view.components.tunings.ITuningListMenuVIew;
    import com.utils.DisplayObjectUtil;

    import feathers.controls.List;
import feathers.controls.ScrollBarDisplayMode;
import feathers.controls.ScrollPolicy;
import feathers.controls.ToggleButton;
import feathers.controls.renderers.IListItemRenderer;
import feathers.core.FeathersControl;
import feathers.data.ListCollection;
import feathers.layout.HorizontalAlign;
import feathers.layout.HorizontalLayout;
import feathers.layout.VerticalAlign;

import starling.display.Canvas;
import starling.display.Image;
import starling.display.Mesh;
import starling.display.Quad;
import starling.display.Sprite;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;
import starling.extensions.TextureMaskStyle;
import starling.filters.BlurFilter;
import starling.filters.DropShadowFilter;
import starling.filters.GlowFilter;
import starling.textures.Texture;

public class ChallengeTuningView extends FeathersControl implements ITuningListMenuVIew {
        private var _slidesList:List;
        private var _snapshots : Array;
        private var _bg:Quad;
        private var _params : Object;
        private var pageIndicatorContainer : Sprite;
        private var _challengeLikeBlueButton:ToggleButton;
        private var _challengeLikeRedButton:ToggleButton;

        private var _vsImage:Image;

        private var _vsTuningLoader:VsImageLoader;
        private var _vsTuningImage:Image;
        private var _user1Label:ChallengeViewUserLabel;
        private var _user2Avatar:UserCircularAvatar;

        private var _timerLabel:ChallengeTimerLabel;
        private var circleBorder:Canvas;
        private var maskCanvas:Canvas;
        private var maskImage:Image;
        private var isSwitched:Boolean = false;
        private var isScrolling:Boolean = false;
        private var _challengeData:ChallengeData;
        public function ChallengeTuningView(params:Object){
            super();
            if (params){
                _params = params.data;
                if(_params.hasOwnProperty("isSwitched")){
                    isSwitched = _params.isSwitched;
                }
                _snapshots = isSwitched?getTuningSnapshots(_params.receiverTuning):getTuningSnapshots(_params.senderTuning);
                _user1Label = new ChallengeViewUserLabel(isSwitched?_params.receiverTuning.userObj:_params.senderTuning.userObj);
                _user2Avatar = new UserCircularAvatar(new User(isSwitched?_params.senderTuning.userObj:_params.receiverTuning.userObj), 34, true, 1.5);
                if(_params.hasOwnProperty("challengeData")){
                    _challengeData = _params.challengeData;
                }
                _user1Label.addEventListener(TouchEvent.TOUCH, labelTouchHandler);
            }
        }

        public function set params(value:Object):void{
            isSwitched = false;
            _params = value;
            if(_params.hasOwnProperty("challengeData")){
                _challengeData = _params.challengeData;
                if(_challengeData.status == ChallengeData.FINISHED){
                    _timerLabel.finished = true;
                }else{
                    _timerLabel.setTime(_challengeData.start_date);
                }

            }
            if(_user1Label){
                _user1Label.removeFromParent(true);
            }
            if(_user2Avatar){
                _user2Avatar.removeFromParent(true);
            }
            if(_vsTuningLoader){
                _vsTuningLoader.removeFromParent(true);
            }
            if(_vsTuningImage){
                _vsTuningImage.mask = null;
                _vsTuningImage.removeFromParent();
                _vsTuningImage.texture.dispose();
                _vsTuningImage.dispose();
                _vsTuningImage = null;
            }
            if(circleBorder){
                circleBorder.removeFromParent();
            }
            if(maskCanvas){
                maskCanvas.removeFromParent();
            }
            if(pageIndicatorContainer){
                while (pageIndicatorContainer.numChildren){
                    pageIndicatorContainer.removeChildAt(0, true);
                }
            }
            _slidesList.dataProvider = new ListCollection([]);
            _slidesList.validate();

            _snapshots = getTuningSnapshots(_params.senderTuning);
            _user1Label = new ChallengeViewUserLabel(_params.senderTuning.userObj);
            _user2Avatar = new UserCircularAvatar(new User(_params.receiverTuning.userObj), 34);

            _vsTuningLoader = new VsImageLoader();
            var url:String = getTuningSnapshots(_params.receiverTuning)[0];
            if(url.indexOf("http:") != 0){
                url = "http:"+url;
            }

            _challengeLikeBlueButton.label = _challengeData.owner.votes;
            _challengeLikeRedButton.label = _challengeData.receiver.votes;

            _vsTuningLoader.source = url;
            _vsTuningLoader.addEventListener(Event.COMPLETE, vsImageCompleteHandler);
            addChild(_vsTuningLoader);

            addChild(_user1Label);
            addChild(_user2Avatar);
            _user2Avatar.visible = _user1Label.visible = false;

            if(!_user1Label.hasEventListener(TouchEvent.TOUCH)) {
                _user1Label.addEventListener(TouchEvent.TOUCH, labelTouchHandler);
            }
            setLikeButtonStates();
            invalidate(INVALIDATION_FLAG_ALL);
        }

        private function labelTouchHandler(event:TouchEvent):void {
            var touch:Touch = event.getTouch(_user1Label, TouchPhase.ENDED);
            if(touch){
                var currentUser:User = UsersProxy.loggedInUser();
                if(currentUser && currentUser.id == _user1Label.user.id){
                    dispatchEvent(new Event(StackedScreen.GARAGE_SELECTED, false));
                }else{
                    dispatchEvent(new Event(StackedScreen.USER_DETAILS_SELECTED, false, {user:_user1Label.user}));
                }
            }
        }

        override protected function initialize():void {
            super.initialize();
            _bg  = new Quad(1,1,0xffffff);
            addChild(_bg);
            _bg.alpha = 1;

            var layout : HorizontalLayout = new HorizontalLayout();
            layout.gap = 0;
            layout.horizontalAlign = HorizontalAlign.CENTER;
            layout.verticalAlign = VerticalAlign.MIDDLE;

            _slidesList = new List();
            _slidesList.minimumPageThrowVelocity = 1;
            _slidesList.dataProvider = new ListCollection();
            _slidesList.layout = layout;
            _slidesList.snapToPages = true;
            _slidesList.hasElasticEdges = false;
            _slidesList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
            _slidesList.horizontalScrollPolicy = ScrollPolicy.ON;
            _slidesList.addEventListener(Event.SCROLL, onCarIndexChanged);
            _slidesList.itemRendererFactory = itemRendererFactory;
            addChild(_slidesList);

            _vsTuningLoader = new VsImageLoader();
            var url:String = isSwitched?getTuningSnapshots(_params.senderTuning)[0]:getTuningSnapshots(_params.receiverTuning)[0];
            if(url.indexOf("http:") != 0){
                url = "http:"+url;
            }
            _vsTuningLoader.source = url;
            _vsTuningLoader.addEventListener(Event.COMPLETE, vsImageCompleteHandler);
            addChild(_vsTuningLoader);

            _challengeLikeBlueButton = ButtonFactory.challengeLiteButton('challenge-like-1', 'challenge-like-selected-1', "left");
            _challengeLikeBlueButton.setSize(60, 30);
            addChild(_challengeLikeBlueButton);
            _challengeLikeBlueButton.addEventListener(Event.TRIGGERED, blueTuningVoteHandler);

            _challengeLikeRedButton = ButtonFactory.challengeLiteButton('challenge-like-2', 'challenge-like-selected-2', "right");
            _challengeLikeRedButton.setSize(60, 30);
            _challengeLikeRedButton.addEventListener(Event.TRIGGERED, redTuningVoteHandler);

            _vsImage = new Image(ButtonFactory.assetManager.getTexture("vs_large"));
            _vsImage.scale = ButtonFactory.assetManager.scaleFactor/2;
            _timerLabel = new ChallengeTimerLabel();
            refreshStatus();
            if(!_timerLabel.fininshed){
                _timerLabel.setTime(_challengeData.start_date);
            }

            addChild(_vsImage);
            addChild(_challengeLikeBlueButton);
            addChild(_challengeLikeRedButton);
            addChild(_timerLabel);

            addChild(_user1Label);
            addChild(_user2Avatar);
            _user2Avatar.visible = _user1Label.visible = false;

            setLikeButtonLabels();
            setLikeButtonStates();
            trace('btn');
        }


    private static function getTuningSnapshots(tuning:Tuning):Array{
        return (tuning.fullHdSnapshots && tuning.fullHdSnapshots.length)?tuning.fullHdSnapshots:tuning.snapshots;
    }

    private function switchTunings():void{
        SoundManager.playSound(SoundManager.SELECT_PART);
        var url:String;
        isSwitched = !isSwitched;
        //
        var tempAvatar:UserCircularAvatar = _user1Label.avatar;
        _user1Label.avatar = _user2Avatar;
        _user2Avatar = tempAvatar;
        addChild(_user2Avatar);
        _user2Avatar.visible = _user1Label.visible = false;
        //
        if(pageIndicatorContainer){
            while (pageIndicatorContainer.numChildren){
                pageIndicatorContainer.removeChildAt(0, true);
            }
        }
        if(isSwitched){
            url = getTuningSnapshots(_params.senderTuning)[0];
            _snapshots = getTuningSnapshots(_params.receiverTuning);
        }else{
            url = getTuningSnapshots(_params.receiverTuning)[0];
            _snapshots = getTuningSnapshots(_params.senderTuning);
        }
        if(url.indexOf("http:") != 0){
            url = "http:"+url;
        }
        if(_vsTuningImage){
            _vsTuningImage.mask = null;
            _vsTuningImage.removeFromParent();
            _vsTuningImage.texture.dispose();
            _vsTuningImage.dispose();
            _vsTuningImage = null;
        }
        if(_vsTuningLoader){
            _vsTuningLoader.removeFromParent(true);
        }
        if(circleBorder){
            circleBorder.removeFromParent();
        }
        if(maskCanvas){
            maskCanvas.removeFromParent();
        }
        _vsTuningLoader = new VsImageLoader();
        _vsTuningLoader.addEventListener(Event.COMPLETE, vsImageCompleteHandler);
        _vsTuningLoader.source = url;
        addChild(_vsTuningLoader);
        _slidesList.dataProvider = new ListCollection([]);
        _slidesList.validate();

        setLikeButtonLabels();
        setLikeButtonStates();
        invalidate(INVALIDATION_FLAG_ALL);
    }

    private function setLikeButtonLabels():void{
        _challengeLikeBlueButton.label = isSwitched?_challengeData.receiver.votes:_challengeData.owner.votes;
        _challengeLikeRedButton.label = isSwitched?_challengeData.owner.votes:_challengeData.receiver.votes;
    }

    private function setLikeButtonStates():void{
        var blueLabel:String = isSwitched?'receiver':'owner';
        var redLabel:String = isSwitched?'owner':'receiver';
        _challengeLikeBlueButton.isSelected = !_challengeData.canIVote(blueLabel) || _timerLabel.isFinished;
        _challengeLikeBlueButton.touchable = _challengeLikeBlueButton.isEnabled = _challengeData.canIVote(blueLabel) || !_timerLabel.isFinished;
        _challengeLikeRedButton.isSelected = !_challengeData.canIVote(redLabel) || _timerLabel.isFinished;
        _challengeLikeRedButton.touchable = _challengeLikeRedButton.isEnabled = _challengeData.canIVote(redLabel) || !_timerLabel.isFinished;
    }

    private function vsImageCompleteHandler(event:Event):void {
        _vsTuningLoader.removeFromParent();
        _vsTuningImage = new Image(_vsTuningLoader.texture);
        if(!circleBorder) {
            circleBorder = new Canvas();
            circleBorder.beginFill(0xFFFFFF, 1);
            circleBorder.drawCircle(70, 70, 70);
            circleBorder.endFill();
            circleBorder.filter = new DropShadowFilter(0, 0);
        }
        _vsTuningImage.scale = 300/_vsTuningImage.width;
        if(!maskCanvas) {
            maskCanvas = new Canvas();
            maskCanvas.beginFill();
            maskCanvas.drawCircle(70, 70, 67);
            maskCanvas.endFill();
            maskCanvas.filter = new BlurFilter( .2, .2, 2);

            maskImage = new Image(Texture.fromBitmapData(maskCanvas.drawToBitmapData()));
            maskImage.style = new TextureMaskStyle(1);
            maskImage.width = maskCanvas.width;
            maskImage.height = maskCanvas.height;
            maskImage.filter = new BlurFilter( .5, .5, 2);
        }

        circleBorder.x = actualWidth - (circleBorder.width + 10);
        circleBorder.y =  actualHeight - (circleBorder.height + 10);
        _vsTuningImage.x = circleBorder.x + (circleBorder.width-_vsTuningImage.width)/2;
        _vsTuningImage.y = circleBorder.y + (circleBorder.height-_vsTuningImage.height)/2;

        addChildAt(circleBorder, getChildIndex(_vsImage));
        addChildAt(_vsTuningImage, getChildIndex(_vsImage));
        addChildAt(maskImage, getChildIndex(_vsImage));
        maskImage.x = circleBorder.x + 3;
        maskImage.y = circleBorder.y + 3;
        _vsTuningImage.mask = maskImage;
        _vsTuningImage.touchable = true;
        _vsTuningImage.addEventListener(TouchEvent.TOUCH, vsTouchHandler);



        _user2Avatar.visible = _user1Label.visible = true;
        circleBorder.filter = new BlurFilter( .2, .2, 2);

        invalidate(INVALIDATION_FLAG_ALL);
    }

    private function vsTouchHandler(event:TouchEvent):void {
        var touch:Touch = event.getTouch(_vsTuningImage, TouchPhase.ENDED);
        if(touch) {
            switchTunings();
        }
    }

        public function initPagingIcons() : void {
            var size:int = 15;
            var lineCount:int = int((actualWidth-160) / size);
            if (!pageIndicatorContainer) {
                pageIndicatorContainer = new Sprite();
                addChild(pageIndicatorContainer);
            }

            if (pageIndicatorContainer.numChildren > 0) {
                var item  : Sprite;
                for (var i : int = 0; i < pageIndicatorContainer.numChildren; i++) {
                    item = (pageIndicatorContainer.getChildAt(i) as Sprite);
                    item.getChildAt(1).visible = (_slidesList.horizontalPageIndex == i);
                    item.x = (i % lineCount) * size;
                    item.y = int(i / lineCount) * size;
                }
            } else {
                var stateContainer:Sprite;
                var state1Image : Image;
                var state2Image : Image;
                for (var ii:int = 0; ii < _slidesList.dataProvider.length; ii++) {
                    stateContainer = new Sprite();
                    state1Image = new Image(ButtonFactory.assetManager.getTexture("pageIndicator"));
                    state2Image = new Image(ButtonFactory.assetManager.getTexture("pageIndicatorSelected"));
                    state1Image.width = state1Image.height = 9;
                    state2Image.width = state2Image.height = 9;
                    stateContainer.addChild(state1Image);
                    stateContainer.addChild(state2Image);
                    if (ii != 0) stateContainer.getChildAt(1).visible = false;
                    stateContainer.x = (ii % lineCount) * size;
                    stateContainer.y = int(ii / lineCount) * size;
                    pageIndicatorContainer.addChild(stateContainer);
                }
            }
        }

        private function onCarIndexChanged(event:Event) : void {
            if(_slidesList.isScrolling && !isScrolling){
                SoundManager.playSound(SoundManager.SELECT_PART);
                isScrolling = true;
            }else if(!_slidesList.isScrolling){
                initPagingIcons();
                isScrolling = false;
            }
        }

        override protected function draw():void {
            super.draw();
            if(isInvalid(INVALIDATION_FLAG_SIZE)) {
                _bg.width =actualWidth;
                _bg.height=actualHeight;
                _slidesList.setSize(actualWidth, actualHeight);
                if (_slidesList.dataProvider.length == 0 && _snapshots.length > 0) {
                    _slidesList.dataProvider = new ListCollection(_snapshots);
                    initPagingIcons();
            }

                if (pageIndicatorContainer) {
                    pageIndicatorContainer.x = (actualWidth - pageIndicatorContainer.width) / 2;
                    pageIndicatorContainer.y = pageIndicatorContainer.height - 9;
                }
            }
            if(_challengeLikeBlueButton){
                _challengeLikeBlueButton.x = 9;
                _challengeLikeBlueButton.y = _challengeLikeRedButton.y = actualHeight - (_challengeLikeBlueButton.height + 9);
                _challengeLikeRedButton.x = actualWidth - (_challengeLikeRedButton.width + 9);
                if(pageIndicatorContainer){
                    pageIndicatorContainer.y = _challengeLikeRedButton.y + (_challengeLikeRedButton.height - pageIndicatorContainer.height)/2;
                }
            }
            if(_vsImage){
                _vsImage.x = 3*actualWidth/4 - _vsImage.width/2;
                _vsImage.y = actualHeight/2;
            }
            if(_user1Label){
                _user1Label.visible = true;
                _user1Label.x = _user1Label.y = 9;
            }
            if(_timerLabel){
                _timerLabel.x = (actualWidth-_timerLabel.width)/2;
                if(_user1Label){
                    _timerLabel.y = _user1Label.y + (_user1Label.height-_timerLabel.height)/2;
                }else{
                    _timerLabel.y = 30-_timerLabel.height/2;
                }
            }
            if(_user2Avatar && circleBorder){
                _user2Avatar.x = circleBorder.x + circleBorder.width/2+_user2Avatar.width/2;
                _user2Avatar.y = circleBorder.y - 5;
                _user2Avatar.visible = true;
            }
        }

        private function itemRendererFactory() : IListItemRenderer {
            var renderer :TimelineTuningViewItemRenderer = new TimelineTuningViewItemRenderer(null);
            var item_w:Number=actualWidth;
            var item_h:Number=actualHeight;
            renderer.setSize(item_w, item_h);
            return renderer;
        }

        public function getVisibleTuningURL():String {
            return _snapshots[_slidesList.horizontalPageIndex];
        }

        private function allowedChallengeVote(userType:String):Boolean{
            if(!_challengeData.canIVote(userType)){
                if(_challengeData.isOwnChallenge) {
                    ProfileAlertView.show("Unsupported action:", "You can't vote on your own challenge.", 0, null, "CLOSE");
                }else if(!_challengeData.getAlreadyVoted(userType)){
                    ProfileAlertView.show("Unsupported action:", "You have already voted for this tuning.", 0, null, "CLOSE");
                }

                _challengeLikeBlueButton.isSelected = true;
                _challengeLikeBlueButton.touchable = false;

                _challengeLikeRedButton.isSelected = true;
                _challengeLikeRedButton.touchable = false;
                return true;
            }
            return false;
        }

        private function blueTuningVoteHandler(event: Event): void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            var challengesProxy:ChallengesProxy = ApplicationFacade.getInstance().retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            var tuningId:String = isSwitched?_challengeData.receiver.tuning._id:_challengeData.owner.tuning._id;
            var userType:String = isSwitched?'receiver':'owner';

            if(allowedChallengeVote(userType)){
                return;
            }

            challengesProxy.voteChallenge(_challengeData._id, userType, tuningId, function (newData: ChallengeData): void {
                _challengeLikeBlueButton.isSelected = true;
                _challengeLikeBlueButton.touchable = false;
                if (newData) {
                    if(isSwitched){
                        _challengeData.receiver.votes = newData.receiver.votes;
                        _challengeLikeBlueButton.label = newData.receiver.votes;
                    }else{
                        _challengeData.owner.votes = newData.owner.votes;
                        _challengeLikeBlueButton.label = newData.owner.votes;
                    }
                    refreshStatus();
                }
            }, null, function():void{
                _challengeLikeBlueButton.isSelected = false;
                _challengeLikeBlueButton.touchable = true;
            })
        }

        private function redTuningVoteHandler(event: Event): void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            var challengesProxy:ChallengesProxy = ApplicationFacade.getInstance().retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            var tuningId:String = isSwitched?_challengeData.owner.tuning._id:_challengeData.receiver.tuning._id;
            var userType:String = isSwitched?'owner':'receiver';

            if(allowedChallengeVote(userType)){
                return;
            }

            challengesProxy.voteChallenge(_challengeData._id, userType, tuningId, function (newData: ChallengeData): void {
                _challengeLikeRedButton.isSelected = true;
                _challengeLikeRedButton.touchable = false;
                if (newData) {
                    if(isSwitched){
                        _challengeData.owner.votes = newData.owner.votes;
                        _challengeLikeRedButton.label = newData.owner.votes;
                    }else{
                        _challengeData.receiver.votes = newData.receiver.votes;
                        _challengeLikeRedButton.label = newData.receiver.votes;
                    }
                    refreshStatus();
                }
            }, null, function():void{
                _challengeLikeRedButton.isSelected = false;
                _challengeLikeRedButton.touchable = true;
            })
        }

        private function refreshStatus():void{
            _timerLabel.finished = (_challengeData.status == ChallengeData.FINISHED || (_challengeData.owner.votes>=ChallengeData.VOTE_MAX_COUNT || _challengeData.receiver.votes>=ChallengeData.VOTE_MAX_COUNT));
            setLikeButtonStates();
        }

    public function get challengeData():ChallengeData {
        return _challengeData;
    }

    public function get challengeType():String{
        return isSwitched?"receiver":"owner";
    }

    override public function dispose():void {
        super.dispose();
        if(_slidesList){
            _slidesList.dataProvider = null;
            _slidesList.removeFromParent(true);
            _slidesList = null;
        }
        if(_bg){
            _bg.removeFromParent(true);
            _bg = null;
        }
        if(pageIndicatorContainer){
            DisplayObjectUtil.dispose(pageIndicatorContainer);
            pageIndicatorContainer.removeFromParent(true);
            pageIndicatorContainer = null;
        }
        if(_challengeLikeBlueButton){
            _challengeLikeBlueButton.removeFromParent(true);
            _challengeLikeBlueButton = null;
        }
        if(_challengeLikeRedButton){
            _challengeLikeRedButton.removeFromParent(true);
            _challengeLikeRedButton = null;
        }
        if(_vsImage){
            _vsImage.removeFromParent(true);
            _vsImage = null;
        }
        if(_vsTuningLoader){
            _vsTuningLoader.removeFromParent(true);
            _vsTuningLoader = null;
        }
        if(_vsTuningImage){
            _vsTuningImage.removeFromParent(true);
            _vsTuningImage = null;
        }
        if(_user1Label){
            _user1Label.removeFromParent(true);
            _user1Label = null;
        }

        if(_user2Avatar){
            _user2Avatar.removeFromParent(true);
            _user2Avatar = null;
        }
        if(_timerLabel){
            _timerLabel.removeFromParent(true);
            _timerLabel = null;
        }
        if(circleBorder){
            circleBorder.removeFromParent(true);
            circleBorder = null;
        }
        if(maskCanvas){
            maskCanvas.removeFromParent(true);
            maskCanvas = null;
        }
    }
}
}

import feathers.controls.ImageLoader;

import starling.textures.Texture;

class VsImageLoader extends ImageLoader{
    public function VsImageLoader(){
        super ();
    }

    public function get texture():Texture{
        return _texture;
    }
}