package com.despark.tuning3d.view.components.challenges.controls {
import com.despark.tuning3d.view.components.challenges.modals.StartChallengePopup;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.utils.TextureUtil;

import feathers.controls.TextInput;
import feathers.controls.text.StageTextTextEditor;
import feathers.core.FeathersControl;
import feathers.core.ITextEditor;
import feathers.data.ListCollection;
import feathers.layout.RelativePosition;
import feathers.layout.VerticalAlign;

import flash.geom.Rectangle;

import starling.core.Starling;
import starling.display.Image;
import starling.events.Event;

public class SearchBox extends FeathersControl{
    private var _searchResult:SearchResultList;
    private var _searchResultCollection:ListCollection;
    private var _defaultCollection:ListCollection;
    private var _searchField:TextInput;
    private var _searchProvider:Function;
    private var _customSearchProvider:Function;
    public var isWaiting:Boolean = false;
    public function SearchBox() {
        _searchResult = new SearchResultList();
        _searchField = new TextInput();
        searchProvider = null;
    }

    public function get length():int{
        _defaultCollection ||= _searchResult.dataProvider as ListCollection;
        if(_defaultCollection){
            return _defaultCollection.length;
        }
        return 0;
    }

    override protected function initialize():void {
        super.initialize();
        addChild(_searchResult);
        _searchField.textEditorFactory = function ():ITextEditor {
            var editor:StageTextTextEditor = new StageTextTextEditor();
            editor.color = 0x383838;
            editor.fontSize = 10;
            editor.fontFamily = 'Arial';
            return editor;
        };
        addChild(_searchField);
        var fieldBg:Image = new Image(ButtonFactory.assetManager.getTexture("decline_btn_bg"));
        var size:Number = StartChallengePopup.INPUT_SIZE;
        TextureUtil.scaleImage(fieldBg, 2*(size/fieldBg.texture.height)/ButtonFactory.assetManager.scaleFactor);
        var rectSize:Number = fieldBg.texture.height/2-0.5;
        fieldBg.scale9Grid = new Rectangle(rectSize, rectSize, fieldBg.texture.width - 2*rectSize, 1);
        _searchField.height = fieldBg.height;
        _searchField.backgroundSkin = fieldBg;
        _searchField.paddingRight = 10;
        _searchField.defaultIcon = new Image(ButtonFactory.assetManager.getTexture("searchIcon"));
        _searchField.iconPosition = RelativePosition.LEFT;
        _searchField.paddingLeft = 5;
        _searchField.paddingTop = 2;
        _searchField.gap = 5;
        _searchField.verticalAlign = VerticalAlign.MIDDLE;
        _searchField.addEventListener(Event.CHANGE, searchFieldChangeHandler);
    }

    private function searchFieldChangeHandler(event:Event):void {
        Starling.juggler.removeDelayedCalls(commitSearch);
        Starling.juggler.delayCall(commitSearch, 1);
    }

    private function commitSearch():void{
        _searchResult.loadingItems = 0;
        if(_searchField.text.length){
            if(_customSearchProvider!=null){
                _defaultCollection ||= _searchResult.dataProvider as ListCollection;
                _customSearchProvider.apply(null, [_searchField.text]);
                return;
            }
            if(_searchProvider!=null) {
                _defaultCollection ||= _searchResult.dataProvider as ListCollection;
                _searchResultCollection = new ListCollection(_searchProvider.apply(null, [_searchField.text, _defaultCollection]));
                _searchResult.dataProvider = _searchResultCollection;
            }
        }else if(_defaultCollection){
            _searchResult.dataProvider = _defaultCollection;
        }
    }

    override protected function draw():void {
        super.draw();
        _searchResult.x = 0;
        _searchResult.y = _searchField.height;
        _searchField.width = actualWidth - 54;
        _searchField.x = (actualWidth-_searchField.width)/2;
        _searchResult.width = actualWidth;
        _searchResult.height = actualHeight - _searchField.height;
    }

    public function get searchResult():SearchResultList {
        return _searchResult;
    }

    public function set searchProvider(value:Function):void {
        if(value == null){
            _searchProvider = function (str:String = null, collection:ListCollection = null):Array {
                return null;
            };
            return;
        }
        _searchProvider = value;
    }

    public function set customSearchProvider(value:Function):void {
        _customSearchProvider = value;
    }

    public function get searchField():TextInput {
        return _searchField;
    }
}
}
