package com.despark.tuning3d.view.components.challenges.modals {
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.model.cars.CarBackground;
import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.Multilingual;
import com.despark.tuning3d.model.packages.PackagesParser;
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.tunings.TuningsProxy;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.challenges.controls.ChallengeViewUserLabel;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.IMainMenuHolder;
import com.despark.tuning3d.view.components.core.MainMenuView;
import com.despark.tuning3d.view.components.core.StackedScreen;
import com.despark.tuning3d.view.components.tunings.ITuningListMenuVIew;
import com.despark.tuning3d.view.components.tunings.TuningItemRenderer;
import com.despark.tuning3d.view.components.tunings.TuningsListMenuView;
import com.despark.tuning3d.view.components.tunings.TuningsListScreen;
import com.greensock.TweenLite;

import feathers.controls.Button;
import feathers.controls.ToggleButton;
import feathers.layout.HorizontalAlign;
import feathers.layout.HorizontalLayout;
import feathers.layout.VerticalAlign;

import starling.core.Starling;
import starling.display.Image;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;

public class ChallengeLeaderBoardTuningScreen extends StackedScreen implements Multilingual, IMainMenuHolder, ITuningListMenuVIew {
    public static const LOAD_BACKGROUNDS:String = 'loadBackgrounds';

    private var _mainMenuView : MainMenuView;
    private var _languages:Array;
    private var _selectedLanguage:Language;
    private var _nextButton:Button;
    private var _prevButton:Button;
    private var _selectedTuning:Tuning;
    private var _snapshots : Array;
    private var tuningListData:Array;
    private var tuningView:TuningItemRenderer;


    private var tuningsListMenuView:TuningsListMenuView;
    private var _challengesList:Object;
    private const _gap:Number = 6;
    private var _selectedUser:User;
    private var _currentUser:User;
    private var _userLabel:ChallengeViewUserLabel;
    private var closeButton:Button;
    private var _container:LeaderBoardContainer;
    private var _backgroundsMap:Object;
    private var _showBanner:ToggleButton;
    private var _menuTween:TweenLite;
    public function ChallengeLeaderBoardTuningScreen() {
        super("ChallengeLeaderBoardTuningScreen");
        var usersProxy:UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
        _container = new LeaderBoardContainer();
        _currentUser = usersProxy.currentUser;
        _snapshots = [];
        addChild(_container);
    }

    override protected function initialize():void {
        super.initialize();
        _container.setSize(actualWidth, actualHeight);

        var layout : HorizontalLayout = new HorizontalLayout();
        layout.gap = 0;
        layout.horizontalAlign = HorizontalAlign.CENTER;
        layout.verticalAlign = VerticalAlign.MIDDLE;

        tuningsListMenuView = new TuningsListMenuView(this);
        tuningsListMenuView.addEventListener(TuningsListMenuView.CAR_SELECTED, onCarSelected);
        tuningsListMenuView.addEventListener(TuningsListMenuView.TUNING_SELECTED, onTuningSelected);
        tuningsListMenuView.addEventListener(TuningsListMenuView.TUNING_SHARE, onTuningShare);
        tuningsListMenuView.addEventListener(TuningsListMenuView.TUNING_DELETE, onTuningDelete);
        tuningsListMenuView.addEventListener(TuningsListMenuView.START_CHALLENGE, challengeHandler);
        _container.addChild(tuningsListMenuView);

        _prevButton = ButtonFactory.arrowButton("slide_arr_lt");
        _prevButton.width=45/2;
        _prevButton.height=70/2;
        _prevButton.addEventListener(Event.TRIGGERED, onPrevButtonHandler);
        _prevButton.alpha = 1;
        Image(_prevButton.defaultSkin).color = 0xFFFFFF;
        _container.addChild(_prevButton);

        _nextButton = ButtonFactory.arrowButton("slide_arr_lt");
        _nextButton.rotation = Math.PI;
        _nextButton.width=45/2;
        _nextButton.height=70/2;
        _nextButton.addEventListener(Event.TRIGGERED, onNextButtonHandler);
        _nextButton.alpha = 1;
        Image(_nextButton.defaultSkin).color = 0xFFFFFF;
        _container.addChild(_nextButton);

        closeButton = ButtonFactory.screenCloseButton();
        closeButton.addEventListener(Event.TRIGGERED, closeButtonHandler);
        _container.addChild(closeButton);

        _showBanner = ButtonFactory.createTuningMenuHideButton();
        _showBanner.isSelected = true;
        _showBanner.addEventListener(Event.CHANGE, showBannerChangeHandler);
        _container.addChild(_showBanner);
    }

    private function showBannerChangeHandler(event:Event):void {
        if(_menuTween){
            _menuTween.kill();
        }
        if(_showBanner.isSelected){
            _menuTween = new TweenLite(tuningsListMenuView, 0.3, {y: actualHeight - tuningsListMenuView.height, onUpdate:menuAnimationUpdate});
        }else{
            _menuTween = new TweenLite(tuningsListMenuView, 0.3, {y: actualHeight, onUpdate:menuAnimationUpdate});
        }
    }

    private function menuAnimationUpdate():void{
        _showBanner.x = (actualWidth - _showBanner.width) / 2;
        _showBanner.y = tuningsListMenuView.y - (_showBanner.height - 10);
        if(tuningView){
            tuningView.y = -(actualHeight - tuningsListMenuView.y) / 2;
        }
        tuningsListMenuView.updateLabelPositions();
        updatePageIndicatorPosition();
    }

    private function updatePageIndicatorPosition():void{
        if(tuningView) {
            tuningView.pageIndicator.y = _showBanner.y - (tuningView.pageIndicator.height + 5) - tuningView.y;
        }
    }


    private function closeButtonHandler(event:Event):void {
        dispatchEvent(new Event(StackedScreen.BACK_BUTTON_SELECTED));
    }

    public function updateNotifications() :void{
        /*if(menuButton){
            menuButton.updateNotification();
        }*/
        if(mainMenuView){
            mainMenuView.updateNotificationCount();
        }
    }

    private function challengeHandler(event:Event):void{
        var tuning:Tuning = (selectedTuning && !Constants.selectedUser)?selectedTuning:null;
        var data:Object  = ChallengeEvent.getData(_currentUser, tuning, Constants.selectedUser, null);
        dispatchEvent(new Event(ChallengeEvent.CHALLENGE_REQUEST, false, data));
    }

    protected function onPrevButtonHandler(event:Event):void {
        var currentIndex:int = tuningListData.indexOf(_selectedTuning);
        if(currentIndex>0){
            selectTuning(tuningListData[currentIndex - 1]);
        }
    }

    protected function onNextButtonHandler(event:Event):void {
        var currentIndex:int = tuningListData.indexOf(_selectedTuning);
        if(currentIndex<tuningListData.length-1){
            selectTuning(tuningListData[currentIndex + 1]);
        }
    }

    protected function onCarSelected(event:Event):void {
        dispatchEventWith(TuningsListMenuView.CAR_SELECTED);
    }

    protected function onTuningSelected(event:Event):void {
        dispatchEventWith(TUNING_SELECTED);
        getSocialData();
    }

    protected function onTuningShare(event:Event):void{
        dispatchEventWith(TuningsListMenuView.TUNING_SHARE);
    }

    protected function onTuningDelete(event:Event):void{
        dispatchEventWith(TuningsListMenuView.TUNING_DELETE, false, event.data);
    }

    public function set likes(value:int) : void {
        tuningsListMenuView.likes = value;
    }

    override protected function draw():void {
        super.draw();
        if (closeButton){
            closeButton.x = actualWidth - (closeButton.width + 6);
            closeButton.y = 6;
        }

        if(tuningsListMenuView) {
            tuningsListMenuView.width = actualWidth;
            tuningsListMenuView.height = 50;
            if (_showBanner) {
                if (_showBanner.isSelected){
                    tuningsListMenuView.y = actualHeight - tuningsListMenuView.height;
                } else {
                    tuningsListMenuView.y = actualHeight;
                }
            }
            tuningsListMenuView.parentHeight = actualHeight;

            tuningsListMenuView.setCarName();
        }
        if(_nextButton){
            _nextButton.x = actualWidth - _nextButton.width;
            _nextButton.y = (actualHeight - _nextButton.height)/2;
        }

        if(_prevButton){
            _prevButton.x = _prevButton.width;
            _prevButton.y = (actualHeight - _prevButton.height)/2;
        }
        updateNavigationButtons();

        if(_userLabel){
            _userLabel.x = _gap;
            _userLabel.y = _gap;
        }

        if(tuningsListMenuView) {
            Starling.juggler.delayCall(function ():void {
                tuningsListMenuView.setCarName();
                updatePageIndicatorPosition();
            }, .01);
            Starling.juggler.delayCall(function ():void {
                tuningsListMenuView.setCarName();
            }, .03);
        }
        menuAnimationUpdate();
    }

    private function updateNavigationButtons():void{
        var currentIndex:int = -1;
        if(tuningListData && tuningListData.length && _selectedTuning) {
            currentIndex = tuningListData.indexOf(_selectedTuning);
        }
        if(_nextButton){
            if(tuningListData && currentIndex >= tuningListData.length-1){
                _nextButton.alpha = 0.4;
                _nextButton.isEnabled = false;
                Image(_nextButton.defaultSkin).color = 0x000000;
            }else{
                _nextButton.alpha = 1;
                _nextButton.isEnabled = true;
                Image(_nextButton.defaultSkin).color = 0xFF0000;
            }
        }

        if(_prevButton){
            if(currentIndex <= 0){
                _prevButton.alpha = 0.4;
                _prevButton.isEnabled = false;
                Image(_prevButton.defaultSkin).color = 0x000000;
            }else{
                _prevButton.alpha = 1;
                _prevButton.isEnabled = true;
                Image(_prevButton.defaultSkin).color = 0xFF0000;
            }
        }
    }

    public function loadTuningList(tunings:Array, user:User, num:int):void{
        _selectedUser = user;
        _userLabel ||= new ChallengeViewUserLabel();
        if(!_userLabel.hasEventListener(TouchEvent.TOUCH)){
            _userLabel.addEventListener(TouchEvent.TOUCH, labelTouchHandler);
        }
        _userLabel.num = num.toString()+". ";
        _userLabel.user = _selectedUser;
        if((_currentUser && _selectedUser && _selectedUser.id != _currentUser.id) || !_currentUser){
            Constants.selectedUser = _selectedUser;
        }else{
            Constants.selectedUser = null;
        }
        tuningListData = tunings.concat();
        var allCarsList:Object = PackagesParser.allCarsList;
        if(!allCarsList){
            return;
        }
        for(var i:int = tuningListData.length - 1; i >= 0; i--){
            if(tuningListData[i]) {
                tuningListData[i] = TuningsProxy.parseTuning(tuningListData[i], true);
                if (tuningListData[i].car && allCarsList[tuningListData[i].car.id]) {
                    tuningListData[i].car = allCarsList[tuningListData[i].car.id];
                }
            }else{
                tuningListData.removeAt(i);
            }
        }
        if(_backgroundsMap) {
            selectTuning(tuningListData[0]);
        }else{
            _selectedTuning = tuningListData[0];
            if(hasEventListener(LOAD_BACKGROUNDS)) {
                dispatchEvent(new Event(LOAD_BACKGROUNDS, false, {user: user}));
            }else{
                Starling.juggler.delayCall(loadTuningList, .03, tunings, user, num);
            }
        }
    }

    private function labelTouchHandler(event:TouchEvent):void {
        var touch:Touch = event.getTouch(_userLabel, TouchPhase.ENDED);
        if(touch){
            dispatchEvent(new Event(StackedScreen.USER_DETAILS_SELECTED));
        }
    }

    private function selectTuning(tuning:Tuning):void{
        _selectedTuning = tuning;
        _selectedTuning.backgroundsMap = _backgroundsMap;
        if(isInitialized) {
            _snapshots = getTuningSnapshots(_selectedTuning);

            tuningsListMenuView.currentTuning = _selectedTuning;
            tuningsListMenuView.currentTuningID = _selectedTuning.id;
            if (_selectedTuning.car) {
                tuningsListMenuView.currentCarID = _selectedTuning.car.id;
            }
            if(_userLabel){
                _container.addChild(_userLabel);
            }
            tuningView ||= new TuningItemRenderer();
            tuningView.data = tuning;
            tuningView.setSize(actualWidth, actualHeight + 50);
            if(!_container.contains(tuningView)) {
                _container.addChildAt(tuningView, 0);
            }

            if(tuningsListMenuView){
                tuningsListMenuView.setCarName();
            }
            updatePageIndicatorPosition();
            invalidate(INVALIDATION_FLAG_SIZE);
        }else{
            Starling.juggler.delayCall(function ():void {
                selectTuning(tuning);
            }, 0.1);
        }
    }

    public function handleCarBackgroundsLoaded(carBackgrounds : Array) : void {
        var background:CarBackground;
        _backgroundsMap = {};
        for(var i:int=0; i < carBackgrounds.length; i++){
            background = carBackgrounds[i];
            _backgroundsMap[background.id]=background;
        }
        selectTuning(_selectedTuning);
    }

    private static function getTuningSnapshots(tuning:Tuning):Array{
        return (tuning.fullHdSnapshots && tuning.fullHdSnapshots.length)?tuning.fullHdSnapshots:tuning.snapshots;
    }

    private function getSocialData() : void {
        if (!Constants.selectedUser) {
            return;
        }
        dispatchEventWith(TuningsListScreen.GET_SOCIAL_DATA);
    }

    public function set comments(value:Array) : void {
        tuningsListMenuView.initComments = value;
    }

    public function getVisibleTuningURL():String {
        return _snapshots[tuningView.selectedIndex];
    }

    public function set languages(value:Array):void{
        _languages=value;
    }

    public function get languages():Array{
        return _languages;
    }

    public function brandsScreenButtonTriggeredHandler(event:Event = null):void {
        dispatchEventWith(StackedScreen.BRANDS_SELECTED);
    }

    public function homeButtonTriggeredHandler(event:Event = null):void {
        dispatchEventWith(StackedScreen.HOME_SELECTED);
    }

    public function enterRowsState():void {

    }

    public function set mainMenuView(value : MainMenuView) : void {
        _mainMenuView = value;
        _mainMenuView.owner = this;
    }

    public function get mainMenuView():MainMenuView {
        return _mainMenuView;
    }

    public function set selectedLanguage(value:Language):void {
        _selectedLanguage = value;
    }

    public function get selectedLanguage():Language {
        return _selectedLanguage;
    }

    public function dispatchExternalEvent(msg:String):void {
        if (mainMenuView) {
            mainMenuView.hide();
        }
        dispatchEventWith(msg);
    }

    public function updateWithLanguage(language:Language):void {
        if (mainMenuView) {
            mainMenuView.updateWithLanguage(language);
        }
    }

    public function get selectedTuning():Tuning {
        return _selectedTuning;
    }

    private function addElement(item:*,index:int=-1) : void {
        for (var i:int = 0; i < numChildren; i++) {
            if (getChildAt(i).name == item.name) return;
        }
        index == -1 ? _container.addChild(item) : _container.addChildAt(item,index);
    }

    public function appendChallenges(challenges:Array):void{
        _challengesList = {};
        for(var i:int = 0;  i < challenges.length; i++){
            _challengesList[challenges[i]] = true;
        }
    }


    override public function dispose():void {
        super.dispose();
        if(_mainMenuView){
            _mainMenuView.removeFromParent(true);
            _mainMenuView = null;
        }

        if(_nextButton){
            _nextButton.removeFromParent(true);
            _nextButton = null;
        }

        if(_prevButton){
            _prevButton.removeFromParent(true);
            _prevButton = null;
        }

        if(tuningsListMenuView){
            tuningsListMenuView.removeFromParent(true);
            tuningsListMenuView = null;
        }

        if(_userLabel){
            _userLabel.removeFromParent(true);
            _userLabel = null;
        }

        if(_container){
            _container.removeFromParent(true);
            _container = null;
        }
        _languages = null;
        _selectedLanguage = null;
        _selectedTuning = null;
        _snapshots = null;
        tuningListData = null;
        _challengesList = null;
        _selectedUser = null;
        _currentUser = null;
    }
}
}

import feathers.core.FeathersControl;

class LeaderBoardContainer extends FeathersControl{
    public function LeaderBoardContainer(){
        super ();
    }
}