package com.despark.tuning3d.view.components.challenges.modals {
    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.model.cars.CarBackgroundsProxy;
    import com.despark.tuning3d.model.challenges.ChallengeData;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.sharing.SharingInstagramProxy;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.tunings.TuningsProxy;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeTuningView;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.ExtentedButton;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.timeline.components.TimelineLoadBar;
    import com.despark.tuning3d.view.components.tunings.ShareView;
    import com.despark.tuning3d.view.components.tunings.editing.TuningScreen;
    import com.utils.RegExpUtil;

    import feathers.controls.Button;

    import flash.display.Loader;
    import flash.events.Event;


    import flash.net.URLRequest;

    import starling.display.Image;
    import starling.events.Event;

    public class ChallengeTuningViewPopup extends PopUp {
        private var _tuningView:ChallengeTuningView;
        private var _onClose:Function;
        private var _data:Object;
        private var _nextButton:Button;
        private var _prevButton:Button;
        private var _shareButton:Button;
        private var _shareView:ShareView;
        private static const MARGIN:int = 9;

        private var _loaderBar:TimelineLoadBar;

        public function ChallengeTuningViewPopup(data:Object = null) {
            super();
            _data = data;
            _tuningView = new ChallengeTuningView(data);
            if (data.closeHandler) {
                _onClose = data.closeHandler;
            } else {
                _onClose = function (...args):void {

                };
            }
        }

        override protected function initialize():void {
            super.initialize();
            setSizeInternal(stage.stageWidth, stage.stageHeight, false);
            var closeIcon:Image = new Image(ButtonFactory.assetManager.getTexture('garage_close'));
            closeIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
            closeButton.width = closeIcon.width;
            closeButton.height = closeIcon.height;
            closeButton.defaultIcon = closeIcon;

            background.color = 0xFFFFFF;
            background.alpha = 1;
            contentBackgroundImage.visible = false;

            _nextButton = ButtonFactory.arrowButton('slide_arr_lt');
            _nextButton.rotation = Math.PI;
            _prevButton = ButtonFactory.arrowButton('slide_arr_lt');
            _prevButton.addEventListener(starling.events.Event.TRIGGERED, prevButtonHandler);
            _nextButton.addEventListener(starling.events.Event.TRIGGERED, nextButtonHandler);
            _prevButton.alpha = 1;
            _nextButton.alpha = 1;

            restoreColor();

            _shareButton = ButtonFactory.circleButton('a_arrow_right_button');
            ExtentedButton(_shareButton).color = 0x000000;
            _shareButton.addEventListener(starling.events.Event.TRIGGERED, shareButtonTriggeredHandler);

            _shareButton.width = 35;
            _shareButton.height = 35;

            _shareView = new ShareView();
            _shareView.color = 0x000000;
            _shareView.setSize(35, (35 * 3) + (7.5 * 2));
            _shareView.addEventListener(ShareView.SHARE_SELECTED, onShareSelected);

            addChild(_tuningView);
            addChild(closeButton);

            addChild(_shareButton);

            addChild(_prevButton);
            addChild(_nextButton);


            _loaderBar ||= new TimelineLoadBar();
            _loaderBar.name = '_timelineLoadBar';
            _loaderBar.setSize(60, 60);
            _loaderBar.x = Constants.STAGE_WIDTH / 2 - _loaderBar.width / 2;
            _loaderBar.y = Constants.STAGE_HEIGHT / 2 - _loaderBar.height / 2;
            _loaderBar.visible = false;
            addChild(_loaderBar);

            //
            var carBackgroundsProxy : CarBackgroundsProxy = ApplicationFacade.getInstance().retrieveProxy(CarBackgroundsProxy.NAME) as CarBackgroundsProxy;
            if(!RegExpUtil.isUrl(_data.data.senderTuning.backgroundId)){
                carBackgroundsProxy.getUrlArrayById(_data.data.senderTuning, function(arr:Array):void{
                    _data.data.senderTuning.customSnapshotsBackgrounds = arr;
                    if(_data.data.senderTuning.customSnapshotsBackgrounds.length){
                        _data.data.senderTuning.backgroundId = _data.data.senderTuning.customSnapshotsBackgrounds[0];
                    }
                });
            }

            if(!RegExpUtil.isUrl(_data.data.receiverTuning.backgroundId)) {
                carBackgroundsProxy.getUrlArrayById(_data.data.receiverTuning, function (arr:Array):void {
                    _data.data.receiverTuning.customSnapshotsBackgrounds = arr;
                    if (_data.data.receiverTuning.customSnapshotsBackgrounds.length) {
                        _data.data.receiverTuning.backgroundId = _data.data.receiverTuning.customSnapshotsBackgrounds[0];
                    }
                });
            }
        }

        private function shareButtonTriggeredHandler(event:starling.events.Event):void {
            if (_shareView.parent) {
                SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
                _shareView.removeFromParent();
                return;
            }
            SoundManager.playSound(SoundManager.PART_MENU);
            addChild(_shareView);
            _shareView.x = _shareButton.x - (_shareView.width - _shareButton.width);
            _shareView.y = _shareButton.y + _shareButton.height + 7.5;
        }

        override public function dispose():void {
            super.dispose();
            if (_tuningView) {
                _tuningView.removeFromParent(true);
                _tuningView = null;
            }
            if (_prevButton) {
                _prevButton.removeFromParent(true);
                _prevButton = null;
            }
            if (_nextButton) {
                _nextButton.removeFromParent(true);
                _nextButton = null;
            }
            if (_shareView) {
                _shareView.removeFromParent(true);
                _shareView = null;
            }
            if (_shareButton) {
                _shareButton.removeFromParent(true);
                _shareButton = null;
            }
            if (_loaderBar) {
                _loaderBar.removeFromParent(true);
                _loaderBar = null;
            }
        }

        public function loadChallenge(challengeData:ChallengeData):void {
            if(!challengeData){
                _nextButton.visible = _prevButton.visible = false;
                return;
            }
            var params:Object = {};
            params.senderTuning = TuningsProxy.parseTuning(challengeData.owner.tuning, false);
            params.senderTuning.userObj = challengeData.owner.user;
            params.receiverTuning = TuningsProxy.parseTuning(challengeData.receiver.tuning, false);
            params.receiverTuning.userObj = challengeData.receiver.user;
            params.challengeData = challengeData;
            _tuningView.params = params;
        }

        private function nextButtonHandler(event:starling.events.Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchEvent(new starling.events.Event(ChallengeEvent.VIEW_NEXT_CHALLENGE_TUNING));
        }

        private function prevButtonHandler(event:starling.events.Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART_BACK);
            dispatchEvent(new starling.events.Event(ChallengeEvent.VIEW_NEXT_CHALLENGE_TUNING));
        }

        private function restoreColor():void {
            Image(_nextButton.defaultSkin).color = 0xFF0000;
            Image(_prevButton.defaultSkin).color = 0xFF0000;
        }

        override protected function draw():void {
            super.draw();
            if (_tuningView) {
                _tuningView.width = actualWidth;
                _tuningView.height = actualHeight;
            }
            closeButton.x = actualWidth - (closeButton.width + MARGIN);
            closeButton.y = MARGIN;

            if (_nextButton) {
                _nextButton.x = actualWidth - 2 * MARGIN;
                _nextButton.y = (actualHeight - _nextButton.height) / 2;
            }
            if (_prevButton) {
                _prevButton.x = _prevButton.width + 2 * MARGIN;
                _prevButton.y = (actualHeight - _prevButton.height) / 2;
            }
            if (_shareButton) {
                _shareButton.x = actualWidth - (_shareButton.width + closeButton.width + 2 * MARGIN);
                _shareButton.y = 9;
            }
        }

        private function onShareSelected(event:starling.events.Event):void {
            if (!_tuningView.challengeData) {
                return;
            }
            var sharingObject:Object = {url: ChallengesProxy.getChallengeShareUrl(_tuningView.challengeData._id, _tuningView.challengeType)};

            if (_shareView.selectedSocialId == 'fb') {
                dispatchEvent(new starling.events.Event(TuningScreen.SHARE_FB_SELECTED, false, sharingObject));
            } else if (_shareView.selectedSocialId == 'vk') {
                dispatchEvent(new starling.events.Event(TuningScreen.SHARE_VK_SELECTED, false, sharingObject));
            } else if (_shareView.selectedSocialId == 'instagram') {
                var sharingInstagramProxy:SharingInstagramProxy = ApplicationFacade.getInstance().retrieveProxy(SharingInstagramProxy.NAME) as SharingInstagramProxy;
                sharingInstagramProxy.prepare();
            }
            _shareView.removeFromParent();
        }

        public function showLoader():void {
            _loaderBar.visible = true;
        }

        public function hideLoader():void {
            _loaderBar.visible = false;
        }

        override protected function closeButtonTriggeredHandler(event:starling.events.Event):void {
            super.closeButtonTriggeredHandler(event);
            SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
            if (_onClose != null) {
                _onClose.call();
            }
        }

        public function get tuningView():ChallengeTuningView {
            return _tuningView;
        }
    }
}
