package com.despark.tuning3d.view.components.challenges.modals {
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.model.challenges.ChallengeData;
import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.tunings.TuningsProxy;
import com.despark.tuning3d.view.components.challenges.controls.ChallengeTuningAvatar;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.PopUp;
import com.distriqt.AdsManager;
import com.greensock.TweenLite;
import com.utils.FontSelector;

import feathers.controls.Button;
import feathers.controls.Label;
import feathers.controls.ToggleButton;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.ITextRenderer;

    import flash.system.Capabilities;

    import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.core.Starling;
import starling.display.DisplayObjectContainer;

import starling.display.Image;
import starling.display.Stage;

import starling.events.Event;
import starling.events.EventDispatcher;

public class ChallengeVotePopup extends PopUp {
    private var _messageLabel:Label;
    private var _nextButton:Button;
    private var _skipButton:Button;
    private var _challengeData:ChallengeData;
    private var _ownerTuning:Tuning;
    private var _receiverTuning:Tuning;
    private var _ownerTuningAvatar:ChallengeTuningAvatar;
    private var _receiverTuningAvatar:ChallengeTuningAvatar;

    private var _challengeLikeBlueButton:ToggleButton;
    private var _challengeLikeRedButton:ToggleButton;
    private var _vsImage:Image;
    private static const GAP:int = 15;
    //
    private static var showDelay:Number = 20;
    private static var retryDelay:Number = 30;
    private static var showBanner:Boolean = false;
    public function ChallengeVotePopup() {
        super();
        var openAds:Boolean = AdsManager.$.bannerHide();
        showBanner ||= openAds;
    }

    public static function requestShowVotePopup():void{
        TweenLite.killDelayedCallsTo(dispatchOpeningEvent);
        TweenLite.delayedCall(showDelay, dispatchOpeningEvent);
    }

    public static function retryShowVotePopup():void{
        TweenLite.killDelayedCallsTo(dispatchOpeningEvent);
        TweenLite.delayedCall(retryDelay, dispatchOpeningEvent);
    }

    private static function dispatchOpeningEvent():void{
        var stage:Stage = Starling.current.stage;
        var eventDispatcher:EventDispatcher = getEventDispatcher(stage, ChallengeEvent.DISPLAY_CHALLENGE_VOTE_POPUP);
        if(eventDispatcher) {
            var challengesProxy:ChallengesProxy = ApplicationFacade.getInstance().retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
            challengesProxy.getRandomChallenge(function (challengeData:ChallengeData):void {
                if (challengeData) {
                    var openAds:Boolean = AdsManager.$.bannerHide();
                    showBanner ||= openAds;
                    eventDispatcher.dispatchEvent(new Event(ChallengeEvent.DISPLAY_CHALLENGE_VOTE_POPUP, false, {challengeData:challengeData}));
                } else {
                    retryShowVotePopup();
                }
            });
        }
    }

    private static function getEventDispatcher(object:DisplayObjectContainer, eventType:String):EventDispatcher{
        for(var i:int = 0; i < object.numChildren; i++){
            if(object.getChildAt(i) is EventDispatcher && EventDispatcher(object.getChildAt(i)).hasEventListener(eventType)){
                return EventDispatcher(object.getChildAt(i));
            }
            if(object.getChildAt(i) is DisplayObjectContainer){
                var result:EventDispatcher = getEventDispatcher(object.getChildAt(i) as DisplayObjectContainer, eventType);
                if(result){
                    return result;
                }
            }
        }
        return null;
    }



    override protected function initialize():void {
        super.initialize();
        setSizeInternal(200, stage.stageHeight, false);
        closeButton.visible = false;

        background.color = 0x000000;
        background.alpha = 0.8;
        contentBackgroundImage.visible = false;

        _messageLabel = new Label();
        _messageLabel.width = actualWidth;
        _messageLabel.height = 40;
        _messageLabel.textRendererFactory = messageFactory;
        _messageLabel.text = "<b>Please take a second</b><br/>to vote for these challengers!";
        _messageLabel.x = _messageLabel.y = 9;
        addChild(_messageLabel);

        _nextButton = new Button();
        _skipButton = new Button();
        _skipButton.addEventListener(Event.TRIGGERED, skipButtonHandler);
        _nextButton.addEventListener(Event.TRIGGERED, nextButtonHandler);
        _nextButton.labelFactory = buttonLabelFactory;
        _skipButton.labelFactory = buttonLabelFactory;
        _nextButton.label = "next";
        _skipButton.label = "skip";
        //
        try{
            var screenRatio:Number = Starling.current.nativeStage.stage.fullScreenWidth / Capabilities.screenDPI;
            if(screenRatio < 3.5){
                _nextButton.scale = _skipButton.scale = 1.35;
            }
        }catch (e:Error) {

        }
        //

        _skipButton.width = _skipButton.height = NaN;
        _skipButton.invalidate(INVALIDATION_FLAG_SIZE);

        _nextButton.width = _nextButton.height = NaN;
        _nextButton.invalidate(INVALIDATION_FLAG_SIZE);

        _skipButton.validate();
        _nextButton.validate();

        _challengeLikeBlueButton = ButtonFactory.challengeLiteButton('challenge-like-1', 'challenge-like-selected-1', "left");
        _challengeLikeBlueButton.setSize(60, 30);
        _challengeLikeBlueButton.addEventListener(Event.TRIGGERED, blueTuningVoteHandler);
        addChild(_challengeLikeBlueButton);

        _challengeLikeRedButton = ButtonFactory.challengeLiteButton('challenge-like-2', 'challenge-like-selected-2', "right");
        _challengeLikeRedButton.setSize(60, 30);
        _challengeLikeRedButton.addEventListener(Event.TRIGGERED, redTuningVoteHandler);
        addChild(_challengeLikeRedButton);

        _vsImage = new Image(ButtonFactory.assetManager.getTexture("vs_large"));
        _vsImage.scale = ButtonFactory.assetManager.scaleFactor/2;

        _challengeLikeBlueButton.visible = _challengeLikeRedButton.visible = false;

        addChild(_skipButton);
        addChild(_nextButton);
    }

    override protected function draw():void {
        super.draw();
        if(_nextButton) {
            _nextButton.x = actualWidth / 3 - _nextButton.width / 2;
            _nextButton.y = actualHeight - (_nextButton.height + GAP);
        }
        if(_skipButton) {
            _skipButton.x = 2 * actualWidth / 3 - _skipButton.width / 2;
            _skipButton.y = actualHeight - (_skipButton.height + GAP);
        }
        if(_challengeData) {
            if(_challengeLikeBlueButton) {
                _challengeLikeBlueButton.label = _challengeData.owner.votes;
            }
            if (_challengeLikeRedButton) {
                _challengeLikeRedButton.label = _challengeData.receiver.votes;
            }
        }
        if(_vsImage){
            _vsImage.x = (actualWidth - _vsImage.width)/2;
            _vsImage.y = (actualHeight - _vsImage.height)/2;
        }
        if(_challengeLikeBlueButton && _ownerTuningAvatar){
            _challengeLikeBlueButton.x = _ownerTuningAvatar.x - _challengeLikeBlueButton.width + 4;
            _challengeLikeBlueButton.y = _ownerTuningAvatar.y + (_ownerTuningAvatar.height - _challengeLikeBlueButton.height)/2;
        }
        if(_challengeLikeRedButton && _receiverTuningAvatar){
            _challengeLikeRedButton.x = _receiverTuningAvatar.x + _receiverTuningAvatar.width - 4;
            _challengeLikeRedButton.y = _receiverTuningAvatar.y + (_receiverTuningAvatar.height - _challengeLikeRedButton.height)/2;
        }
    }

    private function nextButtonHandler(event:Event):void {
        SoundManager.playSound(SoundManager.SELECT_PART);
        dispatchEvent(new Event(ChallengeEvent.VIEW_NEXT_CHALLENGE_TUNING));
    }

    private function skipButtonHandler(event:Event):void {
        SoundManager.playSound(SoundManager.SELECT_PART_BACK);
        if(showBanner){
            AdsManager.$.bannerShow(AdsManager.$.currentBannerScreen);
        }
        skip();
    }

    private static function buttonLabelFactory():ITextRenderer {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 14, 0xFFFFFF);
        textRenderer.textFormat.align = TextFormatAlign.CENTER;
        textRenderer.textFormat.leftMargin = 0;
        textRenderer.textFormat.bold = false;
        textRenderer.textFormat.underline = true;
        textRenderer.embedFonts = true;
        return textRenderer;
    }

    private static function messageFactory():ITextRenderer {
        var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
        textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 12, 0xFFFFFF);
        textRenderer.textFormat.align = TextFormatAlign.LEFT;
        textRenderer.textFormat.leftMargin = 0;
        textRenderer.textFormat.bold = false;
        textRenderer.wordWrap = true;
        textRenderer.isHTML = true;
        textRenderer.embedFonts = true;
        return textRenderer;
    }

    private function blueTuningVoteHandler(event: Event): void {
        SoundManager.playSound(SoundManager.SELECT_PART);

        var challengesProxy:ChallengesProxy = ApplicationFacade.getInstance().retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
        var tuningId:String = _ownerTuning.id;
        var userType:String = 'owner';
        challengesProxy.voteChallenge(_challengeData._id, userType, tuningId, function (newData: ChallengeData): void {
            _challengeLikeBlueButton.isSelected = true;
            _challengeLikeBlueButton.touchable = false;
            if (newData) {
                _challengeData.owner.votes = newData.owner.votes;
                _challengeLikeBlueButton.label = newData.owner.votes;
            }
        }, null, function():void{
            _challengeLikeBlueButton.isSelected = false;
            _challengeLikeBlueButton.touchable = true;
        })
    }

    private function redTuningVoteHandler(event: Event): void {
        SoundManager.playSound(SoundManager.SELECT_PART);

        var challengesProxy:ChallengesProxy = ApplicationFacade.getInstance().retrieveProxy(ChallengesProxy.NAME) as ChallengesProxy;
        var tuningId:String = _receiverTuning.id;
        var userType:String = 'receiver';
        challengesProxy.voteChallenge(_challengeData._id, userType, tuningId, function (newData: ChallengeData): void {
            _challengeLikeRedButton.isSelected = true;
            _challengeLikeRedButton.touchable = false;
            if (newData) {
                _challengeData.receiver.votes = newData.receiver.votes;
                _challengeLikeRedButton.label = newData.receiver.votes;
            }
        }, null, function():void{
            _challengeLikeRedButton.isSelected = false;
            _challengeLikeRedButton.touchable = true;
        })
    }

    private function setLikeButtonStates():void{
        var blueLabel:String = 'owner';
        var redLabel:String = 'receiver';
        _challengeLikeBlueButton.isSelected = !_challengeData.canIVote(blueLabel);
        _challengeLikeBlueButton.touchable = _challengeData.canIVote(blueLabel);
        _challengeLikeRedButton.isSelected = !_challengeData.canIVote(redLabel);
        _challengeLikeRedButton.touchable = _challengeData.canIVote(redLabel);
    }

    public function loadChallenge(data:ChallengeData):void {
        if(!data){
            closeButtonTriggeredHandler(null);
            return;
        }
        if(!_challengeLikeBlueButton || !_challengeLikeRedButton){
            dispose();
            return;
        }
        _challengeData = data;
        _ownerTuningAvatar ||= new ChallengeTuningAvatar();
        _receiverTuningAvatar ||= new ChallengeTuningAvatar();
        if(!_ownerTuningAvatar.hasEventListener(Event.TRIGGERED)) {
            _ownerTuningAvatar.addEventListener(Event.TRIGGERED, avatarTriggerHandler);
        }
        if(!_receiverTuningAvatar.hasEventListener(Event.TRIGGERED)){
            _receiverTuningAvatar.addEventListener(Event.TRIGGERED, avatarTriggerHandler);
        }

        _ownerTuning = TuningsProxy.parseTuning(_challengeData.owner.tuning, false);
        _ownerTuning.userObj = _challengeData.owner.user;
        _receiverTuning = TuningsProxy.parseTuning(_challengeData.receiver.tuning, false);
        _receiverTuning.userObj = _challengeData.receiver.user;

        _receiverTuningAvatar.loadComplete = function():void{
            _challengeLikeRedButton.visible = true;
        };
        _ownerTuningAvatar.loadComplete = function():void{
            _challengeLikeBlueButton.visible = true;
        };
        _ownerTuningAvatar.tuning = _ownerTuning;
        _receiverTuningAvatar.tuning = _receiverTuning;

        addChild(_challengeLikeBlueButton);
        addChild(_ownerTuningAvatar);

        addChild(_challengeLikeRedButton);
        addChild(_receiverTuningAvatar);

        _challengeLikeBlueButton.visible = _challengeLikeRedButton.visible = false;

        addChild(_vsImage);
        _ownerTuningAvatar.x = actualWidth - (_ownerTuningAvatar.width + GAP);
        _receiverTuningAvatar.x = GAP;
        if(_messageLabel){
            _receiverTuningAvatar.y = _messageLabel.y + _messageLabel.height;
        }
        _ownerTuningAvatar.y = actualHeight/2 + 10;
        _receiverTuningAvatar.y = actualHeight/2 - (_receiverTuningAvatar.height + 10);

        setLikeButtonStates();
        draw();
    }

    private function avatarTriggerHandler(event:Event):void {
        SoundManager.playSound(SoundManager.PART_MENU);
        var isSwitched:Boolean = (event.target == _receiverTuningAvatar);
        var eventData:Object = {challengeData:_challengeData, receiverTuning: _receiverTuning, senderTuning: _ownerTuning, isSwitched:isSwitched};
        dispatchEvent(new Event(ChallengeEvent.VIEW_CHALLENGE_TUNING, false, eventData));
    }

    public function skip():void{
        showBanner = false;
        closeButtonTriggeredHandler(null);
        if(showDelay == 20){
            showDelay = 300;
        }else if(showDelay == 300){
            showDelay = 600;
        }else if(showDelay == 600){
            showDelay = -1;
        }
        if(showDelay != -1){
            requestShowVotePopup();
        }
    }

    override public function dispose():void {
        super.dispose();
        if(_messageLabel){
            _messageLabel.removeFromParent(true);
            _messageLabel = null;
        }
        if(_nextButton){
            _nextButton.removeFromParent(true);
            _nextButton = null;
        }
        if(_skipButton){
            _skipButton.removeFromParent(true);
            _skipButton = null;
        }
        _challengeData = null;
        _ownerTuning = null;
        _receiverTuning = null;
        if(_ownerTuningAvatar){
            _ownerTuningAvatar.removeFromParent(true);
            _ownerTuningAvatar = null;
        }
        if(_receiverTuningAvatar){
            _receiverTuningAvatar.removeFromParent(true);
            _receiverTuningAvatar = null;
        }
        if(_challengeLikeBlueButton){
            _challengeLikeBlueButton.removeFromParent(true);
            _challengeLikeBlueButton = null;
        }
        if(_challengeLikeRedButton){
            _challengeLikeRedButton.removeFromParent(true);
            _challengeLikeRedButton = null;
        }
        if(_vsImage){
            _vsImage.removeFromParent(true);
            _vsImage = null;
        }
    }
}
}
