package com.despark.tuning3d.view.components.challenges.modals {
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.model.challenges.ChallengeEvent;
import com.despark.tuning3d.model.challenges.ChallengesProxy;
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.friend.Friend;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.tunings.TuningsProxy;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.challenges.TuningChallengeType;
import com.despark.tuning3d.view.components.challenges.controls.ChallengeTuningItemRenderer;
import com.despark.tuning3d.view.components.challenges.controls.ChallengeUserItemRenderer;
import com.despark.tuning3d.view.components.challenges.controls.SearchBox;
import com.despark.tuning3d.view.components.challenges.controls.SearchResultList;
import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
import com.despark.tuning3d.view.components.core.PopUp;
import com.despark.tuning3d.view.components.instructions.SimpleImageProvider;
import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
    import com.distriqt.AdsManager;
    import com.utils.FontSelector;
import com.utils.ObjectUtils;
import com.utils.TextureUtil;

import feathers.controls.Button;
import feathers.controls.Label;
import feathers.controls.LayoutGroup;
import feathers.controls.Radio;
import feathers.controls.ToggleButton;
import feathers.controls.renderers.IListItemRenderer;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.ITextRenderer;
import feathers.core.ToggleGroup;
import feathers.data.ListCollection;
import feathers.data.ListCollection;
import feathers.layout.AnchorLayout;
import feathers.layout.HorizontalLayout;
import feathers.layout.RelativePosition;
import feathers.layout.VerticalAlign;


import flash.geom.Rectangle;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;
import flash.utils.setTimeout;

import starling.display.Image;
import starling.events.Event;
import starling.utils.ScaleMode;

public class StartChallengePopup extends PopUp {
    private var bgImage:Image;
    private var _container:LayoutGroup;
    private static var _instance:StartChallengePopup;
    //
    private var _confirmButton:Button;
    private var _cancelButton:Button;
    private var _data:Object;
    private var _avatar : UserCircularAvatar;
    private var _selectedCar:ImageLoaderWithStates;
    private var _senderCar:ImageLoaderWithStates;
    private var _randomChallengeSelect:ToggleButton;

    private var _challengeTypeGroup:ToggleGroup;
    private var _challengeTypeLayout:LayoutGroup;
    private var _challengeAllButton:ToggleButton;
    private var _challengeSameCarButton:ToggleButton;

    private var _recipientLabel:Button;
    private var _senderLabel:Button;
    private var _challengesList:Object = {};
    private var _currentUserTuningList:Array = [];
    private var _currentUserFriendList:Array = [];
    private var _tuningList:SearchBox;
    private var _userList:SearchBox;
    private var _friendsRequestedIndex:int;

    public static const INPUT_SIZE:int = 10;

    private var _tuning:Tuning;
    private var _user:User;
    private var _currentUser:User;
    private var _isInvitation:Boolean;
    private var _showBannerAfterExit:Boolean;
    private var _bannerScreen:String;
    public const tuningLimit:int = 100;

    private var _carY:Number = -10;
    private var _tuningChallengeType:String;
    private var _emptyMessageLabel:Label;
    public function StartChallengePopup(popupData:Object = null) {
        super();
        var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
        _currentUser = usersProxy.currentUser;
        _data = popupData;
        _showBannerAfterExit = _data.hasOwnProperty("showBannerAfterExit") && _data.showBannerAfterExit;
        _bannerScreen = (popupData && popupData.hasOwnProperty("bannerScreen"))?popupData["bannerScreen"]:BannerScreen.BANNER_SCREEN;
    }

    public function get tuningSkip():int{
        return _tuningList.length;
    }

    override protected function initialize():void {
        super.initialize();
        setSizeInternal(stage.stageWidth, stage.stageHeight,false);
        _carY = actualHeight/5 + 20;
        _isInvitation = _data.hasOwnProperty("isInvitation") && _data.isInvitation;


        _container ||= new LayoutGroup();
        background.color = 0x555555;
        background.alpha = 0.8;
        contentBackgroundImage.visible = false;

        bgImage ||= new Image(ButtonFactory.assetManager.getTexture("challenges_start_accept_bg"));
        _container.layout = new AnchorLayout();
        addChild(_container);
        _container.addChild(bgImage);

        _recipientLabel = getLabel();
        if(_isInvitation) {
            _recipientLabel.addEventListener(Event.TRIGGERED, selectTuningHandler);
            _tuningChallengeType = ObjectUtils.getProperty(_data, 'challengeData.challengeType')
        }else{
            _recipientLabel.addEventListener(Event.TRIGGERED, selectUserHandler);
        }

        _senderLabel = getLabel(_isInvitation);
        if(!_isInvitation) {
            _senderLabel.addEventListener(Event.TRIGGERED, selectTuningHandler);
        }

        addChild(_senderLabel);
        addChild(_recipientLabel);
        if(_isInvitation && _data.sender.user){
            setSelectedUser(_data.sender.user as User);
        }else if(_data.recipient.user) {
            if(_currentUser && _data.recipient.user && _currentUser.id != User(_data.recipient.user).id) {
                setSelectedUser(_data.recipient.user as User);
            }
        }
        //
        var acceptLabel:String = "Invite";
        var challengeData:Object = {selectTuning: true};
        if(_data && _data.hasOwnProperty("isInvitation") && _data.isInvitation){
            acceptLabel = "Accept";
            challengeData.selectTuning = true;
            _senderCar = new ImageLoaderWithStates();
            _senderCar.scaleMode = ScaleMode.NONE;
            var simpleImageProvider : SimpleImageProvider = new SimpleImageProvider();
            simpleImageProvider.urlForDownloading = _data.sender.tuning.thumbnailURL;
            _senderCar.persistentImageProvider = simpleImageProvider;
            _senderCar.completeHandler = function():void{
                _senderCar.width = 200;
                _senderCar.height = _senderCar.width * _senderCar.originalSourceHeight / _senderCar.originalSourceWidth;
                _senderCar.x = _senderLabel.x + (_senderLabel.width - _senderCar.width)/2;
                _senderCar.y = _senderLabel.y + 10 - _senderCar.height - (_senderCar.height < 125? (125 - _senderCar.height) / 2: 0);
                if(_data.isInvitation){
                    _senderCar.scaleMode = ScaleMode.NONE;
                    _senderCar.scaleX = -1;
                    _senderCar.x += _senderCar.width;
                }
            };
            _senderCar.errorHandler = function():void{

            };
            _container.addChild(_senderCar);
            _senderLabel.label = _data.sender.tuning.name;
        }else{
            if(_data) {
                challengeData.selectTuning = !(_data.sender.tuning == null);
            }else{
                challengeData.selectTuning = false;
            }
        }

        createChallengeTypes();

        _confirmButton = ButtonFactory.challengeAcceptButton(acceptLabel);
        _cancelButton = ButtonFactory.challengeDeclineButton("Decline");
        _container.addChild(_confirmButton);
        _container.addChild(_cancelButton);
        _cancelButton.addEventListener(Event.TRIGGERED, onCancelTriggered);
        if(_data && _data.hasOwnProperty("isInvitation") && _data.isInvitation){
            _confirmButton.addEventListener(Event.TRIGGERED, acceptButtonHandler);
        }else {
            _confirmButton.addEventListener(Event.TRIGGERED, inviteButtonHandler);
        }
        _tuningList = new SearchBox();
        _tuningList.searchResult.addEventListener(Event.CHANGE, tuningSelectHandler);
        _tuningList.searchResult.itemRendererFactory = function() : IListItemRenderer {
            var renderer : ChallengeTuningItemRenderer = new ChallengeTuningItemRenderer();
            var item_w:Number=(_tuningList.searchResult.width/4);
            var item_h:Number=(_tuningList.searchResult.height/2);
            renderer.setSize(item_w, item_h);
            return renderer;
        };
        _tuningList.customSearchProvider = function(searchTerm:String):void{
            dispatchEvent(new Event(ChallengeEvent.SEARCH_TUNINGS, false, {searchTerm:searchTerm, skip:0, completeHandler:function (tuningsList:Array):void {
                    var tunings:Array = [];
                    for(var i:int = 0; i < tuningsList.length; i++){
                        var tng:Tuning = TuningsProxy.parseTuning(tuningsList[i], true);
                        if(_tuningChallengeType && _tuningChallengeType == TuningChallengeType.SAME_CAR){
                            if(tng.car.id == _data.sender.tuning.carObj._id && !_challengesList.hasOwnProperty(tng.id)){
                                tunings.push(tng);
                            }
                        }else if(!_challengesList.hasOwnProperty(tng.id)) {
                            tunings.push(tng);
                        }
                    }
                    _tuningList.searchResult.dataProvider = new ListCollection(tunings);
                }
            }));
        };

        _tuningList.searchResult.addEventListener(SearchResultList.END_REACHED, tuningListEndReachedHandler);
        _container.addChild(_tuningList);

        _userList = new SearchBox();
        _userList.searchResult.addEventListener(Event.CHANGE, userSelectHandler);
        _userList.setSize(actualWidth, getListHeight());
        ChallengesProxy.USER_REQUEST_COUNT = 2*Math.ceil(_userList.width/80)*Math.ceil(_userList.height/60);
        _userList.searchResult.itemRendererFactory = function() : IListItemRenderer {
            var renderer : ChallengeUserItemRenderer = new ChallengeUserItemRenderer();
            var item_w:Number=80;//(_userList.searchResult.width/5);
            var item_h:Number=60;//(_userList.searchResult.height/3);
            renderer.setSize(item_w, item_h);
            return renderer;
        };
        _container.addChild(_userList);
        _friendsRequestedIndex = 0;
        _currentUserFriendList = [];
        _userList.searchResult.addEventListener(Event.SCROLL, onUserListScroll);

        _userList.visible = _tuningList.visible= false;
        _userList.customSearchProvider = function(searchTerm:String):void{
            dispatchEvent(new Event(ChallengeEvent.SEARCH_USERS, false, {searchTerm:searchTerm, skip:0, completeHandler:function (friendList:Array):void {
                    for(var i:int = friendList.length-1; i >= 0; i--){
                        var friend:Friend = friendList[i];
                        if(friend.user.id == _currentUser.id){
                            friendList.removeAt(i);
                        }
                    }
                    _userList.searchResult.dataProvider = new ListCollection(friendList);
                }
            }));
        };

        var closeIcon:Image = new Image(ButtonFactory.assetManager.getTexture("garage_close"));
        closeIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
        closeButton.width = closeIcon.width;
        closeButton.height = closeIcon.height;
        closeButton.defaultIcon = closeIcon;
        addChild(closeButton);

        _emptyMessageLabel = new Label();
        _emptyMessageLabel.wordWrap = true;
        _emptyMessageLabel.textRendererFactory = function () : ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMonserratSemiBold(), 11, 0x424242);
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = false;
            textRenderer.embedFonts = true;
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            return textRenderer;
        };

        dispatchEvent(new Event(ChallengeEvent.LOAD_USER_CHALLENGES, false, challengeData));
        dispatchEvent(new Event(ChallengeEvent.LOAD_USER_FRIENDS, false, {skip:0}));
    }

    private function createChallengeTypes():void{
        _challengeTypeGroup = new ToggleGroup();
        _challengeTypeLayout = new LayoutGroup();
        var challengeLayout:HorizontalLayout = new HorizontalLayout();
        challengeLayout.gap = 30;
        _challengeTypeLayout.layout = challengeLayout;

        _challengeAllButton = ButtonFactory.challengeRadioButton('All Cars');
        _challengeAllButton.name = TuningChallengeType.ALL;
        _challengeSameCarButton = ButtonFactory.challengeRadioButton('Same Car');
        _challengeSameCarButton.name = TuningChallengeType.SAME_CAR;
        _challengeTypeGroup.addItem(_challengeAllButton);
        _challengeTypeGroup.addItem(_challengeSameCarButton);
        _challengeTypeGroup.addEventListener(Event.CHANGE, challengeTypeChangeHandler);
        _challengeTypeLayout.addChild(_challengeAllButton);
        _challengeTypeLayout.addChild(_challengeSameCarButton);
        _container.addChild(_challengeTypeLayout);
        _challengeTypeLayout.validate();
        if(_isInvitation && _tuningChallengeType){
            for(var i:int = 0; i < _challengeTypeGroup.numItems; i++){
                if(_challengeTypeGroup.getItemAt(i).name == _tuningChallengeType){
                    _challengeTypeGroup.selectedItem = _challengeTypeGroup.getItemAt(i);
                    break;
                }
            }
            _challengeTypeLayout.isEnabled = false;
        }
    }

    private function challengeTypeChangeHandler(event:Event):void {
        if(_challengeTypeGroup.selectedItem){
            SoundManager.playSound(SoundManager.SELECT_PART);
        }
    }

    private function tuningListEndReachedHandler(event:Event):void {
        dispatchEvent(new Event(ChallengeEvent.LOAD_USER_ALLOWED_TUNINGS));
    }

    private function inviteButtonHandler(event:Event):void {
        var challengeType:String = ((_challengeTypeGroup && _challengeTypeGroup.selectedItem)?_challengeTypeGroup.selectedItem.name:'all');
        var eventData:Object = {};
        if(_tuning){
            eventData = {
                userId:_tuning.userObj._id,
                carId: _tuning.car.id,
                carType: _tuning.car.type,
                challengeType: challengeType,
                tuningId:_tuning.id
            };
        }
        if(_randomChallengeSelect && _randomChallengeSelect.isSelected && _tuning){
            dispatchEvent(new Event(ChallengeEvent.RANDOM_CHALLENGE, false, eventData));
            _confirmButton.alpha = _cancelButton.alpha = 0.6;
            _confirmButton.isEnabled = _cancelButton.isEnabled = false;
            setTimeout(closeButtonTriggeredHandler, 200, null);
        }else if(_tuning && _user){
            eventData.userId = _user.id;
            dispatchEvent(new Event(ChallengeEvent.INVITE_FOR_CHALLENGE, false, eventData));
            _confirmButton.alpha = _cancelButton.alpha = 0.6;
            _confirmButton.isEnabled = _cancelButton.isEnabled = false;
            setTimeout(closeButtonTriggeredHandler, 200, null);
        }else{
            if(!_tuning){
                AlertView.show("Error: No tuning selected.");
            }else{
                AlertView.show("Error. No user selected.");
            }
        }
    }

    override protected function closeButtonTriggeredHandler(event:Event):void{
        super .closeButtonTriggeredHandler(event);
        SoundManager.playSound(SoundManager.SELECT_PART_BACK);
        if(_showBannerAfterExit){
            if(_bannerScreen == BannerScreen.BANNER_SCREEN){
                BannerStackedScreen.showAdBanner();
            }else if(_bannerScreen == BannerScreen.HOME_SCREEN){
                AdsManager.$.bannerShow(BannerScreen.HOME_SCREEN);
            }
        }
    }

    private function acceptButtonHandler(event:Event):void {
        SoundManager.playSound(SoundManager.SELECT_PART);
        closeButtonTriggeredHandler(null);
        dispatchEvent(new Event(ChallengeEvent.ACCEPT_CHALLENGE, false, {challengeId:_data.challengeId, tuningId:_tuning.id, notification:_data.notification}));
    }

    private function onUserListScroll(event:Event):void {
        if(_userList.isWaiting){
            return;
        }
        if(_userList.searchResult.verticalScrollPosition>=_userList.searchResult.maxVerticalScrollPosition) {
            if (_userList.searchField.text.length > 0) {
                if(!_userList.searchResult.dataProvider){
                    _userList.searchResult.dataProvider = new ListCollection([]);
                }
                var skip:int = _userList.searchResult.dataProvider.length;
                _userList.isWaiting = true;
                _userList.searchResult.loadingItems = 1;
                dispatchEvent(new Event(ChallengeEvent.SEARCH_USERS, false, {
                    searchTerm: _userList.searchField.text, skip: skip, completeHandler: function (friendList:Array):void {
                        for(var i:int = friendList.length-1; i >= 0; i--){
                            var friend:Friend = friendList[i];
                            if(friend.user.id == _currentUser.id){
                                friendList.removeAt(i);
                            }
                        }
                        _userList.searchResult.dataProvider.addAll(new ListCollection(friendList));
                        _userList.isWaiting = false;
                        _userList.searchResult.loadingItems = 0;
                    }
                }));
            }else if (_friendsRequestedIndex < _currentUserFriendList.length) {
                _userList.searchResult.loadingItems = 1;
                dispatchEvent(new Event(ChallengeEvent.LOAD_USER_FRIENDS, false, {skip: _currentUserFriendList.length}));
                _friendsRequestedIndex = _currentUserFriendList.length;
            }
        }
    }

    private function selectUserHandler(event:Event):void {
        if(_tuningList)
            _tuningList.visible = false;
        if(_userList) {
            _userList.visible = !_userList.visible;
            if(_userList.visible){
                SoundManager.playSound(SoundManager.PART_MENU);
            }else{
                SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
            }
            if(_userList && _userList.parent && _userList.parent.getChildIndex(_userList)<_userList.parent.numChildren-1){
                _userList.parent.setChildIndex(_userList, _userList.parent.numChildren-1);
            }
        }
    }

    private function selectTuningHandler(event:Event):void {
        if(_userList)
            _userList.visible = false;
        if(_tuningList) {
            _tuningList.visible = !_tuningList.visible;
            if(_tuningList.visible){
                SoundManager.playSound(SoundManager.PART_MENU);
            }else{
                SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
            }
            if(_tuningList && _tuningList.parent && _tuningList.parent.getChildIndex(_tuningList)<_tuningList.parent.numChildren-1){
                _tuningList.parent.setChildIndex(_tuningList, _tuningList.parent.numChildren-1);
            }
        }

    }

    private function onCancelTriggered(event:Event):void {
        closeButtonTriggeredHandler(null);
        if(_isInvitation){
            dispatchEvent(new Event(ChallengeEvent.DECLINE_CHALLENGE, false, {challengeId:_data.challengeId, tuningId:null, notification:_data.notification}));
        }
    }

    override public function dispose():void {
        super.dispose();
        removeFromParent();
        if(bgImage){
            bgImage.removeFromParent(true);
            bgImage = null;
            ButtonFactory.assetManager.removeTexture("challenges_start_accept_bg", true);
        }

        if(_emptyMessageLabel){
            _emptyMessageLabel.removeFromParent(true);
            _emptyMessageLabel = null;
        }

        if(_challengeAllButton){
            _challengeAllButton.removeFromParent(true);
            _challengeAllButton = null;
        }

        if(_challengeSameCarButton){
            _challengeSameCarButton.removeFromParent(true);
            _challengeSameCarButton = null;
        }

        if(_challengeTypeLayout){
            _challengeTypeLayout.removeFromParent(true);
            _challengeTypeLayout = null;
        }

        if(_challengeTypeGroup){
            _challengeTypeGroup.removeAllItems();
            _challengeTypeGroup = null;
        }

        _instance = null;
    }

    public function appendChallenges(challenges:Array):void{
        _challengesList = {};
        for(var i:int = 0;  i < challenges.length; i++){
            _challengesList[challenges[i]] = true;
        }
    }

    public function setUserTunings(tunings:Array, selectTuning:Boolean = false):void{
        var latestFreeTuning:Tuning;
        _currentUserTuningList ||= [];
        for(var i:int = 0; i < tunings.length; i++){
            if(!_challengesList.hasOwnProperty(Tuning(tunings[i]).id)){
                if(_tuningChallengeType == TuningChallengeType.SAME_CAR){
                    if(Tuning(tunings[i]).car.id == _data.sender.tuning.carObj._id){
                        _currentUserTuningList.push(tunings[i]);
                    }
                }else{
                    _currentUserTuningList.push(tunings[i]);
                }
            }
        }
        _tuningList.searchResult.validate();

        if(selectTuning){
            if(_currentUserTuningList.length) {
                latestFreeTuning = _currentUserTuningList[0];
                if(!_isInvitation && _data.sender && _data.sender.tuning){
                    setSelectedTuning(_data.sender.tuning);
                }else {
                    setSelectedTuning(latestFreeTuning);
                }
                if(_emptyMessageLabel){
                    _emptyMessageLabel.removeFromParent();
                }
            }else{
                setSelectedTuning(null);
                if(_emptyMessageLabel && _tuningChallengeType){
                    _tuningList.addChild(_emptyMessageLabel);
                    _emptyMessageLabel.text = TuningChallengeType.getMessage(_tuningChallengeType);
                    _emptyMessageLabel.width = actualWidth / 2;
                    _emptyMessageLabel.height = NaN;
                    _emptyMessageLabel.validate();
                    _emptyMessageLabel.x = (_tuningList.width - _emptyMessageLabel.width) / 2;
                    _emptyMessageLabel.y = (_tuningList.height - _emptyMessageLabel.height) / 2;
                }
            }
        }else{
            setSelectedTuning(null);
        }
    }

    public function setUserFriends(friends:Array):void{
        _currentUserFriendList ||= [];
        _currentUserFriendList = _currentUserFriendList.concat(friends);
        var collection:ListCollection = (_userList.searchResult.dataProvider as ListCollection)||new ListCollection([]);
        collection.addAll(new ListCollection(friends));
        _userList.searchResult.dataProvider = collection;
        _userList.searchResult.loadingItems = 0;
    }

    private function setSelectedUser(user:User):void{
        _user = user;
        if(_avatar){
            _avatar.removeFromParent(true);
        }
        _avatar = new UserCircularAvatar(user, _isInvitation?30: 54);
        _container.addChild(_avatar);
        if(!_isInvitation){
            _recipientLabel.label = user.nameToShow;
        }
        if(_userList) {
            _userList.setSize(actualWidth, getListHeight());
            _userList.y = actualHeight - _userList.height;
        }
    }

    private function setSelectedTuning(tuning:Tuning):void {
        if(tuning && _senderLabel){
            _tuning = tuning;
            if(_selectedCar){
                _selectedCar.removeFromParent(true);
            }
            if(_isInvitation){
                _recipientLabel.label = tuning.name;
            }else{
                _senderLabel.label = tuning.name;
            }
            _selectedCar = new ImageLoaderWithStates();
            var simpleImageProvider : SimpleImageProvider = new SimpleImageProvider();
            simpleImageProvider.urlForDownloading = tuning.thumbnailURL;
            _selectedCar.persistentImageProvider = simpleImageProvider;
            _selectedCar.completeHandler = function():void{
                _selectedCar.width = 200;
                _selectedCar.height = _selectedCar.width * _selectedCar.originalSourceHeight / _selectedCar.originalSourceWidth;
                _selectedCar.x = _recipientLabel.x + (_recipientLabel.width - _selectedCar.width)/2;
                _selectedCar.y = _recipientLabel.y + 10 - _selectedCar.height - (_selectedCar.height < 125? (125 - _selectedCar.height) / 2:0);
                if(!_data.isInvitation){
                    _selectedCar.x = _senderLabel.x + (_senderLabel.width - _selectedCar.width)/2;
                    _selectedCar.scaleMode = ScaleMode.NONE;
                    _selectedCar.scaleX = -1;
                    _selectedCar.x += _selectedCar.width;
                }
            };
            _selectedCar.errorHandler = function():void{

            };
            _container.addChild(_selectedCar);
        }
        if(_currentUserTuningList && _tuningList){
            if(!_tuningList.searchResult.dataProvider || _tuningList.searchResult.dataProvider.length==0){
                _tuningList.searchResult.dataProvider = new ListCollection(_currentUserTuningList);
            }
            _tuningList.setSize(actualWidth, getListHeight());
            _tuningList.y = actualHeight - _tuningList.height;
            _container.addChild(_tuningList);
            _tuningList.visible = (tuning==null);
        }
    }

    private function getListHeight():Number{
        return actualHeight/2 - 10;
    }


    private function searchTuning(str:String, collection:ListCollection):Array{
        var words:Array = str.toLowerCase().split(" ");
        var searchFunction:Function = function (element:*, index:int, arr:Array):Boolean {
            var tuning:Tuning = element as Tuning;
            var result:Boolean = true;
            var name:String = tuning.name.toLowerCase();
            for(var i:int = 0; i < words.length; i++){
                if(name.indexOf(words[i])==-1){
                    result = false;
                    break;
                }
            }
            return result;
        };
        var dataArray:Array = collection.data as Array;
        return dataArray.filter(searchFunction);
    }

    private function tuningSelectHandler(event:Event):void {
        if(_tuningList.searchResult.selectedItem) {
            SoundManager.playSound(SoundManager.SELECT_PART);
            setSelectedTuning(_tuningList.searchResult.selectedItem as Tuning);
        }
    }

    private function userSelectHandler(event:Event):void {
        if(_userList.searchResult.selectedItem) {
            SoundManager.playSound(SoundManager.SELECT_PART);
            setSelectedUser(Friend(_userList.searchResult.selectedItem).user);
            _avatar.x = _recipientLabel.x + _recipientLabel.width / 2 - _avatar.width / 2;
            _avatar.y = _recipientLabel.y - (_avatar.height + 15);
            _userList.visible = false;
        }
    }

    override protected function draw():void {
        super.draw();
        var ratio:Number = (actualWidth/actualHeight)/(bgImage.width/bgImage.height);
        if(ratio>=1){
            bgImage.width = actualWidth;
            bgImage.height  = actualHeight;
        }else{
            bgImage.height = actualHeight;
            bgImage.width  = actualWidth;
        }
        var gap:Number = 8;
        bgImage.x = (actualWidth-bgImage.width)/2;
        bgImage.y = (actualHeight-bgImage.height)/2;
        _confirmButton.x = actualWidth/2 - (_confirmButton.defaultSkin.width+gap);
        _cancelButton.x = actualWidth/2 + gap;
        var bottomGap:Number = 12;
        _confirmButton.y = _cancelButton.y = actualHeight - (_confirmButton.defaultSkin.height+bottomGap);
        _recipientLabel.y = _senderLabel.y = 273 / 2/*actualHeight/4 - _senderLabel.height*/;
        var margin:Number = 29;
        _senderLabel.x = margin;
        _recipientLabel.x = actualWidth - (_recipientLabel.width + margin);
        if(_data.isInvitation){
            if(_avatar){
                _avatar.x = _senderLabel.x;
                _avatar.y = _senderLabel.y - 115;
            }
        }else{
            if(_avatar){
                _avatar.x = _recipientLabel.x + _recipientLabel.width / 2 - _avatar.width / 2;
                _avatar.y = _recipientLabel.y - (_avatar.height + 15);
            }
        }

        if(_userList){
            _userList.setSize(actualWidth, getListHeight());
            _userList.y = actualHeight - _userList.height;
        }

        if(closeButton){
            closeButton.x = actualWidth - (closeButton.width + gap);
            closeButton.y = gap;
        }

        if(!_isInvitation){
            if(!_randomChallengeSelect) {
                _randomChallengeSelect = ButtonFactory.challengeRadioButton('Random Challenge');
                _container.addChild(_randomChallengeSelect);
                _randomChallengeSelect.addEventListener(Event.CHANGE, randomChallengeChangeHandler);
                setTimeout(function():void{
                    invalidate("RandomChallenge");
                }, 30);
            }
            _randomChallengeSelect.x = _recipientLabel.x + 5;
            _randomChallengeSelect.y = _recipientLabel.y + _recipientLabel.height + 5;
        }

        if(_challengeTypeLayout){
            _challengeTypeLayout.x = _senderLabel.x + (_senderLabel.width - _challengeTypeLayout.width) / 2;
            _challengeTypeLayout.y = _senderLabel.y + _senderLabel.height + 5;
        }
    }

    private function randomChallengeChangeHandler(event:Event):void {
        if(_randomChallengeSelect.isSelected){
            SoundManager.playSound(SoundManager.SELECT_PART);
            if(_avatar){
                _avatar.alpha = 0.6;
            }
            if(_recipientLabel){
                _recipientLabel.alpha = 0.6;
            }
        }else{
            SoundManager.playSound(SoundManager.SELECT_PART_BACK);
            if(_avatar){
                _avatar.alpha = 1;
            }
            if(_recipientLabel){
                _recipientLabel.alpha = 1;
            }
        }
        if(_recipientLabel) {
            _recipientLabel.isEnabled = !_randomChallengeSelect.isSelected;
        }
    }

    public static function getInstance(data:Object = null):StartChallengePopup{
        if(_instance){
            _instance.dispose();
        }
        _instance = new StartChallengePopup(data);
        return _instance;
    }

    private static function getLabel(isInvitation:Boolean = false):Button{
        var button:Button = new Button();
        var textureId:String = (isInvitation?"invitation_label_bg":"decline_btn_bg");
        var bg:Image = new Image(ButtonFactory.assetManager.getTexture(textureId));
        TextureUtil.scaleImage(bg, 0.5);
        var rectSize:Number = bg.texture.height/2-.5;
        bg.scale9Grid = new Rectangle(rectSize, rectSize, bg.texture.width-2*rectSize, 1);
        button.defaultSkin = bg;
        button.width = 325 / 2;
        button.height = bg.height;
        button.paddingLeft = bg.texture.height/2;
        button.verticalAlign = VerticalAlign.MIDDLE;
        button.gap = 10;
        if(!isInvitation) {
            button.iconOffsetX = -10;
            var arrow:Image = new Image(ButtonFactory.assetManager.getTexture("arrow_down"));
            arrow.scale = arrow.texture.scale / 2;
            button.defaultIcon = arrow;
            button.iconPosition = RelativePosition.RIGHT;
        }
        button.label = " ";
        var textColor:uint = isInvitation?0xFFFFFF:0x383838;
        button.labelFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMonserratMedium(), 10, textColor);
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = false;
            textRenderer.embedFonts = true;
            textRenderer.wordWrap = false;
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.width = button.width-(button.paddingRight+button.paddingLeft+button.gap);
            return textRenderer;
        };
        return button;
    }

}
}
