package com.despark.tuning3d.view.components.core
{
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.view.components.brands.PartInfoPopUp;
import com.utils.FontSelector;

import feathers.core.FeathersControl;

import flash.geom.Rectangle;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.animation.Transitions;
import starling.animation.Tween;
import starling.core.Starling;
import starling.display.Image;
import starling.display.Quad;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;
import starling.textures.Texture;
import starling.utils.AssetManager;

public class AlertView extends FeathersControl implements Multilingual
	{
		public static const MIN_HEIGHT : int = 100;
		public static var assetManager : AssetManager;
		public static var language : Language;
		
		public var simpleAlert : Boolean;
		public var resultHandler : Function;
		
		private var backgroundImage:Image;
		private var label : MultilingualLabel;
		private var _text : String;
		private var closeButton : MultilingualButton;
		private var yesButton : MultilingualButton;
		private var dismissingView : Quad;
		
		public function AlertView() {
			super();
			simpleAlert = true;
		}
		
		public function get text():String {
			return _text;
		}

		public function set text(value:String):void {
            _text = value;
			if(label){
                label.invalidate();
			}
			invalidate("text");
		}
		
		public function updateWithLanguage(language :Language) : void {
			label.updateWithLanguage(language);
		}

		protected override function initialize() : void {
			super.initialize();

			var backgroundTexture : Texture = assetManager.getTexture("popUpBackground");
			//var backgroundTextures : Scale9Textures = new Scale9Textures(backgroundTexture, new Rectangle(24, 22, 11, 7));
			
			backgroundImage = new Image(backgroundTexture);
			backgroundImage.scale9Grid = new Rectangle(24, 22, 11, 7);
			backgroundImage.alpha = simpleAlert ? 0.8 : 1.0;
			addChild(backgroundImage);	
			
			label = new MultilingualLabel();
			label.textFormat = new TextFormat(FontSelector.getFont(), simpleAlert ? 15 : 10, 0xFFFFFF );
			label.textFormat.align = TextFormatAlign.CENTER;
			label.wordWrap = true;
			
			addChild(label);

			if(!AlertView.language) //TODO , bug accured when you dont have internet, and start app, 
			{
				AlertView.language = new Language(null);
                AlertView.language.code = "en-US";
			}
			
			if(!simpleAlert)
			{
				closeButton = ButtonFactory.defaultButton((resultHandler ? 
					AlertView.language.translate("mob_signup", "no").toUpperCase() :
					AlertView.language.translate("mob_signup", "close")).toUpperCase(), 0xB2BDC1);
				closeButton.updateWithLanguage(AlertView.language);
				closeButton.addEventListener(Event.TRIGGERED, closeButtonTriggeredHandler);
				addChild(closeButton);
				
				if(resultHandler)
				{
					yesButton = ButtonFactory.defaultButton(AlertView.language.translate("mob_signup", "yes").toUpperCase(), 0xED3F25);
					yesButton.updateWithLanguage(AlertView.language);
					yesButton.addEventListener(Event.TRIGGERED, yesButtonTriggeredHandler);
					addChild(yesButton);
				}
				
				dismissingView = new Quad(stage.stageWidth, stage.stageHeight, 0xFF0000);
				dismissingView.alpha = 0.0;
				dismissingView.addEventListener(TouchEvent.TOUCH, dismissingViewTouchHandler);
				stage.addChildAt(dismissingView, stage.getChildIndex(this));
			}
		}
		
		private function dismissingViewTouchHandler(event : TouchEvent) : void
		{
			var touch : Touch = event.getTouch(dismissingView);
			
			if(touch && touch.phase == TouchPhase.ENDED)
			{
				hide();
			}
		}
		
		override protected function draw() : void {
			super.draw();
			if(isInvalid("text")) {
				var l:Language;
				if(AlertView.language) {
					label.updateWithLanguage(AlertView.language);
					l=AlertView.language;
				} else {
					l = new Language(null);
					l.code = "en-US";
					label.updateWithLanguage(l);
				}

				if(l!=null){
					var size:int=calcOptimalFontSizeForWidth(text, int(label.textFormat.size), actualWidth - 20,l);
					//label.updateWithLanguage(l);
					
					label.textFormat.size=size;// = new TextFormat(FontSelector.getFont(),size, 0xFFFFFF );
					//label.textFormat.horizontalAlign = TextFormatAlign.CENTER;
					//label.wordWrap = true;
					//label.isHTML=true;
					//label.paddingLeft=0;
					//label.paddingRight=0;
					
					label.updateWithLanguage(l);
				}
				
				label.text = text;
				
				label.width= actualWidth - 20;   
				label.height=NaN;
				label.validate();
				
				label.x = (actualWidth - label.width) / 2;  
				if(actualHeight == MIN_HEIGHT && simpleAlert)
				{
					label.y = (actualHeight - label.height) / 2;
				}else{
					label.y = 10;
				}
				backgroundImage.width = actualWidth;
				backgroundImage.height = actualHeight;  
				if(stage){
					x = (stage.stageWidth - actualWidth) / 2;
					y = (stage.stageHeight - actualHeight) / 2;    
				}
			}
			
			var autoSizeNeeded : Boolean = autoSizeIfNeeded();
			
			if(autoSizeNeeded || isInvalid(INVALIDATION_FLAG_SIZE)) {
				backgroundImage.width = actualWidth;
				backgroundImage.height = actualHeight;

				label.width = actualWidth - 20;
				
				label.x = (actualWidth - label.width) / 2;
				label.y = 10;
				
				if(actualHeight == MIN_HEIGHT && simpleAlert)
				{
					label.y = (actualHeight - label.height) / 2;
				}
				
				// UGLY FIX FOR AIR TEXT MEASURING PROBLEMS
				if(AlertView.language && label.text == AlertView.language.translate("mob_alertsAndOverlays", "savedToCameraRoll"))
				{
					label.y = 20;
				}
				
				
				// UGLY FIX FOR AIR TEXT MEASURING PROBLEMS
				if(AlertView.language && label.text == AlertView.language.translate("mob_alertsAndOverlays", "loadingParts"))
				{
					label.y = 28;
				}
				
				if(AlertView.language && label.text == "Сache deleted")
				{
					label.y = 28;
				}
				
				
				if(closeButton)
				{
					closeButton.y = actualHeight - closeButton.height - 10;
					
					if(resultHandler)
					{
						yesButton.y = closeButton.y;
						yesButton.x = 10;
						yesButton.width = (actualWidth - 20) / 2 - 5;
						
						closeButton.width = yesButton.width;
						closeButton.x = yesButton.x + yesButton.width + 10;
						
						
					}
					else
					{
						closeButton.x = 10;
						
						closeButton.width = actualWidth - 20;
					}
				}
				
				x = (stage.stageWidth - actualWidth) / 2;
				y = (stage.stageHeight - actualHeight) / 2;
			}
		}
		
		protected function autoSizeIfNeeded() : Boolean
		{
			//return false;
			var needsHeight : Boolean = isNaN(explicitHeight);
			
			if(!needsHeight)
			{
				return false;
			}
			
			label.width = explicitWidth-20;
			label.height = NaN;
			label.validate();
			
			var newHeight : Number = explicitHeight;
			
			if(needsHeight)
			{
				newHeight = Math.max(simpleAlert ? MIN_HEIGHT : 0, 20 +  Math.ceil(label.height) + 10 + (closeButton ? closeButton.height + 20 : 0));
			}
			
						
			return setSizeInternal(explicitWidth, newHeight, true);
		}
		
		
		public function hide() : void
		{
			AlertView.hideAlertView(this);
		}
		
		public static function calcOptimalFontSizeForWidth(text:String,  fontSize:uint, maxWidth:Number, language:Language):uint{
			
			
			var words:Array = text.split(" ");
			var test_word:String
			
			if(words){
				var max_len:int=(words[0] as String).length;
				var max_len_index:int=0;
				
				for(var i:int=0; i<words.length; i++){
					
					if((words[i] as String).length>max_len){
						
						max_len_index=i;
						max_len=(words[i] as String).length;
					}
				}
				
				test_word=words[max_len_index];
				
			}else{
				
				test_word=text;
			}
			
			
			var labelTmp : MultilingualLabel=new MultilingualLabel();
			
			labelTmp.updateWithLanguage(AlertView.language);
			labelTmp.textFormat = new TextFormat(FontSelector.getFont(), fontSize, 0xFFFFFF );
			labelTmp.textFormat.align = TextFormatAlign.CENTER;
			labelTmp.wordWrap = false;
			labelTmp.text=test_word;
			labelTmp.validate();
			
			if(labelTmp.width>maxWidth){
				
				return calcOptimalFontSizeForWidth(test_word, fontSize-1, maxWidth, language );
			}else{
				
				return fontSize;
			}
			
		}
		
		
		
		public static function show(text : String, hideTimeout : Number = 1.0, simple : Boolean = true, resultHandler : Function = null) : AlertView
		{
			
			
			
			
			var alertView : AlertView = new AlertView();
			
			alertView.text = text;
			alertView.width = simple ? 100 : 200;
			alertView.alpha = 0.0;
			alertView.simpleAlert = simple;
			alertView.resultHandler = resultHandler;
			alertView.touchable = !simple;
			
			Starling.current.stage.addChild(alertView);
			
			var tween : Tween = new Tween(alertView, 0.35, Transitions.EASE_IN_OUT);
			tween.fadeTo(1.0);
			Starling.juggler.add(tween);
			
			if(hideTimeout > 0)
			{
				Starling.juggler.delayCall(function() : void
				{
					hideAlertView(alertView);
				}, hideTimeout);
			}
				
				
			try{
				
				if(PartInfoPopUp.WEBVIEW){
						
					PartInfoPopUp.WEBVIEW_CLOSE_ALERT=alertView;
					PartInfoPopUp.WEBVIEW.visible=false;
						
					PartInfoPopUp.WEBVIEW_HIDE=true;
				}
				
			}catch(e:Error){
					
					
			}
			
			
			return alertView;
		}
		
		public static function hideAlertView(alertView : AlertView) : void
		{
			if(alertView.closeButton)
			{
				alertView.closeButton.visible = false;
			}
			
			if(alertView.yesButton)
			{
				alertView.yesButton.visible = false;
			}
			
			var tween : Tween = new Tween(alertView, 0.35, Transitions.EASE_IN_OUT);
			tween.fadeTo(0.0);
			tween.onComplete = function() : void
			{
				Starling.current.stage.removeChild(alertView);
				Starling.current.stage.removeChild(alertView.dismissingView);
				
				if(alertView==PartInfoPopUp.WEBVIEW_CLOSE_ALERT){
					try{
						if(PartInfoPopUp.WEBVIEW && PartInfoPopUp.WEBVIEW_HIDE){
							
							PartInfoPopUp.WEBVIEW.visible=true;
							
							PartInfoPopUp.WEBVIEW_HIDE=false;
							
							PartInfoPopUp.WEBVIEW_CLOSE_ALERT=null;
						}
					}catch(e:Error){
						
						
					}
				}
			}
			Starling.juggler.add(tween);
		}
		
		private function closeButtonTriggeredHandler(event : Event) : void {
			SoundManager.playSound(SoundManager.SELECT_PART_BACK);
			hide();
			
			if(resultHandler)
			{
				resultHandler(false);
			}
		}
		
		private function yesButtonTriggeredHandler(event : Event) : void
		{
			SoundManager.playSound(SoundManager.SELECT_PART);
			hide();

			if(resultHandler)
			{
				resultHandler(true);
			}
		}
	}
}