package com.despark.tuning3d.view.components.core {
	import com.utils.BitmapDataUtils;

	import feathers.core.FeathersControl;

	import flash.display.BitmapData;
	import flash.events.PermissionEvent;
	import flash.geom.Matrix;
	import flash.media.Camera;
	import flash.permissions.PermissionStatus;

	import starling.display.Image;
	import starling.textures.Texture;

	public class BackgroundContainer extends FeathersControl {
		
		public static const EVENT_CAMERA_ERROR:String = "EVENT_CAMERA_ERROR";
		
		private var _background:Image;
		private var _backgroundFromCamera:Image = null;
		private var _scale:Number = 1;
		
		private var _handleCameraTexture:Texture = null;
		private var _camera:Camera;
		private var _textureList:Vector.<Texture>;
		public function BackgroundContainer() {
			super();
            _textureList = new Vector.<Texture>();
		}

		private function onPermissionStatusChange(event:PermissionEvent):void{
			if(event.status==PermissionStatus.GRANTED){
				setCameraMode();
			} else {
				this.dispatchEventWith(EVENT_CAMERA_ERROR, false);
				_camera = null;
			}
		}
		
		
		private function setCameraMode():void{
			if(_camera){
				_camera.setMode(1920, 1080, 30);
				_camera.setQuality(0, 100);
				_handleCameraTexture = Texture.fromCamera(_camera,1, function(texture:Texture):void{
					_backgroundFromCamera = new Image(texture)
					addChild(_backgroundFromCamera);
					invalidate();
				});
			}
		}
		
		
		public function startCamera():void{
			_camera = Camera.getCamera();
			if(Camera.permissionStatus==PermissionStatus.UNKNOWN || Camera.permissionStatus==PermissionStatus.DENIED){
				_camera.addEventListener(PermissionEvent.PERMISSION_STATUS, onPermissionStatusChange);
				_camera.requestPermission();
				return;
			}
			
			setCameraMode();
		}
		
		public function stopCamera(getBitmapData:Boolean):BitmapData{
			
			if(!_camera) return null;
			
			var bitmapData:BitmapData = null;
			if(getBitmapData){
				
				bitmapData = new BitmapData(_camera.width, _camera.height, false);
				_camera.drawToBitmapData(bitmapData);
				
				if(_camera.width!=1920 || _camera.height!=1080){
					var scaleX:Number = 1920/_camera.width;
					var scaleY:Number = 1080/_camera.height;
					bitmapData = BitmapDataUtils.scaleBitmapDataXY(bitmapData,scaleX,scaleY, true);
				}
			
				var texture:Texture = Texture.fromBitmapData(bitmapData);
				updateBackground(texture);
				
			}
			
			
			if(_handleCameraTexture){
				
				_handleCameraTexture.root.attachCamera(null);
				_handleCameraTexture.dispose();
				_handleCameraTexture = null;
			}
			
			if(_backgroundFromCamera){
				
				_backgroundFromCamera.removeFromParent(true);
				_backgroundFromCamera = null;
			}
			
			
			
			_camera = null;
			
			return bitmapData;
			
		}

		public function uploadCustomBackground(bitmapData:BitmapData):BitmapData{
			if(bitmapData.width!=1920 || bitmapData.height!=1080){
				var scaledBitmapData:BitmapData = new BitmapData(1920, 1080, false);
				var m:Matrix = new Matrix();
				var sc:Number;
				var tx:Number = 0;
				var ty:Number = 0;
				if(bitmapData.width/bitmapData.height >= 1920/1080){
					sc = 1080 / bitmapData.height;
					tx = (1920 - bitmapData.width * sc)/ 2;
				}else{
					sc = 1920 / bitmapData.width;
					ty = (1080 - bitmapData.height * sc) / 2;
				}
				m.scale(sc, sc);
				m.tx = tx;
				m.ty = ty;
				scaledBitmapData.draw(bitmapData, m);
				bitmapData.dispose();
				bitmapData = scaledBitmapData;
				//
				/*var scaleX:Number = 1920/bitmapData.width;
				var scaleY:Number = 1080/bitmapData.height;
				bitmapData = BitmapDataUtils.scaleBitmapDataXY(bitmapData,scaleX,scaleY, true);*/
			}
			var texture:Texture = Texture.fromBitmapData(bitmapData);
			updateBackground(texture);
			return bitmapData;
		}
		
		
		override protected function initialize():void{
			super.initialize();
			_background ||= new Image(Texture.fromColor(100,100,0xf7f7f7));
			addChild(_background);
		}
		
		override public function dispose():void{
			super.dispose();
			if(_background){
				if(_background.texture){
                    _background.texture.dispose();
                    _background.texture = null;
				}
                _background.removeFromParent(true);
                _background = null;
			}
			if(_textureList){
				for(var i:int = 0; i < _textureList.length; i++){
                    _textureList[i].dispose();
                    _textureList[i] = null;
				}
                _textureList = null;
			}
		}
		
		public function updateBackground(texture:Texture, disposeOld:Boolean = true):void{
			if(_background){
				if(_background.texture!=texture){
					if(disposeOld && _background.texture ){
						_background.texture.dispose();
					}else if(_background.texture){
                        _textureList.push(_background.texture);
					}
					_background.texture = texture;
					_background.readjustSize();
					invalidate(INVALIDATION_FLAG_ALL);
				}
			}
		}
		
		public function get texture():Texture{
			if(_background && _background.texture) return _background.texture;
			else return null;
		}
		
		public function set myScale(value:Number):void{
			_scale = value;
			this.invalidate(INVALIDATION_FLAG_SIZE);
		}

		override protected function draw():void{
			super.draw();
			if(this.isInvalid(INVALIDATION_FLAG_SIZE)){
				_background.width = this.actualWidth * _scale;
				_background.height = this.actualHeight * _scale;

				_background.x = (this.actualWidth - _background.width)/2;
				_background.y = ( this.actualHeight-_background.height)/2;
				
				
				if(_backgroundFromCamera){
					_backgroundFromCamera.width = _background.width;
					_backgroundFromCamera.height = _background.height;
				
					_backgroundFromCamera.x = _background.x;
					_backgroundFromCamera.y = _background.y;
				}
			}
		}

		public function get background():Image {
			return _background;
		}
	}
}