package com.despark.tuning3d.view.components.core
{
import com.despark.tuning3d.model.core.Constants;

import feathers.core.FeathersControl;

import flash.geom.Point;
import flash.geom.Rectangle;

import starling.animation.Transitions;
import starling.animation.Tween;
import starling.core.Starling;
import starling.display.Image;
import starling.display.Quad;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;
import starling.textures.Texture;

public class BrandMenuScreen extends StackedScreen
	{
		public var locked : Boolean = false;
		public var handleTouchable : Boolean;

		protected var tween : Tween;
		protected var tweening : Boolean;
		protected var menuBackground : Image; //Scale3Image

		private var startY : int;
		private var startHeight : int;
		private var beganOutside : Boolean;
		private var handleHitArea : Rectangle;
		private var touchPoint : Point;
		private var _handleTitle : String;
		private var touchMoved : Boolean;
		
		public function BrandMenuScreen() 
		{
			super('BrandMenuScreen');
			
			handleHitArea = new Rectangle();
			touchPoint = new Point();
			handleTouchable = true;
		}
		
		override protected function initialize() : void 
		{
			super.initialize();
			addEventListener(TouchEvent.TOUCH, touchHandler);     
		}
		
		override public function dispose():void
		{
			removeEventListener(TouchEvent.TOUCH, touchHandler);     
			super.dispose();
		}
		
		public function set handleTitle(value : String) : void 
		{
			_handleTitle = value;
			invalidate(FeathersControl.INVALIDATION_FLAG_DATA);
		}
		
		public function get handleTitle() : String
		{
			return _handleTitle;
		}
		
		protected function get handleTexture() : Texture 
		{
			return null;
		}
		
		protected function get handleWidth() : int 
		{
			return handleTexture.width;
		}
		
		protected function get menuBackgroundTexture() : Texture
		{
			return null;
		}
		
		protected function get menuHeight() : int
		{
			return 0;
		}
		
		protected override function draw() : void 
		{
			var sizeInvalid : Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE);
			
			if (sizeInvalid) 
			{
				this.width  = this.actualWidth
				this.height = this.actualHeight;
				
				_newFrameImageHeight = this.height;
			} 
			
			if (isInvalid("layout_menu_views")){
				layoutMenuViews();
			}
			
			var dataInvalid : Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_DATA);
			if (dataInvalid) {
				//handleTitleLabel.text = handleTitle;
			}
		}
		
		protected function layoutMenuViews() : void 
		{
			if (menuBackground)
			{
				menuBackground.width = actualWidth+1;
				menuBackground.y = int(this.height);    

				if (Constants.SCALE_FACTOR == 3.375){
					menuBackground.y = int(this.height) - 1;  
				}
				
				menuBackground.height = menuHeight;
			}
			
			checkAndInformForMenuStatus();
		}
		
		private var originalFrameImageHeight : int;
		
		protected var _newFrameImageHeight:Number=0;
		
		protected function touchHandler(event : TouchEvent) : void
		{
			return;
			const touch : Touch = event.getTouch(this);

			if (!touch)	return;

			if (touch.phase == TouchPhase.BEGAN)
			{
				originalFrameImageHeight = _newFrameImageHeight;
				touchMoved = false;

				touchPoint.setTo(touch.globalX, touch.globalY);

				if (!handleTouchable || !handleHitArea.containsPoint(touchPoint)){
					beganOutside = true;
					return;
				}

				startY = touch.globalY;
				startHeight = _newFrameImageHeight;
			}
			else if(touch.phase == TouchPhase.MOVED)
			{
				if (beganOutside) return;

				_newFrameImageHeight=int(Math.min(actualHeight, Math.max(actualHeight - menuHeight, startHeight + touch.globalY - startY)));
				checkAndInformForMenuStatus();

				this.invalidate("layout_menu_views");
				//layoutMenuViews()
			}
			else if(touch.phase == TouchPhase.ENDED)
			{
				if (beganOutside){
					beganOutside = false;
					return;
				}

				if (!tweening)
				{
					var dY : int = touch.globalY - (actualHeight - menuHeight / 2);

					touchMoved = originalFrameImageHeight - this.height != 0;

					if (!touchMoved && this.height == actualHeight){
						dY = -1;
					}
					else if(!touchMoved && this.height == actualHeight - menuHeight){
						dY = 1;
					}
					animate(dY);
				}
			}
		}

		protected function completeAnimation():void
		{
		}
		
		private var _targetObject:Object={height:0};
		
		private function animate(dY : int, resultHandler : Function = null, animated:Boolean=true) : void
		{
			tweening = true;
			
			if (dY < 0) 
			{	
				var time:Number=0.3;
				if (!animated){
					time = 0.01;
				}
				tween = new Tween(_targetObject, time, Transitions.LINEAR);
			}
			else
			{
				tween = new Tween(_targetObject, 0.3, Transitions.LINEAR);
			}
			
			tween.onComplete = function() : void 
			{
				tweening = false;
				checkAndInformForMenuStatus();
				
				if (resultHandler)
				{
					resultHandler();
					completeAnimation();
				}
			}
			
			tween.onUpdate = function() : void 
			{
				//checkAndInformForMenuStatus();
				_newFrameImageHeight = _targetObject.height;
				//(actualHeight - menuHeight)*tween.progress;
				invalidate("layout_menu_views");
				//layoutMenuViews();
			}
			
			if (dY < 0) 
			{
				_targetObject.height=_newFrameImageHeight;
				tween.animate("height", actualHeight - menuHeight);
			}
			else 
			{
				_targetObject.height=_newFrameImageHeight;
				tween.animate("height", actualHeight);
			}
			
			Starling.juggler.add(tween);
		}
		
		public function checkAndInformForMenuStatus() : void
		{
		}
		
		protected function menuHidden() : void
		{
			if (menuBackground)	menuBackground.visible = false;  
		}
		
		protected function menuShown() : void
		{
			if (menuBackground)	menuBackground.visible = true;
		}
		
		protected function closeMenu(animated : Boolean = false, resultHandler : Function = null) : void
		{
			if (animated){
				animate(1, resultHandler);
			}
			else 
			{
				_newFrameImageHeight = actualHeight;
				invalidate("layout_menu_views");
				//checkAndInformForMenuStatus();
			}
		}
		
		protected function openMenu(animated : Boolean = true) : void {
			animate(-1, null, animated);
		}
	}
}
