package com.despark.tuning3d.view.components.core {
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.view.core.HomeScreenPopUp;
    import com.utils.FontSelector;
    import com.utils.ImageUtils;

    import feathers.controls.Button;
    import feathers.controls.LayoutGroup;
    import feathers.controls.List;
    import feathers.controls.PageIndicator;
    import feathers.controls.PageIndicatorInteractionMode;
    import feathers.controls.ScrollBarDisplayMode;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.controls.text.TextFieldTextRenderer;
    import feathers.core.ITextRenderer;
    import feathers.data.ListCollection;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.VerticalAlign;
    import feathers.layout.VerticalLayout;

    import flash.text.TextFormat;
    import flash.text.TextFormatAlign;

    import starling.core.Starling;
    import starling.display.Canvas;

    import starling.display.DisplayObject;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.textures.Texture;

    public class HomeScreenButton extends LayoutGroup{
        private var _mainImageTexture : Texture;
        private var _topButton:Button;
        private var _tapToChoose:Button;
        private var _wide : Boolean;
        private var _newsList:List;
        private var _pageIndicator:PageIndicator;
        private var _newsItemGroup:LayoutGroup;
        private static var dashLineTexture:Texture;
        private var _width:Number;
        private var _screen:HomeScreenPopUp;
        private var _screenMode:String;
        public function HomeScreenButton(mainImageTexture : Texture) {
            this._mainImageTexture = mainImageTexture;
            this._wide = Constants.isWide;
            _width = 150;
            dashLineTexture ||= ImageUtils.getDashLine(_width, 1, 2, 1);
        }


        override protected function initialize():void {
            super.initialize();
            //
            var vLayout:VerticalLayout = new VerticalLayout();
            vLayout.horizontalAlign = HorizontalAlign.CENTER;
            vLayout.gap = 9;
            layout = vLayout;
            //
            initTopButton();
            initContent();
            initNewsList();
            _tapToChoose.addEventListener(Event.TRIGGERED, onButtonSelected);
            _topButton.addEventListener(Event.TRIGGERED, onButtonSelected);
            //
            // newsData = [{time:'08.04', car:"18' MERCEDES G-CLASS", msg: "NEW BODY KITS AVAILABLE"},{time:'08.04', car:"18' MERCEDES G-CLASS", msg: "NEW BODY KITS AVAILABLE"},{time:'08.04', car:"18' MERCEDES G-CLASS", msg: "NEW BODY KITS AVAILABLE"}];
            //

        }

        private function initContent():void {
            _tapToChoose = new Button();
            _tapToChoose.labelFactory = function ():ITextRenderer {
                var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
                textRenderer.textFormat = new TextFormat(FontSelector.getMonserratMedium(true), 7, 0x969696);
                textRenderer.textFormat.align = TextFormatAlign.CENTER;
                textRenderer.textFormat.italic = true;
                textRenderer.textFormat.bold = false;
                textRenderer.embedFonts = true;
                return textRenderer;
            };
            _tapToChoose.horizontalAlign = HorizontalAlign.CENTER;
            _tapToChoose.label = "tap to choose";
            _tapToChoose.addEventListener(Event.TRIGGERED, onButtonSelected);
            _tapToChoose.width = _topButton.width;
            _tapToChoose.height = 12;
            addChild(_tapToChoose);
        }

        private function initTopButton():void {
            var defaultIcon : Image = new Image(_mainImageTexture);
            var downIcon : Image = new Image(_mainImageTexture);
            downIcon.scale = defaultIcon.scale = HomeScreenPopUp.HOME_SCREEN_SCALE;
            //
            defaultIcon.color = 0xFF0000;
            downIcon.color = 0xFF0000;
            //
            _topButton = new Button();
            _topButton.defaultIcon = defaultIcon;
            _topButton.downIcon = downIcon;
            _topButton.horizontalAlign = HorizontalAlign.CENTER;
            _topButton.addEventListener(Event.TRIGGERED, onButtonSelected);
            //
            var btnSkin:Quad = new Quad(1,1,0xFFFFFF);
            btnSkin.alpha = 0;
            _topButton.defaultSkin = btnSkin;
            //
            _topButton.width = defaultIcon.width;
            _topButton.height = defaultIcon.height;
            width = _width;
            addChild(_topButton);
            _topButton.validate();
        }

        private function initNewsList():void {
            _newsItemGroup = new LayoutGroup();
            var newsLayout:VerticalLayout = new VerticalLayout();
            newsLayout.horizontalAlign = HorizontalAlign.CENTER;
            newsLayout.gap = 4;
            _newsItemGroup.layout = newsLayout;
            //
            var topBorder:Image = new Image(dashLineTexture);
            _newsItemGroup.addChild(topBorder);
            //

            _newsList = new List();
            var listLayout:HorizontalLayout = new HorizontalLayout();
            listLayout.gap = 0;
            listLayout.horizontalAlign = HorizontalAlign.CENTER;
            listLayout.verticalAlign = VerticalAlign.MIDDLE;

            _newsList.clipContent = true;
            _newsList.width = actualWidth;
            _newsList.height = 24;
            _newsList.minimumPageThrowVelocity = 1;
            _newsList.layout = listLayout;
            _newsList.snapToPages = true;
            _newsList.hasElasticEdges = false;
            _newsList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
            _newsList.horizontalScrollPolicy = ScrollPolicy.ON;
            _newsList.verticalScrollPolicy = ScrollPolicy.OFF;
            _newsList.dataProvider = new ListCollection([]);
            _newsList.itemRendererFactory = function ():IListItemRenderer {
                var renderer:HomeNewsItemRenderer = new HomeNewsItemRenderer();
                renderer.setSize(_newsList.width, _newsList.height);
                renderer.homeScreen = _screen;
                renderer.screenMode = _screenMode;
                return renderer;
            };
            restoreScrollHandling();
            _newsItemGroup.addChild(_newsList);
            //
            var bottomBorder:Image = new Image(dashLineTexture);
            _newsItemGroup.addChild(bottomBorder);
            //
            _pageIndicator = new PageIndicator();
            _pageIndicator.normalSymbolFactory = function ():DisplayObject {
                var img:Image = new Image(ButtonFactory.assetManager.getTexture("home_screen_page_indicator_def"));
                img.scale = ButtonFactory.assetManager.scaleFactor / 2;
                return img;
            };

            _pageIndicator.selectedSymbolFactory = function ():DisplayObject {
                var img:Image = new Image(AlertView.assetManager.getTexture("home_screen_page_indicator_sel"));
                img.scale = ButtonFactory.assetManager.scaleFactor / 2;
                return img;
            };

            _pageIndicator.gap = 9;
            _pageIndicator.horizontalAlign = HorizontalAlign.CENTER;
            _pageIndicator.verticalAlign = VerticalAlign.MIDDLE;


            _pageIndicator.selectedIndex = -1;
            _pageIndicator.pageCount = 0;
            _pageIndicator.interactionMode = PageIndicatorInteractionMode.PRECISE;
            _pageIndicator.addEventListener(Event.CHANGE, pageChangeHandler);
            _newsItemGroup.addChild(_pageIndicator);
            addChild(_newsItemGroup);
        }

        private function pageChangeHandler(event:Event):void {
            _newsList.removeEventListener(Event.SCROLL, slidesListScrollHandler);
            _newsList.scrollToPageIndex(_pageIndicator.selectedIndex, 0, 0.5);
            restoreScrollHandling();
        }

        private function restoreScrollHandling():void{
            if(_newsList.isScrolling){
                Starling.juggler.delayCall(restoreScrollHandling, 0.01);
            }else if(!_newsList.hasEventListener(Event.SCROLL, slidesListScrollHandler)){
                _newsList.addEventListener(Event.SCROLL, slidesListScrollHandler);
            }
        }

        protected function slidesListScrollHandler(event:Event):void {
            setPageIndicatorPage(_newsList.horizontalPageIndex);
        }

        private function setPageIndicatorPage(value:int):void{
            _pageIndicator.removeEventListener(Event.CHANGE, pageChangeHandler);
            _pageIndicator.selectedIndex = value;
            _pageIndicator.addEventListener(Event.CHANGE, pageChangeHandler);
        }

        private function onButtonSelected(event:Event):void {
            dispatchEventWith(Event.TRIGGERED);
        }


        override public function dispose():void {
            super.dispose();
        }
        override public function validate():void {
            if(_topButton){
                _topButton.validate();
            }
            if(_tapToChoose){
                _tapToChoose.validate();
            }
            if(_newsList){
                _newsList.validate();
            }
            if(_pageIndicator){
                _pageIndicator.validate();
            }
            if(_newsItemGroup){
                _newsItemGroup.validate();
            }
            height = NaN;
            super.validate();
        }

        public function set newsData(value:Array):void {
            _newsList.dataProvider = new ListCollection(value);
            _newsList.selectedIndex = _pageIndicator.selectedIndex = 0;
            _pageIndicator.pageCount = value.length;
        }

        public function set screen(value:HomeScreenPopUp):void {
            _screen = value;
        }

        public function set screenMode(value:String):void {
            _screenMode = value;
        }
    }
}
