package com.despark.tuning3d.view.components.core {
import com.despark.tuning3d.model.core.Device;
import com.despark.tuning3d.view.components.NotificationView.InAppNotificationPopUp;
import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.timeline.FeedData;
    import com.utils.FontSelector;

import feathers.controls.Button;
import feathers.controls.Label;
import feathers.controls.LayoutGroup;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.ITextRenderer;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.display.Image;
import starling.display.Quad;

import starling.display.Sprite;
import starling.textures.Texture;

public class MainMenuItem extends Sprite {
    private var _noteCountGroup:LayoutGroup;
    private var _noteCountLabel:Label;
    public function MainMenuItem(textureName:String, prop:Object = null, countable:Boolean = false, iconColor:int = -1) {
        super();
        var _wt:Number = 60;
        var _ht:Number = 30;
        var _tx:String = "";

        if (prop) {
            _wt = prop.w;
            _ht = prop.h;
            _tx = prop.t;
        }

        touchable = true;

        var buttonTexture:Texture = ButtonFactory.assetManager.getTexture(textureName);
        var buttonImage:Image = new Image(buttonTexture);
        if(iconColor != -1){
            buttonImage.color = iconColor;
        }

        var hitItem:Quad = new Quad(_wt, _ht, 0xffffff);
        hitItem.name = "background";
        hitItem.alpha = 0;
        addChild(hitItem);

        var iconButton:Button = new Button();
        iconButton.name = "icon";
        iconButton.width = iconButton.height = _ht;
        iconButton.defaultSkin = buttonImage;
        iconButton.y = 5;
        addChild(iconButton);

        var qd : Quad = new Quad(1, 1, 0xCCCCCC);
        qd.alpha = .1;
        var labelButton:MultilingualButton = new MultilingualButton();
        labelButton.downSkin = qd;
        labelButton.name = "label";
        labelButton.label = _tx;
        labelButton.labelFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMonserratSemiBold(), 15, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            textRenderer.textFormat.leftMargin = 0;
            textRenderer.textFormat.letterSpacing = -0.1;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        labelButton.y = 0;
        labelButton.height = _ht + 20;
        labelButton.y = 0;
        labelButton.x = iconButton.x + iconButton.width + 14;
        if(Device.isIphoneXVariant()){
            labelButton.x-= 9;
        }
        addChild(labelButton);

        iconButton.y = (labelButton.height - iconButton.height) / 2;

        var touchButton:MultilingualButton = new MultilingualButton();
        touchButton.downSkin = qd;
        touchButton.name = "touch";
        touchButton.label = "";
        touchButton.y = 0;
        touchButton.width = _wt - 2;
        touchButton.height = _ht + 20;
        touchButton.paddingLeft = 0;
        touchButton.labelOffsetX = 10;
        touchButton.y = 0;
        addChild(touchButton);

        var seperator:Quad = new Quad(_wt, 1.2, 0xffffff);
        seperator.alpha = .55;
        seperator.y = labelButton.y + labelButton.height + 1;
        seperator.x = iconButton.x - 2;
        addChild(seperator);
        if(countable){
            _noteCountGroup = InAppNotificationPopUp.generateNotificationIcon();
            _noteCountGroup.visible = false;
            _noteCountGroup.x = 16;
            _noteCountGroup.y = 4;
            addChild(_noteCountGroup);
            for(var i:int = 0; i < _noteCountGroup.numChildren; i++){
                if(_noteCountGroup.getChildAt(i) is Label){
                    _noteCountLabel = _noteCountGroup.getChildAt(i) as Label;
                    break;
                }
            }
        }
    }

    public function set noteCount(val:int):void{
        if(_noteCountGroup) {
            _noteCountGroup.visible = (val > 0);
            if(_noteCountLabel)  {
                _noteCountLabel.text = FeedData.getNumberString(val);
            }
        }
    }

    public function get noteCount():int{
        if(_noteCountLabel)  {
            return parseInt(_noteCountLabel.text);
        }
        return 0;
    }
}
}
