
package com.despark.tuning3d.view.components.core 
{
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.Multilingual;
import com.utils.FontSelector;

import feathers.controls.Button;
import feathers.core.FeathersControl;

import flash.geom.Rectangle;
import flash.text.TextFormat;

import starling.display.Image;
import starling.events.Event;
import starling.textures.Texture;

public class MultilingualToggleSwitch extends FeathersControl implements Multilingual
	{
		public var textFormat : TextFormat;
		public var isHTML : Boolean;
		
		public var leftIsSelected: Boolean;
		private var _leftTitle: String;
		private var _rightTitle: String;
		private var _topTitle: String;
		
		private var leftLabel : MultilingualButton;
		private var rightLabel : MultilingualButton;
		private var topLabel : MultilingualButton
		
		private var thumb : Button;
		
		private var button:HitExtButton;
		
		private var blackBackground:Image;
		
		private var isAnimating: Boolean = false;
		
		public function set leftTitle(value:String):void
		{
			_leftTitle = value;
			invalidate("leftTitle");
		}
		
		public function set rightTitle(value:String):void
		{
			_rightTitle = value;
			invalidate("rightTitle");
		}
		
		public function set topTitle(value:String):void
		{
			_topTitle = value;
			invalidate("topTitle");
		}
		
		
		public function updateWithLanguage(language : Language) : void
		{
			leftLabel.updateWithLanguage(language);
			rightLabel.updateWithLanguage(language);
			topLabel.updateWithLanguage(language);
		}
		
		public function MultilingualToggleSwitch(__leftIsSelected: Boolean)
		{
			super();
			leftIsSelected = __leftIsSelected;
			setSizeInternal(26, 40, false);
		}
		
		protected override function initialize() : void
		{
			super.initialize();
			
			button = new HitExtButton();//ButtonFactory.defaultButton2(null, 0x0);//0xED3F25
			button.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
			addChild(button);
			
			var buttonTexture : Texture = ButtonFactory.assetManager.getTexture("button2");
			//var buttonTextures : Scale3Textures = new Scale3Textures(buttonTexture, 14, 15, Scale3Textures.DIRECTION_HORIZONTAL);
			var buttonImage :Image = new Image(buttonTexture);
			
			buttonImage.scale9Grid = new Rectangle(14,0,15, buttonTexture.frameHeight);
			buttonImage.color = 0x0;
			
			blackBackground = buttonImage;
			blackBackground.touchable = false;
			addChild(blackBackground);
			
			thumb = ButtonFactory.defaultButton2(null, 0xb01313);
			thumb.touchable = false;
			addChild(thumb);
			
			leftLabel = new MultilingualButton();
			leftLabel.touchable = false;
			leftLabel.textFormat = new TextFormat(FontSelector.getFont(), 7, 0xFEFFFF);
			
			addChild(leftLabel);
			
			rightLabel = new MultilingualButton();
			rightLabel.touchable = false;
			rightLabel.textFormat = new TextFormat(FontSelector.getFont(), 7, 0xFEFFFF);
			
			addChild(rightLabel);
			
			topLabel = new MultilingualButton();
			topLabel.touchable = false;
			topLabel.textFormat = new TextFormat(FontSelector.getFont(), 8, 0xFEFFFF);
			topLabel.alpha = 0.7;
			topLabel.height = 12;
			addChild(topLabel);
		}
		
		private function buttonTriggeredHandler(event : Event) : void
		{
			if(isAnimating){
				return;
			}
			isAnimating = true;
			stage.addEventListener(Event.ENTER_FRAME,updateThumb);
			
		}
		
		
		protected function updateThumb(e:Event):void {
			
			var offset: Number = 10;
			
			if(!isAnimating){
				return;
			}
			
			if(leftIsSelected){
				thumb.x +=offset;
				
				if(thumb.x>=thumb.width){
					thumb.x = thumb.width;
					isAnimating = false;
					leftIsSelected = false;
					dispatchEventWith(Event.CHANGE);
				}
				
			}else{
				thumb.x -=offset;
				
				if(thumb.x<=0){
					thumb.x = 0;
					isAnimating = false;
					leftIsSelected = true;
					dispatchEventWith(Event.CHANGE);
				}
			}
		}
		
		
		protected override function draw() : void
		{
			super.draw();
			
			if(isInvalid("leftTitle"))
			{
				leftLabel.label = _leftTitle;
			}
			
			if(isInvalid("rightTitle"))
			{
				rightLabel.label = _rightTitle;
			}
			
			if(isInvalid("topTitle"))
			{
				topLabel.label = _topTitle;
			}
			
			if( !isAnimating && isInvalid(INVALIDATION_FLAG_SIZE) || isInvalid("leftTitle") || isInvalid("rightTitle") )//
			{
				button.y = 0;
				button.height = actualHeight;
				button.width = actualWidth;
				
				//
				topLabel.width = actualWidth;
				topLabel.x = 0;
				topLabel.y = 0;
				
				
				var topOffset: int = topLabel.height;
				
				//
				blackBackground.y = topOffset;
				blackBackground.height = actualHeight - topOffset;
				blackBackground.width = actualWidth;
				
				leftLabel.y = topOffset;
				leftLabel.x = 0;
				leftLabel.width = actualWidth/2;
				leftLabel.height = actualHeight - topOffset;
				
				rightLabel.y =topOffset;
				rightLabel.x = actualWidth/2;
				rightLabel.width = actualWidth/2;
				rightLabel.height = actualHeight - topOffset;
				
				thumb.width = actualWidth/2;
				thumb.height = actualHeight - topOffset;
				thumb.y = topOffset;
				if(leftIsSelected){
					thumb.x = 0;
				}else{
					thumb.x = actualWidth/2;
				}
			}
		}
	}
}

