package com.despark.tuning3d.view.components.core
{
import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.view.BrandsScreenMediator;
import com.utils.BitmapDataUtils;

import feathers.controls.ScreenNavigator;
import feathers.controls.ScreenNavigatorItem;
import feathers.events.FeathersEventType;

import flash.display.BitmapData;
	import flash.utils.getDefinitionByName;
	import flash.utils.setTimeout;

import starling.display.Image;
import starling.events.Event;
import starling.textures.Texture;

public class StackedScreenNavigator extends ScreenNavigator
	{
		public static const SCREEN_PUSHED : String = "ScreenPushed";
		public static const SCREEN_POPPED : String = "ScreenPopped";

        public static const BACK_TO_SCREEN : String = "BackToScreen";
		
		public var inTransition : Boolean;
		
		private var stack : Array;
		private var popSelected : Boolean;
		private var popToRootSelected : Boolean;
		private var popToRootLevel : int;
		private var selectedScreen : StackedScreen;
		
		public static var POP_TO_ROOT_FROM_REGISTER:Boolean = false;
		
		public function StackedScreenNavigator()
		{
			super();
			
			stack = [];
			
			addEventListener(FeathersEventType.TRANSITION_START, transitionStartHandler);
		}
		
		public function pushScreen(screen : StackedScreen, show : Boolean = true, deleteOthers : Boolean = true) : void
		{

			if(!inTransition)
			{
				screen.hasBackButton = stack.length > 0;
				stack.push(screen);
				dispatchEventWith(SCREEN_PUSHED, false, screen);
				if(deleteOthers)
				{
                    selectedScreen = screen;
				}

				Tracer.print("pushScreen: " + stack[stack.length-1].name);

                addScreen(screen.screenName, new ScreenNavigatorItem(screen));
				showScreen(screen.screenName);
				
				if(show)
				{
					screen.ShowDone();
				}
			}
		}
		
		public function popScreen() : void
		{
			if (!inTransition && stack.length > 1)
			{
				popSelected = true;
				
				var screen : StackedScreen = stack[stack.length - 2];
				showScreen(screen.screenName);
				screen.ShowDone();
			}
		}

		public function screenTypeExists(classType:Class):Boolean{
			for(var i:int = 0; i < stack.length; i++){
				if(stack[i] is classType){
					return true;
				}
			}
			return false;
		}
		
		public function get stackSize() : int 
		{
			return stack.length;
		}
		
		private var _blockerTexture:Texture = null;
		private var _blockerImage:Image = null;
		private var _blockerView:BlockerView = null;
		
		public function showBlocker():void{
			try {
                var bitmapData:BitmapData = this.stage.drawToBitmapData();

                if (bitmapData.width > 2048 || bitmapData.height > 2048) {
                    var ts:Number = 2048 / bitmapData.width;
                    var newBitmapData:BitmapData = BitmapDataUtils.scaleBitmapData(bitmapData, ts);
                    bitmapData.dispose();
                    bitmapData = newBitmapData;
                }


                _blockerTexture = Texture.fromBitmapData(bitmapData, false, false, 1);
                bitmapData.dispose();
                _blockerImage = new Image(_blockerTexture);

                _blockerView = new BlockerView();

                _blockerView.width = stage.width;
                _blockerView.height = stage.height;

                _blockerImage.width = stage.width;
                _blockerImage.height = stage.height;


                _blockerView.addImage(_blockerImage);
                stage.addChild(_blockerView);
            }catch (e:Error) {
				
            }
			
		}
		
		public function hideBlocker():void{
			
			if(_blockerImage){
				_blockerImage.removeFromParent(true);
				_blockerImage = null;
			}
			
			if(_blockerTexture){
				_blockerTexture.dispose();
				_blockerTexture=null;
			}
			
			if(_blockerView){
				
				_blockerView.removeFromParent(true);
				_blockerView=null;
			}
		}
		
		
		public function popToRootScreen(level : int = 0) : void
		{
			if (!inTransition && stack.length > level + 1) 
			{
				popToRootSelected = true;
				popToRootLevel = level;
				
				if(!POP_TO_ROOT_FROM_REGISTER)
				showBlocker();
				//blockerView:BlockerView=new BlockerView();
				
				var screen : StackedScreen = stack[level];
				BrandsScreenMediator.SECOND_LOAD_COMPLETE = false;
				showScreen(screen.screenName);
				screen.ShowDone();
				
				
				if(!POP_TO_ROOT_FROM_REGISTER)
				setTimeout(checkRootLoaded, 200, screen);

			}
		}
		
		private function checkRootLoaded(screen : StackedScreen):void{
			
			if(BrandsScreenMediator.SECOND_LOAD_COMPLETE){
				hideBlocker();
				//showScreen(screen.screenName);
			}else{
				//trace("BrandsScreenMediator.SECOND_LOAD_COMPLETE:", BrandsScreenMediator.SECOND_LOAD_COMPLETE);
				setTimeout(checkRootLoaded, 200, screen);
			}
		}
		
		private function transitionStartHandler(event : Event) : void
		{
			inTransition = true;
		}
		
		override protected function transitionComplete(cancelTransition:Boolean=false) : void  //fixed by maxb add param cancelTransition:Boolean=false
		{
			super.transitionComplete();
			
			

			if(selectedScreen)
			{
				for(var i : int = 0; i < stack.length; i++)
				{
					if(!stack[i].isRoot && stack[i] != selectedScreen)
					{
						Tracer.print("removing screen - " + stack[i].name, "StackedScreenNavigator");
						removeScreenFromStack(i);

						i--;
					}
				}
                selectedScreen = null;
			}
			if(popSelected) 
			{
				removeLastScreenFromStack();
			}
			else if(popToRootSelected) 
			{
				while(stack.length > popToRootLevel + 1)
				{
					removeLastScreenFromStack();
				}
			}
			
			inTransition = false;
			popSelected = false;
			popToRootSelected = false;
		}

		private function removeScreenFromStack(index : int) : void
		{
			if(index < 0 && index >= stack.length)
			{
				Tracer.print("Invalid screen index", "StackedScreenNavigator");
				return;
			}
            var screen : StackedScreen = stack[index];

            removeScreen(screen.screenName);
            stack.removeAt(index);
            dispatchEventWith(SCREEN_POPPED, false, screen);

            screen=null;
		}
		
		private function removeLastScreenFromStack() : void
		{
			var screen : StackedScreen = stack[stack.length - 1];
			
			removeScreen(screen.screenName);
			stack.pop();
			dispatchEventWith(SCREEN_POPPED, false, screen);
			
			screen=null;
		}
	}
}