package com.despark.tuning3d.view.components.core
{
	import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.core.images.BitmapDataUtil;
	
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.filesystem.File;
	import flash.net.URLRequest;
	import flash.utils.ByteArray;
	
	import deng.fzip.FZip;
	import deng.fzip.FZipErrorEvent;
	import deng.fzip.FZipFile;
	

	import starling.events.EventDispatcher;
	
	public class ZipManager extends EventDispatcher
	{
		private static var ref:ZipManager=null;
		private var fileDataList:Object=null;
		private var urlRequest:URLRequest;
		private var zipsList:Object = null;
		private var fileLoaded:int=0;
		private var _isEncrypted:Boolean = true;
		private var _useForThisCar:Boolean = false;
		public var useZipSplitCam: Boolean = false;

		public function ZipManager()
		{
			urlRequest = new URLRequest();
			fileDataList = {};
			zipsList = {};
			super();
		}
		
		public static function getInctance():ZipManager{
			
			if(!ref) { 
				ref = new ZipManager();
			}
			return ref;
		}
		
		public function set useForThisCar(value:Boolean):void{
			_useForThisCar = value;
		}
		
		public function get useForThisCar():Boolean{
			return _useForThisCar;
		}
		
		public function set isEncrypted(value:Boolean):void{
			_isEncrypted = value;
		}
		
		public function get isEncrypted():Boolean{
			return _isEncrypted;
		}
		
		public function getFileDataByPath(url:String):Object{
			
			if(fileDataList[url]) {
				return fileDataList[url];
			}
			
			var zipFileUrl:String=fileUrlToZipUrl(url);
			var zipData:Object = zipsList[zipFileUrl];
			
			if(!zipData){
				
				zipData = {loading:true, error:false};
				var fzip:FZip = new FZip();
				fzip.addEventListener(Event.COMPLETE, onZipComplete);
				fzip.addEventListener(IOErrorEvent.IO_ERROR, onZipError);
				fzip.addEventListener(FZipErrorEvent.PARSE_ERROR, onZipParseError);
				
				zipData.zip = fzip;
				zipData.url = zipFileUrl;
				zipsList[zipFileUrl] = zipData;
				var zipRealUrl: String = Constants.IMAGES_URL+zipFileUrl.replace(File.cacheDirectory.url + '/cars/', '/cars/') + '?v=3';
				trace('zipRealUrl' + zipRealUrl);
				fzip.load(new URLRequest(zipRealUrl));
				return {content:null, error:false};
			}else if(zipData.error){
				return {content:null, error:true};
			}else{
				return {content:null, error:false};
			}
			
		}
		
		private function onZipParseError(event:FZipErrorEvent):void{
			
			trace("onZipParseError");
			var fzip:FZip = event.target as FZip;
			
			var zipUrl:String=getZipUrlByFZip(fzip);
			var zipData:Object = zipsList[zipUrl];
			if(zipData) {
				zipData.error = true;
				zipData.loading = false;
			}
			
			fzip.removeEventListener(Event.COMPLETE, onZipComplete);
			fzip.removeEventListener(IOErrorEvent.IO_ERROR, onZipError);
			fzip.removeEventListener(FZipErrorEvent.PARSE_ERROR, onZipParseError);
		}
		
		private function onZipError(event:IOErrorEvent):void{
			
			trace("onZipError");
			var fzip:FZip = event.target as FZip;
			
			var zipUrl:String =getZipUrlByFZip(fzip);
			var zipData:Object = zipsList[zipUrl];
			if(zipData) {
				zipData.error = true;
				zipData.loading = false;
			}
			
			fzip.removeEventListener(Event.COMPLETE, onZipComplete);
			fzip.removeEventListener(IOErrorEvent.IO_ERROR, onZipError);
			fzip.removeEventListener(FZipErrorEvent.PARSE_ERROR, onZipParseError);
		}
		
		private function getZipUrlByFZip(fzip:FZip):String{
			
			for (var url:String in zipsList){
				
				var zipData:Object = zipsList[url];
				
				if(zipData.zip==fzip){
					
					return zipData.url;
				}
			}
			return null;
		}
		
		private function onZipComplete(event:Event):void{
			
			var fzip:FZip = event.target as FZip;
			var fZipFile:FZipFile;
			fzip.removeEventListener(Event.COMPLETE, onZipComplete);
			fzip.removeEventListener(IOErrorEvent.IO_ERROR, onZipError);
			fzip.removeEventListener(FZipErrorEvent.PARSE_ERROR, onZipParseError);
			
			var zipUrl:String =getZipUrlByFZip(fzip);
			var zipData:Object = zipsList[zipUrl];
			if(!zipData)
			{
                Tracer.print("Couldn't find zip data object", "ZIP MANAGER");
                return;
            }


			var baseUrl: String;
			if (!useZipSplitCam) {
				baseUrl = zipUrl.replace(".zip", "/");
			} else  {
				baseUrl = zipUrl.replace(".zip", "/").replace('_1', '').replace('_2', '');

			}

			var filename:String ="";
			var filePath:String = "";
			var file:File;
			var comps:Array;
			
			for(var i:int=0; i<fzip.getFileCount(); i++){
				
				fZipFile = fzip.getFileAt(i);
				
				if(fZipFile.filename.indexOf(".png")<0) continue;
				
				comps =  fZipFile.filename.split("/");
				
				if(comps.length>1) filename = comps[comps.length-1];
				else
				filename = fZipFile.filename;
				
				filePath = "Cars/" + baseUrl+filename;
				
				file = File.cacheDirectory.resolvePath(filePath);
				file.preventBackup = true;
				
				BitmapDataUtil.saveImageBytes(fZipFile.content, file);
				
				if(isEncrypted){
					BitmapDataUtil.decryptImageBytes(fZipFile.content, baseUrl+filename,null);
				}
				
				fileDataList[baseUrl+filename]= {content:fZipFile.content, error:false};
			}
			
			fileLoaded+=fzip.getFileCount();
			zipData.loading = false;
			fzip.close();
		}
		
		private function fileUrlToZipUrl(url:String):String{
			
			var comps:Array=url.split("/");
			var newUrl:String="";
			var i:int=0;
			if(comps){
				
				for(i=0; i<comps.length-1; i++){
					
					newUrl+=comps[i];
					if(i!=comps.length-2){
						newUrl+="/";
					}else{
						if (useZipSplitCam) {
							if (url.indexOf('a1') == -1) {
								newUrl += '_1.zip';
							} else {
								newUrl += '_2.zip';
							}
						} else{
							newUrl+=".zip";
						}

					}
				}
			}
			
			return newUrl;
		}
		
		
		public function reset():void{
			
			//trace("ZipManager reset")
			fileDataList=null;
			fileDataList={};
			
			zipsList = null;
			zipsList = {};
			
			fileLoaded=0;
		}
		
	}
}