package com.despark.tuning3d.view.components.downloads
{
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.model.parts.PartSlot;
	import com.despark.tuning3d.view.components.core.MultilingualCheck;
	import com.utils.FontSelector;
	
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	import feathers.controls.Check;
	import feathers.controls.Label;
	import feathers.controls.text.BitmapFontTextRenderer;
	import feathers.controls.text.TextFieldTextRenderer;
	import feathers.core.FeathersControl;
	import feathers.core.ITextRenderer;
	import feathers.skins.ImageStateValueSelector;
	import feathers.text.BitmapFontTextFormat;
	
	import starling.display.DisplayObject;
	import starling.display.Image;
	import starling.display.Quad;
	import starling.events.Event;
	import starling.text.TextField;
	import starling.utils.AssetManager;

	public class DownloadsPartSlotView extends FeathersControl implements Multilingual
	{
		public static const SELECTION_STATE_CHANAGED : String = "SelectionChanged";
		
		public var assetManager : AssetManager;
		public var isSelectedForDownload : Boolean;
		public var fontSize : int = 7;
		public var bias : int;
		
		private var currentLanguage : Language;
		private var check : MultilingualCheck;
		private var sizeLabel : Label;
		private var _partSlot : PartSlot;
		private var q:Quad;
		private var testRest : Rectangle = new Rectangle();
		
		public function get partSlot() : PartSlot
		{
			return _partSlot;
		}

		public function set partSlot(value : PartSlot) : void
		{
			_partSlot = value;
			
			invalidate("partSlot");
		}
		
		public function updateWithLanguage(language : Language) : void
		{
			currentLanguage = language;
		}

		override protected function initialize() : void
		{
			super.initialize();
			
			q = new Quad(1, 1, 0x00FF00);
			q.alpha = 0.2;
			//addChild(q);
			q.touchable = false;
			var currentY : int = 1;
			
			var iconSelector : ImageStateValueSelector = new ImageStateValueSelector();
			iconSelector.defaultValue = assetManager.getTexture("check");
			iconSelector.defaultSelectedValue = assetManager.getTexture("checkSelected");
			
			check = new MultilingualCheck();
			check.labelOffsetX = 5;
			check.stateToIconFunction = iconSelector.updateValue;
			check.textFormat = new TextFormat(FontSelector.getFont(), fontSize, 0x222728);
			check.textFormat.align = TextFormatAlign.LEFT;
			check.defaultIcon = new Image(assetManager.getTexture("check"));
			check.selectedUpIcon = new Image(assetManager.getTexture("checkSelected"));
			check.selectedDownIcon = check.selectedUpIcon;
			check.addEventListener(Event.CHANGE, checkChangeHandler);
		
			check.x = 0;
			addChild(check);
			
			sizeLabel = new Label();
			
			sizeLabel.textRendererFactory =  function() : ITextRenderer
			{
				/*var textRenderer:BitmapFontTextRenderer = new BitmapFontTextRenderer();
				
				textRenderer.textFormat = new BitmapFontTextFormat(TextField.getBitmapFont("openSans"), fontSize, 0x96999B);
				textRenderer.textFormat.horizontalAlign = "right";
				textRenderer.useSeparateBatch = false;*/
				
				var textRenderer : TextFieldTextRenderer = new TextFieldTextRenderer();
				
				textRenderer.textFormat = new TextFormat(FontSelector.getFont(), fontSize, 0x96999B);
				textRenderer.textFormat.align = TextFormatAlign.RIGHT;
				textRenderer.embedFonts = Language.supportsEmbeddedFont;
				
				return textRenderer;
			};
			addChild(sizeLabel);
		}
		
		public function refresh() : void
		{
			if(partSlot)
			{
				invalidate("partSlot");
			}
		}
		
		override protected function draw() : void
		{
			if(isInvalid(INVALIDATION_FLAG_SIZE))
			{
				sizeLabel.width = 55;
				sizeLabel.x = actualWidth - sizeLabel.width;
				sizeLabel.y = int((actualHeight + bias - 11) / 2);
				
				q.width = actualWidth;
				q.height = actualHeight;
				
				check.y = sizeLabel.y;
			}
			
			if(isInvalid("partSlot") && partSlot)
			{
				var mb:Number=0;
				mb=partSlot.sizeInMegabytes;
				
				
				if(mb>1024){
					mb=((mb/1024)/1024);
				}
				
				/*if(partSlot.percentLoaded == 0)
				{
									
					sizeLabel.text = partSlot.possibleParts.length+"";//mb.toFixed(2)+" MB";
				}
				else
				{*/
					
				sizeLabel.text = (partSlot.percentLoaded * partSlot.possibleParts.length).toFixed(0) + " / " + partSlot.possibleParts.length.toFixed(0);
				//}
					
				check.label = currentLanguage.translate("componentTypes", partSlot.name).toUpperCase();
				
				if(partSlot.nameSuffix)
				{
					check.label += partSlot.nameSuffix;
				}
				
				check.label += "    ";
				
				check.updateWithLanguage(currentLanguage);
			}
		}
		
		private function checkChangeHandler(event : Event) : void
		{
			var check : Check = Check(event.target);	
			isSelectedForDownload = check.isSelected;
		
			dispatchEventWith(SELECTION_STATE_CHANAGED, true);
		}
		
		public override function hitTest(localPoint : Point) : DisplayObject
		{
			if(getBounds(this, testRest).containsPoint(localPoint))
			{
				return check;
			}
			
			return super.hitTest(localPoint);
		}
	}
}