package com.despark.tuning3d.view.components.gallery
{
    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.UIColors;
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.languages.Multilingual;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.model.tunings.TuningsProxy;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.IMainMenuHolder;
    import com.despark.tuning3d.view.components.core.MainMenuView;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.gallery.data.RequestPeriod;
    import com.despark.tuning3d.view.components.gallery.data.RequestSort;
    import com.despark.tuning3d.view.components.timeline.components.TimelineLoadBar;
    import com.despark.tuning3d.view.components.tunings.ITuningListMenuVIew;
    import com.distriqt.AdsManager;

    import feathers.controls.Button;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.controls.List;
    import feathers.controls.Radio;
    import feathers.controls.ScrollBarDisplayMode;
    import feathers.controls.ScrollPolicy;
    import feathers.controls.ToggleButton;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.core.ToggleGroup;
    import feathers.data.ListCollection;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.VerticalAlign;
    import feathers.layout.VerticalLayout;

    import flash.geom.Rectangle;
    import flash.utils.setTimeout;

    import starling.core.Starling;
    import starling.display.DisplayObjectContainer;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.display.Sprite;
    import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.filters.DropShadowFilter;
    import starling.textures.Texture;

    public class GalleryScreen extends StackedScreen implements Multilingual, IMainMenuHolder, ITuningListMenuVIew {
        private var _mainMenuView:MainMenuView;
        private var _selectedLanguage:Language;
        private var _languages:Array;
        private var menuButton:Button;
        private var _background:Quad;
        private var _bannerBackground:Image;
        private var _loadingBar: TimelineLoadBar;
        private var titleLabel:Label;

        private var viewToggleGroup:ToggleGroup;
        private var viewLayout:LayoutGroup;

        private var sortByToggleGroup:ToggleGroup;
        private var sortByLayout:LayoutGroup;
        private var backgroundButton:ToggleButton;

        private var showByButton:ToggleButton;
        private var showByLayout:LayoutGroup;
        private var showByMenu:Sprite;
        private var showByGroup:LayoutGroup;
        private var showByToggleGroup:ToggleGroup;

        private var showByCountryButton:ToggleButton;
        private var showByWorldButton:ToggleButton;
        private var showByRegionLayout:LayoutGroup;
        private var showByRegionToggleGroup:ToggleGroup;

        private var searchInput:SearchInput;

        private var tuningsList : List;
        private var _prevButton:Button;
        private var _nextButton:Button;

        private static const BANNER_SIZE:int = 47;
        private static const TOOLBAR_SIZE:int = 31;
        //
        private var _countPerPage:int = 4;
        private var _data:Array;
        private var _tunings:Array;
        private var _scrollToNextPage:Boolean;
        //
        private var _limit:int;
        private var _skip:int;
        private var _sortBy:String;
        private var _period:String;
        private var _userSignedIn:Boolean;
        private var _selectedTuning:Tuning;
        private var _timestamp:Number;
        private var _query:String;

        private var _tuningsProxy:TuningsProxy;
        private var _isLoading:Boolean;
        private var _noMoreTuning:Boolean;

        private var _noResultLabel:Label;
        private static var NO_RESULT_LABEL:String = 'No results found';

        private var sortByBorderList:Array;
        private var showByBorderList:Array;
        public var countryData:Object;
        private var isScrolling:Boolean = false;

        public function GalleryScreen() {
            super("GalleryScreen");
        }

        override public function dispose():void {
            super.dispose();

            if(viewToggleGroup){
                viewToggleGroup.removeEventListeners();
            }

            if(sortByToggleGroup){
                sortByToggleGroup.removeEventListeners();
            }

            if(showByRegionToggleGroup){
                showByRegionToggleGroup.removeEventListeners();
            }

            if(showByToggleGroup){
                showByToggleGroup.removeEventListeners();
            }

            if(_mainMenuView) {
                _mainMenuView.removeFromParent(true);
                _mainMenuView = null;
            }
            if(menuButton) {
                menuButton.removeFromParent(true);
                menuButton = null;
            }
            if(_background) {
                _background.removeFromParent(true);
                _background = null;
            }
            if(_bannerBackground) {
                _bannerBackground.removeFromParent(true);
                _bannerBackground = null;
            }
            if(_loadingBar) {
                _loadingBar.removeFromParent(true);
                _loadingBar = null;
            }
            if(titleLabel) {
                titleLabel.removeFromParent(true);
                titleLabel = null;
            }
            if(viewToggleGroup) {
                viewToggleGroup.removeAllItems();
                viewToggleGroup = null;
            }
            if(viewLayout) {
                viewLayout.removeFromParent(true);
                viewLayout = null;
            }

            if(sortByToggleGroup){
                sortByToggleGroup.removeAllItems();
                sortByToggleGroup = null;
            }
            if(sortByLayout){
                sortByLayout.removeFromParent(true);
                sortByLayout = null;
            }

            if(backgroundButton){
                backgroundButton.removeFromParent(true);
                backgroundButton = null;
            }

            if(showByButton){
                showByButton.removeFromParent(true);
                showByButton = null;
            }

            if(showByLayout){
                showByLayout.removeFromParent(true);
                showByLayout = null;
            }

            if(showByMenu){
                showByMenu.removeFromParent(true);
                showByMenu = null;
            }

            if(showByGroup){
                showByGroup.removeFromParent(true);
                showByGroup = null;
            }

            if(showByToggleGroup){
                showByToggleGroup.removeAllItems();
                showByToggleGroup = null;
            }

            if(searchInput){
                searchInput.removeFromParent(true);
                searchInput = null;
            }

            if(tuningsList){
                tuningsList.dataProvider = null;
                tuningsList.removeFromParent(true);
                tuningsList = null;
            }

            if(_prevButton){
                _prevButton.removeFromParent(true);
                _prevButton = null;
            }

            if(_nextButton){
                _nextButton.removeFromParent(true);
                _nextButton = null;
            }

            if(_noResultLabel){
                _noResultLabel.removeFromParent(true);
                _noResultLabel = null;
            }

            if(showByCountryButton){
                showByCountryButton.removeFromParent(true);
                showByCountryButton = null;
            }

            if(showByWorldButton){
                showByWorldButton.removeFromParent(true);
                showByWorldButton = null;
            }
            if(showByRegionLayout){
                showByRegionLayout.removeFromParent(true);
                showByRegionLayout = null;
            }
            if(showByRegionToggleGroup){
                showByRegionToggleGroup.removeAllItems();
                showByRegionToggleGroup = null;
            }

            if(stage){
                stage.removeEventListener(TouchEvent.TOUCH, stageTouchHandler);
            }

            _data = null;
            _tunings = null;
            _selectedTuning = null;
            _tuningsProxy = null;
        }

        public function updateWithLanguage(language : Language) : void {
            if (mainMenuView){
                mainMenuView.updateWithLanguage(language);
            }
        }

        public function updateNotifications() :void{
            if(mainMenuView){
                mainMenuView.updateNotificationCount();
            }
        }

        override protected function initialize() : void {
            super.initialize();
            _noMoreTuning = false;
            _timestamp = new Date().time/1000;

            _background = new Quad(1, 1, 0xFFFFFF);
            addChild(_background);

            _bannerBackground = new Image(Texture.fromColor(1,1, UIColors.BANNER_BACKGROUND_COLOR));
            addChild(_bannerBackground);

            titleLabel = new Label();
            titleLabel.textRendererFactory = GalleryLabelFactory.screenTitle;
            titleLabel.text = "GALLERY";
            titleLabel.validate();
            titleLabel.paddingBottom = 3;
            addChild(titleLabel);

            menuButton = ButtonFactory.menuBigButton("home_screen_main_menu", 0xFFFFFE);
            menuButton.width = 35;
            menuButton.height = 35;
            menuButton.addEventListener(Event.TRIGGERED, onMenuButton);
            addChild(menuButton);

            menuButton.x = menuButton.y = (BANNER_SIZE - menuButton.height) / 2;

            /*menuButton.x=6;
            menuButton.y=6;
            if(Device.isIphoneX()){
                menuButton.x+=6;
                menuButton.y+=6;
            }*/



            var layout : HorizontalLayout = new HorizontalLayout();
            layout.gap = 0;
            layout.horizontalAlign = HorizontalAlign.CENTER;
            layout.verticalAlign = VerticalAlign.MIDDLE;
            layout.useVirtualLayout = true;

            var screen:GalleryScreen = this;
            tuningsList = new List();
            tuningsList.minimumPageThrowVelocity=1;
            tuningsList.layout = layout;
            tuningsList.snapToPages = true;
            tuningsList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
            tuningsList.horizontalScrollPolicy = ScrollPolicy.ON;
            tuningsList.verticalScrollPolicy = ScrollPolicy.OFF;
            tuningsList.itemRendererFactory = function() : IListItemRenderer {
                var renderer : GridItemRenderer = new GridItemRenderer();
                renderer.count = _countPerPage;
                renderer.owner = tuningsList;
                var item_w:Number = actualWidth;
                var item_h:Number = (actualHeight - BANNER_SIZE - TOOLBAR_SIZE);
                renderer.setSize(item_w, item_h);
                renderer.eventDispatcher = screen;
                return renderer;
            };
            tuningsList.addEventListener(Event.SCROLL, tuningsListScrollHandler);
            addChild(tuningsList);

            _limit = 30;
            _sortBy = RequestSort.LIKES;
            _period = RequestPeriod.TODAY;

            createViewSection();
            createSortBySection();
            createShowBySection();
            createNavigationButtons();
            createSearchView();

            _loadingBar ||= new TimelineLoadBar();
            _loadingBar.name = '_timelineLoadBar';
            _loadingBar.setSize(60, 60);
            _loadingBar.x = Constants.STAGE_WIDTH / 2 - _loadingBar.width / 2;
            _loadingBar.y = Constants.STAGE_HEIGHT / 2 - _loadingBar.height / 2;
            _loadingBar.visible = false;
            addChild(_loadingBar);

            _scrollToNextPage = false;
            updateSortByBorders();
            updateShowByBorders();
            loadContent(refreshTuningsCompleteHandler);
        }

        private function tuningsListScrollHandler(event:Event):void {
            if(!isScrolling && tuningsList.isScrolling){
                isScrolling = true;
                SoundManager.playSound(SoundManager.SELECT_PART);
            }else if(!tuningsList.isScrolling){
                isScrolling = false;
            }
            if(!_noMoreTuning) {
                if (tuningsList.horizontalPageIndex == tuningsList.horizontalPageCount - 1 && !_loadingBar.visible && tuningsList.horizontalScrollPosition > tuningsList.maxHorizontalScrollPosition + 25) {
                    _scrollToNextPage = true;
                    loadContent(tuningLoadComplete, false);
                }
            }
            _nextButton.visible = !(_noMoreTuning && tuningsList.horizontalPageIndex == tuningsList.horizontalPageCount - 1);
            _prevButton.visible = (tuningsList.horizontalPageIndex > 0);
        }


        // *******
        private function loadContent(completeHandler:Function, refresh:Boolean = true):void{
            if(refresh){
                _timestamp = new Date().time/1000;
                _noMoreTuning = false;
            }
            if(_isLoading || _noMoreTuning){
                return;
            }
            _noResultLabel.visible = false;
            _loadingBar.visible = _isLoading = true;
            touchable = false;

            _limit = _countPerPage * int(10 / _countPerPage);
            if(refresh || !_tunings){
                _skip = 0;
            }else if(_tunings){
                _skip = _tunings.length;
            }
            _tuningsProxy ||= ApplicationFacade.getInstance().retrieveProxy(TuningsProxy.NAME) as TuningsProxy;
            _tuningsProxy.loadGalleryTunings(completeHandler, _timestamp, _limit, _skip, _sortBy, _period, backgroundButton.isSelected, _query, showByCountryButton.isSelected);
        }

        private function createNavigationButtons():void {
            var defaultSkin:Image;

            defaultSkin = new Image(ButtonFactory.assetManager.getTexture("gallery_dir_button"));
            defaultSkin.scale = ButtonFactory.assetManager.scaleFactor/2;
            _nextButton = new Button();
            _nextButton.defaultIcon = defaultSkin;

            defaultSkin = new Image(ButtonFactory.assetManager.getTexture("gallery_dir_button"));
            defaultSkin.scale = ButtonFactory.assetManager.scaleFactor/2;
            defaultSkin.alpha = 0.6;
            _nextButton.downIcon = defaultSkin;
            _nextButton.disabledIcon = defaultSkin;
            _nextButton.addEventListener(Event.TRIGGERED, nextButtonTriggeredHandler);

            defaultSkin = new Image(ButtonFactory.assetManager.getTexture("gallery_dir_button"));
            defaultSkin.scale = ButtonFactory.assetManager.scaleFactor/2;
            defaultSkin.scaleX *= -1;
            _prevButton = new Button();
            _prevButton.defaultIcon = defaultSkin;

            defaultSkin = new Image(ButtonFactory.assetManager.getTexture("gallery_dir_button"));
            defaultSkin.scale = ButtonFactory.assetManager.scaleFactor/2;
            defaultSkin.scaleX *= -1;
            defaultSkin.alpha = 0.6;
            _prevButton.downIcon = defaultSkin;
            _prevButton.disabledIcon = defaultSkin;

            _prevButton.iconOffsetX = defaultSkin.width;
            _prevButton.x = -defaultSkin.width;

            _prevButton.addEventListener(Event.TRIGGERED, prevButtonTriggeredHandler);

            addChild(_prevButton);
            addChild(_nextButton);
            _nextButton.visible = _prevButton.visible = false;
        }

        private function prevButtonTriggeredHandler(event:Event):void {
            if(tuningsList && tuningsList.horizontalPageIndex > 0){
                SoundManager.playSound(SoundManager.SELECT_PART_BACK);
                tuningsList.scrollToPageIndex(tuningsList.horizontalPageIndex - 1, 0, 0.3);
            }
        }

        private function nextButtonTriggeredHandler(event:Event):void {
            if(tuningsList) {
                SoundManager.playSound(SoundManager.SELECT_PART);
                if (tuningsList.horizontalPageIndex < tuningsList.horizontalPageCount - 1) {
                    tuningsList.scrollToPageIndex(tuningsList.horizontalPageIndex + 1, 0, 0.3);
                }else if(tuningsList.horizontalPageIndex == tuningsList.horizontalPageCount - 1){
                    _scrollToNextPage = true;
                    loadContent(tuningLoadComplete, false);
                }
            }
        }

        public function get userSignedIn():Boolean{
            return _userSignedIn;
        }

        public function set userSignedIn(value:Boolean):void {
            _userSignedIn = value;
            if (!owner)	return;
            invalidate("userSignedIn");
        }
        private function refreshTuningsCompleteHandler(value:Object):void{
            clearContent();
            setTimeout(function():void{
                tuningLoadComplete(value);
            }, 100);
        }

        private function tuningLoadComplete(value:Object):void{
            _tunings ||= [];
            _tunings = _tunings.concat(value);
            _data ||= [];
            var count:int = Math.min(value.length, _countPerPage);
            var ln:int = Math.ceil(value.length / _countPerPage);
            for(var i:int = 0; i < ln; i++){
                var items:Array = [];
                for(var j:int = 0; j < count; j++){
                    var k:int = i * count + j;
                    if(k < value.length && value[k]) {
                        var tuning:Tuning = TuningsProxy.parseTuning(value[k]);
                        if (value[k].hasOwnProperty('tuningId')) {
                            tuning.id = value[k].tuningId;
                        }
                        items.push(tuning);
                    }
                }
                _data.push(items);
            }
            if(tuningsList.dataProvider == null){
                tuningsList.dataProvider = new ListCollection(_data);
            }
            tuningsList.invalidate(INVALIDATION_FLAG_DATA);
            tuningsList.invalidate(INVALIDATION_FLAG_SCROLL);
            //
            if(_scrollToNextPage) {
                Starling.juggler.delayCall(function ():void {
                    if (tuningsList.horizontalPageIndex < tuningsList.horizontalPageCount) {
                        tuningsList.scrollToPageIndex(tuningsList.horizontalPageIndex + 1, 0, 0.5);
                    }
                }, 0.1);
                _scrollToNextPage = false;
            }

            _noMoreTuning = (value.length < _limit);

            Starling.juggler.delayCall(function():void{
                _loadingBar.visible = _isLoading = false;
                touchable = true;
                _noResultLabel.visible = (count == 0 && _tunings.length == 0);
                _nextButton.visible = !_noMoreTuning;
                _prevButton.visible = (tuningsList.horizontalPageIndex > 0);
            }, 0.05);
        }

        public function dispatchExternalEvent(msg:String):void {
            if (mainMenuView) {
                mainMenuView.hide();
            }
            dispatchEventWith(msg);
        }

        override public function ShowDone():void {
            BannerStackedScreen.showAdBanner();
        }

        private function menuOpenHandler(event:Event):void {
            BannerStackedScreen.hideAdBanner(false);
        }

        private function menuCloseHandler(event:Event):void {
            BannerStackedScreen.showAdBanner();
        }

        override protected function screen_removedFromStageHandler(event:Event):void{
            super.screen_removedFromStageHandler(event);
            BannerStackedScreen.hideAdBanner(true);
        }

        override protected function draw() : void {
            super.draw();
            if (!mainMenuView){
                mainMenuView = new MainMenuView();
                mainMenuView.name = "mainMenuView";
                mainMenuView.setSize(MainMenuView.MENU_WIDTH, actualHeight);
                mainMenuView.parentActualWidth = actualWidth;
                mainMenuView.parentActualHeight = actualHeight;
                addChild(mainMenuView);

                mainMenuView.addEventListener(MainMenuView.START_SHOW, menuOpenHandler);
                mainMenuView.addEventListener(Event.CLOSE, menuCloseHandler);
            }

            if(_background){
                _background.width = actualWidth;
                _background.height = actualHeight;
            }

            if(_bannerBackground){
                _bannerBackground.width = actualWidth;
                _bannerBackground.height = BANNER_SIZE;
            }

            if(titleLabel){
                titleLabel.x = 50;
                titleLabel.y = (BANNER_SIZE - titleLabel.height)/2;
            }

            if(viewLayout){
                viewLayout.x = actualWidth - viewLayout.width - 7.5;
                viewLayout.y = (BANNER_SIZE - viewLayout.height)/2;
            }

            if(sortByLayout){
                sortByLayout.x = 10;
                sortByLayout.y = BANNER_SIZE + (TOOLBAR_SIZE - (sortByLayout.height + HorizontalLayout(sortByLayout.layout).padding)) / 2;
            }

            if(showByLayout){
                showByLayout.x = actualWidth/2;
                showByLayout.y = BANNER_SIZE + (TOOLBAR_SIZE - (showByLayout.height + HorizontalLayout(showByLayout.layout).padding)) / 2;
                if(showByMenu){
                    showByMenu.x = actualWidth / 2;
                    showByMenu.y = BANNER_SIZE + TOOLBAR_SIZE;
                }

                if(showByRegionLayout){
                    showByRegionLayout.x = showByLayout.x + showByLayout.width;
                    showByRegionLayout.y = showByLayout.y;
                }
            }



            if(searchInput){
                searchInput.x = actualWidth - searchInput.width - 10;
                searchInput.y = BANNER_SIZE + (TOOLBAR_SIZE - searchInput.height) / 2;
            }

            if(tuningsList){
                tuningsList.width = actualWidth;
                tuningsList.height = actualHeight - (BANNER_SIZE + TOOLBAR_SIZE);
                tuningsList.x = 0;
                tuningsList.y = actualHeight - tuningsList.height;
            }

            if(_prevButton){
                _prevButton.x = 0;
                _prevButton.y = (BANNER_SIZE + TOOLBAR_SIZE)/2 + (actualHeight - _prevButton.height)/2;
            }

            if(_nextButton){
                _nextButton.x = actualWidth - _nextButton.width;
                _nextButton.y = (BANNER_SIZE + TOOLBAR_SIZE)/2 + (actualHeight - _nextButton.height)/2;
            }

            if(_noResultLabel){
                _noResultLabel.x = (actualWidth - _noResultLabel.width) / 2;
                if(tuningsList){
                    _noResultLabel.y = tuningsList.y + (tuningsList.height - _noResultLabel.height) / 2;
                }else{
                    _noResultLabel.y = (actualHeight - _noResultLabel.height) / 2;
                }
            }
        }

        private function createViewSection():void{
            viewLayout = new LayoutGroup();
            var hLayout:HorizontalLayout = new HorizontalLayout();
            hLayout.horizontalAlign = HorizontalAlign.RIGHT;
            hLayout.verticalAlign = VerticalAlign.MIDDLE;
            viewLayout.layout = hLayout;

            /*var viewLabel:Label = new Label();
            viewLabel.textRendererFactory = GalleryLabelFactory.sectionTitleLabel;
            viewLabel.paddingLeft = 2;
            viewLabel.paddingRight = 5;
            viewLabel.paddingBottom = 3;
            viewLabel.text = "View";

            viewLayout.addChild(viewLabel);*/

            viewToggleGroup = new ToggleGroup();

            var toggleButton:ToggleButton;
            var defaultIcon:Image;
            var selectedIcon:Image;

            toggleButton = new ToggleButton();
            toggleButton.paddingLeft = toggleButton.paddingRight = 2.5;
            toggleButton.name = "4";
            defaultIcon = new Image(ButtonFactory.assetManager.getTexture("gallery_grid_4"));
            defaultIcon.color = 0xFEFFFF;
            defaultIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            toggleButton.defaultIcon = defaultIcon;


            selectedIcon = new Image(ButtonFactory.assetManager.getTexture("gallery_grid_4"));
            selectedIcon.color = 0x51A1F2;
            selectedIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            toggleButton.defaultSelectedIcon = selectedIcon;
            toggleButton.isSelected = true;
            viewToggleGroup.addItem(toggleButton);
            viewLayout.addChild(toggleButton);

            toggleButton = new ToggleButton();
            toggleButton.paddingLeft = toggleButton.paddingRight = 2.5;
            toggleButton.name = "9";
            defaultIcon = new Image(ButtonFactory.assetManager.getTexture("gallery_grid_9"));
            defaultIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            defaultIcon.color = 0xFEFFFF;
            toggleButton.defaultIcon = defaultIcon;

            selectedIcon = new Image(ButtonFactory.assetManager.getTexture("gallery_grid_9"));
            selectedIcon.color = 0x51A1F2;
            selectedIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            toggleButton.defaultSelectedIcon = selectedIcon;
            viewToggleGroup.addItem(toggleButton);
            viewLayout.addChild(toggleButton);

            toggleButton = new ToggleButton();
            toggleButton.paddingLeft = toggleButton.paddingRight = 2.5;
            toggleButton.name = "1";
            defaultIcon = new Image(ButtonFactory.assetManager.getTexture("gallery_grid_1"));
            defaultIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            defaultIcon.color = 0xFEFFFF;
            toggleButton.defaultIcon = defaultIcon;

            selectedIcon = new Image(ButtonFactory.assetManager.getTexture("gallery_grid_1"));
            selectedIcon.color = 0x51A1F2;
            selectedIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            toggleButton.defaultSelectedIcon = selectedIcon;
            viewToggleGroup.addItem(toggleButton);
            viewLayout.addChild(toggleButton);

            viewToggleGroup.addEventListener(Event.CHANGE, viewChangeHandler);
            addChild(viewLayout);
        }

        private function clearContent():void{
            if(tuningsList.dataProvider){
                tuningsList.dataProvider.removeAll();
            }
            tuningsList.dataProvider = null;
            _data = _tunings = null;
        }

        private function viewChangeHandler(event:Event):void {
            if(viewToggleGroup.selectedItem){
                SoundManager.playSound(SoundManager.SELECT_PART);
                _countPerPage = parseInt(viewToggleGroup.selectedItem.name);
                if(_countPerPage){
                    loadContent(refreshTuningsCompleteHandler);
                }
            }
        }


        private function createSortBySection():void{
            var border:Quad;
            sortByBorderList = [];
            sortByLayout = new LayoutGroup();
            var hLayout:HorizontalLayout = new HorizontalLayout();
            hLayout.horizontalAlign = HorizontalAlign.RIGHT;
            hLayout.verticalAlign = VerticalAlign.MIDDLE;
            hLayout.gap = 0;
            hLayout.paddingTop = 3;
            sortByLayout.layout = hLayout;

            var viewLabel:Label = new Label();
            viewLabel.textRendererFactory = GalleryLabelFactory.sortByTitleLabel;
            viewLabel.paddingLeft = 2;
            viewLabel.paddingRight = 5;
            viewLabel.paddingBottom = 3;
            viewLabel.text = "Sort by";

            sortByLayout.addChild(viewLabel);

            sortByToggleGroup = new ToggleGroup();

            var toggleButton:ToggleButton;

            toggleButton = createSortByButton("gallery_eye", RequestSort.VIEWS);
            toggleButton.isSelected = true;
            sortByToggleGroup.addItem(toggleButton);
            sortByLayout.addChild(toggleButton);
            border = createBorder();
            sortByBorderList.push(border);
            border.height = toggleButton.defaultSkin.height;
            sortByLayout.addChildAt(border, 1);
            border = createBorder();
            sortByBorderList.push(border);
            border.height = toggleButton.defaultSkin.height;
            sortByLayout.addChild(border);
            if(_sortBy == RequestSort.VIEWS){
                toggleButton.isSelected = true;
            }

            toggleButton = createSortByButton("gallery_like", RequestSort.LIKES);
            sortByToggleGroup.addItem(toggleButton);
            sortByLayout.addChild(toggleButton);
            border = createBorder();
            sortByBorderList.push(border);
            border.height = toggleButton.defaultSkin.height;
            sortByLayout.addChild(border);
            if(_sortBy == RequestSort.LIKES){
                toggleButton.isSelected = true;
            }

            toggleButton = createSortByButton("gallery_comments", RequestSort.COMMENTS);
            sortByToggleGroup.addItem(toggleButton);
            sortByLayout.addChild(toggleButton);
            border = createBorder();
            sortByBorderList.push(border);
            border.height = toggleButton.defaultSkin.height;
            sortByLayout.addChild(border);
            if(_sortBy == RequestSort.COMMENTS){
                toggleButton.isSelected = true;
            }

            toggleButton = createSortByButton("gallery_date", RequestSort.DATE);
            sortByToggleGroup.addItem(toggleButton);
            sortByLayout.addChild(toggleButton);
            border = createBorder();
            sortByBorderList.push(border);
            border.height = toggleButton.defaultSkin.height;
            sortByLayout.addChild(border);
            if(_sortBy == RequestSort.DATE){
                toggleButton.isSelected = true;
            }

            sortByToggleGroup.addEventListener(Event.CHANGE, sortByGroupChangeHandler);


            backgroundButton = createSortByButton("gallery_background", "background");
            sortByLayout.addChild(backgroundButton);
            border = createBorder();
            sortByBorderList.push(border);
            border.height = backgroundButton.defaultSkin.height;
            sortByLayout.addChild(border);
            backgroundButton.isSelected = false;
            backgroundButton.addEventListener(Event.CHANGE, backgroundChangeHandler);

            addChild(sortByLayout);
        }

        private function sortByGroupChangeHandler(event:Event):void {
            if(sortByToggleGroup.selectedItem) {
                SoundManager.playSound(SoundManager.SELECT_PART);
                _sortBy = sortByToggleGroup.selectedItem.name;
                loadContent(refreshTuningsCompleteHandler);
            }
            updateSortByBorders();
        }

        private function sortByRegionChangeHandler(event:Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            loadContent(refreshTuningsCompleteHandler);
        }

        private function updateSortByBorders():void{
            var nextIndex:int = sortByToggleGroup.selectedIndex + 1;
            if(sortByBorderList && sortByToggleGroup){
                for(var i:int = 0; i < sortByBorderList.length; i++){
                    if(i == sortByToggleGroup.selectedIndex || i == nextIndex){
                        sortByBorderList[i].alpha = 0;
                    }else{
                        sortByBorderList[i].alpha = 1;
                    }
                }
            }
            if(backgroundButton.isSelected){
                sortByBorderList[sortByBorderList.length - 2].alpha = sortByBorderList[sortByBorderList.length - 1].alpha = 0;
            }
        }

        private function updateShowByBorders():void{
            if(showByBorderList && showByButton){
                if(showByButton.isSelected){
                    showByBorderList[0].alpha = showByBorderList[1].alpha = 0;
                }else{
                    showByBorderList[0].alpha = showByBorderList[1].alpha = 1;
                }
            }
        }

        private function createShowBySection():void{
            showByBorderList = [];
            var border:Quad;
            showByLayout = new LayoutGroup();
            var hLayout:HorizontalLayout = new HorizontalLayout();
            hLayout.horizontalAlign = HorizontalAlign.RIGHT;
            hLayout.verticalAlign = VerticalAlign.MIDDLE;
            hLayout.gap = 0;
            hLayout.paddingTop = 3;
            showByLayout.layout = hLayout;

            var viewLabel:Label = new Label();
            viewLabel.textRendererFactory = GalleryLabelFactory.sortByTitleLabel;
            viewLabel.paddingLeft = viewLabel.paddingRight = 6;
            viewLabel.paddingBottom = 3;
            viewLabel.text = "Show by";

            showByLayout.addChild(viewLabel);

            showByButton = createSortByButton("gallery_clock", "show");
            showByLayout.addChild(showByButton);
            showByButton.addEventListener(Event.CHANGE, showByChangeHandler);
            border = createBorder();
            showByBorderList.push(border);
            border.height = showByButton.defaultSkin.height;
            showByLayout.addChildAt(border, 1);

            border = createBorder();
            showByBorderList.push(border);
            border.height = showByButton.defaultSkin.height;
            showByLayout.addChild(border);
            showByLayout.validate();

            addChild(showByLayout);


            showByMenu = new Sprite();
            var menuBackground:Image = new Image(ButtonFactory.assetManager.getTexture("menu_background"));
            menuBackground.scale9Grid = new Rectangle(menuBackground.width/3, menuBackground.height/3, menuBackground.width/3, menuBackground.height/3);
            menuBackground.width = showByLayout.width;
            menuBackground.height = 190;
            menuBackground.filter = new DropShadowFilter(2, Math.PI/4, 0, 0.2);
            menuBackground.y = -1;

            var menuTopBorder:Quad = new Quad( 1, 1, 0xFFFFFF);
            menuTopBorder.width = menuBackground.width + 10;
            menuTopBorder.height = 5;
            menuTopBorder.x = -5;
            menuTopBorder.y = -5;

            showByMenu.addChild(menuBackground);
            showByMenu.addChild(menuTopBorder);

            showByGroup = new LayoutGroup();
            var layout:VerticalLayout = new VerticalLayout();
            layout.gap = 5;
            layout.paddingLeft = layout.paddingRight = 5;
            layout.paddingTop = 6;
            showByGroup.layout = layout;
            showByToggleGroup = new ToggleGroup();

            var radio:Radio;


            radio = createRadio("today", RequestPeriod.TODAY);
            //showByGroup.addChild(createRadioBorder());
            showByGroup.addChild(radio);
            showByToggleGroup.addItem(radio);
            if(_period == RequestPeriod.TODAY){
                showByToggleGroup.selectedItem = radio;
            }


            radio = createRadio("yesterday", RequestPeriod.YESTERDAY);
            showByGroup.addChild(createRadioBorder());
            showByGroup.addChild(radio);
            showByToggleGroup.addItem(radio);
            if(_period == RequestPeriod.YESTERDAY){
                showByToggleGroup.selectedItem = radio;
            }

            radio = createRadio("this week", RequestPeriod.THIS_WEEK);
            showByGroup.addChild(createRadioBorder());
            showByGroup.addChild(radio);
            showByToggleGroup.addItem(radio);
            if(_period == RequestPeriod.THIS_WEEK){
                showByToggleGroup.selectedItem = radio;
            }


            radio = createRadio("this month", RequestPeriod.THIS_MONTH);
            showByGroup.addChild(createRadioBorder());
            showByGroup.addChild(radio);
            showByToggleGroup.addItem(radio);
            if(_period == RequestPeriod.THIS_MONTH){
                showByToggleGroup.selectedItem = radio;
            }


            radio = createRadio("last month", RequestPeriod.LAST_MONTH);
            showByGroup.addChild(createRadioBorder());
            showByGroup.addChild(radio);
            showByToggleGroup.addItem(radio);
            if(_period == RequestPeriod.LAST_MONTH){
                showByToggleGroup.selectedItem = radio;
            }


            radio = createRadio("this year", RequestPeriod.THIS_YEAR);
            showByGroup.addChild(createRadioBorder());
            showByGroup.addChild(radio);
            showByToggleGroup.addItem(radio);
            if(_period == RequestPeriod.THIS_YEAR){
                showByToggleGroup.selectedItem = radio;
            }


            radio = createRadio("show all", RequestPeriod.ALL);
            showByGroup.addChild(createRadioBorder());
            showByGroup.addChild(radio);
            showByToggleGroup.addItem(radio);
            if(_period == RequestPeriod.ALL){
                showByToggleGroup.selectedItem = radio;
            }

            showByGroup.validate();
            menuBackground.height = showByGroup.height + 5;

            showByMenu.addChild(showByGroup);
            addChild(showByMenu);
            showByMenu.visible = false;
            showByToggleGroup.addEventListener(Event.CHANGE, showByGroupChangeHandler);


            showByCountryButton = createSortByButton(String(countryData.country).toLowerCase(), "showByCountry", 16, false);
            showByWorldButton = createSortByButton("timeline_world_icon", "showByWorld");
            showByRegionLayout = new LayoutGroup();
            showByRegionLayout.addChild(showByCountryButton);
            showByRegionLayout.addChild(showByWorldButton);

            border = createBorder();
            border.height = showByButton.defaultSkin.height;
            showByRegionLayout.addChild(border);

            var regionLayout:HorizontalLayout = new HorizontalLayout();
            regionLayout.paddingTop = 3;
            showByRegionLayout.layout = regionLayout;
            showByRegionLayout.validate();
            showByRegionToggleGroup = new ToggleGroup();
            showByRegionToggleGroup.addItem(showByCountryButton);
            showByRegionToggleGroup.addItem(showByWorldButton);
            showByRegionToggleGroup.addEventListener(Event.CHANGE, sortByRegionChangeHandler);

            addChild(showByRegionLayout);
        }

        private function showByGroupChangeHandler(event:Event):void {
            if(showByToggleGroup.selectedItem){
                _period = showByToggleGroup.selectedItem.name;
                loadContent(refreshTuningsCompleteHandler);
                if(showByButton.isSelected){
                    Starling.juggler.delayCall(function():void{
                        showByButton.isSelected = false;
                    }, 0.1);
                }
            }
        }

        private function createSearchView():void{
            searchInput = new SearchInput();
            searchInput.x = 5;
            searchInput.addEventListener(Event.CHANGE, searchInputChangeHandler);

            _noResultLabel = new Label();
            _noResultLabel.textRendererFactory = GalleryLabelFactory.sortByTitleLabel;
            _noResultLabel.text = NO_RESULT_LABEL;

            addChild(searchInput);
            addChild(_noResultLabel);
            _noResultLabel.visible = false;
        }

        private function searchInputChangeHandler(event:Event):void {
            Starling.juggler.removeDelayedCalls(startSearch);
            Starling.juggler.delayCall(startSearch, 1);
        }

        private function startSearch():void{
            _query = searchInput.text;
            loadContent(refreshTuningsCompleteHandler);
        }

        private function showByChangeHandler(event:Event):void {
            showByMenu.visible = showByButton.isSelected;
            if(showByButton.isSelected){
                SoundManager.playSound(SoundManager.PART_MENU);
            }else{
                SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
            }
            updateShowByBorders();
            if(showByButton.isSelected){
                Starling.juggler.delayCall(function():void{
                    stage.addEventListener(TouchEvent.TOUCH, stageTouchHandler);
                }, .1);
            }
        }

        private function stageTouchHandler(event:TouchEvent):void {
            if(!stage){
                event.currentTarget.removeEventListener(event.type, stageTouchHandler);
                return;
            }
            var menuTouch:Touch = event.getTouch(showByMenu);
            var buttonTouch:Touch = event.getTouch(showByButton);
            var touchBegan:Touch = event.getTouch(stage, TouchPhase.BEGAN);
            if(touchBegan && !menuTouch && !buttonTouch){
                showByButton.isSelected = false;
                stage.removeEventListener(event.type, stageTouchHandler);
                event.stopImmediatePropagation();
            }
        }


        public static function createSortByButton(texture:String, name:String, iconHeight:Number = 0, colorizeIcon:Boolean = true, selectedSkinColor:int = -1, hasOrder:Boolean = false, buttonWidth:Number = 35):ToggleButton{
            var toggleButton:ToggleButton;
            var defaultIcon:Image;
            var selectedIcon:Image;
            var defaultSkin:Quad;
            var selectedSkin:Quad;
            if(hasOrder){
                var orderIcon:Image = new Image(ButtonFactory.assetManager.getTexture("drop_down_arrow_2"));
                orderIcon.name = "order_icon";
            }

            toggleButton = new ToggleButton();
            toggleButton.name = name;
            defaultIcon = new Image(ButtonFactory.assetManager.getTexture(texture));
            defaultIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            if(colorizeIcon){
                defaultIcon.color = 0x999999;
                if(orderIcon){
                   orderIcon.color = 0x999999;
                }
            }

            var ratio:Number = iconHeight / defaultIcon.height;
            if(ratio > 0){
                defaultIcon.height = iconHeight;
                defaultIcon.width *= ratio;
            }

            toggleButton.defaultIcon = defaultIcon;
            defaultSkin = new Quad(1,1, 0xFFFFFF);
            defaultSkin.width = buttonWidth;
            defaultSkin.height = 20;
            if(ratio){
                defaultSkin.width = defaultIcon.width + 10;
            }
            toggleButton.defaultSkin = defaultSkin;

            selectedIcon = new Image(ButtonFactory.assetManager.getTexture(texture));
            if(colorizeIcon){
                selectedIcon.color = 0xFEFFFF;
            }
            selectedIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            if(ratio > 0){
                selectedIcon.height = iconHeight;
                selectedIcon.width *= ratio;
            }
            toggleButton.defaultSelectedIcon = selectedIcon;
            selectedSkin = new Quad(1, 1, selectedSkinColor==-1?0x026edc:selectedSkinColor);
            selectedSkin.width = defaultSkin.width;
            selectedSkin.height = defaultSkin.height;
            toggleButton.defaultSelectedSkin = selectedSkin;

            if(orderIcon){
                toggleButton.iconOffsetX = 3;
                toggleButton.validate();
                orderIcon.width = orderIcon.height = 1.5 * toggleButton.height / 2;
                toggleButton.addChild(orderIcon);
                orderIcon.x = 0;
                orderIcon.y = (toggleButton.height - toggleButton.scaleY * orderIcon.height) / 2;
            }
            return toggleButton;
        }

        public static function setOrderIconColor(toggleButton:ToggleButton):void{
            var icon:Image = toggleButton.getChildByName("order_icon") as Image;
            if(icon){
                icon.color = toggleButton.isSelected?0xFEFFFF:0x999999;
            }
        }

        public static function setOrderIconDirection(toggleButton:ToggleButton):Boolean{
            var icon:Image = toggleButton.getChildByName("order_icon") as Image;
            if(icon){
                if(icon.scaleY < 0){
                    icon.scaleY *= -1;
                    icon.y = (toggleButton.height - toggleButton.scaleY * icon.height) / 2;
                    return true;
                }else if(icon.scaleY > 0){
                    icon.scaleY *= -1;
                    icon.y = toggleButton.height - (toggleButton.height - toggleButton.scaleY * icon.height) / 2;
                    return false;
                }
            }
            return false;
        }

        public static function getOrderIconDirection(toggleButton:ToggleButton):Boolean{
            var icon:Image = toggleButton.getChildByName("order_icon") as Image;
            if(icon){
                return (icon.scaleY > 0);
            }
            return true;
        }

        private static function createRadio(label:String, value:String):Radio{
            var radio:Radio = new Radio();
            var defaultIcon:Image = new Image(ButtonFactory.assetManager.getTexture("gallery_radio_default"));
            defaultIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            defaultIcon.color = 0x888888;

            var selectedIcon:Image = new Image(ButtonFactory.assetManager.getTexture("gallery_radio_selected"));
            selectedIcon.scale = ButtonFactory.assetManager.scaleFactor/2;
            selectedIcon.color = 0x026edc;

            radio.defaultIcon = defaultIcon;
            radio.defaultSelectedIcon = selectedIcon;
            radio.labelFactory = GalleryLabelFactory.showByRadioDefaultLabel;
            radio.labelOffsetX = 5;
            radio.iconOffsetY = 1;
            radio.label = label;
            radio.name = value;
            radio.paddingTop = -2;
            radio.paddingBottom = 2;
            return radio;
        }

        public static function createBorder():Quad{
            var border:Quad = new Quad(1, 1, 0xCCCCCC);
            border.width = 1;
            return border;
        }

        private static function createRadioBorder():Quad{
            var border:Quad = new Quad(1, 1, 0xCCCCCC);
            border.height = 1;
            border.width = 85;
            return border;
        }

        private function onMenuButton(event:Event):void{
            mainMenuView.toggle();
        }

        private function backgroundChangeHandler(event:Event):void {
            if(backgroundButton.isSelected){
                SoundManager.playSound(SoundManager.SELECT_PART);
            }else{
                SoundManager.playSound(SoundManager.SELECT_PART_BACK);
            }
            loadContent(refreshTuningsCompleteHandler);
            updateSortByBorders();
        }

        public function set mainMenuView(value : MainMenuView) :void {
            _mainMenuView = value;
            _mainMenuView.owner = this;
        }

        public function get mainMenuView():MainMenuView {
            return _mainMenuView;
        }

        public function set selectedLanguage(value:Language):void {
            _selectedLanguage =value;
        }

        public function get selectedLanguage():Language {
            return _selectedLanguage;
        }

        public function set languages(value:Array):void{
            _languages = value;
        }
        public function get languages():Array{
            return _languages;
        }

        public function brandsScreenButtonTriggeredHandler(event:Event = null):void{
            dispatchEventWith(StackedScreen.BRANDS_SELECTED);
        }

        public function homeButtonTriggeredHandler(event:Event = null):void {
            dispatchEventWith(StackedScreen.HOME_SELECTED);
        }

        public function enterRowsState():void{

        }

        public function getVisibleTuningURL():String{
            return '';
        }

        public function get selectedTuning():Tuning {
            return _selectedTuning;
        }

        public function set selectedTuning(value:Tuning):void {
            _selectedTuning = value;
        }

        public function invalidateTunings():void {
            var _parent:DisplayObjectContainer = parent;
            var gallery:GalleryScreen = this;
            removeFromParent();
            clearContent();
            Starling.juggler.delayCall(function():void{
                if(_parent){
                    _parent.addChild(gallery);
                }
                loadContent(refreshTuningsCompleteHandler);
            }, .15);
        }


    }
}
