package com.despark.tuning3d.view.components.gallery {
    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.challenges.ChallengeEvent;
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.languages.Multilingual;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.timeline.TimelineProxy;
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.challenges.controls.ChallengeViewUserLabel;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.IMainMenuHolder;
    import com.despark.tuning3d.view.components.core.MainMenuView;
    import com.despark.tuning3d.view.components.core.PopUp;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.mailbox.MailsScreen;
    import com.despark.tuning3d.view.components.timeline.FeedData;
    import com.despark.tuning3d.view.components.timeline.TimelineScreen;
    import com.despark.tuning3d.view.components.tunings.ITuningListMenuVIew;
    import com.despark.tuning3d.view.components.tunings.TuningItemRenderer;
    import com.despark.tuning3d.view.components.tunings.TuningsListMenuView;
    import com.despark.tuning3d.view.components.tunings.TuningsListScreen;
    import com.greensock.TweenLite;

    import feathers.controls.ToggleButton;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.HorizontalLayout;
    import feathers.layout.VerticalAlign;

    import starling.core.Starling;
    import starling.display.Image;
    import starling.events.Event;
    import starling.events.EventDispatcher;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;

    public class GalleryTuningPopup extends PopUp implements Multilingual, IMainMenuHolder, ITuningListMenuVIew {
    private var _mainMenuView : MainMenuView;
    private var _languages:Array;
    private var _selectedLanguage:Language;
    private var _selectedTuning:Tuning;
    private var tuningsListMenuView:TuningsListMenuView;
    private const _gap:Number = 6;
    private var _selectedUser:User;
    private var _currentUser:User;
    private var _userLabel:ChallengeViewUserLabel;
    private var _container:MainContainer;
    private var _closeHandler:Function;
    private var tuningView:TuningItemRenderer;
    private var _eventDispatcher:EventDispatcher;

    private var _showBanner:ToggleButton;
    private var _menuTween:TweenLite;
    private var _showBannerAfterExit:Boolean = false;
    private var _bannerScreen:String;
    public function GalleryTuningPopup(data:Object = null) {
        var usersProxy:UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
        _container = new MainContainer();
        _currentUser = usersProxy.currentUser;
        FeedData.CURRENT_USER = _currentUser;
        _showBannerAfterExit = data.hasOwnProperty("showBannerAfterExit") && data.showBannerAfterExit;
        _bannerScreen = (data && data.hasOwnProperty("bannerScreen"))?data["bannerScreen"]:BannerScreen.BANNER_SCREEN;
    }

    override protected function initialize():void {
        super.initialize();
        setSizeInternal(stage.stageWidth, stage.stageHeight, false);

        if(contentBackgroundImage){
            contentBackgroundImage.removeFromParent();
        }

        background.color = 0xFFFFFF;
        background.alpha = 1;
        _container.setSize(actualWidth, actualHeight);

        var layout : HorizontalLayout = new HorizontalLayout();
        layout.gap = 0;
        layout.horizontalAlign = HorizontalAlign.CENTER;
        layout.verticalAlign = VerticalAlign.MIDDLE;

        tuningsListMenuView = new TuningsListMenuView(this);
        tuningsListMenuView.addEventListener(TuningsListMenuView.CAR_SELECTED, onCarSelected);
        tuningsListMenuView.addEventListener(TuningsListMenuView.TUNING_SELECTED, onTuningSelected);
        tuningsListMenuView.addEventListener(TuningsListMenuView.TUNING_SHARE, onTuningShare);
        tuningsListMenuView.addEventListener(TuningsListMenuView.TUNING_DELETE, onTuningDelete);
        tuningsListMenuView.addEventListener(TuningsListMenuView.START_CHALLENGE, challengeHandler);
        tuningsListMenuView.addEventListener(TuningsListMenuView.POST_TO_TIMELINE, postToTimelineButtonHandler);
        tuningsListMenuView.addEventListener(TuningsListMenuView.FILTER_SELECTED, filterButtonHandler);


        tuningsListMenuView.addEventListener(TuningsListMenuView.COMMENT_SELECTED, commentSelectHandler);
        tuningsListMenuView.addEventListener(TuningsListMenuView.LIKE_SELECTED, likeSelectHandler);
        tuningsListMenuView.owner = _eventDispatcher;

        _container.addChild(tuningsListMenuView);
        if(background.parent && background.parent == this){
            addChildAt(_container, getChildIndex(background)+1);
        }

        var closeIcon:Image = new Image(ButtonFactory.assetManager.getTexture('garage_close'));
        closeIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        closeButton.width = closeIcon.width;
        closeButton.height = closeIcon.height;
        closeButton.defaultIcon = closeIcon;
        _container.addChild(closeButton);

        _showBanner = new ToggleButton();

        var defaultIcon:Image = new Image(ButtonFactory.assetManager.getTexture("a_open_menu"));
        defaultIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        _showBanner.defaultIcon = defaultIcon;

        defaultIcon = new Image(ButtonFactory.assetManager.getTexture("a_open_menu"));
        defaultIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        defaultIcon.alpha = 0.6;
        _showBanner.downIcon = defaultIcon;

        var selectedIcon:Image = new Image(ButtonFactory.assetManager.getTexture("a_close_menu"));
        selectedIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        _showBanner.defaultSelectedIcon = selectedIcon;

        selectedIcon = new Image(ButtonFactory.assetManager.getTexture("a_close_menu"));
        selectedIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
        selectedIcon.alpha = 0.6;
        _showBanner.selectedDownIcon = selectedIcon;
        _showBanner.width = selectedIcon.width;
        _showBanner.height = selectedIcon.height + 10;
        _showBanner.iconOffsetY = -5;

        _showBanner.isSelected = true;
        _showBanner.addEventListener(Event.CHANGE, showBannerChangeHandler);
        _container.addChild(_showBanner);
    }

    private function sendMailButtonTriggeredHandler(event:Event):void {
        dispatchEvent(new Event(MailsScreen.SEND_INSTANT_MAIL, false, {toUser: Constants.selectedUser}));
    }

    private function likeSelectHandler(event:Event):void {
        visible = false;
        name = 'restore';
        closeButtonTriggeredHandler(null);
        dispatchEvent(new Event(TuningsListMenuView.LIKE_SELECTED));
    }

    private function commentSelectHandler(event:Event):void {
        visible = false;
        name = 'restore';
        closeButtonTriggeredHandler(null);
        dispatchEvent(new Event(TuningsListMenuView.COMMENT_SELECTED));
    }

    private function showBannerChangeHandler(event:Event):void {
        if(_menuTween){
            _menuTween.kill();
        }
        if(_showBanner.isSelected){
            _menuTween = new TweenLite(tuningsListMenuView, 0.3, {y: actualHeight - tuningsListMenuView.height, onUpdate:menuAnimationUpdate});
        }else{
            _menuTween = new TweenLite(tuningsListMenuView, 0.3, {y: actualHeight, onUpdate:menuAnimationUpdate});
        }
    }

    private function menuAnimationUpdate():void{
        _showBanner.y = tuningsListMenuView.y - (_showBanner.height - 10);
        if(tuningView){
            tuningView.y = - (actualHeight - tuningsListMenuView.y) / 2;
        }
        tuningsListMenuView.updateLabelPositions();
        updatePageIndicatorPosition();
    }

    public function updatePageIndicatorPosition():Boolean{
        if(tuningView && tuningView.pageIndicator && _showBanner){
            tuningView.pageIndicator.y = _showBanner.y - (tuningView.pageIndicator.height + 5) - tuningView.y;
            return true;
        }
        return false;
    }

    private function challengeHandler(event:Event):void{
        var user : User  = (ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy).currentUser;
        if(!user){
            visible = false;
            name = 'restore';
            closeButtonTriggeredHandler(null);
            dispatchEvent(new Event(TuningsListMenuView.CHALLENGE_SELECTED));
            return;
        }
        var tuning:Tuning;
        var tuningUserId:String;
        if(selectedTuning.userObj){
            if(selectedTuning.userObj is User){
                tuningUserId = User(selectedTuning.userObj).id;
            }else if(selectedTuning.userObj is String){
                tuningUserId = selectedTuning.userObj as String;
            }else if(selectedTuning.userObj is Object && selectedTuning.userObj.hasOwnProperty('_id')){
                tuningUserId = selectedTuning.userObj._id;
            }
        }else if(selectedTuning.tuningUserID){
            tuningUserId = selectedTuning.tuningUserID;
        }
        selectedTuning.tuningUserID ||= tuningUserId;
        if(user && selectedTuning && selectedTuning.tuningUserID == user.id){
            tuning = selectedTuning;
            Constants.selectedUser = null;
        }else{
            tuning = null;
        }
        var data:Object  = ChallengeEvent.getData(user, tuning, Constants.selectedUser, null);
        dispatchEvent(new Event(ChallengeEvent.CHALLENGE_REQUEST, false, data));
    }

    public function get selectedTuningFrameIndex():int{
        return tuningView.selectedIndex;
    }


    private function filterButtonHandler(event:Event):void {
        if (!_selectedTuning) {
            return;
        }
        var userName:String = _selectedTuning.userObj.username;
        var shortId:String = _selectedTuning.shortId;
        var car:Car = _selectedTuning.car;

        var urlToSend:String = car.getEditorLink(userName, shortId, tuningView.selectedIndex);
        var obj:Object = {
            url: urlToSend,
            tuning: _selectedTuning
        };

        ApplicationFacade.getInstance().sendNotification(Notifications.DISPLAY_TUNING_EDITOR_POPUP, obj);
    }

    private function postToTimelineButtonHandler(event:Event):void {
        if (!_selectedTuning) {
            return;
        }
        AlertView.show('Are you sure you want to share this tuning on 3DTuning\' feed', -1, false, function (yesSelected:Boolean):void {
            if (yesSelected) {
                var timelineProxy : TimelineProxy = ApplicationFacade.getInstance().retrieveProxy(TimelineProxy.NAME) as TimelineProxy;
                var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
                var snapshotIndex:int = tuningView.selectedIndex;
                var urls:Array = [{url: _selectedTuning.snapshots[snapshotIndex]}];
                /*for(var i:int = 0; i < tuning.snapshots.length; i++){
                    urls.push({url: tuning.snapshots[i]});
                }*/
                timelineProxy.newPost('', usersProxy.currentUser, urls);
            }
        });
    }

    protected function onCarSelected(event:Event):void {
        dispatchEventWith(TuningsListMenuView.CAR_SELECTED);
    }

    override protected function closeButtonTriggeredHandler(event:Event):void {
        super.closeButtonTriggeredHandler(event);
        SoundManager.playSound(SoundManager.SELECT_PART_BACK);
        if(_closeHandler != null){
            _closeHandler.call();
        }
        if(_eventDispatcher && ((_eventDispatcher is TimelineScreen && _showBannerAfterExit) || (_eventDispatcher is GalleryScreen))){
            BannerStackedScreen.showAdBanner();
        }
    }

    protected function onTuningSelected(event:Event):void {
        _closeHandler = null;
        closeButtonTriggeredHandler(null);
        dispatchEventWith(StackedScreen.TUNING_SELECTED);
        getSocialData();
    }

    protected function onTuningShare(event:Event):void{
        dispatchEventWith(TuningsListMenuView.TUNING_SHARE);
    }

    protected function onTuningDelete(event:Event):void{
        dispatchEventWith(TuningsListMenuView.TUNING_DELETE, false, event.data);
    }

    public function updateTuning():void{
        tuningView.data = null;
        tuningView.data = _selectedTuning;
        tuningView.invalidate();
        _showBanner.isSelected = true;
    }

    public function set likes(value:int) : void {
        tuningsListMenuView.likes = value;
    }

    override protected function draw():void {
        super.draw();
        if(closeButton){
            if(!closeButton.parent){
                addChild(closeButton);
            }
            closeButton.visible = true;
            closeButton.x = actualWidth - (closeButton.width + 9);
            closeButton.y = 9;
        }

        if(tuningsListMenuView) {
            tuningsListMenuView.width = actualWidth;
            tuningsListMenuView.height = 50;
            tuningsListMenuView.y = actualHeight - tuningsListMenuView.height;
            tuningsListMenuView.parentHeight = actualHeight;

            if(_showBanner){
                _showBanner.y = tuningsListMenuView.y - _showBanner.height;
                _showBanner.x = tuningsListMenuView.x + (tuningsListMenuView.width - _showBanner.width)/2;
            }
            tuningsListMenuView.setCarName();
        }

        if(_userLabel){
            _userLabel.x = _gap;
            _userLabel.y = _gap;
        }

        menuAnimationUpdate();
        var setPageIndicatorPosition:Function = function():void{
            if(!updatePageIndicatorPosition()){
                Starling.juggler.delayCall(function():void{
                    setPageIndicatorPosition.call();
                }, .03);
            }
        };
        setPageIndicatorPosition.call();
    }

    private function labelTouchHandler(event:TouchEvent):void {
        var touch:Touch = event.getTouch(_userLabel, TouchPhase.ENDED);
        if(touch){
            var facade : ApplicationFacade = ApplicationFacade.getInstance();
            var usersProxy:UsersProxy= facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            var currentUser:User =usersProxy.currentUser;
            if(currentUser && _selectedUser && _selectedUser.id == currentUser.id){
                Constants.selectedUser = null;
            }else{
                Constants.selectedUser = _selectedUser;
                _eventDispatcher.dispatchEvent(new Event(StackedScreen.USER_DETAILS_SELECTED, false, {user: _selectedUser}));
            }
            if(_eventDispatcher) {
                // closeButtonTriggeredHandler(null);
            }
        }
    }

    public function selectTuning(tuning:Tuning, user:User):void{
        tuningView = new TuningItemRenderer();
        tuningView.data = tuning;
        tuningView.setSize(actualWidth, actualHeight + 50);
        tuningView.y = -25;
        _container.addChildAt(tuningView, 0);

        _selectedTuning = tuning;
        _selectedUser = user;
        _selectedTuning.car ||= Car.jsonToCar(_selectedTuning.carObj);

        _userLabel ||= new ChallengeViewUserLabel();
        if(!_userLabel.hasEventListener(TouchEvent.TOUCH)){
            _userLabel.addEventListener(TouchEvent.TOUCH, labelTouchHandler);
        }
        _userLabel.num = "";
        _userLabel.user = _selectedUser;
        if((_currentUser && _selectedUser && _selectedUser.id != _currentUser.id) || !_currentUser){
            Constants.selectedUser = _selectedUser;
        }else{
            Constants.selectedUser = null;
            _userLabel.visible = false;
        }

        if(isInitialized) {
            _selectedTuning.userObj = _selectedUser;
            tuningsListMenuView.requireLogin = (_currentUser==null);
            tuningsListMenuView.currentTuning = _selectedTuning;
            tuningsListMenuView.currentTuningID = _selectedTuning.id;
            if (_selectedTuning.car) {
                tuningsListMenuView.currentCarID = _selectedTuning.car.id;
            }
            if(_userLabel){
                _container.addChild(_userLabel);
            }
            invalidate();
            validate();
        }else{
            Starling.juggler.delayCall(function ():void {
                selectTuning(tuning, user);
            }, 0.1);
        }
    }

    private function getSocialData() : void {
        if (!Constants.selectedUser) {
            return;
        }
        dispatchEventWith(TuningsListScreen.GET_SOCIAL_DATA);
    }

    public function set comments(value:Array) : void {
        tuningsListMenuView.initComments = value;
    }

    public function getVisibleTuningURL():String {
        return "";
    }

    public function set languages(value:Array):void{
        _languages=value;
    }

    public function get languages():Array{
        return _languages;
    }

    public function brandsScreenButtonTriggeredHandler(event:Event = null):void {
        dispatchEventWith(StackedScreen.BRANDS_SELECTED);
    }

    public function homeButtonTriggeredHandler(event:Event = null):void {
        dispatchEventWith(StackedScreen.HOME_SELECTED);
    }

    public function enterRowsState():void {

    }

    public function set mainMenuView(value : MainMenuView) : void {
        _mainMenuView = value;
        _mainMenuView.owner = this;
    }

    public function get mainMenuView():MainMenuView {
        return _mainMenuView;
    }

    public function set selectedLanguage(value:Language):void {
        _selectedLanguage = value;
    }

    public function get selectedLanguage():Language {
        return _selectedLanguage;
    }

    public function dispatchExternalEvent(msg:String):void {
        if (mainMenuView) {
            mainMenuView.hide();
        }
        dispatchEventWith(msg);
    }

    public function updateWithLanguage(language:Language):void {
        if (mainMenuView) {
            mainMenuView.updateWithLanguage(language);
        }
    }

    override public function dispose():void {
        super.dispose();
        if(_mainMenuView){
            _mainMenuView.removeFromParent(true);
            _mainMenuView = null;
        }

        if(tuningsListMenuView){
            tuningsListMenuView.removeFromParent(true);
            tuningsListMenuView = null;
        }

        if(_userLabel){
            _userLabel.removeFromParent(true);
            _userLabel = null;
        }

        if(tuningView){
            tuningView.removeFromParent(true);
            tuningView = null;
        }

        if(_container){
            _container.removeFromParent(true);
            _container = null;
        }
        _languages = null;
        _selectedLanguage = null;
        _selectedTuning = null;
        _selectedUser = null;
        _currentUser = null;
    }

    public function get selectedTuning():Tuning {
        return _selectedTuning;
    }

    public function set closeHandler(value:Function):void {
        _closeHandler = value;
    }

    public function set eventDispatcher(value:EventDispatcher):void {
        _eventDispatcher = value;
        if(tuningsListMenuView){
            tuningsListMenuView.owner = _eventDispatcher;
        }
    }

    public function set requireLogin(value:Boolean):void{
        if(tuningsListMenuView) {
            tuningsListMenuView.requireLogin = value;
        }
    }

    public function close():void{
        closeButtonTriggeredHandler(null);
    }

    public function set selectedTuning(value:Tuning):void {
        _selectedTuning = value;
    }
}
}

import feathers.core.FeathersControl;

class MainContainer extends FeathersControl{
    public function MainContainer(){
        super ();
    }
}