package com.despark.tuning3d.view.components.gallery {
    import com.despark.tuning3d.model.core.UIColors;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.greensock.TweenLite;

import feathers.controls.Label;

import feathers.controls.TextInput;

import feathers.core.FeathersControl;
import feathers.events.FeathersEventType;

import flash.geom.Rectangle;

import starling.core.Starling;

import starling.display.Image;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;

public class SearchInput extends FeathersControl {
    private var searchInput:TextInput;
    private var searchInputPlaceholder:Label;
    private var searchBackground:Image;
    private var tween:TweenLite;
    private var _isOpen:Boolean;
    private static const MAX_WIDTH:int = 150;
    private static var SEARCH_LABEL:String = 'search';
    private var _isFixed:Boolean;
    private var _expandedWidth:Number = 0;
    public function SearchInput(isFixed:Boolean = false) {
        super();
        _isFixed = isFixed;
    }


    override protected function initialize():void {
        super.initialize();
        width = height = 22;
        _isOpen = false;

        searchBackground = new Image(ButtonFactory.assetManager.getTexture("searchBg"));
        searchBackground.color = UIColors.UI_COMPONENT_GRAY;
        searchBackground.scale = ButtonFactory.assetManager.scaleFactor / 2;
        searchBackground.scale9Grid = new Rectangle(searchBackground.texture.frameWidth / 12, 0, 10 * searchBackground.texture.frameWidth / 12, searchBackground.texture.frameHeight);

        addChild(searchBackground);
        searchBackground.width = 22;
        searchBackground.height = 22;

        searchInput = new TextInput();
        searchInput.textEditorFactory = GalleryLabelFactory.searchFieldLabel;


        var searchIcon:Image = new Image(ButtonFactory.assetManager.getTexture("src"));
        searchIcon.height = searchBackground.height - 6;
        searchIcon.scale = searchIcon.scaleY;
        searchIcon.x = (searchBackground.width - searchIcon.width) / 2;
        searchIcon.y = (searchBackground.height - searchIcon.height) / 2;
        searchIcon.color = UIColors.UI_COMPONENT_GRAY;

        addChild(searchIcon);
        searchInput.width = expandedWidth - 25;
        searchInput.height = searchIcon.height;
        searchInput.x = 5;
        searchInput.y = searchBackground.y + (searchBackground.height - searchInput.height)/2;
        searchInput.addEventListener(FeathersEventType.FOCUS_IN, searchFieldFocusHandler);
        searchInput.addEventListener(FeathersEventType.FOCUS_OUT, searchFieldFocusHandler);
        searchInput.addEventListener(Event.CHANGE, searchInputChangeHandler);
        searchInput.visible = false;
        addChild(searchInput);

        searchInputPlaceholder = new Label();
        searchInputPlaceholder.textRendererFactory = GalleryLabelFactory.searchPlaceholderDefaultLabel;
        searchInputPlaceholder.text = SEARCH_LABEL;
        searchInputPlaceholder.height = searchInput.height;
        searchInputPlaceholder.validate();
        addChild(searchInputPlaceholder);
        searchInputPlaceholder.visible = false;
        searchInputPlaceholder.touchable = searchInputPlaceholder.isEnabled = false;

        if(_isFixed){
            open();
        }
    }

    override protected function feathersControl_addedToStageHandler(event:Event):void {
        super.feathersControl_addedToStageHandler(event);
        if(!_isFixed){
            stage.addEventListener(TouchEvent.TOUCH, touchHandler);
        }
    }

    override protected function feathersControl_removedFromStageHandler(event:Event):void {
        super.feathersControl_addedToStageHandler(event);
        stage.removeEventListener(TouchEvent.TOUCH, touchHandler);
    }

    private function touchHandler(event:TouchEvent):void{
        var touch:Touch = event.getTouch(this, TouchPhase.ENDED);
        var touchEnd:Touch = event.getTouch(stage, TouchPhase.ENDED);
        if(touch && !_isOpen){
            open();
        }else if(!touch && _isOpen && touchEnd){
            close();
        }
    }

    public function close():void{
        if(!tween){
            SoundManager.playSound(SoundManager.SELECT_PART_BACK);
            searchInput.visible = false;
            searchInputPlaceholder.visible = false;
            tween = TweenLite.to(searchBackground, 0.25, {width:22, onUpdate:function():void{
                    searchBackground.x = (22 - searchBackground.width);
                }, onComplete:function():void{
                    tween = null;
                    _isOpen = false;
                }});
        }
    }

    public function open():void{
        if(!tween){
            if(isInitialized){
                SoundManager.playSound(SoundManager.SELECT_PART);
            }
            tween = TweenLite.to(searchBackground, 0.25, {width: expandedWidth, onUpdate:function():void{
                    searchBackground.x = (22 - searchBackground.width);
                }, onComplete:function():void{
                    tween = null;
                    _isOpen = true;
                    searchInput.x = 22 - searchBackground.width + 5;
                    searchInput.visible = true;
                    searchInputPlaceholder.x = 22 - searchBackground.width + 10;
                    searchInputPlaceholder.y = 2;
                    if(text == "" || !text){
                        searchInputPlaceholder.visible = true;
                    }
                }});
        }
    }

    private function searchFieldFocusHandler(event:Event):void {
        if(event.type == FeathersEventType.FOCUS_IN && searchInput.text == ''){
            SoundManager.playSound(SoundManager.SELECT_PART);
            searchInputPlaceholder.visible = false;
        }else if(event.type == FeathersEventType.FOCUS_OUT && searchInput.text == ''){
            if(isInitialized){
                SoundManager.playSound(SoundManager.SELECT_PART_BACK);
            }
            searchInputPlaceholder.visible = true;
        }
    }

    private function searchInputChangeHandler(event:Event):void {
        dispatchEvent(new Event(event.type, event.bubbles, event.data));
    }

    public function set text(value:String):void{
        searchInput.text = value;
    }

    public function get text():String{
        return searchInput.text;
    }

    public function set expandedWidth(value:Number):void {
        _expandedWidth = value;
    }

    public function get expandedWidth():Number {
        return _expandedWidth>0?_expandedWidth:MAX_WIDTH;
    }
}
}
