package com.despark.tuning3d.view.components.mailbox.mails
{
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.users.User;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.LabelFactory;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.utils.DateParser;

    import feathers.controls.ImageLoader;
    import feathers.controls.Label;
    import feathers.controls.LayoutGroup;
    import feathers.controls.List;
    import feathers.controls.text.StageTextTextEditor;
    import feathers.core.FeathersControl;
    import feathers.layout.AnchorLayout;
    import feathers.layout.AnchorLayoutData;
    import feathers.layout.HorizontalAlign;
    import feathers.layout.VerticalLayout;

    import flash.geom.Rectangle;

    import starling.display.Image;
    import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.filters.DropShadowFilter;

    public class MailBodyItemRenderer extends MailItemRenderer{
        private var _user:User;

        private static const _gap : int = 5;

        protected var _textLabel: FeathersControl;
        protected var _dateLabel: Label;

        private var _mainContainer   : LayoutGroup;

        private var _background:Image;
        private var _currentUserId:String;
        private var _prevData:Object;
        private var _imageContainer:LayoutGroup;
        private var _metaContainer:MessageLinkView;
        private var _isItemSelected:Boolean;
        public function MailBodyItemRenderer(){

        }

		protected override function initialize() : void {
			super.initialize();

            var contentWidth:Number = ownerWidth - (marginLeft + marginRight);

            _background ||= new Image(ButtonFactory.assetManager.getTexture("button2"));//new Image(Texture.fromColor(1, 1, 0xFF0000));
            _background.scale9Grid = new Rectangle(_background.texture.width / 4, _background.texture.height / 3,2 * _background.texture.width / 4, _background.texture.height / 3);
            addChild(_background);
            _background.filter = new DropShadowFilter(0, 0, 0, 0.2, 1, 1);

            _mainContainer ||= new LayoutGroup();
            _mainContainer.layout = new AnchorLayout();
            _mainContainer.width = contentWidth;
            addChild(_mainContainer);

            var layData : AnchorLayoutData;
            /**
             * Message label
             */
            layData = new AnchorLayoutData();
            layData.left = 2 * _gap;
            layData.top = 2 * _gap;
            _textLabel = LabelFactory.createLabelMailBody();
            _textLabel.layoutData = layData;
            _textLabel.addEventListener(TouchEvent.TOUCH, onFriendSelectHandler);
            _textLabel.maxWidth = 2 * contentWidth / 3;
            _mainContainer.addChild(_textLabel);

            /**
             * Date label
             */
            layData = new AnchorLayoutData();
            layData.bottom = _gap / 2;
            layData.left = _gap;
            _dateLabel ||= new Label();
            _dateLabel.textRendererFactory = LabelFactory.mailDateLabel;
            _dateLabel.layoutData = layData;
            _mainContainer.addChild(_dateLabel);

            _isItemSelected = false;
            addEventListener(TouchEvent.TOUCH, triggeredHandler);
		}

        private function triggeredHandler(event:TouchEvent):void {
            var touch:Touch = event.getTouch(this, TouchPhase.ENDED);
            if(touch){
                if(_isItemSelected && isSelected){
                    isSelected = false;
                }
                _isItemSelected = !_isItemSelected;
            }
        }

        override protected function commitSelection():void {
            super .commitSelection();
            if(!isSelected){
                _isItemSelected = false;
            }
        }

		protected override function draw() : void {
			super.draw();
            updateView();
		}

        private function get customHeight():Number{
            return _textLabel.height + 4 * _gap + _dateLabel.height;
        }

        private function updateView():void{
            /*if(Device.isIos()){
                _textLabel.width = 3 * actualWidth / 4;
            }else{
                _textLabel.height = NaN;
            }*/
            _textLabel.height = NaN;
            _textLabel.width = NaN;
            _textLabel.validate();
            /*if(Device.isIos()){
                _textLabel.width = Math.min(StageTextTextEditor(_textLabel).measureText().x, 3 * actualWidth / 4);
                _textLabel.validate();
            }*/
            _textLabel.width = Math.min(_textLabel.maxWidth, _textLabel.width + 5);
            _textLabel.height += 3;
            //
            _mainContainer.width = ownerWidth;
            _mainContainer.height = customHeight + metaContentHeight;
            //
            _background.width = _textLabel.width + 2 * _gap;
            _background.height = customHeight - 2 * _gap - _dateLabel.height;
            _background.y = _gap;
            //
            var selectionColor:uint = 0xbaba54;
            if(isOwnMessage){
                _background.color = isSelected? selectionColor: 0xd3f8ba;
                _background.x = ownerWidth - _background.width - _gap - marginRight;
                AnchorLayoutData(_dateLabel.layoutData).left = NaN;
                AnchorLayoutData(_dateLabel.layoutData).right = 2 * _gap + marginRight;

                AnchorLayoutData(_textLabel.layoutData).left = NaN;
                AnchorLayoutData(_textLabel.layoutData).right = 2 * _gap + marginRight;
            }else{
                _background.color = isSelected? selectionColor: 0xFFFFFE;
                _background.x = _gap + marginLeft;
                AnchorLayoutData(_dateLabel.layoutData).left = 2 * _gap + marginLeft;
                AnchorLayoutData(_dateLabel.layoutData).right = NaN;

                AnchorLayoutData(_textLabel.layoutData).left = 2 * _gap + marginLeft;
                AnchorLayoutData(_textLabel.layoutData).right = NaN;
            }
            var textBottom:Number = _background.y + _background.height + _gap;
            if(_metaContainer){
                _metaContainer.y = textBottom;
            }

            if(_imageContainer){
                _imageContainer.y = _metaContainer?(_metaContainer.height + textBottom):textBottom;
            }
        }

        private function onFriendSelectHandler(event:TouchEvent) : void {
            var touch : Touch = event.touches[0] as Touch;
            if (touch.phase == TouchPhase.ENDED && !this.owner.isScrolling) {
                Constants.selectedUser = _user.copy();
                eventDispatcher.dispatchEventWith(StackedScreen.GARAGE_SELECTED);
            }
        }

		public override function dispose() : void {
            if (_textLabel){
                _textLabel.removeFromParent(true);
                _textLabel = null;
            }

            if (_dateLabel){
                _dateLabel.removeFromParent(true);
                _dateLabel = null;
            }

            if (_mainContainer){
                _mainContainer.removeFromParent(true);
                _mainContainer = null;
            }

            if (_background){
                _background.removeFromParent(true);
                _background = null;
            }

            if (_imageContainer){
                _imageContainer.removeFromParent(true);
                _imageContainer = null;
            }
            super.dispose();
        }

		protected override function commitData() : void {
            super.commitData();
            if(!data){
                return;
            }

            var contentWidth:Number = ownerWidth - (marginLeft + marginRight);
            if(data != _prevData){
                _user = new User(data);
                _user.id = data.from;
                _textLabel.height = NaN;
                LabelFactory.setLabelText(_textLabel, data.text);
                var time:Date = DateParser.parseDate(data.created);
                _dateLabel.text = DateParser.getTimeAppropriateText(time);
                _dateLabel.validate();
                //
                _prevData = data;
                updateSize();
                //
                var hasImages:Boolean = false;
                if(_imageContainer){
                    _imageContainer.removeFromParent(true);
                }
                if(data.hasOwnProperty("metadata") && data.metadata is Array){
                    for(var i:int = 0; i < data.metadata.length; i++){
                        if(data.metadata[i].type == "images"){
                            hasImages = true;
                            _imageContainer = new LayoutGroup();
                            var vLayout:VerticalLayout = new VerticalLayout();
                            vLayout.gap = _gap;
                            vLayout.paddingLeft = vLayout.paddingRight = vLayout.paddingTop = _gap;
                            _imageContainer.layout = vLayout;
                            var lData:AnchorLayoutData = new AnchorLayoutData();
                            if(isOwnMessage){
                                lData.right = marginRight;
                                vLayout.horizontalAlign = HorizontalAlign.RIGHT;
                            }else{
                                lData.left = marginLeft;
                                vLayout.horizontalAlign = HorizontalAlign.LEFT;
                            }
                            lData.top = lData.bottom = NaN;
                            lData.topAnchorDisplayObject = _textLabel;
                            lData.bottomAnchorDisplayObject = _dateLabel;
                            _imageContainer.layoutData = lData;
                            _mainContainer.addChildAt(_imageContainer, _mainContainer.getChildIndex(_dateLabel));
                            //
                            var photos:Array = data.metadata[i].photos;
                            for(var j:int = 0; j < photos.length; j++){
                                var imageLoader:ImageLoader = new ImageLoader();
                                imageLoader.source = 'http:' + photos[j];
                                imageLoader.addEventListener(Event.COMPLETE, function(event:Event):void{
                                    var ldr:ImageLoader = event.currentTarget as ImageLoader;
                                    ldr.validate();
                                    var ratio:Number = ldr.width / ldr.height;
                                    if(ldr.width > 2 * contentWidth / 3){
                                        ldr.width = 2 * contentWidth / 3;
                                        ldr.height = ldr.width / ratio;
                                    }
                                    ldr.removeEventListeners();
                                    _imageContainer.validate();
                                    updateSize();
                                    updateView();
                                })
                                _imageContainer.addChild(imageLoader);
                            }
                        }else if(data.metadata[i].type == "url"){
                            _metaContainer = new MessageLinkView(data.metadata[i].meta);
                            _mainContainer.addChildAt(_metaContainer, _mainContainer.getChildIndex(_dateLabel));
                            _metaContainer.width = 2 * contentWidth / 3;
                            _metaContainer.list = owner;
                            _metaContainer.addEventListener(Event.UPDATE, function(e:Event):void{
                               updateSize();
                            });
                            var metaLayout:AnchorLayoutData = new AnchorLayoutData();
                            if(isOwnMessage){
                                metaLayout.right = _gap + marginRight;
                            }else{
                                metaLayout.left = _gap + marginLeft;
                            }
                            _metaContainer.layoutData = metaLayout;
                        }
                    }
                }
            }
		}

        private function get metaContentHeight():Number{
            return (_imageContainer?(_imageContainer.height + _gap):0) + (_metaContainer?(_metaContainer.height + _gap):0);
        }

        private function updateSize():void{
            setSize(ownerWidth, customHeight + metaContentHeight);
        }

        private function get isOwnMessage():Boolean {
            return _currentUserId == (_user?_user.id:null);
        }

        public function set currentUserId(value:String):void {
            _currentUserId = value;
        }
    }
}

import com.despark.tuning3d.view.components.core.LabelFactory;
import com.utils.ObjectUtils;

import feathers.controls.ImageLoader;
import feathers.controls.Label;
import feathers.controls.LayoutGroup;
import feathers.controls.List;
import feathers.core.FeathersControl;
import feathers.layout.AnchorLayoutData;
import feathers.layout.HorizontalAlign;
import feathers.layout.VerticalLayout;

import flash.net.URLRequest;

import flash.net.navigateToURL;

import starling.display.Quad;
import starling.events.Event;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;
import starling.filters.DropShadowFilter;
import starling.filters.GlowFilter;

class MessageLinkView extends FeathersControl{
    private var _titleField:Label;
    private var _urlField:Label;
    private var _descriptionField:Label;
    private var _labelContainer:LayoutGroup;
    private var _data:Object;
    private var _bg:Quad;
    private var _image:ImageLoader;
    private var _list:List;
    public function MessageLinkView(data:Object){
        _data = data;
    }

    override protected function initialize():void{
        super .initialize();
        _bg = new Quad(1, 1, 0xF6F6F6);
        _bg.filter = new GlowFilter(0x000000, 1, 0.2, 1);//new DropShadowFilter(0,0,0,1,0.1,0.1);
        addChild(_bg);

        _labelContainer = new LayoutGroup();
        var vLayout:VerticalLayout = new VerticalLayout();
        vLayout.horizontalAlign = HorizontalAlign.LEFT;
        _labelContainer.layout = vLayout;
        addChild(_labelContainer);

        _titleField = new Label();
        _titleField.textRendererFactory = LabelFactory.mailBodyLinkTitleLabel;
        var label:String = ObjectUtils.getValue(_data, "title")?ObjectUtils.getValue(_data, "title"):ObjectUtils.getValue(_data, "url");
        _titleField.text = label?label:"[URL]";
        _titleField.paddingLeft = 5;
        _titleField.paddingTop = 5;
        _labelContainer.addChild(_titleField);
        _titleField.validate();

        _urlField = new Label();
        _urlField.textRendererFactory = LabelFactory.mailBodyLinkURLLabel;
        _urlField.text = ObjectUtils.getValue(_data, "url");
        _urlField.paddingLeft = 5;
        _urlField.paddingBottom = 5;
        _labelContainer.addChild(_urlField);
        _urlField.validate();

        if(ObjectUtils.getValue(_data, "description")){
            _descriptionField = new Label();
            _descriptionField.paddingLeft = 5;
            _descriptionField.paddingBottom = 5;
            _descriptionField.textRendererFactory = LabelFactory.mailBodyLinkDescLabel;
            _descriptionField.maxWidth = actualWidth - 10;
            _descriptionField.text = _data["description"];
            _labelContainer.addChild(_descriptionField);
            _descriptionField.validate();
        }

        if(ObjectUtils.getValue(_data, "image")){
            _image = new ImageLoader();
            _image.addEventListener(Event.COMPLETE, completeHandler);
            _image.source = _data["image"];
            _image.padding = 2.5;
            var imgLD:AnchorLayoutData = new AnchorLayoutData();
            imgLD.top = _titleField.height + _titleField.paddingTop;
            _image.layoutData = imgLD;
            addChild(_image);
        }

        _labelContainer.validate();
        addEventListener(TouchEvent.TOUCH, touchHandler);
    }

    private function touchHandler(event:TouchEvent):void {
        var touch:Touch = event.getTouch(this, TouchPhase.ENDED);
        if(touch && !_list.isScrolling){
            navigateToURL(new URLRequest(_data.url));
        }
    }

    private function completeHandler(event:Event):void {
        _image.removeEventListener(Event.COMPLETE, completeHandler);
        _image.validate();
        updateView();
    }

    private function updateView():void{
        if(_titleField){
            _titleField.width = actualWidth - 10;
        }
        if(_urlField){
            _urlField.width = actualWidth - 10;
        }
        if(_descriptionField){
            _descriptionField.width = actualWidth - 10;
        }
        if(_labelContainer){
            _labelContainer.validate();
        }
        if(_image && _image.width){
            var ratio:Number = _image.width / _image.height;
            _image.width = Math.min(_image.width, actualWidth);
            _image.height = _image.width / ratio;
            _image.x = (actualWidth - _image.width) / 2;
            _image.y = _labelContainer.height;
        }

        //
        height = _labelContainer.height + (_image?_image.height:0);
        _bg.width = width;
        _bg.height = height;
        dispatchEvent(new Event(Event.UPDATE));
    }

    override protected function draw():void{
        super.draw();

        if(isInitialized){
            updateView();
        }
    }

    public function set list(list:List):void {
        _list = list;
    }
}