package com.despark.tuning3d.view.components.parts
{
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.model.assets.AssetsProxy;
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.parts.PartSlot;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.ListItemRenderer;
import com.utils.DisplayObjectUtil;

import flash.geom.Point;
import flash.geom.Rectangle;
	import flash.utils.getTimer;

	import starling.display.DisplayObject;
import starling.display.Image;
import starling.display.Quad;
import starling.textures.Texture;

public class PartSlotItemRenderer extends ListItemRenderer
	{
		public var partSlotNameWidthsMap : Object;
		public var language : Language;
		
		private var quad : Quad;
		private var HELPER_RECTANGLE : Rectangle = new Rectangle();
		
		public static const WIDTH:Number=105;
		public static const HEIGHT:Number=50;

		private var bagroundSelectedState:Image;
		
		private var bagroundState:Image;
		private var iconImage:Image;
		private var border:Quad;
		private var iconId:String;
		private var _noBorder:Boolean = false;
		private static const iconIdList:Object = {};
		
		override public function dispose():void{
			HELPER_RECTANGLE=null;
			if(partSlotNameWidthsMap){
				for(var id:String in partSlotNameWidthsMap){
					delete partSlotNameWidthsMap[id];
				}
			}
			partSlotNameWidthsMap=null;
			language=null;
			data=null;
			
			super.dispose();
            if(bagroundSelectedState){
                bagroundSelectedState.dispose();
                bagroundSelectedState = null;
            }

            if(bagroundState){
                bagroundState.dispose();
                bagroundState = null;
            }

            if(quad){
                quad.dispose();
				quad = null;
            }

			if(iconImage){
				iconImage.removeFromParent(true);
				iconImage = null;
			}

			if(border){
                border.removeFromParent(true);
				border = null;
			}
		}
				
		public function PartSlotItemRenderer() {
			super();
			fontSize = 9;
			_textColor = 0xffffff;
			_bold=false;
			width = WIDTH;
            height = HEIGHT;
		}

		protected override function initialize() : void {
			quad = new Quad( 1,  1, 0xffffff);
			quad.alpha=0;
			addChild(quad);
			super.initialize();

			label.paddingBottom=0;
			label.paddingTop=0;
			label.paddingLeft=0;
			label.paddingRight=0;    
			imageLoader.visible = false;
			if(!_noBorder){
				border = new Quad(1,1,0xCCCCCC);
				border.width = 1;
				border.height = HEIGHT;
				border.x = WIDTH - 1;
				addChild(border);
			}
		}
		
		public function get partSlot() : PartSlot {
			return data as PartSlot;
		}
		
		protected override function commitData() : void {
			super.commitData();
			if(!language || !partSlot){
				return;
			}
			
			if(partSlot.name != "ALL") {
				label.text = language.translate("componentTypes", partSlot.name);
                label.visible=true;
			} else {
				label.visible=false;
				width = height = 0;
			}

			if(!iconImage || iconId!=partSlot.name.toLowerCase()) {
				if(iconImage){
					iconImage.removeFromParent(true);
					iconImage = null;
				}
                iconId=partSlot.name.toLowerCase();
				if(!loadAndSetIcon() && iconId!='all' && !iconIdList.hasOwnProperty(iconId)){
                    var assetsProxy : AssetsProxy = ApplicationFacade.getInstance().retrieveProxy(AssetsProxy.NAME) as AssetsProxy;
                    assetsProxy.loadComponentIcon(iconId, function():void{
                        loadAndSetIcon();
					});
				}else{
					loadAndSetIcon();
				}
				iconIdList[iconId] = true;
            }
		}
		
		private function loadAndSetIcon():Boolean{
            var iconTexture:Texture = ButtonFactory.assetManager.getTexture(iconId);
            if(iconTexture) {
                iconImage ||= new Image(iconTexture);
                iconImage.scale = ButtonFactory.assetManager.scaleFactor * 40 / 120;
                addChild(iconImage);
                invalidate("iconImage");
                return true;
            }
            return false;
        }

		override protected function draw():void{
			super.draw();
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			var iconInvalid : Boolean = isInvalid("iconImage");
			if(sizeInvalid) {
				quad.width = actualWidth;
				quad.height = actualHeight;
			}
			if(iconImage && iconInvalid){
                iconImage.x = (actualWidth - iconImage.width)/2;
                iconImage.y = Math.max((40 - iconImage.height)/2, 0);
			}
			if(label){
                label.paddingLeft = label.paddingRight = 3;
                label.width = actualWidth;
                label.height = 13;
				label.x = (actualWidth - label.width)/2;
                label.y = actualHeight - label.height;
			}

			//commitSelection();
		}

		protected override function layout() : void {
			super.layout();
			label.width = actualWidth;
            label.height = 13;
            label.y=actualHeight-label.height;
		}
		
		override public function hitTest(localPoint : Point) : DisplayObject {
			if(getBounds(this, HELPER_RECTANGLE).containsPoint(localPoint)) {
				return label;
			}
			
			return super.hitTest(localPoint);
		}
		protected override function commitSelection() :void {
			super.commitSelection();
			textColor = 0xffffff;
			/*if(isSelected){
			}else{
				textColor = 0x414141;
			}*/
		}
		
		public override function get useEmbeddedFont() : Boolean {
			if(!language){
				return null;
			}
			return language.supportsEmbeddedFont;
		}

		public function set noBorder(noBorder:Boolean):void {
			_noBorder = noBorder;
		}
	}
}



