package com.despark.tuning3d.view.components.timeline.ItemRenderers.Abstract {
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.despark.tuning3d.view.components.timeline.FeedData;
import com.despark.tuning3d.view.components.timeline.FeedData;
import com.despark.tuning3d.view.components.timeline.ItemRenderers.Comments.CommentData;
import com.despark.tuning3d.view.components.timeline.ItemRenderers.Comments.CommentInput;
import com.despark.tuning3d.view.components.timeline.ItemRenderers.Comments.CommentItemRenderer;
import com.despark.tuning3d.view.components.timeline.ItemRenderers.IPostFeedItemRenderer;
import com.despark.tuning3d.view.components.timeline.ItemRenderers.Likes.LikeRenderer;
import com.despark.tuning3d.view.components.timeline.components.TimelineLabelFactory;
import com.utils.DisplayObjectUtil;
import com.utils.ObjectUtils;

import feathers.controls.Button;

import feathers.controls.LayoutGroup;
import feathers.controls.List;
import feathers.controls.ScrollBarDisplayMode;
import feathers.controls.ScrollPolicy;
import feathers.controls.ToggleButton;
import feathers.controls.renderers.IListItemRenderer;
import feathers.core.FeathersControl;
import feathers.data.ListCollection;
import feathers.layout.AnchorLayout;
import feathers.layout.HorizontalAlign;
import feathers.layout.HorizontalLayout;
import feathers.layout.HorizontalLayoutData;
import feathers.layout.VerticalAlign;
import feathers.layout.VerticalLayout;

import flash.geom.Point;

import starling.core.Starling;

import starling.display.Image;

import starling.display.Quad;
import starling.display.Sprite;
import starling.events.Event;

public class ATimelineCommentLikeItemRender extends ATimelineBaseItemRenderer {

    public static const COMMENTS_MORE_CYCLE : int = 3;

    protected var _commentsList :         List;
    protected var _commentsTopQuad:       Quad;
    protected var _commentButton: Button;
    protected var _showMoreCommentsButton:    Button;
    protected var _likeButtonNew: ToggleButton;
    protected var _inputComment : CommentInput;
    protected var _inputCommentContainer : LayoutGroup;
    protected var _socialContainer:LayoutGroup;

    protected var _neededPosition : Point;
    protected var likeListWidth : int = 135;
    protected var likeRendererHeight : int = 18;
    protected var _middleLineQuad:Quad;

    private var _commentLoadBar:Sprite;
    public function ATimelineCommentLikeItemRender() {
        super();
    }

    protected override function initAllComponents():void {
        super.initAllComponents();

        var socialLayout : VerticalLayout = new VerticalLayout();
        socialLayout.gap = _bigGap;
        socialLayout.verticalAlign = VerticalAlign.MIDDLE;


        var horizontalLayout3 : HorizontalLayout = new HorizontalLayout();
        horizontalLayout3.paddingBottom = 10;
        horizontalLayout3.paddingTop = 10;
        horizontalLayout3.verticalAlign = VerticalAlign.MIDDLE;

        _socialContainer = new LayoutGroup();
        _socialContainer.layout = socialLayout;
        _infoContainer.addChild(_socialContainer);

        _bottomContainer = new LayoutGroup(); // Like comment button
        _bottomContainer.layout = horizontalLayout3;
        _mainContainer.addChild(_bottomContainer);

        var horizontalLayout : HorizontalLayout = new HorizontalLayout();
        horizontalLayout.gap = 2;
        horizontalLayout.paddingTop = 10;
        horizontalLayout.verticalAlign = VerticalAlign.MIDDLE;

        _middleLineQuad = new Quad(1,2,0x939090);
        _likeButtonNew = ButtonFactory.timelineLikeButtonNew("garage_like");
        _likeButtonNew.addEventListener(Event.TRIGGERED, likeButtonHandler);
        _likeButtonNew.setSize(50, 30);
        _socialContainer.addChild(_likeButtonNew);

        _commentButton = ButtonFactory.timelineCommentButtonNew("garage_comment");
        _commentButton.addEventListener(Event.TRIGGERED, commentButtonTriggeredHandler);
        _commentButton.setSize(50, 30);
        _commentButton.label = "0";
        _socialContainer.addChild(_commentButton);

        _showMoreCommentsButton = ButtonFactory.timelineSocialButton();
        _showMoreCommentsButton.addEventListener(Event.TRIGGERED, onShowAllCommentsHandler);
        _bottomContainer.addChild(_showMoreCommentsButton);

        initializeCommentsList();
    }

    protected function initInputComment():void {
        if(!_inputComment) {
            _inputCommentContainer = new LayoutGroup();

            _inputComment = new CommentInput(true);
            _inputCommentContainer.addChild(_inputComment);
            _inputComment.owner  = feedData;
            _inputComment.setSize(_mainContainer.width, 44);

            var index : int = _mainContainer.getChildIndex(_bottomContainer);
            _mainContainer.addChildAt(_inputCommentContainer, 2);
            _inputComment.width = _mainContainer.width; //actualWidth - 2 * _avatarXpos;

            _inputComment.addEventListener(FeedData.COMMENT_SEND, sendCommentHandler);
            _inputComment.addEventListener(FeedData.COMMENT_CANCEL, cancelCommentHandler);

            _inputComment.visible = false;
            _inputComment.height = 0;
        }

    }

    protected function initializeCommentsList() : void {
        var layout : VerticalLayout = new VerticalLayout();
        layout.gap = _gap;
        layout.hasVariableItemDimensions = true;
        layout.horizontalAlign = HorizontalAlign.LEFT;
        layout.verticalAlign = VerticalAlign.TOP;
        layout.paddingTop = 0;
        layout.paddingBottom = 0;

        _commentsList = new List();
        _commentsList.setSize(contentWidth, 1);
        _commentsList.dataProvider = new ListCollection();
        _commentsList.layout = layout;
        _commentsList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
        _commentsList.horizontalScrollPolicy = ScrollPolicy.OFF;
        _commentsList.verticalScrollPolicy = ScrollPolicy.OFF;
        _commentsList.addEventListener(FeedData.COMMENT_DELETE, deleteCommentHandler);
        _commentsList.addEventListener(FeedData.COMMENT_EDIT, editCommentHandler);
        _commentsList.itemRendererFactory =  commentItemRendererFactory;

        _commentsTopQuad = new Quad(1,1,TimelineLabelFactory.COLOR_TEXT_LITE_GREY);
        _commentsTopQuad.alpha = 0.5;
        _commentsTopQuad.visible = true;

        _commentsList.addChild(_commentsTopQuad);
        _commentsList.visible = false;
        _commentsList.height = 1;
        _commentsList.width = contentWidth;

        var bottomIndex:int = _mainContainer.getChildIndex(_bottomContainer);
        if(bottomIndex != -1){
            _mainContainer.addChildAt(_commentsList,  bottomIndex);
        }else{
            _mainContainer.addChild(_commentsList);
        }
        _mainContainer.height = NaN;
        _mainContainer.validate();
    }

    protected function commentItemRendererFactory() : IListItemRenderer {
        if(!stage) {
            return null;
        }
        var renderer : CommentItemRenderer = new CommentItemRenderer();
        renderer.setSize(contentWidth, NaN);
        renderer.eventDispatcher = eventDispatcher;
        return renderer;
    }

    public function set neededPosition(value:Point) : void{
        _neededPosition = value;
    }
    public function get neededPosition() : Point{
        return _neededPosition;
    }

    protected function likeListRenderer() : IListItemRenderer{
        var renderer : LikeRenderer = new LikeRenderer();
        renderer.setSize(likeListWidth-2,likeRendererHeight);
        renderer.eventDispatcher = this;
        return renderer;
    }

    override protected function autoSizeIfNeeded():Boolean
    {
        var needsWidth : Boolean = isNaN(explicitWidth);
        var needsHeight : Boolean = true;//isNaN(explicitHeight);

        if(!needsWidth && !needsHeight)
        {
            return false;
        }

        _topContainer.height = NaN;
        _topContainer.validate();

         _mainContainer.height = NaN;
        _mainContainer.validate();

        _bottomContainer.height = NaN;
        _bottomContainer.validate();

        _stageGroup.width = itemWidth;
        _stageGroup.height = NaN;
        _stageGroup.validate();

        var newWidth : Number = _stageGroup.width;//explicitWidth;
        var newHeight : Number = _stageGroup.height;//measureHeight();//explicitHeight;


        var dh:Number = 0;
        var autoSize:Boolean = setSizeInternal(newWidth, newHeight+dh, false);

        if(autoSize)
            updateSize();

        return autoSize;
    }


    override protected function commitData():void {
        if(!feedData)
            return;
        //
        if(feedData){
            feedData.holder = this;
            feedData.holder.validate();
        }
        super.commitData();

//        update();
        if(_inputComment && _inputComment.visible)
        {hideCommentInput();}
        if(feedData) {
            _likeButtonNew.isEnabled = true;
            _likeButtonNew.isSelected = feedData.iLiked;
        }else{
            _likeButtonNew.isEnabled = false;
        }
    }


    override public function validate():void {
        super.validate();
        updateLikeView();
    }

    protected function updateLikeView():void {
        if(feedData && _likeButtonNew){
            var counters:Object = ObjectUtils.getProperty(feedData, "tuning.counters");
            if(counters && counters.hasOwnProperty("likes")){
                _likeButtonNew.label = counters.likes.toString();
            }else if (feedData.likes.length > 0) {
                _likeButtonNew.label = feedData.getLikesCountString();
            } else {
                _likeButtonNew.label = "0";
            }
        }
        if(feedData){
            _likeButtonNew.isSelected = feedData.iLiked;
        }
    }


    protected override function updateSize():void{
        super .updateSize();
        if(_inputComment) {
            _inputComment.width = _mainContainer.width; //actualWidth - 2 * _avatarXpos;
            _inputComment.height = _inputComment.height;
        }
        if(_commentsList) {
            _commentsList.width = contentWidth;
        }

        _middleLineQuad.width =  _mainContainer.width;//actualWidth - (_avatarXpos + 2 * _gap + _bigGap + _dateGroup.width);

        _bottomLineQuad.y = actualHeight - _bottomLineQuad.height;
        _bottomLineQuad.width = actualWidth;

        _bgQuad.width = actualWidth;
        _bgQuad.height = actualHeight;

        if(_commentsTopQuad) {
            _commentsTopQuad.width = _mainContainer.width;
            _commentsTopQuad.y = 0;
        }
    }

    override protected function draw():void {
        super.draw();
        layoutInfoContainer();
        _middleLineQuad.width =  _dataContainer.width;
    }

    protected function layoutInfoContainer():void{
        if(_infoContainer){
            var h:Number = _bigGap + _gap;
            for(var i:int = 0; i < _infoContainer.numChildren; i++){
                h += _infoContainer.getChildAt(i).height;
            }
            _infoContainer.height = Math.max(_dataContainer.height + _feedTitleGroup.height + 2 * _gap + 2, h);
            if(_socialContainer && _socialContainer.parent == _infoContainer){
                _socialContainer.y = _infoContainer.height - _socialContainer.height - _gap;
            }
        }
    }

    protected function updateCommentsList() : void {
        if(_commentLoadBar){
            _commentLoadBar.removeFromParent();
        }
        if (feedData){
            if(feedData.commentsPocket && feedData.commentsPocket.totalLength > 0){
                _commentButton.label = FeedData.getNumberString(feedData.commentsPocket.totalLength);
                _commentsList.dataProvider = new ListCollection(feedData.commentsPocket.comments);
                _commentsList.visible = true;
                _commentsList.height = NaN;
                _commentsList.validate();
                if(this is IPostFeedItemRenderer){
                    IPostFeedItemRenderer(this).updateLabel();
                }
            } else {
                _commentButton.label = "0";
                _commentsList.visible = false;
                _commentsList.dataProvider = new ListCollection();
                _commentsList.height = 1;
                _commentsList.validate();
            }

            if(feedData.commentsPocket && feedData.commentsPocket.currentLength < feedData.commentsPocket.totalLength){
                _showMoreCommentsButton.label = feedData.getMoreLabelText();
                if(!_bottomContainer.parent){
                    _mainContainer.addChild(_bottomContainer);
                }
            }else {
                if(_bottomContainer.parent){
                   _bottomContainer.removeFromParent();
                }
            }
        }
        invalidateItem();
    }

    protected function showCommentInput():void{
        _inputCommentContainer.height = 44;
        _inputComment.height = 44;
        _inputComment.visible = true;
        _inputComment.activate();
    }

    protected function hideCommentInput():void{
        if(_inputComment) {
            _inputCommentContainer.height = 0;
            _inputComment.height = 0;
            _inputComment.deactivate();
            _inputComment.visible = false;
        }
    }

    protected function cancelCommentHandler(event:Event):void {
        SoundManager.playSound(SoundManager.SELECT_PART_BACK);
        hideCommentInput();
        invalidateLikeCommentsViews();
    }

    protected function sendCommentHandler(event:Event):void{
        SoundManager.playSound(SoundManager.SELECT_PART);
        hideCommentInput();

        var text : String = event.data.text as String;
        // feedData.commentsPocket.generateComment(text);

        this.owner.dispatchEventWith(FeedData.COMMENT_SEND, false , {data : feedData, text : text});
    }

    protected function commentButtonTriggeredHandler(event:Event):void {
        initInputComment();
        SoundManager.playSound(SoundManager.SELECT_PART);
        if(!_inputComment.visible){
            showCommentInput();
        } else {
            hideCommentInput();
        }
        invalidateLikeCommentsViews(true);
    }

    protected function onShowAllCommentsHandler(event:Event):void {
        if(!feedData) return;
        if(feedData.commentsPocket && feedData.commentsPocket.isNotFull){
            SoundManager.playSound(SoundManager.SELECT_PART);
            createLoadBar();
            _commentLoadBar.removeFromParent();
            _commentLoadBar.x = 2 * _showMoreCommentsButton.width/3+10;
            _commentLoadBar.y = _showMoreCommentsButton.y + (_showMoreCommentsButton.height-_commentLoadBar.height)/2+2;
            _showMoreCommentsButton.addChild(_commentLoadBar);
            addEventListener(Event.ENTER_FRAME, spinLoader);
            this.owner.dispatchEventWith(FeedData.COMMENT_MORE, false, this.feedData);
        }
    }

    private function spinLoader(event:Event):void {
        if(!_commentLoadBar.parent){
            removeEventListener(Event.ENTER_FRAME, spinLoader);
            return;
        }
        _commentLoadBar.rotation += Math.PI / 10;
    }

    private function createLoadBar():void{
        _commentLoadBar ||= new Sprite();
        if(!_commentLoadBar.numChildren){
            var img:Image = new Image(ButtonFactory.assetManager.getTexture("spinner"));
            img.width = img.height = 16;
            img.x = img.y = -img.width/2;
            _commentLoadBar.addChild(img);
        }
    }

    public function invalidateLikeCommentsViews(autoSize:Boolean = false):void {
        updateCommentsList();
        updateLikeView();
        if(autoSize){
            autoSizeIfNeeded();
        }
    }

    protected function likeButtonHandler(event:Event):void {
        this.owner.dispatchEventWith(FeedData.LIKE, false, feedData);
        _likeButtonNew.isEnabled = false;
        _likeButtonNew.isSelected = feedData.iLiked;
        var counters:Object = ObjectUtils.getProperty(feedData, "tuning.counters");
        if(feedData.iLiked){
            SoundManager.playSound(SoundManager.SELECT_PART_BACK);
        }else{
            SoundManager.playSound(SoundManager.SELECT_PART);
        }
        if(counters && counters.hasOwnProperty("likes")){
            if(feedData.iLiked){
                counters.likes = Math.max(counters.likes-1, 0);
            }else{
                counters.likes = Math.max(counters.likes+1, 0);
            }
        }

    }

    protected function editCommentHandler(e : Event) : void {
        SoundManager.playSound(SoundManager.SELECT_PART);
        this.owner.dispatchEventWith(FeedData.COMMENT_EDIT, false, {comment : e.data, feedData : feedData});
    }

    protected function deleteCommentHandler(event:Event):void {
        SoundManager.playSound(SoundManager.SELECT_PART);
        var commentData : CommentData = event.data as CommentData;
        if(feedData.commentsPocket.deleteComment(commentData)){
            this.owner.dispatchEventWith(FeedData.COMMENT_DELETE, false, {comment : commentData, feedData : feedData});
        } else {
            //tracer.warn("Comment not found to delete");
        }
    }

    override public function dispose():void {
        super.dispose();

        if (_commentButton){
            _commentButton.removeFromParent(true);
            _commentButton.removeEventListener(Event.TRIGGERED, commentButtonTriggeredHandler);
            _commentButton = null;
        }

        if (_showMoreCommentsButton){
            _showMoreCommentsButton.removeFromParent(true);
            _showMoreCommentsButton.removeEventListener(Event.TRIGGERED, onShowAllCommentsHandler);
            _showMoreCommentsButton = null;
        }

        if (_likeButtonNew){
            _likeButtonNew.removeFromParent(true);
            _likeButtonNew.removeEventListener(Event.TRIGGERED, likeButtonHandler);
            _likeButtonNew = null;
        }

        if (_inputComment) {
            _inputComment.removeEventListener(FeedData.COMMENT_SEND, sendCommentHandler);
            _inputComment.removeEventListener(FeedData.COMMENT_CANCEL, cancelCommentHandler);
            _inputComment.removeFromParent(true);
            _inputComment = null;
        }

        if (_commentsList) {
            _commentsList.dataProvider = null;
            _commentsList.removeEventListener(FeedData.COMMENT_DELETE, deleteCommentHandler);
            _commentsList.removeEventListener(FeedData.COMMENT_EDIT, editCommentHandler);
            _commentsList.removeFromParent(true);
            _commentsList = null;
        }
        DisplayObjectUtil.dispose(this, true, true);
    }

    public function enableLike():void {
        _likeButtonNew.isEnabled = true;
    }
}
}
