/**
 * Created by Render-2 on 8/9/2017.
 */
package com.despark.tuning3d.view.components.timeline.ItemRenderers.Comments
{
import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.LabelFactory;
import com.despark.tuning3d.view.components.core.ListItemRenderer;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.despark.tuning3d.view.components.core.MultilingualLabel;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.timeline.*;
import com.despark.tuning3d.view.components.timeline.components.TimelineLabelFactory;
import com.despark.tuning3d.view.components.timeline.components.UserCircularAvatar;
import com.utils.DateParser;

import feathers.controls.Label;
import feathers.controls.LayoutGroup;
import feathers.controls.TextInput;
    import feathers.controls.text.StageTextTextEditor;
    import feathers.core.FeathersControl;
    import feathers.layout.AnchorLayout;
import feathers.layout.AnchorLayoutData;
import feathers.layout.HorizontalLayout;
import feathers.layout.VerticalAlign;
import feathers.layout.VerticalLayout;

    import flash.display.Stage;

    import flash.text.TextFormat;

import starling.display.Image;
import starling.display.Quad;
import starling.events.Event;
    import starling.events.EventDispatcher;
    import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;

public class CommentItemRenderer extends ListItemRenderer {

    protected var _nameLabel:FeathersControl;
    protected var _commentLabel:FeathersControl;
    protected var _dateLabel:Label;
    protected var _gap:int;
    protected var _bottomQuad:Quad;
    protected var _editButton:MultilingualButton;
    protected var _deleteButton:MultilingualButton;
    protected var _stageGroup:LayoutGroup;
    protected var _bottomLayoutGroup:LayoutGroup;

    protected static const buttonWidth: Number = 35;
    protected static const buttonheight: Number = 15;

    protected static const avatarSize : int = 26;
    protected var _topLayout:LayoutGroup;
    protected var _avatar:UserCircularAvatar;

    protected var _eventDispatcher : EventDispatcher;
    private var _textInput:EditCommentInput;

    protected var _isEditEnabled:Boolean;

    public function CommentItemRenderer() {
        _gap = 8;
        _isEditEnabled = true;
    }

    protected function get commentData() : CommentData {
        return _data as CommentData;
    }

    override protected function initialize():void
    {
        super.initialize();


        var verticalLayout : VerticalLayout = new VerticalLayout();
        verticalLayout.gap = _gap;
        verticalLayout.paddingTop = 0;

        _stageGroup = new LayoutGroup();
        _stageGroup.layout = verticalLayout;
        addChild(_stageGroup);

        var topL : HorizontalLayout = new HorizontalLayout();
        topL.gap = _gap;
        topL.verticalAlign = VerticalAlign.MIDDLE;

        _topLayout = new LayoutGroup();
        _topLayout.layout = topL;
        _stageGroup.addChild(_topLayout);


        _nameLabel = LabelFactory.createLabelComment();
        _nameLabel.addEventListener(TouchEvent.TOUCH, onCreatorGarageSelectHandler);
        _topLayout.addChild(_nameLabel);

        if(Constants.isIOS) {
            _commentLabel = TimelineLabelFactory.textInputRenderer() as FeathersControl;
        }else{
            _commentLabel = new Label();
            Label(_commentLabel).textRendererFactory = TimelineLabelFactory.commentInputRenderer;
            Label(_commentLabel).wordWrap = true;
        }
        _stageGroup.addChild(_commentLabel);

        _bottomLayoutGroup = new LayoutGroup();
        _stageGroup.addChild(_bottomLayoutGroup);
        _bottomLayoutGroup.layout = new AnchorLayout();

        var dateLayout : HorizontalLayout = new HorizontalLayout();
        dateLayout.gap = 3;
        dateLayout.verticalAlign = VerticalAlign.MIDDLE;

        var dateContainer : LayoutGroup = new LayoutGroup();
        dateContainer.layout = dateLayout;
        _bottomLayoutGroup.addChild(dateContainer);

        var _dateIcon : Image = new Image(AlertView.assetManager.getTexture("dateIcon"));
        _dateIcon.width = _dateIcon.height = 11;
        dateContainer.addChild(_dateIcon);

        _dateLabel = new Label();
        _dateLabel.textRendererFactory = TimelineLabelFactory.commentDateTextRenderer;
        dateContainer.addChild(_dateLabel);
        _bottomLayoutGroup.width = actualWidth;

        if(_isEditEnabled)
        {
            var buttonsLayoutData : AnchorLayoutData = new AnchorLayoutData();
            buttonsLayoutData.right = 15;

            var buttonsContainer : LayoutGroup = new LayoutGroup();
            buttonsContainer.layout = new HorizontalLayout();
            buttonsContainer.layoutData = buttonsLayoutData;
            _bottomLayoutGroup.addChild(buttonsContainer);

            _editButton = ButtonFactory.timelineCommentButtons("edit  |");
            _editButton.labelFactory = TimelineLabelFactory.commentEditTextRenderer;
            _editButton.setSize(30,buttonheight);
            _editButton.addEventListener(Event.TRIGGERED, onEditSelectedHandler);
            buttonsContainer.addChild(_editButton);

            _deleteButton = ButtonFactory.timelineCommentButtons("delete ");
            _deleteButton.labelFactory = TimelineLabelFactory.commentEditTextRenderer;
            _deleteButton.addEventListener(Event.TRIGGERED, onDeleteButtonTriggeredHandler);
            _deleteButton.setSize(40,buttonheight);
            buttonsContainer.addChild(_deleteButton);
        }


        _bottomQuad = new Quad(1,1,TimelineLabelFactory.COLOR_TEXT_LITE_GREY);
        _bottomQuad.alpha = 0.5;
        _stageGroup.addChild(_bottomQuad);

    }

    protected function updateAvatar():void {
        return;
        if(!_avatar )
        {
            _avatar = new UserCircularAvatar(commentData.owner, avatarSize);
            _topLayout.addChildAt(_avatar, 0);

        }else {
            _avatar.user = commentData.owner;
        }
    }

    public function set eventDispatcher(value:EventDispatcher) : void{
        _eventDispatcher = value;
    }
    public function get eventDispatcher() : EventDispatcher{
        return _eventDispatcher;
    }

    protected function onCreatorGarageSelectHandler(event:TouchEvent) : void {
        var touch : Touch = event.getTouch(_nameLabel, TouchPhase.ENDED);
        if (touch && !_owner.isScrolling) {
            Constants.selectedUser =  commentData.owner;
            eventDispatcher.dispatchEventWith(StackedScreen.USER_DETAILS_SELECTED, false, {user: Constants.selectedUser});
        }
    }

    override protected function autoSizeIfNeeded():Boolean{
        var needsWidth : Boolean = isNaN(explicitWidth);
        var needsHeight : Boolean = true;//isNaN(explicitHeight);

        if(!needsWidth && !needsHeight)
        {
            return false;
        }

        _stageGroup.width = stage.width;
        _stageGroup.height = NaN;
        _stageGroup.validate();

        var newWidth : Number = _stageGroup.width;//explicitWidth;
        var newHeight : Number = _stageGroup.height;//measureHeight();//explicitHeight;
        var dh:Number = 0;
        var autoSize:Boolean = setSizeInternal(newWidth, newHeight+dh, false);

        height = newHeight;
        return autoSize;
    }

    override protected function layout():void {
        super.layout();

        _bottomQuad.width = actualWidth;
    }


    protected function onEditSelectedHandler(event:Event):void {
        SoundManager.playSound(SoundManager.SELECT_PART);
        if(!_textInput)
        {
            initTextInput();
        }
        editState();
        _textInput.text = _commentLabel["text"];
    }

    protected function initTextInput () : void
    {
        _textInput = new EditCommentInput();
        _textInput.setSize(actualWidth-18, actualHeight);
        _textInput.addEventListener(FeedData.COMMENT_EDIT, onUpdateComment);
        _textInput.addEventListener(EditCommentInput.HIDE, onHideEditComment);
    }


    protected function editState() : void{

        this.addChild(_textInput);

        _stageGroup.visible = false;
        _textInput.visible = true;
        _textInput.activate();
    }
    protected function stableState() : void
    {
        _stageGroup.visible = true;
        if(_textInput && _textInput.parent)
        {
            this.removeChild(_textInput);
            _textInput.deactivate();
            _textInput.visible = false;
        }
    }

    protected function onDeleteButtonTriggeredHandler(event:Event):void {
        SoundManager.playSound(SoundManager.SELECT_PART);
        this.owner.dispatchEventWith(FeedData.COMMENT_DELETE, false, commentData);
    }

    override protected function commitSelection():void {

    }

    override protected function commitData():void {
        super.commitData();

        update();

    }

    protected function update() : void {
        if(commentData) {
            commentData.itemRenderer = this;
            stableState();
            updateAvatar();
            _nameLabel["text"] = commentData.owner.name ? commentData.owner.name : commentData.owner.username;
            _nameLabel.height = NaN;
            _nameLabel.validate();

            _commentLabel["text"] = commentData.comment;
            _commentLabel.width = actualWidth - (2 * _gap);
            _commentLabel.height = NaN;
            if(_commentLabel is StageTextTextEditor){
                StageTextTextEditor(_commentLabel).height = StageTextTextEditor(_commentLabel).measureText().y + 3;
            }
            _commentLabel.validate();

            _dateLabel.text = DateParser.getDateAppropriateText(commentData.availableDate);
            _dateLabel.height = NaN;
            _dateLabel.validate();

            if(_isEditEnabled)
            {
                if(commentData.owner.id && commentData.owner.id == FeedData.CURRENT_USER.id)
                {
                    _editButton.visible = true;
                    _deleteButton.visible = true;
                }else {
                    _editButton.visible = false;
                    _deleteButton.visible = false;
                }
            }


            if(commentData.id == null || commentData.id.length == 0)
            {
                if(_isEditEnabled)
                {
                    this._deleteButton.isEnabled = false;
                    this._editButton.isEnabled = false;
                }
                this.alpha = .5;
            }else {
                if(_isEditEnabled)
                {
                    this._deleteButton.isEnabled = true;
                    this._editButton.isEnabled = true;
                }
                this.alpha = 1;
            }


            autoSizeIfNeeded();
        }
    }

    override public function dispose():void
    {
        if(_avatar)
        {
            _avatar.removeFromParent(true);
            _avatar = null;
        }

        if (_nameLabel){
            _nameLabel.removeEventListener(TouchEvent.TOUCH, onCreatorGarageSelectHandler);
            _nameLabel.removeFromParent(true);
            _nameLabel = null;
        }

        if (_editButton){
            _editButton.removeEventListener(Event.TRIGGERED, onEditSelectedHandler);
            _editButton.removeFromParent(true);
            _editButton = null;
        }

        if (_deleteButton) {
            _deleteButton.removeEventListener(Event.TRIGGERED, onDeleteButtonTriggeredHandler);
            _deleteButton.removeFromParent(true) ;
            _deleteButton = null;
        }


        if (_textInput) {
            _textInput.removeEventListener(FeedData.COMMENT_EDIT, onUpdateComment);
            _textInput.removeEventListener(EditCommentInput.HIDE, onHideEditComment);
            _textInput.removeFromParent(true);
            _textInput = null;
        }

        super.dispose();
    }

    private function onHideEditComment(event:Event):void {
        SoundManager.playSound(SoundManager.SELECT_PART_BACK);
        stableState();
    }

    private function onUpdateComment(event:Event):void {
        commentData.comment = _textInput.text;
        stableState();
        update();
        this.owner.dispatchEventWith(FeedData.COMMENT_EDIT, false, commentData);
    }

    override protected function draw():void
    {
        super.draw();
        if(isInvalid(INVALIDATION_FLAG_SIZE) || isInvalid(INVALIDATION_FLAG_DATA)) {

            update();
            _bottomLayoutGroup.width = actualWidth;
            _bottomLayoutGroup.validate();
            _bottomLayoutGroup.readjustLayout();
        }
    }
}
}
