package com.despark.tuning3d.view.components.timeline.ItemRenderers {
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.model.core.Constants;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.LabelFactory;
import com.despark.tuning3d.view.components.timeline.FeedData;
import com.despark.tuning3d.view.components.timeline.ItemRenderers.Abstract.ATimelineCommentLikeItemRender;
import com.despark.tuning3d.view.components.timeline.components.TimelineLabelFactory;
import com.despark.tuning3d.view.components.timeline.components.TimelineURLSnippet;
import com.utils.DateParser;

import feathers.controls.Label;
import feathers.controls.text.StageTextTextEditor;
import feathers.core.FeathersControl;
import feathers.layout.VerticalLayout;
import feathers.layout.VerticalLayout;

import flash.geom.Rectangle;
import flash.net.URLRequest;
import flash.net.navigateToURL;

import starling.display.Image;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;

public class FeedTextItemRenderer extends ATimelineCommentLikeItemRender implements IPostFeedItemRenderer{

    protected var _postText : FeathersControl;
    
    private var user:User;
    private var creator:User;
    private var _postLabel:FeathersControl;
    private var _snippet:TimelineURLSnippet;
    public function FeedTextItemRenderer() {
        super();
    }

    override protected function initAllComponents():void {
        super.initAllComponents();
        if(Constants.isIOS){
            _postText = TimelineLabelFactory.textInputRenderer() as FeathersControl;
        }else {
            _postText = new Label();
            Label(_postText).textRendererFactory = TimelineLabelFactory.textLabelRenderer;
            Label(_postText).wordWrap = true;
        }
        _postText.width = _mainContainer.width;
        _dataContainer.addChildAt(_postText, 0);
        var dataLayout:VerticalLayout = new VerticalLayout();
        dataLayout.padding = _gap;
        dataLayout.paddingBottom = 0;
        _dataContainer.layout = dataLayout;

        _border = new Image(ButtonFactory.assetManager.getTexture('timeline_feed_border'));
        _border.scale9Grid = new Rectangle(_border.texture.frameWidth / 3, _border.texture.frameHeight / 3, _border.texture.frameWidth / 3, _border.texture.frameHeight / 3);
        //
        _dataContainer.backgroundSkin = _border;
    }

    override protected function draw():void {
        super.draw();
        if(_dataContainer.height < _infoContainer.height - (_feedTitleGroup.height + 2 * _gap + 2)){
            _dataContainer.height = _infoContainer.height - (_feedTitleGroup.height + 2 * _gap + 2);
        }
    }

    override protected function update():void {
        if(!feedData) {
            return;
        }
        updatePostText();
        updateLikeView();
        updateCommentsList();
        _dateLabel.text = DateParser.getDateAppropriateText(feedData.availableDate);
        _dateLabel.visible = true;
        if(!feedData.iLiked && feedData.likes && feedData.likes.length){
            for(var i:int = 0; i < feedData.likes.length; i++){
                var obj:Object = feedData.likes[i];
                if(obj.hasOwnProperty("user") && obj.user.hasOwnProperty("_id") && user && obj.user._id == user.id){
                    feedData.iLiked = true;
                    break;
                }
            }
        }
        if(feedData.feedActualType == FeedData.TYPE_POST_COMMENT || feedData.feedActualType == FeedData.TYPE_POST_LIKE){
            updateLabel();
            //return;
        }else{
            _nameLabel["text"] = feedData.nameLabel;
            _nameLabel.visible = true;

            _descriptionLabel["text"] = feedData.descriptionLabel;
            _descriptionLabel.visible = true;

            _friendNameLabel["text"] = "" ;
            _friendNameLabel.visible = false;
        }


        if(feedData.metadataList && feedData.metadataList.length){
            if(_snippet){
                _snippet.update(feedData.metadataList[0]);
            }else{
                _snippet = new TimelineURLSnippet(feedData.metadataList[0]);
                _snippet.width = contentWidth;
                _snippet.addEventListener(TouchEvent.TOUCH, snippetSelectHandler);
                VerticalLayout(_dataContainer.layout).paddingBottom = _gap;
            }
            if(!_snippet.parent){
                _dataContainer.addChild(_snippet);
            }
        }else if(_snippet){
            _snippet.removeFromParent(true);
            _snippet = null;
        }
        _dataContainer.validate();
        if(imageLoader && imageLoader.parent){
            imageLoader.removeFromParent();
        }
        height = NaN;
        if(_bgQuad){
            _bgQuad.height = NaN;
        }
        customAutosize();
    }

    private function snippetSelectHandler(event:TouchEvent):void{
        if(_snippet && !owner.isScrolling) {
            var touch:Touch = event.getTouch(_snippet, TouchPhase.ENDED);
            if(touch){
                navigateToURL(new URLRequest(_snippet.url));
            }
        }
    }

    private function getDescription():String{
        if(feedData.feedActualType == FeedData.TYPE_POST_COMMENT){
            return "commented on";
        }
        if(feedData.feedActualType == FeedData.TYPE_POST_LIKE){
            return "liked";
        }
        if(feedData.feedActualType == FeedData.TYPE_TEXT){
            return "added a new post";
        }
        return '';
    }

    public function updateLabel():void{
        // feedData.creator
        // feedData.owner
        creator ||= feedData.creator;
        user  = (ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy).currentUser;
        // feedData.creator = feedData.commentsPocket.comments[feedData.commentsPocket.comments.length-1].owner;
        _nameLabel["text"] = getNameToShow(feedData.creator);
        _descriptionLabel["text"] = getDescription();
        if(feedData.feedActualType == FeedData.TYPE_POST_COMMENT || feedData.feedActualType == FeedData.TYPE_POST_LIKE){
            feedData.owner ||= feedData.currentUser;

        }
        _friendNameLabel["text"] = getNameToShow(feedData.owner, true);
        _friendNameLabel.visible = true;
        _postLabel ||= LabelFactory.createLabelTimelineDesc();
        _postLabel["text"] = (feedData.feedActualType != FeedData.TYPE_TEXT && feedData.feedActualType != FeedData.TYPE_TEXT_AND_IMAGES?"post":"");
        _postLabel.x = _friendNameLabel.x + _friendNameLabel.width;
        _postLabel.y = _friendNameLabel.y;
        _feedTitleGroup.addChild(_postLabel);
        //
        if(_snippet){
            _snippet.width = contentWidth;
        }
        //

        _nameLabel.invalidate(INVALIDATION_FLAG_ALL);
        _descriptionLabel.invalidate(INVALIDATION_FLAG_ALL);
        _friendNameLabel.invalidate(INVALIDATION_FLAG_ALL);

        height = NaN;
        setRequiresRedraw();
    }


    override public function dispose():void {
        super.dispose();
        if(_snippet){
            _snippet.removeFromParent(true);
            _snippet = null;
        }
        if(_postLabel){
            _postLabel.removeFromParent(true);
            _postLabel = null;
        }
    }

    private function getNameToShow(userData:User, possessive:Boolean = false):String{
        if(!userData){
            return "";
        }
        if(userData.id == user.id){
            if(possessive){
                return "My";
            }
            return "I";
        }
        return userData.nameToShow + (possessive?"'s":"");
    }

    protected function updatePostText() : void {
        if(feedData.text){
            _postText.touchable ||= true;
            _postText.width = contentWidth;
            //_postText.height = NaN;
            if(_postText.hasOwnProperty("htmlText")){
                _postText["htmlText"] = feedData.text + '<br/> ';
            }else{
                _postText["text"] = feedData.text + '\n ';
            }
            if(_postText is StageTextTextEditor){
                StageTextTextEditor(_postText).height = StageTextTextEditor(_postText).measureText().y+3;
            }
            _postText.validate();
            invalidateItem();
        }
    }



    override protected function initIcon() : void{
        _postIcon = new Image(iconsMap[FeedData.TYPE_TEXT]);
        layoutIcon();
    }

    public function get snippet():TimelineURLSnippet {
        return _snippet;
    }
}
}
