package com.despark.tuning3d.view.components.timeline.ItemRenderers {
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ListItemRenderer;

import feathers.controls.Button;
import feathers.core.FeathersControl;

import starling.display.DisplayObject;
import starling.display.Image;
import starling.events.Event;
import starling.textures.Texture;

public class UploadImageItemRenderer extends ListItemRenderer{

    public function UploadImageItemRenderer(){

    }

    public static const CLOSE_IMAGE : String = 'closeImage';

    private var _itemWidth : Number;
    private var _texture : Texture;
    private var _image : DisplayObject;
    private var _closeButtonTexture : Texture;
    private var _closeButton:Button;

    override protected function initialize():void {
        super.initialize();

        _closeButtonTexture = AlertView.assetManager.getTexture("garage_close");
    }

    override public function dispose():void {
        super.dispose();

        _image.removeFromParent(true);
        if(_texture){
            _texture.dispose();
        }
        _closeButton.removeFromParent(true);
    }


    override protected function commitData():void {
        super.commitData();



        if(data.bitmapData && !_image){

            _texture = Texture.fromBitmapData(data.bitmapData);
            _image = new Image(_texture);

        }else if(data.quad) {
            _image = data.quad;
        }

        if(!_image.parent){
            addChild(_image);
        }

        var aspect : Number = _image.width / _image.height;
        _image.width = _itemWidth;
        _image.height = _itemWidth / aspect;

        if(!_closeButton){
            _closeButton = new Button();
            _closeButton.defaultSkin = new Image(_closeButtonTexture);
            _closeButton.downSkin = new Image(_closeButtonTexture);
            _closeButton.downSkin.alpha = 0.6;
            _closeButton.addEventListener(Event.TRIGGERED, closeButtonTriggered);
            addChild(_closeButton)
        }

        _closeButton.width = _closeButton.height = 20;
        _closeButton.x = _itemWidth - _closeButton.width - 5;
        _closeButton.y = 5;
    }

    private function closeButtonTriggered(event:Event):void {
        if(this.owner){
            this.owner.dispatchEventWith(CLOSE_IMAGE, false, data);
        }
    }


    override protected function draw():void {
        super.draw();

        if(_image){

            _closeButton.width = _closeButton.height = 20;
            _closeButton.x = _itemWidth - _closeButton.width - 5;
            _closeButton.y = 5;
        }
    }

    public function set itemWidth(value:Number):void {
        _itemWidth = value;
    }

    override protected function autoSizeIfNeeded():Boolean {
        if(_image){
            setSizeInternal(this._image.width, this._image.height, false);
        }
        return true;
    }
}
}
