package com.despark.tuning3d.view.components.timeline.TimelineGarage {
import com.despark.tuning3d.model.friend.Friend;
import com.despark.tuning3d.model.friend.IFriendHolder;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.view.components.auth.ProfileAlertView;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.CustomAlertView;

    import feathers.controls.Button;

import starling.display.DisplayObject;

import starling.display.Image;
import starling.events.Event;

public class TimelineAddFriendButton extends Button implements IFriendHolder{

    private var _imageMap : Object;
    private var _friend : Friend;

    public function TimelineAddFriendButton(friend:Friend) {
        _friend = friend;
        _friend.holder = this;

        _imageMap = {};
        this.addEventListener(Event.TRIGGERED, triggeredHandler);

        Friend.PROXY.getUserFriendStatus(_friend);
    }

    public function updateFriendState(state:String):void {
        updateFriendButtonState();
    }

    private function triggeredHandler(event:Event):void {
        switch (_friend.state) {
            case Friend.STATE_FRIEND:
                ProfileAlertView.show("Confirm Unfriend", "Are you sure you want to unfriend this user?", 0, function():void{
                    Friend.PROXY.undo_friend(_friend);
                })
                return;
            case Friend.STATE_NO_FRIEND:
                SoundManager.playSound(SoundManager.SELECT_PART);
                Friend.PROXY.add_friend(_friend);
                break;
            case Friend.STATE_REQUEST_SENT:
                return;
            case Friend.STATE_PENDING:
                SoundManager.playSound(SoundManager.SELECT_PART);
                Friend.PROXY.approveRequest(_friend);
                break;
        }
    }



    override protected function initialize():void {
        super.initialize();

        updateFriendButtonState();
    }

    private function updateFriendButtonState() : void
    {
        if(_friend)
        {
            this.defaultSkin = getTextureByState(_friend.state);
        }
    }



    private function getTextureByState(st : String) : DisplayObject
    {
        if(_imageMap[st])
                return _imageMap[st];

        switch (st)
        {
            case Friend.STATE_FRIEND:
                    _imageMap[st]= ButtonFactory.imageByTexture("garage_already_friend");
                break;
            case Friend.STATE_NO_FRIEND:
                    _imageMap[st]= ButtonFactory.imageByTexture("garage_add_friend");

                break;
            case Friend.STATE_REQUEST_SENT:
            case Friend.STATE_PENDING:
                _imageMap[Friend.STATE_REQUEST_SENT]= ButtonFactory.imageByTexture("garage_pending_friend");
                _imageMap[Friend.STATE_PENDING] = _imageMap[Friend.STATE_REQUEST_SENT];
                break;
        }

        return _imageMap[st];
    }

    override public function dispose():void {
        super.dispose();

        for each (var i : Image in _imageMap)
        {
            i.dispose();
        }
    }
}
}
