package com.despark.tuning3d.view.components.tunings {
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.utils.FontSelector;

import feathers.controls.Label;
import feathers.controls.LayoutGroup;
import feathers.controls.ToggleButton;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.ITextRenderer;
import feathers.core.ToggleGroup;
import feathers.layout.HorizontalLayout;
import feathers.layout.VerticalAlign;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.display.Quad;
import starling.events.Event;

public class TuningDeletePrompt extends LayoutGroup {
    private var _onChange:Function;
    private var _onCancel:Function;
    //
    private var _areSureWantToDeleteLabel:MultilingualButton;
    private var currentFrameButton : ToggleButton;
    private var tuningButton : ToggleButton;
    private var orLabel:Label;
    private var toggleGroup:ToggleGroup;
    //
    private var currentText:String = '1 x Snapshot';
    private var entireText:String = 'Full Tuning';
    //
    public function TuningDeletePrompt() {
        super();
    }

    override protected function initialize():void {
        super.initialize();

        _areSureWantToDeleteLabel = new MultilingualButton();
        _areSureWantToDeleteLabel.textFormat = new TextFormat(FontSelector.getMonserratMedium(), 11, 0xFFFFFF);
        _areSureWantToDeleteLabel.textFormat.align = TextFormatAlign.LEFT;
        _areSureWantToDeleteLabel.textFormat.bold = false;
        _areSureWantToDeleteLabel.paddingLeft = 8;
        _areSureWantToDeleteLabel.paddingRight = 8;
        _areSureWantToDeleteLabel.wordWrap = true;
        _areSureWantToDeleteLabel.defaultSkin = new Quad(1, 1, 0x000000);

        _areSureWantToDeleteLabel.defaultSkin.alpha = 0;
        _areSureWantToDeleteLabel.downSkin = new Quad(1, 1, 0x000000);
        _areSureWantToDeleteLabel.downSkin.alpha = 0.0;
        _areSureWantToDeleteLabel.updateWithLanguage(AlertView.language);
        _areSureWantToDeleteLabel.isHTML = true;
        _areSureWantToDeleteLabel.label = 'Choose and press <font color="#FF0000">Delete</font>';

        _areSureWantToDeleteLabel.width = 70;
        _areSureWantToDeleteLabel.height = NaN;

        orLabel = new Label();
        orLabel.text = 'or';
        orLabel.textRendererFactory = function (): ITextRenderer {
            var textRenderer: TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMonserratMedium(), 11, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts =  true;
            return textRenderer;
        };

        currentFrameButton = ButtonFactory.tuningDeleteButton(currentText, "frame_del_icon");
        currentFrameButton.width = 110;

        tuningButton = ButtonFactory.tuningDeleteButton(entireText, "tuning_del_icon");
        tuningButton.iconOffsetY = -1;
        tuningButton.width = 110;

        toggleGroup = new ToggleGroup();
        toggleGroup.isSelectionRequired = false;
        tuningButton.toggleGroup = toggleGroup;
        currentFrameButton.toggleGroup = toggleGroup;
        toggleGroup.addEventListener(Event.CHANGE, changeHandler);
        //
        var bLayout:HorizontalLayout = new HorizontalLayout();
        bLayout.padding = 5;
        bLayout.gap = 10;
        bLayout.verticalAlign = VerticalAlign.MIDDLE;
        //
        addChild(_areSureWantToDeleteLabel);
        addChild(tuningButton);
        addChild(orLabel);
        addChild(currentFrameButton);
        layout = bLayout;

    }

    override public function dispose():void {
        super.dispose();
        if(_areSureWantToDeleteLabel){
            _areSureWantToDeleteLabel.removeFromParent(true);
            _areSureWantToDeleteLabel = null;
        }

        if(orLabel){
            orLabel.removeFromParent(true);
            orLabel = null;
        }

        if(currentFrameButton){
            currentFrameButton.removeFromParent(true);
            currentFrameButton = null;
        }

        if(tuningButton){
            tuningButton.removeFromParent(true);
            tuningButton = null;
        }

        toggleGroup = null;
    }

    private function changeHandler(event:Event):void {
        if(_onChange != null){
            _onChange.call();
        }
    }

    public function get isCurrentFrame():Boolean{
        return currentFrameButton.isSelected;
    }

    public function get isSelected():Boolean{
        return currentFrameButton.isSelected || tuningButton.isSelected;
    }

    public function set onChange(value:Function):void {
        _onChange = value;
    }

    public function set onCancel(value:Function):void {
        _onCancel = value;
    }

    public function set isSelected(value:Boolean):void {
        if(value) {
            tuningButton.isSelected = value;
        }else{
            currentFrameButton.isSelected = tuningButton.isSelected = value;
        }
    }
}
}
