package com.despark.tuning3d.view.components.tunings {
import com.despark.tuning3d.ApplicationFacade;
import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.timeline.TimelineProxy;
import com.despark.tuning3d.model.tunings.Tuning;
import com.despark.tuning3d.model.users.User;
import com.despark.tuning3d.model.users.UsersProxy;
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.GridPageIndicator;
import com.despark.tuning3d.view.components.core.ListItemRenderer;
import com.distriqt.extension.application.Application;
import com.utils.DisplayObjectUtil;

import feathers.controls.List;
import feathers.controls.PageIndicator;
import feathers.controls.PageIndicatorInteractionMode;
import feathers.controls.ScrollBarDisplayMode;
import feathers.controls.ScrollPolicy;
import feathers.controls.renderers.IListItemRenderer;
import feathers.data.ListCollection;
import feathers.layout.HorizontalAlign;
import feathers.layout.HorizontalLayout;
import feathers.layout.VerticalAlign;

import starling.core.Starling;
import starling.display.DisplayObject;
import starling.display.Image;
import starling.display.Quad;
import starling.events.Event;

public class TuningItemRenderer extends ListItemRenderer {
    private var _pageIndicator:PageIndicator;
    private var slidesList:List;

    private var clipRectQuad:Quad;
    private var challengeIcon:Image;
    public var clipBackground:Boolean = false;
    private var _overrideDispose:Boolean = true;
    private var _indicatorMargin:Number = 100;
    private var _fitContent:Boolean = false;
    private var scrollStarted:Boolean = false;

    public function get tuning():Tuning {
        return data as Tuning;
    }

    protected override function initialize():void {
        clipRectQuad = new Quad(1, 1);
        super.initialize();
        overrideDispose = _overrideDispose;
        this.label.visible = false;
        var layout:HorizontalLayout = new HorizontalLayout();
        layout.gap = 0;
        layout.horizontalAlign = HorizontalAlign.CENTER;
        layout.verticalAlign = VerticalAlign.MIDDLE;
        slidesList = new List();
        slidesList.minimumPageThrowVelocity = 1;
        slidesList.dataProvider = new ListCollection();

        slidesList.layout = layout;
        slidesList.snapToPages = true;
        slidesList.hasElasticEdges = false;
        slidesList.scrollBarDisplayMode = ScrollBarDisplayMode.NONE;
        slidesList.horizontalScrollPolicy = ScrollPolicy.ON;
        slidesList.verticalScrollPolicy = ScrollPolicy.OFF;
        slidesList.addEventListener(Event.SCROLL, slidesListScrollHandler);
        addEventListener(Event.CHANGE, slidesListChangeHandler);
        slidesList.itemRendererFactory = function ():IListItemRenderer {
            var renderer:TuningSlideItemRenderer = new TuningSlideItemRenderer();
            var item_w:Number = (actualWidth);
            var item_h:Number = (actualHeight);
            renderer.overrideDispose = _overrideDispose;
            renderer.clipBackground = clipBackground;
            renderer.setSize(item_w, item_h);
            renderer.fitContent = fitContent;
            return renderer;
        };

        addChild(slidesList);

        _pageIndicator = new GridPageIndicator();
        //_pageIndicator.visible=false;
        _pageIndicator.width = actualWidth - 2 * _indicatorMargin;
        _pageIndicator.normalSymbolFactory = function ():DisplayObject {
            var img:Image = new Image(AlertView.assetManager.getTexture("pageIndicator"));
            img.width = 9;
            img.height = 9;
            return img;
        };

        _pageIndicator.selectedSymbolFactory = function ():DisplayObject {
            var img:Image = new Image(AlertView.assetManager.getTexture("pageIndicatorSelected"));
            img.width = 9;
            img.height = 9;
            return img;
        };

        _pageIndicator.gap = 6;

        _pageIndicator.horizontalAlign = HorizontalAlign.CENTER;
        _pageIndicator.verticalAlign = VerticalAlign.MIDDLE;


        _pageIndicator.selectedIndex = -1;
        _pageIndicator.pageCount = 0;
        _pageIndicator.interactionMode = PageIndicatorInteractionMode.PRECISE;
        _pageIndicator.addEventListener(Event.CHANGE, pageChangeHandler);
        this.addChild(_pageIndicator);

        challengeIcon = new Image(ButtonFactory.assetManager.getTexture("gone_to_challenge_lable"));
        challengeIcon.width = actualWidth/4;
        challengeIcon.scale = challengeIcon.scaleX;
        this.addChild(challengeIcon);
    }

    private function pageChangeHandler(event:Event):void {
        slidesList.removeEventListener(Event.SCROLL, slidesListScrollHandler);
        slidesList.scrollToPageIndex(_pageIndicator.selectedIndex, 0, 0.5);
        restoreScrollHandling();
        dispatchEventWith(Event.CHANGE, false);
    }

    override public function setSize(w:Number, h:Number):void{
        super .setSize(w, h);
        if(slidesList){
            slidesList.width = actualWidth;
            slidesList.height = actualHeight;
            slidesList.validate();
            slidesList.scrollToPageIndex(slidesList.horizontalPageIndex, 0);
        }
    }

    private function restoreScrollHandling():void{
        if(slidesList.isScrolling){
            Starling.juggler.delayCall(restoreScrollHandling, 0.03);
        }else if(!slidesList.hasEventListener(Event.SCROLL, slidesListScrollHandler)){
            slidesList.addEventListener(Event.SCROLL, slidesListScrollHandler);
        }
    }

    private function postToTimelineButtonHandler(event:Event):void {
        if (!data) {
            return;
        }
        AlertView.show('Are you sure you want to share this tuning on 3DTuning\' feed', -1, false, function (yesSelected:Boolean):void {
            if (yesSelected) {
                var timelineProxy : TimelineProxy = ApplicationFacade.getInstance().retrieveProxy(TimelineProxy.NAME) as TimelineProxy;
                timelineProxy.postToTimeline(tuning.id);
                /*var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
                var snapshotIndex:int = (slidesList.horizontalPageIndex>-1?slidesList.horizontalPageIndex:0);
                var urls:Array = [{url: tuning.snapshots[snapshotIndex]}];*/
                /*for(var i:int = 0; i < tuning.snapshots.length; i++){
                    urls.push({url: tuning.snapshots[i]});
                }*/
                //timelineProxy.newPost('', usersProxy.currentUser, urls);
            }
        });
    }

    protected function slidesListScrollHandler(event:Event):void {
        if(!scrollStarted && isScrolling){
            scrollStarted = true;
            SoundManager.playSound(SoundManager.SELECT_PART);
        }else if(!isScrolling){
            scrollStarted = false;
        }
        setPageIndicatorPage(slidesList.horizontalPageIndex);
    }

    protected function slidesListChangeHandler(event:Event):void {
        var renderer:TuningSlideItemRenderer = slidesList.itemToItemRenderer(slidesList.dataProvider.getItemAt(slidesList.horizontalPageIndex)) as TuningSlideItemRenderer;
        if(renderer){
            renderer.updatePositions();
        }
    }

    private function setPageIndicatorPage(value:int):void{
        _pageIndicator.removeEventListener(Event.CHANGE, pageChangeHandler);
        if(_pageIndicator.selectedIndex != value){
            _pageIndicator.selectedIndex = value;
            dispatchEventWith(Event.CHANGE, false);
        }
        _pageIndicator.addEventListener(Event.CHANGE, pageChangeHandler);
    }

    protected override function draw():void {
        super.draw();
        var sizeInvalid:Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
        if (sizeInvalid) {
            if (!this.mask) {
                clipRectQuad.x = 0;
                clipRectQuad.width = actualWidth;
                if(clipBackground){
                    addChild(clipRectQuad);
                    clipRectQuad.height = actualHeight + 50;
                    clipRectQuad.y = -25;
                }else{
                    clipRectQuad.height = actualHeight;
                    clipRectQuad.y = 0;
                }
                this.mask = clipRectQuad;
            }else if(clipBackground){
                if(mask.height != actualHeight + 50){
                    mask.height = actualHeight + 50;
                }
                if(mask.y != -25){
                    mask.y = -25;
                }
                if(!mask.parent){
                    addChild(mask);
                }
            }

            _pageIndicator.validate();

            _pageIndicator.x = (actualWidth - _pageIndicator.width) / 2;
            _pageIndicator.y = (actualHeight - 10 - _pageIndicator.height);

            imageLoader.width = actualWidth * 0.75;
            imageLoader.height = actualHeight * 0.75;


            slidesList.width = this.actualWidth;
            slidesList.height = this.actualHeight;
        }
        if(data && Tuning(data).isChallenging && challengeIcon){

            challengeIcon.visible = true;
        }else if(data && !Tuning(data).isChallenging && challengeIcon){

            challengeIcon.visible = false;
        }

    }


    public function get isScrolling():Boolean{
        if(!slidesList){
            return false;
        }
        return slidesList.isScrolling;
    }

    protected override function commitData():void {
        super.commitData();


        if (!tuning) return;

        var hdsize:Boolean = false;
        hdsize = tuning.hdsize;


        var list:Array = [];

        if (tuning.snapshots && tuning.snapshots.length == 0) {
            trace("no snapshots");
        }


        for each(var url:String in tuning.snapshots) {
            list.push({
                url: url,
                hdsize: hdsize,
                tuning: tuning,
                index: list.length
                //isDefaultTexture:
            });
        }

        if (list.length <= 0) {

            list.push({
                url: tuning.imageURL,
                hdsize: hdsize,
                tuning: tuning,
                index: list.length
            });
        }
        slidesList.dataProvider = new ListCollection(list);
        _pageIndicator.pageCount = tuning.snapshots.length;
        _pageIndicator.selectedIndex = 0;
        _pageIndicator.validate();
        if(_pageIndicator.x == 0){
            _pageIndicator.x = (actualWidth - _pageIndicator.width) / 2;
        }
        _pageIndicator.y = (actualHeight - 15 - _pageIndicator.height);
    }

    override public function dispose():void {
        if(_pageIndicator){
            DisplayObjectUtil.dispose(_pageIndicator);
            _pageIndicator.dispose();
            _pageIndicator = null;
        }
        if(slidesList){
            slidesList.dataProvider = null;
            slidesList.itemRendererFactory = null;
            slidesList.dispose();
        }
        if(clipRectQuad){
            clipRectQuad.dispose();
            clipRectQuad = null;
        }
        DisplayObjectUtil.dispose(this);
        if (challengeIcon) {
            challengeIcon.removeFromParent(true);
            challengeIcon = null;
        }
        super.dispose();
    }

    public function get pageIndicator():PageIndicator {
        return _pageIndicator;
    }

    public function get selectedIndex():Number {
        return slidesList.horizontalPageIndex;
    }

    public function set overrideDispose(overrideDispose:Boolean):void {
        if(imageLoader){
            imageLoader.overrideDispose = overrideDispose;
        }
        _overrideDispose = overrideDispose;
    }

    public function set indicatorMargin(value:Number):void {
        _indicatorMargin = value;
    }

    public function set fitContent(value:Boolean):void {
        _fitContent = value;
    }

    public function get fitContent():Boolean {
        return _fitContent;
    }
}
}