package com.despark.tuning3d.view.components.tunings {
import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.MultilingualLabel;
import com.utils.FontSelector;

import feathers.core.FeathersControl;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;

public class TuningNameLabel extends FeathersControl {
    private var labels:Vector.<MultilingualLabel>;
    private var isBold:Boolean;
    public var relocate:Function;
    public function TuningNameLabel(bold:Boolean) {
        super();
        isBold = bold;
    }

    override protected function draw():void {
        super.draw();
        var w:Number = 0;
        var h:Number = 0;
        var isValid:Boolean = false;
        for(var i:int = 0; i < labels.length; i++){
            isValid ||= !labels[i].isInvalid();
            w = Math.max(w, labels[i].width);
            h = Math.max(h, labels[i].height);
        }
        if(w && h) {
            width = w;
            height = h;
            if(relocate != null){
                relocate.call();
            }
        }
    }

    override protected function initialize():void {
        super.initialize();
        labels = new <MultilingualLabel>[];
        var label:MultilingualLabel;
        for(var i:int = 0; i < 4; i++){
            label = createLabel(0x666666, isBold);
            label.x = (i%2==0?0.5:-0.5);
            label.y = (int(i/2)==0?0.5:-0.5);
            labels.push(label);
            addChild(label);
        }
        label = createLabel(0xFFFFFF, isBold);
        labels.push(label);
        addChild(label);
    }

    public function set text(value:String):void{
        for(var i:int = 0; i < labels.length; i++){
            labels[i].text = value;
            labels[i].invalidate();
        }
        invalidate();
    }

    private static function createLabel(color:uint = 0xffffff, bold:Boolean = false):MultilingualLabel{
        var _label:MultilingualLabel = new MultilingualLabel();
        _label.textFormat = new TextFormat(bold?FontSelector.getMonserratSemiBold():FontSelector.getMonserratMedium(), 10, color);
        _label.textFormat.align = TextFormatAlign.LEFT;
        _label.textFormat.bold = bold;
        _label.width = _label.height = NaN;
        _label.text = ' ';
        _label.updateWithLanguage(AlertView.language);
        return _label;
    }
}
}
