package com.despark.tuning3d.view.components.tunings
{
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.languages.Language;
	import com.despark.tuning3d.model.languages.Multilingual;
	import com.despark.tuning3d.view.components.parts.TunedPartView;
	
	import feathers.controls.ScrollContainer;
import feathers.controls.ScrollPolicy;
import feathers.core.FeathersControl;
	import feathers.system.DeviceCapabilities;
	
	import starling.core.Starling;
	import starling.display.Quad;

	public class TuningPartsView extends FeathersControl implements Multilingual
	{
		private var container : ScrollContainer;
		private var _car : Car;
		private var currentLanguage : Language;
		
		public var initialHeight : int;
		
		public function TuningPartsView()
		{
			super();
		}
		
		public function get car() : Car
		{
			return _car;
		}

		public function set car(value : Car):void
		{
			_car = value;
			invalidate("car");
		}
		
		public function updateWithLanguage(language : Language) : void
		{
			currentLanguage = language;
		}

		override protected function initialize() : void
		{
			super.initialize();
			
			container = new ScrollContainer();
			container.verticalScrollPolicy = ScrollPolicy.OFF;
			container.padding = 10;
			addChild(container);
			//container.snapToPages = true;
		}
		
		public function addParts() : void
		{
			const itemsPerColumn : int = DeviceCapabilities.isPhone(Starling.current.nativeStage) ? 3 : 4;
			const verticalGap : int = 2;
			const horizontalGap : int = 20;
			const itemHeight : int = int((initialHeight - 2 * container.padding) / itemsPerColumn - verticalGap);
			
			var currentX : int = 0;
			var currentY : int = 3;
			
			container.removeChildren();
			
			for(var i : int = 0; i < car.editablePartSlots.length; i++)
			{
				var tunedPartView : TunedPartView = new TunedPartView();
				tunedPartView.updateWithLanguage(currentLanguage);
				tunedPartView.x = currentX;
				tunedPartView.y = currentY + 2;
				tunedPartView.width = 170;
				tunedPartView.height = itemHeight;
				tunedPartView.partSlot = car.editablePartSlots[i];
				container.addChild(tunedPartView);
				
				if((i + 1) % itemsPerColumn == 0 && i > 0 && i < car.editablePartSlots.length - 1) 
				{
					currentY = 3;
					currentX += tunedPartView.width + horizontalGap / 2;
						
					var separator : Quad = new Quad(1, initialHeight - 2 * container.padding, 0xDADBDB);
					separator.x = currentX;
					separator.y = currentY;
					container.addChild( separator );
					
					currentX += horizontalGap / 2;
				}
				else 
				{
					currentY += tunedPartView.height + verticalGap;
				}
			}
		}
		
		override protected function draw() : void
		{
			super.draw();
			
			if(isInvalid(INVALIDATION_FLAG_SIZE))
			{
				container.width = actualWidth;
				container.height = actualHeight;
			}
			
			if(car && isInvalid("car"))
			{
				addParts();
			}
		}
	}
}