package com.despark.tuning3d.view.components.tunings
{
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.cars.CarBackground;
	import com.despark.tuning3d.model.core.Device;
	import com.despark.tuning3d.model.tunings.Tuning;
	import com.despark.tuning3d.view.components.core.AlertView;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
	import com.despark.tuning3d.view.components.core.ListItemRenderer;
	import com.despark.tuning3d.view.components.core.MultilingualLabel;
	import com.utils.DisplayObjectUtil;
	import com.utils.FontSelector;

	import flash.filesystem.File;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;

	import starling.display.Image;
	import starling.display.Quad;
	import starling.textures.TextureSmoothing;

	public class TuningRowsItemRenderer extends ListItemRenderer {
		private var _scale:Number=0.75;
        private var _posY:Number=0;
		protected var nameBg:Quad;
		private var clipRectQuad:Quad;
		protected var nameLabel:MultilingualLabel;
		private var backgroundImage:ImageLoaderWithStates=null;
        private var challengeIcon:Image;
		public var pageContentCount:int = 1;
		//
		private var leftMargin:Number = 0;
		private var rightMargin:Number = 0;
		private var topMargin:Number = 0;
		private var bottomMargin:Number = 0;
		private var marginSize:Number = 0.75;
		//
		private var itemIndex:int = 0;
		//
		public function get tuning() : Tuning {
			return data as Tuning;
		}
		
		protected override function initialize() : void {

			backgroundImage = new ImageLoaderWithStates();
			backgroundImage.overrideDispose = true;
			backgroundImage.visible = false;
			backgroundImage.completeHandler = function():void{
				var bgRatio:Number = backgroundImage.currentTexture.width / backgroundImage.currentTexture.height;
				var actualRatio:Number = actualWidth / actualHeight;
				if(actualRatio < bgRatio){
					if(backgroundImage.height != actualHeight){
						backgroundImage.height = actualHeight;
						backgroundImage.width = actualHeight * bgRatio;
					}
				}else{
					if(backgroundImage.width != actualWidth){
						backgroundImage.width = actualWidth;
						backgroundImage.height = actualWidth / bgRatio;
					}
				}

				backgroundImage.x = (actualWidth - backgroundImage.width)/2;
				backgroundImage.y = (actualHeight - backgroundImage.height)/2;
			};
			addChild(backgroundImage);
			
			super.initialize();
			imageLoader.overrideDispose = true;
			//imageLoader.visible=false;

			label.visible = false;
			nameBg = new Quad(1, 1, 0x000000);
			nameBg.alpha = 0.4;
			addChild(nameBg);
			
			
			
			nameLabel = new MultilingualLabel();
			nameLabel.textFormat = new TextFormat(FontSelector.getFont(false), 10, 0xffffff);
			nameLabel.textFormat.align = TextFormatAlign.LEFT;
			if(Device.isIphoneXVariant()){
				nameLabel.textFormat.align = TextFormatAlign.CENTER;
			}
			nameLabel.textFormat.bold = false;
			addChild(nameLabel);
			nameLabel.updateWithLanguage(AlertView.language);

			challengeIcon = new Image(ButtonFactory.assetManager.getTexture("gone_to_challenge_lable"));
			challengeIcon.width = actualWidth/2;
			challengeIcon.scale = challengeIcon.scaleX;
			this.addChild(challengeIcon);

			clipRectQuad = new Quad(1,1,0x000000);
			addChild(clipRectQuad);
		}
		

        override public function dispose():void {
            DisplayObjectUtil.disposeMesh(nameBg);
			nameBg = null;
            DisplayObjectUtil.disposeMesh(clipRectQuad);
			clipRectQuad = null;
            if(nameLabel)
				nameLabel.dispose();
			nameLabel = null;
            if(backgroundImage)
				backgroundImage.dispose();
			backgroundImage = null;
			if (challengeIcon) {
				challengeIcon.removeFromParent(true);
				challengeIcon = null;
			}

			super.dispose();


        }

        protected override function draw() : void {
			super.draw();
			var dataInvalid : Boolean = isInvalid(INVALIDATION_FLAG_DATA);
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			clipRectQuad.width = actualWidth - (leftMargin + rightMargin);
			clipRectQuad.height = actualHeight - (topMargin + bottomMargin);
			clipRectQuad.x = rightMargin;
			clipRectQuad.y = topMargin;
			mask = clipRectQuad;

			if(sizeInvalid){
				nameBg.width = actualWidth;
				nameBg.height = 20;
				nameBg.x = 0;
				nameBg.y = actualHeight - nameBg.height;
			}



			if(dataInvalid){
				if(tuning){
					nameLabel.text=tuning.name;
				}
				nameLabel.height=NaN;
				nameLabel.width=actualWidth-10;
				nameLabel.validate();
				nameLabel.x=5;
				nameLabel.y=(actualHeight-nameBg.height)+((20-nameLabel.height)/2);
			}
			if(data && Tuning(data).isChallenging && challengeIcon){

				challengeIcon.visible = true;
			}else if(data && !Tuning(data).isChallenging && challengeIcon){

				challengeIcon.visible = false;
			}
		}
		
		protected function completeLoadImage(imageWidth:Number, imageHeight:Number):void{
			/*var scale:Number = actualWidth/this.stage.width;
			imageLoader.width = (imageWidth/2)*scale;
			imageLoader.height = (imageHeight/2)*scale;*/
			var nActualHeight:Number = actualHeight * 1.3;
			var bgRatio:Number = imageWidth / imageHeight;
			var actualRatio:Number = actualWidth / nActualHeight;
			if(actualRatio > bgRatio){
				imageLoader.height = nActualHeight;
				imageLoader.width = nActualHeight * bgRatio;
			}else{
				imageLoader.width = actualWidth;
				imageLoader.height = actualWidth / bgRatio;
			}
			
			imageLoader.x=(actualWidth-imageLoader.width)/2;
			imageLoader.y=(actualHeight-imageLoader.height)/2;
		}
		
		protected override function commitData() : void{
			super.commitData();
			var carBackground:CarBackground;
			if(tuning){
				itemIndex = owner.dataProvider.getItemIndex(data) % pageContentCount;
				if(pageContentCount == 1 || pageContentCount == 6){
					leftMargin = rightMargin = topMargin = bottomMargin = 0;
				}else if(pageContentCount == 4){
					if(itemIndex % 2 == 0){
						leftMargin = marginSize;
						rightMargin = 0;
					}else if(itemIndex % 2 == 1){
						leftMargin = 0;
						rightMargin = marginSize;
					}

					if(int(itemIndex / 2) == 0){
						topMargin = 0;
						bottomMargin = 1;
					}else if(int(itemIndex / 2) == 1){
						topMargin = marginSize;
						bottomMargin = 0;
					}
				}else if(pageContentCount == 9){
					if(itemIndex % 3 == 0){
						leftMargin = marginSize;
						rightMargin = 0;
					}else if(itemIndex % 3 == 1){
						leftMargin = marginSize;
						rightMargin = marginSize;
					}else{
						leftMargin = 0;
						rightMargin = marginSize;
					}

					if(int(itemIndex / 3) == 0){
						topMargin = 0;
						bottomMargin = marginSize;
					}else if(int(itemIndex / 3) == 1){
						topMargin = marginSize;
						bottomMargin = marginSize;
					}else{
						topMargin = marginSize;
						bottomMargin = 0;
					}
				}

			}
			var cameraIndex:int;
			if(tuning && tuning.backgroundId && tuning.backgroundId != CarBackground.DEFAULT_ID && tuning.backgroundsMap && tuning.backgroundsMap[tuning.backgroundId]){
				carBackground = tuning.backgroundsMap[tuning.backgroundId];
				cameraIndex = 0;
				if(tuning.customSnapshotsCamera && tuning.customSnapshotsCamera.length>0 && carBackground.imagePathSecondCam && carBackground.imagePathSecondCam.length>0){
					cameraIndex = tuning.customSnapshotsCamera[0];
				}
				carBackground.showThumbnailUrl=false;
				carBackground.selectedCamera = cameraIndex;
				backgroundImage.smoothing = TextureSmoothing.BILINEAR;
				backgroundImage.persistentImageProvider = carBackground;
				backgroundImage.visible = true;
				_scale=0.75;
				_posY=0.025;
				//updateImageSize();
			}else if(tuning){
				var randomSplashUrl:String;
				if(tuning && tuning.carChooserBgName){
					randomSplashUrl = File.applicationDirectory.resolvePath("assets/textures/bgs/"+tuning.carChooserBgName.replace('.jpg', '') + ".jpg").url;
					if(tuning.customSnapshotsCamera && tuning.customSnapshotsCamera.length>0){
						cameraIndex = tuning.customSnapshotsCamera[0];
						if(tuning.carObj.defaultBkgMobile[cameraIndex]){
							randomSplashUrl = tuning.carObj.defaultBkgMobile[cameraIndex];
						}
					}
					carBackground = new CarBackground();
					carBackground.mainUrl = randomSplashUrl;
					carBackground.isLocalFile = true;
					carBackground.showThumbnailUrl = false;
					backgroundImage.persistentImageProvider = carBackground;
				}else{
					randomSplashUrl = File.applicationDirectory.resolvePath("assets/textures/bgs/a_bridge.jpg").url;
					carBackground = new CarBackground();
					carBackground.mainUrl = randomSplashUrl;
					carBackground.isLocalFile = true;
					carBackground.showThumbnailUrl = false;
					backgroundImage.persistentImageProvider = carBackground;
				}
				backgroundImage.visible=true;
				
				_scale=0.85;
				
				_posY=0.05;
				//updateImageSize();
			}
			
			if(tuning){
				
				var car:Car = new Car();
				car.imageURL = tuning.imageURL;
				
				imageLoader.recieveImageSize = completeLoadImage;
				imageLoader.bitmapScale = 0.4;
				imageLoader.persistentImageProvider = car;
			}
		}
	}
}
