package com.despark.tuning3d.view.components.tunings.colors
{
	import com.despark.tuning3d.ApplicationFacade;
	import com.despark.tuning3d.model.cars.Car;
	import com.despark.tuning3d.model.cars.CarBackgroundsProxy;
	import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.Multilingual;
import com.despark.tuning3d.model.parts.Part;
import com.despark.tuning3d.model.parts.PartColor;
import com.despark.tuning3d.model.parts.PartMaterial;
import com.despark.tuning3d.model.parts.PartMaterial;
import com.despark.tuning3d.model.parts.PartSlot;
	import com.despark.tuning3d.model.sounds.SoundManager;
	import com.despark.tuning3d.model.users.UsersProxy;
	import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.MaterialItemRenderer;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.despark.tuning3d.view.components.tunings.editing.SuspensionMenu;
import com.despark.tuning3d.view.components.tunings.editing.TuningSuspensionControl;
import com.google.analytics.debug.Align;
import com.utils.FontSelector;

import feathers.layout.VerticalAlign;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import feathers.controls.Button;
import feathers.controls.List;
import feathers.controls.ScrollContainer;
import feathers.controls.renderers.IListItemRenderer;
import feathers.core.FeathersControl;
import feathers.data.ListCollection;
import feathers.layout.HorizontalLayout;

import starling.display.Image;
import starling.display.Quad;
import starling.events.Event;
import starling.textures.Texture;
import starling.utils.AssetManager;

public class TuningColorsView extends FeathersControl implements Multilingual {
		public static const COLOR_SELECTED : String = "ColorSelected";
		public static const COLOR_PICKER_SELECTED : String = "ColorPickerSelected";
		
		public static const ALL_PART_SLOT_SELECTED : String = "ALL_PART_SLOT_SELECTED";
		
		public static const MATERIAL_SELECTED : String = "MaterialSelected";
		public var initialHeight: int;
		public var assetManager : AssetManager;
		private var _partColors : Array;
		public var selectedColor : PartColor;
		
		private var currentLanguage : Language;
		private var colorContainer : ScrollContainer;
		private var quad : Quad;
		
		private var moreColorsButton:MultilingualButton;
		
		private var moreColorsSeparator:Image;
		private var partColorsSeparator:Image;

		private var partColorButton:PartColorButton;
		public var materialList:List;

		//данные
		private var _partSlot:PartSlot;
		
		private var _materialChangeBlocker:Boolean=false;
		private var _backgroundsIsDownloaded: Boolean = false;

		public function TuningColorsView() {
			currentLanguage=AlertView.language;
			super();
		}

		public function set partColors(value:Array):void{
			if(value == null) {
				return;
			}
			_partColors=value;
			invalidate("partColors");
		}
		
		public function get partColors():Array{
			return _partColors;
		}
		
		public function updateWithLanguage(language : Language) : void {
			currentLanguage = language;
		}
		
		override protected function initialize() : void {
			super.initialize();
			quad = new Quad(1, 1, 0x000000);
			quad.alpha = 0.1;
			addChild(quad);
			
			var layout : HorizontalLayout = new HorizontalLayout();
			layout.verticalAlign = VerticalAlign.MIDDLE;
			layout.gap = 0;
			layout.paddingLeft = 0;
			layout.paddingRight = 0;
			
			colorContainer = new ScrollContainer();
			colorContainer.clipContent = true;
			colorContainer.layout = layout;
			addChild(colorContainer);
			
			moreColorsButton=new MultilingualButton();
			moreColorsButton.textFormat = new TextFormat(FontSelector.getFont(true), 8, 0xFFFFFF );
			moreColorsButton.textFormat.align = TextFormatAlign.LEFT;
			moreColorsButton.textFormat.bold = false;
			moreColorsButton.paddingLeft = 8;
			moreColorsButton.wordWrap = true;
			moreColorsButton.label = "More Colors";
			
			moreColorsButton.defaultSkin = new Quad(1,1,0x000000);
			moreColorsButton.defaultSkin.alpha = 0;
			moreColorsButton.downSkin = new Quad(1,1,0x000000);
			moreColorsButton.downSkin.alpha = 0.3;
			
			moreColorsButton.updateWithLanguage(AlertView.language);
			moreColorsSeparator = createSeporator();
			addChild(moreColorsSeparator);
			
			
			partColorButton=new PartColorButton();
			partColorButton.addEventListener(Event.TRIGGERED, onSelectAllPartSlots);
			
			partColorButton.paddingLeft = 3;
			partColorButton.paddingRight = 3;
			addChild(partColorButton);
			
			partColorsSeparator=createSeporator();
			addChild(partColorsSeparator);
			
			var possiblePartSlotsLayout : HorizontalLayout = new HorizontalLayout();
			possiblePartSlotsLayout.useVirtualLayout = false;
			possiblePartSlotsLayout.gap=16;
			possiblePartSlotsLayout.paddingLeft=0;
			possiblePartSlotsLayout.paddingRight=0;
			
			
			materialList = new List();
			materialList.snapScrollPositionsToPixels = true;
			materialList.hasElasticEdges=false;
			materialList.clipContent = true;
			materialList.layout = possiblePartSlotsLayout;
			materialList.addEventListener(Event.CHANGE, materialListChangeHandler);
			materialList.addEventListener(Event.SCROLL, materialListScrollHandler);
			materialList.addEventListener("itemSelectedAgain", materialListItemSelectedAgain);
			materialList.itemRendererFactory = function() : IListItemRenderer {
				var renderer :MaterialItemRenderer = new MaterialItemRenderer();
				renderer.language = AlertView.language;
				return renderer;
			}
			addChild(materialList);
		}

		protected function onSelectAllPartSlots(event:Event):void{
			this.dispatchEventWith(ALL_PART_SLOT_SELECTED);
		}

		protected function onToggleState(event:Event):void{
			this.dispatchEventWith(EffectMenu.TOGGLE_STATE, false, event.data);
		}
		
		protected function onErrorToggleState(event:Event):void{
			this.dispatchEventWith(EffectMenu.TOGGLE_STATE_ERROR, false, event.data);
		}
		
		public function set partSlot(value:PartSlot):void{
			_partSlot = value;
			if(_partSlot && _partSlot.selectedPart){
				isEnabled = hasPaintMaterial;
			}else{
				isEnabled = false;
			}
			this.invalidate("updatePartSlot");
		}
		
		public function createSeporator():Image{
			
			var seporatorTexture:Texture=assetManager.getTexture("a_seporator_1");

			var seporatorImage:Image=new Image(seporatorTexture);
			
			seporatorImage.width=1.5;
			seporatorImage.height=34;
			
			return seporatorImage;
		}
		
		private function refresh() : void
		{
			
			if(!partColors)
				return;
			
			var i:int=0;
			var tuningSelectedColorView : TuningSelectedColorView;
			
			
			
			if(colorContainer){
				
				
				while( colorContainer.numChildren>0){
					
					
					tuningSelectedColorView=colorContainer.getChildAt(0) as TuningSelectedColorView;
					
					tuningSelectedColorView.removeEventListener(Event.SELECT, tuningSelectedColorViewSelectHandler);
						
					colorContainer.removeChild(tuningSelectedColorView);
					
					
					tuningSelectedColorView.assetManager=null;
					tuningSelectedColorView.dispose();
					tuningSelectedColorView=null;
				}
			}
			
			
			
			var itemHeight : int = initialHeight;

			var currentX : int = 0;

			for( i  = 0; i < partColors.length; i++)
				{
				tuningSelectedColorView  = new TuningSelectedColorView();


				tuningSelectedColorView.assetManager = assetManager;
				tuningSelectedColorView.x = currentX;
				tuningSelectedColorView.width = 40.5;
				tuningSelectedColorView.height = itemHeight;
				tuningSelectedColorView.showsMaterial = false;
				tuningSelectedColorView.partColor = partColors[i].copy();
				tuningSelectedColorView.addEventListener(Event.SELECT, tuningSelectedColorViewSelectHandler);
				
				colorContainer.addChild(tuningSelectedColorView);
				
				/*if(i < partColors.length - 1) 
				{
					var sprite : Sprite = new Sprite();
					
					var separator : Quad = new Quad(1, initialHeight - 2 * 6, 0xDADBDB);
					separator.alpha = 0.99;
					separator.y = 5;
					sprite.addChild(separator);
					
					container.addChild( sprite );
					
				}*/
			}
		}
		
		override protected function draw() : void {
			super.draw();
			
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			
			if(sizeInvalid) {
				quad.width = actualWidth;
				quad.height = actualHeight;

				partColorButton.width = 40;
				partColorButton.height = actualHeight;

				colorContainer.width = (actualWidth - partColorButton.width - 10 + 2 * moreColorsSeparator.width) / 2;
				materialList.width = (actualWidth - partColorButton.width - 10 + 2 * moreColorsSeparator.width) / 2;
				colorContainer.height = actualHeight;
				
				partColorButton.x = (actualWidth - partColorButton.width) / 2;
				partColorButton.y = 0;

				moreColorsSeparator.x = partColorButton.x;
				moreColorsSeparator.y = (actualHeight - moreColorsSeparator.height) / 2;

				partColorsSeparator.x = partColorButton.x + partColorButton.width - partColorsSeparator.width;
				partColorsSeparator.y = (actualHeight - partColorsSeparator.height) / 2;

				materialList.x = partColorsSeparator.x + partColorsSeparator.width + 2.5;
				materialList.y = 0;
				
				materialList.height = actualHeight;
			}
			
			if(isInvalid("partColors")) {
				refresh();
			}
			
			if(isInvalid("updatePartSlot")){
				var i:int;
				if(_partSlot && _partSlot.selectedPart && _partSlot.selectedPart.possibleMaterials && _partSlot.selectedPart.possibleMaterials.length>0){
					var list:Array=[];
					var partMaterial:PartMaterial;
					//
					var fixed:Array = new Array(PartMaterial.FIXED_MATERIAL_TYPES.length);
					var others:Array = [];
					//
					for(i=0; i<_partSlot.selectedPart.possibleMaterials.length; i++){
						partMaterial=_partSlot.selectedPart.possibleMaterials[i];
						if(partMaterial.name=="metal" && _partSlot.dummy && !_partSlot.parentCar.isChopper)
							continue;
						if(partMaterial.name!="mask" && partMaterial.name!="antimask" /*&&  partMaterial.name!="glossy" &&  partMaterial.name!="white"*/) {
                            if(PartMaterial.FIXED_MATERIAL_TYPES.indexOf(partMaterial.name) !=-1){
                                fixed[PartMaterial.FIXED_MATERIAL_TYPES.indexOf(partMaterial.name)] = partMaterial;
                            }else {
                                others.push(_partSlot.selectedPart.possibleMaterials[i]);
							}
						}
					}
					for(i = fixed.length - 1; i >= 0; i--){
						if(!fixed[i]){
							fixed.removeAt(i);
						}
					}
					others.sortOn(["name"], Array.CASEINSENSITIVE);
					list = fixed.concat(others);

					//возможно нужно зафильтровать
					materialList.dataProvider=new ListCollection(list);
					if(_partSlot.selectedPart.selectedMaterial){
						_materialChangeBlocker=true;
						//materialList.selectedIndex=list.indexOf(_partSlot.selectedPart.selectedMaterial);
						for(i = 0; i < list.length; i++)
						{
                            if(_partSlot.selectedPart.selectedMaterial.name == list[i].name)
							{
                                materialList.selectedItem=list[i];
                                _materialChangeBlocker=false;
								break;
							}
						}
//						materialList.selectedItem=list[list.indexOf(_partSlot.selectedPart.selectedMaterial)];
						_materialChangeBlocker=false;
					}
					
				}else{
					
					materialList.dataProvider=new ListCollection();
				}
				
				if(_partSlot){
					
					if(_partSlot.name != "ALL")
					{
						partColorButton.label = AlertView.language.translate("componentTypes", _partSlot.name);
					}
					else 
					{
						partColorButton.label = AlertView.language.translate("mob_saving", "all");
					}
					
					
					//partColorButton.label=_partSlot.name;
				}
				if(materialList.selectedIndex!=-1){
                    materialList.scrollToDisplayIndex(materialList.selectedIndex);
				}
			}
			
			
			/*if(_partSlot && _partSlot.selectedPart){
				
				var selectedPart:Part
				if(_partSlot.selectedPart && _partSlot.selectedPart.selectedState){
					selectedPart = _partSlot.selectedPart.selectedState;
				}else{
					
					selectedPart = _partSlot.selectedPart;
				}
				
				
				var isExistWhiteMaterial:Boolean=selectedPart.getPossibleMaterial("white");
				var isExistGlossyMaterial:Boolean=selectedPart.getPossibleMaterial("glossy");
				
				
				var partMaterial:PartMaterial;
				
				if(_partSlot.selectedPart && _partSlot.selectedPart.selectedState){
					partMaterial = _partSlot.selectedPart.selectedState.selectedMaterial;
				}else if( _partSlot.selectedPart){
					partMaterial = _partSlot.selectedPart.selectedMaterial;
				}
				
				if(partMaterial.name=="metal" && !isExistGlossyMaterial && !isExistWhiteMaterial){
					
					this.colorContainer.touchable=false;
					this.colorContainer.alpha=0.5;
				}else{
					
					this.colorContainer.touchable=true;
					this.colorContainer.alpha=1;
				}
				
			}else if(colorContainer){
				
				this.colorContainer.touchable=false;
				this.colorContainer.alpha=0.5;
			}*/
		}

    protected function materialListItemSelectedAgain(event:Event):void{

        if(materialList.selectedIndex<0) return;

        if(!_partSlot) return;

        if(!_partSlot.selectedPart) return;
		var part : Part = (_partSlot.selectedPart.selectedState == null) ? _partSlot.selectedPart : _partSlot.selectedPart.selectedState;

        var isExistGlossyMaterial:Boolean=part.getPossibleMaterial("glossy");
        if((materialList.selectedItem as PartMaterial).name!="glossy" && isExistGlossyMaterial ){


            materialList.selectedIndex=-1;//_partSlot.selectedPart.getPossibleMaterial("glossy");

            dispatchEventWith(MATERIAL_SELECTED, true, {selectedMaterial: _partSlot.selectedPart.getPossibleMaterial("glossy"), partSlot: _partSlot, selectedColor:_partSlot.previousColor});
			
			var selectedMaterial:PartMaterial =part.getPossibleMaterial("glossy");
			
			//var isExistWhiteMaterial:Boolean= part.getPossibleMaterial("white");
			//var isExistGlossyMaterial:Boolean= _partSlot.selectedPart.getPossibleMaterial("glossy");
			
			this.colorContainer.touchable=true;
			this.colorContainer.alpha=1;
			

        }
    }


        override public function set isEnabled(value:Boolean):void {
            super.isEnabled = value;
            if(!value){
                this.colorContainer.touchable=false;
                this.colorContainer.alpha=0.5;
            }else{
                this.colorContainer.touchable=true;
                this.colorContainer.alpha=1;
            }
        }

        private function materialListChangeHandler(event:Event):void{
			if(_materialChangeBlocker) {
				return;
            }
			if(_partSlot && _partSlot.selectedPart && _partSlot.selectedPart.possibleMaterials && _partSlot.selectedPart.possibleMaterials.length>0 ){
				var selectedMaterial : PartMaterial;
				var possibleMaterials:Array=_partSlot.selectedPart.possibleMaterials;
				selectedMaterial = possibleMaterials[possibleMaterials.indexOf(materialList.selectedItem)];
				if(!selectedMaterial) {
					return;
                }
				if(_partSlot.selectedPart.selectedState) {
					var selectedState : Part = _partSlot.selectedPart.selectedState;
					var length : int = selectedState.possibleMaterials.length;
					var materialIsFound : Boolean = false;
					if(length <= 0) {
						return;
					}
					for(var i : int  = 0; i < length; i ++) {
						if(selectedState.possibleMaterials[i].name == selectedMaterial.name) {
							materialIsFound = true;
						}
					}
					if(!materialIsFound) {
						return;
					}
				}

				SoundManager.playSound(SoundManager.SELECT_PART);
				dispatchEventWith(MATERIAL_SELECTED, true, {selectedMaterial: selectedMaterial, partSlot: _partSlot, selectedColor:_partSlot.previousColor});
				isEnabled = hasPaintMaterial;
			}
		}

        public function get hasPaintMaterial():Boolean{
			var materials:Array;
			if(_partSlot.selectedPart && _partSlot.selectedPart.selectedState){
				materials = _partSlot.selectedPart.selectedState.possibleMaterials;
			}else if( _partSlot.selectedPart){
				materials = _partSlot.selectedPart.possibleMaterials;
			}else{
				return false;
			}
			var value:Boolean = false;
			for(var i:int = 0; i < materials.length; i++){
				if(PartMaterial(materials[i]).isPaintable && PartMaterial(materials[i]).name != "mask"){
					value = true;
					break;
				}
			}
            return value;
        }
		
		private function materialListScrollHandler(event:Event):void{
			
			
		}
		
		/*public function selectMaterialForState(partSlot:PartSlot):void {
			
			var materialName:String;
			var materialOfState:PartMaterial;
			if (partSlot.selectedPart.selectedMaterial) {
				//если у не state детали есть выбранный материал то пытаемся найти этот же материал у state детали
				materialName = partSlot.selectedPart.selectedMaterial.name;
				materialOfState = partSlot.selectedPart.selectedState.getPossibleMaterial(materialName);
				
				trace("selectMaterialForState", materialName);
			}
			
			if (!materialOfState) {
				//если не нашли материал то берем первый возможный
				materialOfState = partSlot.selectedPart.selectedState.possibleMaterials[0];//временный код на всякий случай
			}
			
			if (materialOfState) {
				//задаем материал
				
				if (partSlot.selectedPart.selectedState.isAdditionState) {
					partSlot.selectedPart.selectedMaterialOfAddState = materialOfState;
				} else {
					
					if (!partSlot.selectedPart.oldSelectedMaterial) {
						//запоминаем цвет и материял основной детали
						partSlot.selectedPart.oldSelectedMaterial = partSlot.selectedPart.selectedMaterial;
						//partSlot.selectedPart.oldSelectedColor = partSlot.selectedColor.copy();
						
						if (partSlot.previousColorOfState) {//востанавливаем цвет состояния если до этого сохраняли
							partSlot.selectedColor = partSlot.previousColorOfState.copy();
							//partSlot.selectedPart.color=partSlot.selectedColor.copy();
						}
						
						
					}
					
					if(partSlot.selectedPart.selectedState){
						partSlot.selectedPart.selectedState.selectedMaterial = materialOfState;
					}
					
					partSlot.selectedPart.selectedMaterial = materialOfState;
				}
				
				
			}
			
			
		}*/
	
		private function tuningSelectedColorViewSelectHandler(event : Event) : void
		{
			SoundManager.playSound(SoundManager.PAINTING);
			var tuningSelectedColorView : TuningSelectedColorView = TuningSelectedColorView(event.currentTarget); 
			selectedColor = tuningSelectedColorView.partColor;

            var part : Part = (_partSlot.selectedPart.selectedState == null) ? _partSlot.selectedPart : _partSlot.selectedPart.selectedState;
			
			//if(_partSlot.selectedPart.selectedState)
			//selectMaterialForState(_partSlot);
			
			var isExistWhiteMaterial:Boolean=part.getPossibleMaterial("white");
			var isExistGlossyMaterial:Boolean=part.getPossibleMaterial("glossy");


			if(!selectedColor.isCustom) 
			{
				
				if(part.selectedMaterial.name=="mate" || part.selectedMaterial.name=="redblack"){
					
					dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
					
				}else if(isExistWhiteMaterial){
					
						
						/*if(selectedColor.value==0xfeffff && part.selectedMaterial.name!="white"){
							
							this.materialList.selectedIndex=-1;
							dispatchEventWith(MATERIAL_SELECTED, true, {selectedMaterial: part.getPossibleMaterial("white"), partSlot: _partSlot});
							
						}else */if(selectedColor.value!=0xfeffff && part.selectedMaterial.name=="white" && isExistGlossyMaterial){
							
							this.materialList.selectedIndex=-1;
							
							dispatchEventWith(MATERIAL_SELECTED, true, {selectedColor: selectedColor, selectedMaterial: part.getPossibleMaterial("glossy"), partSlot: _partSlot, onComplete:function():void{
								dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
								
							}});
							//
							
						}else if(part.selectedMaterial.name=="metal" && isExistGlossyMaterial){
							
							this.materialList.selectedIndex=-1;
							
							dispatchEventWith(MATERIAL_SELECTED, true, {selectedMaterial: part.getPossibleMaterial("glossy"), selectedColor: selectedColor, partSlot: _partSlot, onComplete:function():void{
								
								dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
								
							}});
							
						}else if(!part.selectedMaterial.isPaintable && isExistGlossyMaterial){
                            this.materialList.selectedIndex=-1;
                            dispatchEventWith(MATERIAL_SELECTED, true, {selectedColor: selectedColor, selectedMaterial: part.getPossibleMaterial("glossy"), partSlot: _partSlot, onComplete:function():void{
                                    dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
                                }});
                        }else{
							
							dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
						}
				}else{
						
						if(part.selectedMaterial.name!="glossy" && isExistGlossyMaterial){
							
							
							
							/*if(!_partSlot.dummy) {
							_partSlot.selectedColor=selectedColor.copy();
							part.color=_partSlot.selectedColor;
							//part.getPossibleMaterial("glossy").getImage(0,0).
							
							
							}*/
							
							this.materialList.selectedIndex=-1;
							dispatchEventWith(MATERIAL_SELECTED, true, {selectedMaterial: _partSlot.selectedPart.getPossibleMaterial("glossy"), selectedColor:selectedColor, partSlot: _partSlot, onComplete:function():void{
								dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
								
							}});
							
							//dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
							
						}else{
							
							dispatchEventWith(COLOR_SELECTED, true, {selectedColor: selectedColor});
						}
				}
				
			}
			else
			{
				dispatchEventWith(COLOR_PICKER_SELECTED, true);
			}
		}
	}
}