package com.despark.tuning3d.view.components.tunings.colors
{
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.Multilingual;
import com.despark.tuning3d.model.parts.Part;
import com.despark.tuning3d.model.parts.PartColor;
import com.despark.tuning3d.model.parts.PartMaterial;
import com.despark.tuning3d.model.parts.PartSlot;
import com.despark.tuning3d.view.components.core.MultilingualLabel;
import com.utils.FontSelector;

import feathers.controls.Button;
import feathers.controls.Radio;
import feathers.controls.ScrollContainer;
import feathers.controls.ScrollPolicy;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.FeathersControl;
import feathers.core.ITextRenderer;
import feathers.core.ToggleGroup;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.animation.Transitions;
import starling.animation.Tween;
import starling.core.Starling;
import starling.display.Image;
import starling.display.Quad;
import starling.events.Event;
import starling.utils.AssetManager;

public class TuningSelectedColorView extends FeathersControl implements Multilingual
	{
		public static const TOGGLE_COLOR_SELECTION : String = "ToggleColorSelection";

		public var assetManager : AssetManager;
		public var showsMaterial : Boolean;
		public var showsIconForCustomColors : Boolean;
		public var showsOnlyMaterials : Boolean;
		
		private var currentLanguage : Language;
		private var _partColor : PartColor;
		private var quad : Quad;
		//private var rightShadow : Scale3Image;
		private var colorSampleImage : Image;
	//	private var colorLabel : MultilingualLabel;
		private var materialLabel : MultilingualLabel;
		private var button : Button;
		private var _opened : Boolean;
		private var _partSlot : PartSlot;
		private var radioButtons : Array;
		private var separator : Quad;
		//private var noPartSelectedLabel : MultilingualLabel;
		
		private var container:ScrollContainer;
		
		private var _possibleMaterials:Array=null;
		
		public function TuningSelectedColorView()
		{
			super();
			
			showsMaterial = true;
			showsIconForCustomColors = true;
			
			radioButtons = [];
		}
		
		override public function dispose():void{
			
			
			colorSampleImage.removeFromParent();
			
			colorSampleImage.dispose();
			
			colorSampleImage=null;
			
			_possibleMaterials=null;
			
			_partSlot=null;
			
			partColor=null;
			
			currentLanguage=null;
			
			
			super.dispose();
		}
		
		public function get opened() : Boolean 
		{
			return _opened;
		}
		
		
		public function get possibleMaterials():Array{
			
			_possibleMaterials=[];
			
			
			var possibleMaterialsOld:Array=null;
			var i:int=0;
			var material:PartMaterial=null;
			
			if(partSlot.selectedPart && partSlot.selectedPart.selectedState){
				possibleMaterialsOld = partSlot.selectedPart.selectedState.possibleMaterials;
				
				for(i=0; i<possibleMaterialsOld.length; i++){
					
					material=possibleMaterialsOld[i];  
					
					if(material.name=="mask")
						continue;
					
					if(material.name=="antimask")
						continue;
					
					_possibleMaterials.push(material);
				}
				
				//trace("possibleMaterials123:", possibleMaterials);
			}else if(partSlot.selectedPart){
				
				possibleMaterialsOld = partSlot.selectedPart.possibleMaterials;  
				
				
				if( possibleMaterialsOld.length<=0 && partSlot.selectedPart.linkedParts &&  partSlot.selectedPart.linkedParts.length>0){
					
					
					
				
					
					
					var mainPart:Part=getMainPartFromLinked();
					
					
					
					if(mainPart)
						possibleMaterialsOld = mainPart.possibleMaterials;  
					else
						possibleMaterialsOld=[];
					
					
					
					
				}
				
				
				for(i=0; i<possibleMaterialsOld.length; i++){
					
					material=possibleMaterialsOld[i];
					
					if(material.name=="mask")  
						continue;
					
					if(material.name=="antimask")
						continue;
					
					_possibleMaterials.push(material);
				}
			}
			
			return _possibleMaterials;
		}
		
		public function set partSlot(value : PartSlot) : void
		{
			_partSlot = value;
			
			if(_partSlot){
				partColor = _partSlot.selectedColor;
				
				
					
			
				
				invalidate("partColor");
				invalidate("partSlot");
			}
			
		}
		
		public function get partSlot() : PartSlot
		{
			return _partSlot;
		}
		
		public function set partColor(value : PartColor) : void
		{
			_partColor = value;
			invalidate("partColor");
		}
		
		public function get partColor() : PartColor
		{
			return _partColor;
		}
		
		public function updateWithLanguage(language : Language) : void
		{
			currentLanguage = language;
			
			if(materialLabel)
			{
				materialLabel.updateWithLanguage(language);
			}
			
			//colorLabel.updateWithLanguage(language);
			
			//noPartSelectedLabel.updateWithLanguage(language);
			//noPartSelectedLabel.text = language.translate("mob_colors", "noPartSelected");
			
			invalidate("partColor");
			invalidate("partSlot");
		}
		
		override protected function initialize() : void
		{
			super.initialize();
			
			if(showsMaterial)
			{
				quad = new Quad(1, 1, 0xFFFFFF);
				quad.alpha=0;
				addChild(quad);
			
				//var shadowTexture : Texture = assetManager.getTexture("shadow")
				//var shadowTextures : Scale3Textures = new Scale3Textures(shadowTexture, 0, 1, Scale3Textures.DIRECTION_VERTICAL);
				//rightShadow = new Scale3Image(shadowTextures);
				//addChild(rightShadow);
			}
			
			colorSampleImage = new Image(assetManager.getTexture("a_color_rect"));
			colorSampleImage.color = 0xFF0000;
			
			colorSampleImage.width=25;
			colorSampleImage.height=25;
			addChild(colorSampleImage);
			
			if(showsMaterial) 
			{
				materialLabel = new MultilingualLabel();
				materialLabel.textFormat = new TextFormat(FontSelector.getFont(), 5, 0x828687);
				materialLabel.textFormat.align = TextFormatAlign.CENTER;
				addChild(materialLabel);
			}
			
			/*colorLabel = new MultilingualLabel();
			colorLabel.textFormat = new TextFormat(FontSelector.getFont(), 7, 0x3F3537);
			colorLabel.textFormat.horizontalAlign = TextFormatAlign.CENTER;
			colorLabel.wordWrap = true;
			addChild(colorLabel);*/
				
			/*noPartSelectedLabel = new MultilingualLabel();
			noPartSelectedLabel.textFormat = new TextFormat(FontSelector.getFont(), 7, 0x3F3537);
			noPartSelectedLabel.textFormat.horizontalAlign = TextFormatAlign.CENTER;
			noPartSelectedLabel.wordWrap = true;
			addChild(noPartSelectedLabel);		
			noPartSelectedLabel.visible = false;*/
			
			
			
			
			
			
			if(showsOnlyMaterials)
			{
				if(materialLabel)
				{
					materialLabel.visible = false;
				}
				
				//colorLabel.visible = false;
				colorSampleImage.visible = false;
				
				
			}
			else
			{
				button = new Button();
				button.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
				addChild(button);
			}
			
			//контейнер для радиокнопок
			container = new ScrollContainer();
			container.horizontalScrollPolicy = ScrollPolicy.OFF;
			//var layout:VerticalLayout = new VerticalLayout();
			//layout.gap = 20;
			//layout.padding = 20;
			//container.layout = layout;
			this.addChild( container );
			container.visible = false;
			
			
			
		}
		
		override protected function draw() : void
		{
			super.draw();
			
			var sizeInvalid : Boolean = isInvalid(INVALIDATION_FLAG_SIZE);
			
			if(sizeInvalid) 
			{
				if(quad)
				{
					quad.width = actualWidth;
					quad.height = actualHeight;
				
					//rightShadow.x = actualWidth;
					//rightShadow.height = actualHeight;
				}
				
				colorSampleImage.x = int((actualWidth - colorSampleImage.width) / 2);
				colorSampleImage.y = (actualHeight - colorSampleImage.height)/2;
				
				//colorLabel.x = 5;
				//colorLabel.width = 54;
				//colorLabel.y = 2;
				//colorLabel.height = 25;
				
				//colorLabel.visible=false;
				
				if(materialLabel)
				{
					materialLabel.x = 5;
					materialLabel.width = 54;
					//materialLabel.y = colorLabel.y + colorLabel.height - 4;
					materialLabel.height = 10;
				
				}
				
				if(!showsOnlyMaterials)
				{
					button.width = actualWidth;
					button.height = actualHeight;
				}
			}
			
			if(container){
				container.width = actualWidth - container.x;
				container.height = actualHeight;
			}
			
			
			if(partSlot && isInvalid("partSlot") && currentLanguage)
			{
				if(materialLabel && partSlot.selectedPart && partSlot.selectedPart.selectedMaterial)
				{
					materialLabel.text = currentLanguage.translate("materials", partSlot.selectedPart.selectedMaterial.name).toUpperCase();
				}
				else if(partSlot.selectedPart && partSlot.selectedPart.linkedParts && partSlot.selectedPart.linkedParts.length>0){
				
					var mainPart:Part=getMainPartFromLinked();
					
					if(mainPart && mainPart.parentSlot && mainPart.parentSlot.selectedPart && mainPart.parentSlot.selectedPart.selectedMaterial){
						
						materialLabel.text = currentLanguage.translate("materials",mainPart.parentSlot.selectedPart.selectedMaterial.name).toUpperCase();
					}else{
						
						materialLabel.text = "";
					}
				
				}else if(materialLabel)
				{
					materialLabel.text = "";
				}
				
				if(opened || showsOnlyMaterials) 
				{
					updateRadioButtons();
				}
			}
						
			if(partColor && isInvalid("partColor")) 
			{
				if(partSlot)
				{
					if(partSlot.selectedColor!=null){   
						
						if(partSlot.selectedColor.value!=0xffffff){
							_partColor = partSlot.selectedColor;
						}else if(partSlot.selectedPart && partSlot.selectedPart.linkedParts && partSlot.selectedPart.linkedParts.length>0){
							
							mainPart=getMainPartFromLinked();
							
							if(mainPart){
								
								_partColor=mainPart.parentSlot.selectedColor;
							}
						}
					}
					
				}
			
				
				if(partColor.isCustom && showsIconForCustomColors)
				{
					colorSampleImage.color = 0xFFFFFF;
					colorSampleImage.texture = assetManager.getTexture("a_custom_color_button");
					
					colorSampleImage.width=25;
					colorSampleImage.height=25;
				}
				else
				{
					
					colorSampleImage.color = partColor.value;
					
					if(materialLabel && partSlot.selectedPart)
					{
						if(partSlot.selectedPart.selectedMaterial && partSlot.selectedPart.selectedMaterial.name == "metal")
						{
							
							
							colorSampleImage.color = PartColor.noColor().value;
						}
					}
				}
			}
		}
		
		private function buttonTriggeredHandler(event : Event) : void
		{
			if(showsMaterial) 
			{
				dispatchEventWith(TOGGLE_COLOR_SELECTION, true);
			}
			else
			{
				dispatchEventWith(Event.SELECT);
			}
		}
		
		public function open() : void
		{
			if(!_opened)
			{
				addSeparator();
				updateRadioButtons();
				
				runTweens();
			}
		}
		
		public function close(animation:Boolean=true) : void
		{
			//trace("_opened"+_opened);
			if(_opened)
			{
				
				if(animation){
					runTweens();
				}else{
					
					var k : int = 0;
					touchable = false;
					
					var radioButtonsCount : int = radioButtons.length;
					
					for( k = 0; k < radioButtonsCount; k++)
					{
						
						
						radioButtons[k].alpha =0
					}
					
					
					//noPartSelectedLabel.alpha = 0;
					
					
					width=actualWidth-70;
					
					separator.alpha=1;
					
					touchable = true;
					
					if(_opened)
					{
						for( k  = 0; k < radioButtonsCount; k++)
						{
							var radio : Radio = radioButtons[k];
							removeChild(radio);
						}
						
						radioButtons = [];
						
						removeChild(separator);
					}
					
					_opened = false;
					
				}
			}
		}
		
		private var materialGroup:ToggleGroup=null;
		private var changeBlock:Boolean=false;
		
		public function cancelChangeMaterial():void{
			
			if(materialGroup && partSlot.selectedPart && partSlot.selectedPart.selectedMaterial){
				changeBlock=true;
				materialGroup.selectedIndex = possibleMaterials.indexOf(partSlot.selectedPart.selectedMaterial);
				changeBlock=false;
			}
		}
		
		private function updateRadioButtons() : void
		{
			if(!container){
				return;
			}
			
			var group : ToggleGroup = new ToggleGroup();
			var mainPart:Part;
			materialGroup=group;
			
			var labelFactory : Function = function() : ITextRenderer
			{
				var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
				
				textRenderer.textFormat = new TextFormat(FontSelector.getFont(), 7, 0x3F3537);
				textRenderer.embedFonts = FontSelector.embedFonts();
				
				return textRenderer;
			};

			for each(var addedRadio : Radio in radioButtons) 
			{
				container.removeChild(addedRadio);
			}
			
			radioButtons = [];
			
			
			
			if(partSlot.selectedPart)
			{	
				//noPartSelectedLabel.visible = false;
				container.visible = true;
				
				container.x = showsOnlyMaterials ? 10 : separator.x + separator.width + 8;
				container.width = actualWidth - container.x;
				
			

				
				if(!showsOnlyMaterials)
				{
					
					if(partSlot.selectedPart.selectedState){
						
						var materialName:String="metal";
						if(partSlot.selectedPart && partSlot.selectedPart.selectedMaterial){
							
							if(partSlot.selectedPart.selectedState.nameOfState.indexOf("neon")<0)
							materialName=partSlot.selectedPart.selectedMaterial.name;
							
							
							var i:int;
							var selI:int=0;
							
							for(i=0; i<possibleMaterials.length; i++){   
								
								if(materialName==possibleMaterials[i].name){
									selI=i;
									break;
								}   
							}
							
							
							group.selectedIndex=selI;     
							
						}else
						if(partSlot.selectedPart.selectedState.selectedMaterial){
							   
							
							
							group.selectedIndex = possibleMaterials.indexOf(partSlot.selectedPart.selectedState.selectedMaterial);
						}else{
							group.selectedIndex = 0;
						}
						
					}else{
						
						if(partSlot && partSlot.selectedPart && !partSlot.selectedPart.selectedMaterial){
							
							mainPart = getMainPartFromLinked();
							
							if(mainPart && mainPart.parentSlot.selectedPart && mainPart.parentSlot.selectedPart.selectedMaterial){
								
								group.selectedIndex = mainPart.parentSlot.selectedPart.possibleMaterials.indexOf(mainPart.parentSlot.selectedPart.selectedMaterial);
							}else{
								
								group.selectedIndex -1;
							}
						}else{
							group.selectedIndex = possibleMaterials.indexOf(partSlot.selectedPart.selectedMaterial);
						}
					}
				}
				else
				{
					//
					if(partSlot.selectedPart.selectedMaterial){
						//group.selectedIndex = possibleMaterials.indexOf(partSlot.selectedPart.selectedMaterial);
						if(partSlot.name=="ALL"){
							group.selectedIndex = possibleMaterials.indexOf(partSlot.selectedPart.selectedMaterial);
						}else{
							group.selectedIndex = -1;
						}
					}
					else{
						
						if(partSlot.selectedPart.linkedParts && partSlot.selectedPart.linkedParts.length>0){
							
							
							mainPart = getMainPartFromLinked();
							
							if(mainPart && mainPart.parentSlot.selectedPart && mainPart.parentSlot.selectedPart.selectedMaterial){
								
								group.selectedIndex = mainPart.parentSlot.selectedPart.possibleMaterials.indexOf(mainPart.parentSlot.selectedPart.selectedMaterial);
							}
							
							
						}else{
						
							group.selectedIndex = -1;
						}
					}
					
				}
				
				group.addEventListener(Event.CHANGE, toggleGroupChangeHandler);
			}
			else {
				
				container.visible = false;
				
				
				//noPartSelectedLabel.height = 30;
				//noPartSelectedLabel.visible = true;
				//noPartSelectedLabel.x = separator.x + separator.width + 5;
				//noPartSelectedLabel.width = 50;
				//noPartSelectedLabel.y = int((actualHeight - noPartSelectedLabel.height) / 2);
			}
		}
		
		private function getMainPartFromLinked():Part{
			
			if(!partSlot){
				return null;
			}
			
			if(!partSlot.selectedPart){
				
				return null;
			}
			
			
			return partSlot.selectedPart.getMainPartFromLinked();
		}
		
		private function toggleGroupChangeHandler(event : Event) : void
		{
			
			if(changeBlock)
				return;
				
			var group : ToggleGroup = ToggleGroup(event.currentTarget);
			
			//trace("toggleGroupChangeHandler:",Radio(event.target).name);       
			var selIndex:int=0;
			
			//trace("group.selectedItem:", Radio(group.selectedItem).name );  
			
			//var radio_name:String=Radio(group.selectedItem).name;
			
			//var mas:Array=radio_name.split("_");
			
			selIndex=group.selectedIndex;//parseInt(mas[1]);
			
			trace("selIndex:", selIndex);  
			
			
			
			var selectedMaterial : PartMaterial
			
			
			
			selectedMaterial = possibleMaterials[selIndex];
			
			if(!selectedMaterial)return;   
			
			if(!selectedMaterial.isPaintable)
			{
				partColor = PartColor.noColor();  
			}
			else
			{
				partColor = partSlot.selectedColor;
			}
			
			materialLabel.text = selectedMaterial.name.toUpperCase();
			
			dispatchEventWith(TuningColorsView.MATERIAL_SELECTED, true, {selectedMaterial: possibleMaterials[selIndex], partSlot: partSlot});
		}
		
		private function addSeparator() : void
		{
			/*separator = new Quad(1, actualHeight - colorLabel.y - 10, 0xE4E5E5);
			separator.x = colorLabel.x + colorLabel.width + 12;
			separator.y = colorLabel.y + 4;
			separator.alpha = 0.0;
			addChild(separator);*/
		}
		
		private function runTweens() : void
		{
		
			touchable = false;
			
			var radioButtonsCount : int = radioButtons.length;
			
			for(var k : int = 0; k < radioButtonsCount; k++)
			{
				//var radioTween : Tween = new Tween(radioButtons[k], 0.2, Transitions.EASE_OUT);
				//radioTween.animate("alpha", _opened ? 0.0 : 1.0);
				//Starling.juggler.add(radioTween);
				
				radioButtons[k].alpha = _opened ? 0.0 : 1.0;
			}
			
			//var noPartSelectedLabelTween : Tween = new Tween(noPartSelectedLabel, 0.2, Transitions.EASE_OUT);
			//noPartSelectedLabelTween.animate("alpha", _opened ? 0.0 : 1.0);
			//Starling.juggler.add(noPartSelectedLabelTween);
			
			//noPartSelectedLabel.alpha = _opened ? 0.0 : 1.0;
			
			var tween : Tween = new Tween(this, 0.3, Transitions.EASE_OUT);
			tween.animate("width", actualWidth + (_opened ? -70 : 70));
			tween.onUpdate = function() : void
			{
				var t : Number = _opened ? 1 - tween.progress : tween.progress;
				separator.alpha = t;
			}
			tween.onComplete = function() : void
			{
				touchable = true;
				
				if(_opened)
				{
					for(var k : int = 0; k < radioButtonsCount; k++)
					{
						var radio : Radio = radioButtons[k];
						removeChild(radio);
					}
					
					radioButtons = [];
					
					removeChild(separator);
				}
				
				_opened = !_opened;
			}
				
			Starling.juggler.add(tween);
		}
	}
}