package com.despark.tuning3d.view.components.tunings.editing {
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.parts.PartSlot;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.parts.PartSlotItemRenderer;

    import feathers.controls.Button;
    import feathers.controls.List;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.core.FeathersControl;
    import feathers.data.IListCollection;
    import feathers.data.ListCollection;
    import feathers.layout.HorizontalLayout;

    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.textures.Texture;

    public class ComponentSelector extends FeathersControl {
        public static const SELECTOR_TRIGGERED:String = "selectorTriggered";

        private var _list:List;
        private var _language:Language;
        private var _partSlotNameWidthsMap:Object;
        private var _dataProvider:IListCollection;
        private var _leftButton:Button;
        private var _rightButton:Button;
        private var _centerButton:Button;
        private var _rightBorder:Image;
        private var _selectedItem:PartSlot;
        private var background:Quad;
        public function ComponentSelector() {
            super();
            _partSlotNameWidthsMap = {};
        }

        override protected function initialize():void{
            background = new Quad(1, 1, 0xFFFFFF);
            background.alpha = 0.05;
            addChild(background);
            //
            _list = new List();
            _list.isSelectable = false;
            _list.isEnabled = false;
            _list.clipContent = true;
            _list.hasElasticEdges = false;
            _list.snapToPages = true;
            _list.addEventListener(Event.CHANGE, listChangeHandler);
            _list.itemRendererFactory = function ():IListItemRenderer {
                var renderer:PartSlotItemRenderer = new PartSlotItemRenderer();
                renderer.language = _language;
                renderer.partSlotNameWidthsMap = _partSlotNameWidthsMap;
                renderer.noBorder = true;
                return renderer;
            };
            var horizontalLayout:HorizontalLayout = new HorizontalLayout();
            horizontalLayout.manageVisibility = true; //tt
            horizontalLayout.useVirtualLayout = false;
            horizontalLayout.gap = 0;
            _list.layout = horizontalLayout;
            addChild(_list);
            //
            _leftButton = new Button();
            _rightButton = new Button();
            _centerButton = new Button();
            //
            var texture:Texture = ButtonFactory.assetManager.getTexture('c_selector_right');
            var rightImage:Image = new Image(texture);
            var rightDownImage:Image = new Image(texture);
            var leftImage:Image = new Image(texture);
            var leftDownImage:Image = new Image(texture);

            rightDownImage.alpha = 0.4;
            leftDownImage.alpha = 0.4;

            _rightButton.defaultSkin = rightImage;
            _rightButton.downSkin = rightDownImage;
            _rightButton.disabledSkin = rightDownImage;

            _leftButton.defaultSkin = leftImage;
            _leftButton.downSkin = leftDownImage;
            _leftButton.disabledSkin = leftDownImage;

            leftImage.pivotX = leftImage.width;
            leftImage.pivotY = leftImage.height;
            leftImage.rotation = Math.PI;

            leftDownImage.pivotX = leftDownImage.width;
            leftDownImage.pivotY = leftDownImage.height;
            leftDownImage.rotation = Math.PI;

            _rightBorder = new Image(Texture.fromColor(1, 1, 0xFFFFFF));
            _rightBorder.width = 0.5;
            //
            var middleImage:Image = new Image(Texture.fromColor(1,1));
            middleImage.alpha = 0;
            _centerButton.defaultSkin = middleImage;
            //

            addChild(_centerButton);
            addChild(_rightButton);
            addChild(_leftButton);
            addChild(_rightBorder);

            _rightButton.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
            _leftButton.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
            _centerButton.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
        }

        private function listChangeHandler(event:Event):void {
            updateLayout();
        }

        private function updateLayout():void{
            var index:int = -1;
            if(selectedItem && _list.dataProvider){
                index = _list.dataProvider.getItemIndex(selectedItem);
            }
            if(index != -1){
                _leftButton.isEnabled = (index > 0);
                _rightButton.isEnabled = (index < _list.dataProvider.length - 1);
            }else{
                _leftButton.isEnabled = false;
                _rightButton.isEnabled = false;
            }
        }

        private function buttonTriggeredHandler(event:Event):void {
            var partSlot:PartSlot;
            var index:int = _list.dataProvider.getItemIndex(_selectedItem);
            switch (event.currentTarget) {
                case _leftButton:
                    if(index > 0 && !_list.isScrolling){
                        partSlot = _list.dataProvider.getItemAt(index - 1) as PartSlot;
                        _list.scrollToDisplayIndex(index - 1, 0.4);
                        _list.selectedIndex = index;
                        _selectedItem = partSlot;
                        dispatchEvent(new Event(Event.CHANGE, false, {selectedItem:_selectedItem}));
                    }
                    break;
                case _rightButton:
                    if(index < _list.dataProvider.length - 1 && !_list.isScrolling){
                        partSlot = _list.dataProvider.getItemAt(index + 1) as PartSlot;
                        _list.scrollToDisplayIndex(index + 1, 0.4);
                        _list.selectedIndex = index;
                        _selectedItem = partSlot;
                        dispatchEvent(new Event(Event.CHANGE, false, {selectedItem:_selectedItem}));
                    }
                    break;
                case _centerButton:
                    dispatchEvent(new Event(SELECTOR_TRIGGERED));
                    break;
            }
            updateLayout();
        }

        override protected function draw():void{
            if(background){
                background.width = actualWidth;
                background.height = actualHeight;
            }

            if(_list){
                _list.width = PartSlotItemRenderer.WIDTH;
                _list.x = (actualWidth - PartSlotItemRenderer.WIDTH) / 2;
            }

            if(_leftButton){
                _leftButton.height = 50;
                _leftButton.width = actualHeight * Image(_leftButton.defaultSkin).texture.frameWidth / Image(_leftButton.defaultSkin).texture.frameHeight;
            }

            if(_rightButton){
                _rightButton.height = 50;
                _rightButton.width = actualHeight * Image(_rightButton.defaultSkin).texture.frameWidth / Image(_rightButton.defaultSkin).texture.frameHeight;
                _rightButton.x = actualWidth - _rightButton.width;
            }

            if(_rightBorder){
                _rightBorder.height = actualHeight;
                _rightBorder.x = actualWidth - _rightBorder.width;
            }

            if(_centerButton){
                _centerButton.x = _leftButton.width;
                _centerButton.width = actualWidth - (_leftButton.width + _rightButton.width);
                _centerButton.height = actualHeight;
            }
        }

        public function push(item:Object):void{
            if(item && !_list.dataProvider.contains(item) && isComponentAllowed(item)){
                _list.dataProvider.push(item);
            }
        }

        public function unshift(item:Object):void{
            if(item && !_list.dataProvider.contains(item) && isComponentAllowed(item)){
                _list.dataProvider.unshift(item);
            }
        }

        public function set selectedItem(value:Object):void{
            if(_list && isEnabled && _list.dataProvider && _list.dataProvider.contains(value)){
                if(value) {
                    var currentIndex:int = _list.selectedIndex;
                    _selectedItem = value as PartSlot;
                    var toIndex:int = _list.dataProvider.getItemIndex(_selectedItem);
                    _list.selectedItem = value;
                    if (toIndex != -1) {
                        _list.scrollToPageIndex(toIndex, 0, 0.2);
                    }
                    if(toIndex > currentIndex){
                        _list.selectedIndex = toIndex - 1;
                    }else if(toIndex < currentIndex){
                        _list.selectedIndex = toIndex + 1;
                    }
                    //_list.dataProvider.updateItemAt(_list.dataProvider.getItemIndex(value));
                    _list.dataProvider.updateAll();
                }
            }
        }

        public function set selectedIndex(value:int):void{
            if(_list){
                _list.selectedIndex = value;
                if(_list.selectedIndex != -1){
                    _list.scrollToDisplayIndex(_list.selectedIndex, 0);
                }
            }
        }

        public function set language(value:Language):void {
            _language = value;
        }

        public function get dataProvider():IListCollection {
            return _dataProvider;
        }

        private function isComponentAllowed(partSlot:Object):Boolean{
            return (partSlot.name != "ALL" && partSlot.name != "Suspension");
        }

        public function set dataProvider(value:IListCollection):void {
            _dataProvider ||= new ListCollection();
            _dataProvider.removeAll();
            _dataProvider.addAll(value);
            for(var i:int = _dataProvider.length - 1; i >= 0; i--){
                var partSlot:PartSlot = _dataProvider.getItemAt(i) as PartSlot;
                if(!isComponentAllowed(partSlot)){
                    _dataProvider.removeItemAt(i);
                }
            }
            if(_list){
                _list.dataProvider = _dataProvider;
                updateLayout();
            }
        }

        public function get selectedItem():Object {
            return _selectedItem as Object;
        }
    }
}
