package com.despark.tuning3d.view.components.tunings.editing {
    import com.despark.tuning3d.view.components.auth.*;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.greensock.TweenLite;
import com.utils.FontSelector;

import feathers.controls.Button;

import feathers.controls.Label;
import feathers.controls.LayoutGroup;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.FeathersControl;
import feathers.core.ITextRenderer;
import feathers.layout.FlowLayout;
import feathers.layout.HorizontalAlign;
import feathers.layout.VerticalAlign;

import flash.geom.Rectangle;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.core.Starling;
import starling.display.Image;
import starling.events.Event;
import starling.textures.Texture;

public class FastModeHelpView extends FeathersControl {
    private var backgroundImage:Image;
    private var closeButton:Button;
    private var titleField:Label;
    private var messageField:Label;
    private var onCLose:Function;
    private var controlGroup:LayoutGroup;
    //
    private var titleText:String;
    private var messageText:String;
    public function FastModeHelpView() {
        super();
        width = 320;
        height = 160;
    }

    public static function show(title:String = '', message:String = '', onClose:Function = null):FastModeHelpView{
        var alertView:FastModeHelpView = new FastModeHelpView();
        alertView.titleText = title;
        alertView.messageText = message;
        alertView.onCLose = onClose;
        alertView.open();
        return alertView;
    }

    private function open():void{
        alpha = 0;
        x = (Starling.current.stage.stageWidth - width)/2;
        y = (Starling.current.stage.stageHeight - height)/2;
        Starling.current.stage.addChild(this);
        var alertView:FastModeHelpView = this;
        TweenLite.to(alertView, .2, {alpha: 1});
    }

    public function close():void{
        TweenLite.killDelayedCallsTo(close);
        var alertView:FastModeHelpView = this;
        TweenLite.to(alertView, .1, {alpha: 0, onComplete: function():void{
                if(alertView){
                    alertView.removeFromParent(true);
                    alertView = null;
                }
                if(onCLose != null){
                    onCLose();
                }
            }});
    }

    override protected function initialize():void {
        super.initialize();
        //
        if(backgroundImage){
            backgroundImage.removeFromParent(true);
        }
        var backgroundTexture : Texture = ButtonFactory.assetManager.getTexture("controlBackground");
        backgroundImage = new Image(backgroundTexture);
        backgroundImage.scale9Grid = new Rectangle(backgroundTexture.frameWidth/3, backgroundTexture.frameHeight/3,backgroundTexture.frameWidth/3, backgroundTexture.frameHeight/3);
        backgroundImage.width = width;
        backgroundImage.height = height;
        backgroundImage.alpha = .9;
        addChildAt(backgroundImage, 0);

        controlGroup = new LayoutGroup();
        var verticalLayout:FlowLayout = new FlowLayout();
        verticalLayout.verticalGap = 12.5;
        verticalLayout.horizontalGap = 4;
        verticalLayout.horizontalAlign = HorizontalAlign.LEFT;
        verticalLayout.rowVerticalAlign = VerticalAlign.MIDDLE;
        controlGroup.layout = verticalLayout;
        controlGroup.width = width - 30;
        controlGroup.height = height - 30;
        controlGroup.x = controlGroup.y = 10;


        titleField = new Label();
        titleField.width = controlGroup.width;
        titleField.text = titleText;
        titleField.wordWrap = true;
        titleField.paddingTop = -1;
        titleField.textRendererFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMonserratSemiBold(), 12, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            textRenderer.isHTML = true;
            return textRenderer;
        };

        messageField = new Label();
        messageField.width = controlGroup.width;
        messageField.text = messageText;
        messageField.wordWrap = true;
        messageField.textRendererFactory = function ():ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            textRenderer.textFormat = new TextFormat(FontSelector.getMonserratSemiBold(), 9, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.LEFT;
            textRenderer.textFormat.bold = false;
            textRenderer.embedFonts = true;
            textRenderer.isHTML = true;
            textRenderer.wordWrap = true;
            return textRenderer;
        };

        closeButton = ButtonFactory.popupCloseButton(10, 10);
        closeButton.addEventListener(Event.TRIGGERED, closeHandler);
        closeButton.x = actualWidth - (closeButton.width + 9);
        closeButton.y = 9;

        controlGroup.addChild(titleField);
        controlGroup.addChild(messageField);
        addChild(controlGroup);
        addChild(closeButton);
    }

    override protected function draw():void{
        if(messageField){
            messageField.width = actualWidth - 2 * messageField.x;
            messageField.validate();
        }
    }

    override public function dispose():void {
        super.dispose();
        if(backgroundImage){
            backgroundImage.removeFromParent(true);
            backgroundImage = null;
        }

        if(closeButton){
            closeButton.removeFromParent(true);
            closeButton = null;
        }

        if(titleField){
            titleField.removeFromParent(true);
            titleField = null;
        }

        if(messageField){
            messageField.removeFromParent(true);
            messageField = null;
        }

        if(controlGroup){
            controlGroup.removeFromParent(true);
            controlGroup = null;
        }
    }

    private function closeHandler(event:Event):void {
        close();
    }
}
}
