package com.despark.tuning3d.view.components.tunings.editing
{
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.LabelFactory;

	import feathers.controls.Button;
	import feathers.controls.Label;
	import feathers.core.FeathersControl;

	import flash.geom.Point;

	import starling.display.Image;
	import starling.display.Quad;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.textures.Texture;
	import starling.utils.AssetManager;

	public class SliderControl extends FeathersControl {
		public static const VALUE_CHANGED : String = "SliderValueChanged";
		private var _handleSize:Number = 200;
		private var _handleMargin:Number = 30;

		public var assetManager : AssetManager;
		
		private var _minValue : int;
		private var _maxValue : int;

		private var backgroundImage:Quad;
		protected var handlerImage:Image;

		private var _value:Number = 0;
		
		private var quad:Quad;
		private var _touchBegan:Boolean = false;
		private var _customValues:Array;
		private var _plusButton:Button;
		private var _minusButton:Button;
		private var _label:Label;
		private var _instantApply:Boolean;
		private static const HANDLE_SIZE:int = 12;
		public function SliderControl(instantApply:Boolean = true) {
			super();
			quad = new Quad(1, 1, 0xFF0000);
			_label = new Label();
			backgroundImage = new Quad(1, 1, 0xFFFFFF);
			_instantApply = instantApply;
			addEventListener(TouchEvent.TOUCH, touchHandler);
			setMinMaxValues(0, 1);
		}
		
		public function setMinMaxValues(min:int, max:int):void{
			_maxValue = max;
			_minValue = min;
			invalidate("value");
		}
		
		public function get maxValue() : int{
			return _maxValue;
		}
		

		public function get minValue() : int {
			return _minValue;
		}
		
		override protected function initialize() : void {
			super.initialize();
			quad.alpha = 0.0;
			addChild(quad);

			backgroundImage.height = 2;
			addChild(backgroundImage);

			_plusButton = ButtonFactory.simpleButtonByTextureNameIcon("slider_plus", true);
			_minusButton = ButtonFactory.simpleButtonByTextureNameIcon("slider_minus", true);
			_plusButton.width = _plusButton.height = _handleMargin;
			_minusButton.width = _minusButton.height = _handleMargin;
			addChild(_minusButton);
			addChild(_plusButton);
			_minusButton.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
			_plusButton.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);

			var handlerTexture:Texture  = ButtonFactory.assetManager.getTexture("a_new_suspension_handler");
			handlerImage = new Image(handlerTexture);
			handlerImage.width = HANDLE_SIZE;
			handlerImage.height = HANDLE_SIZE;
			handlerImage.touchable = false;
			addChild(handlerImage);

			_label.textRendererFactory = LabelFactory.tuningSettingLabel;
			addChild(_label);
		}

		private function buttonTriggeredHandler(event:Event):void {
			switch (event.currentTarget) {
				case _minusButton:
					if(value < maxValue){
						value++;
						dispatchEvent(new Event(VALUE_CHANGED, true, {value: -value}));
					}
					break;
				case _plusButton:
					if(value > minValue){
						value--;
						dispatchEvent(new Event(VALUE_CHANGED, true, {value: -value}));
					}
					break;
			}
		}
		
		override public function dispose():void{
			removeEventListener(TouchEvent.TOUCH, touchHandler);
			super.dispose();
			
		}


		override protected function draw() : void {
			super.draw();
			if(isInvalid(INVALIDATION_FLAG_SIZE)) {
				backgroundImage.width = actualWidth - (_plusButton.width + _minusButton.width + 2);
				backgroundImage.x = _plusButton.width;
				backgroundImage.y = (actualHeight - backgroundImage.height)/2 + 3;

				_plusButton.x = actualWidth - _plusButton.width;

				quad.width = actualWidth;
				quad.height = actualHeight;

				_label.x = (actualWidth - _label.width) / 2;
				_label.y = -_label.height + 6;

				_plusButton.y = _minusButton.y = backgroundImage.y + backgroundImage.height / 2 - _minusButton.height / 2;
			}

			if(isInvalid("value") || isInvalid(INVALIDATION_FLAG_SIZE)){
				handlerImage.x = posForValue(_value);
				handlerImage.y = backgroundImage.y + backgroundImage.height / 2 - handlerImage.height / 2;
			}
		}
		
		public function set value(val:Number):void{
			_value = val;
			invalidate("value");
		}
		
		public function get value():Number{
			return _value;
		}

		override public function setSize(w:Number, h:Number):void{
			super .setSize(w, h);
			_handleSize = w - HANDLE_SIZE - 2 * _handleMargin;
		}
		
		public function posForValue(val : int) : Number {
			var dest:Number = (minValue - maxValue);
			if(dest == 0) {
				return _handleMargin;
			}
			return (((val - maxValue) / dest) * _handleSize) + _handleMargin;
		}
		
		public function valueForPos(posX : Number) : int {
			var dest:Number = (minValue - maxValue);
			return  Math.round((dest - (((posX - _handleMargin) / _handleSize) * dest))-minValue);
		}
		
		protected function touchHandler(event : TouchEvent) : void {
			var touch : Touch = event.getTouch(this);
			if(minValue - maxValue == 0) {
				return;
			}

			if(!touch) {
                _touchBegan = false;
				return;
			}	
            var touchX : Number = this.globalToLocal(new Point(touch.globalX, touch.globalY)).x;
			var dX : Number;
			if(touch.phase == TouchPhase.BEGAN) {
				dX = touchX;
				if(dX < _handleMargin || dX > _handleSize + _handleMargin + HANDLE_SIZE) {
					return;
				}
				_touchBegan = true;
			}
			if((touch.phase == TouchPhase.BEGAN || touch.phase == TouchPhase.MOVED) && _touchBegan) {
				dX = touchX;
				if(dX < _handleMargin){
					dX = _handleMargin;
				} else if(dX > _handleSize + _handleMargin) {
					dX = _handleSize + _handleMargin;
				}
				value = -valueForPos(dX);
				if(_instantApply){
					dispatchEvent(new Event(VALUE_CHANGED, true, {value: valueForPos(dX)}));
				}
			} else {
				if(_touchBegan && !_instantApply){
					dispatchEvent(new Event(VALUE_CHANGED, true, {value: -value}));
				}
                _touchBegan = false;
			}
		}

		public function get customValue():*{
			if(_customValues && _customValues[value]){
				return _customValues[value];
			}
			return null;
		}

		public function get customValues():Array {
			return _customValues;
		}

		public function set customValues(value:Array):void {
			_customValues = value;
			if(_customValues && _customValues.length){
				setMinMaxValues(0, _customValues.length - 1);
			}
		}

		public function set text(val:String):void {
			if(_label){
				_label.text = val;
				_label.validate();
			}
		}
	}
}

