package com.despark.tuning3d.view.components.tunings.editing
{
import feathers.core.FeathersControl;

import starling.display.Image;
import starling.display.Quad;
import starling.events.Touch;
import starling.events.TouchEvent;
import starling.events.TouchPhase;
import starling.utils.AssetManager;

public class TuningOffsetControl extends FeathersControl
	{
		public static const FRONT_OFFSET_CHANGED : String = "FRONT_OFFSET_CHANGED";
		
		public var MAX_HEIGHT : int = 100;
		public var MIN_HEIGHT : int = 36;
		public var PIXELS_FOR_CHANGE : int = 1;
		
		public var assetManager : AssetManager;
		
		private var _maxSuspension : int;
		private var _minSuspension : int;

		private var lines : Array;
		private var touchStartY : int;
		private var startHeight : int;
		private var startY : int;
		private var quad : Quad;
		
		public function TuningOffsetControl()
		{
			super();
			
			addEventListener(TouchEvent.TOUCH, touchHandler);
			lines = [];
			/*_maxSuspension = 15;
			
			_minSuspension=-15;
			*/
			setMinMaxSuspension(15, -15);
			
			
		}
		
		public function setMinMaxSuspension(min:int, max:int):void{
			
			_minSuspension=max;
			_maxSuspension=min;

		}
		
		public function get minSuspension() : int{
			
			return _minSuspension;
		}
		
		
		
		public function get maxSuspension() : int
		{
			return _maxSuspension;
		}
		

		
		override protected function initialize() : void
		{
			super.initialize();
			
			quad = new Quad(1, 1, 0xFF0000);
			quad.alpha = 0.0;
			addChild(quad);
		}
		
		override protected function draw() : void
		{
			super.draw();
			
			if(isInvalid(INVALIDATION_FLAG_SIZE))
			{
				var quadPadding : int = 9;
				
				quad.y = -quadPadding;
				quad.x = -quadPadding;

				quad.height = actualHeight + 2 * quadPadding;
				
				var currentY : int = 10;
				var d : int = (actualHeight - 2*9) / lines.length;
				
				for(var i : int = 0; i < lines.length; i++) 
				{
					var lineImage : Image = Image(lines[i]);
					
					lineImage.y = int(currentY + (d - lineImage.height) / 2);
					currentY += d;
				}
				

			}
		}
		
		public function heightForOffset(suspension : int) : int
		{
			

			
			var DEST_PX:Number=MAX_HEIGHT-MIN_HEIGHT;
			var dest:Number=(maxSuspension-minSuspension);
			
			//trace("heightForSuspension:"+int((((suspension-maxValue)/dest)*DEST_PX)+MIN_HEIGHT));
			return int((((suspension-minSuspension)/dest)*DEST_PX)+MIN_HEIGHT);  
		}
		
		public function offsetForHeight(height : int) : int
		{
			
			//trace("suspensionForHeight height:"+height);
			var DEST_PX:Number=MAX_HEIGHT-MIN_HEIGHT;
			var dest:Number=(maxSuspension-minSuspension);
			//trace("suspensionForHeight:"+(dest-((((height - MIN_HEIGHT) / DEST_PX)*dest))-minValue));
			return  (dest-((((height - MIN_HEIGHT) / DEST_PX)*dest)))-maxSuspension;   // PIXELS_FOR_CHANGE;  
		}
		
		private function touchHandler(event : TouchEvent) : void 
		{
			const touch : Touch = event.getTouch(this);
			
			if(!touch)
			{
				return;
			}	
			
			if(touch.phase == TouchPhase.BEGAN)
			{		
				touchStartY = touch.globalY;
				
				startHeight = actualHeight;
				startY = y;
			}
			else if(touch.phase == TouchPhase.MOVED)
			{
				var dY : int = touchStartY - touch.globalY;
				
				height = startHeight + dY;
				
				if(height > MAX_HEIGHT)
				{
					height = MAX_HEIGHT;
					dY = MAX_HEIGHT - startHeight;
				}
				else if(height < MIN_HEIGHT)
				{
					height = MIN_HEIGHT;
					dY = MIN_HEIGHT - startHeight;
				}
				
				y = startY - dY;
				
				dispatchEventWith(FRONT_OFFSET_CHANGED, true, {offset: offsetForHeight(height)});
			}
		}
	}
}


