package com.despark.tuning3d.view.components.tunings.editing {
    import api.com.adobe.images.PNGEncoder;

    import com.adobe.utils.DateUtil;

    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.model.cars.Car;
    import com.despark.tuning3d.model.cars.CarBackground;
    import com.despark.tuning3d.model.cars.CarBackgroundsProxy;
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.core.PromoVideoManager;
    import com.despark.tuning3d.model.core.Tracer;
    import com.despark.tuning3d.model.core.images.Base64;
    import com.despark.tuning3d.model.disks.Disk;
    import com.despark.tuning3d.model.disks.DiskBrand;
    import com.despark.tuning3d.model.disks.DiskBrandsProxy;
    import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.languages.Multilingual;
    import com.despark.tuning3d.model.packages.PackagesProxy;
    import com.despark.tuning3d.model.packages.Packet;
    import com.despark.tuning3d.model.packages.Purchase;
    import com.despark.tuning3d.model.parts.Part;
    import com.despark.tuning3d.model.parts.PartColor;
    import com.despark.tuning3d.model.parts.PartDisk;
    import com.despark.tuning3d.model.parts.PartMaterial;
    import com.despark.tuning3d.model.parts.PartSlot;
    import com.despark.tuning3d.model.parts.PartSlotDisk;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.view.components.brands.BrandsScreen;
    import com.despark.tuning3d.view.components.cars.CarView;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.BackgroundContainer;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.BlockerView;
    import com.despark.tuning3d.view.components.core.ButtonFactory;
    import com.despark.tuning3d.view.components.core.CustomAlertView;
    import com.despark.tuning3d.view.components.core.ExtentedButton;
    import com.despark.tuning3d.view.components.core.FramedMenuScreen;
    import com.despark.tuning3d.view.components.core.IMainMenuHolder;
    import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
    import com.despark.tuning3d.view.components.core.MainMenuView;
    import com.despark.tuning3d.view.components.core.MenuBackButton;
    import com.despark.tuning3d.view.components.core.NotificationView;
    import com.despark.tuning3d.view.components.core.PackagesView;
    import com.despark.tuning3d.view.components.core.SpinnerView;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.core.ZipManager;
    import com.despark.tuning3d.view.components.partinfo.DetailInfoView;
    import com.despark.tuning3d.view.components.parts.BrandPartItemRenderer;
    import com.despark.tuning3d.view.components.parts.PartDiskItemRenderer;
    import com.despark.tuning3d.view.components.parts.PartItemRenderer;
    import com.despark.tuning3d.view.components.parts.PartSlotItemRenderer;
    import com.despark.tuning3d.view.components.tunings.ShareView;
    import com.despark.tuning3d.view.components.tunings.colors.EffectMenu;
    import com.despark.tuning3d.view.components.tunings.colors.TuningBackgroundMenuView;
    import com.despark.tuning3d.view.components.tunings.colors.TuningColorsView;
    import com.despark.tuning3d.view.components.tunings.colors.TuningDiskSettingsView;
    import com.despark.tuning3d.view.components.tunings.colors.TuningSelectedStateView;
    import com.distriqt.AdsManager;
    import com.distriqt.extension.camerarollextended.CameraRollExtended;
    import com.utils.DateParser;
    import com.utils.FileUtil;
    import com.utils.MemoryTracker;
    import com.utils.ObjectUtils;
    import com.utils.RegExpUtil;

    import feathers.controls.Button;
    import feathers.controls.List;
    import feathers.controls.ToggleButton;
    import feathers.controls.renderers.IListItemRenderer;
    import feathers.core.FeathersControl;
    import feathers.data.ListCollection;
    import feathers.data.ListCollection;
    import feathers.layout.HorizontalLayout;

    import flash.display.BitmapData;
    import flash.display.StageOrientation;
    import flash.events.Event;
    import flash.events.IOErrorEvent;
    import flash.filesystem.File;
    import flash.geom.Matrix;
    import flash.geom.Point;
    import flash.media.Sound;
    import flash.media.SoundChannel;
    import flash.media.SoundTransform;
    import flash.net.URLRequest;
    import flash.system.System;
    import flash.utils.ByteArray;
    import flash.utils.Dictionary;
    import flash.utils.clearTimeout;
    import flash.utils.getTimer;
    import flash.utils.setTimeout;

    import org.as3commons.lang.ArrayUtils;

    import starling.animation.Transitions;
    import starling.animation.Tween;
    import starling.core.Starling;
    import starling.display.DisplayObject;
    import starling.display.Image;
    import starling.display.Quad;
    import starling.events.Event;
    import starling.events.Touch;
    import starling.events.TouchEvent;
    import starling.events.TouchPhase;
    import starling.textures.Texture;
    import starling.utils.MathUtil;

    public class TuningScreen extends FramedMenuScreen implements Multilingual, IMainMenuHolder {

        public static const CAR_PART_LOADING_PROGRESS:String = 'CAR_PART_LOADING_PROGRESS';
        public static const CAR_PART_FULL_LOADED:String = 'CAR_PART_FULL_LOADED';
        public static const CAR_PART_LOCK:String = 'CAR_PART_LOCK';
        public static const PART_SELECTED:String = 'PartSelected';
        public static const PART_UNSELECTED:String = 'PartUnselected';
        public static const PART_SELECTED_DATA_CHAGE:String = 'PartSelectedDataChage';
        public static const PART_DISK_INFO_POP_UP:String = 'PartDiskInfoPopUp';

        public static const PART_INFO_POP_UP:String = 'PART_INFO_POP_UP';
        public static const PART_SLOT_SELECTED:String = 'PartSlotSelected';
        public static const PHOTO_BUTTON_SELECTED:String = 'PhotoButtonSelected';
        public static const SAVE_PROMO_VIDEO_SELECTED:String = 'SavePromoVideoSelected';
        public static const SAVE_TUNING_SELECTED:String = 'SaveTuningSelected';
        public static const DOWNLOADS_BUTTON_SELECTED:String = 'DownloadsButtonSelected';
        public static const SETTINGS_SELECTED:String = 'SettingsSelected';
        //INSTRUCTION_SELECTED
        private static const PART_DISK_WIDTH:int = 87;
        private static var PART_DISK_HEIGHT:int = 55;
        private static const PART_LIST_MARGIN:int = PartSlotItemRenderer.WIDTH + 30;

        public static const FULLSCREEN_SELECTED:String = 'FullscreenSelected';

        public static const ON_OPEN_SHARE_POP_UP:String = 'ON_OPEN_SHARE_POP_UP';
        public static const SHARE_VK_SELECTED:String = 'SHARE_VK_SELECTED';
        public static const SHARE_FB_SELECTED:String = 'SHARE_FB_SELECTED';
        public static const SHARE_OK_SELECTED:String = 'SHARE_OK_SELECTED';
        public static const SHARE_PIN_SELECTED:String = 'SHARE_PIN_SELECTED';
        public static const SHARE_INSTAGRAM_SELECTED:String = 'SHARE_INSTAGRAM_SELECTED';

        public static const BACKGROUND_SELECTED:String = 'BACKGROUND_SELECTED';
        public static const BACKGROUND_UPLOAD:String = 'BACKGROUND_UPLOAD';
        public static const BACKGROUND_UPLOAD_COMPLETE:String = 'BACKGROUND_UPLOAD_COMPLETE';
        public static const SNAPSHOT_BACKGROUND:String = 'SNAPSHOT_BACKGROUND';

        public static const BACKGROUND_DELETE:String = 'BACKGROUND_DELETE';
        public static const EXIT_FAST_MODE:String = 'EXIT_FAST_MODE';


        //public static const BACKGROUND_UNSELECTED:String="BACKGROUND_UNSELECTED";


        public static const SHAVING_SELECTED:String = 'SHAVING_SELECTED';

        public static const ACCEPT_CAMERA_SELECTED:String = 'ACCEPT_CAMERA_SELECTED';
        public static const CANCEL_CAMERA_SELECTED:String = 'CANCEL_CAMERA_SELECTED';

        public static const BUY_NOW_PACKET:String = 'onBuyNowPacket';

        private static var MENU_HEIGHT:int = 50;
        private static var PART_WIDTH:int = 80;
        private static var PART_HEIGHT:int = 50;
        private static const DISTANCE_BETWEEN_CAR_AND_MENU:int = 35;
        private static const DISTANCE_BETWEEN_CAR_AND_TOP:int = 10;
        private static const BUTTON_GAP:int = 10;
        private static var BUTTON_HORIZONTAL_GAP:int = 10;
        private static const BUTTON_SIDE_MARGIN:int = 7;
        public static const BUTTON_MENU_GAP:Number = 7.5;


        public var _mainMenuView:MainMenuView;

        public var bitmapdataFromCamera:BitmapData;
        public var _carView:CarView;
        public var tunning:Tuning;
        private var _selectedPart:Part;
        private var _selectedPartSlot:PartSlot;
        public var loadedCar:Car;

        public var frontWheelModified:Boolean;
        public var rearWheelModified:Boolean;


        private var possiblePartSlotsList:List;
        private var _possiblePartSlots:Array;
        private var partsList:List;
        private var componentSelector:ComponentSelector;
        public var partSlotNameWidthsMap:Object;
        private var _possibleScrollPosition:Number;

        public var tuningColorsView:TuningColorsView;
        private var _colorsViewEnabled:Boolean;
        private var initialScrollEventPassed:Boolean;
        public var suspensionControl:TuningSuspensionControl;
        public var tintControl:SliderControl;

        private var _car:Car;
        private var finishedScreenShotTweens:int;

        public var saveButton:Button;
        public var fastModeButton:ToggleButton;
        private var fastModeHelpButton:Button;
        private var fastModeHelpView:FastModeHelpView;

        public var cameraButton:Button;
        public var arButton:Button;
        public var currentScreenshotButton:Button;
        public var cameraScreenshotButton:Button;
        public var acceptFromCameraButton:Button;
        public var cancelFromCameraButton:Button;
        public var saveVideoPreviewButton1:Button;
        public var saveVideoPreviewButton2:Button;

        public var powerOnButton:ToggleButton;
        public var gasButton:Button;
        private var revSoundButton:ToggleButton;
        private var restorePosition:Number;

        public var allPartsColorsView:TuningColorsView;
        private var tuningSettingsPanel:TuningSettingsPanel;
        public var backgroundMenuView:TuningBackgroundMenuView;
        public var partDiskSettingView:TuningDiskSettingsView = null;
        public var progressBar:TuningProgressBar;
        public var progressFlatBar:TuningFlatProgressBar;
        private var maxAlphaForHiddenView:Dictionary;
        private var partSlotSelectedByCarTouch:Boolean;

        public var shareButton:Button;

        public var colorMenuButton:Button;
        private var partsMenuButton:Button;
        private var settingsMenuButton:Button;

        private var menuBackButton:MenuBackButton;

        public var language:Language;
        private var _suspensionEnabled:Boolean;

        private var diskBrandList:List;
        private var brandListCollection:Object;
        private var currentFrame:int = 0;

        private var _brandCollection:ListCollection;
        private var _vinylCollection:ListCollection;

        private var _partsCollection:ListCollection;
        public var packagesView:PackagesView;

        private var _packages:Array;
        private var _filteredPackages:Array;

        public var packet:Packet = null;
        public var bodyPartSlot:PartSlot = null;


        public var blockerView:BlockerView;
        private var lineBg:Quad;

        private var notificationView:NotificationView;
        private var _carBackgrounds:Array;

        public var selectedBackground:CarBackground = null;
        public var selectedFillColor:Object = null;

        private var shareView:ShareView;
        public var _selectedLanguage:Language;

        private var _languages:Array;
        private var _carSplashImage:ImageLoaderWithStates;

        public var infoView:DetailInfoView;
        public var fastModeEnabled:Boolean = false;
        public var rotationEnabled:Boolean = true;
        private var viewManager:TuningScreenViewManager;
        private var _diskChangeData:Object;
        private var videoSpinnerView:SpinnerView;
        public var savingAlert:SpinnerView;

        public function get selectedPart():Part {
            return _selectedPart;
        }

        public function set selectedPart(p:Part):void {
            _selectedPart = p;
        }

        public function get selectedPartSlot():PartSlot {
            return _selectedPartSlot;
        }

        public function set selectedPartSlot(value:PartSlot):void {
            _selectedPartSlot = value;
        }

        public function set carSplashImage(value:ImageLoaderWithStates):void {
            _carSplashImage = value;
            if (this.carView) {
                carView.setCarSplash(_carSplashImage);
            }
        }


        public function set languages(value:Array):void {

            _languages = value;
        }

        public function get languages():Array {

            return _languages;
        }


        public function set carView(value:CarView):void {
            _carView = value;
            if (_carView) {
                _carView.addEventListener(CarView.ROTATION_START, carViewRotationStarted);
                _carView.addEventListener(CarView.ROTATION_END, carViewRotationEnded);
            }
        }

        private function carViewRotationStarted(event:starling.events.Event):void {
            partsList.touchable = partsList.isEnabled = false;
        }

        private function carViewRotationEnded(event:starling.events.Event):void {
            partsList.touchable = partsList.isEnabled = true;
        }

        public function get selectedLanguage():Language {
            return _selectedLanguage;
        }

        public function set selectedLanguage(value:Language):void {
            this._selectedLanguage = value;
        }


        public function enterRowsState():void {
            Tracer.print('Tuning screen doesn\'t have ROW STATE')
        }

        public function homeButtonTriggeredHandler(event:starling.events.Event = null):void {

            this.dispatchEventWith(StackedScreen.HOME_SELECTED);
        }

        public function brandsScreenButtonTriggeredHandler(event:starling.events.Event = null):void {
            this.dispatchEventWith(StackedScreen.BRANDS_SELECTED);
        }


        public function get carView():CarView {

            return _carView
        }

        public var progressDefaultX:Number = 0;
        public var progressDeltaX:Number = 0;

        public function moveTo(delta:Number, target:*):void {
            var tween:Tween = new Tween(target, .35, Transitions.EASE_IN_OUT);
            tween.moveTo(delta, 0);
            tween.onComplete = completeMove;

            Starling.juggler.add(tween);

            if (delta == 0 && !infoView.isOpened) {
                colorMenuButton.visible = true;
                partsMenuButton.visible = true;
                settingsMenuButton.visible = true;
                backButton.visible = true;
                progressBar.x = progressDefaultX;

            } else {
                colorMenuButton.visible = false;
                partsMenuButton.visible = false;
                settingsMenuButton.visible = false;
                backButton.visible = false;
                progressBar.x = progressDeltaX;
                mainMenuView.visible = false;
            }
        }

        private function completeMove():void {
            mainMenuView.visible = backButton.visible;
        }

        private function set partsListVisible(value:Boolean):void {
            if (value && selectedPartSlot && selectedPartSlot.selectedPart && Car.DISKS_SLOTS.indexOf(selectedPartSlot.selectedPart.parentSlot.name) != -1) {
                var listenerRemoved:Boolean = false;
                if (partsList.hasEventListener(starling.events.Event.CHANGE, partsListChangeHandler)) {
                    partsList.removeEventListener(starling.events.Event.CHANGE, partsListChangeHandler);
                    listenerRemoved = true;
                }
                if (partsList.dataProvider.contains(selectedPartSlot.selectedPart)) {
                    partsList.selectedItem = selectedPartSlot.selectedPart;
                    partsList.scrollToDisplayIndex(partsList.selectedIndex);
                }
                if (listenerRemoved) {
                    partsList.addEventListener(starling.events.Event.CHANGE, partsListChangeHandler);
                }
            }
        }

        private function setSelectedPart(part:Part):void{
            var listenerRemoved:Boolean = false;
            if (partsList.hasEventListener(starling.events.Event.CHANGE, partsListChangeHandler)) {
                partsList.removeEventListener(starling.events.Event.CHANGE, partsListChangeHandler);
                listenerRemoved = true;
            }
            if (partsList.dataProvider.contains(part)) {
                partsList.selectedItem = part;
                partsList.scrollToDisplayIndex(partsList.selectedIndex);
            }
            if (listenerRemoved) {
                partsList.addEventListener(starling.events.Event.CHANGE, partsListChangeHandler);
            }
        }

        private function set brandCollection(value:ListCollection):void {
            _brandCollection = value;
            invalidate('brand_data_provider');
        }

        private function get brandCollection():ListCollection {
            return _brandCollection;
        }

        override public function invalidate(flag:String = INVALIDATION_FLAG_ALL):void {
            super.invalidate(flag);
        }


        public function setPartListTouchable(enabled:Boolean):void {
            if (partsList) {
                partsList.touchable = enabled;
            }

            /*if (diskBrandList) {
                diskBrandList.touchable = enabled;
            }*/
        }

        public function updatePartList():void {
            var i:int = 0;

            if (partsList && partsList.dataProvider) {

                partsList.dataProvider.refresh();


                partsList.invalidate();


            }
        }

        public function updateLists():void {
            var i:int = 0;
            if (possiblePartSlotsList && possiblePartSlotsList.dataProvider) {
                for (i = 0; i < possiblePartSlotsList.dataProvider.length; i++) {
                    possiblePartSlotsList.dataProvider.updateItemAt(i);
                }
                possiblePartSlotsList.invalidate();
            }

            if (partsList && partsList.dataProvider) {
                for (i = 0; i < partsList.dataProvider.length; i++) {
                    partsList.dataProvider.updateItemAt(i);
                }
                partsList.invalidate();
            }

            if (tuningColorsView) {
                tuningColorsView.updateWithLanguage(AlertView.language);
            }

            if (allPartsColorsView) {
                allPartsColorsView.updateWithLanguage(AlertView.language);
            }

            if (packagesView) {
                packagesView.updateWithLanguage(AlertView.language);
                packagesView.updateLists();
            }
        }

        private function set partsCollection(value:ListCollection):void {
            var item:Part;
            var i:int;
            var j:int = 0;

            _partsCollection = value;
            var facade:ApplicationFacade = ApplicationFacade.getInstance();
            if (car.packet && car.packet.lock) {
                packet = car.packet;
                if (packet && packet.type != Packet.TYPE_SUBCOMPONENT) {
                    for (i = 0; i < _partsCollection.length; i++) {
                        item = _partsCollection.getItemAt(i) as Part;
                        if (!item.isBack) {
                            item.lock = true;
                        }
                    }
                } else if (packet) {
                    for (i = 0; i < _partsCollection.length; i++) {
                        item = _partsCollection.getItemAt(i) as Part;
                        if (packet.slotName) {
                            for (j = 0; j < packet.slotName.length; j++) {
                                if (item.parentSlot.name == packet.slotName[j]) {
                                    if (packet.parts != null && packet.parts[packet.slotName[j]] && packet.parts[packet.slotName[j]].length > 0) {
                                        if (packet.parts[packet.slotName[j]].indexOf(item.name) >= 0) {
                                            if (packet.funcName == 'camber') {
                                                item.camberLock = true;
                                                item.lock = false;
                                            } else {
                                                item.lock = true;
                                                item.camberLock = false;
                                            }
                                        } else {
                                            item.lock = false;
                                        }
                                    } else {
                                        if (packet.funcName == 'camber') {
                                            item.camberLock = true;
                                            item.lock = false;
                                        } else {
                                            item.lock = true;
                                            item.camberLock = false;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            } else if (car.packet) {
                packet = car.packet;
                for (i = 0; i < _partsCollection.length; i++) {
                    item = _partsCollection.getItemAt(i) as Part;
                    if (packet && packet.slotName) {
                        for (j = 0; j < packet.slotName.length; j++) {
                            if (item.parentSlot.name == packet.slotName[j]) {
                                if (packet.parts != null && packet.parts[packet.slotName[j]] && packet.parts[packet.slotName[j]].length > 0) {
                                    if (packet.parts[packet.slotName[j]].indexOf(item.name) >= 0) {
                                        if (packet.funcName == 'camber') {
                                            item.camberLock = false;
                                            item.lock = false;
                                        } else {
                                            item.lock = false;
                                            item.camberLock = false;
                                        }
                                    } else {
                                        item.lock = false;
                                    }
                                } else {
                                    item.lock = false;
                                }
                            }
                        }
                    }
                }
            }

            var packets:Array = car.packets;
            if (packets) {
                var packet:Packet
                var z:int = 0;
                for (i = 0; i < packets.length; i++) {

                    packet = packets[i];

                    if (packet && packet.type == Packet.TYPE_SUBCOMPONENT && packet.lock && packet.funcName == 'standart') {


                        for (j = 0; j < _partsCollection.length; j++) {

                            item = _partsCollection.getItemAt(j) as Part;

                            if (packet.slotName) {
                                for (z = 0; z < packet.slotName.length; z++) {

                                    if (item.parentSlot.name != packet.slotName[z]) {

                                        continue;
                                    }

                                    if (packet.parts && packet.parts[packet.slotName[z]] && packet.parts[packet.slotName[z]].length > 0) {

                                        if (packet.parts[packet.slotName[z]].indexOf(item.name) >= 0) {
                                            item.lock = true;
                                            item.lockPacket = packet;
                                        } else {

                                            /*if(car && car.packet && car.packet.type==Packet.TYPE_COMPONENTS && !car.packet.lock){
												item.lock=false;
											}*/

                                        }
                                    } else {

                                        item.lock = true;
                                        item.lockPacket = packet;
                                    }

                                }

                            }

                        }

                    }
                }


                for (i = 0; i < packets.length; i++) {

                    packet = packets[i];

                    if (packet && packet.type == Packet.TYPE_SUBCOMPONENT && !packet.lock && packet.funcName == 'standart') {


                        for (j = 0; j < _partsCollection.length; j++) {

                            item = _partsCollection.getItemAt(j) as Part;


                            if (packet.slotName) {
                                for (z = 0; z < packet.slotName.length; z++) {

                                    if (item.parentSlot.name != packet.slotName[z]) {

                                        continue;
                                    }

                                    if (packet.parts && packet.parts[packet.slotName[z]] && packet.parts[packet.slotName[z]].length > 0) {

                                        if (packet.parts[packet.slotName[z]].indexOf(item.name) >= 0) {
                                            item.lock = false;
                                            item.lockPacket = null;
                                        }
                                    } else {

                                        item.lock = false;

                                        item.lockPacket = null;
                                    }
                                }
                            }

                        }

                    }
                }

            }


            invalidate('parts_data_provider');
        }

        public function set packages(value:Array):void {

            _packages = value;

            _filteredPackages = Packet.filteringPackages(_packages);

            if (packagesView)
                packagesView.dataProvider = _filteredPackages;
        }

        private function get partsCollection():ListCollection {

            return _partsCollection;
        }

        private var normalRenderer:Function = function ():IListItemRenderer {
            var renderer:PartItemRenderer = new PartItemRenderer();
            //renderer.setChopper(car.isChopper);
            if (car.isChopper) {
                renderer.setSize(130, 50);
            } else {
                renderer.setSize(100, 50);
            }

            return renderer;
        }
        // Расширенный режим отображения
        private var diskRenderer:Function = function ():IListItemRenderer {
            var renderer:PartItemRenderer = new PartDiskItemRenderer();
            //renderer.setChopper(car.isChopper);

            if (car.isChopper) {
                //renderer.setSize(150, 50);
            } else {
                //renderer.setSize(PART_DISK_WIDTH, PART_DISK_HEIGHT);
            }


            return renderer;
        }


        private var _carLoaded:Boolean = false;

        public function TuningScreen() {
            super('TuningScreen');
            frontWheelModified = rearWheelModified = false;
            showOpenHandleFlag = true;
            carLoaded = false;
            name = 'TuningScreen';

            partSlotNameWidthsMap = {};
            maxAlphaForHiddenView = new Dictionary(true);
            colorsViewEnabled = true;
            this.backButtonHandler = function ():void {

            }
        }

        public function clearSound():void{
            powerOn = false;
            powerOnButton.isSelected = false;
            gasButton.visible = false;

            if(startAudioChannel){
                startAudioChannel.stop();
            }

            if(gasAudioChannel){
                gasAudioChannel.stop();
            }

            if(downAudioChannel){
                downAudioChannel.stop();
            }
            destroyRevSound();

            startAudioChannel = null;
            gasAudioChannel = null;
            downAudioChannel = null;
        }

        override public function dispose():void {
            AdsManager.$.bannerEnabled = false;
            clearSound();
            if (infoView) {
                infoView.clear();
                infoView.removeFromParent(true);
            }


            if (carView) {
                carView.removeEventListener(CarView.PART_SLOT_SELECTED, carViewPartSlotSelectedHandler);

                if (Constants.SHOW_FRAME_INDEX_IN_TUNING_TITLE) {
                    carView.removeEventListener(CarView.SHOW_FRAME, carViewOnShowFrame);
                }

                carView.removeEventListener(BackgroundContainer.EVENT_CAMERA_ERROR, onCameraError);
            }


            if (partsList) {
                partsList.removeEventListener(starling.events.Event.CHANGE, partsListChangeHandler);
                partsList.removeEventListener(PartDiskItemRenderer.EVENT_CHANGE_PART_DATA, partsListChangeSelectedPartDataHandler);
                partsList.removeEventListener('itemSelectedAgain', partsListItemSelectedAgain);
                partsList.removeEventListener(PartDiskItemRenderer.EVENT_OPEN_DISK_INFO_POP_UP, partsListOpenDiskInfoPopUpHandler);
                partsList.removeEventListener(PartDiskItemRenderer.EVENT_OPEN_DISK_SETTINGS_VIEW, partsListOpenDiskSettingsViewHandler);
                partsList.removeEventListener(PartItemRenderer.EVENT_OPEN_INFO_POP_UP, partsListOpenPopupInfoHandler);
                partsList.removeEventListener(starling.events.Event.SCROLL, partsScrollEventHandler);
                if (partsList.dataProvider) {
                    partsList.dataProvider.removeAll();
                    partsList.dataProvider = null;
                }

                partsList.removeFromParent(true);
                partsList = null;
            }

            if (shareButton) {
                shareButton.removeFromParent(true);
                shareButton = null;
            }

            if (saveButton) {
                saveButton.removeFromParent(true);
                saveButton = null;
            }

            if (cameraButton) {
                cameraButton.removeFromParent(true);
                cameraButton = null;
            }

            if (acceptFromCameraButton) {
                acceptFromCameraButton.removeFromParent(true);
                acceptFromCameraButton = null;
            }

            if (cancelFromCameraButton) {
                cancelFromCameraButton.removeFromParent(true);
                cancelFromCameraButton = null;
            }

            if (colorMenuButton) {
                colorMenuButton.removeFromParent(true);
                colorMenuButton = null;
            }

            if (partsMenuButton) {
                partsMenuButton.removeFromParent(true);
                partsMenuButton = null;
            }

            if (settingsMenuButton) {
                settingsMenuButton.removeFromParent(true);
                settingsMenuButton = null;
            }


            if (possiblePartSlotsList) {
                possiblePartSlotsList.removeEventListener(starling.events.Event.CHANGE, possiblePartSlotsListChangeHandler);
                possiblePartSlotsList.removeEventListener(starling.events.Event.SCROLL, possiblePartSlotsListScrollHandler);
                if (possiblePartSlotsList.dataProvider){
                    possiblePartSlotsList.dataProvider.removeAll();
                }
                possiblePartSlotsList.removeFromParent(true);
                possiblePartSlotsList = null;
            }

            if (diskBrandList) {
                diskBrandList.removeEventListener(starling.events.Event.CHANGE, brandsListItemChangeHandler);
                if (diskBrandList.dataProvider){
                    diskBrandList.dataProvider.removeAll();
                }
                diskBrandList.removeFromParent(true);
                diskBrandList = null;
            }

            if (packagesView) {
                packagesView.removeEventListener(PackagesView.BUY_NOW_PACKET, onBuyNowPacket);
                packagesView.removeEventListener(PackagesView.VIEW_CAR_360, viewCar360);
                packagesView.removeEventListener(starling.events.Event.OPEN, packagesViewOpen);
                packagesView.removeEventListener(starling.events.Event.CLOSE, onClosePackagesView);
                packagesView.removeEventListener(PackagesView.SHOW_CONFIRM_POP_UP, displayRewardConfirmPopUp);
                packagesView.removeEventListener(PackagesView.SHOW_NO_COIN_POP_UP, displayRewardNoCoinPopUp);
                packagesView.removeFromParent(true);
                packagesView = null;
            }


            if (_car) {
                if(_car.packet && _car.packet.tempOpened){
                    _car.packet.lock = true;
                    _car.packet.tempOpened = false;
                }
                _car.dispose();
            }
            _car = null;

            if (loadedCar) {
                loadedCar.dispose();
            }

            loadedCar = null;
            tunning = null;
            selectedPart = null;
            selectedPartSlot = null;

            if (partSlotNameWidthsMap) {
                for (var id:String in partSlotNameWidthsMap) {
                    delete partSlotNameWidthsMap[id];
                    partSlotNameWidthsMap[id] = null;
                }
            }

            partSlotNameWidthsMap = null;


            if (tuningColorsView) {
                tuningColorsView.removeFromParent(true);
                tuningColorsView = null;
            }


            if (partDiskSettingView) {
                partDiskSettingView.partSlot = null;
                partDiskSettingView.removeEventListener(starling.events.Event.CLOSE, closePartDiskSettingsView);
                partDiskSettingView.removeFromParent(true);
                partDiskSettingView = null;
            }

            if (progressBar) {
                progressBar.removeFromParent(true);
                progressBar = null;
            }

            if (maxAlphaForHiddenView) {
                maxAlphaForHiddenView = null;
            }

            language = null;
            if (packagesView) {
                packagesView.dataProvider = null;
                packagesView.removeFromParent(true);
                packagesView = null;
            }

            bodyPartSlot = null;
            packet = null;
            if (blockerView) {
                blockerView.removeFromParent(true);
                blockerView = null;
            }

            _partsCollection = null;
            _brandCollection = null;
            _packages = null;
            assetManager = null;
            _carView.removeFromParent(true);
            _carView = null;
            _possiblePartSlots = null;

            ZipManager.getInctance().reset();
            if (startAudio){
                startAudio.removeEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
            }
            if (gasAudio) {
                gasAudio.removeEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
            }

            if (downAudio) {
                downAudio.removeEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
            }

            if (startAudioChannel) {
                startAudioChannel.removeEventListener(flash.events.Event.SOUND_COMPLETE, onPowerSoundComplete);
            }
            removeEventListener(starling.events.Event.ENTER_FRAME, onEnterFrame);
            if (powerOnButton) {
                powerOnButton.removeFromParent(true);
                powerOnButton = null;
            }

            if (gasButton) {
                gasButton.removeFromParent(true);
                gasButton = null;
            }

            if(fastModeButton){
                fastModeButton.removeFromParent(true);
                fastModeButton = null;
            }

            if(fastModeHelpButton){
                fastModeHelpButton.removeFromParent(true);
                fastModeHelpButton = null;
            }

            super.dispose();

            MemoryTracker.gcAndCheck('TuningScreen.dispose()');
        }

        private function displayRewardConfirmPopUp(event:starling.events.Event):void {
            var packet:Packet = event.data.packet;
            AdsManager.$.bannerHide();
            dispatchEventWith(PackagesView.SHOW_CONFIRM_POP_UP, false, {packet: packet});
        }

        private function displayRewardNoCoinPopUp(event:starling.events.Event):void {
            AdsManager.$.bannerHide();
            dispatchEventWith(PackagesView.SHOW_NO_COIN_POP_UP);
        }

        public function set car(value:Car):void {
            _car = value;
            if (_car && !_car.packet) {
                var facade:ApplicationFacade = ApplicationFacade.getInstance();
                var packagesProxy:PackagesProxy = facade.retrieveProxy(PackagesProxy.NAME) as PackagesProxy;
                var packs:Array = packagesProxy.findPackagesByCarId(_car.id);
                for (var k:int = 0; k < packs.length; k++) {
                    _car.addPacket(packs[k]);
                }
            }
            invalidate('car');
            validateCarDrawing();
        }

        public function get car():Car {
            return _car;
        }

        public function set selectedPartSlotIndex(value:int):void {
            possiblePartSlotsList.selectedIndex = value;
        }

        private var _topButtonPadding:Number = 0;

        public function set topButtonPadding(value:Number):void {
            if (value != _topButtonPadding) {
                _topButtonPadding = value;
                invalidate('ads');
            }
        }

        public function get topButtonPadding():Number {
            return _topButtonPadding;
        }


        public var loginOpen:Boolean = false;

        override public function ShowDone():void {
            ImageLoaderWithStates.disposeWhenRemovedFromStage = false;
            if (loginOpen) {
                this.carView.visible = true;
                this.invalidate();
            }

            if (mainMenuView) {
                mainMenuView.updateWithLanguage(AlertView.language);
            }
        }

        public function set possiblePartSlots(value:Array):void {
            try {
                var newBrandDisk:Array = [];
                var vinylSlots:Array = [];
                for each(var partSlot:PartSlot in value) {

                    if (partSlot.name == 'masks') {
                        carView.partSlotMask = partSlot;
                    }

                    if (partSlot.name && PartSlot.VINYLS.indexOf(partSlot.name) != -1) {
                        vinylSlots.push(partSlot);
                    }
                    if (vinylSlots.indexOf(partSlot) != -1 && Car.TIRE_VINYL_PARTS.indexOf(partSlot.name) != -1 && partSlot.name != 'tire_vinyl_front_left') {
                        vinylSlots.pop();
                    }

                    if (partSlot.name == 'disks' || partSlot.name == 'disks_front' || partSlot.name == 'disk_front_left') {

                        var index:int = 0;
                        for each(var diskBrand:DiskBrand in Disk.brandList.data) {


                            var diskBrandEmpty:Boolean = true;

                            if (index == 0) {

                                diskBrandEmpty = false;
                            }


                            var camberEx:Boolean = false;


                            if (diskBrand.id == 'factory') {

                                var t:int = 9;
                                t = 1;

                            }


                            for each(var part:Part in partSlot.possibleParts) {


                                if (diskBrand.id == 'factory' && part.manufacturer == null) {

                                    diskBrand.imageUrl = car.brandImageUrl;
                                    diskBrandEmpty = false;


                                    if (part.existCamber && part.name != null && part.name.indexOf('factory') >= 0) {

                                        camberEx = true;
                                        diskBrand.existCamber = true;

                                    }
                                } else if (part.manufacturer == diskBrand.id) {


                                    if (part.existCamber) {

                                        camberEx = true;
                                        diskBrand.existCamber = true;

                                    }


                                    diskBrandEmpty = false;
                                }

                            }


                            if (diskBrand.name == null) {

                                diskBrandEmpty = true;
                            }

                            if (!diskBrandEmpty) {


                                diskBrand.existCamber = camberEx;

                                if (!Device.isIos() && Constants.DISABLED_CAMBER_FOR_ANDROID_TRUCK_SUV) {
                                    if (car.type == 'Truck' || car.type == 'Suv') {

                                        diskBrand.existCamber = false;
                                    }
                                }
                                newBrandDisk.push(diskBrand);
                            } else {

                            }

                            index++;
                        }
                    }


                    if (partSlot.name == 'body') {

                        if (partSlot.selectedColor.value != 0xffffff)
                            car.bodyPartColor.value = partSlot.selectedColor.value;

                        bodyPartSlot = partSlot;
                        carView.bodyPartSlot = bodyPartSlot;
                    }


                }


                function orderSortKey(a:Object, b:Object):int {
                    var a_i:int = int(a.sortKey);
                    var b_i:int = int(b.sortKey);

                    if (a_i < b_i) return -1;
                    if (a_i > b_i) return 1;

                    return 0;
                }

                if (newBrandDisk.length > 2) {

                    var firstBrandDisk:DiskBrand = newBrandDisk[2];

                    if (firstBrandDisk.sortKey >= 0) {

                        for each(var dB:DiskBrand in newBrandDisk) {

                            if (!dB.isAllBrands && dB.sortKey < 0) dB.sortKey = 1;
                        }


                        newBrandDisk.sort(orderSortKey);
                    }
                }

                brandCollection = new ListCollection(newBrandDisk);
                var newPossiblePartSlots:Array = [];
                for each(partSlot in value) {
                    if (Car.WINDOW_SLOTS.indexOf(partSlot.name) != -1) {
                        partSlot.hideFromMenu = true;
                    }
                    if (partSlot.hideFromMenu) {
                        trace('PartSlot hide')
                        continue;
                    }

                    if (partSlot.name == 'right_doors' || partSlot.name == 'right_windows')
                        continue;
                    if (car.isChopper) {
                        if (partSlot.name != 'masks' && partSlot.name != 'frame_back' && partSlot.name != 'body' && PartSlot.VINYLS.indexOf(partSlot.name) == -1) {
                            newPossiblePartSlots.push(partSlot);
                        }
                    } else if (car.suvTires) { // combine
                        if (partSlot.name != 'masks' && CarView.NOT_SELECTABLE_PART_SLOTS.indexOf(partSlot.name) < 0 && PartSlot.VINYLS.indexOf(partSlot.name) == -1) {

                            /**
                             * Cause we combine all the tires as one tire, 4 disks to 2-2 front and rear  and 4 brakes to 2-2 front and rear
                             * we show only one of the in part list menu
                             */
                            if (PartSlot.PART_SHADOWS.indexOf(partSlot.name) != -1 || (partSlot.name == 'disk_front_right' || partSlot.name == 'disk_rear_right') ||
                                    (Car.TIRES_SLOTS.indexOf(partSlot.name) >= 0 && partSlot.name != 'tire_front_left') ||
                                    (partSlot.name == 'brake_front_right' || partSlot.name == 'brake_rear_right') ||
                                    partSlot.name == 'tire_vinyl_front_right' || partSlot.name == 'tire_vinyl_rear_right'
                            ) {
                                Tracer.print('part has skiped: ' + partSlot.name);
                            } else {
                                newPossiblePartSlots.push(partSlot);

                            }
                        }
                    } else {
                        if (partSlot.name != 'masks' && CarView.NOT_SELECTABLE_PART_SLOTS.indexOf(partSlot.name) < 0 && PartSlot.VINYLS.indexOf(partSlot.name) == -1) {
                            newPossiblePartSlots.push(partSlot);
                        }
                    }


                }

                _possiblePartSlots = newPossiblePartSlots;
                _vinylCollection = new ListCollection(vinylSlots);

                if (vinylSlots.length) {
                    var banner:PartSlot = new PartSlot(car);
                    banner.sortKey = 0;
                    banner.name = "banner";
                    vinylSlots.push(banner);


                    vinylSlots.sort(orderSortKey);
                    var vinylLabel:PartSlot = new PartSlot(car);
                    vinylLabel.displayName = 'Vinyls';
                    vinylLabel.name = PartSlot.VINYL;
                    if (PartSlot.VINYLS[PartSlot.VINYLS.length - 1].hasOwnProperty('sortKey')) {
                        vinylLabel.sortKey = PartSlot.VINYLS[PartSlot.VINYLS.length - 1]['sortKey'];
                    }
                    _possiblePartSlots.push(vinylLabel);
                }
                if (_possiblePartSlots) {
                    _possiblePartSlots.sort(orderSortKey);
                }

                arrowImage.visible = _possiblePartSlots.length > 0;
                for (var i:int = _possiblePartSlots.length - 1; i >= 0; i--) {
                    var pSlot:PartSlot = _possiblePartSlots[i] as PartSlot;
                    if (pSlot && (pSlot.name == 'mirror_right' || pSlot.name == 'tire_vinyl_rear_left')) {
                        _possiblePartSlots.removeAt(i);
                    }
                }
                possiblePartSlotsList.dataProvider = new ListCollection(_possiblePartSlots);
                Starling.juggler.delayCall(function():void{
                    componentSelector.dataProvider = possiblePartSlotsList.dataProvider;
                }, 0.1);

            } catch (e:Error) {

                Tracer.print('error:', e.getStackTrace());
            }
        }

        public function get possiblePartSlots():Array {
            return _possiblePartSlots;
        }

        public function dispatchExternalEvent(msg:String):void {
            if (mainMenuView) {
                mainMenuView.hide();
            }

            dispatchEventWith(msg);
        }

        public function updateWithLanguage(_language:Language):void {
            language = _language;

            if (mainMenuView) {
                mainMenuView.updateWithLanguage(language);
            }

            if (tuningColorsView) {
                tuningColorsView.updateWithLanguage(language);
            }

            if (allPartsColorsView) {
                allPartsColorsView.updateWithLanguage(language);
            }


            if(componentSelector){
                componentSelector.language = language;
            }
        }

        public function copyToBitmap():BitmapData {
            var scaleFactor:Number = assetManager.scaleFactor;
            var bitmapData2:BitmapData = new BitmapData(Starling.context.backBufferWidth, Starling.context.backBufferHeight, false);
            this.drawToBitmapData(bitmapData2);
            return bitmapData2;
        }


        override protected function initialize():void {
            if (!carView) {
                carView = new CarView();
                carView.assetManager = assetManager;
                carView.visible = false;
            }

            carView.canTouchParts = true;
            carView.addEventListener(CarView.PART_SLOT_SELECTED, carViewPartSlotSelectedHandler);
            carView.addEventListener(BackgroundContainer.EVENT_CAMERA_ERROR, onCameraError);

            if (Constants.SHOW_FRAME_INDEX_IN_TUNING_TITLE) {
                carView.addEventListener(CarView.SHOW_FRAME, carViewOnShowFrame);
            }

            carView.progressHandler = progressHandler;

            addChild(carView);

            super.initialize();

            handleBarImage.visible = false;
            frameImage.visible = false;
            frameImage.touchable = false;

            saveButton = createUICircleButton("a_save_button");
            saveButton.addEventListener(starling.events.Event.TRIGGERED, saveButtonTriggeredHandler);
            addChild(saveButton);


            fastModeButton = new ToggleButton();
            fastModeButton.defaultSkin = new Image(ButtonFactory.assetManager.getTexture('a_car_fast_mode'));
            fastModeButton.downSkin = new Image(ButtonFactory.assetManager.getTexture('a_car_fast_mode'));
            fastModeButton.downSkin.alpha = 0.4;
            fastModeButton.defaultSelectedSkin = new Image(ButtonFactory.assetManager.getTexture('a_car_fast_mode_selected'));
            fastModeButton.selectedDownSkin = new Image(ButtonFactory.assetManager.getTexture('a_car_fast_mode_selected'));
            Image(fastModeButton.selectedDownSkin).color = 0xFF0000;
            fastModeButton.selectedDownSkin.alpha = 0.4;
            fastModeButton.width = fastModeButton.height = 35;
            fastModeButton.addEventListener(starling.events.Event.CHANGE, fastModeButtonChangedHandler);
            fastModeButton.visible = false;
            Image(fastModeButton.defaultSkin).color = 0x000000;
            Image(fastModeButton.downSkin).color = 0x000000;
            addChild(fastModeButton);

            fastModeHelpButton = new Button();
            fastModeHelpButton.defaultSkin = new Image(ButtonFactory.assetManager.getTexture('a_fast_mode_help'));
            fastModeHelpButton.width = fastModeHelpButton.height = 20;
            fastModeHelpButton.scaleWhenDown = 0.9;
            fastModeHelpButton.addEventListener(starling.events.Event.TRIGGERED, fastModeHelpButtonChangedHandler);
            fastModeHelpButton.visible = false;
            addChild(fastModeHelpButton);


            // Camera and screenshot buttons
            cameraButton = createUICircleButton('a_photo_accept_button');
            cameraButton.addEventListener(starling.events.Event.TRIGGERED, cameraButtonTriggeredHandler);
            addChild(cameraButton);
            //
            var newIcon:Image;
            newIcon = new Image(ButtonFactory.assetManager.getTexture("new_icon"));
            newIcon.width = 25;
            newIcon.height = 25;
            newIcon.name = "new_icon";
            newIcon.visible = false;
            cameraButton.addChild(newIcon);
            //
            arButton = createUICircleButton('a_screenshot_ar_con');
            arButton.addEventListener(starling.events.Event.TRIGGERED, arButtonTriggeredHandler);
            addChild(arButton);

            acceptFromCameraButton = createUICircleButton('a_save_button');
            acceptFromCameraButton.addEventListener(starling.events.Event.TRIGGERED, acceptScreenshotButtonTriggeredHandler);
            addChild(acceptFromCameraButton);

            cancelFromCameraButton = createUICircleButton('a_photo_cancel_button');
            cancelFromCameraButton.addEventListener(starling.events.Event.TRIGGERED, cancelScreenshotButtonTriggeredHandler);
            addChild(cancelFromCameraButton);

            currentScreenshotButton = createUICircleButton('a_camera_screenshot');
            currentScreenshotButton.addEventListener(starling.events.Event.TRIGGERED, currentScreenshotButtonTriggeredHandler);
            addChild(currentScreenshotButton);

            cameraScreenshotButton = createUICircleButton('a_camera_screenshot');
            cameraScreenshotButton.addEventListener(starling.events.Event.TRIGGERED, currentScreenshotButtonTriggeredHandler);
            addChild(cameraScreenshotButton);

            saveVideoPreviewButton1 = createUICircleButton('video_icon_1');
            saveVideoPreviewButton1.addEventListener(starling.events.Event.TRIGGERED, saveVideoButtonTriggeredHandler);
            newIcon = new Image(ButtonFactory.assetManager.getTexture("new_icon"));
            newIcon.width = 25;
            newIcon.height = 25;
            newIcon.name = "new_icon";
            saveVideoPreviewButton1.addChild(newIcon);
            addChild(saveVideoPreviewButton1);
            saveVideoPreviewButton1.visible = false;

            saveVideoPreviewButton2 = createUICircleButton('video_icon_2');
            saveVideoPreviewButton2.addEventListener(starling.events.Event.TRIGGERED, saveVideoButtonTriggeredHandler);
            newIcon = new Image(ButtonFactory.assetManager.getTexture("new_icon"));
            newIcon.width = 25;
            newIcon.height = 25;
            newIcon.name = "new_icon";
            saveVideoPreviewButton2.addChild(newIcon);
            addChild(saveVideoPreviewButton2);
            saveVideoPreviewButton2.visible = false;


            // end of camera and screenshot buttons

            powerOnButton = ButtonFactory.soundButtonEngine('engine_active', 'engine_inactive');
            powerOnButton.width = 35;
            powerOnButton.height = 35;
            powerOnButton.visible = false;
            powerOnButton.isSelected = false;
            powerOnButton.addEventListener(starling.events.Event.TRIGGERED, powerOnButtonButtonTriggeredHandler);
            addChild(powerOnButton);

            gasButton = ButtonFactory.soundButtonGaz('gas');
            gasButton.width = 35;
            gasButton.height = 35;
            gasButton.visible = false;
            gasButton.addEventListener(TouchEvent.TOUCH, onGazButton);
            addChild(gasButton);

            revSoundButton = ButtonFactory.soundButtonEngine('engine_sound_rev_stop', 'engine_sound_rev_start');
            revSoundButton.width = 35;
            revSoundButton.height = 35;
            revSoundButton.visible = false;
            revSoundButton.addEventListener(starling.events.Event.CHANGE, revSoundButtonHandler);
            addChild(revSoundButton);

            colorMenuButton = createUICircleButton('a_paint_button');
            colorMenuButton.addEventListener(starling.events.Event.TRIGGERED, colorMenuButtonTriggeredHandler);
            addChild(colorMenuButton);

            partsMenuButton = createUICircleButton('a_partslot_button');
            partsMenuButton.addEventListener(starling.events.Event.TRIGGERED, partsMenuButtonTriggeredHandler);
            addChild(partsMenuButton);

            settingsMenuButton = createUICircleButton('a_car_settings_button');
            settingsMenuButton.addEventListener(starling.events.Event.TRIGGERED, carSettingsButtonTriggeredHandler);
            addChild(settingsMenuButton);

            allPartsColorsView = new TuningColorsView();
            allPartsColorsView.updateWithLanguage(language);
            allPartsColorsView.assetManager = assetManager;
            allPartsColorsView.initialHeight = menuBackground.height;
            allPartsColorsView.addEventListener(TuningColorsView.ALL_PART_SLOT_SELECTED, onAllPartSlotSelectedHandler);
            allPartsColorsView.visible = false;
            addChild(allPartsColorsView);

            tuningSettingsPanel = new TuningSettingsPanel();
            tuningSettingsPanel.updateWithLanguage(language);
            tuningSettingsPanel.width = actualWidth;
            tuningSettingsPanel.height = menuBackground.height;
            tuningSettingsPanel.visible = false;
            tuningSettingsPanel.addEventListener(TuningSettingsPanel.OPEN_BACKGROUND, onOpenBackgroundHandler);
            tuningSettingsPanel.addEventListener(EffectMenu.TOGGLE_STATE, onToggleState);
            tuningSettingsPanel.addEventListener(EffectMenu.TOGGLE_STATE_ERROR, onErrorToggleState);
            tuningSettingsPanel.validate();
            addChild(tuningSettingsPanel);

            suspensionControl = tuningSettingsPanel.suspensionControl;
            tintControl = tuningSettingsPanel.tintControl;
            tintControl.addEventListener(SliderControl.VALUE_CHANGED, tintChangedHandler);


            backgroundMenuView = new TuningBackgroundMenuView();
            backgroundMenuView.updateWithLanguage(language);
            backgroundMenuView.close();
            backgroundMenuView.addEventListener(TuningBackgroundMenuView.ON_CLOSE, onCloseBackground);
            backgroundMenuView.addEventListener(TuningBackgroundMenuView.BACKGROUND_SELECTED, onBackgroundSelected);
            backgroundMenuView.addEventListener(TuningBackgroundMenuView.BACKGROUND_UNSELECTED, onBackgroundUnselected);
            backgroundMenuView.addEventListener(TuningBackgroundMenuView.BACKGROUND_UPLOAD, onBackgroundUpload);
            backgroundMenuView.addEventListener(TuningBackgroundMenuView.BACKGROUND_UPLOAD_COMPLETE, onBackgroundUploadComplete);
            backgroundMenuView.addEventListener(TuningBackgroundMenuView.SNAPSHOT_CAMERA_SELECTED, onSnapShotBackgroundUpload);
            backgroundMenuView.addEventListener(TuningBackgroundMenuView.FILL_COLOR_SELECTED, onFillColorSelected);
            backgroundMenuView.addEventListener(TuningBackgroundMenuView.BACKGROUND_DELETE, onBackgroundDelete);
            addChild(backgroundMenuView);


            var possiblePartSlotsLayout:HorizontalLayout = new HorizontalLayout();
            possiblePartSlotsLayout.manageVisibility = true; //tt
            possiblePartSlotsLayout.useVirtualLayout = false;
            possiblePartSlotsLayout.gap = 0;
            possiblePartSlotsLayout.paddingLeft = -4;
            possiblePartSlotsLayout.paddingRight = 11;

            possiblePartSlotsList = new List();
            possiblePartSlotsList.visible = true;
            possiblePartSlotsList.snapScrollPositionsToPixels = true;
            possiblePartSlotsList.hasElasticEdges = false;
            possiblePartSlotsList.clipContent = false;
            possiblePartSlotsList.layout = possiblePartSlotsLayout;
            possiblePartSlotsList.addEventListener(starling.events.Event.CHANGE, possiblePartSlotsListChangeHandler);
            possiblePartSlotsList.addEventListener(starling.events.Event.SCROLL, possiblePartSlotsListScrollHandler);
            possiblePartSlotsList.itemRendererFactory = function ():IListItemRenderer {
                var renderer:PartSlotItemRenderer = new PartSlotItemRenderer();
                renderer.language = language;
                renderer.partSlotNameWidthsMap = partSlotNameWidthsMap;
                return renderer;
            };
            addChild(possiblePartSlotsList);

            var partsListLayout:HorizontalLayout = new HorizontalLayout();
            partsListLayout.gap = 0;
            partsListLayout.paddingLeft = 0;
            partsListLayout.paddingRight = 5;
            partsListLayout.manageVisibility = true;
            partsListLayout.useVirtualLayout = true;
            partsListLayout.hasVariableItemDimensions = true;
            //	partsListLayout.c


            partsList = new List();
            partsList.addEventListener('itemSelectedAgain', partsListItemSelectedAgain);
            partsList.clipContent = true;
            partsList.hasElasticEdges = false;
            partsList.visible = false;
            partsList.layout = partsListLayout;
            partsList.paddingLeft = 0;
            partsList.paddingRight = 0;
            partsList.addEventListener(starling.events.Event.CHANGE, partsListChangeHandler);
            partsList.addEventListener(PartDiskItemRenderer.EVENT_CHANGE_PART_DATA, partsListChangeSelectedPartDataHandler);
            partsList.addEventListener(PartDiskItemRenderer.EVENT_OPEN_DISK_INFO_POP_UP, partsListOpenDiskInfoPopUpHandler);
            partsList.addEventListener(PartDiskItemRenderer.EVENT_OPEN_DISK_SETTINGS_VIEW, partsListOpenDiskSettingsViewHandler);
            partsList.addEventListener(PartItemRenderer.EVENT_OPEN_INFO_POP_UP, partsListOpenPopupInfoHandler);
            partsList.addEventListener(starling.events.Event.SCROLL, partsScrollEventHandler);
            partsList.itemRendererFactory = normalRenderer;
            addChild(partsList);

            // Контейнер для списка брендов дисков
            var diskBrandListLayout:HorizontalLayout = new HorizontalLayout();
            diskBrandListLayout.gap = 0;
            diskBrandListLayout.paddingLeft = 0;

            // Создаем список брендов дисков и привязываем его к источнику
            diskBrandList = new List();
            diskBrandList.addEventListener(starling.events.Event.CHANGE, brandsListItemChangeHandler);
            diskBrandList.layout = diskBrandListLayout;
            diskBrandList.clipContent = true;
            diskBrandList.itemRendererFactory = function ():IListItemRenderer {
                var renderer:BrandPartItemRenderer = new BrandPartItemRenderer();
                renderer.setSize(120, PART_HEIGHT);
                return renderer;
            }

            addChild(diskBrandList);
            arrowImage.visible = false;
            partsList.itemRendererFactory = function ():IListItemRenderer {
                var renderer:PartItemRenderer = new PartItemRenderer();
                renderer.setSize(PART_WIDTH, PART_HEIGHT);
                return renderer;
            };
            partsList.height = menuHeight;
            addChild(partsList);
            componentSelector = new ComponentSelector();
            if(language){
                componentSelector.language = language;
            }
            componentSelector.addEventListener(starling.events.Event.CHANGE, componentSelectorChangeHandler);
            componentSelector.addEventListener(ComponentSelector.SELECTOR_TRIGGERED, componentSelectorTriggeredHandler);
            componentSelector.width = PART_LIST_MARGIN;
            componentSelector.height = menuHeight;
            addChild(componentSelector);
            componentSelector.visible = false;

            menuBackButton = new MenuBackButton();
            menuBackButton.addEventListener(starling.events.Event.TRIGGERED, onMenuBackHandler);
            menuBackButton.visible = false;
            menuBackButton.setSize(50, menuHeight);
            addChild(menuBackButton);

            arrowImage.visible = true;



            progressBar = new TuningProgressBar();
            progressBar.assetManager = assetManager;
            progressBar.touchable = false;
            progressBar.visible = false;
            addChild(progressBar);

            progressFlatBar = new TuningFlatProgressBar();
            progressFlatBar.height = 29;
            progressFlatBar.assetManager = assetManager;
            progressFlatBar.validate();
            addChild(progressFlatBar);

            shareButton = createUICircleButton('a_arrow_right_button');
            shareButton.addEventListener(starling.events.Event.TRIGGERED, shareButtonTriggeredHandler);
            addChild(shareButton);
            initializeBackButton();
            setChildIndex(arrowImage, numChildren);

            notificationView = new NotificationView();
            notificationView.visible = false;
            addChild(notificationView);
            menuHidden();

            packagesView = new PackagesView();
            PackagesView.calacItemCount = false;
            packagesView.setSize(PackagesView.PACKAGE_ITEM_WIDTH, actualHeight);
            packagesView.hide();
            
            packagesView.parentActualWith = actualWidth;
            packagesView.parentActualHeight = actualHeight;
            packagesView.addEventListener(PackagesView.BUY_NOW_PACKET, onBuyNowPacket);
            packagesView.addEventListener(PackagesView.VIEW_CAR_360, viewCar360);
            packagesView.addEventListener(starling.events.Event.OPEN, packagesViewOpen);
            packagesView.addEventListener(starling.events.Event.CLOSE, onClosePackagesView);
            packagesView.addEventListener(PackagesView.SHOW_CONFIRM_POP_UP, displayRewardConfirmPopUp);
            packagesView.addEventListener(PackagesView.SHOW_NO_COIN_POP_UP, displayRewardNoCoinPopUp);

            addChild(packagesView);
            if (_filteredPackages) {
                packagesView.dataProvider = _filteredPackages;
            }

            if (AlertView.language) {
                packagesView.updateWithLanguage(AlertView.language);
            }

            blockerView = new BlockerView();
            blockerView.visible = false;
            addChild(blockerView);
            setChildIndex(backButton, getChildIndex(blockerView));

            shareView = new ShareView();
            shareView.visible = false;
            shareView.color = 0x000000;
            shareView.addEventListener(ShareView.SHARE_SELECTED, onShareSelected);
            addChild(shareView);
            if(!AdsManager.$.haveRewardAds){
                AdsManager.$.checkRewardAds(function(status:Boolean):void{

                });
            }
            //
            viewManager = new TuningScreenViewManager();
            viewManager.addMode(TuningScreenViewManager.PART_SLOTS_VIEW, function():void{
                AdsManager.$.bannerHide();

                partsList.visible = false;
                diskBrandList.visible = false;
                menuBackButton.visible = false;
                allPartsColorsView.visible = false;
                componentSelector.visible = false;
                backgroundMenuView.visible = false;
                tuningSettingsPanel.visible = false;

                if(!possiblePartSlotsList.dataProvider.contains(_possiblePartSlots[0])){
                    possiblePartSlotsList.dataProvider = new ListCollection(_possiblePartSlots);
                }
                if(componentSelector.selectedItem){
                    possiblePartSlotsList.scrollToDisplayIndex(possiblePartSlotsList.dataProvider.getItemIndex(componentSelector.selectedItem), 0);
                }
                possiblePartSlotsList.width = wantedWidth;
                possiblePartSlotsList.x = 0;
                possiblePartSlotsList.visible = true;
            });

            viewManager.addMode(TuningScreenViewManager.PAINTS_VIEW, function():void{
                SoundManager.playSound(SoundManager.PART_MENU);
                AdsManager.$.bannerHide();

                partsList.visible = false;
                diskBrandList.visible = false;
                menuBackButton.visible = false;
                componentSelector.visible = false;
                possiblePartSlotsList.visible = false;
                backgroundMenuView.visible = false;
                tuningSettingsPanel.visible = false;

                allPartsColorsView.visible = true;
            });

            viewManager.addMode(TuningScreenViewManager.CAR_SETTINGS_VIEW, function():void{
                SoundManager.playSound(SoundManager.PART_MENU);
                AdsManager.$.bannerHide();
                checkDiskChanges(null);

                partsList.visible = false;
                diskBrandList.visible = false;
                menuBackButton.visible = false;
                componentSelector.visible = false;
                possiblePartSlotsList.visible = false;
                allPartsColorsView.visible = false;

                tuningSettingsPanel.visible = true;
            });

            viewManager.addMode(TuningScreenViewManager.DISK_BRANDS_VIEW, function():void{
                AdsManager.$.bannerHide();
                if(selectedPartSlot == null || selectedPartSlot.name.toLowerCase() == "all"){
                    viewManager.setMode(TuningScreenViewManager.PART_SLOTS_VIEW);
                    return;
                }
                partsList.visible = false;
                menuBackButton.visible = false;
                possiblePartSlotsList.visible = false;
                allPartsColorsView.visible = false;
                backgroundMenuView.visible = false;
                tuningSettingsPanel.visible = false;

                componentSelector.visible = true;
                diskBrandList.visible = true;
            });

            viewManager.addMode(TuningScreenViewManager.MANUFACTURERS_VIEW, function():void{
                AdsManager.$.bannerHide();
                if(selectedPartSlot == null || selectedPartSlot.name.toLowerCase() == "all"){
                    viewManager.setMode(TuningScreenViewManager.PART_SLOTS_VIEW);
                    return;
                }
                partsList.visible = false;
                menuBackButton.visible = false;
                possiblePartSlotsList.visible = false;
                allPartsColorsView.visible = false;
                backgroundMenuView.visible = false;
                tuningSettingsPanel.visible = false;

                componentSelector.visible = true;
                diskBrandList.visible = true;
            });

            viewManager.addMode(TuningScreenViewManager.PARTS_VIEW, function():void{
                if(selectedPartSlot == null || selectedPartSlot.name.toLowerCase() == "all"){
                    viewManager.setMode(TuningScreenViewManager.PART_SLOTS_VIEW);
                    return;
                }
                diskBrandList.visible = false;
                menuBackButton.visible = false;
                possiblePartSlotsList.visible = false;
                allPartsColorsView.visible = false;
                backgroundMenuView.visible = false;
                tuningSettingsPanel.visible = false;

                componentSelector.visible = true;
                partsList.width = wantedWidth - PART_LIST_MARGIN;
                partsList.x = PART_LIST_MARGIN;
                partsList.visible = true;

            });

            viewManager.addMode(TuningScreenViewManager.DISK_PARTS_VIEW, function():void{
                if(selectedPartSlot == null || selectedPartSlot.name.toLowerCase() == "all"){
                    viewManager.setMode(TuningScreenViewManager.PART_SLOTS_VIEW);
                    return;
                }
                SoundManager.playSound(SoundManager.PART_MENU);
                diskBrandList.visible = false;
                possiblePartSlotsList.visible = false;
                allPartsColorsView.visible = false;
                backgroundMenuView.visible = false;
                tuningSettingsPanel.visible = false;
                /*if(selectedPartSlot != componentSelector.selectedItem){
                    if(!possiblePartSlotsList.dataProvider.contains(componentSelector.selectedItem)){
                        possiblePartSlotsList.dataProvider = new ListCollection(_possiblePartSlots);
                    }
                    selectedPart = selectedPartSlot.selectedPart;
                    possiblePartSlotsList.selectedItem = componentSelector.selectedItem;
                    partsList.selectedItem = selectedPart;
                }*/

                partsList.width = wantedWidth - (menuBackButton.width + PART_LIST_MARGIN);
                partsList.x = menuBackButton.width + PART_LIST_MARGIN;
                menuBackButton.x = PART_LIST_MARGIN;
                componentSelector.visible = true;
                menuBackButton.visible = true;
                partsList.visible = true;
            });

            viewManager.addMode(TuningScreenViewManager.VINYL_PARTS_VIEW, function():void{
                if(selectedPartSlot == null || selectedPartSlot.name.toLowerCase() == "all"){
                    viewManager.setMode(TuningScreenViewManager.PART_SLOTS_VIEW);
                    return;
                }

                diskBrandList.visible = false;
                possiblePartSlotsList.visible = false;
                allPartsColorsView.visible = false;
                tuningSettingsPanel.visible = false;
                backgroundMenuView.visible = false;

                partsList.width = wantedWidth - (menuBackButton.width + PART_LIST_MARGIN);
                partsList.x = menuBackButton.width + PART_LIST_MARGIN;
                menuBackButton.x = PART_LIST_MARGIN;
                componentSelector.visible = true;
                menuBackButton.visible = true;
                partsList.visible = true;
            });
        }

        private function tintChangedHandler(event:starling.events.Event):void {
            var windowPart:Part = tintControl.customValue as Part;
            if(windowPart){
                selectedPart = windowPart;
                selectedPartSlot = windowPart.parentSlot;
                dispatchEventWith(PART_SELECTED, false, {beforeRemovePartSlot: null});
            }
        }

        private function removeExtraItems():void{
            var cSelectedIndex:int = componentSelector.dataProvider.getItemIndex(componentSelector.selectedItem);
            for(var i:int = componentSelector.dataProvider.length - 1; i >= 0; i--){
                if(possiblePartSlots.indexOf(componentSelector.dataProvider.getItemAt(i)) == -1){
                    componentSelector.dataProvider.removeItemAt(i);
                    if(cSelectedIndex >= i){
                        cSelectedIndex--;
                    }
                }
            }
            cSelectedIndex = MathUtil.clamp(cSelectedIndex, 0, componentSelector.dataProvider.length - 1);
            componentSelector.selectedIndex = cSelectedIndex;
        }

        private function componentSelectorChangeHandler(event:starling.events.Event):void {
            removeExtraItems();
            componentSelector.isEnabled = false;
            if(!possiblePartSlotsList.dataProvider.contains(componentSelector.selectedItem) && possiblePartSlots.indexOf(componentSelector.selectedItem) != -1){
                possiblePartSlotsList.dataProvider = new ListCollection(possiblePartSlots);
            }
            possiblePartSlotsList.selectedItem = componentSelector.selectedItem;
            componentSelector.isEnabled = true;
        }

        private static function createUICircleButton(textureId:String):Button {
            var button:Button = ButtonFactory.circleButton(textureId);
            (button as ExtentedButton).color = 0x000000;
            button.width = 35;
            button.height = 35;
            button.visible = false;
            return  button;
        }

        private function onCameraError(event:starling.events.Event):void {
            this.dispatchEventWith(TuningScreen.CANCEL_CAMERA_SELECTED);
        }

        private function onShareSelected(event:starling.events.Event):void {
            if (shareView.selectedSocialId == 'fb') {
                dispatchEventWith(SHARE_FB_SELECTED);
            } else if (shareView.selectedSocialId == 'vk') {
                dispatchEventWith(SHARE_VK_SELECTED);
            } else if (shareView.selectedSocialId == 'instagram') {
                dispatchEventWith(SHARE_INSTAGRAM_SELECTED);
            }
            shareView.visible = false;


        }


        public function startCamera():void {
            cancelFromCameraButton.visible = true;
            acceptFromCameraButton.visible = true;
            cameraScreenshotButton.visible = true;
            carView.startCamera();
        }

        public function stopCamera(getBitmapData:Boolean = false):BitmapData {
            acceptFromCameraButton.visible = false;
            cancelFromCameraButton.visible = false;
            cameraScreenshotButton.visible = false;
            return carView.stopCamera(getBitmapData);
        }

        public function setCustomBackground(bitmapData:BitmapData):BitmapData {
            return carView.setCustomBackground(bitmapData);
        }


        private function onFillColorSelected(event:starling.events.Event):void {
            var color:uint = 0x000000;
            if (this.backgroundMenuView.selectedFillColor) color = this.backgroundMenuView.selectedFillColor.color;
            (saveButton as ExtentedButton).color = color;
            (backButton as ExtentedButton).color = color;
            (shareButton as ExtentedButton).color = color;
            (partsMenuButton as ExtentedButton).color = color;
            (colorMenuButton as ExtentedButton).color = color;
            (acceptFromCameraButton as ExtentedButton).color = color;
            (cancelFromCameraButton as ExtentedButton).color = color;
            shareView.color = color;
        }

        private function onAllPartSlotSelectedHandler(event:starling.events.Event):void {
            if (!possiblePartSlotsList.dataProvider.contains(possiblePartSlots[0])) {
                possiblePartSlotsList.dataProvider = new ListCollection(possiblePartSlots);
                componentSelector.dataProvider = possiblePartSlotsList.dataProvider;
            }
            possiblePartSlotsList.selectedIndex = 0;
        }

        private function onBackgroundUpload(event:starling.events.Event):void {
            this.dispatchEventWith(BACKGROUND_UPLOAD);
        }

        private function onBackgroundUploadComplete(event:starling.events.Event):void {
            var bitmapData:BitmapData = setCustomBackground(event.data.bitmapData);
            dispatchEvent(new starling.events.Event(BACKGROUND_UPLOAD_COMPLETE, false, {bitmapData: bitmapData}));
        }

        private function onSnapShotBackgroundUpload(event:starling.events.Event):void {
            this.dispatchEventWith(SNAPSHOT_BACKGROUND);
        }

        private function onBackgroundDelete(event:starling.events.Event):void {
            selectedBackground = backgroundMenuView.selectedBackground;
            car.selectedBackground = null;
            dispatchEventWith(BACKGROUND_DELETE);
        }

        private function onBackgroundSelected(event:starling.events.Event):void {
            selectedBackground = backgroundMenuView.selectedBackground;
            selectedFillColor = backgroundMenuView.selectedFillColor;
            dispatchEventWith(BACKGROUND_SELECTED);
        }

        private function onBackgroundUnselected(event:starling.events.Event):void {
            selectedBackground = null;
            selectedFillColor = null;
            dispatchEventWith(BACKGROUND_SELECTED);
        }

        private function onOpenBackgroundHandler(event:starling.events.Event):void {
            tuningSettingsPanel.visible = false;
            backgroundMenuView.open();
        }

        private function onCloseBackground(event:starling.events.Event):void {
            tuningSettingsPanel.visible = true;
            backgroundMenuView.close();
        }

        private function onMenuBackHandler(event:starling.events.Event):void {
            AdsManager.$.bannerHide();
            SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
            if (Car.DISKS_SLOTS.indexOf(this.selectedPartSlot.name) >= 0 && !car.isChopper/* && !disksBrandsManufacturesAreNull*/) {
                viewManager.setMode(TuningScreenViewManager.DISK_BRANDS_VIEW);
                diskBrandList.touchable = true;
            } else if (brandListCollection[selectedPartSlot.name]) {
                viewManager.setMode(TuningScreenViewManager.DISK_BRANDS_VIEW);
                diskBrandList.touchable = true;
            } else {
                var scrollPosition:Number = _possibleScrollPosition;
                switchVinylList(true);
                componentSelector.visible = true;
                _possibleScrollPosition = scrollPosition;

                partsList.visible = false;
                possiblePartSlotsList.visible = true;
                menuBackButton.visible = false;
            }
            removePartDiskSettingsView();
        }

        private function componentSelectorTriggeredHandler(event:starling.events.Event):void {
            AdsManager.$.bannerHide();
            SoundManager.playSound(SoundManager.PART_MENU);
            viewManager.setMode(TuningScreenViewManager.PART_SLOTS_VIEW);
            removePartDiskSettingsView();
            //
            if(componentSelector.selectedItem && possiblePartSlotsList.dataProvider.contains(componentSelector.selectedItem)){
                setListScrollPosition(possiblePartSlotsList, componentSelector.selectedItem);
            }
        }

        private static function setListScrollPosition(list:List, item:Object):void{
            list.validate();
            var index:int = list.dataProvider.getItemIndex(item);
            var scrollPosition:Number = 0;
            for(var i:int = 0; i < index; i++){
                var renderer:PartSlotItemRenderer = list.itemToItemRenderer(list.dataProvider.getItemAt(i)) as PartSlotItemRenderer;
                if(renderer){
                    scrollPosition += renderer.width;
                }
            }
            scrollPosition = Math.min(list.maxHorizontalScrollPosition, scrollPosition);
            list.scrollToPosition(scrollPosition, 0, 0);
        }

        public var oldSelectedPartSlot:PartSlot = null;

        protected function onToggleState(event:starling.events.Event):void {
            // ************************************
            carView.resetFastModeCurrentLoaded();
            // ************************************
            var stateName:String = event.data.stateName;
            var newValue:Boolean = event.data.newValue;
            var partSlotName:String;

            oldSelectedPartSlot = selectedPartSlot;
            //нужно установить стейт


            if (stateName == 'rear_light' || stateName == 'front_light' || stateName == 'disk_neon') {
                var nameOfState:String = null;
                if (stateName == 'rear_light') {
                    var rearLightPartSlot:PartSlot = _car.getPossiblePartSlot('rear_lights');
                    selectedPartSlot = rearLightPartSlot;
                    if (newValue) {
                        //ставим
                        if (rearLightPartSlot.selectedPart && rearLightPartSlot.selectedPart.states && rearLightPartSlot.selectedPart.states['lights']) {
                            nameOfState = 'lights';
                        }
                    } else {
                        nameOfState = null;
                    }
                } else if (stateName == 'front_light') {
                    var frontLightPartSlot:PartSlot = _car.getPossiblePartSlot('front_lights');
                    selectedPartSlot = frontLightPartSlot;
                    if (newValue) {
                        //ставим
                        if (frontLightPartSlot.selectedPart && frontLightPartSlot.selectedPart.states && frontLightPartSlot.selectedPart.states['lights']) {
                            nameOfState = 'lights';
                        } else {
                            for (var i:int = 0; i < _car.possiblePartSlots.length; i++) {
                                var partSlot:PartSlot = _car.possiblePartSlots[i] as PartSlot;
                                if (partSlot && partSlot.selectedPart && partSlot.selectedPart.states && partSlot.selectedPart.states['lights'] && partSlot.selectedPart.selectedState == null) {
                                    nameOfState = 'lights';
                                    break;
                                }
                            }
                        }
                    } else {
                        nameOfState = null;
                    }
                } else if (stateName == 'disk_neon') {
                    var disksPartSlot:PartSlot;
                    trace('---')
                    for each(partSlotName in Car.CAR_DISKS_SLOTS) {
                        disksPartSlot = _car.getPossiblePartSlot(partSlotName);
                        if (!disksPartSlot) {
                            continue;
                        }
                        trace('partSlotName:', partSlotName, disksPartSlot.selectedPart && disksPartSlot.selectedPart.selectedState ? disksPartSlot.selectedPart.selectedState.nameOfState : 'null');
                    }
                    trace('---')


                    for each(partSlotName in Car.CAR_DISKS_SLOTS) {
                        disksPartSlot = _car.getPossiblePartSlot(partSlotName);
                        trace('partSlotName:', partSlotName, disksPartSlot)
                        if (!disksPartSlot) continue;
                        selectedPartSlot = disksPartSlot;
                        if (newValue) {
                            //ставим
                            var name:String = 'neon';
                            nameOfState = name;
                        } else {
                            nameOfState = null;
                        }

                        if ((newValue && nameOfState) || (!newValue && !nameOfState)) {
                            break;
                        }
                    }
                }

                dispatchEventWith(TuningSelectedStateView.STATE_SELECTED, true, {
                    selectedState: nameOfState,
                    part: Part,
                    stateType: stateName
                });
            } else {
                var neonPartSlot:PartSlot = _car.getPossiblePartSlot('neon');
                if (newValue) {
                    setNeonFromEffectMenu = true;
                    var neonIndex:int = possiblePartSlotsList.dataProvider.getItemIndex(_car.getPossiblePartSlot('neon'));
                    possiblePartSlotsList.selectedIndex = neonIndex;
                    partsList.selectedIndex = 0;
                } else {
                    this.carView.removePartSlot(neonPartSlot);
                    neonPartSlot.selectedPart = null;
                }
            }
        }

        private var setNeonFromEffectMenu:Boolean = false;

        protected function onErrorToggleState(event:starling.events.Event):void {
            var stateName:String = event.data.stateName;
            if (stateName == 'disk_neon') {
                this.notificationView.title = 'Disk\'s neon is not available';
                this.notificationView.text = 'Please change disks to enable this feature';

                addChild(notificationView)
                notificationView.visible = true;


            } else if (stateName == 'neon') {

                this.notificationView.title = 'Neon is not available';
                this.notificationView.text = 'Please change car to enable this feature';
                addChild(notificationView)
                notificationView.visible = true;
            } else if (stateName == 'rear_light') {

                this.notificationView.title = 'Lights is not available';
                this.notificationView.text = 'Please change rear lights to enable this feature';
                addChild(notificationView)
                notificationView.visible = true;
            } else if (stateName == 'front_light') {

                this.notificationView.title = 'Lights is not available';
                this.notificationView.text = 'Please change front lights to enable this feature';
                addChild(notificationView)
                notificationView.visible = true;
            }

            //notificationView


            //Можем выводить ошибку через нотификации
        }

        private function colorMenuButtonTriggeredHandler(event:starling.events.Event = null):void {
            //trace("partsMenuButtonTriggeredHandler:", this.frameImage.height, this.actualHeight)
            if (this.selectedPartSlot == null) {
                this.selectedPartSlotIndex = 0;
            }

            if (Math.round(this.frameImage.height) == this.actualHeight) {
                viewManager.setMode(TuningScreenViewManager.PAINTS_VIEW);
                this.openMenu(event);
            } else {
                tuningSettingsPanel.car = car;
                allPartsColorsView.partSlot = selectedPartSlot;
                allPartsColorsView.partColors = car.possiblePartColors;
                showColorMenuPanel(true);
            }
        }


        private function partsMenuButtonTriggeredHandler(event:starling.events.Event):void {
            SoundManager.playSound(SoundManager.PART_MENU);
            if(viewManager.currentMode == TuningScreenViewManager.PART_SLOTS_VIEW){
                openMenu(true);
            }else if(TuningScreenViewManager.COMPONENTS_GROUP.indexOf(viewManager.currentMode) != -1){
                viewManager.setMode(TuningScreenViewManager.PART_SLOTS_VIEW);
                openMenu(true);
            }else if (Math.round(this.frameImage.height) == this.actualHeight) {
                viewManager.setMode(TuningScreenViewManager.PART_SLOTS_VIEW);
                openMenu(true);
            } else {
                showColorMenuPanel(false);
            }
        }

        private function carSettingsButtonTriggeredHandler(event:starling.events.Event):void {
            openMenu(true);
            viewManager.setMode(TuningScreenViewManager.CAR_SETTINGS_VIEW, false);
        }


        public function showColorMenuPanel(show:Boolean):void {
            if (show) {
                AdsManager.$.bannerHide();
                if(viewManager){
                    viewManager.setMode(TuningScreenViewManager.PAINTS_VIEW);
                }
            } else {
                toggleBanner();
                if (allPartsColorsView && backgroundMenuView) {
                    allPartsColorsView.visible = false;
                    backgroundMenuView.visible = false;
                }

                var componentView:String = viewManager.getGroupLastMode(TuningScreenViewManager.COMPONENTS_GROUP);
                if(TuningScreenViewManager.COMPONENTS_GROUP.indexOf(viewManager.currentMode) == -1 && componentView){
                    viewManager.setMode(componentView);
                }else if (selectedPartSlot && Car.DISKS_SLOTS.indexOf(this.selectedPartSlot.name) >= 0) {
                    var wheelModified:Boolean = false;
                    if (selectedPart) {
                        if (selectedPart.parentSlot.name == 'disks' || selectedPart.parentSlot.name.indexOf('front') != -1) {
                            wheelModified = frontWheelModified;
                        } else {
                            wheelModified = rearWheelModified;
                        }
                    }
                    if (wheelModified) {
                        if (selectedPartSlot.selectedPart && selectedPartSlot.selectedPart.name.indexOf('factory') >= 0) {
                            viewManager.setMode(TuningScreenViewManager.DISK_PARTS_VIEW);
                        } else {
                            viewManager.setMode(TuningScreenViewManager.DISK_PARTS_VIEW);
                        }
                    } else {
                        viewManager.setMode(TuningScreenViewManager.DISK_BRANDS_VIEW);
                    }
                    possiblePartSlotsList.visible = false;
                } else if (this.selectedPartSlot && !selectedPartSlot.dummy) {
                    if(PartSlot.VINYLS.indexOf(selectedPartSlot.name) != -1){
                        viewManager.setMode(TuningScreenViewManager.VINYL_PARTS_VIEW);
                    }else if(brandListCollection[selectedPartSlot.name]){
                        diskBrandList.dataProvider = new ListCollection(brandListCollection[selectedPartSlot.name]);
                        viewManager.setMode(TuningScreenViewManager.MANUFACTURERS_VIEW);
                    }else{
                        viewManager.setMode(TuningScreenViewManager.PARTS_VIEW);
                    }
                } else {
                    viewManager.setMode(TuningScreenViewManager.PART_SLOTS_VIEW);
                    if (possiblePartSlotsList.layout is HorizontalLayout) {
                        HorizontalLayout(possiblePartSlotsList.layout).paddingLeft = -4;
                    }
                }
            }
        }


        override public function initializeBackButton():void {
            if (hasBackButton) {
                backButton = createUICircleButton('a_burger_button');
                backButton.x = BUTTON_SIDE_MARGIN;
                backButton.y = 6;
                backButton.addEventListener(starling.events.Event.TRIGGERED, onMenuButton);
                if (Device.isIphoneXVariant()) {
                    backButton.x += 6;
                    backButton.y += 6;
                }
                addChild(backButton);
            }
        }


        private function onMenuButton(event:starling.events.Event):void {

            if (mainMenuView)
                mainMenuView.toggle();
        }


        private function onClosePackagesView(event:starling.events.Event):void {


            backButton.visible = true;

        }

        public function reloadCar():void {


            /*if(carLoaded){

			this.carView.invalidate();
			}else*/
            {

                var facade:ApplicationFacade = ApplicationFacade.getInstance();

                /*var carsProxy:CarsProxy= facade.retrieveProxy(CarsProxy.NAME) as CarsProxy;


				carsProxy.loadCar(car);   */

                var diskBrandsProxy:DiskBrandsProxy = facade.retrieveProxy(DiskBrandsProxy.NAME) as DiskBrandsProxy;

                diskBrandsProxy.loadDiskBrands(car.id);


            }
        }

        //private

        private function viewCar360(event:starling.events.Event):void {

            var car:Car = event.data.car;
            var packet:Packet = event.data.packet;
            packagesView.hide();

            BrandsScreen.SELECTED_CAR = car;
            BrandsScreen.SELECTED_PACKET = packet;


            backButtonTriggeredHandler(new starling.events.Event(''));


            //this.dispatchEventWith(PackagesView.VIEW_CAR_360, false, {car:car});

        }

        private function onBuyNowPacket(event:starling.events.Event):void {


            var packet:Packet = event.data.packet;

            this.dispatchEventWith(BUY_NOW_PACKET, false, {packet: packet});
        }

        protected function packagesViewOpen(event:starling.events.Event):void {

            //this.purchaseView.visible=false;
            AdsManager.$.bannerHide();
            backButton.visible = false;
        }

        private function carViewOnShowFrame(event:starling.events.Event):void {
            currentFrame = event.data.currentFrame;
        }

        private function acceptScreenshotButtonTriggeredHandler(event:starling.events.Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchEventWith(ACCEPT_CAMERA_SELECTED);
        }

        private function cancelScreenshotButtonTriggeredHandler(event:starling.events.Event):void {
            SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
            dispatchEventWith(CANCEL_CAMERA_SELECTED);
        }

        private function shavingButtonTriggeredHandler(event:starling.events.Event):void {
            dispatchEventWith(SHAVING_SELECTED);
        }

        public function setSuspensionControlEnabled(enabled:Boolean):void {
            _suspensionEnabled = enabled;
            if (suspensionControl) {
                suspensionControl.alpha = enabled ? 1.0 : 0.5;
                suspensionControl.touchable = enabled;
            }

            if (tintControl) {
                tintControl.alpha = enabled ? 1.0 : 0.5;
                tintControl.touchable = enabled;
            }

            if (!_suspensionEnabled) {
                carView.car.suspension = 0;
                carView.updateSuspension();
            }
        }

        private var carPartLoading:Boolean = false;

        public function updateProgressPosition():void {
            if (_carSplashImage && _carSplashImage.image && _carSplashImage.image.height) {
                var yRatio:Number = 970 / 1080;
                progressFlatBar.y = _carSplashImage.image.y + yRatio * _carSplashImage.image.height + 7;
            } else {
                progressFlatBar.y = -progressFlatBar.height;
            }
        }

        private function progressHandler(progress:Number):void {
            {//костыль - всем костылям костыль
                //если по какойнибудь причине после исчезновения прогрес опять стал видимым не много то игнорируем
                if ((progress != 0) && (progressBar.progress > 0.7) && (progressBar.progress > progress)) {
                    return;
                }
            }

            if (progress > 1.0){
                progress = 1.0;
            }

            progressBar.progress = progress;
            enableBottomMenu(false);
            var p:Number = progress;
            if (this.car.partMaterialZip) {
                p = 0.41 + (progress * 0.59);
            } else {
                p = 0.03 + (progress * 0.97)
            }

            if (p > 1) {
                p = 1;
            }
            progressFlatBar.progress = p;
            updateProgressPosition();

            if (progressBar.visible && carLoaded) {
                if (partDiskSettingView) {
                    partDiskSettingView.enableButtons(false);
                }
            }


            if (!carView.visible) {
                carPartLoading = (progress >= 0 && progress < 1.0);
                dispatchEventWith(CAR_PART_LOADING_PROGRESS, false, {progress: progress});
            }

            if (progress >= 1.0) {
                carLoaded = true;
                if (setNeonFromEffectMenu) {
                    setNeonFromEffectMenu = false;
                    colorMenuButtonTriggeredHandler();
                }

                if (partDiskSettingView) {
                    partDiskSettingView.enableButtons(true);
                }

                if (allPartsColorsView && allPartsColorsView.materialList){
                    allPartsColorsView.materialList.touchable = true;
                }

                enableBottomMenu(true);

                if (partsList) {
                    partsList.touchable = true;
                }

                if (!carView.visible) {
                    handleBarImage.visible = false;
                    handleOpenBarImage.visible = true;

                    progressFlatBar.visible = false;
                    colorMenuButton.visible = true;
                    partsMenuButton.visible = true;
                    settingsMenuButton.visible = true;
                    fastModeButton.visible = fastModeHelpButton.visible = true;
                    backButton.visible = true;
                    shareButton.visible = true;
                    saveButton.visible = true;
                    cameraButton.visible = true;
                    carView.resetCarScale();
                    carView.removeCarSplash();

                    carView.visible = true;
                    CarView.CAR_FULL_LOADED = true;
                    AdsManager.$.requestBanner();
                    dispatchEventWith(CAR_PART_FULL_LOADED);

                    setTimeout(function():void{
                        if (car.haveSound) {
                            loadSounds();
                        }

                    }, 300)
                }

                if(carView.visible && carLoaded && !menuBackground.visible && !progressBar.visible){
                    BannerStackedScreen.showAdBanner();
                }

                ZipManager.getInctance().reset();

                System.gc();
                System.gc();

            } else if (progress > 0 && carView.visible && progress <= 1.0) {
                progressBar.visible = true;
            }
        }

        private var startAudio:Sound = new Sound();
        private var startAudioChannel:SoundChannel;
        private var powerOn: Boolean = false;

        private var gasAudio:Sound = new Sound();
        private var gasAudioChannel:SoundChannel;

        private var downAudio:Sound = new Sound();
        private var downAudioChannel:SoundChannel;

        private var revSound:Sound = new Sound();
        private var revSoundChannel:SoundChannel;

        private var downloadedSoundsCount: int = 0;
        private function loadSounds(): void {
            try {
                var startSoundUrl:String = Constants.SOUNDS_URL + car.path.replace('Cars/', '') + 'start.mp3';
                startAudio = new Sound();
                startAudio.addEventListener(IOErrorEvent.IO_ERROR, onSoundLoadError);
                startAudio.addEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
                startAudio.load(new URLRequest(startSoundUrl));

                if(car.hasSoundRevs){
                    var revUrl:String = Constants.SOUNDS_URL + car.path.replace('Cars/', '') + 'rev.mp3';
                    revSound = new Sound(new URLRequest(revUrl));
                    revSound.addEventListener(IOErrorEvent.IO_ERROR, onSoundLoadError);
                    revSound.addEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
                }else{
                    gasAudio = new Sound(new URLRequest(Constants.SOUNDS_URL + car.path.replace('Cars/', '') + 'gas.mp3'));
                    downAudio = new Sound(new URLRequest(Constants.SOUNDS_URL + car.path.replace('Cars/', '') + 'down.mp3'));

                    gasAudio.addEventListener(IOErrorEvent.IO_ERROR, onSoundLoadError);
                    downAudio.addEventListener(IOErrorEvent.IO_ERROR, onSoundLoadError);

                    gasAudio.addEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
                    downAudio.addEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
                }

            } catch (e: *) {
                onSoundLoadError();
            }

        }

        private function onSoundLoadError(e: IOErrorEvent = null):void {
            startAudio.removeEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
            gasAudio.removeEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
            downAudio.removeEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
            revSound.removeEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);

            startAudio.removeEventListener(IOErrorEvent.IO_ERROR, onSoundLoadError);
            gasAudio.removeEventListener(IOErrorEvent.IO_ERROR, onSoundLoadError);
            downAudio.removeEventListener(IOErrorEvent.IO_ERROR, onSoundLoadError);
            revSound.removeEventListener(IOErrorEvent.IO_ERROR, onSoundLoadError);

        }

        private function onSoundLoadComplete(e: flash.events.Event):void {
            if(!car){
                return;
            }
            downloadedSoundsCount++;
            var count:int = car.hasSoundRevs?2:3;
            if (downloadedSoundsCount == count) {
                startAudio.removeEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
                startAudio.removeEventListener(IOErrorEvent.IO_ERROR, onSoundLoadError);
                if(car.hasSoundRevs){
                    revSound.removeEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
                    revSound.removeEventListener(IOErrorEvent.IO_ERROR, onSoundLoadError);
                }else{
                    gasAudio.removeEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
                    downAudio.removeEventListener(flash.events.Event.COMPLETE, onSoundLoadComplete);
                    gasAudio.removeEventListener(IOErrorEvent.IO_ERROR, onSoundLoadError);
                    downAudio.removeEventListener(IOErrorEvent.IO_ERROR, onSoundLoadError);
                }

                powerOnButton.visible = true;
                powerOnButton.isSelected = false;
                gasButton.visible = false;
                fastModeButton.x = powerOnButton.x - (10 + fastModeButton.width);
                fastModeHelpButton.x = fastModeButton.x - (fastModeHelpButton.width + 5);
                addEventListener(starling.events.Event.ENTER_FRAME, onEnterFrame);
            }
        }

        public function showBlocker():void {

            if (this.blockerView) {
                blockerView.visible = true;
            }
        }

        public function hideBlocker():void {

            if (this.blockerView) {
                blockerView.visible = false;
            }
        }

        public function updateButtonPositions():void{
            if(!stage){
                return;
            }
            var marginSize:Number = 24;
            var leftMargin:Number = 0;
            var rightMargin:Number = 0;
            if(stage.starling.nativeStage.orientation == StageOrientation.ROTATED_RIGHT){
                leftMargin = marginSize;
            }else if(stage.starling.nativeStage.orientation == StageOrientation.ROTATED_LEFT){
                rightMargin = marginSize;
            }

            partsMenuButton.x = BUTTON_SIDE_MARGIN + leftMargin;
            colorMenuButton.x = partsMenuButton.x + partsMenuButton.width + BUTTON_GAP;
            settingsMenuButton.x = colorMenuButton.x + settingsMenuButton.width + BUTTON_GAP;
            backButton.x = BUTTON_SIDE_MARGIN + leftMargin;
            //
            var rightX:Number = actualWidth - saveButton.width - BUTTON_SIDE_MARGIN - rightMargin;
            updateScreenshotButtons();
            saveButton.x = rightX;
            shareButton.x = saveButton.x - (shareButton.width + BUTTON_GAP);
            cameraButton.x = shareButton.x - (cameraButton.width + BUTTON_GAP);
            //
            updateScreenshotButtons();

            powerOnButton.x = rightX;
            gasButton.x = powerOnButton.x - (BUTTON_GAP + gasButton.width);
            revSoundButton.x = powerOnButton.x - (BUTTON_GAP + revSoundButton.width);
            //
            if(powerOnButton.visible){
                fastModeButton.x = powerOnButton.x - (BUTTON_GAP + fastModeButton.width);
            }else{
                fastModeButton.x = rightX;
            }
            fastModeHelpButton.x = fastModeButton.x - (fastModeHelpButton.width + BUTTON_GAP/2);
            acceptFromCameraButton.x = saveButton.x;
            cameraScreenshotButton.x = acceptFromCameraButton.x - acceptFromCameraButton.width - BUTTON_GAP;
            cancelFromCameraButton.x = saveButton.x;
        }

        protected function updateNewFbSharingButton():void {

        }

        private function addElement(item:*, index:int = -1):void {
            for (var i:int = 0; i < numChildren; i++) {
                if (getChildAt(i).name == item.name) return;
            }
            index == -1 ? addChild(item) : addChildAt(item, index);
        }

        public function get wantedWidth():Number {
            return _wantedWidth > 0 ? _wantedWidth : actualWidth;
        }

        public function get mainMenuView():MainMenuView {
            return _mainMenuView;
        }

        private function updateScreenshotButtons():void{
            arButton.x = currentScreenshotButton.x = cameraButton.x;
            arButton.y = arButton.height + BUTTON_MENU_GAP + cameraButton.y;
            currentScreenshotButton.y = currentScreenshotButton.height + 10 + arButton.y;
        }

        protected override function draw():void {
            super.draw();

            if (!mainMenuView) {
                _mainMenuView = new MainMenuView();
                mainMenuView.owner = this;
                mainMenuView.name = 'mainMenuView';
                mainMenuView.setSize(MainMenuView.MENU_WIDTH, actualHeight);
                mainMenuView.parentActualWidth = actualWidth;
                mainMenuView.parentActualHeight = actualHeight;

                mainMenuView.addEventListener(MainMenuView.START_SHOW, menuOpenHandler);
                mainMenuView.addEventListener(starling.events.Event.CLOSE, menuCloseHandler);

                addElement(mainMenuView, this.numChildren - 1);
                //mainMenuView.toggleButton.visible=false;
            }

            var sizeInvalid:Boolean = isInvalid(FeathersControl.INVALIDATION_FLAG_SIZE);
            progressBar.width = 100; //actualWidth / 4;
            var frameHeight:Number = handleBarImage.y + handleBarImage.height;

            if (frameHeight != frameImage.height) {
                frameImage.height = frameHeight;
                _newFrameImageHeight = frameImage.height;
            }


            if (sizeInvalid) {

                var rightX:Number = actualWidth - saveButton.width - BUTTON_SIDE_MARGIN;
                var bottomY:Number = frameImage.height - colorMenuButton.height - 6.5;

                notificationView.x = 0;
                notificationView.y = 0;

                notificationView.width = this.actualWidth;
                notificationView.height = 40;

                //backgroundQuad.width = actualWidth;
                //backgroundQuad.height = actualHeight;

                carView.width = actualWidth;
                carView.height = actualHeight;


                updateScreenshotButtons();
                if (Device.isIphoneXVariant()) {
                    rightX -= BUTTON_SIDE_MARGIN;
                }
                BUTTON_HORIZONTAL_GAP = (actualWidth < 600?5:BUTTON_GAP);
                saveButton.x = rightX;
                shareButton.x = saveButton.x - (shareButton.width + BUTTON_HORIZONTAL_GAP);
                cameraButton.x = shareButton.x - (cameraButton.width + BUTTON_HORIZONTAL_GAP);
                //
                updateScreenshotButtons();

                powerOnButton.x = rightX;
                gasButton.x = powerOnButton.x - (BUTTON_GAP + gasButton.width);

                powerOnButton.y = bottomY;
                revSoundButton.y = bottomY;
                gasButton.y = bottomY;
                fastModeButton.y = bottomY;

                revSoundButton.x = powerOnButton.x - (BUTTON_GAP + revSoundButton.width);
                revSoundButton.y = powerOnButton.y;

                //
                if(powerOnButton.visible){
                    fastModeButton.x = powerOnButton.x - (BUTTON_GAP + fastModeButton.width);
                }else{
                    fastModeButton.x = rightX;
                }

                fastModeHelpButton.x = fastModeButton.x - (fastModeHelpButton.width + BUTTON_GAP/2);
                fastModeHelpButton.y = fastModeButton.y + (fastModeButton.height - fastModeHelpButton.height) / 2;
                //
                acceptFromCameraButton.x = saveButton.x;
                cameraScreenshotButton.x = acceptFromCameraButton.x - acceptFromCameraButton.width - BUTTON_GAP;
                cancelFromCameraButton.x = saveButton.x;

                updateTopButtonsYCoord();
                partsList.width = wantedWidth - PART_LIST_MARGIN;
                if (packagesView) {
                    packagesView.setSize(packagesView.width, actualHeight);
                    packagesView.x = 0;
                    packagesView.y = 0;
                    packagesView.parentActualWith = actualWidth;
                    packagesView.parentActualHeight = actualHeight;
                }

                allPartsColorsView.height = menuBackground.height;
                allPartsColorsView.width = actualWidth;
                allPartsColorsView.x = 0;
                allPartsColorsView.y = menuBackground.y;

                tuningSettingsPanel.x = 0;
                tuningSettingsPanel.y = menuBackground.y;

                backgroundMenuView.x = 0;
                backgroundMenuView.y = menuBackground.y;
                backgroundMenuView.height = menuBackground.height;
                backgroundMenuView.width = actualWidth;

                blockerView.setSize(this.actualWidth, this.actualHeight);

                menuBackButton.x = PART_LIST_MARGIN;
                menuBackButton.y = this.menuBackground.y;
                menuBackButton.setSize(50, menuHeight);
                try {
                    carView.topOffset = 30;
                } catch (e:Error) {
                    if (mainMenuView) {
                        mainMenuView.y = topButtonPadding;
                        mainMenuView.setSize(MainMenuView.MENU_WIDTH, actualHeight - topButtonPadding);
                        mainMenuView.parentActualWidth = actualWidth;
                        mainMenuView.parentActualHeight = actualHeight - topButtonPadding;
                    }
                    Tracer.print('error:', e.getStackTrace());
                }
                if (possiblePartSlotsList && menuBackButton && allPartsColorsView && backgroundMenuView) {

                }
            }

            if (isInvalid('ads') || sizeInvalid) {
                updateTopButtonsYCoord();

                if (mainMenuView) {
                    mainMenuView.y = topButtonPadding;
                    mainMenuView.setSize(MainMenuView.MENU_WIDTH, actualHeight - topButtonPadding);
                    mainMenuView.parentActualWidth = actualWidth;
                    mainMenuView.parentActualHeight = actualHeight - topButtonPadding;
                }
            }

            if (isInvalid('colorsViewEnabled')) {
                commitColorsViewEnabled();
            }

            if (isInvalid('car')) {

                if (!allPartsColorsView) {
                    setTimeout(invalidate, 500, 'car');
                    return;
                }

                if (!suspensionControl) {
                    setTimeout(invalidate, 500, 'car');
                    return;
                }

                if (tuningSettingsPanel){
                    tuningSettingsPanel.car = car;
                }

                if (backgroundMenuView) {
                    backgroundMenuView.car = car;
                }

                if (car) {

                    var maxSuspension:int = car.limiterMaxSuspension;
                    var minSuspension:int = car.limiterMinSuspension;
                    var suspension:int = car.suspension;

                    suspensionControl.setMinMaxValues(minSuspension, maxSuspension);
                    suspensionControl.value = -suspension;
                    layoutMenuViews();
                    isModifed = true;
                }


            }
            validateCarDrawing();


            if (isInvalid('carBackgrounds')) {
                backgroundMenuView.carBackgrounds = _carBackgrounds;
            }

            if (isInvalid('brand_data_provider')) {
                diskBrandList.dataProvider = brandCollection;
            }


            if (isInvalid('parts_data_provider')) {
                partsList.dataProvider = partsCollection;
                if (selectedPartSlot && selectedPartSlot.selectedPart/* && Car.DISKS_SLOTS.indexOf(selectedPartSlot.name) != -1*/) {
                    var listenerRemoved:Boolean = false;
                    if (partsList.hasEventListener(starling.events.Event.CHANGE, partsListChangeHandler)) {
                        partsList.removeEventListener(starling.events.Event.CHANGE, partsListChangeHandler);
                        listenerRemoved = true;
                    }
                    if (partsList.dataProvider.contains(selectedPartSlot.selectedPart)) {
                        partsList.selectedItem = selectedPartSlot.selectedPart;
                    } else {
                        for (var j:int = 0; j < partsList.dataProvider.length; j++) {
                            var p:Part = partsList.dataProvider.getItemAt(j) as Part;
                            if (p && p.name == selectedPartSlot.selectedPart.name) {
                                partsList.selectedIndex = j;
                                break;
                            }
                        }
                    }
                    if (partsList.selectedIndex != -1) {
                        partsList.scrollToDisplayIndex(partsList.selectedIndex);
                    }
                    if (listenerRemoved) {
                        partsList.addEventListener(starling.events.Event.CHANGE, partsListChangeHandler);
                    }
                }
            }

            /*if (tintControl) {
                tintControl.x = 20;
                tintControl.y = (tuningSettingsPanel.height - tintControl.height) / 2 + 5;
            }

            if (suspensionControl) {
                suspensionControl.x = tintControl.x + tintControl.width + 20;
                suspensionControl.y = (tuningSettingsPanel.height - suspensionControl.height) / 2 + 5;
            }*/

            if (carView && !carView.visible && this.handleBarImage) {
                this.handleBarImage.visible = false;
            }
        }

        private function updateTopButtonsYCoord():void {
            var gap:Number = 5.2;
            var yPos:Number = gap + topButtonPadding + frameImage.height - actualHeight;
            yPos = (yPos < gap) ? gap : yPos;
            backButton.y = yPos;
            saveButton.y = yPos;
            shareButton.y = yPos;
            cameraButton.y = yPos;
            cancelFromCameraButton.y = yPos;

            if (Device.isIphoneXVariant()) {
                backButton.y += 6;
                saveButton.y += 6;
                cameraButton.y += 6;
                shareButton.y += 6;
                cancelFromCameraButton.y += 6;
            }
        }

        private function validateCarDrawing():void {
            if(tuningSettingsPanel && menuBackground){
                tuningSettingsPanel.width = actualWidth;
                tuningSettingsPanel.height = menuBackground.height;
                tuningSettingsPanel.car = car;
                tuningSettingsPanel.invalidate(INVALIDATION_FLAG_SIZE);
            }
            if(tintControl){
                for(var i:int = 0; i < Car.WINDOW_SLOTS.length; i++){
                    var window:PartSlot = car.getPossiblePartSlot(Car.WINDOW_SLOTS[i]);
                    var parts:Array = [];
                    var j:int;
                    if(window && window.possibleParts && window.possibleParts.length > 1 && window.selectedPart){
                        var result:Array = RegExpUtil.getWindowTintState(window.selectedPart.folder);
                        if(result && result.length){
                            var doorPartSlot:PartSlot = car.getPossiblePartSlot('left_doors');
                            var folderName:String = doorPartSlot.selectedPart.folder;
                            for(j = 0; j < window.possibleParts.length; j++){
                                if(Part(window.possibleParts[j]).folder.indexOf(folderName) != -1){
                                    parts.push(window.possibleParts[j]);
                                }
                            }
                        }else{
                            parts = window.possibleParts.concat();
                        }
                        for(j = 0; j < parts.length; j++){
                            parts[j] = {p:parts[j], index: RegExpUtil.getWindowTintValue(parts[j].name)};
                        }
                        parts.sortOn("index", Array.NUMERIC|Array.DESCENDING);
                        for(j = 0; j < parts.length; j++){
                            parts[j] = parts[j].p;
                        }
                        tintControl.customValues = parts;
                        var selectedIndex:int = parts.indexOf(window.selectedPart);
                        if(selectedIndex == -1){
                            selectedIndex = 0;
                        }
                        tintControl.value = selectedIndex;
                        break;
                    }
                }
            }
        }

        private var isModifed:Boolean = false;


        private function commitColorsViewEnabled():void {

            if (colorsViewEnabled) {
                this._colorsViewEnabled = false;
                if (tuningColorsView) {
                    tuningColorsView.alpha = 0.5;
                    tuningColorsView.touchable = false;
                }
                return;
            }

            if (tuningColorsView) {
                tuningColorsView.alpha = colorsViewEnabled ? 1.0 : 0.5;
                tuningColorsView.touchable = colorsViewEnabled;
            }
        }

        protected override function layoutMenuViews():void {
            super.layoutMenuViews();

            var bottomY:Number = frameImage.height - colorMenuButton.height - 6.5;
            var rightX:Number = actualWidth - saveButton.width - BUTTON_SIDE_MARGIN;

            handleBarImage.y = frameImage.height - handleBarImage.height;
            handleTitleLabel.y = handleBarImage.y + 2;

            if (Constants.SCALE_FACTOR == 1.5) {
                possiblePartSlotsList.y = menuBackground.y;
            } else {
                possiblePartSlotsList.y = menuBackground.y;
            }


            possiblePartSlotsList.width = wantedWidth;
            possiblePartSlotsList.height = menuHeight;

            partsList.y = menuBackground.y;
            componentSelector.y = menuBackground.y;

            // Список брендов дисков
            diskBrandList.x = PART_LIST_MARGIN;
            diskBrandList.y = menuBackground.y;
            diskBrandList.width = wantedWidth - PART_LIST_MARGIN;
            diskBrandList.height = menuHeight;

            if (tuningColorsView) {
                tuningColorsView.y = menuBackground.y;
            }

            if (allPartsColorsView) {
                allPartsColorsView.y = menuBackground.y;
            }

            if(tuningSettingsPanel){
                tuningSettingsPanel.y = menuBackground.y;
            }

            if (backgroundMenuView) {
                backgroundMenuView.y = menuBackground.y;
            }

            arrowImage.y = possiblePartSlotsList.y + possiblePartSlotsList.height;
            arrowImage.visible = false;

            partsMenuButton.x = 6;
            partsMenuButton.y = bottomY;

            colorMenuButton.x = partsMenuButton.x + partsMenuButton.width + BUTTON_GAP;
            colorMenuButton.y = bottomY;

            settingsMenuButton.x = colorMenuButton.x + colorMenuButton.width + BUTTON_GAP;
            settingsMenuButton.y = bottomY;

            shareButton.y = saveButton.y;
            if (Device.isIphoneXVariant()) {
                colorMenuButton.y -= 6;
                partsMenuButton.y -= 6;
                settingsMenuButton.y -= 6;

                rightX -= 6;
                bottomY -= 6;
            }

            gasButton.y = bottomY;
            powerOnButton.y = bottomY;
            revSoundButton.y = bottomY;
            fastModeButton.y = bottomY;
            fastModeHelpButton.y = fastModeButton.y + (fastModeButton.height - fastModeHelpButton.height) / 2;

            if(powerOnButton.visible){
                fastModeButton.x = powerOnButton.x - (10 + fastModeButton.width);
            }else{
                fastModeButton.x = rightX;
            }
            fastModeHelpButton.x = fastModeButton.x - (fastModeHelpButton.width + 5);
            acceptFromCameraButton.y = frameImage.height - colorMenuButton.height - 6.5;
            cameraScreenshotButton.y = acceptFromCameraButton.y;
            shareView.y = shareButton.y + (shareButton.height + BUTTON_MENU_GAP);

            if (Device.isIphoneXVariant()) {
                acceptFromCameraButton.y -= 6;
                cameraScreenshotButton.y -= 6;
            }
            menuBackButton.y = this.menuBackground.y;
            progressBar.width = 100;
            progressBar.x = int(((wantedWidth) - progressBar.width) / 2);
            progressBar.y = frameImage.height - 30;

            progressFlatBar.x = (actualWidth - progressFlatBar.width) / 2;
            updateProgressPosition();
            updateTopButtonsYCoord();
            updateCarViewY();
            if (frameImage.height != carView.frameHeight) {
                carView.frameHeight = frameImage.height;
            }

            try {
                carView.topOffset = 30;
            } catch (e:Error) {
                Tracer.print('error:', e.getStackTrace());
            }

            if (Device.isIphoneXVariant()) {
                updateButtonPositions();
            }
        }

        override protected function completeAnimation():void {

            if (frameImage.height == actualHeight) {

                carView.topOffset = 0;

            } else {

                carView.topOffset = 30;//(_adsButton.y + _adsButton.height + 5);
            }
        }

        public function getCarViewY():Number {

            return int(frameImage.height / 2) + 20;
        }


        public function updateCarViewY():void {
            carView.updateCarY();
        }

        protected override function get handleTexture():Texture {
            return assetManager.getTexture('a_close_menu');
        }


        protected var _menuTexture:Texture;

        protected override function get menuBackgroundTexture():Texture {
            if (!_menuTexture) {
                _menuTexture = Texture.fromColor(10, 100, 0, 0.5);
            }
            return _menuTexture;
        }

        protected override function get menuHeight():int {
            if (Device.isIphoneXVariant()) {
                return 60;
            }

            if (Constants.SCALE_FACTOR == 1.5) {

                return 50;//assetManager.getTexture("a_menu_bg").height*1.15;
            } else {
                return 50;//assetManager.getTexture("a_menu_bg").height;
            }

        }

        public function get diskChangeData():Object {
            return _diskChangeData;
        }

        public function setDiskProperties(slotDisk:PartSlotDisk):Boolean{
            if(!slotDisk){
                if(_diskChangeData && !ObjectUtils.isEmpty(_diskChangeData)){
                    _diskChangeData = {};
                    return true;
                }
                return false;
            }
            if(slotDisk.name.indexOf("front") == -1 && slotDisk.name.indexOf("rear") == -1){
                if(_diskChangeData && !ObjectUtils.isEmpty(_diskChangeData)){
                    _diskChangeData = {};
                }
                return false;
            }
            if(_diskChangeData && _diskChangeData["type"] == slotDisk.name){
                return false;
            }
            var selectedStateName:String = slotDisk.selectedPart.selectedState?slotDisk.selectedPart.selectedState.nameOfState:null;
            _diskChangeData = {};
            _diskChangeData["type"] = slotDisk.name;
            _diskChangeData["name"] = slotDisk.selectedPart.name;
            _diskChangeData["profile"] = PartDisk(slotDisk.selectedPart).profile;
            _diskChangeData["material"] = slotDisk.selectedPart.selectedMaterial.name;
            _diskChangeData["state"] = RegExpUtil.getWheelStraightState(selectedStateName);
            _diskChangeData["color"] = slotDisk.selectedColor.value;
            return true;
        }

        public function isDiskChanged():Boolean{
            if(_diskChangeData && !ObjectUtils.isEmpty(_diskChangeData)) {
                var diskName:String = _diskChangeData["type"];
                var slotDisk:PartSlotDisk = car.getPossiblePartSlot(diskName) as PartSlotDisk;
                var selectedStateName:String = slotDisk.selectedPart.selectedState?slotDisk.selectedPart.selectedState.nameOfState:null;
                selectedStateName = RegExpUtil.getWheelStraightState(selectedStateName);
                return (_diskChangeData["name"] != slotDisk.selectedPart.name ||
                    _diskChangeData["profile"] != PartDisk(slotDisk.selectedPart).profile ||
                    _diskChangeData["material"] != slotDisk.selectedPart.selectedMaterial.name ||
                    RegExpUtil.getWheelStraightState(_diskChangeData["state"]) != selectedStateName ||
                        (_diskChangeData["color"] != slotDisk.selectedColor.value && PartMaterial.isColorable(_diskChangeData["material"]))
                );
            }
            return false;
        }

        public function getDiskChangedData():Object{
            var result:Object = {};
            if(!_diskChangeData || !car){
                return result;
            }
            var partSlotDisk:PartSlotDisk = car.getPossiblePartSlot(_diskChangeData["type"]) as PartSlotDisk;
            if(!partSlotDisk || !partSlotDisk.selectedPart){
                return result;
            }
            var selectedDisk:PartDisk = partSlotDisk.selectedPart as PartDisk;
            if (_diskChangeData["name"] != selectedDisk.name){
                result["name"] = selectedDisk.name;
            }
            if(_diskChangeData["material"] != selectedDisk.selectedMaterial.name){
                result["material"] = selectedDisk.selectedMaterial.name;
            }
            if(_diskChangeData["profile"] != selectedDisk.profile){
                result["profile"] = selectedDisk.profile;
            }
            var selectedState:String = partSlotDisk.selectedPart.selectedState?partSlotDisk.selectedPart.selectedState.nameOfState:null;
            selectedState = RegExpUtil.getWheelStraightState(selectedState);
            if(_diskChangeData["state"] != selectedState){
                result["state"] = selectedState;
            }

            if(PartMaterial.isColorable(_diskChangeData["material"]) &&  _diskChangeData["color"] != partSlotDisk.selectedColor.value){
                result["color"] = partSlotDisk.selectedColor.value;
            }
            return result;
        }

        public function getDiskDifferenceData(partSlotDisk:PartSlotDisk, data:Object):Object{
            var result:Object = {};
            for(var kay:String in data){
                result[kay] = data[kay];
            }
            if(!partSlotDisk || !partSlotDisk.selectedPart){
                return result;
            }
            var selectedDisk:PartDisk = partSlotDisk.selectedPart as PartDisk;
            if (data.hasOwnProperty("name") && data["name"] == selectedDisk.name){
                delete result["name"];
            }
            if(data.hasOwnProperty("material") && data["material"] == selectedDisk.selectedMaterial.name){
                delete result["material"];
            }
            if(data.hasOwnProperty("profile") && data["profile"] == selectedDisk.profile){
                delete result["profile"];
            }
            var selectedState:String = partSlotDisk.selectedPart.selectedState?partSlotDisk.selectedPart.selectedState.nameOfState:null;
            selectedState = RegExpUtil.getWheelStraightState(selectedState);
            if(data.hasOwnProperty("state") && data["state"] == selectedState){
                delete result["state"];
            }

            if(data.hasOwnProperty("color")){
                if((data.hasOwnProperty("material") && !PartMaterial.isColorable(data["material"])) || data["color"] == partSlotDisk.selectedColor.value){
                    delete result["color"];
                }
            }
            return result;
        }

        private function partsListOpenDiskInfoPopUpHandler(event:starling.events.Event):void {
            var packet:Packet = new Packet();
            packet.id = '12344';
            packet.body = '';
            packet.cars = [];
            packet.title = 'Wheel info';
            this.dispatchEventWith(PART_DISK_INFO_POP_UP, false, {packet: packet});
        }

        private function hidePartDiskSettingsView():void {
            partsList.isEnabled = true;
            if (partDiskSettingView) {
                if(partDiskSettingView.visible){
                    SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
                }
                partDiskSettingView.visible = false;
            }
            if (restoreAds) {
                AdsManager.$.bannerShow(BannerScreen.TUNING_SCREEN);
                restoreAds = false;
            }
        }

        private function removePartDiskSettingsView():void {
            if (partDiskSettingView) {
                partDiskSettingView.removeFromParent(true);
                partDiskSettingView = null;
            }
        }


        private function partsScrollEventHandler(event:starling.events.Event):void {
            toggleBanner();
        }

        private function toggleBanner():void {
            if (AdsManager.$.bannerHeight > 1) {
                if (partsList.visible && partsList.dataProvider.length > 0 && partsList.width > 0) {
                    Starling.juggler.delayCall(function ():void {
                        if(!partsList){
                            return;
                        }
                        if (partsList.visible && (partsList.maxHorizontalScrollPosition == 0 || partsList.maxHorizontalScrollPosition <= partsList.horizontalScrollPosition)) {
                            var position:Number = actualHeight - menuHeight + ((menuHeight - AdsManager.$.bannerHeight) / 2);
                            AdsManager.$.setBannerPosition(position * Starling.current.contentScaleFactor, BannerScreen.TUNING_SCREEN);
                            AdsManager.$.bannerShow(BannerScreen.TUNING_SCREEN);
                        } else {
                            AdsManager.$.bannerHide();
                        }
                    }, 0.1);

                } else {
                    AdsManager.$.bannerHide();
                }
            } else {
                AdsManager.$.bannerHide();
            }
        }

        private function partsListOpenPopupInfoHandler(event:starling.events.Event):void {


            var item:PartItemRenderer = event.data.part as PartItemRenderer;

            this.dispatchEventWith(PART_INFO_POP_UP, false, {part: item});


        }


        private var camberDiskSelectedIndex:int = -1;
        private var restoreAds:Boolean = false;

        private function partsListOpenDiskSettingsViewHandler(event:starling.events.Event):void {
            var item:PartDiskItemRenderer = event.data.part as PartDiskItemRenderer;
            hidePartDiskSettingsView();

            if (partDiskSettingView) {
                partDiskSettingView.removeFromParent(true);
                partDiskSettingView = null;
            }

            partDiskSettingView = new TuningDiskSettingsView();
            partDiskSettingView.updateWithLanguage(language);
            partDiskSettingView.assetManager = assetManager;
            //partDiskSettingView.initialHeight = tuningSelectedColorView.height;
            partDiskSettingView.height = 50;
            partDiskSettingView.width = 370;
            partDiskSettingView.x = 0;//(TuningScreen.PART_DISK_WIDTH-partDiskSettingView.width)/2;


            partDiskSettingView.y = this.menuBackground.x;
            partsList.addChild(partDiskSettingView);

            partDiskSettingView.partSlot = selectedPartSlot;
            partsList.isEnabled = false;


            partDiskSettingView.addEventListener(TuningDiskSettingsView.CLOSE_DISK_SETTINGS, closePartDiskSettingsView);
            item.partDiskMenu = partDiskSettingView;

            camberDiskSelectedIndex = partsList.selectedIndex;
            var point:Point = partsList.globalToLocal(item.parent.localToGlobal(new Point(item.x)));
            var maxX:Number = partsList.width - partDiskSettingView.width;
            partDiskSettingView.x = Math.min(maxX, Math.max(0, point.x));

            if (partDiskSettingView.width > item.width) {
                restoreAds = AdsManager.$.bannerHide();
            }
        }

        private function closePartDiskSettingsView(event:starling.events.Event):void {
            hidePartDiskSettingsView();
        }

        private function partsListChangeSelectedPartDataHandler(event:starling.events.Event):void {
            if (event.data && event.data.disk && event.data.disk.part) {
                selectedPart = event.data.disk.part;
            }
            this.partsList.dataProvider.updateItemAt(this.partsList.selectedIndex);


            if (this.carView.progressHandler) {

                this.carView.progressHandler(0);
                this.carView.progressHandler(0.01);
            }

            setTimeout(function ():void {
                dispatchEventWith(PART_SELECTED_DATA_CHAGE);
                if (partDiskSettingView){
                    partDiskSettingView.partSlot = selectedPartSlot;
                }
            }, 200);
        }

        public function setPurchases(value:Array, resetPackages:Boolean):void {
            var idList:Object = {};
            var j:int;
            var haveVipPack: Boolean = false;
            for(j = 0; j < value.length; j++){
                if (value[j].type == Packet.TYPE_ALLPACKET) {
                    haveVipPack = true;
                    break;
                }
            }

            if (car.packets && car.packets.length) {
                for (j = 0; j < car.packets.length; j++) {
                    if (idList.hasOwnProperty(Packet(car.packets[j]).id)) {
                        Packet(car.packets[j]).lock = false;
                    }
                }
            }

            if (car.packet && (idList.hasOwnProperty(car.packet.id) || haveVipPack)) {
                car.packet.lock = false;
            }

            if (packagesView && packagesView.dataProvider) {
                if(value.length == 0 && resetPackages){
                    packagesView.clearPurchases();
                }else{
                    for (var i:int = 0; i < value.length; i++) {
                        packagesView.packetPurchaseUpdate(value[i] as Purchase);
                    }
                }
            }
            packagesView.invalidate();
        }


        public function removePartLock(packet:Packet):void {


            var k:int = 0;
            if (packagesView && packagesView.visible)
                packagesView.hide(true);


            if (packet && !packet.lock) {

                for each (var ps:PartSlot in car.possiblePartSlots) {
                    if (!ps) continue;
                    if (packet.parts && !packet.parts.hasOwnProperty(ps.name) && packet.type != Packet.TYPE_ALLPACKET) {
                        continue;
                    }
                    for (var i:int = 0; i < ps.possibleParts.length; i++) {
                        var item:Part = ps.possibleParts[i];
                        if (packet.type != Packet.TYPE_SUBCOMPONENT) {
                            item.lock = false;
                            trace('removePartLock1');
                        } else {
                            trace('removePartLock!!!:', item.parentSlot.name, packet.parts);
                            if (packet.parts != null && packet.parts[item.parentSlot.name] && packet.parts[item.parentSlot.name].length > 0) {
                                if (packet.parts[item.parentSlot.name].indexOf(item.name) >= 0) {
                                    if (packet.funcName == 'camber') {
                                        item.camberLock = true;
                                        item.lock = false;
                                        trace('removePartLock2');
                                    } else {
                                        item.lock = true;
                                        item.camberLock = false;
                                        trace('removePartLock3');
                                    }
                                } else {
                                    trace('removePartLock4');
                                    item.lock = false;
                                }
                            } else {
                                if (packet.funcName == 'camber') {
                                    trace('removePartLock5');
                                    item.camberLock = false;
                                    item.lock = false;
                                } else {
                                    trace('removePartLock6');
                                    item.lock = false;
                                    item.camberLock = false;
                                }
                            }
                        }
                    }
                    if (packet.funcName == 'camber') {
                        if (this.partDiskSettingView) {
                            trace('removePartLock7');
                            partDiskSettingView.removeCamberLock();
                        }
                    }
                }
            }

            var packets:Array = car.packets;
            var j:int = 0;
            if (packets) {
                for (i = 0; i < packets.length; i++) {
                    packet = packets[i];
                    if (packet && packet.type == Packet.TYPE_SUBCOMPONENT && !packet.lock && packet.funcName == 'standart' && _partsCollection) {
                        for (j = 0; j < _partsCollection.length; j++) {
                            item = _partsCollection.getItemAt(j) as Part;
                            if (packet.slotName) {
                                for (k = 0; k < packet.slotName.length; k++) {
                                    if (item.parentSlot.name != packet.slotName[k]) {
                                        continue;
                                    }

                                    if (packet.parts && packet.parts[packet.slotName[k]] && packet.parts[packet.slotName[k]].length > 0) {
                                        if (packet.parts[packet.slotName[k]].indexOf(item.name) >= 0) {
                                            item.lock = false;
                                            item.lockPacket = null;
                                        }
                                    } else {
                                        item.lock = false;
                                        item.lockPacket = null;
                                    }
                                }
                            }
                        }
                    }
                }
            }

            if (this.partsList) {
                this.partsList.dataProvider = partsCollection;
                if (partsList.dataProvider) {
                    for (i = 0; i < partsList.dataProvider.length; i++) {
                        partsList.dataProvider.updateItemAt(i);
                    }
                }
                partsList.validate();
            }

            if (pendingSelectedPart) {
                partsList.selectedItem = pendingSelectedPart;
                pendingSelectedPart = null;
            }
            enableButtons(true);
            if ((car.packet && car.packet.type != Packet.TYPE_SUBCOMPONENT && car.packet.lock)) {
                //saveButton.//push(saveButton);
                saveButton.alpha = 0.5;
                saveButton.touchable = false;
            }
        }
        //public var lastAerographyPart:Part = null;
        public var lastAerographyParts:Object = {};
        private var partsListChangeOneBlocker:Boolean = false;
        private function getNoMultiselectPart(list:*):Part {
            for each(var part:Part in list) {
                if (!part.multiselect)
                    return part;
            }
            return null;
        }

        //pendingForAuthAction
        public var pendingSelectedPart:Part = null;
        private function setCurrentRadiusDisk(diskPart:PartDisk):void {
            var rad:Number = (diskPart.parentSlot.selectedPart as PartDisk).selectedDiskSize;
            var currentPart:PartDisk;

            var p:Part = diskPart.parentSlotDisk.getPossiblePartForAGivenNameAndRadius(diskPart.name, rad);
            if (p) {
                var partObj:Object = diskPart.parentSlotDisk.findPartDiskForRadius(diskPart.name, rad, PartDisk(diskPart.parentSlotDisk.selectedPart).profile);
                currentPart = partObj ? partObj.part : null;//diskPart.parentSlotDisk.findMainPartDiskByName(diskPart.name);
                if (!currentPart) {
                    var prefix:String = PartSlotDisk.getPrefixDep(diskPart);
                    currentPart = diskPart.parentSlotDisk.findMainPartDiskByName(diskPart.name + prefix);
                }
                if (currentPart) {
                    selectedPart = currentPart;
                    (currentPart as PartDisk).selectedDiskSize = rad;
                }

            }
        }


        private function partsListChangeHandler(event:starling.events.Event):void {
            if (partsList.selectedItem && !partsList.selectedItem.enabled) {
                partsList.removeEventListener(event.type, partsListChangeHandler);
                partsList.selectedItem = selectedPart;
                partsList.addEventListener(event.type, partsListChangeHandler);
                return;
            }
            var i:int;
            if (partsList.selectedItem)


                if (partsListChangeOneBlocker) {
                    partsListChangeOneBlocker = false;
                    return;
                }
            trace('partsListChangeHandler');
            var beforeRemovePartSlot:PartSlot = null;


            if (partsList.dataProvider) {
                for (i = 0; i < partsList.dataProvider.length; i++) {
                    partsList.dataProvider.updateItemAt(i);
                }

            }

            if (partDiskSettingView && partDiskSettingView.visible) {
                partsList.selectedIndex = this.camberDiskSelectedIndex;
                return;
            }


            hidePartDiskSettingsView();


            if (!possiblePartSlotsListChangeFlag && partsList.selectedItem && Part(partsList.selectedItem).lock) {
                SoundManager.playSound(SoundManager.DISABLED_PART);
                pendingSelectedPart = Part(partsList.selectedItem);
                partsList.selectedIndex = -1;
                dispatchEventWith(CAR_PART_LOCK);
                if (infoView && infoView.isOpened) {
                    infoView.hide(true);
                }
                return;
            }


            if (possiblePartSlotsListChangeFlag) {
                selectedPart = Part(partsList.selectedItem);
                if (car.suvTires && selectedPart && Car.DISKS_SLOTS.indexOf(selectedPart.parentSlot.name) >= 0) {
                    setCurrentRadiusDisk(selectedPart as PartDisk);
                }
                return;
            }


            if (!partsList.allowMultipleSelection) {

                if (selectedPart && partsList.selectedItem != selectedPart) {
                    selectedPart.unloadPart();
                }

                selectedPart = Part(partsList.selectedItem);
                if (selectedPart && car.suvTires && Car.DISKS_SLOTS.indexOf(selectedPart.parentSlot.name) >= 0) {
                    setCurrentRadiusDisk(selectedPart as PartDisk);
                }

                if (selectedPart && (!selectedPart.enabled && selectedPart.isDependentlyDisabled)) {
                    partsList.selectedIndex = -1;
                    return;
                }


            } else {
                selectedPart = Part(partsList.selectedItems.pop());

                if (selectedPart) {
                    if (!selectedPart.parentSlot.selectedParts)
                        selectedPart.parentSlot.selectedParts = [];

                    if (selectedPart.parentSlot.name.indexOf('aerography') >= 0)
                        if (selectedPart.parentSlot.selectedParts.length > 0 && selectedPart.parentSlot.name != 'aerography') {
                            if (!selectedPart.multiselect) {
                                var oldPart:Part = getNoMultiselectPart(selectedPart.parentSlot.selectedParts);

                                if (oldPart && oldPart != selectedPart) {
                                    var newSelectedItems:Vector.<Object> = new Vector.<Object>;
                                    newSelectedItems.push(selectedPart);

                                    for (i = 0; i < selectedPart.parentSlot.selectedParts.length; i++) {
                                        if (oldPart != selectedPart.parentSlot.selectedParts[i])
                                            newSelectedItems.push(selectedPart.parentSlot.selectedParts[i]);
                                    }
                                    partsListChangeOneBlocker = true;
                                    partsList.selectedItems = newSelectedItems;
                                    partsList.invalidate();


                                    var newSelectedParts:Array = [];
                                    for (i = 0; i < oldPart.parentSlot.selectedParts.length; i++) {

                                        if (oldPart.parentSlot.selectedParts[i] != oldPart) {

                                            newSelectedParts.push(oldPart.parentSlot.selectedParts[i]);
                                        }
                                    }

                                    oldPart.parentSlot.selectedParts = newSelectedParts;

                                    if (newSelectedParts.length > 0) {
                                        lastAerographyParts[oldPart.parentSlot.name] = newSelectedParts[newSelectedParts.length - 1];
                                    } else {
                                        lastAerographyParts[oldPart.parentSlot.name] = null
                                    }

                                    carView.saveSelectedPart = null;
                                    carView.saveSelectedColor = null;

                                    oldPart.parentSlot.selectedPart = oldPart;
                                    beforeRemovePartSlot = oldPart.parentSlot;
                                }

                            } else {

                                if (lastAerographyParts && lastAerographyParts[selectedPart.parentSlot.name])
                                    selectedPart.parentSlot.selectedPart = lastAerographyParts[selectedPart.parentSlot.name];
                            }

                        }


                    if (selectedPart.parentSlot.selectedParts.indexOf(selectedPart) < 0) {
                        selectedPart.parentSlot.selectedParts.push(selectedPart);

                        lastAerographyParts[selectedPart.parentSlot.name] = selectedPart;
                    }


                    if (!selectedPart.enabled || selectedPart.isDependentlyDisabled) {
                        partsList.selectedIndex = -1;
                        return;
                    }
                }
            }


            if (selectedPart && selectedPart.states && (objectLength(selectedPart.states) > 0)) {
                showTuningSelectedStateView();
            } else {
                //tuningSelectedStateView.visible = false;
            }

            if (selectedPart && selectedPart.parentSlot && selectedPart.parentSlot.isExcluded) {
                return;
            }

            if (selectedPart) {


                if (selectedPart && fastModeEnabled) {

                    selectedPart.isFastModeLoaded = !selectedPart.isFastModeLoaded;
                    carView.isFastModeEnable = selectedPart.isFastModeLoaded; ///setFastModeFrameMap(part.isFastModeLoaded,-1);

                    //if(!part.isFastModeLoaded)
                    if (selectedPart.parentSlot.name.indexOf('aerography') >= 0) {

                    } else {
                        if (carView.fastModeLastSetPartSlot == selectedPart.parentSlot)
                            carView.removePartFromFastModeLoadedByPartSlot(selectedPart.parentSlot);
                    }

                    if (selectedPart.isFastModeLoaded) {
                        carView.fastModeLastSetPartSlot = selectedPart.parentSlot;
                        carView.addPartToFastModeLoaded(selectedPart);
                    } else {
                        carView.removePartFromFastModeLoaded(carView.fastModeLastSetPartSlot.selectedPart);
                        carView.fastModeLastSetPartSlot = null;
                    }

                    updatePartList()

                }


                //if (selectedPart.parentSlot.name == "packs") {
                if (selectedPart.parentSlot.name == 'packs' /*|| selectedPart.parentSlot.name == "aerography" */) {
                    partsList.touchable = false;
                    trace('partsList.touchable = false')
                }

                if (this.carView.progressHandler) {

                    this.carView.progressHandler(0);
                    this.carView.progressHandler(0.01);
                }

                SoundManager.playSound(SoundManager.SELECT_PART);
                var timeoutDelay:int = carView.previewMode?0: 200;
                setTimeout(function (beforeRemovePartSlot2:*):void {
                    dispatchEventWith(PART_SELECTED, false, {beforeRemovePartSlot: beforeRemovePartSlot2});
                }, timeoutDelay, beforeRemovePartSlot);
            }
        }

        private function showTuningSelectedStateView():void {
            // partsList.width = wantedWidth - PART_LIST_MARGIN; //- (partsList.x+tuningSelectedStateView.width);
        }


        private function partsListItemSelectedAgain(event:starling.events.Event):void {

            if (this.partDiskSettingView && partDiskSettingView.visible)
                return;

            if (carView.isLoading) {
                return;
            }
            if (!partsList.allowMultipleSelection) {

                if (selectedPart && selectedPart.parentSlot && !selectedPart.parentSlot.isMandatory) {
                    SoundManager.playSound(SoundManager.SELECT_PART);
                    partsList.selectedIndex = -1;
                    selectedPart = null;
                    dispatchEventWith(PART_UNSELECTED);
                }


            } else {

                var item:PartItemRenderer = event.target as PartItemRenderer;

                //item.

                item.isSelected = false;
                var unselectedPart:Part = partsList.dataProvider.getItemAt(item.index) as Part;


                unselectedPart.parentSlot.selectedPart = unselectedPart;

                var newSelectedParts:Array = [];
                if(unselectedPart.parentSlot.selectedParts){
                    for (var i:int = 0; i < unselectedPart.parentSlot.selectedParts.length; i++) {
                        if (unselectedPart.parentSlot.selectedParts[i] != unselectedPart) {
                            newSelectedParts.push(unselectedPart.parentSlot.selectedParts[i]);
                        }
                    }
                }

                unselectedPart.parentSlot.selectedParts = newSelectedParts;
                if (newSelectedParts.length > 0) {
                    lastAerographyParts[unselectedPart.parentSlot.name] = newSelectedParts[newSelectedParts.length - 1];
                    //lastAerographyPart = newSelectedParts[newSelectedParts.length - 1];
                    //this.partsList.touchable = false;
                } else {
                    lastAerographyParts[unselectedPart.parentSlot.name] = null
                    //lastAerographyPart = null;
                }
                trace('partsListItemSelectedAgain');
                SoundManager.playSound(SoundManager.SELECT_PART);
                selectedPart = null;
                carView.saveSelectedPart = null;
                dispatchEventWith(PART_UNSELECTED);
            }
        }

        private function brandsListItemChangeHandler(event:starling.events.Event):void {
            if (diskBrandList.selectedIndex == -1) {
                return;
            }
            // После появления API possibleParts брать из выбранного бренда
            var possibleParts:Array = [];
            var allBrands:Boolean = (diskBrandList.selectedIndex == 0);
            var diskBrand:DiskBrand = diskBrandList.selectedItem as DiskBrand;//brandCollection.getItemAt(diskBrandList.selectedIndex) as DiskBrand;//Disk.brandList.getItemAt(diskBrandList.selectedIndex) as DiskBrand;
            var manufacturer:String = diskBrand.id;


            var isFactory:Boolean = false;

            if (diskBrandList && diskBrandList.selectedItem && diskBrandList.selectedItem.id == 'factory') {

                isFactory = true;
            }

            var nameList:Object = {};
            for each (var part:Part in selectedPartSlot.possibleParts) {
                if(!nameList.hasOwnProperty(part.name)){
                    nameList[part.name] = true;
                    if (allBrands || (part.manufacturer == manufacturer) || (isFactory && (part.name.indexOf('factory') >= 0 || !part.manufacturer))) {
                        possibleParts.push(part);
                    }
                }
            }

            var sort:Boolean = false;
            if (possibleParts.length > 1) {

                var tmpPart:Part = possibleParts[1];

                if (tmpPart.sortKey != -1) {
                    sort = true;
                }
            }

            if (sort) {
                function orderSortKey(a:Object, b:Object):int {
                    var a_i:int = int(a.sortKey);
                    var b_i:int = int(b.sortKey);

                    if (a_i < b_i) return -1;
                    if (a_i > b_i) return 1;

                    return 0;
                }

                if (possibleParts) {
                    possibleParts.sort(orderSortKey);
                }
            }

            var bannerPart:Part = new Part(selectedPartSlot);
            bannerPart.name = 'banner';
            bannerPart.id = 'banner';
            bannerPart.active = true;
            bannerPart.enabled = false;
            bannerPart.sortKey = 1000000;
            possibleParts.push(bannerPart);

            possibleParts.sortOn(['sortKey', 'folder', 'name'], [Array.NUMERIC, Array.CASEINSENSITIVE, Array.CASEINSENSITIVE]);
            partsCollection = new ListCollection(possibleParts);
            diskBrandList.selectedIndex = -1;
            //
            viewManager.setMode(TuningScreenViewManager.DISK_PARTS_VIEW);
        }

        // Установка видимости списка выбора бренда дисков
        public function setBrandsPanelVisible(value:Boolean):void {
            if (Constants.SHOW_DISK_BRANDS) {
                diskBrandList.visible = value;
                possiblePartSlotsList.visible = !value;
                partsListVisible = !value;
                if(componentSelector){
                    componentSelector.visible = (diskBrandList.visible || partsList.visible);
                }
                // Снимаем выделение выбранного бренда
            } else {
                diskBrandList.visible = false;
                partsListVisible = true;
            }
        }

        public function showBuyPacket(packet:Packet):void {

            backButton.visible = false;
            this.packagesView.showPacket(packet);
        }


        private var possiblePartSlotsListChangeFlag:Boolean = false;

        public var allColorIsChange:Boolean = false;
        public var lastAllColor:uint = 0;
        private var disksBrandsManufacturesAreNull:Boolean = false;

        private function arrayInArray(in1:Array, subject:Array):Boolean {
            var index:int = 0;
            while (subject.indexOf(in1[index]) > -1 && index < in1.length) {
                index++;
            }
            return index >= in1.length;
        }

        public function dependendFilter(possibleParts:Array, selectedPartSlot:PartSlot):Array {


            var newPossibleParts:Array = [];
            var partSlot:PartSlot;
            var part:Part;

            var selectedIds:Array = [];
            var dependence:*;
            var currentPartName:String;
            var i:int, j:int;

            for each (partSlot in car.possiblePartSlots) {
                if (!partSlot)
                    continue;

                if (!partSlot.selectedPart)
                    continue;

                selectedIds.push(partSlot.selectedPart.realId);
            }

            var uniquePartName:Array = [];
            for (i = 0; i < possibleParts.length; i++) {
                part = possibleParts[i];
                if (uniquePartName.indexOf(part.name) > -1) continue;
                uniquePartName.push(part.name);
            }


            var addedPartName:Array = [];

            for (i = 0; i < possibleParts.length; i++) {

                part = possibleParts[i];

                if (addedPartName.indexOf(part.name) > -1) continue;

                if (!part.properties) {

                    addedPartName.push(part.name);
                    newPossibleParts.push(part);

                    part.enabled = true;
                    part.isDependentlyDisabled = false;
                    continue;
                }

                if (!part.properties.dependence) {

                    addedPartName.push(part.name);
                    newPossibleParts.push(part);

                    part.enabled = true;
                    part.isDependentlyDisabled = false;

                    continue;
                }

                dependence = part.properties.dependence;

                if (dependence as String) {

                    if (selectedIds.indexOf(dependence) > -1) {
                        addedPartName.push(part.name);
                        newPossibleParts.push(part);
                        part.enabled = true;
                        part.isDependentlyDisabled = false;
                    }
                } else {

                    //////////////////
                    if (dependence.length <= 0) {
                        addedPartName.push(part.name);
                        newPossibleParts.push(part);
                        part.enabled = true;
                        part.isDependentlyDisabled = false;
                        continue;
                    }/*else{
					addedPartName.push(part.name);
					newPossibleParts.push(part);
					part.enabled = false;
					part.isDependentlyDisabled = true;
					continue;
				}*/
                    //////////////////

                    if (dependence.length == 1) {

                        if (selectedIds.indexOf(dependence[0]) > -1) {
                            addedPartName.push(part.name);
                            newPossibleParts.push(part);
                            part.enabled = true;
                            part.isDependentlyDisabled = false;
                            continue;
                        } else {

                            //addedPartName.push(part.name);
                            //newPossibleParts.push(part);
                            part.enabled = false;
                            part.isDependentlyDisabled = true;
                            continue;
                        }

                    } else if (arrayInArray(dependence, selectedIds)) {
                        addedPartName.push(part.name);
                        newPossibleParts.push(part);
                        part.enabled = true;
                        part.isDependentlyDisabled = false;
                        continue;
                    } else {
                        //addedPartName.push(part.name);
                        //newPossibleParts.push(part);
                        part.enabled = false;
                        part.isDependentlyDisabled = true;
                        continue;
                    }
                }
            }


            for (j = 0; j < uniquePartName.length; j++) {

                var nameIsExist:Boolean = false;
                currentPartName = uniquePartName[j];
                for (i = 0; i < newPossibleParts.length; i++) {
                    part = newPossibleParts[i];
                    if (currentPartName == part.name) {
                        nameIsExist = true;
                        break;
                    }
                }

                if (!nameIsExist) {

                    for (i = 0; i < possibleParts.length; i++) {

                        part = possibleParts[i];

                        if (currentPartName == part.name) {

                            part.enabled = false;
                            part.isDependentlyDisabled = true;
                            newPossibleParts.push(part);
                            break;
                        }
                    }
                }
            }

            return newPossibleParts;
        }

        private function disableDependedPartsToSelect(newPossibleParts:Array):void {
            var possiblePartSlot:PartSlot;
            for each (var part:Part in newPossibleParts) {
                var possiblePartSlotsForDependence:Array = car.possiblePartSlots;
                var isDependentComponentActive:Boolean = false;
                if (part.isDependence) {
                    if (part.dependence is String) {
                        for each (possiblePartSlot in possiblePartSlotsForDependence) {

                            if (possiblePartSlot.selectedPart && possiblePartSlot.selectedPart.id == part.dependence) {
                                isDependentComponentActive = true;
                                break;
                            }
                        }
                    } else if (part.dependence is Array) {
                        for each (possiblePartSlot in possiblePartSlotsForDependence) {
                            for each(var dependenceId:String in part.dependence) {
                                if (possiblePartSlot.selectedPart && possiblePartSlot.selectedPart.id == dependenceId) {
                                    isDependentComponentActive = true;
                                    break;
                                }
                            }
                            if (isDependentComponentActive) break;
                        }
                    } else {
                        isDependentComponentActive = true;
                    }


                    part.isDependentlyDisabled = !isDependentComponentActive;
                } else {
                    part.isDependentlyDisabled = false;
                }
            }

        }


        /**
         * For example, if we want to put mirrors, and the doors aren't installed, we should disable all the mirrors
         *
         *"autoRemoveLinked" indicated those, which are connected tied by linked, and the one without other could not be installed
         * @param possibleParts - all the parts of current selected partSlot
         * @param selectedPartSlot
         */
        public function autoRemoveLinkedDependentFilter(possibleParts:Array, selectedPartSlot:PartSlot):Array {
            var part:Part;
            var parentLinked:Part;
            var isAutoRemoveLinked:Boolean = false;
            var enable:Boolean = true;

            for each(part in possibleParts) {
                if (part.parentLinkedParts && part.parentLinkedParts.length) {
                    for each(parentLinked in part.parentLinkedParts) {
                        if (parentLinked.properties &&
                                parentLinked.properties.hasOwnProperty('autoRemoveLinked') &&
                                parentLinked.properties['autoRemoveLinked']) {
                            isAutoRemoveLinked = true;


                            enable = (parentLinked.parentSlot.selectedPart != null && parentLinked.id == parentLinked.parentSlot.selectedPart.id);

                            if (enable)
                                break;
                        }
                    }

                    if (isAutoRemoveLinked) {
                        break;
                    }
                }
            }

            if (isAutoRemoveLinked && !enable) {
                Tracer.print(selectedPartSlot.name + ' is DISABLED', 'autoRemoveLinkedDependentFilter');
            }
            return [isAutoRemoveLinked, enable];
        }

        public function tireRadiusFilter(possibleParts:Array, selectedPartSlot:PartSlot):void {
            var prefix:String;
            var part:Part;
            if (Car.TIRES_SLOTS.indexOf(selectedPartSlot.name) >= 0) {
                prefix = selectedPartSlot.name.indexOf('tires') >= 0 ? selectedPartSlot.name.replace('tires', '') : selectedPartSlot.name.replace('tire', '')
            } else if (Car.BRAKES_SLOTS.indexOf(selectedPartSlot.name) >= 0) {
                prefix = selectedPartSlot.name.replace('brake', '')
            } else if (Car.TIRE_VINYL_PARTS.indexOf(selectedPartSlot.name) >= 0) {
                prefix = selectedPartSlot.name.replace('tire_vinyl', '')
            } else {
                return;
            }

            var diskSlot:PartSlot = car.getPossiblePartSlot('disk' + prefix);
            if (!diskSlot)
                diskSlot = car.getPossiblePartSlot('disk' + prefix);

            if (diskSlot) {
                var radius:int = (diskSlot.selectedPart as PartDisk).selectedDiskSize;
                for (var i:int = 0; i < possibleParts.length; i++) {
                    part = possibleParts[i];
                    if (part.properties.size == radius) {
                        part.enabled = true;
                    } else {
                        var p:Part = selectedPartSlot.getPossiblePartForAGivenNameAndRadius(part.name, radius);
                        if (p != null) {
                            possibleParts[i] = p;
                            p.enabled = true;
                        } else {
                            part.enabled = false;
                        }
                    }

                }
                for each (part in possibleParts) {
                    part.enabled = (part.properties.size == radius) || selectedPartSlot.getPossiblePartForAGivenNameAndRadius(part.name, radius);
                }
            }
        }

        public function wideFilter(possibleParts:Array, selectedPartSlot:PartSlot):Array {
            if (!car.isChopper) return possibleParts;
            var newPossibleParts:Array = [];
            var partSlot:PartSlot;
            var part:Part;
            for each (part in possibleParts) {
                if (part.properties && part.properties.wide) {
                    for each (partSlot in car.possiblePartSlotsDisks) {
                        if (!partSlot)
                            continue;
                        if (partSlot.name !== 'rear_wheel' && partSlot.name !== 'front_wheel')
                            continue;
                        if (!partSlot.selectedPart)
                            continue;
                        if (partSlot.name == selectedPartSlot.name) {
                            continue;
                        }
                        if (partSlot.selectedPart.wide && part.wide + '' == (partSlot.selectedPart as PartDisk).selectedDiskSize + '') {
                            newPossibleParts.push(part);
                        }
                    }
                } else {
                    newPossibleParts.push(part);
                }
            }


            var addedPartName:Array = [];
            if (newPossibleParts.length <= 0) {
                for each (part in possibleParts) {
                    if (addedPartName.indexOf(part.name) < 0) {
                        newPossibleParts.push(part);
                        addedPartName.push(part.name);
                    }
                }
            }
            return newPossibleParts;
        }

        private static function orderSortKey(a:*, b:*):int {
            var a_i:int = int(a.sortKey);
            var b_i:int = int(b.sortKey);

            if (a_i < b_i) return -1;
            if (a_i > b_i) return 1;

            return 0;
        }

        private function checkDiskChanges(disk:PartSlotDisk):void{
            var isDiskPropsChanged:Boolean = isDiskChanged();
            var prevDiskName:String = diskChangeData?diskChangeData["type"]:null;
            var currDiskName:String = disk?disk.name:null;
            if(currDiskName && currDiskName.indexOf("front") == -1 && currDiskName.indexOf("rear") == -1){
                return;
            }
            if(prevDiskName){
                if(prevDiskName.indexOf("rear") != -1){
                    currDiskName = prevDiskName.replace("rear", "front");
                }else if(prevDiskName.indexOf("front") != -1){
                    currDiskName = prevDiskName.replace("front", "rear");
                }
            }
            var oppDisk:PartSlotDisk = car.getPossiblePartSlot(currDiskName) as PartSlotDisk;
            var diskChangedData:Object = getDiskChangedData();
            var dataDifference:Object = getDiskDifferenceData(oppDisk, diskChangedData);
            var isPartChanged:Boolean = disk?(disk.name != prevDiskName):true;
            if(isDiskPropsChanged && isPartChanged && prevDiskName && !ObjectUtils.isEmpty(dataDifference)){
               //var diskNameStr:String;
                var diskNameStr2:String;
                if(Car.FRONT_DISKS_SLOTS.indexOf(prevDiskName) != -1){
                    //diskNameStr = "front discs";
                    diskNameStr2 = "rear";
                }else if(Car.REAR_DISKS_SLOTS.indexOf(prevDiskName) != -1){
                    //diskNameStr = "rear discs";
                    diskNameStr2 = "front";
                }
                var msg:String = "Apply these changes to diskNameStr2 wheels?";
                // msg = msg.replace('diskNameStr', diskNameStr);
                msg = msg.replace('diskNameStr2', diskNameStr2);
                CustomAlertView.show(msg, "Yes", "No", null, function():void{
                    applyDiskChanges(disk);
                }, function():void{
                    setDiskProperties(disk);
                });
            }else if(isPartChanged){
                setDiskProperties(disk);
            }
        }

        private function applyDiskChanges(currDisk:PartSlotDisk):void{
            // apply opposite disk changes
            var currentSelectedPartSlot:PartSlot = selectedPartSlot;
            var currentSelectedPart:Part = selectedPart;
            //
            var isDiskPropsChanged:Boolean = isDiskChanged();
            var diskChangedData:Object;
            var currName:String;
            var applyHandlerList:Array = [];
            if(isDiskPropsChanged){
                diskChangedData = getDiskChangedData();
            }
            var diskName:String = diskChangeData?diskChangeData["type"]:null;
            if(!diskName || !diskChangedData){
                return;
            }
            if(!currDisk){
                if(diskName.indexOf("front") != -1){
                    currName = diskName.replace("front", "rear");
                }else if(diskName.indexOf("rear") != -1){
                    currName = diskName.replace("rear", "front");
                }
                currDisk = car.getPossiblePartSlot(currName) as PartSlotDisk;
            }else{
                currName = currDisk.name;
            }
            var currSelectedDisk:PartDisk = currDisk.selectedPart as PartDisk;
            var prevDisk:PartSlotDisk = car.getPossiblePartSlot(diskName) as PartSlotDisk;
            var prevSelectedDisk:PartDisk = prevDisk.selectedPart as PartDisk;
            var partName:String = diskChangedData.hasOwnProperty("name")?prevSelectedDisk.name:currSelectedDisk.name;
            var profile:String = diskChangedData.hasOwnProperty("profile")?prevSelectedDisk.profile:currSelectedDisk.profile;
            var currDiskObj:Object = currDisk.findPartDiskForRadius(partName, prevSelectedDisk.radius, profile);
            currSelectedDisk = currDiskObj?currDiskObj.part:null;
            if(!currSelectedDisk){
                currSelectedDisk = currDisk.findPartDisk(partName, prevSelectedDisk.radius.toString(), profile, prevSelectedDisk.folder);
            }
            var applyChangedStateHandler:Function = null;
            if(diskChangedData.hasOwnProperty("state")){
                var rotPatter:RegExp = /-?(trr|trl)/gmi;
                var currStateName:String = currDisk.selectedPart.selectedState?currDisk.selectedPart.selectedState.nameOfState:null;
                var prevStateName:String = prevDisk.selectedPart.selectedState?prevDisk.selectedPart.selectedState.nameOfState:null;
                var matches:Array;
                if(currName.indexOf("front") != -1){
                    matches = currStateName?currStateName.match(rotPatter):[];
                    if(matches.length){
                        if(prevStateName && prevStateName.length && matches[0].indexOf("-") == -1){
                            matches[0] = "-" + matches[0];
                        }else if((!prevStateName || !prevStateName.length) && matches[0].indexOf("-") != -1){
                            matches[0] = matches[0].replace("-", '');
                        }
                        currStateName = ((prevStateName && prevStateName.length)?prevStateName + matches[0]:matches[0]);
                    }else{
                        currStateName = prevStateName;
                    }
                }else if(currName.indexOf("rear") != -1){
                    matches = prevStateName?prevStateName.match(rotPatter):[];
                    if(matches.length){
                        currStateName = prevStateName.replace(rotPatter, '');
                    }else{
                        currStateName = prevStateName;
                    }
                }
                if(currStateName && currStateName.length == 0){
                    currStateName = null;
                }
                if((currStateName && currSelectedDisk.states.hasOwnProperty(currStateName)) || !currStateName) {
                    applyChangedStateHandler = function ():void {
                        selectedPartSlot = currDisk;
                        selectedPart = currSelectedDisk;
                        dispatchEventWith(TuningSelectedStateView.STATE_SELECTED, true, {
                            selectedState: currStateName,
                            part: Part
                        });
                    };
                    applyHandlerList.push(applyChangedStateHandler);
                }
            }
            var currPairName:String;
            if(currDisk.name.indexOf("left") != -1){
                currPairName = currDisk.name.replace("left", "right");
            }else if(currDisk.name.indexOf("right") != -1){
                currPairName = currDisk.name.replace("right", "left");
            }
            if(diskChangedData.hasOwnProperty("color")){
                if(currSelectedDisk.selectedMaterial.name != prevSelectedDisk.selectedMaterial.name){
                    diskChangedData["material"] = prevSelectedDisk.selectedMaterial.name;
                }else{
                    var currPair:PartSlotDisk = car.getPossiblePartSlot(currPairName) as PartSlotDisk;
                    currDisk.selectedColor = prevDisk.selectedColor.copy();
                    currPair.selectedColor = prevDisk.selectedColor.copy();
                    carView.updateWithPartSlots([currDisk, currPair], true);
                }
            }

            var recallMaterial:Boolean = !selectedPartSlot.selectedPart.getPossibleMaterial(prevDisk.selectedPart.selectedMaterial.name);
            if(diskChangedData.hasOwnProperty("name") || diskChangedData.hasOwnProperty("profile")){
                applyHandlerList.push(function():void{
                    selectedPartSlot = currDisk;
                    selectedPart = currSelectedDisk;
                    var eventData:Object = {beforeRemovePartSlot: null}
                    var recallMaterialFunc:Function;
                    if(diskChangedData.hasOwnProperty("material") && recallMaterial){
                        var prevMaterial:PartMaterial = prevDisk.selectedPart.selectedMaterial;
                        recallMaterialFunc = function():void{
                            Starling.juggler.delayCall(function ():void {
                                selectedPartSlot = currDisk;
                                selectedPart = currSelectedDisk;
                                progressBar.visible = false;
                                dispatchEventWith(TuningColorsView.MATERIAL_SELECTED, true, {selectedMaterial: prevMaterial, partSlot: currDisk, selectedColor:prevDisk.selectedColor});
                            }, 2);
                        };
                    }

                    if(applyChangedStateHandler != null){
                        if(recallMaterialFunc != null){
                            eventData["loadCompleteHandler"] = function():void{
                                applyChangedStateHandler();
                                recallMaterialFunc();
                            }
                        }else{
                            eventData["loadCompleteHandler"] = applyChangedStateHandler;
                        }
                    }else if(recallMaterialFunc != null){
                        eventData["loadCompleteHandler"] = recallMaterialFunc;
                    }
                    if(diskChangedData.hasOwnProperty("material")){

                    }
                    dispatchEventWith(PART_SELECTED, false, eventData);
                });
            }
            if(diskChangedData.hasOwnProperty("material") && !recallMaterial){
                applyHandlerList.push(function():void{
                    selectedPartSlot = currDisk;
                    selectedPart = currSelectedDisk;
                    dispatchEventWith(TuningColorsView.MATERIAL_SELECTED, true, {selectedMaterial: prevDisk.selectedPart.selectedMaterial, partSlot: currDisk, selectedColor:prevDisk.selectedColor});
                });
            }
            carView.addEventListener(CarView.ALL_IMAGES_LOADED, commitDiskChangeHandler);
            commitDiskChangeHandler(null);
            function exitCommitChange():void{
                setDiskProperties(currDisk);
                carView.removeEventListener(CarView.ALL_IMAGES_LOADED, commitDiskChangeHandler);
                selectedPartSlot = currentSelectedPartSlot;
                if(currentSelectedPartSlot is PartSlotDisk){
                    setTimeout(function():void{
                        for(var i:int = 0; i < partsList.dataProvider.length; i++){
                            if(partsList.dataProvider.getItemAt(i).name == currSelectedDisk.name){
                                currSelectedDisk = partsList.dataProvider.getItemAt(i) as PartDisk;
                                break;
                            }
                        }
                        if(currSelectedDisk && partsList.dataProvider.contains(currSelectedDisk)){
                            selectedPart = currSelectedDisk;
                            setSelectedPart(currSelectedDisk);
                        }
                    }, 0.2);
                }else{
                    selectedPart = currentSelectedPart;
                }
            }
            function commitDiskChangeHandler(event:starling.events.Event):void{
                trace("applyHandlerList", applyHandlerList.length);
                if(applyHandlerList.length){
                    setTimeout(function():void{
                        if(applyHandlerList.length){
                            applyHandlerList[applyHandlerList.length - 1]();
                            applyHandlerList.pop();
                        }else{
                            exitCommitChange();
                        }
                    }, 0.1);
                }else{
                    exitCommitChange();
                }
            }
        }

        private function possiblePartSlotsListChangeHandler(event:starling.events.Event):void {
            //
            if(possiblePartSlotsList.selectedItem && selectedPartSlot && possiblePartSlotsList.selectedItem != selectedPartSlot){
                SoundManager.playSound(SoundManager.PART_MENU);
            }
            //
            toggleBanner();
            var sP:PartSlot = PartSlot(possiblePartSlotsList.selectedItem);
            var i:int;
            var j:int;
            var part2:Part;
            var tmpPart:Part;
            // Color bug fix
            if(sP && sP.selectedPart && sP.selectedPart.color){
                if(!sP.selectedColor || sP.selectedColor.value != sP.selectedPart.color.value){
                    sP.selectedColor = sP.selectedPart.color.copy();
                }
            }
            //
            if(componentSelector){
                if(sP && !componentSelector.dataProvider.contains(sP)){
                    // componentSelector.dataProvider = possiblePartSlotsList.dataProvider;
                    componentSelector.unshift(sP);
                }
                componentSelector.selectedItem = sP;
            }

            if (sP && sP.name == PartSlot.VINYL) {
                _possibleScrollPosition = possiblePartSlotsList.horizontalScrollPosition;
                possiblePartSlotsList.removeEventListener(starling.events.Event.CHANGE, possiblePartSlotsListChangeHandler);
                possiblePartSlotsList.selectedIndex = -1;
                partsList.visible = diskBrandList.visible = false;
                componentSelector.visible = true;
                switchVinylList(true);
                possiblePartSlotsList.addEventListener(starling.events.Event.CHANGE, possiblePartSlotsListChangeHandler);
                return;
            }

            if (_possibleScrollPosition && sP && PartSlot.VINYLS.indexOf(sP.name) == -1) {
                switchVinylList(false);
            }

            if (possiblePartSlotsList.selectedIndex < 0) {
                return;
            }

            possiblePartSlotsListChangeFlag = true;

            hidePartDiskSettingsView();
            if (selectedPartSlot) {
                if (selectedPartSlot.name.indexOf('aerography') >= 0 && lastAerographyParts[selectedPartSlot.name] != null) {
                    selectedPartSlot.selectedPart = lastAerographyParts[selectedPartSlot.name];
                    if (lastAerographyParts[selectedPartSlot.name].color) {
                        selectedPartSlot.selectedColor = lastAerographyParts[selectedPartSlot.name].color.copy();
                    } else {
                        selectedPartSlot.selectedColor = PartColor.noColor();
                    }
                }
            }
            selectedPartSlot = PartSlot(possiblePartSlotsList.selectedItem);
            if (!selectedPartSlot) {
                return;
            }

            checkDiskChanges(selectedPartSlot as PartSlotDisk);

            // Если это диски, устанавливаем расширенный режим отображения, иначе обычный
            if (Car.DISKS_SLOTS.indexOf(selectedPartSlot.name) >= 0) {
                partsList.itemRendererFactory = diskRenderer;
            } else {
                partsList.itemRendererFactory = normalRenderer;
            }
            var showBrandsPanel:Boolean = false;
            var isDiskSlot:Boolean = Car.DISKS_SLOTS.indexOf(selectedPartSlot.name) >= 0;
            var cond1:Boolean = (selectedPartSlot.selectedPart != null && selectedPartSlot.selectedPart.manufacturer == null);
            var cond2:Boolean = (selectedPartSlot.selectedPart == null);
            var cond3:Boolean = (selectedPartSlot.selectedPart && selectedPartSlot.selectedPart.name && selectedPartSlot.selectedPart.name.indexOf('factory') >= 0);
            if (isDiskSlot && (cond1 || cond2 || cond3)) {
                showBrandsPanel = true;
                if (!disksBrandsManufacturesAreNull) {
                    for (i = 0; i < selectedPartSlot.possibleParts.length; i++) {
                        if (!selectedPartSlot.possibleParts[i].manufacturer || selectedPartSlot.possibleParts[i].manufacturer == null) {
                            showBrandsPanel = false;
                            disksBrandsManufacturesAreNull = true;
                        } else {
                            showBrandsPanel = true;
                            disksBrandsManufacturesAreNull = false;
                            break;
                        }
                    }
                }

            } else {
                //	partsCollection = new ListCollection(selectedPartSlot.possibleParts);
            }

            if (disksBrandsManufacturesAreNull) {
                frontWheelModified = rearWheelModified = true;
            }
            setBrandsPanelVisible(showBrandsPanel);
            if(showBrandsPanel){
                possiblePartSlotsList.removeEventListener(starling.events.Event.CHANGE, possiblePartSlotsListChangeHandler);
                possiblePartSlotsList.selectedIndex = -1;
                possiblePartSlotsList.addEventListener(starling.events.Event.CHANGE, possiblePartSlotsListChangeHandler);
            }
            var reA:RegExp = /[^a-zA-Z]/g;
            var reN:RegExp = /[^0-9]/g;
            var sortAlphaNum:Function = function (a, b):int {
                var aA:String = a.name.replace(reA, '');
                var bA:String = b.name.replace(reA, '');
                aA = aA.replace('factory', '1factory');
                bA = bA.replace('factory', '1factory');


                if (aA === bA) {
                    var aN:int = parseInt(a.name.replace(reN, ''), 10);
                    var bN:int = parseInt(b.name.replace(reN, ''), 10);
                    return aN === bN ? 0 : aN > bN ? 1 : -1;
                } else {
                    return aA > bA ? 1 : -1;
                }
            };


            function orderNameKey(a:*, b:*):int {
                var a_i:int = int(a.name);
                var b_i:int = int(b.name);

                if (a_i < b_i) return -1;
                if (a_i > b_i) return 1;

                return 0;
            }

            if (selectedPartSlot.possibleParts.length > 1) {

                tmpPart = selectedPartSlot.possibleParts[1];

            }

            if (tmpPart && tmpPart.sortKey >= 0) {
                selectedPartSlot.possibleParts.sort(orderSortKey);
            } else {
                selectedPartSlot.possibleParts.sort(sortAlphaNum);
            }

            partsList.allowMultipleSelection = selectedPartSlot.name.indexOf('aerography') >= 0;

            var newPossibleParts:Array = null;
            var disksPartSlot:PartSlotDisk;
            var radius:*;

            if (!disksBrandsManufacturesAreNull && Car.CAR_DISKS_SLOTS.indexOf(selectedPartSlot.name) >= 0 && selectedPartSlot.selectedPart) {
                var possibleParts:Array = [];
                var allBrands:Boolean = (diskBrandList.selectedIndex == 0);
                var manufacturer:String = selectedPartSlot.selectedPart.manufacturer;//diskBrand.id;
                for each (var part:Part in selectedPartSlot.possibleParts) {
                    if (allBrands || (part.manufacturer == manufacturer)) {
                        possibleParts.push(part);
                    }
                }
                newPossibleParts = possibleParts;
            } else {
                newPossibleParts = selectedPartSlot.possibleParts;
            }

            var copyPosibleParts:Array = ArrayUtils.clone(newPossibleParts);
            if (selectedPartSlot.name == 'tires_front') {
                disksPartSlot = car.getPossiblePartSlot('disks_front') as PartSlotDisk;
                radius = (disksPartSlot.selectedPart as PartDisk).selectedDiskSize;
                for (i = 0; i < copyPosibleParts.length; i++) {
                    if (copyPosibleParts[i].properties.size != radius) {
                        copyPosibleParts.splice(i, 1);
                    }
                }
            }
            if (selectedPartSlot.name == 'tires_rear') {
                disksPartSlot = car.getPossiblePartSlot('disks_rear') as PartSlotDisk;
                radius = (disksPartSlot.selectedPart as PartDisk).selectedDiskSize;
                for (i = 0; i < copyPosibleParts.length; i++) {
                    if (copyPosibleParts[i].properties.size != radius) {
                        copyPosibleParts.splice(i, 1);
                    }
                }
            }

            newPossibleParts = copyPosibleParts;
            if (car.componentsDependencies && car.componentsDependencies.length > 0) {
                var ids:Array = car.getDependencyPartIds(selectedPartSlot.name);
                if (ids.length > 0) {
                    var newPossibleParts2:Array = [];
                    for (i = 0; i < ids.length; i++) {
                        for (j = 0; j < newPossibleParts.length; j++) {
                            part2 = newPossibleParts[j];
                            if (part2.states['state_' + ids[i]]) {
                                part2.enabled = true;
                                part2.selectedState = part2.states['state_' + ids[i]];
                            } else {
                                part2.enabled = false;
                            }
                            newPossibleParts2.push(part2);
                        }
                    }
                    newPossibleParts = newPossibleParts2;
                } else {
                    for (j = 0; j < newPossibleParts.length; j++) {
                        part2 = newPossibleParts[j];
                        part2.enabled = true;
                    }
                }
            } else {
                for (j = 0; j < newPossibleParts.length; j++) {
                    part2 = newPossibleParts[j];
                    part2.enabled = true;
                }
            }

            // This part was only for chopper, now we enable it for cars too
            var arr:Array = autoRemoveLinkedDependentFilter(newPossibleParts, selectedPartSlot);
            newPossibleParts = dependendFilter(newPossibleParts, selectedPartSlot);
            newPossibleParts = wideFilter(newPossibleParts, selectedPartSlot);
            tireRadiusFilter(newPossibleParts, selectedPartSlot);

            if (arr[0]) {
                for each (var pppppp:Part in newPossibleParts) {
                    pppppp.enabled = arr[1];
                }
            }


            if (selectedPartSlot.possibleParts.length > 1) {
                tmpPart = selectedPartSlot.possibleParts[1];
            }
            if (car.isChopper) {
                selectedPartSlot.possibleParts.sort(orderNameKey);
            } else {
                if (tmpPart && tmpPart.sortKey >= 0) {
                    trace('orderSortKey');
                    selectedPartSlot.possibleParts.sort(orderSortKey);
                } else {
                    trace('sortAlphaNum');
                    selectedPartSlot.possibleParts.sort(sortAlphaNum);
                }
            }
            var bannerPart:Part = new Part(selectedPartSlot);
            bannerPart.name = 'banner';
            bannerPart.id = 'banner';
            bannerPart.sortKey = 10000000;
            bannerPart.active = true;
            bannerPart.enabled = false;
            newPossibleParts.push(bannerPart);

            // newPossibleParts.sort(orderSortKey);
            newPossibleParts.sortOn(['sortKey', 'folder', 'name'], [Array.NUMERIC, Array.CASEINSENSITIVE, Array.CASEINSENSITIVE]);
            partsCollection = new ListCollection(newPossibleParts);
            this.partsList.dataProvider = partsCollection;


            if (!partsList.allowMultipleSelection) {
                if (selectedPartSlot.selectedPart) {
                    if (selectedPartSlot.name != 'windows') {
                        if (Car.DISKS_SLOTS.indexOf(selectedPartSlot.name)) {
                            for (j = 0; j < partsList.dataProvider.length; j++) {
                                var p:Part = partsList.dataProvider.getItemAt(j) as Part;
                                if (p && p.name == selectedPartSlot.selectedPart.name) {
                                    partsList.selectedIndex = j;
                                    break;
                                }
                            }
                        } else {
                            partsList.selectedItem = selectedPartSlot.selectedPart;
                        }
                    } else {
                        var selectedIndex:int = -1;
                        for (var k:int = 0; k < this.partsCollection.length; k++) {
                            var item:Part = partsCollection.getItemAt(k) as Part;
                            if (item.brightness == selectedPartSlot.selectedPart.brightness) {
                                selectedIndex = k;
                                break;
                            }
                        }
                        partsList.selectedIndex = selectedIndex;
                    }
                    if (partsList.selectedIndex != -1) {
                        partsList.scrollToDisplayIndex(partsList.selectedIndex);
                    }
                } else {
                    partsList.selectedIndex = -1;
                }
            } else {
                if (selectedPartSlot.selectedParts) {
                    var selectedIndices:Vector.<int> = new <int>[];
                    for (var selectedPartIndex:int in selectedPartSlot.selectedParts) {
                        var selectedPartItem:Part = selectedPartSlot.selectedParts[selectedPartIndex];
                        var index:int = partsList.dataProvider.getItemIndex(selectedPartItem);
                        selectedIndices.push(index);
                    }

                    partsList.selectedIndices = selectedIndices;
                } else if (selectedPartSlot.selectedPart) {
                    partsList.selectedIndex = partsList.dataProvider.getItemIndex(selectedPartSlot.selectedPart);
                } else {
                    partsList.selectedIndices = null;
                }
            }

            if (!selectedPartSlot.dummy) {
                if (!selectedPartSlot.isMandatory && allColorIsChange) {
                    if (!selectedPartSlot.selectedPart) {
                        selectedPartSlot.selectedColor = PartColor.noColor();
                        selectedPartSlot.selectedColor.value = lastAllColor;
                    }
                }


                if (selectedPart && selectedPart.states && (objectLength(selectedPart.states) > 0)) {
                    showTuningSelectedStateView();
                }
                allPartsColorsView.partSlot = selectedPartSlot;
                allPartsColorsView.partColors = car.possiblePartColors;
                tuningSettingsPanel.car = car;
                showColorMenuPanel(!(!allPartsColorsView.visible && !backgroundMenuView.visible));
            } else {
                allPartsColorsView.partSlot = selectedPartSlot;
                allPartsColorsView.partColors = car.possiblePartColors;
                tuningSettingsPanel.car = car;
                showColorMenuPanel(!(!allPartsColorsView.visible && !backgroundMenuView.visible));
            }

            if (initialScrollEventPassed && !partSlotSelectedByCarTouch) {
                moveArrowImageToSelectedIndex(possiblePartSlotsList.selectedIndex, true);
            }

            if (partsList.selectedIndex >= 3) {
                partsList.scrollToDisplayIndex(partsList.selectedIndex, 0);
            }
            initialScrollEventPassed = true;
            dispatchEventWith(PART_SLOT_SELECTED, false, {selectedByCarTouch: partSlotSelectedByCarTouch});
            possiblePartSlotsListChangeFlag = false;
            possiblePartSlotsList.selectedIndex = -1;

            // ********************************
            brandListCollection ||= {};
            if(!brandListCollection.hasOwnProperty(selectedPartSlot.name)){
                var manufacturers:Array = Disk.brandList.data as Array;
                var manufacturerList:Array = [];
                if(manufacturers.length >= 2){
                    manufacturerList.push(manufacturers[0]);
                    manufacturerList.push(manufacturers[1]);
                }
                for(i = 0; i < selectedPartSlot.possibleParts.length; i++){
                    part2 = selectedPartSlot.possibleParts[i] as Part;
                    if(part2.manufacturer){
                        for(j = 0; j < manufacturers.length; j++){
                            if(manufacturers[j].id == part2.manufacturer && manufacturerList.indexOf(manufacturers[j]) == -1){
                                manufacturerList.push(manufacturers[j]);
                            }
                        }
                    }
                }
                if(manufacturerList.length > 2){
                    brandListCollection[selectedPartSlot.name] = manufacturerList;
                }else{
                    brandListCollection[selectedPartSlot.name] = null;
                }
            }
            if(!showBrandsPanel && brandListCollection[selectedPartSlot.name] && !(selectedPartSlot is PartSlotDisk)){
                diskBrandList.dataProvider = new ListCollection(brandListCollection[selectedPartSlot.name]);
                viewManager.setMode(TuningScreenViewManager.DISK_BRANDS_VIEW);
            }else{
                diskBrandList.dataProvider = brandCollection;
            }
        }


        private function objectLength(myObject:Object):int {
            var cnt:int = 0;

            for (var s:String in myObject) cnt++;

            return cnt;
        }

        private function possiblePartSlotsListScrollHandler(event:starling.events.Event):void {
            moveArrowImageToSelectedIndex(possiblePartSlotsList.selectedIndex, false);
        }

        private function moveArrowImageToSelectedIndex(index:int, animated:Boolean = true):void {
            var selectedPartSlot:PartSlot = PartSlot(possiblePartSlotsList.selectedItem);

            if (!selectedPartSlot) {
                return;
            }

            var itemWidth:int = partSlotNameWidthsMap[selectedPartSlot.name];
            var prevSlotsWidths:int = 0;

            for (var i:int = 0; i < possiblePartSlotsList.selectedIndex; i++) {
                var partSlot:PartSlot = possiblePartSlots[i];
                prevSlotsWidths += partSlotNameWidthsMap[partSlot.name];
            }

            var newX:int = -int(possiblePartSlotsList.horizontalScrollPosition) + prevSlotsWidths + itemWidth / 2;

            if (animated) {
                var tween:Tween = new Tween(arrowImage, 0.2, Transitions.EASE_OUT);
                tween.animate('x', newX);
                Starling.juggler.add(tween);
            } else {
                arrowImage.x = newX;
            }
        }

        private function settingsButtonTriggeredHandler(event:starling.events.Event):void {
            dispatchEventWith(SETTINGS_SELECTED);
        }

        public function updateNotifications():void {
            if (mainMenuView) {
                mainMenuView.updateNotificationCount();
            }
        }

        private var powerSoundStopPosition: Number = 0;

        private function powerOnButtonButtonTriggeredHandler(event: starling.events.Event): void {
            if (startAudioChannel) {
                startAudioChannel.removeEventListener(flash.events.Event.SOUND_COMPLETE, onPowerSoundComplete);
            }
            if (powerOn) {
                destroyRevSound();
                if (startAudioChannel) {
                    startAudioChannel.stop();
                    startAudioChannel = null;
                }

                if (downAudioChannel) {
                    downAudioChannel.stop();
                }

                if (gasAudioChannel) {
                    gasAudioChannel.stop();
                }
                gasButton.visible = false;
                revSoundButton.visible = false;
                powerOn = false;
            } else {
                startAudioChannel = startAudio.play();
                startAudioChannel.addEventListener(flash.events.Event.SOUND_COMPLETE, onPowerSoundComplete);
                powerOn = true;
                if(car.hasSoundRevs){
                    revSoundButton.visible = true;
                    revSoundButton.isSelected = false;
                }else{
                    gasButton.visible = true;
                }
            }
            fastModeButton.visible = fastModeHelpButton.visible = !powerOn;
        }

        private function revSoundButtonHandler(event: starling.events.Event): void {
            if(startAudioChannel){
                restorePosition = startAudioChannel.position;
            }
            stopPowerSound();
            destroyRevSound();
            if(revSoundButton.isSelected) {
                revSoundChannel = revSound.play();
                revSoundChannel.addEventListener(flash.events.Event.SOUND_COMPLETE, revSoundCompleteHandler);
            }else{
                restorePowerSound();
            }
        }

        private function stopPowerSound():void{
            if (startAudioChannel) {
                startAudioChannel.removeEventListener(flash.events.Event.SOUND_COMPLETE, onPowerSoundComplete);
                startAudioChannel.stop();
                startAudioChannel = null;
            }

            if (downAudioChannel) {
                downAudioChannel.stop();
            }

            if (gasAudioChannel) {
                gasAudioChannel.stop();
            }
        }

        private function restorePowerSound():void{
            powerOnButton.isSelected = true;
            startAudioChannel = startAudio.play(restorePosition);
            startAudioChannel.addEventListener(flash.events.Event.SOUND_COMPLETE, onPowerSoundComplete);
            powerOn = true;
        }

        private function destroyRevSound():void{
            if(revSoundChannel){
                revSoundChannel.stop();
                revSoundChannel.removeEventListener(flash.events.Event.SOUND_COMPLETE, revSoundCompleteHandler);
                revSoundChannel = null;
            }
        }

        private function revSoundCompleteHandler(e: flash.events.Event): void {
            revSoundButton.isSelected = false;
        }

        private function onPowerSoundComplete(e: flash.events.Event): void {
            e.currentTarget.removeEventListener(e.type, onPowerSoundComplete);
            powerOn = false;
            powerOnButton.isSelected = false;
            gasButton.visible = false;
            revSoundButton.visible = false;
        }
        private var startTimeout: *;
        private var gazTimeout: *;
        private function onGazButton(e: TouchEvent): void {
            if (gasAudioChannel) {
                gasAudioChannel.removeEventListener(flash.events.Event.SOUND_COMPLETE, onUpSoundComplete);
            }
            if(!powerOn){
                return;
            }
            var touch: Touch = e.getTouch(gasButton);
            if (touch && touch.phase == TouchPhase.BEGAN) {
                if (gasAudioChannel) {
                    gasAudioChannel.stop();
                }
                gasAudioChannel = gasAudio.play(0.3);
                if (startAudioChannel) {
                    powerSoundStopPosition = startAudioChannel.position;
                    startAudioChannel.stop();
                    startAudioChannel.removeEventListener(flash.events.Event.SOUND_COMPLETE, onPowerSoundComplete);
                    startAudioChannel = null;
                }

                if (downAudioChannel) {
                    downAudioChannel.stop();
                }

                if (gasAudioChannel){
                    gasAudioChannel.addEventListener(flash.events.Event.SOUND_COMPLETE, onUpSoundComplete)
                }

            } else if (touch &&  touch.phase === TouchPhase.ENDED) {
                downAudioChannel = downAudio.play(0.3);
                if (startAudioChannel) {
                    powerSoundStopPosition = startAudioChannel.position;
                    startAudioChannel.stop();
                    startAudioChannel.removeEventListener(flash.events.Event.SOUND_COMPLETE, onPowerSoundComplete);
                    startAudioChannel = null;
                }

                if (gazTimeout) {
                    clearTimeout(gazTimeout);
                    gazTimeout = null;
                }
                gasAudioChannel.stop();
                gasAudioChannel = null

            }
        }

        private function onUpSoundComplete(e: flash.events.Event): void {
            downAudioChannel.removeEventListener(flash.events.Event.SOUND_COMPLETE, onUpSoundComplete);
            downAudioChannel = downAudio.play();

        }

        private function onEnterFrame(e: starling.events.Event): void {
            if (downAudioChannel && downAudioChannel.position > 0) {
                var percent:Number =  1 - (downAudioChannel.position / downAudio.length);
                var volumeAdjust: SoundTransform = new SoundTransform();
                if (percent > 0.8) {
                    percent = 1;
                }
                volumeAdjust.volume = percent;
                downAudioChannel.soundTransform = volumeAdjust;
                if ((downAudio.length - downAudioChannel.position) <=  250 && (downAudio.length - downAudioChannel.position) >  0) {
                    if (!startAudioChannel) {
                        startAudioChannel = startAudio.play(powerSoundStopPosition);
                        startAudioChannel.addEventListener(flash.events.Event.SOUND_COMPLETE, onPowerSoundComplete);
                    }
                    downAudioChannel.stop();
                    downAudioChannel = null;

                }
            }
        }

        private function fastModeButtonChangedHandler(event:starling.events.Event):void {
            if(carView){
                carView.previewMode = fastModeButton.isSelected;
            }
            if(!carView.previewMode){
                SoundManager.playSound(SoundManager.SELECT_PART_BACK);
                dispatchEvent(new starling.events.Event(EXIT_FAST_MODE));
            }else{
                SoundManager.playSound(SoundManager.SELECT_PART);
                carView.setFastModeFullLoaded();
            }
        }

        private function fastModeHelpButtonChangedHandler(event:starling.events.Event):void {
            if(fastModeHelpView){
                // SoundManager.playSound(SoundManager.SELECT_PART_BACK);
                fastModeHelpView.close();
                fastModeHelpView = null;
            }else{
                SoundManager.playSound(SoundManager.SELECT_PART);
                fastModeHelpView = FastModeHelpView.show("FAST MODE ENABLES FASTER LOADING OF THE TUNING PARTS!", "When turned 'On', the car can be rotated only by selecting components in the Part Menu or by selecting a certain part of the car. When done browsing through tuning parts, turn 'Off' the fast mode to enable free rotation of the car.", function():void{
                    SoundManager.playSound(SoundManager.SELECT_PART_BACK);
                    fastModeHelpView = null;
                });
            }
        }

        // AR and screenshot event handlers
        private function cameraButtonTriggeredHandler(event:starling.events.Event):void {
            var isVisible:Boolean = true;
            if(currentScreenshotButton.alpha == 1){
                isVisible = false;
            }
            if(arButton.visible){
                SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
            }else{
                SoundManager.playSound(SoundManager.PART_MENU);
            }
            arButton.alpha = currentScreenshotButton.alpha = 1;
            arButton.visible = currentScreenshotButton.visible = !arButton.visible || isVisible;
            if(car.hdsize){
                if(arButton.visible){
                    setVideoMenuVisibility(true);
                }else{
                    setVideoMenuVisibility(false);
                }
            }

            shareView.visible = false;
        }

        private function arButtonTriggeredHandler(event:starling.events.Event):void {
            SoundManager.playSound(SoundManager.PART_MENU);
            backgroundMenuView.dispatchEventWith(TuningBackgroundMenuView.SNAPSHOT_CAMERA_SELECTED);
        }

        private function currentScreenshotButtonTriggeredHandler(event:starling.events.Event):void {
            SoundManager.playSound(SoundManager.SELECT_PART);
            dispatchEventWith(PHOTO_BUTTON_SELECTED);
        }
        // end of AR and screenshot event handlers

        private function setVideoMenuVisibility(value:Boolean):void {
            var newIcon:Image = cameraButton.getChildByName("new_icon") as Image;
            if(newIcon){
                newIcon.visible = car.hdsize && !value;
            }
            if(value){
                updateScreenshotButtons();
                saveVideoPreviewButton1.x = currentScreenshotButton.x;
                saveVideoPreviewButton1.y = saveVideoPreviewButton1 .height + 10 + currentScreenshotButton.y;
                saveVideoPreviewButton2.x = currentScreenshotButton.x;
                saveVideoPreviewButton2.y = saveVideoPreviewButton2.height + 10 + saveVideoPreviewButton1.y;
            }
            saveVideoPreviewButton1.visible = saveVideoPreviewButton2.visible = value;
        }

        private function saveVideoButtonTriggeredHandler(event:starling.events.Event):void {
            cameraButtonTriggeredHandler(null);
            //
            var videoSettings:File = File.applicationStorageDirectory.resolvePath("video_settings.json");
            var videoDataStr:String;
            if(videoSettings.exists){
                videoDataStr = FileUtil.fileGetContent(videoSettings);
            }else{
                videoDataStr = "";
            }
            var videoData:Object;
            var today:Date = new Date();
            var lastDate:Date;
            if(videoDataStr && videoDataStr.length){
                videoData = JSON.parse(videoDataStr);
                lastDate = DateUtil.parseW3CDTF(videoData.date);
            }else{
                videoData = {date: DateUtil.toW3CDTF(today), count: 0};
                lastDate = today;
            }
            // videoData.count = 0;
            if(lastDate.fullYear != today.fullYear || lastDate.month != today.month || lastDate.date != today.date){
                videoData = {date: DateUtil.toW3CDTF(today), count: 0};
            }

            if(videoData.count >= 10){
                AlertView.show("Sorry, you have reached the daily limit of 10 videos.\nBut you will be able to make more videos tomorrow!", -1, false);
                //return;
            }
            dispatchEventWith(SAVE_PROMO_VIDEO_SELECTED, false, {button:event.currentTarget, settingsFile:videoSettings, videoData: videoData});
        }

        public function getVideoData(selectedButton:Button, completeHandler:Function):void{
            CameraRollExtended.service.requestAccess();
            removeVideoSpinner();
            var frames:Array;
            var theme:String;
            if(selectedButton == saveVideoPreviewButton1){
                frames = [[0, 0], [1, 0], [0, 2 * 10], [0, 2 * 6], [0, 2 * 2], [0, 2 * 18], [0, 2 * 20]];
                theme = "Template1";
            }else if(selectedButton == saveVideoPreviewButton2){
                frames = [[0, 2 * 18], [0, 2 * 12], [0, 0]];
                theme = "Template2";
            }
            var snapshots:Array = [];
            var currAngle:int = carView.currentAngelIndex;
            var currIndex:int = carView.currentFrame;
            var i:int = 0;
            var tempImage:Image = new Image(Texture.fromBitmapData(carView.copyToBitmapAll()));
            tempImage.width = carView.width;
            tempImage.scale = tempImage.scaleX;
            addChildAt(tempImage, getChildIndex(carView) + 1);
            var frameCreateFunc:Function = function():void{
                var bd1:BitmapData = carView.createFrameImage(frames[i][0], frames[i][1]);
                var png1:ByteArray = PNGEncoder.encode(bd1);
                var b641:String = Base64.encode(png1);
                snapshots.push({snapshot:b641});
                png1.clear();
                bd1.dispose();
                i++;
                if(i < frames.length){
                    Starling.juggler.delayCall(frameCreateFunc, 0.1);
                }else{
                    tempImage.texture.dispose();
                    tempImage.removeFromParent(true);

                    carView.showFrame(currIndex, currAngle);
                    completeHandler({snapshots: snapshots, theme: theme});
                }
            };
            frameCreateFunc();
        }

        private function saveButtonTriggeredHandler(event:starling.events.Event):void {
            SoundManager.playSound(SoundManager.PART_MENU);
            dispatchEventWith(SAVE_TUNING_SELECTED);
        }

        public function addVideoSpinner():void{
            removeVideoSpinner();
            savingAlert = new SpinnerView();
            savingAlert.name = "video_spinner";
            cameraButton.addChildAt(savingAlert, Math.max(cameraButton.numChildren - 2, 0));
        }

        public function removeVideoSpinner():void{
            savingAlert = cameraButton.getChildByName("video_spinner") as SpinnerView;
            if(savingAlert){
                savingAlert.hide();
                savingAlert = null;
            }
        }

        private function newFbSharingTriggeredHandler(event:starling.events.Event):void {
            //this.carView.ImageForVideo.testCamera();
            dispatchEventWith(SHARE_FB_SELECTED, false, {challenge: true});
        }

        private function photoButtonTriggeredHandler(event:starling.events.Event):void {
            //this.carView.ImageForVideo.testCamera();
            dispatchEventWith(PHOTO_BUTTON_SELECTED);
        }

        public function get colorSelectionOpened():Boolean {
            return true;
        }

        public function closeColorSelection():void {
            var tween:Tween = new Tween(tuningColorsView, 0.3);
            tween.animate('x', -tuningColorsView.width);
            if (selectedPart && selectedPart.states && (objectLength(selectedPart.states) > 0)) {
                showTuningSelectedStateView();
            }

            tween.onComplete = function ():void {
                removeChild(tuningColorsView);
                tuningColorsView = null;
            };
            Starling.juggler.add(tween);
        }

        public function openColorSelection():void {

        }

        public function lockMenu(__lock:Boolean):void {
            locked = __lock;
        }

        protected override function menuHidden():void {
            super.menuHidden();
            updateMenuVisible(false);
            AdsManager.$.bannerHide();
            if(carView.visible && carLoaded){
                BannerStackedScreen.showAdBanner();
            }
        }

        protected override function menuShown():void {
            super.menuShown();
            AdsManager.$.bannerHide();
            updateMenuVisible(true);
            toggleBanner();
        }

        private function updateMenuVisible(menuVisible:Boolean):void {

        }

        public function updateMenuListWidth(wt:Number):void {
            _wantedWidth = wt;

            if (frameImage) {
                frameImage.width = wt;
            }
            this.layoutMenuViews();

            invalidate(INVALIDATION_FLAG_SIZE);
        }

        public function set colorsViewEnabled(value:Boolean):void {
            _colorsViewEnabled = value;
            invalidate('colorsViewEnabled');
        }

        public function get colorsViewEnabled():Boolean {
            return _colorsViewEnabled;
        }

        public function updateVisibleShaving():void {

        }

        private function objectsForHideBeforeScreenShot(includeBackButton:Boolean = true):Array {
            var objects:Array = [
                colorMenuButton, partsMenuButton, settingsMenuButton,
                menuBackground, frameImage, possiblePartSlotsList,
                partsList, handleBarImage, handleOpenBarImage,
                saveButton, fastModeButton, fastModeHelpButton, cameraButton,
                shareButton,
                powerOnButton, gasButton
            ];

            if (includeBackButton) {
                objects.push(backButton);
            }

            if (tuningColorsView) {
                objects.push(tuningColorsView);
            }

            if (allPartsColorsView) {
                objects.push(allPartsColorsView);
            }

            if (arrowImage) {
                objects.push(arrowImage);
            }

            return objects;
        }

        public function beginScreenShot(additionalViewsToHide:Array, resultHandler:Function = null, touchWhileInFullscreen:Boolean = false, hideBackButton:Boolean = true, uiTouchable:Boolean = false):void {
            if (!uiTouchable) {
                touchable = false;
                carView.touchable = false;
            }

            menuAlwaysOpen = true;
            finishedScreenShotTweens = 0;

            var objects:Array = objectsForHideBeforeScreenShot(hideBackButton);
            var arr:Array = [arButton, currentScreenshotButton];
            if(car.hdsize){
                arr.push(saveVideoPreviewButton1, saveVideoPreviewButton2);
            }
            for(var i:int = 0; i < arr.length; i++){
                arr[i].name = "0";
            }
            objects = objects.concat(arr);
            if (additionalViewsToHide) {
                objects = objects.concat(additionalViewsToHide);
            }

            shareButton.touchable = false;
            saveButton.touchable = false;
            cameraButton.touchable = false;
            fastModeButton.touchable = false;
            fastModeHelpButton.touchable = false;

            powerOnButton.touchable = false;
            gasButton.touchable = false;


            //instructionButton.touchable = false;

            this.closeMenu(false);

            for each(var displayObject:DisplayObject in objects) {
                displayObject.name = displayObject.alpha.toString();
                var tween:Tween = new Tween(displayObject, 0.3, Transitions.EASE_IN_OUT);
                //(Math.random()*10000000000).toFixed(0);
                //maxAlphaForHiddenView[displayObject.name] = displayObject.alpha;
                tween.fadeTo(0);
                tween.onComplete = function ():void {
                    //if(handleBarImage)
                    //handleBarImage.visible = false;

                    finishedScreenShotTweens++;

                    if (resultHandler && finishedScreenShotTweens == objects.length) {
                        resultHandler();
                    }
                }

                Starling.juggler.add(tween);
            }
        }

        public function endScreenShot(additionalViewsToShow:Array = null, delay:Number = 0.5, incaludeBackButton:Boolean = true, resetNames:Boolean = false):void {
            //if(handleBarImage)
            //handleBarImage.visible = true;
            menuAlwaysOpen = false;
            var objects:Array = objectsForHideBeforeScreenShot(incaludeBackButton);
            if (additionalViewsToShow) {
                objects = objects.concat(additionalViewsToShow);
            }

            if(resetNames){
                for(var i:int = 0; i < objects.length; i++){
                    objects[i].name = "1";
                }
            }

            touchable = true;
            carView.touchable = true;
            shareButton.touchable = true;
            saveButton.touchable = true;
            cameraButton.touchable = true;
            arButton.touchable = true;
            currentScreenshotButton.touchable = true;
            if(car.hdsize){
                saveVideoPreviewButton1.touchable = true;
                saveVideoPreviewButton2.touchable = true;
            }
            fastModeButton.touchable = true;
            fastModeHelpButton.touchable = true;

            powerOnButton.touchable = true;
            gasButton.touchable = true;

            for each(var displayObject:DisplayObject in objects) {
                var tween:Tween = new Tween(displayObject, 0.3, Transitions.EASE_IN_OUT);

                tween.delay = delay;
                tween.fadeTo(parseFloat(displayObject.name));

                Starling.juggler.add(tween);
            }
        }

        public function updateSelectedColorForPartSlot(partSlot:PartSlot):void {

        }

        private function downloadsButtonTriggeredHandler(event:starling.events.Event):void {
            dispatchEventWith(DOWNLOADS_BUTTON_SELECTED);
        }

        private var carViewPartSlotSelectedHandlerStart:Boolean = false;
        private function carViewPartSlotSelectedHandler(event:starling.events.Event):void {
            Starling.juggler.delayCall(toggleBanner, .1);
            var partSlot:PartSlot = event.data.partSlot;

            if (partSlot.name == 'right_windows') {
                partSlot = car.getPossiblePartSlot('left_windows');
            }

            if (partSlot.name == 'right_doors') {
                partSlot = car.getPossiblePartSlot('left_doors');
            }

            if (partSlot.name == 'mirror_right') {
                partSlot = car.getPossiblePartSlot('mirror_left');
            }

            if (car.suvTires){ // cause we combine these types of part slots, so we choose basis partSlot
                switch (partSlot.name) {
                    case 'disk_front_right':
                        partSlot = car.getPossiblePartSlot('disk_front_left');
                        break;
                    case 'disk_rear_right':
                        partSlot = car.getPossiblePartSlot('disk_rear_left');
                        break;
                    case 'brake_front_right':
                        partSlot = car.getPossiblePartSlot('brake_front_left');
                        break;
                    case 'brake_rear_right':
                        partSlot = car.getPossiblePartSlot('brake_rear_left');
                        break;
                    case 'tire_rear_left':
                    case 'tire_front_right':
                    case 'tire_rear_right':
                        partSlot = car.getPossiblePartSlot('tire_front_left');
                        break;
                    case 'tire_vinyl_front_right':
                    case 'tire_vinyl_rear_left':
                    case 'tire_vinyl_rear_right':
                        partSlot = car.getPossiblePartSlot('tire_vinyl_front_left');
                        break;
                }

            }

            var partIndex:int = _possiblePartSlots.indexOf(partSlot);
            if(partIndex == -1) {
                removeExtraItems();
                possiblePartSlotsList.dataProvider = new ListCollection([partSlot]);
                possiblePartSlotsList.selectedItem = partSlot;
                componentSelector.unshift(partSlot);
                /*if(!componentSelector.dataProvider.contains(partSlot)){
                    componentSelector.dataProvider = possiblePartSlotsList.dataProvider;
                }*/
                if (!menuBackground.visible) {
                    openMenu();
                }
                return;
            }
            possiblePartSlotsList.dataProvider = new ListCollection(_possiblePartSlots);
            componentSelector.dataProvider = possiblePartSlotsList.dataProvider;



            if (carViewPartSlotSelectedHandlerStart)
                return;

            if (partSlot.name.indexOf('aerographyAddState') >= 0) {
                return;
            }

            carViewPartSlotSelectedHandlerStart = true;
            partSlotSelectedByCarTouch = true;
            if (!possiblePartSlotsList.dataProvider.contains(partSlot)) {
                if (_vinylCollection.contains(partSlot)) {
                    _possibleScrollPosition = 0;
                    switchVinylList(true);
                } else {
                    switchVinylList(false);
                }
            }


            if (partSlot.name.indexOf('aerography') >= 0 || PartSlot.VINYLS.indexOf(partSlot.name) != -1) {
                this.selectedPart = carView.saveSelectedPart;
                //partMaterialImage.parentMaterial.parentPart.parentSlot.selectedPart=partMaterialImage.parentMaterial.parentPart;

                if (partSlot.selectedPart && partSlot.selectedPart.color)
                    partSlot.selectedColor = partSlot.selectedPart.color.copy();


                if (possiblePartSlotsList.selectedItem && possiblePartSlotsList.selectedItem.name.indexOf('aerography') < 0) {
                    //selectedPartSlot=partSlot;
                    possiblePartSlotsList.selectedItem = partSlot;
                } else if (!possiblePartSlotsList.selectedItem) {
                    possiblePartSlotsList.selectedItem = partSlot;
                    //selectedPartSlot=partSlot;
                } else {
                    dispatchEventWith(PART_SELECTED);
                }
            } else {
                possiblePartSlotsList.selectedItem = partSlot;
            }

            scrollToPartSlot(partSlot);

            partSlotSelectedByCarTouch = false;

            Starling.juggler.delayCall(function ():void {
                carViewPartSlotSelectedHandlerStart = false
            }, 0.5);


            if (!menuBackground.visible) {
                openMenu();
            }

            //_alertSound.play();
        }

        private function switchVinylList(isOn:Boolean):void {
            if (isOn) {
                if (possiblePartSlotsList.horizontalScrollPosition) {
                    _possibleScrollPosition = possiblePartSlotsList.horizontalScrollPosition;
                }
                possiblePartSlotsList.clipContent = true;
                if (possiblePartSlotsList.dataProvider != _vinylCollection) {
                    possiblePartSlotsList.dataProvider = _vinylCollection;
                    possiblePartSlotsList.scrollToPosition(0, 0, 0);
                }
                possiblePartSlotsList.width = actualWidth - PART_LIST_MARGIN;
                possiblePartSlotsList.x = PART_LIST_MARGIN;
                if (possiblePartSlotsList.layout is HorizontalLayout) {
                    HorizontalLayout(possiblePartSlotsList.layout).paddingLeft = HorizontalLayout(possiblePartSlotsList.layout).paddingRight;
                }
                possiblePartSlotsList.visible = true;
            } else {
                if (possiblePartSlotsList.dataProvider.getItemIndex(_possiblePartSlots[0]) != 0) {
                    possiblePartSlotsList.dataProvider = new ListCollection(_possiblePartSlots);
                    componentSelector.dataProvider = possiblePartSlotsList.dataProvider;
                } else {
                    _possibleScrollPosition = possiblePartSlotsList.horizontalScrollPosition;
                }
                possiblePartSlotsList.width = actualWidth;
                possiblePartSlotsList.x = 0;
                possiblePartSlotsList.clipContent = false;
                if (possiblePartSlotsList.layout is HorizontalLayout) {
                    HorizontalLayout(possiblePartSlotsList.layout).paddingLeft = -4;
                }
                if (_possibleScrollPosition) {
                    possiblePartSlotsList.scrollToPosition(_possibleScrollPosition, 0, 0);
                    _possibleScrollPosition = 0;
                } else {
                    possiblePartSlotsList.scrollToPosition(0, 0, 0);
                }
            }
        }


        private function scrollToPartSlot(partSlot:PartSlot):void {
            var widthSumBeforeSelected:int = 0;
            var partSlotWidth:int = 0;
            var widthSum:int = 0;

            for (var i:int = 0; i < possiblePartSlots.length; i++) {
                var currentPartSlot:PartSlot = possiblePartSlots[i];
                if (!currentPartSlot){
                    continue;
                }
                var currentPartSlotWidth:int = partSlotNameWidthsMap[currentPartSlot.name];
                widthSum += currentPartSlotWidth;
                if (currentPartSlot != partSlot && partSlotWidth == 0) {
                    widthSumBeforeSelected += partSlotNameWidthsMap[currentPartSlot.name];
                } else {
                    partSlotWidth = currentPartSlotWidth;
                }
            }

            if (widthSum - widthSumBeforeSelected < actualWidth) {
                widthSumBeforeSelected = widthSum - actualWidth;
            }

            possiblePartSlotsList.scrollToPosition(widthSumBeforeSelected, 0, 0.3);
        }

        public function enableBottomMenu(value:Boolean):void{
            var itemList:Vector.<FeathersControl> = new <FeathersControl>[possiblePartSlotsList, partsList, allPartsColorsView];
            for(var i:int = 0; i < itemList.length; i++){
                if(itemList[i]){
                    itemList[i].isEnabled = itemList[i].touchable = value;
                    itemList[i].alpha = value? 1: 0.6;

                }
            }
        }

        public function enableButtons(value:Boolean):void {
            var buttons:Array = [colorMenuButton, partsMenuButton, settingsMenuButton, shareButton, backButton];
            buttons.push(saveButton);
            buttons.push(fastModeButton);
            buttons.push(fastModeHelpButton);
            buttons.push(cameraButton);
            buttons.push(arButton);
            buttons.push(currentScreenshotButton);
            if(car.hdsize){
                buttons.push(saveVideoPreviewButton1);
                buttons.push(saveVideoPreviewButton2);
            }
            buttons.push(powerOnButton);
            buttons.push(gasButton);


            for each(var button:DisplayObject in buttons) {
                if (button) {
                    button.alpha = value ? 1.0 : 0.5;
                    button.touchable = value;
                }
            }
        }

        private function fullscreenButtonTriggeredHandler(event:starling.events.Event):void {
            dispatchEventWith(FULLSCREEN_SELECTED);
        }

        private function shareButtonTriggeredHandler(event:starling.events.Event):void {
            if (shareView.visible) {
                SoundManager.playSound(SoundManager.PART_MENU_CLOSE);
                shareView.visible = false
                return;
            }
            SoundManager.playSound(SoundManager.PART_MENU);

            shareView.x = shareButton.x;
            shareView.y = shareButton.y + (shareButton.height + 7.5);
            shareView.visible = true;
            arButton.visible = currentScreenshotButton.visible = false;
            setVideoMenuVisibility(false);
        }


        private function shareToPinHandler(event:starling.events.Event):void {
            dispatchEventWith(SHARE_PIN_SELECTED);
        }

        private function shareToOkHandler(event:starling.events.Event):void {
            dispatchEventWith(SHARE_OK_SELECTED);
        }


        private function shareToVkBookHandler(event:starling.events.Event):void {
            dispatchEventWith(SHARE_VK_SELECTED);
        }

        private function shareToFaceBookHandler(event:starling.events.Event):void {
            dispatchEventWith(SHARE_FB_SELECTED);
        }

        private function shareToInstagramHandler(event:starling.events.Event):void {
            dispatchEventWith(SHARE_INSTAGRAM_SELECTED);
        }


        public function set carBackgrounds(value:Array):void {


            _carBackgrounds = value;
            if (backgroundMenuView) backgroundMenuView.carBackgrounds = _carBackgrounds;

            if (CarBackgroundsProxy.uploaded_background_id) {
                for (var i:int = 0; i < _carBackgrounds.length; i++) {
                    var bg:CarBackground = _carBackgrounds[i];
                    if (bg.id == CarBackgroundsProxy.uploaded_background_id) {
                        car.selectedBackground = bg;
                        backgroundMenuView.setBackground(bg);

                        break;
                    }
                }
                CarBackgroundsProxy.uploaded_background_id = null;
            }


            this.invalidate('carBackgrounds');
        }


        public function get carBackgrounds():Array {

            return _carBackgrounds;
        }

        private function menuOpenHandler(event:starling.events.Event):void {
            AdsManager.$.bannerHide();
        }

        private function menuCloseHandler(event:starling.events.Event):void {
            if(AdsManager.$.lastBannerStatus){
                AdsManager.$.bannerShow(AdsManager.$.currentBannerScreen);
            }
        }

        private function carLoadCompleteHandler():void{
            if(cameraButton){
                var newIcon:Image = cameraButton.getChildByName("new_icon") as Image;
                if(newIcon){
                    newIcon.visible = car.hdsize;
                }

                if(PromoVideoManager.inProcess){
                    addVideoSpinner();
                    enableVideoButtons(false);
                }
            }

            if(car && carView && carView.visible && car.packet && tempOpenPackages[car.packet.id]){
                car.packet.tempOpened = true;
                packagesView.updateTempOpenPacket(car.packet);
                dispatchEventWith(BUY_NOW_PACKET, false, {packet: car.packet});
                delete tempOpenPackages[car.packet.id];
            }
        }


        override protected function touchHandler(event:TouchEvent):void {

            /**
             * if the progress bar is visible, means something is loading and we do not want to rotate car
             *
             * ONE Exception - when the first camera is loaded, and we showed the car, we want to user only be able to rotate the car, that's why we check <!cameraSecondCameraIsLoading>
             */
            if (progressBar && progressBar.visible && !carView.cameraSecondCameraIsLoading)
                return;


            if (!carView.zooming) {
                if (this.carLoaded)
                    super.touchHandler(event);
            }

            carView.touchHandler(event);
        }

        public function get carLoaded():Boolean {
            return _carLoaded;
        }

        public function set carLoaded(value:Boolean):void {
            _carLoaded = value;
            carLoadCompleteHandler();
        }

        public function enableVideoButtons(value:Boolean):void{
            if(saveVideoPreviewButton1){
                saveVideoPreviewButton1.isEnabled = value;
            }
            if(saveVideoPreviewButton2){
                saveVideoPreviewButton2.isEnabled = value;
            }
        }
    }
}


