package com.despark.tuning3d.view.components.tunings.editing
{
import com.despark.tuning3d.model.cars.Car;
import com.despark.tuning3d.model.cars.CarBackground;
import com.despark.tuning3d.model.core.Device;
import com.despark.tuning3d.model.languages.Language;
import com.despark.tuning3d.model.languages.Multilingual;
import com.despark.tuning3d.model.settings.Settings;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.ImageLoaderWithStates;
import com.despark.tuning3d.view.components.core.MultilingualButton;
import com.despark.tuning3d.view.components.core.MultilingualCheck;
import com.despark.tuning3d.view.components.core.MultilingualLabel;
import com.despark.tuning3d.view.components.core.MultilingualToggleSwitch;
import com.despark.tuning3d.view.components.core.PopUp;
import com.despark.tuning3d.view.components.core.TextInputView;
import com.utils.FontSelector;

import feathers.controls.Button;
import feathers.controls.ImageLoader;
import feathers.controls.PickerList;
import feathers.controls.ScrollContainer;
import feathers.layout.HorizontalLayout;
import feathers.layout.VerticalAlign;
import feathers.skins.ImageStateValueSelector;

import flash.display.BitmapData;
import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.display.DisplayObject;
import starling.display.Image;
import starling.display.Quad;
import starling.display.Sprite;
import starling.events.Event;

public class TuningSettingsPopUp extends PopUp implements Multilingual
	{
		public static const BACKGROUND_SELECTED : String = "BackgroundSelected";
		public static const LANGUAGE_CHANGED : String = "LanguageChanged";
		public static const UPLOAD_BACKGROUND_SELECTED : String = "UploadBackgroundSelected";
		public static const DELETE_BACKGROUND_SELECTED : String = "DeleteBackgroundSelected";
		public static const SNAPSHOT_CAMERA_SELECTED : String = "SNAPSHOT_CAMERA_SELECTED";
		
		public static const MINIFRAME_SELECTED : String = "MINIFRAME_SELECTED";
		
		public var car : Car;
		public var bitmapdataFromCamera : BitmapData;
		public var selectedLanguage : Language;
		
		private var currentLanguage : Language;
		private var _carBackgrounds : Array;
		private var scrollContainer : ScrollContainer;
		private var selectedImageLoader : ImageLoader;
		private var _selectedBackground : CarBackground;
		private var buttons : Array;
		private var deleteButtons : Array;
		private var backgroundsLabel : MultilingualLabel;
		private var autoscrollCheck : MultilingualCheck;
		private var defaultBackgroundCheck : MultilingualCheck;
		private var _settings : Settings;
		private var languagePickerList : PickerList;
		private var _languages : Array;
		public var uploadBackgroundButton : MultilingualButton;
		private var infoLabel : MultilingualLabel;
		private var languageChangedHandlerDisabled : Boolean;
		private var pickerButton : MultilingualButton;
		
		private var photoButton : MultilingualButton;
		
		public var miniFrameCheck : MultilingualToggleSwitch;
		
		public var testInputView1 : TextInputView;
		public var testInputView2 : TextInputView;
		public var testInputView3 : TextInputView;
		
		public function get languages() : Array
		{
			return _languages;
		}

		public function set languages(value : Array):void
		{
			_languages = value;
			invalidate("languages");
		}

		public function get settings() : Settings
		{
			return _settings;
		}

		public function set settings(value : Settings):void
		{
			_settings = value;
			invalidate("settings");
		}

		public function get selectedBackground() : CarBackground
		{
			return _selectedBackground;
		}
		
		public function TuningSettingsPopUp()
		{
			super();
			
			setSizeInternal(300, 200, false);
		}

		public function get carBackgrounds() : Array
		{
			return _carBackgrounds;
		}

		public function set carBackgrounds(value : Array) : void
		{
			_carBackgrounds = value;
			invalidate("carBackgrounds");
		}
		
		protected override function initialize() : void
		{
			super.initialize();
			
			languageChangedHandlerDisabled = true;
			
			backgroundsLabel = new MultilingualLabel();
			backgroundsLabel.touchable = false;
			backgroundsLabel.height = 20;
			backgroundsLabel.textFormat = new TextFormat(FontSelector.getFont(), 8, 0x515D63);
			addChild(backgroundsLabel);
			
			var layout : HorizontalLayout = new HorizontalLayout();
			layout.padding = 10;
			layout.gap = 5;
			//layout.manageVisibility = true; //tt
			layout.verticalAlign = VerticalAlign.MIDDLE;
			
			scrollContainer = new ScrollContainer();
			scrollContainer.layout = layout;
			addChild(scrollContainer);
			
			
			autoscrollCheck = checkWithLabel("Autoscroll     ");
			addChild(autoscrollCheck);
			
			
			defaultBackgroundCheck = checkWithLabel("Use default background      ");
			defaultBackgroundCheck.isSelected = car.selectedBackground == null;
			defaultBackgroundCheck.touchable = !defaultBackgroundCheck.isSelected;
			defaultBackgroundCheck.addEventListener(Event.CHANGE, defaultBackgroundCheckChangeHandler);
			addChild(defaultBackgroundCheck);
			
			//languagePickerList = new PickerList();
			//addChild(languagePickerList);
			
			//languagePickerList.labelField = "name";
			//languagePickerList.popUpContentManager = new VerticalCenteredPopUpContentManager();
			//languagePickerList.addEventListener(Event.CHANGE, languagePickerListChangeHandler);
			
			/*pickerButton = ButtonFactory.defaultButton("Language", 0x959291);
				
			languagePickerList.buttonFactory = function() : Button
			{
				return pickerButton;
			};
			
			languagePickerList.listFactory = function():List
			{
				var list : List = new List();
				
				var layout : VerticalLayout = new VerticalLayout();
				layout.useVirtualLayout = false;
				list.layout = layout;

				list.paddingTop = 5;
				list.paddingBottom = 5;
				list.setSize(150, 150);
				
				var backgroundTextures : Scale9Textures = new Scale9Textures(assetManager.getTexture("popUpBackground"), new Rectangle(24, 22, 11, 7));
				list.backgroundSkin = new Scale9Image( backgroundTextures ); 
				
				list.itemRendererFactory = function() : IListItemRenderer
				{
					var renderer :	LanguageItemRenderer = new LanguageItemRenderer();
					renderer.setSize(150, 35);
					
					return renderer;
				};
				
				return list; 
			};
			*/
			
			uploadBackgroundButton = ButtonFactory.defaultButton(null, 0xed3f25);
			uploadBackgroundButton.width = 110;
			uploadBackgroundButton.addEventListener(Event.TRIGGERED, uploadBackgroundButtonTriggeredHandler);
			addChild(uploadBackgroundButton);
			
			infoLabel = new MultilingualLabel();
			infoLabel.touchable = false;
			infoLabel.height = 20;
			infoLabel.visible = false;
			infoLabel.textFormat = new TextFormat(FontSelector.getFont(), 12, 0xFFFFFF);
			infoLabel.textFormat.align = TextFormatAlign.CENTER;
			addChild(infoLabel);
			
			
			photoButton = ButtonFactory.defaultButton("Take photo", 0x0f87b3);//ButtonFactory.circleButton("redPhoto");
			photoButton.width = 110;
			photoButton.addEventListener(Event.TRIGGERED, photoButtonTriggeredHandler);
			//photoButton.width = uploadBackgroundButton.height;
			photoButton.height = uploadBackgroundButton.height;
			addChild(photoButton);
			
			
			
			{//test
				
				testInputView1 = new TextInputView();
				//testInputView1.addEventListener(Event.CHANGE, testInputViewChangeHandler1);
				testInputView1.width = 200;
				//addChild(testInputView1);
				testInputView1.text = "16384";
				
				
				testInputView2 = new TextInputView();
				//testInputView2.addEventListener(Event.CHANGE, testInputViewChangeHandler2);
				testInputView2.width = 200;
				//addChild(testInputView2);
				testInputView2.text = "100";
				
				testInputView3 = new TextInputView();
				//testInputView3.addEventListener(Event.CHANGE, testInputViewChangeHandler3);
				testInputView3.width = 200;
				//addChild(testInputView3);
				testInputView3.text = "15";
				
				
				
				
			}
			
			
			
			if(!Car.weakDevice){
				var isSelected: Boolean =false;
				
				var w:Number= Device.isIos() || assetManager.scaleFactor <= 2 ? 30 : 25;
				
				miniFrameCheck = customCheckFactory("BASIC","HIGH","connection",isSelected);
				miniFrameCheck.width = w*2*1.4;
				miniFrameCheck.height = w*0.85;
				//miniFrameCheck.isSelected = (Car.stepOfFrameIndex_global>1);
				
				miniFrameCheck.addEventListener(Event.CHANGE, miniframeButtonTriggeredHandler);
				addChild(miniFrameCheck);
			}
		}
		
		private function miniframeButtonTriggeredHandler(event : Event) : void
		{
			dispatchEventWith(MINIFRAME_SELECTED);
		}
		
		public static  function customCheckFactory(leftTitle : String, rightTitle : String, topTitle : String, isSelected: Boolean) : MultilingualToggleSwitch
		{
			
			
			var check : MultilingualToggleSwitch = new MultilingualToggleSwitch(isSelected);
			
			check.isHTML = true;
			
			check.x = 0;
			
			
			check.leftTitle = leftTitle;//;
			check.rightTitle = rightTitle;//;
			check.topTitle = topTitle;//;
			
			
			
			return check;
		}
		
		private function photoButtonTriggeredHandler(event : Event) : void
		{
			
			
			dispatchEventWith(SNAPSHOT_CAMERA_SELECTED);
			
		}
		
		public function updateWithLanguage(language : Language) : void
		{
			currentLanguage = language;
			
			title = language.translate("mob_settings", "settings");
			titleLabel.updateWithLanguage(language);
			
			infoLabel.updateWithLanguage(language);
			
			//pickerButton.updateWithLanguage(language);
			
			uploadBackgroundButton.label = language.translate("mob_saving", "uploadBackground");
			uploadBackgroundButton.updateWithLanguage(language);
			
			defaultBackgroundCheck.label = language.translate("mob_saving", "defaultBackground") + "    ";
			defaultBackgroundCheck.updateWithLanguage(language);
			
			autoscrollCheck.label = language.translate("mob_settings", "autoscroll") + "          ";
			autoscrollCheck.updateWithLanguage(language);
			
			backgroundsLabel.text = language.translate("mob_saving", "selectBackground");
			backgroundsLabel.updateWithLanguage(language);
			
			photoButton.label = "Take photo";
			photoButton.updateWithLanguage(language);
			
			if(miniFrameCheck){
				miniFrameCheck.updateWithLanguage(language);
			}
			
			
			if(miniFrameCheck){
				var text_basic : String = language.translate("mob_features", "basic").toUpperCase();
				miniFrameCheck.leftTitle = text_basic;
				
				var text_high : String = language.translate("mob_features", "high").toUpperCase();
				miniFrameCheck.rightTitle = text_high;
			}  
			
			
			
		}
		
		private function checkWithLabel(label : String) : MultilingualCheck
		{
			var iconSelector : ImageStateValueSelector = new ImageStateValueSelector();
			iconSelector.defaultValue = assetManager.getTexture("check");
			iconSelector.defaultSelectedValue = assetManager.getTexture("checkSelected");
			
			var check : MultilingualCheck = new MultilingualCheck();
			check.isQuickHitAreaEnabled = true;
			check.minTouchHeight = 50;
			check.labelOffsetX = 5;
			check.stateToIconFunction = iconSelector.updateValue;
			check.label = label;
			check.textFormat = new TextFormat(FontSelector.getFont(), 8, 0xFFFFFF);
			check.isHTML = true;
			
			
			check.defaultIcon = new Image(assetManager.getTexture("check"));
			check.selectedUpIcon = new Image(assetManager.getTexture("checkSelected"));
			check.selectedDownIcon = check.selectedUpIcon;
			
			return check;
		}
		
		/*private function languagePickerListChangeHandler(event : Event) : void
		{
			if(languageChangedHandlerDisabled)
			{
				return;
			}
			
			selectedLanguage = languagePickerList.selectedItem as Language;
	
			dispatchEventWith(LANGUAGE_CHANGED);
		}*/
		
		public function closeLanguagePickerList() : void
		{
			
			//languagePickerList.popUpContentManager.close();
			//languageChangedHandlerDisabled = false;
			
		}
		
		public function updateCarBackgroundsViews() : void
		{
			scrollContainer.removeChildren();
			
			buttons = [];
			deleteButtons = [];
			
			for each(var carBackground : CarBackground in carBackgrounds)
			{
				var carBackgroundView : Sprite = new Sprite();
				scrollContainer.addChild(carBackgroundView);
				
				var imageLoader : ImageLoaderWithStates = new ImageLoaderWithStates();	
				imageLoader.maintainAspectRatio = true;
				//imageLoader.scaleMode = ScaleMode.NO_BORDER; /* cometn by maxb*/
				//imageLoader.clipRect = new Rectangle(0, 0, 80, 64); //tt
				imageLoader.setSize(80, 64);
				imageLoader.persistentImageProvider = carBackground;
				carBackgroundView.addChild(imageLoader);
				
				var button : Button = new Button();
				button.setSize(80, 64);
				button.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
				carBackgroundView.addChild(button);
				buttons.push(button);
				


				
				if(imageLoader.alpha == 1.0)
				{
					selectedImageLoader = imageLoader;
				}
				
				if(carBackground != carBackgrounds[carBackgrounds.length - 1])
				{
					var quad : Quad = new Quad(1, button.height - 10, 0xF2F2F2);
					scrollContainer.addChild(quad);
				}
			}
		}
		
		protected override function draw() : void
		{
			super.draw();
			
			if(carBackgrounds && isInvalid("carBackgrounds"))
			{
				updateCarBackgroundsViews();
			}
			
			if(isInvalid(INVALIDATION_FLAG_SIZE))
			{
				backgroundsLabel.width = 
				contentBackgroundImage.height = 107;
				
				backgroundsLabel.x = contentBackgroundImage.x + 10;
				backgroundsLabel.y = contentBackgroundImage.y + 10;
				backgroundsLabel.width = contentBackgroundImage.width - 20;
				
				scrollContainer.x = contentBackgroundImage.x;
				scrollContainer.y = backgroundsLabel.y + backgroundsLabel.height - 10;
				scrollContainer.width = contentBackgroundImage.width;
				scrollContainer.height = 90;
				
				photoButton.y = contentBackgroundImage.y + contentBackgroundImage.height + 10;
				photoButton.x = contentBackgroundImage.x + contentBackgroundImage.width - photoButton.width + contentBackgroundImage.x/2;
				
				uploadBackgroundButton.y = photoButton.y + photoButton.height + 5;
				uploadBackgroundButton.x = photoButton.x; 
				
				//languagePickerList.width = 80;
				
				//languagePickerList.y = uploadBackgroundButton.y;
				//languagePickerList.x = photoButton.x - 5 - languagePickerList.width;
				
				//pickerButton.width = languagePickerList.width;
				
				
				defaultBackgroundCheck.y = photoButton.y + 22;   
				defaultBackgroundCheck.x = contentBackgroundImage.x;
				
				autoscrollCheck.x = contentBackgroundImage.x;  
				autoscrollCheck.y = defaultBackgroundCheck.y + 14 ;       
				
				infoLabel.width = actualWidth;
				infoLabel.y = int((actualHeight - infoLabel.height) / 2);
				
//				photoButton.x = uploadBackgroundButton.x - photoButton.width - 10;
//				photoButton.y = uploadBackgroundButton.y;
				
				
				testInputView1.x = 0;
				testInputView1.y = 0;
				testInputView1.width = contentBackgroundImage.width;
				
				testInputView2.x = 0;
				testInputView2.y = testInputView1.y + testInputView1.height + 3;
				testInputView2.width = contentBackgroundImage.width;
				
				testInputView3.x = 0;
				testInputView3.y = testInputView2.y + testInputView2.height + 3;
				testInputView3.width = contentBackgroundImage.width;
				
				
				
				if(!Car.weakDevice){
					miniFrameCheck.y =  photoButton.y -8;
					miniFrameCheck.x = contentBackgroundImage.x;  
					//miniFrameCheck.visible=false;
				}
			}
			
			if(isInvalid("settings") && settings)
			{
				autoscrollCheck.isSelected = settings.autoscrollEnabled;
			}
			
			if(selectedLanguage && isInvalid("languages") && languages)
			{	
				var index : int = -1;
				
				for(var k : int = 0; k < languages.length; k++)
				{
					if(selectedLanguage.code == languages[k].code)
					{
						index = k;
					}
				}
			
				//languagePickerList.dataProvider = new ListCollection(languages);
				//languagePickerList.selectedIndex = index;
				
				//languageChangedHandlerDisabled = false;
			}
		}
		
		public function get autoscrollEnabled() : Boolean
		{
			return autoscrollCheck.isSelected;
		}
		
		private function buttonTriggeredHandler(event : Event) : void
		{
			var button : Button = Button(event.target);
			
		
				
			_selectedBackground = carBackgrounds[buttons.indexOf(button)];
			dispatchEventWith(BACKGROUND_SELECTED);
		}
		
		private function uploadBackgroundButtonTriggeredHandler(event : Event) : void
		{
			dispatchEventWith(UPLOAD_BACKGROUND_SELECTED);
		}
		
		public function get objectsToHideWhileUploading() : Array
		{
			return [
				scrollContainer, 
				backgroundsLabel, 
				autoscrollCheck, 
				defaultBackgroundCheck, 
				uploadBackgroundButton, 
				contentBackgroundImage, 
				
				photoButton
			];
		}
		
		public function enterUploadingState() : void
		{
			infoLabel.visible = true;
			infoLabel.text = currentLanguage.translate("mob_alertsAndOverlays", "uploading").toUpperCase();
			
			for each(var objectToHide : DisplayObject in objectsToHideWhileUploading)
			{
				objectToHide.visible = false;
			}
		}
		
		public function enterDeletingState() : void
		{
			infoLabel.visible = true;
			infoLabel.text = currentLanguage.translate("mob_downloads", "deleting").toUpperCase();
			
			for each(var objectToHide : DisplayObject in objectsToHideWhileUploading)
			{
				objectToHide.visible = false;
			}
		}
		
		public function enterNormalState() : void
		{
			infoLabel.visible = false;
			
			for each(var objectToHide : DisplayObject in objectsToHideWhileUploading)
			{
				objectToHide.visible = true;
			}
		}
		
		private function deleteButtonTriggeredHandler(event : Event) : void
		{
			var deleteButton : Button = Button(event.target);
			
			_selectedBackground = carBackgrounds[deleteButtons.indexOf(deleteButton)];
			dispatchEventWith(DELETE_BACKGROUND_SELECTED);
		}
		
		private function defaultBackgroundCheckChangeHandler(event : Event) : void
		{
			_selectedBackground = null;
			dispatchEventWith(BACKGROUND_SELECTED);
		}
	}
}