package com.despark.tuning3d.view.components.tunings.editing {
	import com.despark.tuning3d.model.core.PromoVideoManager;
	import com.despark.tuning3d.view.components.core.ButtonFactory;
	import com.despark.tuning3d.view.components.core.LabelFactory;
	import com.despark.tuning3d.view.components.core.PopUp;
	import com.utils.FileUtil;

	import feathers.controls.Button;
	import feathers.controls.Label;
	import feathers.controls.LayoutGroup;
	import feathers.layout.AnchorLayout;
	import feathers.layout.AnchorLayoutData;
	import feathers.layout.HorizontalLayout;
	import feathers.layout.VerticalAlign;

	import flash.filesystem.File;

	import starling.display.Image;
	import starling.events.Event;

	public class TuningVideoPopup extends PopUp {
		private var _mainContainer:LayoutGroup;
		private var _confirmButton:Button;
		private var _cancelButton:Button;
		private var _okButton:Button;
		private var _buttonContainer:LayoutGroup;
		private var _messageField:Label;
		private var _mode:String;
		private var _selectedButton:Button;
		private var _header:LayoutGroup;
		private var _videoIcon:Image;
		private var _videoLabel:Label;
		private var _settingsFile:File;
		private var _videoData:Object;
		//
		public static const START:String = "start";
		public static const COMPLETE:String = "complete";
		public static const SAVE_VIDEO:String = "save_video";
		public function TuningVideoPopup(mode:String = null, data:Object = null) {
			super();
			_mode = mode||START;
			if(data){
				_selectedButton = data.button;
				_settingsFile = data.settingsFile;
				_videoData = data.videoData;
			}
		}

		override protected function initialize() : void {
			super.initialize();
			setSizeInternal(stage.stageWidth, stage.stageHeight, false);
			contentBackgroundImage.visible = false;
			closeButton.visible = false;
			//
			_mainContainer = new LayoutGroup();
			_mainContainer.layout = new AnchorLayout();
			addChild(_mainContainer);
			//
			var startMsg:String = "Video will be downloaded to your phone's \n" +
					"gallery. You can continue using 3DTuning \n" +
					"while video is being prepared.";
			var completeMsg:String = "Video downloaded to \n" + "your phone\'s gallery.";
			//

			_videoIcon = new Image(ButtonFactory.assetManager.getTexture("video_icon"));
			_videoIcon.scale = ButtonFactory.assetManager.scaleFactor / 2;
			_header = new LayoutGroup();
			_header.addChild(_videoIcon);
			_mainContainer.addChild(_header);

			_videoLabel = new Label();
			_videoLabel.textRendererFactory = LabelFactory.saveVideoHeaderLabel;
			_videoLabel.text = "Video";
			_header.addChild(_videoLabel);
			_header.validate();
			_header.layout = new HorizontalLayout();
			_header.layoutData = new AnchorLayoutData();
			HorizontalLayout(_header.layout).gap = 5;
			HorizontalLayout(_header.layout).verticalAlign = VerticalAlign.MIDDLE;
			AnchorLayoutData(_header.layoutData).horizontalCenter = 0;
			AnchorLayoutData(_header.layoutData).top = actualHeight / 3 - _header.height - 10;


			_messageField = new Label();
			_messageField.textRendererFactory = LabelFactory.saveVideoMessageLabel;
			_messageField.width = 260;
			_mainContainer.addChild(_messageField);
			var fieldLayout:AnchorLayoutData = new AnchorLayoutData();
			fieldLayout.horizontalCenter = 0;
			fieldLayout.verticalCenter = -15;
			_messageField.layoutData = fieldLayout;
			//
			_buttonContainer = new LayoutGroup();
			if(_mode == START){
				_confirmButton = ButtonFactory.simpleRoundButton("Prepare & Download", 0x0070b0);
				_cancelButton = ButtonFactory.simpleRoundButton("Cancel", -1, "a_transparent_bg_button");

				_confirmButton.width = 135;
				_confirmButton.height = 25;
				_confirmButton.addEventListener(Event.TRIGGERED, confirmTriggeredHandler);

				_cancelButton.width = 70;
				_cancelButton.height = 25;
				_cancelButton.addEventListener(Event.TRIGGERED, cancelButtonTriggeredHandler);

				_buttonContainer.addChild(_confirmButton);
				_buttonContainer.addChild(_cancelButton);
				_messageField.text = startMsg;
			}else{
				_okButton = ButtonFactory.simpleRoundButton("OK, got it", 0x0070b0);

				_okButton.width = 75;
				_okButton.height = 25;
				_okButton.addEventListener(Event.TRIGGERED, cancelButtonTriggeredHandler);

				_buttonContainer.addChild(_okButton);
				_messageField.text = completeMsg;
			}
			_mainContainer.addChild(_buttonContainer);
			_buttonContainer.validate();
			//
			var layout:HorizontalLayout = new HorizontalLayout();
			layout.gap = 14;
			var btnLayData:AnchorLayoutData = new AnchorLayoutData();
			btnLayData.bottom = actualHeight / 3 - _buttonContainer.height / 2;
			btnLayData.horizontalCenter = 0;
			_buttonContainer.layout = layout;
			_buttonContainer.layoutData = btnLayData;
		}

		private function confirmTriggeredHandler(event:Event):void {
			_videoData.count ++;
			FileUtil.filePutContent(_settingsFile, JSON.stringify(_videoData));
			PromoVideoManager.inProcess = true;

			closeButtonTriggeredHandler(null);
			dispatchEvent(new Event(SAVE_VIDEO, false, {button:_selectedButton}));
		}

		override public function dispose():void{
			if(_confirmButton){
				_confirmButton.removeFromParent(true);
				_confirmButton = null;
			}

			if(_cancelButton){
				_cancelButton.removeFromParent(true);
				_cancelButton = null;
			}

			if(_okButton){
				_okButton.removeFromParent(true);
				_okButton = null;
			}

			if(_buttonContainer){
				_buttonContainer.removeFromParent(true);
				_buttonContainer = null;
			}

			if(_mainContainer){
				_mainContainer.removeFromParent(true);
				_mainContainer = null;
			}
		}
		
		protected function cancelButtonTriggeredHandler(event:Event):void{
			dispatchEventWith(CLOSE_BUTTON_SELECTED);
		}

		override protected function draw() : void {
			super.draw();
			if(isInvalid(INVALIDATION_FLAG_SIZE)) {
				_mainContainer.width = actualWidth;
				_mainContainer.height = actualHeight;
			}
		}
	}
}

