package com.despark.tuning3d.view.core {
import com.despark.tuning3d.model.core.AppMode;
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Device;
    import com.despark.tuning3d.model.core.Tracer;
import com.despark.tuning3d.model.languages.Language;
    import com.despark.tuning3d.model.sounds.SoundManager;
    import com.despark.tuning3d.model.tunings.Tuning;
    import com.despark.tuning3d.view.components.challenges.InstagramChallengePopup;
    import com.despark.tuning3d.view.components.core.AlertView;
import com.despark.tuning3d.view.components.core.AlertViewHL;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.BlockerView;
import com.despark.tuning3d.view.components.core.ButtonFactory;
import com.despark.tuning3d.view.components.core.CustomAlertView;
import com.despark.tuning3d.view.components.core.HomeScreenButton;
import com.despark.tuning3d.view.components.core.IMainMenuHolder;
import com.despark.tuning3d.view.components.core.MainMenuButton;
import com.despark.tuning3d.view.components.core.MainMenuView;
import com.despark.tuning3d.view.components.core.PopUp;
import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.distriqt.AdsManager;
    import com.greensock.TweenLite;
    import com.utils.FontSelector;

import feathers.controls.Button;

import feathers.controls.Label;
import feathers.controls.LayoutGroup;
import feathers.controls.text.TextFieldTextRenderer;
import feathers.core.ITextRenderer;
import feathers.layout.HorizontalAlign;
import feathers.layout.VerticalAlign;
import feathers.layout.VerticalLayout;
import feathers.layout.VerticalLayoutData;

import flash.text.TextFormat;
import flash.text.TextFormatAlign;

import starling.core.Starling;

import starling.display.Image;
import starling.display.Quad;
import starling.events.Event;

public class HomeScreenPopUp extends StackedScreen implements IMainMenuHolder{

    public static const MODE_SELECTED : String = "ModeSelected";

    public static var UPDATE_OBJECT : Object;

    private var _tuningButton : HomeScreenButton;
    private var _truckConfiguratorButton : HomeScreenButton;
    private var _menuButton:MainMenuButton;
    private var _mainMenuView:MainMenuView;
    private var _blockerView : BlockerView;


    private var _waitingForBrandsLoad : Boolean = false;
    private var _headerContainer:LayoutGroup;
    private var _titleContainer:LayoutGroup;
    private var _wide : Boolean; // true if the device is rectangular ( iphone 7(aspectRation > 1.5)) false if square (iPad)
    private var _isSuperWide : Boolean; // true if the device is too wide( (aspectRation > 1.8)) false if square (iPad)


    public static const HOME_SCREEN_SCALE:Number = 130/184;// 138
    public static const BUTTON_DIST:Number = 50;// 138
    private var _selectedLanguage:Language;
    private var _languages:Array;
    private var _language:Language = null;
    private var _userSignedIn:Boolean;
    private var _selectedTuning:Tuning;

    private var title : Label;
    private var desc : Label;

//
    public function HomeScreenPopUp() {
        super("HomeScreenPopUp");
        AppMode.self.setModeHome();
    }

    private function initBg() : void {
        var newBg : Image = new Image(assetManager.getTexture("firstScreenBg"));
        var aspect : Number = newBg.width / newBg.height;
        var needY : Number = _wide ? 0 : 0;
        var neededHeight : Number = actualHeight - needY; // for IPAD we need to move y a bit
        var currentAspect : Number = actualWidth / actualHeight;
        if(currentAspect > aspect) {
            newBg.width = actualWidth;
            newBg.height = actualWidth / aspect;
        } else {
            newBg.height = actualHeight;
            newBg.width = actualHeight * aspect;
        }
        newBg.x = (actualWidth - newBg.width) /2 ;
        newBg.y = needY;
        addChild(newBg);
        /*background.color = 0x222222;
        background.alpha = 1;*/
    }


    override protected function initialize():void {
        super.initialize();

        _wide = this.stage.stageWidth / this.stage.stageHeight > 1.5;
        _isSuperWide = this.stage.stageWidth / this.stage.stageHeight > 1.8;

        setSizeInternal(this.stage.stageWidth, this.stage.stageHeight, false);
        /*closeButton.visible = false;
        contentBackgroundImage.visible = false;*/

        initBg();
        initHeader();
        _tuningButton = ButtonFactory.homeScreenButton("tuning");
        _tuningButton.screen = this;
        _tuningButton.screenMode = AppMode.MODE_TUNING;
        _tuningButton.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
        addChild(_tuningButton);


        _truckConfiguratorButton = ButtonFactory.homeScreenButton("truck");
        _truckConfiguratorButton.screen = this;
        _truckConfiguratorButton.screenMode = AppMode.MODE_TRUCK_CONFIGURATOR;
        _truckConfiguratorButton.addEventListener(Event.TRIGGERED, buttonTriggeredHandler);
        addChild(_truckConfiguratorButton);

        locateMainButtons();
        //
        if(UPDATE_OBJECT) {
            _tuningButton.newsData = UPDATE_OBJECT.tuning;
            _truckConfiguratorButton.newsData = UPDATE_OBJECT.pickups;
        }
        //
        _menuButton = ButtonFactory.menuBigButton("home_screen_main_menu", 0x5a5a5a);
        _menuButton.addEventListener(Event.TRIGGERED, menuButtonTriggeredHandler);
        addChild(_menuButton);

        if (!_mainMenuView){
            mainMenuView = new MainMenuView();
            mainMenuView.name = "mainMenuView";
            mainMenuView.setSize(MainMenuView.MENU_WIDTH, actualHeight);
            mainMenuView.parentActualWidth = actualWidth;
            mainMenuView.parentActualHeight = actualHeight;
            addChild(_mainMenuView);
            _mainMenuView.addEventListener(Event.CLOSE, menuCloseHandler);
        }

        initTitle();

        updateNotificationCount();
        //
        Starling.current.juggler.delayCall(function():void{
            invalidate(INVALIDATION_FLAG_SIZE);
        }, 0.1);
    }

    private function menuCloseHandler(event:Event):void {
        BannerStackedScreen.showAdBanner();
    }

    private function instagramTriggeredHandler(event:Event):void {
        var instagramPopup:InstagramChallengePopup = new InstagramChallengePopup();
        addChild(instagramPopup);
        instagramPopup.height = (stage.stageHeight - _headerContainer.height - 20);
        instagramPopup.x = (stage.stageWidth - instagramPopup.width) /2;
        instagramPopup.y = _headerContainer.height + (stage.stageHeight - _headerContainer.height - instagramPopup.height) /2;
        instagramPopup.open();
        instagramPopup.addEventListener(Event.CLOSE, function (e:Event):void {
            showMainButtons();
        });
        //
        hideMainButtons();
    }

    private function hideMainButtons():void{
        TweenLite.to(_tuningButton, 0.3, {alpha:0});
        TweenLite.to(_truckConfiguratorButton, 0.3, {alpha:0});
    }

    private function showMainButtons():void{
        TweenLite.to(_tuningButton, 0.3, {alpha:1});
        TweenLite.to(_truckConfiguratorButton, 0.3, {alpha:1});
    }

    private function locateMainButtons():void{
        if(stage) {
            _tuningButton.x = Math.max(stage.stageWidth / 2 - (_tuningButton.width + BUTTON_DIST), 5);
            _truckConfiguratorButton.x = Math.min(stage.stageWidth / 2 + BUTTON_DIST, stage.stageWidth - _truckConfiguratorButton.width - 5);
        }
    }

    private function menuButtonTriggeredHandler(event:Event):void {
        AdsManager.$.bannerHide();
        _mainMenuView.toggle();
    }

    public function updateNotificationCount() :void{
        if(mainMenuView){
            mainMenuView.updateNotificationCount();
        }
        _menuButton.updateNotification();
    }

    override protected function draw():void {
        super.draw();

        if(_headerContainer && _tuningButton && _truckConfiguratorButton) {
            var yCoord : Number ;
            _headerContainer.validate();
            _headerContainer.validate();
            _tuningButton.validate();
            _truckConfiguratorButton.validate();
            yCoord = (stage.stageHeight + _headerContainer.height - _tuningButton.height) / 2 + 8;

            _tuningButton.y = yCoord;
            _truckConfiguratorButton.y = yCoord;
            locateMainButtons();
        }

        if(_menuButton){
            _menuButton.x = _menuButton.y = (_headerContainer.height - _menuButton.height) / 2;
        }


    }

    private function initHeader():void {
        var padding : Number = _wide ? actualHeight * 0.02: actualHeight * 0.02;
        padding = _isSuperWide ? 1 : padding;

        var verticalLayout : VerticalLayout = new VerticalLayout();
        verticalLayout.horizontalAlign = HorizontalAlign.RIGHT;
        verticalLayout.verticalAlign = VerticalAlign.MIDDLE;
        verticalLayout.gap = -4;

        _headerContainer  = new LayoutGroup();
        _headerContainer.backgroundSkin = new Quad(1,1,0xfeffff);
        _headerContainer.width = actualWidth;
        _headerContainer.height = 47;
        _headerContainer.layout = verticalLayout;
        addChild(_headerContainer);
    }

    private function initTitle():void {
        var padding : Number = _wide ? actualHeight * 0.02: actualHeight * 0.02;
        padding = _isSuperWide ? 1 : padding;


        var verticalLayout : VerticalLayout = new VerticalLayout();
        verticalLayout.horizontalAlign = HorizontalAlign.RIGHT;
        verticalLayout.verticalAlign = VerticalAlign.MIDDLE;
        verticalLayout.gap = 0;

        _titleContainer  = new LayoutGroup();
        _titleContainer.width = actualWidth;
        _titleContainer.height = 40;
        _titleContainer.layout = verticalLayout;
        _titleContainer.y = _titleContainer.height + 6;
        addChild(_titleContainer);

        title = new Label();
        title.width = actualWidth;
        title.textRendererFactory = function () :ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            //textRenderer.textFormat = new TextFormat(FontSelector.getMonserratMedium(), _wide ? _isSuperWide ? 14 : 17 : 17.5, 0x5a5a5a);
            textRenderer.textFormat = new TextFormat(FontSelector.getMonserratMedium(), 13, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = false;
            textRenderer.embedFonts = true;
            return textRenderer;
        };
        title.text = "WELCOME TO 3DTUNING";
        _titleContainer.addChild(title);


        var vertLayData : VerticalLayoutData = new VerticalLayoutData();
        vertLayData.percentWidth = 99.8;

        desc = new Label();
        desc.layoutData = vertLayData;
        desc.textRendererFactory = function () :ITextRenderer {
            var textRenderer:TextFieldTextRenderer = new TextFieldTextRenderer();
            //textRenderer.textFormat = new TextFormat(FontSelector.getMonserratLight(), _wide ? _isSuperWide ? 9 : 10: 11, 0x5a5a5a);
            textRenderer.textFormat = new TextFormat(FontSelector.getMonserratLight(), 8, 0xFFFFFF);
            textRenderer.textFormat.align = TextFormatAlign.CENTER;
            textRenderer.textFormat.letterSpacing = 1; //_wide ? _isSuperWide ?  0.8 : 1.49 : 1.35;
            textRenderer.textFormat.italic = false;
            textRenderer.textFormat.bold = true;
            textRenderer.embedFonts = true;
            return textRenderer;
        };

        desc.text = "probably the best car configurator";
        _titleContainer.addChild(desc);
    }

    public function brandsLoaded() : Boolean {
        if(_waitingForBrandsLoad) {
            Tracer.timeStamp('HomeButton', "Opened", true);
            // dispatchEventWith(CLOSE_BUTTON_SELECTED);
            Starling.juggler.delayCall(function () : void {
                _blockerView.hide();
                _blockerView = null;
            }, 1);
            return true;
        }
        return false;
    }


    private function buttonTriggeredHandler(e:Event):void {
        if(_waitingForBrandsLoad) {
            return;
        }

        switch (e.currentTarget){
            case _tuningButton:
                    SoundManager.playSound(SoundManager.PART_MENU);
                    Tracer.timeStamp('HomeButton', "Tuning");
                    AppMode.self.setModeTuning();
                break;
            case _truckConfiguratorButton:
                    SoundManager.playSound(SoundManager.PART_MENU);
                    Tracer.timeStamp('HomeButton', "Truck");
                    AppMode.self.setModeTruckConfigurator();
                break;
            /*case _wheelConfiguratorButton:
                    AppMode.self.setModeWheelConfigurator();
                break;*/
        }

        _blockerView = BlockerView.show();

        _waitingForBrandsLoad = true;
        Starling.juggler.delayCall(function () : void  {
            dispatchEventWith(MODE_SELECTED);
        }, 1)

    }

    public function get userSignedIn():Boolean{
        return _userSignedIn;
    }

    public function set userSignedIn(value:Boolean):void {
        _userSignedIn = value;
        if (!stage)	{
            return;
        }
        invalidate("userSignedIn");
    }

    override public function dispose():void {
        super.dispose();
    }

    public function updateWithLanguage(language:Language) : void {
        _language = language;

        AlertView.language = language;
        CustomAlertView.language = language;
        AlertViewHL.language = language;

        if (mainMenuView){
            mainMenuView.updateWithLanguage(language);
        }
    }

    public function dispatchExternalEvent(msg:String):void {
        if (mainMenuView) mainMenuView.hide();
        dispatchEventWith(msg);
    }

    public function set mainMenuView(value : MainMenuView): void {
        _mainMenuView = value;
        _mainMenuView.owner = this;
    }

    public function get mainMenuView() : MainMenuView{
        return _mainMenuView;
    }
    public function brandsScreenButtonTriggeredHandler(event : Event=null) : void{
        mainMenuView.hide(false);
        _blockerView = BlockerView.show();
        _waitingForBrandsLoad = true;
        Starling.juggler.delayCall(function () : void  {
            dispatchEventWith(MODE_SELECTED);
        }, 1)
    }

    public function homeButtonTriggeredHandler(event : Event=null) : void{
        mainMenuView.toggle();
    }

    public function enterRowsState() : void{

    }

    public function get languages() : Array{
        return _languages;
    }

    public function set languages(value : Array):void {
        _languages = value;
        invalidate("languages");
    }

    public function set selectedLanguage(value:Language):void {
        _selectedLanguage = value;
    }

    public function get selectedLanguage():Language {
        return _selectedLanguage;
    }

    public function get selectedTuning():Tuning {
        return _selectedTuning;
    }
}
}
