package com.despark.tuning3d.view.core
{
    import com.despark.tuning3d.Notifications;
    import com.despark.tuning3d.model.assets.AssetsProxy;
    import com.despark.tuning3d.model.core.BannerScreen;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.BrandsScreenMediator;
    import com.despark.tuning3d.view.HomeScreenPopUpMediator;
    import com.despark.tuning3d.view.components.NotificationView.InAppNotificationPopUp;
    import com.despark.tuning3d.view.components.brands.BrandsScreen;
    import com.despark.tuning3d.view.components.challenges.modals.ChallengeNotificationsPopup;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.despark.tuning3d.view.components.core.BannerStackedScreen;
    import com.despark.tuning3d.view.components.core.StackedScreen;
    import com.despark.tuning3d.view.components.friends.FriendsListPopUp;
    import com.despark.tuning3d.view.components.mailbox.MailsScreen;
    import com.despark.tuning3d.view.components.timeline.TimelineScreen;
    import com.despark.tuning3d.view.components.tunings.TuningsListScreen;
    import com.distriqt.AdsManager;
    import com.utils.ArrayUtil;

    import org.puremvc.as3.interfaces.INotification;

    import starling.core.Starling;
    import starling.events.Event;

    public class StackedScreenExtendedMediator extends StackedScreenMediator {
        public static const AUTH_ACTION_MAILS : String = "mails";
        public static const AUTH_ACTION_MAIL_BOX : String = "mail_box";
        private static var _SCREEN_DISPLAY_DELAY:Number = 0.2;
		public function StackedScreenExtendedMediator(stackedScreen : StackedScreen) {
			super(stackedScreen);
			stackedScreen.assetManager = (facade.retrieveProxy(AssetsProxy.NAME) as AssetsProxy).assetManager;
		}
		
		override public function onRegister() : void {
			super.onRegister();
            /**
             * Screens and popups requiring authentication
             */
            stackedScreen.addEventListener(StackedScreen.MAILS_SELECTED, mailSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.USER_GARAGE_SELECTED, userGarageSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.FRIENDS_SELECTED, friendsSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.TIMELINE_SECLECTED, timelineSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.NOTIFICATION_SELECTED, notificationsSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.CHALLENGE_NOTIFICATIONS_SELECTED, challengeNotesSelectHandler);

            /**
             * Screens and popups not requiring authentication
             */
            stackedScreen.addEventListener(StackedScreen.HOME_SELECTED, homeSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.GARAGE_SELECTED, garageSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.BRANDS_SELECTED, brandsSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.VOTE_SELECTED, votesSelectedHandler);
            stackedScreen.addEventListener(StackedScreen.GALLERY_SELECTED, gallerySelectHandler);
            stackedScreen.addEventListener(StackedScreen.CHALLENGES_SELECTED, challengeSelectHandler);
            stackedScreen.addEventListener(StackedScreen.LEADER_BOARD_SELECTED, leaderBoardSelectHandler);

            /**
             * Login/logout
             */
            stackedScreen.addEventListener(BrandsScreen.LOGIN_SELECTED, loginSelectedHandler);
            stackedScreen.addEventListener(BrandsScreen.LOGOUT_SELECTED, logoutSelectedHandler);
		}

        override public function onRemove() : void {
            super.onRemove();
            /**
             * Screens and popups requiring authentication
             */
            stackedScreen.removeEventListener(StackedScreen.MAILS_SELECTED, mailSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.USER_GARAGE_SELECTED, userGarageSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.FRIENDS_SELECTED, friendsSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.TIMELINE_SECLECTED, timelineSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.NOTIFICATION_SELECTED, notificationsSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.CHALLENGE_NOTIFICATIONS_SELECTED, challengeNotesSelectHandler);

            /**
             * Screens and popups not requiring authentication
             */
            stackedScreen.removeEventListener(StackedScreen.HOME_SELECTED, homeSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.GARAGE_SELECTED, garageSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.BRANDS_SELECTED, brandsSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.VOTE_SELECTED, votesSelectedHandler);
            stackedScreen.removeEventListener(StackedScreen.GALLERY_SELECTED, gallerySelectHandler);
            stackedScreen.removeEventListener(StackedScreen.CHALLENGES_SELECTED, challengeSelectHandler);
            stackedScreen.removeEventListener(StackedScreen.LEADER_BOARD_SELECTED, leaderBoardSelectHandler);

            /**
             * Login/logout
             */
            stackedScreen.removeEventListener(BrandsScreen.LOGIN_SELECTED, loginSelectedHandler);
            stackedScreen.removeEventListener(BrandsScreen.LOGOUT_SELECTED, logoutSelectedHandler);
        }

        private function friendsSelectedHandler(event : Event) : void {
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(!usersProxy.currentUser) {
                _SCREEN_DISPLAY_DELAY = 1.5;
            } else {
                _SCREEN_DISPLAY_DELAY = 0.2;
            }
            pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_FRIENDS;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        private function timelineSelectedHandler( e : Event) : void {
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(!usersProxy.currentUser) {
                _SCREEN_DISPLAY_DELAY = 1.5;
            } else {
                _SCREEN_DISPLAY_DELAY = 0.2;
            }
            pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_TIMELINE;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        private function userGarageSelectedHandler(event : Event): void {
            if(Constants.selectedUser) {
                _SCREEN_DISPLAY_DELAY = 1.5;
                pendingForAuthAction = null;
                sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN);
            } else {
                _SCREEN_DISPLAY_DELAY = 0.2;
                pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_GARAGE;
                sendNotification(Notifications.AUTHENTICATE_USER);
            }
        }

        private function notificationsSelectedHandler(event : Event) : void{
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(!usersProxy.currentUser) {
                _SCREEN_DISPLAY_DELAY = 1.5;
            } else {
                _SCREEN_DISPLAY_DELAY = 0.2;
            }
            pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_NOTIFICATIONS;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        private function challengeNotesSelectHandler(event : Event) : void {
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(!usersProxy.currentUser) {
                _SCREEN_DISPLAY_DELAY = 1.5;
            } else {
                _SCREEN_DISPLAY_DELAY = 0.2;
            }
            pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_CHALLENGE_NOTIFICATIONS;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        private function mailSelectedHandler(event:Event):void {
            pendingForAuthAction = AUTH_ACTION_MAILS;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        /**
         * Screens and popups not requiring authentication
         */
        private function homeSelectedHandler(event:Event):void {
            sendNotification(Notifications.DISPLAY_HOME_SCREEN_POPUP);
        }

        private function brandsSelectedHandler(event:Event):void {
            if(HomeScreenPopUpMediator.isFirstOpen) {
                sendNotification(Notifications.DISPLAY_BRANDS_SCREEN);
            } else {
                sendNotification(Notifications.DISPLAY_ROOT_SCREEN);
            }
            HomeScreenPopUpMediator.isFirstOpen = false;
        }

        private function garageSelectedHandler(event : Event): void {
            Constants.selectedUser = null;
            _SCREEN_DISPLAY_DELAY = 0.2;
            pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_GARAGE;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        private function votesSelectedHandler(event:Event):void {
            sendNotification(Notifications.DISPLAY_VOTES_LIST_SCREEN, null);
        }

        private function gallerySelectHandler(event : Event) : void {
            sendNotification(Notifications.DISPLAY_GALLERY_SCREEN);
        }

        private function challengeSelectHandler(event : Event) : void {
            sendNotification(Notifications.DISPLAY_CHALLENGE_SCREEN);
        }

        private function leaderBoardSelectHandler(event : Event) : void {
            BannerStackedScreen.hideAdBanner(false);
            sendNotification(Notifications.DISPLAY_CHALLENGE_LEADERBOARD_POPUP, {showBannerAfterExit:true});
        }

        /**
         * Login/Logout
         */
        private function loginSelectedHandler(event : Event) : void {
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            if(!usersProxy.currentUser) {
                _SCREEN_DISPLAY_DELAY = 1.5;
            } else {
                _SCREEN_DISPLAY_DELAY = 0.2;
            }
            pendingForAuthAction = BrandsScreenMediator.AUTH_ACTION_LOGIN;
            sendNotification(Notifications.AUTHENTICATE_USER);
        }

        private function logoutSelectedHandler(e:Event):void {
            var usersProxy : UsersProxy = facade.retrieveProxy(UsersProxy.NAME) as UsersProxy;
            usersProxy.logoutCurrentUser();
            AlertView.show(AlertView.language.translate("mob_saving", "loggedOut"), -1, false);
            if(stackedScreen.hasOwnProperty("updateNotifications")){
                stackedScreen["updateNotifications"]();
            }
            stackedScreen.ShowDone();
            if(stackedScreen is MailsScreen || stackedScreen is TimelineScreen){
                homeSelectedHandler(null);
            }
        }


        override public function listNotificationInterests():Array {
            return ArrayUtil.uniqueConcat(super.listNotificationInterests(), [
                Notifications.USER_AUTHENTICATED
            ])
        }

        override public function handleNotification(note:INotification):void {
            super.handleNotification(note);
			switch (note.getName()){
                case Notifications.USER_AUTHENTICATED:
                    handleUserAuthenticated();
                    break;
			}
        }

        private function handleUserAuthenticated():void{
            if(!stackedScreen || !stackedScreen.stage){
                return;
            }
            var popupData:Object = {showBannerAfterExit: true};
            if(stackedScreen is BrandsScreen){
                popupData.bannerScreen = BannerScreen.HOME_SCREEN;
                BrandsScreen(stackedScreen).locateAdBanner();
            }
            switch(pendingForAuthAction){
                case AUTH_ACTION_MAILS:
                    if(!(stackedScreen is MailsScreen)){
                        Starling.juggler.delayCall(function():void{
                            sendNotification(Notifications.DISPLAY_MAIL_SCREEN);
                        }, _SCREEN_DISPLAY_DELAY);
                    }
                    break;
                case BrandsScreenMediator.AUTH_ACTION_FRIENDS:
                    Starling.juggler.delayCall(function() : void{
                        BannerStackedScreen.hideAdBanner(false);
                        sendNotification(Notifications.DISPLAY_FRIENDS_LIST_SCREEN, popupData);
                    }, _SCREEN_DISPLAY_DELAY);
                    break;
                case BrandsScreenMediator.AUTH_ACTION_GARAGE:
                    if(!(stackedScreen is TuningsListScreen)){
                        Starling.juggler.delayCall(function():void{
                            sendNotification(Notifications.DISPLAY_TUNINGS_LIST_SCREEN);
                        }, _SCREEN_DISPLAY_DELAY);
                    }
                   break;
                case BrandsScreenMediator.AUTH_ACTION_TIMELINE:
                    if(!(stackedScreen is TimelineScreen)){
                        Starling.juggler.delayCall(function():void{
                            sendNotification(Notifications.DISPLAY_TIMELINE_SCREEN);
                        }, _SCREEN_DISPLAY_DELAY);
                    }
                   break;
                case BrandsScreenMediator.AUTH_ACTION_NOTIFICATIONS:
                    if(!(stackedScreen is InAppNotificationPopUp)){
                        Starling.juggler.delayCall(function() : void{
                            BannerStackedScreen.hideAdBanner(false);
                            sendNotification(Notifications.DISPLAY_IN_APP_NOTIFICATION_POPUP, popupData);
                        }, _SCREEN_DISPLAY_DELAY);
                    }
                   break;
                case BrandsScreenMediator.AUTH_ACTION_CHALLENGE_NOTIFICATIONS:
                    if(!(stackedScreen is ChallengeNotificationsPopup)){
                        Starling.juggler.delayCall(function() : void{
                            BannerStackedScreen.hideAdBanner(false);
                            sendNotification(Notifications.DISPLAY_CHALLENGE_NOTIFICATION_POPUP, popupData);
                        }, _SCREEN_DISPLAY_DELAY);
                    }
                   break;
            }

        }
	}
}