package com.distriqt {
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.core.Tracer;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.despark.tuning3d.view.components.core.AlertView;
    import com.distriqt.extension.facebookapi.FacebookAPI;
    import com.distriqt.extension.facebookapi.events.FacebookAPISessionEvent;
    import com.distriqt.extension.facebookapi.events.ShareDialogEvent;
    import com.distriqt.extension.facebookapi.share.builders.ShareLinkContentBuilder;

//import com.despark.tuning3d.model.core.Constants;
//import com.despark.tuning3d.model.core.Tracer;
//import com.despark.tuning3d.model.users.UsersProxy;
//import com.despark.tuning3d.view.components.core.AlertView;
//import com.distriqt.extension.facebookapi.FacebookAPI;
//import com.distriqt.extension.facebookapi.appevents.FacebookAppEvent;
//import com.distriqt.extension.facebookapi.events.FacebookAPISessionEvent;
//import com.distriqt.extension.facebookapi.events.ShareDialogEvent;
//import com.distriqt.extension.facebookapi.share.builders.ShareLinkContentBuilder;

public class FacebookManager {

    private static var self : FacebookManager;

    private var _tracer : Tracer;
    private var _isInitialized : Boolean;
    private var _loginCallback:Function;
    private var _profileObj:Object;
    private var _shareUrl:String;

    public static function get $() : FacebookManager{

        if(!self)
        {
            self = new FacebookManager();
        }
        return self;
    }

    public function FacebookManager() {

        if(self) {
            throw new Error("Facebook manager is singleton, do not attempt to create instance");
        }

        _tracer = new Tracer("Facebook");

        try
        {
            DistriqtController.init();
            FacebookAPI.init(DistriqtController.getKey());
            if (FacebookAPI.isSupported)
            {
                _tracer.print("IS SUPPORTED");
                if(FacebookAPI.service.initialiseApp( Constants.FACEBOOK_APP_ID ))
                {
                    _isInitialized = true;
                    _tracer.print("Is initialized", FacebookAPI.VERSION);

                    FacebookAPI.service.addEventListener( FacebookAPISessionEvent.SESSION_OPENED, facebook_sessionHandler );
                    FacebookAPI.service.addEventListener( FacebookAPISessionEvent.SESSION_INFO, facebook_sessionHandler );
                    FacebookAPI.service.addEventListener( FacebookAPISessionEvent.SESSION_OPEN_ERROR, facebook_sessionHandler );
                    FacebookAPI.service.addEventListener( FacebookAPISessionEvent.SESSION_OPEN_DISABLED, facebook_sessionHandler );
                    FacebookAPI.service.addEventListener( FacebookAPISessionEvent.SESSION_OPEN_CANCELLED, facebook_sessionHandler );

                    FacebookAPI.service.addEventListener( FacebookAPISessionEvent.SESSION_CLOSED, facebook_sessionHandler );
                    FacebookAPI.service.addEventListener( FacebookAPISessionEvent.SESSION_CLOSE_ERROR, facebook_sessionHandler );

                    FacebookAPI.service.addEventListener( FacebookAPISessionEvent.REQUEST_PERMISSIONS_COMPLETED, requestPermissionsHandler );
                    FacebookAPI.service.addEventListener( FacebookAPISessionEvent.REQUEST_PERMISSIONS_CANCELLED, requestPermissionsHandler );
                    FacebookAPI.service.addEventListener( FacebookAPISessionEvent.REQUEST_PERMISSIONS_ERROR, requestPermissionsHandler );

                }else {
                    _isInitialized = false;
                    _tracer.print("Not initialized");
                }
            }
            else {
                _isInitialized = false;
                _tracer.print("Not supported")
            }

        }
        catch (e:Error){
            _tracer.print("init error", e.getStackTrace());
            _isInitialized = false;
        }

    }

    internal function requestPermissionsHandler(event:FacebookAPISessionEvent ):void
    {
        switch (event.type)
        {
            case FacebookAPISessionEvent.REQUEST_PERMISSIONS_COMPLETED:
                _tracer.print('Permission completed');
                _tracer.print("User ID:         " + event.accessToken.userId );
                _tracer.print("Expiration date: " + event.accessToken.expirationTimestamp );
                _tracer.print("Permissions:     " + event.accessToken.permissions.join(", "));
                _tracer.print("Access token:    " + event.accessToken.token);


                break;

            case FacebookAPISessionEvent.REQUEST_PERMISSIONS_CANCELLED:
                // User cancelled the request
                break;

            case FacebookAPISessionEvent.REQUEST_PERMISSIONS_ERROR:
                // There was an error
                _tracer.print( "ERROR: "+ event.errorMessage );
                break;
        }
    }

    private function shareDialogEventHandler( event:ShareDialogEvent ):void
    {
        FacebookAPI.service.shareDialog.removeEventListener( ShareDialogEvent.DIALOG_COMPLETED, shareDialogEventHandler );
        FacebookAPI.service.shareDialog.removeEventListener( ShareDialogEvent.DIALOG_CANCELLED, shareDialogEventHandler );
        FacebookAPI.service.shareDialog.removeEventListener( ShareDialogEvent.DIALOG_ERROR, shareDialogEventHandler );

        _tracer.print('share: ' + event.type);
        if (event.postId != null) _tracer.print( "postId="+event.postId);
    }

    public function isLoggedIn() : Boolean {
        return FacebookAPI.isSupported && _isInitialized && FacebookAPI.service.isSessionOpen() && _profileObj;
    }

    public function shareURL(url : String) : void {
        _tracer.print(url);
        _shareUrl = url;

        if (FacebookAPI.service.shareDialog.canShow( ShareLinkContentBuilder.TYPE ))
        {
            FacebookAPI.service.shareDialog.addEventListener( ShareDialogEvent.DIALOG_COMPLETED, shareDialogEventHandler );
            FacebookAPI.service.shareDialog.addEventListener( ShareDialogEvent.DIALOG_CANCELLED, shareDialogEventHandler );
            FacebookAPI.service.shareDialog.addEventListener( ShareDialogEvent.DIALOG_ERROR, shareDialogEventHandler );

            var builder:ShareLinkContentBuilder = new ShareLinkContentBuilder()
                    .setContentTitle("3DTuning.com")
                    .setContentDescription("Share your own customized cars" )
                    .setContentUrl(_shareUrl);

            var success:Boolean = FacebookAPI.service.shareDialog.show( builder.build() );

            if (success) {
                _tracer.print("Share show succeeded")
            }

        } else {
            _tracer.print("Can not share");
        }
    }

    public function login(callback : Function) : void{
        if (FacebookAPI.isSupported && _isInitialized) {
            if(isLoggedIn()) {
                callback(_profileObj);
            } else {
                logOut();
            }
            _loginCallback = callback;
            var permissions:Array = [ "public_profile", "email"];
            var readOnly:Boolean = true;

            FacebookAPI.service.createSession( permissions, readOnly );
        }
        else
        {
            _tracer.print( "Can not start session, isSupported:", FacebookAPI.isSupported)
            AlertView.show("Facebook login is not supported on your device.");
        }
    }
    public function logOut() : void
    {
        if(FacebookAPI.isSupported && _isInitialized && FacebookAPI.service.isSessionOpen())
        {
            // Do not close the current session, cause it can be reused
            FacebookAPI.service.closeSession();
            _profileObj = null;
        }
    }

    private function callBackWithData(data : FacebookAPISessionEvent) : void
    {
        if(_loginCallback && data && data.profile && data.accessToken)
        {
            var profileObj : Object = {};
            profileObj.typeOfSN = UsersProxy.typeOfSN_FB;
            profileObj.user_id = data.profile.id;
            profileObj.access_token = data.accessToken.token;
            profileObj.oauth_token = "";
            profileObj.oauth_token_secret = "";
            profileObj.oauth_verifier = "";

            _profileObj = profileObj;
            _loginCallback(profileObj);
            _loginCallback = null;

        }
    }


    private function facebook_sessionHandler( event:FacebookAPISessionEvent ):void
    {
        switch (event.type)
        {
            case FacebookAPISessionEvent.SESSION_OPENED:
                // A session was opened successfully and you will have an access token
                _tracer.print("User ID:         " + event.accessToken.userId );
                _tracer.print("Expiration date: " + event.accessToken.expirationTimestamp );
                _tracer.print("Permissions:     " + event.accessToken.permissions.join(", "));
                _tracer.print("Access token:    " + event.accessToken.token);
                callBackWithData(event);
                break;

            case FacebookAPISessionEvent.SESSION_INFO:
                // More information about the user's session/profile is available
                _tracer.print( "User name: " + event.profile.name );
                callBackWithData(event);
                break;

            case FacebookAPISessionEvent.SESSION_OPEN_ERROR:
                // Triggered when a login error occurred
                _tracer.print("Session open error: " + event.errorMessage);
                break;

            case FacebookAPISessionEvent.SESSION_OPEN_DISABLED:
                // Triggered when the user has disabled Facebook login for your application in iOS settings
                _tracer.print("Session login is disabled");
                break;

            case FacebookAPISessionEvent.SESSION_OPEN_CANCELLED:
                // Triggered when the user cancels the sign-in process from Facebook
                _tracer.print("User cancelled sign-in");
                break;
            case FacebookAPISessionEvent.SESSION_CLOSED:
                // The session was closed
                _tracer.print("Session closed");
                break;

            case FacebookAPISessionEvent.SESSION_CLOSE_ERROR:
                // An error occurred when closing the session
                _tracer.print( "There was an error closing the session" );
                break;
        }
    }
}
}
