package com.distriqt {
    import com.despark.tuning3d.ApplicationFacade;
    import com.despark.tuning3d.model.core.Constants;
    import com.despark.tuning3d.model.users.UsersProxy;
    import com.distriqt.extension.pushnotifications.AuthorisationStatus;
    import com.distriqt.extension.pushnotifications.PushNotifications;
    import com.distriqt.extension.pushnotifications.Service;
    import com.distriqt.extension.pushnotifications.builders.ChannelBuilder;
    import com.distriqt.extension.pushnotifications.channels.Channel;
    import com.distriqt.extension.pushnotifications.events.AuthorisationEvent;
    import com.distriqt.extension.pushnotifications.events.RegistrationEvent;

    public class PushNotification {
        public var isInited: Boolean = false;
        public function PushNotification() {
            try {
                DistriqtController.init();
                if (PushNotifications.isSupported) {
                    PushNotifications.init(DistriqtController.getKey());
                    PushNotifications.service.addEventListener( RegistrationEvent.REGISTERING, registeringHandler );
                    PushNotifications.service.addEventListener( RegistrationEvent.REGISTER_SUCCESS, registerSuccessHandler );
                    PushNotifications.service.addEventListener( RegistrationEvent.CHANGED, registrationChangedHandler );
                    PushNotifications.service.addEventListener( RegistrationEvent.REGISTER_FAILED, registerFailedHandler );
                    PushNotifications.service.addEventListener( RegistrationEvent.ERROR, errorHandler );
                    PushNotifications.service.addEventListener( AuthorisationEvent.CHANGED, authorisationChangedHandler );

                    var service:Service = new Service(Service.FCM);
                    service.sandboxMode = false;
                    service.enableNotificationsWhenActive = true;
                    service.channels.push(
                            new ChannelBuilder()
                                    .setId("tuning_channel")
                                    .setName("3D Tuning Channel")
                                    .setImportance(Channel.IMPORTANCE_HIGH)
                                    .enableVibration(true)
                                    .enableLights(true)
                                    .setShowBadge(true)
                                    .build());

                    PushNotifications.service.setup(service);

                    switch (PushNotifications.service.authorisationStatus())
                    {
                        case AuthorisationStatus.AUTHORISED:
                            PushNotifications.service.register();
                            break;

                        case AuthorisationStatus.NOT_DETERMINED:
                            PushNotifications.service.requestAuthorisation();
                            break;

                        case AuthorisationStatus.DENIED:
//                            if (PushNotifications.service.canOpenDeviceSettings)
//                            {
//                                PushNotifications.service.openDeviceSettings();
//                            }
                            break;
                    }



                }

            } catch (e:Error) {
                trace( e );
            }
        }

        internal function authorisationChangedHandler(event:AuthorisationEvent) : void {
            PushNotifications.service.register();
        }

        internal function registeringHandler(event:RegistrationEvent ):void
        {
            trace( "Registration started" );
        }

        internal function registerSuccessHandler(event:RegistrationEvent ):void
        {
            isInited = true;
            trace( "Registration succeeded with ID: " + event.data  );
            var usersProxy : UsersProxy = ApplicationFacade.getInstance().retrieveProxy(UsersProxy.NAME) as UsersProxy;
            usersProxy.sendTokenId();
        }

        internal function registrationChangedHandler(event:RegistrationEvent ):void
        {
            trace( "Registration ID has changed: " + event.data );
        }

        internal function registerFailedHandler(event:RegistrationEvent ):void
        {
            trace( "Registration failed" );
        }

        internal function errorHandler(event:RegistrationEvent ):void
        {
            trace( "Registration error: "+event.data );
        }
    }
}
