package com.utils {
import feathers.controls.Button;

import starling.display.DisplayObject;
import starling.display.DisplayObjectContainer;
import starling.display.Mesh;
import starling.display.Stage;
import starling.textures.Texture;

public class DisplayObjectUtil {
    public function DisplayObjectUtil() {
    }

    public static function dispose(object:DisplayObjectContainer, skipButtons:Boolean = true, disposeTexture:Boolean = false):void{
        if(skipButtons && object is Button){
            return;
        }
        while(object.numChildren){
            var child:DisplayObject = object.removeChildAt(0);
            if(skipButtons && child is Button){
                continue;
            }
            if(child is DisplayObjectContainer){
                dispose(child as DisplayObjectContainer);
            }else if(disposeTexture){
                if (child.hasOwnProperty("texture") && child["texture"] is Texture) {
                    Texture(child["texture"]).dispose();
                    child["texture"] = null;
                }
            }
            child.dispose();
        }
    }

    public static function collectionDispose(collection:*):void{
        for(var key:* in collection){
            if(collection[key] is DisplayObject){
                if(collection[key] is DisplayObjectContainer){
                    dispose(collection[key], true, true);
                }
                if (collection[key].hasOwnProperty("texture") && collection[key]["texture"] is Texture) {
                    Texture(collection[key]["texture"]).dispose();
                    collection[key]["texture"] = null;
                }
                DisplayObject(collection[key]).dispose();
            }else{
                collectionDispose(collection[key]);
            }
            collection[key] = null;
        }
    }

    public static function disposeMesh(mesh:Mesh, disposeTexture:Boolean = true):void{
        if(mesh){
            if(disposeTexture && mesh.texture){
                mesh.texture.dispose();
                mesh.texture = null;
            }
            mesh.dispose();
        }
    }

    public static function bringToFront(object:DisplayObject):void{
        if(object.parent){
            object.parent.setChildIndex(object, object.parent.numChildren-1);
        }
    }

    public static function removeChildren(obj:DisplayObjectContainer):void {
        while (obj.numChildren){
            obj.removeChildAt(0);
        }
    }

    public static function redrawAll(displayObjectContainer:DisplayObjectContainer):void {
        var displayObject:DisplayObject;
        if(displayObjectContainer.numChildren){
            for(var i:int = 0; i < displayObjectContainer.numChildren; i++){
                displayObject = displayObjectContainer.getChildAt(i);
                if(displayObject is DisplayObjectContainer){
                    redrawAll(displayObject as DisplayObjectContainer);
                }
                displayObject.setRequiresRedraw();
            }
        }
    }
}
}
